/*
 * This source file (SQLServerConfiguration.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Utilities;
using System.IO;

namespace Bamboo.Cubed.AddIns.DatabaseConnectors
{
    /// <summary>
    /// This class configures the SQL Server Connector
    /// </summary>
    [XmlRoot("SQLServerConfiguration")]
    public partial class SQLServerConfiguration : CubedDatabaseConnectorConfiguration
    {
        /// <summary>
        /// If the screen update function is currently running, disable events
        /// </summary>
        private bool screenupdating = true;
        
        /// <summary>
        /// Initialise the new Add In Configuration conntrol
        /// </summary>
        public SQLServerConfiguration()
        {
            InitializeComponent();
            _properties = new SQLServerProperties();
            InitialiseProperties(String.Empty, 1.0);
            UpdateScreen();
        }

        /// <summary>
        /// Initialises a new instance of the Add In configuration control, based on the passed in 
        /// XML representing the current configuration and the version of the Add In the configuration was
        /// created under
        /// </summary>
        /// <param name="configurationxml">The Base64 encoded XML string containing the stored _properties of
        /// the Add In</param>
        /// <param name="configurationversion">The version the _properties were created for</param>
        public SQLServerConfiguration(string configurationxml, double configurationversion)
        {
            InitializeComponent();
            _properties = new SQLServerProperties();
            InitialiseProperties(configurationxml, configurationversion);
            UpdateScreen();
        }

        /// <summary>
        /// Update the screen items with the _properties
        /// </summary>
        private void UpdateScreen()
        {
            // Disable our auto property writing events
            screenupdating = true;

            // The authentication Mode
            if (_properties.AuthenticationMode == AuthenticationModes.Integrated)
            {
                IntegratedButton.Enabled = true;
            }
            else
            {
                SQLServerButton.Enabled = true;
            }

            // Then the database server and instance
            DatabaseServer.Text = _properties.Server;
            DatabaseInstance.Text = _properties.Instance;

            // The the database
            Database.Text = _properties.Database;

            // Finally the UserID and Password, but only if not INTEGRATED authentication
            if (_properties.AuthenticationMode != AuthenticationModes.Integrated)
            {
                UserID.Text = _properties.UserID;
                Password.Text = _properties.Password;
            }

            // Re-enable our events
            screenupdating = false;
        }

        /// <summary>
        /// The unique identifier of the SQL Server Connector
        /// </summary>
        [XmlElement("ID")]
        public override Guid ID
        {
            get { return new Guid("e9ed3fc3-be76-4de3-9022-67f766800509"); }
        }

        /// <summary>
        /// The name of the SQL Server Connector
        /// </summary>
        [XmlElement("Name")]
        public override string AddInName
        {
            get
            {
                return "SQL Server 2000/2005";
            }
        }        

        /// <summary>
        /// The current version of the SQL Server Connector
        /// </summary>
        [XmlElement("CurrentVersion")]
        public override double CurrentVersion
        {
            get { return 1.0; }
        }

        private SQLServerProperties _properties;
        /// <summary>
        /// This represents the _properties of the Add In
        /// </summary>
        public SQLServerProperties Properties
        {
            get { return _properties; }
            set { _properties = value; }
        }

        /// <summary>
        /// This deserialises the current properties of the Add In from the serialised string
        /// </summary>
        /// <param name="configurationstring">The serialised XML string</param>
        /// <returns>True if the _properties were deserialised successfully</returns>
        public override bool WriteConfiguration(string configurationstring)
        {
            if (configurationstring.Length > 0)
            {
                string[] configurationelements = configurationstring.Split(';');
                if (configurationelements.Length == 6)
                {
                    // The configuration string is complete, so lets use it
                    _properties.AuthenticationMode = (AuthenticationModes)Enum.Parse(typeof(AuthenticationModes), configurationelements[0]);
                    _properties.Database = configurationelements[1];
                    _properties.Instance = configurationelements[2];
                    _properties.Password = configurationelements[3];
                    _properties.Server = configurationelements[4];
                    _properties.UserID = configurationelements[5];

                    // Return true to indicate it was read correctly
                    UpdateScreen();
                    return true;
                }
                else
                    return false;
            }
            else
                return false;
        }

        /// <summary>
        /// This reads the current configuration and returns a string containing the configuration
        /// </summary>
        /// <returns>A serialised string</returns>
        public override string ReadConfiguration()
        {
            return _properties.AuthenticationMode.ToString() + ";" + _properties.Database + ";" +
                _properties.Instance + ";" + _properties.Password + ";" + _properties.Server + ";" +
                _properties.UserID;
        }

        /// <summary>
        /// This updates a version of the Add In from its' previous configuration to the latest
        /// </summary>
        /// <param name="currentconfiguration">The saved (previous) configuration XML</param>
        /// <param name="configurationversion">The current version of the Add In</param>
        /// <returns>The updated configuration</returns>
        public override string UpdateConfiguration(string currentconfiguration, double configurationversion)
        {
            // As this is the first release, there should be no need to update
            return currentconfiguration;
        }

        private void IntegratedButton_CheckedChanged(object sender, EventArgs e)
        {
            // Check if our screen is currently updating
            if (!screenupdating)
            {
                if (IntegratedButton.Checked)
                {
                    // Integrated Authentication has been chosen
                    _properties.AuthenticationMode = AuthenticationModes.Integrated;
                    Password.Enabled = false;
                    PasswordLabel.Enabled = false;
                    Password.Text = String.Empty;
                    UserID.Enabled = false;
                    UserIDLabel.Enabled = false;
                    UserID.Text = String.Empty;
                }
                else
                {
                    // SQL Server Authentication has been chosen
                    _properties.AuthenticationMode = AuthenticationModes.Authenticated;
                    Password.Enabled = true;
                    PasswordLabel.Enabled = true;
                    Password.Text = String.Empty;
                    UserID.Enabled = true;
                    UserIDLabel.Enabled = true;
                    UserID.Text = "sa";
                }
            }
        }

        private void SQLServerButton_CheckedChanged(object sender, EventArgs e)
        {
            // Check if our screen is currently updating
            if (!screenupdating)
            {
                if (IntegratedButton.Checked)
                {
                    // Integrated Authentication has been chosen
                    _properties.AuthenticationMode = AuthenticationModes.Integrated;
                    Password.Enabled = false;
                    PasswordLabel.Enabled = false;
                    Password.Text = String.Empty;
                    UserID.Enabled = false;
                    UserIDLabel.Enabled = false;
                    UserID.Text = String.Empty;
                }
                else
                {
                    // SQL Server Authentication has been chosen
                    _properties.AuthenticationMode = AuthenticationModes.Authenticated;
                    Password.Enabled = true;
                    PasswordLabel.Enabled = true;
                    Password.Text = String.Empty;
                    UserID.Enabled = true;
                    UserIDLabel.Enabled = true;
                    UserID.Text = "sa";
                }
            }
        }

        private void DatabaseServer_TextChanged(object sender, EventArgs e)
        {
            // Check if our screen is currently updating
            if (!screenupdating)
            {
                _properties.Server = DatabaseServer.Text;
            }
        }

        private void DatabaseInstance_TextChanged(object sender, EventArgs e)
        {
            // Check if our screen is currently updating
            if (!screenupdating)
            {
                _properties.Instance = DatabaseInstance.Text;
            }
        }

        private void Database_TextChanged(object sender, EventArgs e)
        {
            // Check if our screen is currently updating
            if (!screenupdating)
            {
                _properties.Database = Database.Text;
            }
        }

        private void UserID_TextChanged(object sender, EventArgs e)
        {
            // Check if our screen is currently updating, and whether integration authentication is selected
            if (!screenupdating && !IntegratedButton.Checked)
                _properties.UserID = UserID.Text;
        }

        private void Password_TextChanged(object sender, EventArgs e)
        {
            // Check if our screen is currently updating, and whether integrated authentication is selected
            if (!screenupdating && !IntegratedButton.Checked)
                _properties.Password = Password.Text;                
        }        
    }
}