﻿/*
 * This source file (Grid.ascx.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.IO;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;
using System.Globalization;
using System.Drawing;

namespace Bamboo.Cubed.AddIns.DesktopResults
{
    /// <summary>
    /// Allows for displaying of results in a standard .NET data grid
    /// </summary>
    public partial class Grid : CubedResult
    {
        #region Properties

        /// <summary>
        /// If this exists it indicates that a "drill down" has been requested onto the given field
        /// </summary>
        public string DrillOn
        {
            get
            {
                if (Session[SearchID.ToString() + "_DrillOn"] == null)
                    Session[SearchID.ToString() + "_DrillOn"] = String.Empty;
                return (string)Session[SearchID.ToString() + "_DrillOn"];
            }
            set
            {
                Session[SearchID.ToString() + "_DrillOn"] = value;
            }
        }

        /// <summary>
        /// This marks whether drill down capable is provided or not
        /// </summary>
        public bool DrillCapable
        {
            get
            {
                if (Session[SearchID.ToString() + "_DrillCapable"] == null)
                    Session[SearchID.ToString() + "_DrillCapable"] = false;
                return (bool)Session[SearchID.ToString() + "_DrillCapable"];
            }
            set
            {
                Session[SearchID.ToString() + "_DrillCapable"] = value;
            }
        }

        /// <summary>
        /// This ensures that when the ID of this Grid Add In gets set, the sub-control grid's ID
        /// also get's set
        /// </summary>
        public override string ID
        {
            get
            {
                return base.ID;
            }
            set
            {
                base.ID = value;
                ResultsGrid.ID = this.ID + "_GridControl";
            }
        }

        /// <summary>
        /// Returns the unique identifier of the Add In
        /// </summary>
        public override Guid AddInID
        {
            get { return new Guid("08bbe1b7-6c05-4f56-b886-89050f25571d"); }
        }

        /// <summary>
        /// Returns the name of the Add In
        /// </summary>
        public override string Name
        {
            get { return "Grid"; }
        }

        /// <summary>
        /// Returns a description of the Add In
        /// </summary>
        public override string Description
        {
            get { return "Allows for displaying of results in a standard grid"; }
        }

        /// <summary>
        /// Returns the current version of the Add In
        /// </summary>
        public override double CurrentVersion
        {
            get { return 1.1; }
        }

        /// <summary>
        /// Returns the name of the user control containing the Add In's markup
        /// </summary>
        public override string UserControlName
        {
            get { return "Grid.ascx"; }
        }

        /// <summary>
        /// Returns the name of the image to display in the search results for this item
        /// </summary>
        public override string SearchLogo
        {
            get { return "grid_preview.png"; }
        }

        /// <summary>
        /// Determines whether this grid should be displayed in dashboard mode (without any
        /// options for change) or standard item mode
        /// </summary>
        public override bool DashboardMode
        {
            get
            {
                if (Session[this.ID + "_GridDashboardMode"] == null)
                    Session[this.ID + "_GridDashboardMode"] = false;
                return (bool)Session[this.ID + "_GridDashboardMode"];
            }
            set
            {
                Session[this.ID + "_GridDashboardMode"] = value;
            }
        }

        /// <summary>
        /// The results which are displayed in this grid
        /// </summary>
        public override object Results
        {
            get
            {
                if (Session[this.ID + "_GridResults"] == null)
                    Session[this.ID + "_GridResults"] = new DataTable();
                return (object)Session[this.ID + "_GridResults"];
            }
            set
            {
                Session[this.ID + "_GridResults"] = value;
            }
        }

        /// <summary>
        /// The current display settings for the grid control
        /// </summary>
        public GridDisplaySettings DisplaySettings
        {
            get
            {
                if (Session[this.ID + "_GridDisplaySettings"] == null)
                    Session[this.ID + "_GridDisplaySettings"] = new GridDisplaySettings();
                return (GridDisplaySettings)Session[this.ID + "_GridDisplaySettings"];
            }
            set
            {
                Session[this.ID + "_GridDisplaySettings"] = value;
            }
        }

        /// <summary>
        /// The identifier of the search this grid is a part of
        /// </summary>
        public override Guid SearchID
        {
            get
            {
                if (Session[this.ID + "_SearchID"] == null)
                    Session[this.ID + "_SearchID"] = Guid.Empty;
                return (Guid)Session[this.ID + "_SearchID"];
            }
            set
            {
                Session[this.ID + "_SearchID"] = value;
            }
        }

        #endregion

        #region Implemented Methods

        /// <summary>
        /// Returns a Base64 encoded version of our current settings
        /// </summary>
        /// <returns>A Base64 encoded string containing the current settings</returns>
        public override string SaveWebDisplaySettings()
        {            
            // First update our settings
            if (SelectedXAxis != null && SelectedYAxis != null &&
                SelectedXAxis.SelectedIndex > -1 && SelectedYAxis.SelectedIndex > -1)
            {
                // Axis members have been selected so choose update our settings
                DisplaySettings.DefaultXAxis = SelectedXAxis.Text;
                DisplaySettings.DefaultYAxis = SelectedYAxis.Text;
            }
            else
            {
                // No members have been selected to set empty defaults
                DisplaySettings.DefaultXAxis = String.Empty;
                DisplaySettings.DefaultYAxis = String.Empty;
            }

            // Then update our decimal places
            int decimalplaces = 2;
            int.TryParse(DecimalPlaces.Text, out decimalplaces);
            DisplaySettings.DecimalPlaces = decimalplaces;

            // Then return our settings
            return DisplaySettings.ToString();
        }

        /// <summary>
        /// Given the Base64 string containing settings, updates the grid with those settings
        /// </summary>
        /// <param name="settings">A Base64 encoded version of the current settings</param>
        /// <returns>True if updated successfully</returns>
        public override bool LoadWebDisplaySettings(string settings)
        {
            // Then read our settings
            DisplaySettings.FromString(settings);

            // Check if our Axes have been loaded and load them if not
            if (SelectedXAxis.Items.Count < 1 || SelectedYAxis.Items.Count < 1)
                PrepareResults();            

            // Then update our Axes if they exist
            if (SelectedXAxis.Items.FindByText(DisplaySettings.DefaultXAxis) != null)
                SelectedXAxis.Text = DisplaySettings.DefaultXAxis;
            if (SelectedYAxis.Items.FindByText(DisplaySettings.DefaultYAxis) != null)
                SelectedYAxis.Text = DisplaySettings.DefaultYAxis;

            // Then update our decimal places
            DecimalPlaces.Text = DisplaySettings.DecimalPlaces.ToString();

            return true;
        }
        /// <summary>
        /// Returns the current result as a bitmap image ready for inserting into a consolidated document
        /// </summary>
        /// <returns>A bitmap representation of the current chart</returns>
        public override Bitmap GetResultImage()
        {
            return new Bitmap(400, 400);
        }


        #endregion

        #region Events

        protected override void OnInit(EventArgs e)
        {
            //PrepareResults();
            base.OnInit(e);
        }

        /// <summary>
        /// Renders the results to the grid if any results have been specified
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            // Then attempt to load our results
            PrepareResults();

            // Then check to see if we should display the pivot options. These should only be displayed
            // when we are using an MDDataSet and we're not in dashboard mode
            if (!DashboardMode && Results is Structures.XMLA.MDDataSet)
            {
                AxisChooserRow.Visible = true;
                DecimalChooserRow.Visible = true;
            }
            else if (!DashboardMode)
            {
                AxisChooserRow.Visible = false;
                DecimalChooserRow.Visible = true;
            }
            else
            {
                AxisChooserRow.Visible = false;
                DecimalChooserRow.Visible = false;
            }

            // Check to see if we have any results and display the no results label if none are 
            // available
            if (ResultsGrid.Items.Count < 1)
            {
                ResultsGrid.Visible = false;
                NoResultsLabel.Visible = true;
                AxisChooserRow.Visible = false;
                DecimalChooserRow.Visible = false;
            }
            else
            {
                ResultsGrid.Visible = true;
                NoResultsLabel.Visible = false;
            }

            // Add a link button to the end of each column to provide "drill down" capability provided the
            // ability is valid
            if (DrillCapable)
            {
                foreach (DataGridItem thisitem in ResultsGrid.Items)
                {
                    // Create a new cell
                    TableCell itemcell = new TableCell();
                    itemcell.ID = this.ID + "_DrillDownCell_" + thisitem.Cells[0].Text;

                    // Create a link button with a tag depending on the item clicked
                    LinkButton drilldown = new LinkButton();
                    drilldown.ID = this.ID + "_DrillDown_" + thisitem.Cells[0].Text;
                    drilldown.CommandArgument = thisitem.Cells[0].Text;
                    drilldown.Text = "Drill Down";
                    drilldown.Click += new EventHandler(drilldown_Click);
                    itemcell.Controls.Add(drilldown);
                    thisitem.Cells.Add(itemcell);
                }
            }
        }

        /// <summary>
        /// Initialises our screen
        /// </summary>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            // Then attempt to load our results
            PrepareResults();

            // Then check to see if we should display the pivot options. These should only be displayed
            // when we are using an MDDataSet and we're not in dashboard mode
            if (!DashboardMode && Results is Structures.XMLA.MDDataSet)
            {
                AxisChooserRow.Visible = true;
                DecimalChooserRow.Visible = true;
            }
            else if (!DashboardMode)
            {
                AxisChooserRow.Visible = false;
                DecimalChooserRow.Visible = true;
            }
            else
            {
                AxisChooserRow.Visible = false;
                DecimalChooserRow.Visible = false;
            }

            // Check to see if we have any results and display the no results label if none are 
            // available
            if (ResultsGrid.Items.Count < 1 || ResultsGrid.DataSource == null)
            {
                ResultsGrid.Visible = false;
                NoResultsLabel.Visible = true;
                AxisChooserRow.Visible = false;
                DecimalChooserRow.Visible = false;
            }
            else
            {
                ResultsGrid.Visible = true;
                NoResultsLabel.Visible = false;
            }

            // Add a link button to the end of each column to provide "drill down" capability
            if (String.IsNullOrEmpty(DrillOn))
            {
                if (DrillCapable)
                {
                    foreach (DataGridItem thisitem in ResultsGrid.Items)
                    {
                        // Create a new cell
                        TableCell itemcell = new TableCell();

                        // Create a link button with a tag depending on the item clicked
                        LinkButton drilldown = new LinkButton();
                        drilldown.ID = this.ID + "_DrillDown_" + thisitem.Cells[0].Text;
                        drilldown.CommandArgument = thisitem.Cells[0].Text;
                        drilldown.Text = "Drill Down";
                        drilldown.Click += new EventHandler(drilldown_Click);
                        itemcell.Controls.Add(drilldown);
                        thisitem.Cells.Add(itemcell);
                    }
                }
            }
            else
                // Make the "cancel drill down" button visible
                CancelDrillDownButton.Visible = true;
        }

        /// <summary>
        /// Initialises the drill down sequence by setting the session variable
        /// </summary>
        protected void drilldown_Click(object sender, EventArgs e)
        {
            // Grab the button which passed the event
            LinkButton eventbutton = (LinkButton)sender;

            // Grab the value of the row clicked and set it in session
            DrillOn = eventbutton.CommandArgument;

            // Make the "cancel drill down" button visible
            CancelDrillDownButton.Visible = true;
        }

        /// <summary>
        /// This cancels any active drill downs by setting the "drill down" property to empty
        /// </summary>
        protected void CancelDrillDownButton_Click(object sender, ImageClickEventArgs e)
        {
            // Clear the drill down
            DrillOn = String.Empty;

            // Make the "cancel drill down" button invisible
            CancelDrillDownButton.Visible = false;
        }

        /// <summary>
        ///  When the selected X axis changes, pivot the results
        /// </summary>
        protected void SelectedXAxis_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Update our display settings
            string oldx = DisplaySettings.DefaultXAxis;
            DisplaySettings.DefaultXAxis = SelectedXAxis.Text;

            // Then, if the newly selected X axis is the same as the Y axis, change the Y to the 
            // old X-Axis
            if (DisplaySettings.DefaultXAxis.ToUpper() == DisplaySettings.DefaultYAxis.ToUpper())
            {
                DisplaySettings.DefaultYAxis = oldx;
                SelectedYAxis.Text = oldx;
            }
        }

        /// <summary>
        ///  When the selected Y axis changes, pivot the results
        /// </summary>
        protected void SelectedYAxis_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Update our display settings
            string oldy = DisplaySettings.DefaultYAxis;
            DisplaySettings.DefaultYAxis = SelectedYAxis.Text;

            // Then, if the newly selected Y axis is the same as the X axis, change the X to the 
            // old Y-Axis
            if (DisplaySettings.DefaultYAxis.ToUpper() == DisplaySettings.DefaultXAxis.ToUpper())
            {
                DisplaySettings.DefaultXAxis = oldy;
                SelectedXAxis.Text = oldy;
            }
        }

        #endregion

        #region Support Methods

        /// <summary>
        /// This prepares our control based on the given results
        /// </summary>
        private void PrepareResults()
        {
            // First prepare our results data table
            DataTable results = new DataTable();
            if (Results is DataTable)
                results = (DataTable)Results;
            else if (Results is DataSet && ((DataSet)Results).Tables.Count > 0)
                results = ((DataSet)Results).Tables[0];
            else if (Results is Structures.XMLA.MDDataSet)
                results = ((Structures.XMLA.MDDataSet)Results).ToDataSet().Tables[0];

            // Then load our Axis definitions for selected values
            SelectedXAxis.Items.Clear();
            SelectedYAxis.Items.Clear();
            if (Results is Structures.XMLA.MDDataSet)
            {
                // Since the results are a multi-dimensional data set, add the axes from the data set
                foreach (Structures.XMLA.Axis thisaxis in ((Structures.XMLA.MDDataSet)Results).Axes)
                {
                    SelectedXAxis.Items.Add(thisaxis.Name);
                    SelectedYAxis.Items.Add(thisaxis.Name);
                }
            }
            else
            {
                // As the results are not a multi-dimensional dataset, just add 2 default axes
                SelectedXAxis.Items.Add("Axis 1");
                SelectedYAxis.Items.Add("Axis 1");
                SelectedXAxis.Items.Add("Axis 2");
                SelectedYAxis.Items.Add("Axis 2");
            }            
         
            // Then, if the selected axes are available, select them
            if (SelectedXAxis.Items.FindByText(DisplaySettings.DefaultXAxis) != null)
                SelectedXAxis.Text = DisplaySettings.DefaultXAxis;
            else if (SelectedXAxis.Items.Count > 0)
            {
                SelectedXAxis.SelectedIndex = 0;
                DisplaySettings.DefaultXAxis = SelectedXAxis.Text;
            }
            if (SelectedYAxis.Items.FindByText(DisplaySettings.DefaultYAxis) != null)
                SelectedYAxis.Text = DisplaySettings.DefaultYAxis;
            else if (SelectedYAxis.Items.Count > 1)
            {
                SelectedYAxis.SelectedIndex = 1;
                DisplaySettings.DefaultYAxis = SelectedYAxis.Text;
            }
            else if (SelectedYAxis.Items.Count > 0)
            {
                SelectedYAxis.SelectedIndex = 0;
                DisplaySettings.DefaultYAxis = SelectedYAxis.Text;
            }

            // Finally, provided axes have been selected and we have an MDDataSet - ensure the
            // correct axes have been pivoted
            if (SelectedXAxis.SelectedIndex > -1 && SelectedYAxis.SelectedIndex > -1 &&
                Results is Structures.XMLA.MDDataSet)
            {
                results = ((Structures.XMLA.MDDataSet)Results).ToDataSet(SelectedXAxis.Text,
                    SelectedYAxis.Text).Tables[0];
            }

            // Then update our results with the necessary decimal places
            if (results != null && results.Rows.Count > 0)
            {
                // As the inbound data table will contain columns with double data types, we need to
                // create a new mirror table which allows full formatting by having string data types
                DataTable newresults = new DataTable(results.TableName);
                foreach (DataColumn thiscolumn in results.Columns)
                    newresults.Columns.Add(new DataColumn(thiscolumn.ColumnName, typeof(string)));

                // Now copy all data across, formatting it appropriately if required
                foreach (DataRow thisrow in results.Rows)
                {
                    // Create a new data row in the new results table
                    DataRow newrow = newresults.NewRow();

                    // Now copy across all values from the current row
                    for (int i = 0; i < results.Columns.Count; i++)
                    {
                        // Check whether the value in this row/column combination is a number
                        double testvalue = 0;
                        if (thisrow[i] != null && double.TryParse(thisrow[i].ToString(), out testvalue))
                        {
                            // This value is a number so round it to the appropriate decimal places
                            string decimalplacestring = String.Empty;
                            for (int dcount = 0; dcount < DisplaySettings.DecimalPlaces; dcount++)
                            {
                                decimalplacestring += "0";
                            }
                            string rowvalue = testvalue.ToString("#,##0." + decimalplacestring, CultureInfo.CurrentCulture.NumberFormat);
                            newrow[i] = rowvalue;
                        }
                        else if (thisrow[i] != null)
                        {
                            // This column does not contain a number so just copy it's value
                            newrow[i] = thisrow[i].ToString();
                        }
                    }

                    // Add the new row to the new results table
                    newresults.Rows.Add(newrow);
                }

                // Finally bind our results to the updated table
                ResultsGrid.DataSource = newresults;
                ResultsGrid.DataBind();
            }
            else
            { 
                // No results were found, so hide our display
                ResultsGrid.Visible = false;
                NoResultsLabel.Visible = true;
                ResultsGrid.DataSource = null;               
            }  
        }

        #endregion
    }
}