/*
 * This source file (MapDisplaySettings.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;
using Microsoft.Live.ServerControls.VE;

namespace Bamboo.Cubed.AddIns.DesktopResults
{
    /// <summary>
    /// This class stores the required settings for displaying the Map
    /// </summary>
    [Serializable]
    public class MapDisplaySettings
    {
        /// <summary>
        /// Initialises a new default instance of the Map display settings
        /// </summary>
        public MapDisplaySettings()
        {
            _mapaxis = String.Empty;
            _valueaxis = String.Empty;
            _decimalplaces = 2;
            _mapcentre = new LatLong(1, 1);
            _mapzoom = 2;
            _mapstyle = MapStyle.Road;
            _trafficlights = false;
            _reversed = false;
            _lowvalue = 0;
            _highvalue = 100;
            _axiszoomlevel = new Dictionary<int, int>();
            _mapcontext = String.Empty;
            _aggregationtype = MapAggregationType.Sum;
        }

        private string _mapaxis;
        /// <summary>
        /// The Axis from the given multi-dimensional dataset to display on the map as locations
        /// </summary>
        public string MapAxis
        {
            get { return _mapaxis; }
            set { _mapaxis = value; }
        }

        private string _valueaxis;
        /// <summary>
        /// The Axis from the given multi-dimensional dataset to display on the map as labels of values
        /// at each location
        /// </summary>
        public string ValueAxis
        {
            get { return _valueaxis; }
            set { _valueaxis = value; }
        }

        private int _decimalplaces;
        /// <summary>
        /// The number of decimal places to format the results in
        /// </summary>
        public int DecimalPlaces
        {
            get { return _decimalplaces; }
            set { _decimalplaces = value; }
        }

        private LatLong _mapcentre;
        /// <summary>
        /// The latitude and longnitude of the centre of the map
        /// </summary>
        public LatLong MapCentre
        {
            get { return _mapcentre; }
            set { _mapcentre = value; }
        }

        private int _mapzoom;
        /// <summary>
        /// The zoom level of the map
        /// </summary>
        public int MapZoom
        {
            get { return _mapzoom; }
            set { _mapzoom = value; }
        }

        private MapStyle _mapstyle;
        /// <summary>
        /// The style of display for the map
        /// </summary>
        public MapStyle MapStyle
        {
            get { return _mapstyle; }
            set { _mapstyle = value; }
        }

        private bool _trafficlights;
        /// <summary>
        /// Whether to use traffic lights (red, green and orange) to denote levels or just a flat
        /// colour
        /// </summary>
        public bool TrafficLights
        {
            get { return _trafficlights; }
            set { _trafficlights = value; }
        }

        private bool _reversed;
        /// <summary>
        /// Whether the values should be reversed in order when using traffic lights
        /// </summary>
        public bool Reversed
        {
            get { return _reversed; }
            set { _reversed = value; }
        }

        private double _lowvalue;
        /// <summary>
        /// When using a traffic light, what is the bottom threshold value
        /// </summary>
        public double LowValue
        {
            get { return _lowvalue; }
            set { _lowvalue = value; }
        }

        private double _highvalue;
        /// <summary>
        /// When using a traffic light, what is the top threshold value
        /// </summary>
        public double HighValue
        {
            get { return _highvalue; }
            set { _highvalue = value; }
        }

        private Dictionary<int, int> _axiszoomlevel;
        /// <summary>
        /// The combinations of axes appropriate zoom levels to display them at
        /// </summary>
        public Dictionary<int, int> AxisZoomLevel
        {
            get { return _axiszoomlevel; }
            set { _axiszoomlevel = value; }
        }

        private string _mapcontext;
        /// <summary>
        /// This allows the setting of a context under which all locations are found (for localising
        /// maps)
        /// </summary>
        public string MapContext
        {
            get { return _mapcontext; }
            set { _mapcontext = value; }
        }

        private MapAggregationType _aggregationtype;
        /// <summary>
        /// This determines the aggregation type used for calculating traffic light scores on the map
        /// </summary>
        public MapAggregationType AggregationType
        {
            get { return _aggregationtype; }
            set { _aggregationtype = value; }
        }

        /// <summary>
        /// This sets the settings based on the passed in Base64 encoded string
        /// </summary>
        /// <param name="settings">A base64 encoded string containing settings</param>
        public void FromString(string settings)
        {
            string regularstring = UnicodeEncoding.Unicode.GetString(Convert.FromBase64String(settings));
            string[] settingsstrings = regularstring.Split(';');
            if (settingsstrings.Length == 13)
            {
                // These are settings
                _mapaxis = settingsstrings[0];
                _valueaxis = settingsstrings[1];
                int decimalplaces = 2;
                int.TryParse(settingsstrings[2], out decimalplaces);
                _decimalplaces = decimalplaces;
                int centrelat = 1;
                int centrelong = 1;
                if (settingsstrings[3].Split('|').Length == 2)
                {
                    int.TryParse(settingsstrings[3].Split('|')[0], out centrelat);
                    int.TryParse(settingsstrings[3].Split('|')[1], out centrelong);
                }
                _mapcentre = new LatLong(centrelat, centrelong);
                int mapzoom = 2;
                int.TryParse(settingsstrings[4], out mapzoom);
                _mapzoom = mapzoom;
                if (new List<string>(Enum.GetNames(typeof(MapStyle))).Contains(settingsstrings[5]))
                    _mapstyle = (MapStyle)Enum.Parse(typeof(MapStyle), settingsstrings[5]);
                bool trafficlights = true;
                bool.TryParse(settingsstrings[6], out trafficlights);
                _trafficlights = trafficlights;
                bool reversed = false;
                bool.TryParse(settingsstrings[7], out reversed);
                _reversed = reversed;
                double bottom = 0;
                double.TryParse(settingsstrings[8], out bottom);
                _lowvalue = bottom;
                double top = 0;
                double.TryParse(settingsstrings[9], out top);
                _highvalue = top;
                _mapcontext = settingsstrings[10];
                _aggregationtype = (MapAggregationType)Enum.Parse(typeof(MapAggregationType), settingsstrings[11]);
                string[] axiszoom = settingsstrings[12].Split('~');
                foreach (string thisaxiszoom in axiszoom)
                    if (thisaxiszoom.Split('|').Length == 2)
                    {
                        // This is a valid axis/zoom level
                        int axislevel = 1;
                        int zoomlevel = 1;
                        if (int.TryParse(thisaxiszoom.Split('|')[0], out axislevel) &&
                            int.TryParse(thisaxiszoom.Split('|')[1], out zoomlevel))
                        {
                            // The axis/zoom have been retrieved successfully so add them
                            _axiszoomlevel.Add(axislevel, zoomlevel);
                        }
                    }
            }
        }

        /// <summary>
        /// This returns the settings as a Base64 encoded string
        /// </summary>
        /// <returns>A base64 encoded string form of the settings</returns>
        public override string ToString()
        {
            string regularsettings = _mapaxis + ";";
            regularsettings += _valueaxis + ";";
            regularsettings += _decimalplaces.ToString() + ";";
            regularsettings += _mapcentre.Latitude.ToString() + "|" + _mapcentre.Longitude.ToString() + ";";
            regularsettings += _mapzoom.ToString() + ";";
            regularsettings += _mapstyle.ToString() + ";";
            regularsettings += _trafficlights.ToString() + ";";
            regularsettings += _reversed.ToString() + ";";
            regularsettings += _lowvalue.ToString() + ";";
            regularsettings += _highvalue.ToString() + ";";
            regularsettings += _mapcontext + ";";
            regularsettings += _aggregationtype.ToString() + ";";
            string axiszoomlevels = String.Empty;
            bool firstlevel = true;
            foreach (int thisaxis in _axiszoomlevel.Keys)
            {
                if (firstlevel)
                    firstlevel = false;
                else
                    axiszoomlevels += "~";
                axiszoomlevels += thisaxis.ToString() + "|" + _axiszoomlevel[thisaxis].ToString();
            }
            regularsettings += axiszoomlevels;
            return Convert.ToBase64String(UnicodeEncoding.Unicode.GetBytes(regularsettings));
        }
    }
}