/*
 * This source file (GridDisplaySettings.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;

namespace Bamboo.Cubed.AddIns.DesktopResults
{
    /// <summary>
    /// This class stores the required settings for displaying the Grid
    /// </summary>
    [Serializable]
    public class PivotGridDisplaySettings
    {
        /// <summary>
        /// Initialises a new default instance of the Grid display settings
        /// </summary>
        public PivotGridDisplaySettings()
        {
            _defaultxaxis = String.Empty;
            _defaultyaxis = String.Empty;
            _decimalplaces = 2;
            _recordsperpage = 10;
            _nopaging = false;
        }

        private string _defaultxaxis;
        /// <summary>
        /// The default X Axis from the given multi-dimensional dataset to display on the grid
        /// </summary>
        public string DefaultXAxis
        {
            get { return _defaultxaxis; }
            set { _defaultxaxis = value; }
        }

        private string _defaultyaxis;
        /// <summary>
        /// The default Y Axis from the given multi-dimensional dataset to display on the grid
        /// </summary>
        public string DefaultYAxis
        {
            get { return _defaultyaxis; }
            set { _defaultyaxis = value; }
        }

        private int _decimalplaces;
        /// <summary>
        /// The number of decimal places to format the results in
        /// </summary>
        public int DecimalPlaces
        {
            get { return _decimalplaces; }
            set { _decimalplaces = value; }
        }

        private int _recordsperpage;
        /// <summary>
        /// The number of records to show per page.
        /// </summary>
        public int RecordsPerPage
        {
            get { return _recordsperpage; }
            set { _recordsperpage = value; }
        }

        private bool _nopaging;
        /// <summary>
        /// Indicates that no paging should be applied to the grid. This property overrides the RecordsPerPage property.
        /// </summary>
        public bool NoPaging
        {
            get { return _nopaging; }
            set { _nopaging = value; }
        }

        private string[] _dataFields;
        /// <summary>
        /// The fields in the Data Area
        /// </summary>
        public string[] DataFields
        {
            get {
                if (_dataFields == null) _dataFields = new string[] { };
                return _dataFields; }
            set { _dataFields = value; }
        }

        private string[] _filterFields;
        /// <summary>
        /// The fields in the Filter Area
        /// </summary>
        public string[] FilterFields
        {
            get {
                if (_filterFields == null) _filterFields = new string[] { };
                return _filterFields;
            }
            set { _filterFields = value; }
        }

        private string[] _rowFields;
        /// <summary>
        /// The fields in the Row Area
        /// </summary>
        public string[] RowFields
        {
            get {
                if (_rowFields == null) _rowFields = new string[] { };
                return _rowFields;
            }
            set { _rowFields = value; }
        }

        private string[] _columnFields;
        /// <summary>
        /// The fields in the Row Area
        /// </summary>
        public string[] ColumnFields
        {
            get {
                if (_columnFields == null) _columnFields = new string[] { };
                return _columnFields;
            }
            set { _columnFields = value; }
        }

        private string _gridLayout;
        /// <summary>
        /// The current layout of the Grid.
        /// </summary>
        public string GridLayout
        {
            get { return _gridLayout; }
            set { _gridLayout = value; }
        }

        /// <summary>
        /// This sets the settings based on the passed in Base64 encoded string
        /// </summary>
        /// <param name="settings">A base64 encoded string containing settings</param>
        public void FromString(string settings)
        {
            string regularstring = UnicodeEncoding.Unicode.GetString(Convert.FromBase64String(settings));
            string[] settingsstrings = regularstring.Split(';');
            if (settingsstrings.Length >= 7)
            {
                // These are new settings
                _defaultxaxis = settingsstrings[0];
                _defaultyaxis = settingsstrings[1];
                int decimalplaces = 2;
                int.TryParse(settingsstrings[2], out decimalplaces);
                _decimalplaces = decimalplaces;

                _dataFields = settingsstrings[3].Split('|');
                _filterFields = settingsstrings[4].Split('|');
                _rowFields = settingsstrings[5].Split('|');
                _columnFields = settingsstrings[6].Split('|');

                // Check the length of the settings to support older versions.
                int recorsperpage = 10;
                bool nopaging = false;
                if (settingsstrings.Length > 7)
                {
                    int.TryParse(settingsstrings[7], out recorsperpage);

                    if (settingsstrings.Length > 8)
                    {
                        bool.TryParse(settingsstrings[8], out nopaging);

                        if (settingsstrings.Length > 9)
                            _gridLayout = settingsstrings[9];
                    }
                }

                _recordsperpage = recorsperpage;
                _nopaging = nopaging;
            }
            else if (settingsstrings.Length == 6)
            {
                _defaultxaxis = settingsstrings[0];
                _defaultyaxis = settingsstrings[1];
                _decimalplaces = 2;
                _dataFields = settingsstrings[2].Split('|');
                _filterFields = settingsstrings[3].Split('|');
                _rowFields = settingsstrings[4].Split('|');
                _columnFields = settingsstrings[5].Split('|');
                _recordsperpage = 10;
                _nopaging = false;
            }
        }

        /// <summary>
        /// This returns the settings as a Base64 encoded string
        /// </summary>
        /// <returns>A base64 encoded string form of the settings</returns>
        public override string ToString()
        {
            string regularsettings = _defaultxaxis + ";";
            regularsettings += _defaultyaxis + ";";
            regularsettings += _decimalplaces.ToString() + ";";
            regularsettings += string.Join("|", _dataFields) + ";";
            regularsettings += string.Join("|", _filterFields) + ";";
            regularsettings += string.Join("|", _rowFields) + ";";
            regularsettings += string.Join("|", _columnFields) + ";";
            regularsettings += _recordsperpage.ToString() + ";";
            regularsettings += _nopaging.ToString() + ";";
            regularsettings += _gridLayout;

            return Convert.ToBase64String(UnicodeEncoding.Unicode.GetBytes(regularsettings));
        }
    }
}