﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Data;
using System.Globalization;
using Microsoft.Live.ServerControls.VE;
using System.Net;

namespace Bamboo.Cubed.AddIns.DesktopResults
{
    /// <summary>
    /// This virtual earth page is required to display the virtual earth control in an iframe due
    /// to problems with AJAX sub display
    /// </summary>
    public partial class VirtualEarth : System.Web.UI.Page
    {
        #region Internal Properties

        /// <summary>
        /// This map ID is taken from the query string for the page
        /// </summary>
        public string MapID
        {
            get
            {
                if (Request.QueryString != null && Request.QueryString.AllKeys.Contains("MapID"))
                {
                    return Request.QueryString["MapID"];
                }
                else
                    return "default";
            }
        }

        /// <summary>
        /// This contains the settings necessary for displaying the map
        /// </summary>
        public MapControlSettings Settings
        {
            get
            {
                if (Session[MapID + "_Settings"] == null)
                    Session[MapID + "_Settings"] = new MapControlSettings();
                return (MapControlSettings)Session[MapID + "_Settings"];
            }
            set
            {
                Session[MapID + "_Settings"] = value;
            }
        }

        #endregion

        #region Events

        /// <summary>
        /// Initialises the map based on the given settings and the current data
        /// </summary>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            // Initialise the map based on our settings
            VirtualEarthMap.Width = Settings.Width;
            VirtualEarthMap.Height = Settings.Height;

            // If the map is in dashboard mode don't display the maps own dashbord
            if (Settings.DashboardMode)
                VirtualEarthMap.Dashboard = false;

            // Set various map style settings based on those saved (on first load)
            if (!Page.IsPostBack)
            {
                VirtualEarthMap.MapStyle = Settings.DisplaySettings.MapStyle;
                VirtualEarthMap.Center = Settings.DisplaySettings.MapCentre;
                VirtualEarthMap.ZoomLevel = Settings.DisplaySettings.MapZoom;
            }

            // Now load our map points (if data exists)
            LoadMapPoints();
        }

        /// <summary>
        /// This captures the current zoom level and sets our settings
        /// </summary>
        protected void VirtualEarthMap_ServerEndZoom(object sender, MapEventArgs e)
        {
            // First update our internal map settings
            Settings.DisplaySettings.MapZoom = VirtualEarthMap.ZoomLevel;

            // Then update the shape layers as we have now zoomed in
            LoadMapPoints();
        }

        /// <summary>
        /// This captures the current map style and sets our settings
        /// </summary>
        protected void VirtualEarthMap_ServerChangeMapStyle(object sender, MapEventArgs e)
        {
            Settings.DisplaySettings.MapStyle = VirtualEarthMap.MapStyle;
        }

        /// <summary>
        /// This captures the current center of the map and sets our setings
        /// </summary>
        protected void VirtualEarthMap_ServerEndPan(object sender, MapEventArgs e)
        {
            Settings.DisplaySettings.MapCentre = VirtualEarthMap.Center;
        }

        #endregion

        #region Support Functions

        /// <summary>
        /// This loads all map points freshly onto the map
        /// </summary>
        private void LoadMapPoints()
        {
            // First clear all current map shapes
            VirtualEarthMap.DeleteAllShapes();
            VirtualEarthMap.DeleteAllShapeLayers();

            // Then load all of the new shapes
            if (Settings.Data != null && Settings.Data.Columns.Count > 0)
                foreach (DataRow thisrow in Settings.Data.Rows)
                {
                    // Grab the name of the location (first column)
                    if (thisrow[0] != null && thisrow[0].ToString() != String.Empty)
                    {
                        // Grab the coordinates
                        LatLong coord = GetLocation(thisrow[0].ToString());

                        // Then add the location to the map
                        LoadLocation(thisrow[0].ToString(), coord.Latitude,
                            coord.Longitude, Settings.Data);
                    }
                }
        }

        /// <summary>
        /// This loads an individual location into the map
        /// </summary>
        /// <param name="location">The name of the location</param>
        /// <param name="latitude">The latitude of the location</param>
        /// <param name="longnitude">The longnitude of the location</param>
        /// <param name="results">The results collection</param>
        private void LoadLocation(string location, double latitude, double longnitude, DataTable results)
        {
            // Return if there are no valid results or there are not enough columns
            if (results.Rows.Count < 1 || results.Columns.Count < 2)
                return;

            // First grab the results for the location (either one or many depending on how many axes
            // are represented)
            Dictionary<string, double> locresults = new Dictionary<string, double>();
            double testresult = 0;

            if (results.Columns.Count == 2)
            {
                // We assume that since this table only has two columns it contains location names in the
                // first column and values in the second
                DataRow thisrow = results.Rows.Cast<DataRow>().FirstOrDefault(testrow => testrow[0].ToString().ToUpper() ==
                    location.ToUpper());
                if (thisrow != null)
                {
                    // We have found a valid data row so grab it's value from the 2nd column
                    double value = 0;
                    if (thisrow[1] != null)
                        double.TryParse(thisrow[1].ToString(), out value);
                    locresults.Add(location, value);
                }
            }
            else if (results.Columns.Count > 2 && !double.TryParse(results.Rows[0][1].ToString(), out testresult))
            {
                // Since this result set has more than two columns, but the second column is not a number,
                // that it contains nested combinations of locations in the first column, value types in the
                // second column, and values in the third column
                foreach (DataRow thisrow in results.Rows.Cast<DataRow>().Where(testrow => testrow[0].ToString().ToUpper() ==
                    location.ToUpper()))
                {
                    // Add each value, assuming the value is in the 3rd column
                    double value = 0;
                    if (thisrow[2] != null)
                        double.TryParse(thisrow[2].ToString(), out value);

                    // Then add the value to the collection with the 2nd column as the identifier
                    if (thisrow[1] != null && !locresults.ContainsKey(thisrow[1].ToString()))
                        locresults.Add(thisrow[1].ToString(), value);
                }
            }
            else if (results.Columns.Count > 2)
            {
                // We assume that, since all columns after the first have values - that this data set
                // contains location names in the first column and values thereafter
                foreach (DataRow thisrow in results.Rows.Cast<DataRow>().Where(testrow => testrow[0].ToString().ToUpper() ==
                    location.ToUpper()))
                {
                    for (int i = 1; i < results.Columns.Count; i++)
                    {
                        // Grab the value of this column if possible
                        double value = 0;
                        if (thisrow[i] != null)
                            double.TryParse(thisrow[i].ToString(), out value);

                        // Then add the value to the collection with the column name as the identifier
                        if (!locresults.ContainsKey(results.Columns[i].ColumnName))
                            locresults.Add(results.Columns[i].ColumnName, value);
                    }
                }
            }

            // Now determine how many results we are adding to the map, and add each result
            if (locresults.Count == 1)
            {
                // There is only one result, so add the title and value with no identifier
                Shape newlocation = new Shape(ShapeType.Pushpin,
                    new LatLongWithAltitude(latitude, longnitude));
                newlocation.CustomIcon = GetMapImage(locresults[locresults.Keys.ElementAt(0)]) + ".png";
                newlocation.Title = location;

                // As there is only one value, add it as the sole description
                string decimalplacestring = String.Empty;
                for (int dcount = 0; dcount < Settings.DisplaySettings.DecimalPlaces; dcount++)
                {
                    decimalplacestring += "0";
                }
                newlocation.Description = locresults[location].ToString("#" + CultureInfo.CurrentCulture.NumberFormat.CurrencyGroupSeparator +
                        "##0" + CultureInfo.CurrentCulture.NumberFormat.CurrencyDecimalSeparator +
                        decimalplacestring);

                // Finally add the shape to the map
                VirtualEarthMap.AddShape(newlocation);
            }
            else
            {
                // There are more than one results, so add each with the title and identifier of each
                // similar result
                Shape newlocation = new Shape(ShapeType.Pushpin,
                    new LatLongWithAltitude(latitude, longnitude));
                newlocation.Title = location;

                // Based on the selected value aggregation method, pass through the aggregated total
                // of given values to the map image generator
                switch (Settings.DisplaySettings.AggregationType)
                {
                    case MapAggregationType.Sum:
                        newlocation.CustomIcon = GetMapImage(locresults.Values.Sum()) + ".png";
                        break;
                    case MapAggregationType.Average:
                        newlocation.CustomIcon = GetMapImage(locresults.Values.Average()) + ".png";
                        break;
                }                
                
                // As there are multiple values, loop through them and add them all to the description
                // with line breaks in between
                string decimalplacestring = String.Empty;
                for (int dcount = 0; dcount < Settings.DisplaySettings.DecimalPlaces; dcount++)
                {
                    decimalplacestring += "0";
                }
                string locvaluestring = String.Empty;
                foreach (string thislocvalue in locresults.Keys)
                    locvaluestring += "<b>" + thislocvalue + "</b>: " +
                        locresults[thislocvalue].ToString("#" + CultureInfo.CurrentCulture.NumberFormat.CurrencyGroupSeparator +
                        "##0" + CultureInfo.CurrentCulture.NumberFormat.CurrencyDecimalSeparator +
                        decimalplacestring) + "<br>";
                newlocation.Description = locvaluestring;

                // Finally add the shape to the map
                VirtualEarthMap.AddShape(newlocation);
            }
        }

        /// <summary>
        /// This returns a string representing the name of the image to use, depending on whether
        /// traffic lights are being used, the value and zoom level
        /// </summary>
        /// <param name="value">The value of the map item</param>
        /// <returns>A string containing the name of the image</returns>
        private string GetMapImage(double value)
        {
            // Determine the image size based on zoom level
            string size = "16";
            //if (VirtualEarthMap.ZoomLevel < 3)
            //    size = "16";
            //else if (VirtualEarthMap.ZoomLevel > 7)
            //    size = "48";

            // Then determine if traffic lights are being used
            if (!Settings.DisplaySettings.TrafficLights)
            {
                // As traffic lights are not being used, just return the purple colour
                return "purple-" + size;
            }
            else
            {
                // As traffic lights are being used, determine if the values are reversed or not
                if (Settings.DisplaySettings.Reversed)
                {
                    // As the settings are reversed, a low value is good
                    if (value > Settings.DisplaySettings.HighValue)
                        return "green-" + size;
                    else if (value < Settings.DisplaySettings.LowValue)
                        return "red-" + size;
                    else
                        return "orange-" + size;
                }
                else
                {
                    // The settings are normal, so a high value is good
                    if (value > Settings.DisplaySettings.HighValue)
                        return "red-" + size;
                    else if (value < Settings.DisplaySettings.LowValue)
                        return "green-" + size;
                    else
                        return "orange-" + size;
                }
            }
        }

        /// <summary>
        /// This returns the latitude and longnitude of a location
        /// </summary>
        /// <param name="locationname">The name of the location to find</param>
        /// <returns>The latitude and longnitude</returns>
        private LatLong GetLocation(string locationname)
        {
            // If there is a set map context, apply it to the location name
            if (Settings.DisplaySettings.MapContext != String.Empty)
                locationname += " " + Settings.DisplaySettings.MapContext;

            // First grab the response details from the Google Maps service
            WebClient web = new WebClient();
            string responsestring = web.DownloadString("http://maps.google.com/maps/geo?q=" + locationname.Replace(" ", "%20") + "&output=CSV");

            // Then we assume that the location, being CSV, is split by commas and includes the lat/long as
            // the last 2 options
            string[] responseparts = responsestring.Split(',');
            double latitude = 1;
            double longnitude = 1;
            double.TryParse(responseparts[responseparts.Length - 2], out latitude);
            double.TryParse(responseparts[responseparts.Length - 1], out longnitude);

            // Then we return the coordinates
            return new LatLong(latitude, longnitude);
        }

        #endregion
    }
}
