﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Bamboo.Cubed.Engine.Persistence;
using Bamboo.Cubed.Structures;

namespace Bamboo.Cubed.AddIns.FormulaEvaluators
{
    /// <summary>
    /// This class provides value parsing functionality for formulas
    /// </summary>
    public static class ValueParser
    {
        /// <summary>
        /// Parses the given source code, replacing references to supported functions with the value that
        /// function would return, prior to compilation
        /// </summary>
        /// <param name="cube">The Cube being consolidated</param>
        /// <param name="member">The Member being evaluated</param>
        /// <param name="valuescollection">The collection of values of other members on the same dimension
        /// <param name="source">The Source Code to parse</param>
        /// evaluated</param>
        public static string ParseMemberValues(Cube cube, Member member,
            Dictionary<string, double> valuescollection, string source)
        {
            // First the PreviousMember() function
            if (source.Contains("PreviousMember()"))
            {
                // There is at least one reference to the PreviousMember() value, 
                // so lets find the previous member's value
                double previousmembervalue = 0;
                int currentmemberindex = cube.Dimensions[member.Dimension].Members.IndexOf(member);
                if (currentmemberindex > 0 && currentmemberindex < cube.Dimensions[member.Dimension].Members.Count &&
                    cube.Dimensions[member.Dimension].Members[currentmemberindex - 1].Level == member.Level &&
                    valuescollection.ContainsKey(cube.Dimensions[member.Dimension].Members[currentmemberindex - 1].Name))
                    previousmembervalue = valuescollection[cube.Dimensions[member.Dimension].Members[currentmemberindex - 1].Name];

                // Now replace all references to the PreviousMember() function with the value returned
                source = source.Replace("PreviousMember()", previousmembervalue.ToString());
            }

            // Then the NextMember() function
            if (source.Contains("NextMember()"))
            {
                // There is at least one reference to the NextMember() value, 
                // so lets find the next member's value
                double nextmembervalue = 0;
                int currentmemberindex = cube.Dimensions[member.Dimension].Members.IndexOf(member);
                if (currentmemberindex > 0 && currentmemberindex < (cube.Dimensions[member.Dimension].Members.Count - 1) &&
                    cube.Dimensions[member.Dimension].Members[currentmemberindex + 1].Level == member.Level &&
                    valuescollection.ContainsKey(cube.Dimensions[member.Dimension].Members[currentmemberindex + 1].Name))
                    nextmembervalue = valuescollection[cube.Dimensions[member.Dimension].Members[currentmemberindex + 1].Name];

                // Now replace all references to the NextMember() function with the value returned
                source = source.Replace("NextMember()", nextmembervalue.ToString());
            }

            // Then the Children() function
            if (source.Contains("Children()"))
            {
                // There is at least one reference to the Children() value, so lets find all member values
                // which are children of this parent
                double totalchildrenvalue = 0;
                foreach (Member thismember in cube.Dimensions[member.Dimension].Members.Where(membertest =>
                    membertest.Parent != null && membertest.Parent == member.ID))
                    if (valuescollection.ContainsKey(thismember.Name))
                        totalchildrenvalue += valuescollection[thismember.Name];

                // Now replace all references to the Children() function with the value returned
                source = source.Replace("Children()", totalchildrenvalue.ToString());
            }

            // Then the specific Member("") function
            if (source.Contains("Member(\""))
            {
                // There is at least one member reference, so replace all the instances with their specified
                // member values by looping through our source looking for the function
                string membername = String.Empty;
                while (GetNextMemberName(source, out membername))
                {
                    // Replace any instances of the member function with the value from the values collection
                    double membervalue = 0;
                    if (valuescollection.ContainsKey(membername))
                        membervalue = valuescollection[membername];
                    source = source.Replace("Member(\"" + membername + "\")", membervalue.ToString());
                }
            }

            // Finally the Member() function
            if (source.Contains("Member()"))
            {
                // There is at least one reference to the Member() value, so lets grab the members current value
                double membervalue = 0;
                if (valuescollection.ContainsKey(member.Name))
                    membervalue = valuescollection[member.Name];

                // Now replace all references to the Member() function with the value returned
                source = source.Replace("Member()", membervalue.ToString());
            }

            return source;
        }

        private static bool GetNextMemberName(string source, out string membername)
        {
            // Set out member name to empty in case we don't find a member function
            membername = String.Empty;

            // Grab the index of the start of the next specific member function
            int startindex = source.IndexOf("Member(\"", 0);

            // If the index is less than zero or more than the end of the source code, then no member function
            // was found
            if (startindex < 0 || startindex > source.Length)
                return false;

            // Add 8 to the start index to move it to the start of the member name
            startindex = startindex + 8;

            // If the index was found, grab the end of the member name
            int endindex = source.IndexOf("\"", startindex);

            // If the index was greater than the length of the source (or less than zero), then no ending was
            // found
            if (endindex < 0 || endindex > source.Length)
                return false;

            // At this point we must have a start and an end index, both within the current source string, so
            // let's use them to grab the member name
            membername = source.Substring(startindex, endindex - startindex);
            return true;
        }

        /// <summary>
        /// Parses the given source code, replacing references to supported functions with the value that
        /// function would return, prior to compilation
        /// </summary>
        /// <param name="cube">The Cube being consolidated</param>
        /// <param name="measure">The Measure being evaluated</param>
        /// <param name="valuescollection">A collection of other member names and their values</param>
        /// <param name="source">The Source Code to parse</param>        
        /// <returns>A string containing a parsed version of the source code</returns>
        public static string ParseMeasureValues(Cube cube, Measure measure, 
            Dictionary<string, double> valuescollection, string source)
        {
            // First the specific Measure("") function
            if (source.Contains("Measure(\""))
            {
                // There is at least one measure reference, so replace all the instances with their specified
                // measure values by looping through our source looking for the function. We need to do this for
                // the given combination of members
                string measurename = String.Empty;
                while (GetNextMeasureName(source, out measurename))
                {
                    // Replace any instances of the measure function with the value from the values collection
                    double measurevalue = 0;
                    if (valuescollection.ContainsKey(measurename))
                        measurevalue = valuescollection[measurename];
                    source = source.Replace("Measure(\"" + measurename + "\")", measurevalue.ToString());
                }
            }

            // Then the Measure() function
            if (source.Contains("Measure()"))
            {
                // There is at least one reference to the Measure() value, so lets grab the measures current value
                double measurevalue = 0;
                if (valuescollection.ContainsKey(measure.Name))
                    measurevalue = valuescollection[measure.Name];

                // Now replace all references to the Measure() function with the value returned
               source = source.Replace("Measure()", measurevalue.ToString());
            }

            return source;
        }

        private static bool GetNextMeasureName(string source, out string measurename)
        {
            // Set our measure name to empty in case we don't find a member function
            measurename = String.Empty;

            // Grab the index of the start of the next specific measure function
            int startindex = source.IndexOf("Measure(\"", 0);

            // If the index is less than zero or more than the end of the source code, then no measure function
            // was found
            if (startindex < 0 || startindex > source.Length)
                return false;

            // Add 9 to the start index to move it to the start of the measure name
            startindex = startindex + 9;

            // If the index was found, grab the end of the measure name
            int endindex = source.IndexOf("\"", startindex);

            // If the index was greater than the length of the source (or less than zero), then no ending was
            // found
            if (endindex < 0 || endindex > source.Length)
                return false;

            // At this point we must have a start and an end index, both within the current source string, so
            // let's use them to grab the measure name
            measurename = source.Substring(startindex, endindex - startindex);
            return true;
        }
    }
}
