/*
 * This source file (Maximum.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;
using System.Data;
using System.CodeDom;
using System.Reflection;
using Bamboo.Cubed.Utilities;
using System.IO;

namespace Bamboo.Cubed.AddIns.Functions
{
    /// <summary>
    /// This AddIn provides a Maximum function
    /// </summary>
    [XmlRoot("Maximum")]
    public class Maximum : CubedFunction
    {
        /// <summary>
        /// Initialises a new default instance of the Add In
        /// </summary>
        public Maximum()
        {
            _properties = new MaximumProperties();
            base.InitialiseProperties(String.Empty, 1.0);
            Parameters = new GenericKeyCollection<string, CubedFunctionParameter>();
        }

        /// <summary>
        /// Initialises the Add In based on the passed in _properties
        /// </summary>
        /// <param name="configurationxml">The Base64 encoded XML string containing the configuration</param>
        /// <param name="configurationversion">The version of the configuration that was used</param>
        /// <param name="parameters">The Parameters for the function</param>
        public Maximum(string configurationxml, double configurationversion, 
            GenericKeyCollection<string, CubedFunctionParameter> parameters)
        {
            _properties = new MaximumProperties();
            base.InitialiseProperties(configurationxml, configurationversion);
            Parameters = parameters;
        }

        /// <summary>
        /// The unique identifier of the Maximum Function
        /// </summary>
        [XmlElement("ID")]
        public override Guid ID
        {
            get { return new Guid("eb4a4ac7-6b2c-42e0-bc6f-2a6ad50b4615"); }
        }

        /// <summary>
        /// The name of the Maximum Function
        /// </summary>
        [XmlElement("Name")]
        public override string Name
        {
            get
            {
                return "Maximum";
            }
        }

        /// <summary>
        /// The description of the Maximum Function
        /// </summary>
        [XmlElement("Description")]
        public override string Description
        {
            get { return "Returns the maximum value of all members specified."; }
        }

        /// <summary>
        /// The current version of the Maximum Function
        /// </summary>
        [XmlElement("CurrentVersion")]
        public override double CurrentVersion
        {
            get { return 1.0; }
        }

        /// <summary>
        /// The configuration control for Maximum Function
        /// </summary>
        [XmlElement("ConfigurationControl")]
        public override CubedFunctionConfiguration ConfigurationControl
        {
            get { return new MaximumConfiguration(); }
        }

        private MaximumProperties _properties;
        /// <summary>
        /// This represents the _properties of the Add In
        /// </summary>
        public MaximumProperties Properties
        {
            get { return _properties; }
            set { _properties = value; }
        }

        /// <summary>
        /// This deserialises the current properties of the Add In from the serialised string
        /// </summary>
        /// <param name="configurationstring">The serialised XML string</param>
        /// <returns>True if the _properties were deserialised successfully</returns>
        public override bool WriteConfiguration(string configurationstring)
        {
            if (configurationstring.Length > 0)
            {
                // The configuration string is complete, so lets use it  
                int newvalue = 0;
                if (int.TryParse(configurationstring, out newvalue))
                    _properties.MaxMembers = newvalue;

                // Return true to indicate it was read correctly
                return true;
            }
            else
                return false;
        }

        /// <summary>
        /// This updates a version of the Add In from its' previous configuration to the latest
        /// </summary>
        /// <param name="currentconfiguration">The saved (previous) configuration XML</param>
        /// <param name="configurationversion">The current version of the Add In</param>
        /// <returns>The updated configuration</returns>
        public override string UpdateConfiguration(string currentconfiguration, double configurationversion)
        {
            // As this is the first release, there should be no need to update
            return currentconfiguration;
        }

        /// <summary>
        /// Evaluates the Maximum of the members in the given set, based on the cube and data given
        /// </summary>
        /// <param name="cube">The cube to process</param>
        /// <param name="data">A data matrix containing the data for the cube</param>
        /// <param name="selectedmembers">The selected members on other axes for the cell to process</param>
        /// <param name="selectedmeasure">The measure selected</param>
        /// <param name="username">The name of the user evaluating the function</param>
        /// <returns>The average of the members for this cell</returns>
        public override object Evaluate(Cube cube, DataMatrix data, List<Member> selectedmembers,
            Measure selectedmeasure, string username)
        {            
            // First we need to look in the parameter collection, assuming they are valid members
            List<Member> maxmembers = new List<Member>();
            int membercounter = 0;
            foreach (CubedFunctionParameter thisparameter in Parameters)
            {
                if (membercounter >= _properties.MaxMembers && _properties.MaxMembers > 0)
                    break;
                object newmembervalue;                
                thisparameter.GetParameterValue(out newmembervalue);
                string newmember = (string)newmembervalue;                
                if (newmember != null && newmember != String.Empty)
                {
                    // Split the string, assuming there is at least the dimension and member name
                    string[] memberparts = newmember.Split('.');
                    if (memberparts.Length > 1)
                    {
                        // There are at least two parts, so we assume the first part to be the dimension of the member,
                        // and the last part to be the name of the member
                        string dimension = memberparts[0];
                        string membername = memberparts[memberparts.Length - 1];
                        if (dimension != String.Empty && membername != String.Empty)
                        {
                            // We have both a member name and a dimension, so lets try and find the member in 
                            // our cube
                            foreach (Dimension thisdimension in cube.Dimensions)
                                if (thisdimension.Name.ToUpper().Equals(dimension.ToUpper()))
                                {
                                    // We have found our dimension, try and find the member
                                    foreach (Member thismember in thisdimension.Members)
                                        if (thismember.Name.ToUpper().Equals(membername.ToUpper()))
                                        {
                                            // We have found our member, add it to our collection to evaluate
                                            maxmembers.Add(thismember);
                                            membercounter++;
                                        }
                                }
                        }
                    }
                }                
            }

            // Now we have processed our parameters, hopefully we have the list of members to find the 
            // maximum of
            if (maxmembers.Count > 0 && selectedmeasure != null && selectedmembers.Count > 0)
            {
                // We have all the required elements. Lets first determine whether all dimensions have
                // been represented, and if not - get the default members of those dimensions
                List<Guid> selecteddimensions = new List<Guid>();
                selecteddimensions.Add(maxmembers[0].Dimension);

                foreach (Member thismember in selectedmembers)
                    selecteddimensions.Add(thismember.Dimension);

                foreach (Dimension thisdimension in cube.Dimensions)
                    if (!selecteddimensions.Contains(thisdimension.ID))
                    {
                        // This dimension has not been represented, so lets add it's default member
                        if (thisdimension.Members.Count > 0)
                        {
                            selectedmembers.Add(thisdimension.Members[0]);
                        }
                    }

                // We should now have a selected members and average members collection which represent each
                // dimension of the cube. Now we should work out the maximum
                double highest = 0;
                foreach (Member thismember in maxmembers)
                {
                    // Create a copy of the selected members
                    List<Member> thisnodemembers = selectedmembers;
                    thisnodemembers.Add(thismember);
                    double thisvalue = data.GetValue(thisnodemembers, selectedmeasure);
                    if (thisvalue > highest)
                        highest = thisvalue;
                }

                // Return the maximum 
                return highest;
            }
            else
                return 0;
        }
    }
}