/*
 * This source file (AddInManager.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Reflection;
using System.IO;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;
using System.Web.UI;

namespace Bamboo.Cubed.AddIns
{
    /// <summary>
    /// The Add In Manager controls the detection and activation of all Add Ins supported by the Cubed system.
    /// </summary>
    [Serializable]
    public static class AddInManager
    {
        private static string addinsdirectory = Information.MyDirectory + @".\AddIns\";
        /// <summary>
        /// This sets the directory where all Add Ins assembly files are located
        /// </summary>
        public static string AddInsDirectory
        {
            get 
            {
                if (addinsdirectory == null || addinsdirectory == String.Empty)
                    addinsdirectory = @".\AddIns\";
                return addinsdirectory; 
            }
            set { addinsdirectory = value; }
        }

        private static GenericKeyCollection<Guid, CubedFunctionDescription> functions = 
            new GenericKeyCollection<Guid, CubedFunctionDescription>();
        /// <summary>
        /// Stores all of the Functions available for use in Cubed
        /// </summary>
        public static GenericKeyCollection<Guid, CubedFunctionDescription> Functions
        {
            get 
            {
                if (functions == null || functions.Count < 1)
                    GetFunctions();
                return functions; 
            }
        }

        private static GenericKeyCollection<Guid, CubedKPIDescription> kpicalculators;
        /// <summary>
        /// Stores all of the KPI Calculators available for use in Cubed
        /// </summary>
        public static GenericKeyCollection<Guid, CubedKPIDescription> KPICalculators
        {
            get 
            {
                if (kpicalculators == null || kpicalculators.Count < 1)
                    GetKPICalculators();
                return kpicalculators;
            }
        }

        private static GenericKeyCollection<Guid, CubedFormulaDescription> formulas;
        /// <summary>
        /// Stores all of the Formula evaluators available for use in Cubed
        /// </summary>
        public static GenericKeyCollection<Guid, CubedFormulaDescription> Formulas
        {
            get
            {
                if (formulas == null || formulas.Count < 1)
                    GetFormulas();
                return formulas;
            }
        }

        private static GenericKeyCollection<Guid, CubedQueryParserDescription> parsers;
        /// <summary>
        /// Stores all of the Query Parsers available for use in Cubed
        /// </summary>
        public static GenericKeyCollection<Guid, CubedQueryParserDescription> Parsers
        {
            get
            {
                if (parsers == null || parsers.Count < 1)
                    GetParsers();
                return parsers;
            }
        }

        private static GenericKeyCollection<Guid, CubedDatabaseConnectorDescription> connectors;
        /// <summary>
        /// Stores all Database Connectors available for use in Cubed
        /// </summary>
        public static GenericKeyCollection<Guid, CubedDatabaseConnectorDescription> Connectors
        {
            get 
            { 
                if (connectors == null || connectors.Count < 1)
                    GetConnectors();
                return connectors;
            }
        }

        private static GenericKeyCollection<Guid, CubedSearchDescription> _searchcontrols;
        /// <summary>
        /// Stores all Cubed Search Controls available for use in Cubed Desktop
        /// </summary>
        public static GenericKeyCollection<Guid, CubedSearchDescription> SearchControls
        {
            get
            {
                if (_searchcontrols == null || _searchcontrols.Count < 1)
                    GetSearchControls();
                return _searchcontrols;
            }
        }

        private static GenericKeyCollection<Guid, CubedResultDescription> _resultcontrols;
        /// <summary>
        /// Stores all Cubed Result Controls available for use in Cubed Desktop
        /// </summary>
        public static GenericKeyCollection<Guid, CubedResultDescription> ResultControls
        {
            get
            {
                if (_resultcontrols == null || _resultcontrols.Count < 1)
                    GetResultControls();
                return _resultcontrols;
            }
        }

        /// <summary>
        /// Recursively trawls through directories to grab all .NET assembly files (assumed to have the extension
        /// *.DLL) which exist under their
        /// </summary>
        /// <param name="StartDirectory">The top level root directory to start searching from</param>
        /// <param name="Assemblies">A string list to add the name of the assembly files to</param>
        public static void GetAssemblyFiles(string StartDirectory, ref List<string> Assemblies)
        {
            // First get all the assemblies in the passed in directory            
            foreach (string thisfile in Directory.GetFiles(StartDirectory))
                if (thisfile.Substring(thisfile.Length - 3, 3).ToUpper().Equals("DLL"))
                    Assemblies.Add(thisfile);

            // Now get a list of all sub directories and recursively call the function to find their assemblies
            foreach (string thisdirectory in Directory.GetDirectories(StartDirectory))                
                GetAssemblyFiles(thisdirectory, ref Assemblies);
        }

        /// <summary>
        /// Returns a list of assemblies which can be initialised from a passed in list of library files
        /// </summary>
        /// <param name="AssemblyFiles">A list of assembly library files to try and open</param>
        /// <returns>A typed list of Assemblies</returns>
        public static List<Assembly> LoadAssemblies(List<string> AssemblyFiles)
        {
            List<Assembly> Assemblies = new List<Assembly>();
            foreach (string thisassemblyfile in AssemblyFiles)
                try
                {
                    Assembly thisassembly = Assembly.LoadFrom(thisassemblyfile);
                    if (thisassembly != null)
                        Assemblies.Add(thisassembly);
                }
                catch
                {
                    // Don't add the assembly to the returned collection as it couldn't be initialised
                }
            return Assemblies;
        }

        /// <summary>
        /// This loads all Function Add Ins from the Add Ins directory
        /// </summary>
        private static void GetFunctions()
        {
            // Initialise the internal functions
            functions = new GenericKeyCollection<Guid, CubedFunctionDescription>();
            
            // Find all assemblies in the given location
            List<string> assemblies = new List<string>();
            GetAssemblyFiles(addinsdirectory, ref assemblies);
            
            // Now that we have all the assemblies, we open them up and see what's inside
            List<Assembly> Assemblies = LoadAssemblies(assemblies);

            // Go through each assembly and try and find any function classes
            foreach (Assembly thisassembly in Assemblies)
            {
                // For each assembly, find all of the modules located within the assembly
                Module[] Modules = thisassembly.GetModules();
                foreach (Module thismodule in Modules)
                {
                    // For each module, locate all types which exist within the module
                    foreach (Type thistype in thismodule.GetTypes())
                    {
                        // Check whether this type has a default, parameterless constructor
                        if (thistype.GetConstructor(System.Type.EmptyTypes) != null)
                        {
                            // Now we instantiate the reflected type as a generic object
                            object thisobject = Activator.CreateInstance(thistype);

                            // Then we compare to see if the object is a descendant of CubedFunction
                            if (thisobject is CubedFunction)
                            {
                                // This is a descendant, so activate it and add it to our collection
                                CubedFunction thisfunction = (CubedFunction)thisobject;
                                CubedFunctionDescription newfunction = new CubedFunctionDescription();
                                newfunction.ID = thisfunction.ID;
                                newfunction.FunctionInstance = thisfunction;
                                newfunction.ConfigurationInstance = thisfunction.ConfigurationControl;
                                newfunction.Name = thisfunction.Name;
                                newfunction.Description = thisfunction.Description;
                                newfunction.CurrentVersion = thisfunction.CurrentVersion;
                                functions.Add(newfunction);
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// This loads all KPI Calculator Add Ins from the Add Ins directory
        /// </summary>
        private static void GetKPICalculators()
        {
            // Initialise the internal KPI calculators
            kpicalculators = new GenericKeyCollection<Guid, CubedKPIDescription>();

            // Find all assemblies in the given location
            List<string> assemblies = new List<string>();
            GetAssemblyFiles(addinsdirectory, ref assemblies);

            // Now that we have all the assemblies, we open them up and see what's inside
            List<Assembly> Assemblies = LoadAssemblies(assemblies);

            // Go through each assembly and try and find any KPI calculator classes
            foreach (Assembly thisassembly in Assemblies)
            {
                // For each assembly, find all of the modules located within the assembly
                Module[] Modules = thisassembly.GetModules();
                foreach (Module thismodule in Modules)
                {
                    // For each module, locate all types which exist within the module
                    foreach (Type thistype in thismodule.GetTypes())
                    {
                        // Check whether this type has a default, parameterless constructor
                        if (thistype.GetConstructor(System.Type.EmptyTypes) != null)
                        {
                            // Now we instantiate the reflected type as a generic object
                            object thisobject = Activator.CreateInstance(thistype);

                            // Then we compare to see if the object is a descendant of CubedKPICalculator
                            if (thisobject is CubedKPICalculator)
                            {
                                // This is a descendant, so activate it and add it to our collection
                                CubedKPICalculator thiskpi = (CubedKPICalculator)thisobject;
                                CubedKPIDescription newkpi = new CubedKPIDescription();
                                newkpi.ID = thiskpi.ID;
                                newkpi.CalculatorInstance = thiskpi;
                                newkpi.ConfigurationInstance = thiskpi.ConfigurationControl;
                                newkpi.Name = thiskpi.Name;
                                newkpi.Description = thiskpi.Description;
                                newkpi.CurrentVersion = thiskpi.CurrentVersion;
                                kpicalculators.Add(newkpi);
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// This loads all Formula Add Ins from the Add Ins directory
        /// </summary>
        private static void GetFormulas()
        {
            // Initialise the internal formulas
            formulas = new GenericKeyCollection<Guid, CubedFormulaDescription>();

            // Find all assemblies in the given location
            List<string> assemblies = new List<string>();
            GetAssemblyFiles(addinsdirectory, ref assemblies);

            // Now that we have all the assemblies, we open them up and see what's inside
            List<Assembly> Assemblies = LoadAssemblies(assemblies);

            // Go through each assembly and try and find any Formula classes
            foreach (Assembly thisassembly in Assemblies)
            {
                // For each assembly, find all of the modules located within the assembly
                Module[] Modules = thisassembly.GetModules();
                foreach (Module thismodule in Modules)
                {
                    // For each module, locate all types which exist within the module
                    foreach (Type thistype in thismodule.GetTypes())
                    {
                        // Check whether this type has a default, parameterless constructor
                        if (thistype.GetConstructor(System.Type.EmptyTypes) != null)
                        {
                            // Now we instantiate the reflected type as a generic object
                            object thisobject = Activator.CreateInstance(thistype);

                            // Then we compare to see if the object is a descendant of CubedFormula
                            if (thisobject is CubedFormula)
                            {
                                // This is a descendant, so activate it and add it to our collection
                                CubedFormula thisformula = (CubedFormula)thisobject;
                                CubedFormulaDescription newformula = new CubedFormulaDescription();
                                newformula.ID = thisformula.ID;
                                newformula.FormulaInstance = thisformula;
                                newformula.ConfigurationInstance = thisformula.ConfigurationControl;
                                newformula.Name = thisformula.Name;
                                newformula.Description = thisformula.Description;
                                newformula.CurrentVersion = thisformula.CurrentVersion;
                                formulas.Add(newformula);
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// This loads all Query Parser Add Ins from the Add Ins directory
        /// </summary>
        private static void GetParsers()
        {
            // Initialise the internal parsers
            parsers = new GenericKeyCollection<Guid, CubedQueryParserDescription>();

            // Find all assemblies in the given location
            List<string> assemblies = new List<string>();
            GetAssemblyFiles(addinsdirectory, ref assemblies);

            // Now that we have all the assemblies, we open them up and see what's inside
            List<Assembly> Assemblies = LoadAssemblies(assemblies);

            // Go through each assembly and try and find any query parser classes
            foreach (Assembly thisassembly in Assemblies)
            {
                // For each assembly, find all of the modules located within the assembly
                Module[] Modules = thisassembly.GetModules();
                foreach (Module thismodule in Modules)
                {
                    // For each module, locate all types which exist within the module
                    foreach (Type thistype in thismodule.GetTypes())
                    {
                        // Check whether this type has a default, parameterless constructor
                        if (thistype.GetConstructor(System.Type.EmptyTypes) != null)
                        {
                            // Now we instantiate the reflected type as a generic object
                            object thisobject = Activator.CreateInstance(thistype);

                            // Then we compare to see if the object is a descendant of CubedQueryParser
                            if (thisobject is CubedQueryParser)
                            {
                                // This is a descendant, so activate it and add it to our collection
                                CubedQueryParser thisparser = (CubedQueryParser)thisobject;
                                CubedQueryParserDescription newparser = new CubedQueryParserDescription();
                                newparser.ID = thisparser.ID;
                                newparser.ParserInstance = thisparser;
                                newparser.ConfigurationInstance = thisparser.ConfigurationControl;
                                newparser.Name = thisparser.Name;
                                newparser.Description = thisparser.Description;
                                newparser.CurrentVersion = thisparser.CurrentVersion;
                                parsers.Add(newparser);
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// This loads all Database Connector Add Ins from the Add Ins directory
        /// </summary>
        private static void GetConnectors()
        {
            // Initialise the internal connectors
            connectors = new GenericKeyCollection<Guid, CubedDatabaseConnectorDescription>();

            // Find all assemblies in the given location
            List<string> assemblies = new List<string>();
            GetAssemblyFiles(addinsdirectory, ref assemblies);

            // Now that we have all the assemblies, we open them up and see what's inside
            List<Assembly> Assemblies = LoadAssemblies(assemblies);

            // Go through each assembly and try and find any database connector classes
            foreach (Assembly thisassembly in Assemblies)
            {
                // For each assembly, find all of the modules located within the assembly
                Module[] Modules = thisassembly.GetModules();
                foreach (Module thismodule in Modules)
                {
                    // For each module, locate all types which exist within the module
                    foreach (Type thistype in thismodule.GetTypes())
                    {
                        // Check whether this type has a default, parameterless constructor
                        if (thistype.GetConstructor(System.Type.EmptyTypes) != null)
                        {
                            // Now we instantiate the reflected type as a generic object
                            object thisobject = Activator.CreateInstance(thistype);

                            // Then we compare to see if the object is a descendant of CubedDatabaseConnector
                            if (thisobject is CubedDatabaseConnector)
                            {
                                // This is a descendant, so activate it and add it to our collection
                                CubedDatabaseConnector thisconnector = (CubedDatabaseConnector)thisobject;
                                CubedDatabaseConnectorDescription newconnector = new CubedDatabaseConnectorDescription();
                                newconnector.ID = thisconnector.ID;
                                newconnector.ConnectorInstance = thisconnector;
                                newconnector.ConfigurationInstance = thisconnector.ConfigurationControl;
                                newconnector.Name = thisconnector.Name;
                                newconnector.Description = thisconnector.Description;
                                newconnector.CurrentVersion = thisconnector.CurrentVersion;
                                connectors.Add(newconnector);
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// This loads all Cubed Search Add Ins from the Add Ins directory
        /// </summary>
        private static void GetSearchControls()
        {
            // Initialise the internal controls
            _searchcontrols = new GenericKeyCollection<Guid, CubedSearchDescription>();

            // Find all assemblies in the given location
            List<string> assemblies = new List<string>();
            GetAssemblyFiles(addinsdirectory, ref assemblies);

            // Now that we have all the assemblies, we open them up and see what's inside
            List<Assembly> Assemblies = LoadAssemblies(assemblies);

            // Go through each assembly and try and find any search control classes
            foreach (Assembly thisassembly in Assemblies)
            {
                // For each assembly, find all of the modules located within the assembly
                Module[] Modules = thisassembly.GetModules();
                foreach (Module thismodule in Modules)
                {
                    // For each module, locate all types which exist within the module
                    foreach (Type thistype in thismodule.GetTypes())
                    {
                        // Check whether this type has a default, parameterless constructor
                        if (thistype.GetConstructor(System.Type.EmptyTypes) != null)
                        {
                            // Now we instantiate the reflected type as a generic object
                            object thisobject = Activator.CreateInstance(thistype);

                            // Then we compare to see if the object is a descendant of CubedSearch
                            if (thisobject is CubedSearch)
                            {
                                // Then we compare to see if the object is a descendant of CubedSearch
                                CubedSearch thissearch = (CubedSearch)thisobject;
                                CubedSearchDescription newsearch = new CubedSearchDescription();
                                newsearch.ID = thissearch.AddInID;
                                newsearch.SearchInstance = thissearch;
                                newsearch.ConfigurationInstance = thissearch.ConfigurationControl;
                                newsearch.Name = thissearch.Name;
                                newsearch.Description = thissearch.Description;
                                newsearch.CurrentVersion = thissearch.CurrentVersion;
                                _searchcontrols.Add(newsearch);
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// This loads all Cubed Result Add Ins from the Add Ins directory
        /// </summary>
        private static void GetResultControls()
        {
            // Initialise the internal controls
            _resultcontrols = new GenericKeyCollection<Guid, CubedResultDescription>();

            // Find all assemblies in the given location
            List<string> assemblies = new List<string>();
            GetAssemblyFiles(addinsdirectory, ref assemblies);

            // Now that we have all the assemblies, we open them up and see what's inside
            List<Assembly> Assemblies = LoadAssemblies(assemblies);

            // Go through each assembly and try and find any search result classes
            foreach (Assembly thisassembly in Assemblies)
            {
                // For each assembly, find all of the modules located within the assembly
                Module[] Modules = thisassembly.GetModules();
                foreach (Module thismodule in Modules)
                {
                    // For each module, locate all types which exist within the module
                    foreach (Type thistype in thismodule.GetTypes())
                    {
                        // Check whether this type has a default, parameterless constructor
                        if (thistype.GetConstructor(System.Type.EmptyTypes) != null)
                        {
                            // Now we instantiate the reflected type as a generic object
                            object thisobject = Activator.CreateInstance(thistype);

                            // Then we compare to see if the object is a descendant of CubedDatabaseConnector
                            if (thisobject is CubedResult)
                            {
                                // This is a descendant, so activate it and add it to our collection
                                CubedResult thisresult = (CubedResult)thisobject;
                                CubedResultDescription newresult = new CubedResultDescription();
                                newresult.ID = thisresult.AddInID;
                                newresult.ResultInstance = thisresult;
                                newresult.ConfigurationInstance = thisresult.ConfigurationControl;
                                newresult.Name = thisresult.Name;
                                newresult.Description = thisresult.Description;
                                newresult.CurrentVersion = thisresult.CurrentVersion;
                                _resultcontrols.Add(newresult);
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Refresh all Add In's by re-loading them from the Add In directories
        /// </summary>
        public static void Refresh()
        {
            GetFunctions();
            GetKPICalculators();
            GetFormulas();
            GetParsers();
            GetConnectors();
            GetSearchControls();
            GetResultControls();
        }
    }
}