/*
 * This source file (CubedDatabaseConnector.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;
using System.Reflection;

namespace Bamboo.Cubed.AddIns
{
    /// <summary>
    /// This class should be used as a base for any Cubed Database Connector Add Ins
    /// </summary>
    [Serializable]
    public abstract class CubedDatabaseConnector : IAddIn
    {
        /// <summary>
        /// Initialises a default new instance of the Database connector
        /// </summary>
        public CubedDatabaseConnector()
        {            
            InitialiseProperties(String.Empty, 1.0);
        }
        
        /// <summary>
        /// Initialises a new instance of the Cubed Database Connection based on the passed in 
        /// configuration Properties
        /// </summary>
        /// <param name="configurationstring">An encrypted serialised string containing the current configuration
        /// properties of the Add In</param>
        /// <param name="configurationversion">The version of the Add In the properties were stored for</param>
        public CubedDatabaseConnector(string configurationstring, double configurationversion)
        {            
            InitialiseProperties(configurationstring, configurationversion);
        }

        /// <summary>
        /// This should be overridden to return the unique identifier of the Add In
        /// </summary>
        [XmlElement("ID")]
        public abstract Guid ID
        {
            get;
        }

        /// <summary>
        /// This should be overridden to return the name of the Add In
        /// </summary>
        [XmlElement("Name")]
        public abstract string Name
        {
            get;
        }

        /// <summary>
        /// This returns the type of Add In, which in this case is a Database Connector
        /// </summary>
        [XmlElement("Type")]
        public AddInType Type
        {
            get { return AddInType.DatabaseConnector; }
        }

        /// <summary>
        /// This should be overridden to return the description of the Add In
        /// </summary>
        [XmlElement("Description")]
        public abstract string Description
        {
            get;
        }

        /// <summary>
        /// This should be overridden to return the current version of the Add In
        /// </summary>
        [XmlElement("CurrentVersion")]
        public abstract double CurrentVersion
        {
            get;
        }

        /// <summary>
        /// This should be overridden to return the configuration User Control of the Add In
        /// </summary>
        public abstract CubedDatabaseConnectorConfiguration ConfigurationControl
        {
            get;
        }

        /// <summary>
        /// This first decrypts the given configuration string, and then passes it to the WriteConfiguration
        /// method to set the configuration of the Add In
        /// </summary>
        /// <param name="configurationstring">A string containing an encrypted serialised string</param>
        /// <returns>True if the configuration was valid, False if there was a problem</returns>
        public bool SetConfiguration(string configurationstring)
        {
            // First we decrypt the given configuration string
            string decryptedconfiguration = EncryptionManager.DecryptString(configurationstring);

            // Then we pass the string to the abstract method for the Add In to handle, and return
            // the results
            return WriteConfiguration(decryptedconfiguration);
        }

        /// <summary>
        /// This writes the configuration specific to a particular Add In to the Add In for use
        /// </summary>
        /// <param name="configurationstring">The configuration string</param>
        /// <returns>True if successful</returns>
        public abstract bool WriteConfiguration(string configurationstring);        

        /// <summary>
        /// This method initialises the Add In by first checking the version of the Add In, and if it is 
        /// different than the version the configuration properties were set for, it calls the 
        /// update configuration method
        /// </summary>
        /// <param name="configurationstring">The encrypted serialised string containing the properties stored
        /// for the Add In</param>
        /// <param name="configurationversion">The version the properties were stored for</param>
        /// <returns>True if the properties were successfully initialised, False if there was a problem</returns>
        public bool InitialiseProperties(string configurationstring, double configurationversion)
        {
            if (configurationstring != null && configurationstring != String.Empty)
            {
                // First we check the versions to see if they match
                if (configurationversion == CurrentVersion)
                {
                    // The properties were created & stored under the current version
                    return SetConfiguration(configurationstring);
                }
                else
                {
                    // The properties were created & stored under a previous version of the Add In, so call
                    // the update method before setting the configuration 
                    string updatedconfiguration = UpdateConfiguration(configurationstring, configurationversion);

                    // Then we call the set configuration method on the updated configuration
                    return SetConfiguration(updatedconfiguration);
                }
            }
            else
            {
                // The passed in XML was empty, so we set the properties to their default values and return false                
                return false;
            }
        }

        /// <summary>
        /// This should be overriden to update the configuration from a previous version. This will automatically 
        /// be called when the version of the Add In is newer than that which was stored when the configuration was 
        /// created. This function should assume the passed in string contains a version of the properties obtained
        /// under the version given, and it should return a string containing a current version of the properties
        /// </summary>
        /// <param name="currentconfiguration">The string containing the previous configuration</param>
        /// <param name="configurationversion">The version the properties were stored under</param>
        /// <returns>A string containing an up to date version of the Properties</returns>
        public abstract string UpdateConfiguration(string currentconfiguration, double configurationversion);

        /// <summary>
        /// This represents the character used to the left of table names, field names, etc. to ensure that any spaces are 
        /// recognised by the database server
        /// </summary>
        public abstract string LeftWrappingCharacter
        {
            get;
        }

        /// <summary>
        /// This represents the character used to the right of table names, field namesm, etc. to ensure that any spaces are
        /// recognised by the database server
        /// </summary>
        public abstract string RightWrappingCharacter
        {
            get;
        }

        /// <summary>
        /// This returns the current state of the internal connection
        /// </summary>
        public abstract ConnectionState State
        {
            get;
        }

        /// <summary>
        /// This connects to the specified data source and begins the data transaction
        /// </summary>
        /// <returns>True if the datasource is successfully opened, False if there was a problem</returns>
        public abstract bool Open();

        /// <summary>
        /// This commits all changes to the underlying datasource and closes the connection
        /// </summary>
        /// <returns>True if successful, False if there was a problem</returns>
        public abstract bool Commit();

        /// <summary>
        /// This abandons all changes to the underlying datasource and closes the connection
        /// </summary>
        /// <returns>True if successful, False if there was a problem</returns>
        public abstract bool Rollback();        

        /// <summary>
        /// Creates the new database based on the current settings
        /// </summary>
        /// <param name="checkifexists">If True, the database will not be created if it already exists; If False, the
        /// database will be dropped before re-creating</param>
        /// <returns>True if successful, False if there was a problem</returns>
        public abstract bool CreateDatabase(bool checkifexists);

        /// <summary>
        /// Creates a new table with the name and structure given, as well as the supplied indexes
        /// </summary>
        /// <param name="table">The name for the new table</param>
        /// <param name="fieldnames">An array of strings containing the names of the new fields (columns) for the table</param>
        /// <param name="fieldtypes">An array of field types for the field names listed above; this array should have an
        /// identical set of indexes to the fieldnames</param>
        /// <param name="checkifexists">If True, the table will not be created (and data will remain untouched) if it 
        /// already exists; If False, the table will be dropped before re-creating</param>
        /// <param name="primarykeyfields">The name(s) of the field(s) to create as the primary key field(s) for the table</param>
        /// <param name="indexes">The collection of indexes to add to the table</param>
        /// <returns>True if successful, False if there was a problem</returns>
        public abstract bool CreateIndexedTable(string table, string[] fieldnames,
            DataCreationTypes[] fieldtypes, bool checkifexists, string[] primarykeyfields,
            DataIndexDefinition[] indexes);

        /// <summary>
        /// Creates a new table with the name and structure given
        /// </summary>
        /// <param name="table">The name for the new table</param>
        /// <param name="fieldnames">An array of strings containing the names of the new fields (columns) for the table</param>
        /// <param name="fieldtypes">An array of field types for the field names listed above; this array should have an
        /// identical set of indexes to the fieldnames</param>
        /// <param name="checkifexists">If True, the table will not be created (and data will remain untouched) if it 
        /// already exists; If False, the table will be dropped before re-creating</param>
        /// <param name="primarykeyfields">The name(s) of the field(s) to create as the primary key field(s) for the table</param>
        /// <returns>True if successful, False if there was a problem</returns>
        public abstract bool CreateTable(string table, string[] fieldnames, 
            DataCreationTypes[] fieldtypes, bool checkifexists, params string[] primarykeyfields);

        /// <summary>
        /// Renames an existing table to the new name specified, overwriting any table with the 
        /// new name if it exists already and is instructed to
        /// </summary>
        /// <param name="originalname">The original (current) name of the table</param>
        /// <param name="newname">The new name to rename</param>
        /// <param name="checkifexists">If this is false, and a table exists already with the 
        /// new name, that table will be dropped before renaming - if true and a table already exists,
        /// then the original table will remain untouched</param>
        /// <returns>True if successful, false if there was a problem</returns>
        public abstract bool RenameTable(string originalname, string newname,
            bool checkifexists);

        /// <summary>
        /// Deletes (drops) a table with the name given, if it's record matches the passed criteria of field names and values
        /// </summary>
        /// <param name="table">The name of the table to delete</param> 
        /// <returns>True if successful, False if there was a problem</returns>
        public abstract bool DeleteTable(string table);

        /// <summary>
        /// Creates a new function with the given parameters
        /// </summary>
        /// <param name="function">The name of the new function</param>
        /// <param name="functiontype">The type of function to create</param>
        /// <param name="parameternames">The list of parameters to pass to the function</param>
        /// <param name="parametertypes">The types of the parameters to pass to the function</param>
        /// <param name="code">The code block to create inside the function, if it is not a simple SELECT</param>
        /// <param name="leftwrappingcharacter">The left wrapping character used in a code block</param>
        /// <param name="rightwrappingcharacter">The right wrapping character used in a code block</param>
        /// <param name="tablegenerator">A SQL Query generator if this is a simple SELECT function</param>
        /// <param name="returntype">If this is a SCALAR value function, what type of value is returned</param>
        /// <param name="checkifexists">Whether to check if the function exists. If this is true and the 
        /// function already exists, the function will not be replaced</param>
        /// <returns>True if successful, false if there was a problem</returns>
        public abstract bool CreateFunction(string function, SQLFunctionTypes functiontype, 
            string[] parameternames, DataCreationTypes[] parametertypes, string code, 
            string leftwrappingcharacter, string rightwrappingcharacter, 
            SQLQueryGenerator tablegenerator, DataCreationTypes returntype, bool checkifexists);

        /// <summary>
        /// Deletes (drops) a function with the name given
        /// </summary>
        /// <param name="function">The name of the function to delete</param>
        /// <returns>True if deleted, false if there was a problem</returns>
        public abstract bool DeleteFunction(string function);

        /// <summary>
        /// This returns all the available tables in a database, sorted alphabetically
        /// </summary>
        /// <returns>The available tables in a database</returns>
        public abstract List<string> DatabaseTables();

        /// <summary>
        /// This returns all the fields in a particular table, sorted alphabetically
        /// </summary>
        /// <param name="table">The table to return the fields for</param>
        /// <returns>All available fields in the table</returns>
        public abstract List<string> DatabaseTableFields(string table);

        /// <summary>
        /// Inserts a new record into the given table with the values given; or updates an existing record if desired
        /// </summary>
        /// <param name="tablename">The name of the table to write the new values to</param>
        /// <param name="fieldnames">An array of names of the fields to write to</param>
        /// <param name="fieldvalues">An array of objects containing values to write to the database; this array should have
        /// an identical set of indexes to the names of the fields and their field types</param>
        /// <param name="fieldtypes">An array of field types for the fields and data listed above; this should also have
        /// an identical set of indexes</param>
        /// <param name="checkifexists">If True, and a record with an identical unique identifier already exists, then the
        /// record will be updated with the new values, rather than inserted as a new record; If False, then a new record
        /// will always be written</param>
        /// <param name="keyfields">The name of the field(s) (from the array of fieldnames above) which is recognised as the "key field"
        /// for this table (to determine if a record already exists</param>
        /// <returns>True if successful, False if there was a problem</returns>
        public abstract bool InsertRecord(string tablename, string[] fieldnames, 
            object[] fieldvalues, DataCreationTypes[] fieldtypes, bool checkifexists, 
            params string[] keyfields);

        /// <summary>
        /// Deletes a record (or series of records) from the table given, based on filter fields and values
        /// </summary>
        /// <param name="tablename">The name of the table to delete records from</param>
        /// <param name="filterfieldnames">A string array containing the names of fields to check against to ensure only
        /// fields with appropriate values are deleted</param>
        /// <param name="filterfieldvalues">An object array containing values to use as filters. This should have an identical
        /// set of indexes to the field names above</param>
        /// <returns>True if successful, False if there was a problem</returns>
        public abstract bool DeleteRecord(string tablename, string[] filterfieldnames, 
            object[] filterfieldvalues);

        /// <summary>
        /// Returns a DataSet filled with results generated by the query
        /// </summary>
        /// <param name="query">A SQL Select statement to run against the database</param>
        /// <returns>A dataset filled with the results of the sql select statement from the database, or an exception
        /// if there is a problem</returns>
        public abstract DataSet SelectRecords(string query);

        /// <summary>
        /// Returns a DataSet filled with the results generated by the query
        /// </summary>
        /// <param name="generator">A SQL Query Generator describing the query to generate</param>
        /// <returns>A dataset filled with the results of the query from the generator given, or an exception
        /// will be thrown if there is a problem</returns>
        public abstract DataSet SelectRecords(SQLQueryGenerator generator);

        /// <summary>
        /// Returns a data reader prepared and ready to start reading the records generated by the given query
        /// </summary>
        /// <param name="query">A SQL Select statement to run against the database</param>
        /// <returns>A data reader ready to start returning the records selected by the query</returns>
        public abstract IDataReader SelectReader(string query);

        /// <summary>
        /// Returns a data reader prepared and ready to start reading the records generated by the given query
        /// </summary>
        /// <param name="generator">A SQL Query Generator describing the query to generate</param>
        /// <returns>A data reader ready to start returning the records selected by the query</returns>
        public abstract IDataReader SelectReader(SQLQueryGenerator generator);

        /// <summary>
        /// Returns a data adapter which can be used to read and/or update records using the given query
        /// </summary>
        /// <param name="query">The query to execute</param>
        /// <returns>A data adapter ready for use</returns>
        public abstract IDataAdapter SelectAdapter(string query);

        /// <summary>
        /// Returns a data adapter which can be used to read and/or update records using the given query
        /// </summary>
        /// <param name="generator">The query to execute</param>
        /// <returns>A data adapter ready for use</returns>
        public abstract IDataAdapter SelectAdapter(SQLQueryGenerator generator);
    }
}
