/*
 * This source file (CubedFunction.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;
using System.Reflection;

namespace Bamboo.Cubed.AddIns
{
    /// <summary>
    /// This class should be used as a base for any Cubed Function Add Ins
    /// </summary>
    [Serializable]
    public abstract class CubedFunction : IAddIn
    {
        /// <summary>
        /// Initialises a default new instance of the Cubed Function
        /// </summary>
        public CubedFunction()
        {
            InitialiseProperties(String.Empty, 1.0);
            parameters = new GenericKeyCollection<string, CubedFunctionParameter>();
        }

        /// <summary>
        /// Initialises a new instance of the Cubed Function based on the passed in Properties
        /// </summary>
        /// <param name="configurationstring">An encrypted serialised string containing the stored Properties for the
        /// Add In</param>
        /// <param name="configurationversion">The version the configuration was created under</param>
        /// <param name="_parameters">The Parameters for the Cubed Function</param>
        public CubedFunction(string configurationstring, double configurationversion,
            GenericKeyCollection<string, CubedFunctionParameter> _parameters)
        {
            InitialiseProperties(configurationstring, configurationversion);
            parameters = _parameters;
        }

        /// <summary>
        /// This should be overridden to return the unique identifier of the Add In
        /// </summary>
        [XmlElement("ID")]
        public abstract Guid ID
        {
            get;
        }

        /// <summary>
        /// This should be overridden to return the name of the Add In
        /// </summary>
        [XmlElement("Name")]
        public abstract string Name
        {
            get;
        }

        /// <summary>
        /// This returns the type of Add In, which in this case is a Function Evaluator
        /// </summary>
        [XmlElement("Type")]
        public AddInType Type
        {
            get { return AddInType.FunctionEvaluator; }
        }

        /// <summary>
        /// This should be overridden to return the description of the Add In
        /// </summary>
        [XmlElement("Description")]
        public abstract string Description
        {
            get;
        }

        /// <summary>
        /// This should be overridden to return the current version of the Add In
        /// </summary>
        [XmlElement("CurrentVersion")]
        public abstract double CurrentVersion
        {
            get;
        }

        /// <summary>
        /// This should be overridden to return the configuration User Control of the Add In
        /// </summary>
        public abstract CubedFunctionConfiguration ConfigurationControl
        {
            get;
        }

        private GenericKeyCollection<string, CubedFunctionParameter> parameters;
        /// <summary>
        /// This contains the collection of parameters used by the function, and their values if they have
        /// been set
        /// </summary>
        [XmlElement("Parameters")]
        public GenericKeyCollection<string, CubedFunctionParameter> Parameters
        {
            get { return parameters; }
            set { parameters = value; }
        }

        /// <summary>
        /// This first decrypts the given configuration string, and then passes it to the WriteConfiguration
        /// method to set the configuration of the Add In
        /// </summary>
        /// <param name="configurationstring">A string containing an encrypted serialised string</param>
        /// <returns>True if the configuration was valid, False if there was a problem</returns>
        public bool SetConfiguration(string configurationstring)
        {
            // First we decrypt the given configuration string
            string decryptedconfiguration = EncryptionManager.DecryptString(configurationstring);

            // Then we pass the string to the abstract method for the Add In to handle, and return
            // the results
            return WriteConfiguration(decryptedconfiguration);
        }

        /// <summary>
        /// This writes the configuration specific to a particular Add In to the Add In for use
        /// </summary>
        /// <param name="configurationstring">The configuration string</param>
        /// <returns>True if successful</returns>
        public abstract bool WriteConfiguration(string configurationstring);

        /// <summary>
        /// This method initialises the Add In by first checking the version of the Add In, and if it is 
        /// different than the version the configuration properties were set for, it calls the 
        /// update configuration method
        /// </summary>
        /// <param name="configurationstring">The encrypted serialised string containing the properties stored
        /// for the Add In</param>
        /// <param name="configurationversion">The version the properties were stored for</param>
        /// <returns>True if the properties were successfully initialised, False if there was a problem</returns>
        public bool InitialiseProperties(string configurationstring, double configurationversion)
        {
            if (configurationstring != null && configurationstring != String.Empty)
            {
                // First we check the versions to see if they match
                if (configurationversion == CurrentVersion)
                {
                    // The properties were created & stored under the current version
                    return SetConfiguration(configurationstring);
                }
                else
                {
                    // The properties were created & stored under a previous version of the Add In, so call
                    // the update method before setting the configuration 
                    string updatedconfiguration = UpdateConfiguration(configurationstring, configurationversion);

                    // Then we call the set configuration method on the updated configuration
                    return SetConfiguration(updatedconfiguration);
                }
            }
            else
            {
                // The passed in XML was empty, so we set the properties to their default values and return false                
                return false;
            }
        }

        /// <summary>
        /// This should be overriden to update the configuration from a previous version. This will automatically 
        /// be called when the version of the Add In is newer than that which was stored when the configuration was 
        /// created. This function should assume the passed in string contains a version of the properties obtained
        /// under the version given, and it should return a string containing a current version of the properties
        /// </summary>
        /// <param name="currentconfiguration">The string containing the previous configuration</param>
        /// <param name="configurationversion">The version the properties were stored under</param>
        /// <returns>A string containing an up to date version of the Properties</returns>
        public abstract string UpdateConfiguration(string currentconfiguration, double configurationversion);
                
        /// <summary>
        /// The type of value this parameter expects
        /// </summary>
        [XmlElement("ResultType")]
        public Type ResultType
        {
            get { return typeof(object); }
        }

        /// <summary>
        /// This should be overridden to evaluate the result of the function, and returns the result in the format
        /// that has specified for the function
        /// </summary>
        /// <param name="cube">The Cube which is part of the query being evaluated</param>
        /// <param name="data">The Data Matrix containing the data for the cube</param>
        /// <param name="selectedmembers">The selected members for the particular cell being evaluated</param>
        /// <param name="selectedmeasure">The selected measure for this particular cell being evaluated</param>
        /// <param name="username">The name of the user evaluating the formula</param>
        /// <returns>An object containing the value returned by the function, which should be typed according
        /// to the ResultType property</returns>
        public abstract object Evaluate(Cube cube, DataMatrix data, List<Member> selectedmembers,
            Measure selectedmeasure, string username);

        /// <summary>
        /// Parses the given source code, replacing references to supported functions with the value that
        /// function would return, prior to evaluating the function
        /// </summary>
        /// <param name="cube">The Cube being queried</param>
        /// <param name="data">A Data Matrix containing all current values for the current Cube of the 
        /// given fixed dimensions</param>        
        /// <param name="source">The Source Code to parse</param>
        /// <param name="measure">The Measure that is being evaluated</param>
        /// <param name="otherdimensionmembers">The Members selected on the other dimensions which are being
        /// evaluated</param>
        /// <param name="username">The name of the user evaluating the query calling this function</param>
        public virtual void ParseMemberValues(Cube cube, DataMatrix data, Measure measure, 
            List<Member> otherdimensionmembers, string source, string username)
        {
            // Then the specific Member("") function
            if (source.Contains("Member(\""))
            {
                // There is at least one member reference, so replace all the instances with their specified
                // member values by looping through our source looking for the function
                int startindex = 0;
                string memberanddimension = String.Empty;
                while (GetNextMemberName(source, ref startindex, out memberanddimension))
                {
                    // First we need to split the member name from the dimension name
                    string[] memberparts = memberanddimension.Split('.');
                    string membername = String.Empty;
                    string dimensionname = String.Empty;
                    if (memberparts.Length > 1)
                    {
                        dimensionname = memberparts[0];
                        membername = memberparts[1];
                    }

                    // Then we need to find the dimension
                    Dimension selecteddimension = null;
                    foreach (Dimension thisdimension in cube.Dimensions)
                        if (thisdimension.Name.ToUpper().Equals(dimensionname.ToUpper()))
                            selecteddimension = thisdimension;                   
                    
                    // A match has been found and the dimension for the match found, so first lets grab
                    // the member (if it exists)
                    double membervalue = 0;
                    
                    foreach (Member thismember in selecteddimension.Members)
                        if (thismember.Name.ToUpper() == membername.ToUpper())
                        {
                            // We have found the member, lets grab its value
                            List<Member> nodemembers = otherdimensionmembers;
                            nodemembers.Add(thismember);
                            membervalue = data.GetValue(nodemembers, measure);
                        }

                    // At this point we have either returned the member value, or we have a zero value if the 
                    // member didnt exist, so either way lets replace the source code segment with the value
                    // obtained
                    source.Replace("Member(\"" + membername + "\")", membervalue.ToString());
                }
            }
        }

        private bool GetNextMemberName(string source, ref int startsearchindex, out string membername)
        {
            // Set out member name to empty in case we don't find a member function
            membername = String.Empty;

            // Grab the index of the start of the next specific member function
            int startindex = source.IndexOf("Member(\"", startsearchindex);

            // If the index is less than zero or more than the end of the source code, then no member function
            // was found
            if (startindex < 0 || startindex > source.Length)
                return false;

            // Add 9 to the start index to move it to the start of the member name
            startindex = startindex + 9;

            // If the index was found, grab the end of the member name
            int endindex = source.IndexOf("\"", startindex);

            // If the index was greater than the length of the source (or less than zero), then no ending was
            // found
            if (endindex < 0 || endindex > source.Length)
                return false;

            // At this point we must have a start and an end index, both within the current source string, so
            // let's use them to grab the member name
            membername = source.Substring(startindex, endindex - startindex);

            // Finally, we update the start search index passed in to move it to the end of the current match, 
            // and we return true to indicate a match
            startsearchindex = endindex;
            return true;
        }
    }
}
