/*
 * This source file (CubedFunctionConfiguration.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.Structures;
using System.Reflection;

namespace Bamboo.Cubed.AddIns
{
    /// <summary>
    /// This class should be used as a base for any Cubed Function Configuration Add In
    /// </summary>
    [Serializable]
    public abstract partial class CubedFunctionConfiguration : UserControl, IAddInConfiguration
    {
        /// <summary>
        /// Initialises a default new instance of the Cubed Function configuration
        /// </summary>
        public CubedFunctionConfiguration()
        {
            InitializeComponent();
            parameters = new GenericKeyCollection<string, CubedFunctionParameter>();
            InitialiseProperties(String.Empty, 1.0);
        }
        
        /// <summary>
        /// Initialises a new instance of the Cubed Function Configuration based on the passed in configuration
        /// Properties and current Parameters
        /// </summary>
        /// <param name="configurationstring">An encrypted serialised string containing the current configuration
        /// properties of the Add In</param>
        /// <param name="configurationversion">The version of the Add In the properties were stored for</param>
        /// <param name="_parameters">The Parameters collection for the Add In</param>
        public CubedFunctionConfiguration(string configurationstring, double configurationversion,
            GenericKeyCollection<string, CubedFunctionParameter> _parameters)
        {
            InitializeComponent();
            parameters = _parameters;
            InitialiseProperties(configurationstring, configurationversion);
        }

        /// <summary>
        /// This should be overridden to give the unique identifier of the Add In which this configuration
        /// control corresponds to
        /// </summary>
        [XmlElement("ID")]
        public abstract Guid ID
        {
            get;
        }

        /// <summary>
        /// This should be overridden to give the name of the Add In which this configuration control
        /// corresponds to
        /// </summary>
        [XmlElement("AddInName")]
        public abstract string AddInName
        {
            get;
        }

        /// <summary>
        /// This is the type of the Add In, which in this case is a Function Evaluator
        /// </summary>
        [XmlElement("Type")]
        public AddInType Type
        {
            get { return AddInType.FunctionEvaluator; }
        }

        /// <summary>
        /// This should be overridden to give the version of the Add In which this configuration control 
        /// corresponds to
        /// </summary>
        public abstract double CurrentVersion
        {
            get;
        }

        /// <summary>
        /// This first decrypts the given configuration string, and then passes it to the WriteConfiguration
        /// method to set the configuration of the Add In
        /// </summary>
        /// <param name="configurationstring">A string containing an encrypted serialised string</param>
        /// <returns>True if the configuration was valid, False if there was a problem</returns>
        public bool SetConfiguration(string configurationstring)
        {
            // First we decrypt the given configuration string
            string decryptedconfiguration = EncryptionManager.DecryptString(configurationstring);

            // Then we pass the string to the abstract method for the Add In to handle, and return
            // the results
            return WriteConfiguration(decryptedconfiguration);
        }

        /// <summary>
        /// This writes the configuration specific to a particular Add In to the Add In for use
        /// </summary>
        /// <param name="configurationstring">The configuration string</param>
        /// <returns>True if successful</returns>
        public abstract bool WriteConfiguration(string configurationstring);

        /// <summary>
        /// This gets the encrypted configuration string for the Add In by first calling ReadConfiguration to
        /// get the serialised configuration and then encrypting the results
        /// </summary>
        /// <returns>An encrypted serialised string containing the configuration properties</returns>
        public string GetConfiguration()
        {
            // First we call the abstract method for for the Add In to handle and return a string representing
            // its' configuration
            string configurationstring = ReadConfiguration();

            // Then we encrypt the string before returning
            return EncryptionManager.EncryptString(configurationstring);
        }

        /// <summary>
        /// This reads the given configuration string from the Add In and returns it
        /// </summary>
        /// <returns>A string representing the configuration of the Add In</returns>
        public abstract string ReadConfiguration();

        /// <summary>
        /// This method initialises the Add In by first checking the version of the Add In, and if it is 
        /// different than the version the configuration properties were set for, it calls the 
        /// update configuration method
        /// </summary>
        /// <param name="configurationstring">The encrypted serialised string containing the properties stored
        /// for the Add In</param>
        /// <param name="configurationversion">The version the properties were stored for</param>
        /// <returns>True if the properties were successfully initialised, False if there was a problem</returns>
        public bool InitialiseProperties(string configurationstring, double configurationversion)
        {
            if (configurationstring != null && configurationstring != String.Empty)
            {
                // First we check the versions to see if they match
                if (configurationversion == CurrentVersion)
                {
                    // The properties were created & stored under the current version
                    return SetConfiguration(configurationstring);
                }
                else
                {
                    // The properties were created & stored under a previous version of the Add In, so call
                    // the update method before setting the configuration 
                    string updatedconfiguration = UpdateConfiguration(configurationstring, configurationversion);

                    // Then we call the set configuration method on the updated configuration
                    return SetConfiguration(updatedconfiguration);
                }
            }
            else
            {
                // The passed in XML was empty, so we set the properties to their default values and return false                
                return false;
            }
        }

        /// <summary>
        /// This should be overriden to update the configuration from a previous version. This will automatically 
        /// be called when the version of the Add In is newer than that which was stored when the configuration was 
        /// created. This function should assume the passed in string contains a version of the properties obtained
        /// under the version given, and it should return a string containing a current version of the properties
        /// </summary>
        /// <param name="currentconfiguration">The string containing the previous configuration</param>
        /// <param name="configurationversion">The version the properties were stored under</param>
        /// <returns>A string containing an up to date version of the Properties</returns>
        public abstract string UpdateConfiguration(string currentconfiguration, double configurationversion);

        private GenericKeyCollection<string, CubedFunctionParameter> parameters;
        /// <summary>
        /// This contains the collection of parameters used by the function, so that the required parameters
        /// can be modified
        /// </summary>
        [XmlElement("Parameters")]
        public GenericKeyCollection<string, CubedFunctionParameter> Parameters
        {
            get { return parameters; }
            set { parameters = value; }
        }
    }
}