/*
 * This source file (CubedResult.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;
using System.Reflection;
using System.Web.UI.WebControls;
using System.Web.SessionState;
using System.Drawing;

namespace Bamboo.Cubed.AddIns
{
    /// <summary>
    /// This class should be used as a base for any Cubed Result Add Ins
    /// </summary>
    [Serializable]
    public abstract class CubedResult : System.Web.UI.UserControl
    {
        /// <summary>
        /// Initialises a default new instance of the Cubed Result
        /// </summary>
        public CubedResult()
        {
            InitialiseProperties(String.Empty, 1.0);
        }

        /// <summary>
        /// Initialises a new instance of the Cubed Result based on the passed in 
        /// configuration Properties
        /// </summary>
        /// <param name="configurationstring">An encrypted serialised string containing the current configuration
        /// properties of the Add In</param>
        /// <param name="configurationversion">The version of the Add In the properties were stored for</param>
        public CubedResult(string configurationstring, double configurationversion)
        {            
            InitialiseProperties(configurationstring, configurationversion);
        }

        /// <summary>
        /// This should be overridden to return the unique identifier of the Add In
        /// </summary>
        public abstract Guid AddInID
        {
            get;
        }

        /// <summary>
        /// This should be overridden to return the name of the Add In
        /// </summary>
        public abstract string Name
        {
            get;
        }

        /// <summary>
        /// This returns the type of Add In, which in this case is a Formula Evaluator
        /// </summary>
        public AddInType Type
        {
            get { return AddInType.DesktopResult; }
        }

        /// <summary>
        /// This should be overridden to return the description of the Add In
        /// </summary>
        public abstract string Description
        {
            get;
        }

        /// <summary>
        /// This should be overridden to return the current version of the Add In
        /// </summary>
        public abstract double CurrentVersion
        {
            get;
        }

        /// <summary>
        /// This is the name of the UserControl (ascx) file which contains the markup for displaying
        /// this control
        /// </summary>
        public abstract string UserControlName
        {
            get;
        }

        /// <summary>
        /// The image that will be displayed in the search results screen to represent this type of result
        /// </summary>
        public abstract string SearchLogo
        {
            get;
        }

        /// <summary>
        /// By setting this it will determine if the result will display in dashboard mode (smaller size,
        /// no configurability) or standard mode
        /// </summary>
        public abstract bool DashboardMode
        {
            get;
            set;
        }

        /// <summary>
        /// The identifier of the search this result is part of
        /// </summary>
        public abstract Guid SearchID
        {
            get;
            set;
        }

        /// <summary>
        /// The current results which will be displayed in the Cubed Result item
        /// </summary>
        public abstract object Results
        {
            get;
            set;
        }

        /// <summary>
        /// This returns null as Cubed Results do not require configuration
        /// </summary>
        public IAddInConfiguration ConfigurationControl
        {
            get
            {
                return null;
            }
        }

        /// <summary>
        /// This method does nothing as Cubed Result's do not require configuration
        /// </summary>
        /// <param name="configurationstring">A string containing the configuration</param>
        /// <returns>True as no configuration is required</returns>
        public bool WriteConfiguration(string configurationstring)
        {
            return true;
        }

        /// <summary>
        /// This method does nothing as Cubed Result's do not require configuration
        /// </summary>
        /// <param name="configurationstring">A string containing an encrypted serialised string</param>
        /// <returns>True if the configuration was valid, False if there was a problem</returns>
        public bool SetConfiguration(string configurationstring)
        {
            return true;
        }

        /// <summary>
        /// This method does nothing as no configuration or properties are required for a Cubed Result Add In
        /// </summary>
        /// <param name="configurationstring">The encrypted serialised string containing the properties stored
        /// for the Add In</param>
        /// <param name="configurationversion">The version the properties were stored for</param>
        /// <returns>True if the properties were successfully initialised, False if there was a problem</returns>
        public bool InitialiseProperties(string configurationstring, double configurationversion)
        {
            return true;
        }

        /// <summary>
        /// This does nothing as no configuration or properties are required for a Cubed Result Add In
        /// </summary>
        /// <param name="currentconfiguration">The string containing the previous configuration</param>
        /// <param name="configurationversion">The version the properties were stored under</param>
        /// <returns>A string containing an up to date version of the Properties</returns>
        public string UpdateConfiguration(string currentconfiguration, double configurationversion)
        {
            return String.Empty;
        }

        /// <summary>
        /// This saves the current display settings for the result and returns them as a string
        /// </summary>
        /// <returns>A string containing the display settings</returns>
        public abstract string SaveWebDisplaySettings();

        /// <summary>
        /// This loads the display settings for the result from the given string
        /// </summary>
        /// <param name="settings">The settings to load</param>
        /// <returns>True if the settings were loaded successfully</returns>
        public abstract bool LoadWebDisplaySettings(string settings);

        /// <summary>
        /// This returns the current result as an image file ready for use in a consolidated document
        /// </summary>
        /// <returns>A bitmap containing teh result image</returns>
        public abstract Bitmap GetResultImage();
    }
}
