/*
 * This source file (MDX.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;
using System.Data;
using System.CodeDom;
using System.Reflection;
using Bamboo.Cubed.Structures.Query;
using Bamboo.Cubed.Structures.XMLA;
using Bamboo.Cubed.Engine.QueryModel;
using System.Text.RegularExpressions;
using Bamboo.Cubed.Utilities;
using System.IO;

namespace Bamboo.Cubed.AddIns.QueryParsers
{
    /// <summary>
    /// This AddIn provides MDX Language support for the Cubed Query Parsing Engine
    /// </summary>
    [XmlRoot("MDX")]
    public class MDX : CubedQueryParser
    {
        /// <summary>
        /// Initialises a new default instance of the Add In
        /// </summary>
        public MDX()
        {
            _properties = new MDXProperties();
            base.InitialiseProperties(String.Empty, 1.0);
            InitialiseParser();
        }

        /// <summary>
        /// Initialises the Add In based on the passed in _properties
        /// </summary>
        /// <param name="configurationxml">The Base64 encoded XML string containing the configuration</param>
        /// <param name="configurationversion">The version of the configuration that was used</param>
        public MDX(string configurationxml, double configurationversion)
        {
            _properties = new MDXProperties();
            base.InitialiseProperties(configurationxml, configurationversion);
            InitialiseParser();
        }

        /// <summary>
        /// The unique identifier of the MDX Parser
        /// </summary>
        [XmlElement("ID")]
        public override Guid ID
        {
            get { return new Guid("9da6e135-6e30-4ef0-8ae8-cba0c20fa672"); }
        }

        /// <summary>
        /// The name of the MDX Parser
        /// </summary>
        [XmlElement("Name")]
        public override string Name
        {
            get
            {
                return "MDX Query Parser";
            }
        }

        /// <summary>
        /// The description of the MDX Parser
        /// </summary>
        [XmlElement("Description")]
        public override string Description
        {
            get { return "Provides the ability to parse MDX queries for exchanging data to/from the cube."; }
        }

        /// <summary>
        /// The current version of the MDX Parser
        /// </summary>
        [XmlElement("CurrentVersion")]
        public override double CurrentVersion
        {
            get { return 1.0; }
        }

        /// <summary>
        /// The configuration control for MDX Parser
        /// </summary>
        [XmlElement("ConfigurationControl")]
        public override CubedQueryParserConfiguration ConfigurationControl
        {
            get { return new MDXConfiguration(); }
        }

        private MDXProperties _properties;
        /// <summary>
        /// This represents the _properties of the Add In
        /// </summary>
        public MDXProperties Properties
        {
            get { return _properties; }
            set { _properties = value; }
        }

        /// <summary>
        /// This deserialises the current properties of the Add In from the serialised string
        /// </summary>
        /// <param name="configurationstring">The serialised XML string</param>
        /// <returns>True if the _properties were deserialised successfully</returns>
        public override bool WriteConfiguration(string configurationstring)
        {
            if (configurationstring.Length > 0)
            {
                // The configuration string is complete, so lets use it                    
                _properties.MDXSupport = (MDXSupportTypes)Enum.Parse(typeof(MDXSupportTypes), configurationstring);

                // Return true to indicate it was read correctly
                return true;
            }
            else
                return false;
        }

        /// <summary>
        /// This updates a version of the Add In from its' previous configuration to the latest
        /// </summary>
        /// <param name="currentconfiguration">The saved (previous) configuration XML</param>
        /// <param name="configurationversion">The current version of the Add In</param>
        /// <returns>The updated configuration</returns>
        public override string UpdateConfiguration(string currentconfiguration, double configurationversion)
        {
            // As this is the first release, there should be no need to update
            return currentconfiguration;
        }

        /// <summary>
        /// The Mdx Tokens are used to parse MDX strings with possible operators. 
        /// </summary>
        private Token[] MDXOperatorTokens;

        /// <summary>
        /// The Statements tokens are used for Splitting the MDX expression with possible statements like select, from or where
        /// </summary>
        private Token[] MDXStatementTokens;

        /// <summary>
        /// Function tokens are used for parse MDX functions 
        /// </summary>
        private Token[] MDXFunctionTokens;

        /// <summary>
        /// Initialises the parser for parsing
        /// </summary>
        private void InitialiseParser()
        {
            MDXOperatorTokens = new Token[]
                {
                    new UnaryOperatorToken ("[Ee][Mm][Pp][Tt][Yy]", null), // NON EMPTY
                    new UnaryOperatorToken ("[Nn][Oo][Nn]", null), // NON
                    new UnaryOperatorToken ("[Nn][Oo][Tt]", null), // NOT
                    new OperatorToken("[Aa][Nn][Dd]", null), // AND
                    new OperatorToken("[Aa][Ss]", null),// AS
                    new OperatorToken("[Ee][Ll][Ss][Ee]", null), // ELSE
                };

            MDXStatementTokens = new Token[]
                    {
                        new StatementToken("[Uu][Pp][Dd][Aa][Tt][Ee]\\s", new TokenSplitter(UpdateSplitter)), //UPDATE
                        new StatementToken("[Ww][Ii][Tt][Hh]\\s", new TokenSplitter(WithSplitter)), // WITH        
                        new StatementToken("[Ss][Ee][Ll][Ee][Cc][Tt]\\s", new TokenSplitter(SelectSplitter)), 
                        new StatementToken("[Cc][Aa][Ss][Ee]\\s", null),   // CASE 
                        new StatementToken("[Ff][Rr][Oo][Mm]\\s", new TokenSplitter(FromSplitter)), // FROM
                        new StatementToken("[Ww][Hh][Ee][Rr][Ee]\\s", new TokenSplitter(WhereSplitter)) // WHERE
                    };

            MDXFunctionTokens = new FunctionToken[]
                   { 
                       new FunctionToken("\\b[Cc][Rr][Oo][Ss][Ss][Jj][Oo][Ii][Nn]\\b",
                                                             new TokenSplitter(TwoSetFunctionSplitter),
                                                             TokenTypes.FUNCTION,
                                                             new TokenParser(parseSetsFunction),
                                                             new string[] { "MdmSet","MdmSet" },"MdmSet"),
                       new FunctionToken("\\b[Nn][Oo][Nn][Ee][Mm][Pp][Tt][Yy][Cc][Rr][Oo][Ss][Ss][Jj][Oo][Ii][Nn]\\b",
                                                             new TokenSplitter(TwoSetFunctionSplitter),
                                                             TokenTypes.FUNCTION,
                                                             new TokenParser(parseSetsFunction),
                                                             new string[] { "MdmSet","MdmSet" },"MdmSet"),
                       new FunctionToken("\\b[Ii][Nn][Tt][Ee][Rr][Ss][Ee][Cc][Tt]\\b",
                                                             new TokenSplitter(TwoSetFunctionSplitter),
                                                             TokenTypes.FUNCTION,
                                                             new TokenParser(parseSetsFunction),
                                                             new string[] { "MdmSet","MdmSet" },"MdmSet"),
                       new FunctionToken("\\b[Hh][Ee][Aa][Dd]\\b",
                                                             new TokenSplitter(SetFunctionSplitter),
                                                             TokenTypes.FUNCTION,
                                                             new TokenParser(parseSetsFunction),
                                                             new string[] { "MdmSet" },"MdmSet"),
                       new FunctionToken("\\b[Hh][Ii][Ee][Rr][Aa][Rr][Cc][Hh][Ii][Zz][Ee]\\b",
                                                             new TokenSplitter(SetFunctionSplitter),
                                                             TokenTypes.FUNCTION,
                                                             new TokenParser(parseSetsFunction),
                                                             new string[] { "MdmSet" },"MdmSet"),
                       new FunctionToken("\\b[Ss][Uu][Mm]\\b",
                                                             new TokenSplitter(SetFunctionSplitter),
                                                             TokenTypes.FUNCTION,
                                                             new TokenParser(parseSetsFunction),
                                                             new string[] { "MdmSet" },"MdmSet"),
                       new FunctionToken("\\b[Oo][Rr][Dd][Ee][Rr]\\b",
                                                             new TokenSplitter(SetFunctionSplitter),
                                                             TokenTypes.FUNCTION,
                                                             new TokenParser(parseSetsFunction),
                                                             new string[] { "MdmSet" },"MdmSet"),
                       new FunctionToken("\\b[Ff][Ii][Ll][Tt][Ee][Rr]\\b",
                                                             new TokenSplitter(FilterFunctionSplitter),
                                                             TokenTypes.FUNCTION,
                                                             new TokenParser(parseFilterFunction),
                                                             new string[] { "MdmSet" , "MdmOperation" },"MdmSet"),
                       new FunctionToken("\\b[Dd][Ii][Ss][Tt][Ii][Nn][Cc][Tt]\\b",
                                                             new TokenSplitter(SetFunctionSplitter),
                                                             TokenTypes.FUNCTION,
                                                             new TokenParser(parseSetsFunction),
                                                             new string[] { "MdmSet"},"MdmSet")
                   };
        }

        /// <summary>
        /// Parses a given MDX command text and returns a valid MDM Statement
        /// </summary>
        /// <param name="commandtext">The command text to parse</param>
        /// <returns>An MDM Statement ready for passing to the second level query engine</returns>
        public override MDMStatement Parse(string commandtext)
        {
            // First we initialise a query expression based on the passed in command text
            QueryExpression Expression = new QueryExpression(commandtext);

            // Call the Apply the evaluation and transformation into the Expression ArrayList and then call to the parse method.
            Normalize(ref Expression);

            // Check if it is a Query, an update or a ALTER Statement.
            MDMStatementTypes statementtype = getStatementType(Expression);
            if (statementtype == MDMStatementTypes.SELECT)
                return parseQuery(Evaluate(Expression));
            else
                if (statementtype == MDMStatementTypes.UPDATE)
                    return parseUpdate(Evaluate(Expression));
            return parseQuery(Evaluate(Expression));
        }

        /// <summary>
        /// Returns a well formed MDX command text string based on the given MDM Statement
        /// </summary>
        /// <param name="statement">An MDM Statement to return the MDX of</param>
        /// <returns>A string containing a well formed MDX command</returns>
        public override string Unparse(MDMStatement statement)
        {
            if (statement.StatementType == MDMStatementTypes.SELECT)
                return UnparseQuery((MDMQuery)statement);
            else
                return UnparseUpdate((MDMUpdate)statement);
        }

        /// <summary>
        /// This validates a given Query Expression to ensure it is well formed
        /// </summary>
        /// <param name="Expression">An expression to validate</param>
        /// <returns>True if the expression is well formed, false if not</returns>
        public override bool Validate(QueryExpression Expression)
        {
            return true;
        }

        /// <summary>
        /// Normalises a given query expression
        /// </summary>
        /// <param name="Expression">The expression to normalize</param>
        /// <returns>True if normalized successfully, false if there was a problem</returns>
        public override bool Normalize(ref QueryExpression Expression)
        {
            // Set up the comments regular expressions
            Regex comments = new Regex("[\\/][\\/].*", RegexOptions.Multiline);
            Expression.Expression = comments.Replace(Expression.Expression, "");

            comments = new Regex("[\\-][\\-].*", RegexOptions.Multiline);
            Expression.Expression = comments.Replace(Expression.Expression, "");

            comments = new Regex("[\\/][\\*].*[\\*][\\/]", RegexOptions.Singleline);
            Expression.Expression = comments.Replace(Expression.Expression, "");

            // Line break replacement expressions
            Expression.Expression = Expression.Expression.Replace(System.Environment.NewLine, " ");

            // Update CUBE WORD elimination
            Expression.Expression = Regex.Replace(Expression.Expression, "[Uu][Pp][Dd][Aa][Tt][Ee]\\s[Cc][Uu][Bb][Ee]\\s", "UPDATE ");

            // Detect whitespaces and normalize them into one
            Expression.Expression = Regex.Replace(Expression.Expression, "\\s\\s*", " ");

            // Add whitespaces befor common operations. (: , + * / - ( ) )
            Regex wr = new Regex("[^\\s][/*-+:,'\\(\\)]");
            int wi = 1;
            foreach (Match wm in wr.Matches(Expression.Expression))
                Expression.Expression = Expression.Expression.Insert(wm.Index + wi++, " ");

            // Add whitespaces after common operations. (: , + * / - )
            Regex rw = new Regex("[/*-+:,'\\(\\)][^\\s]");
            int iw = 1;
            foreach (Match mw in rw.Matches(Expression.Expression))
                Expression.Expression = Expression.Expression.Insert(mw.Index + iw++, " ");

            // Replace whitespaces inside [ ] 
            // The white space inside bracks will be replaced by @whitespace@ as a convention. 
            // Denormalization process has to be done in order to get the correct member name
            Regex rwi = new Regex("\\[(?>[^\\[\\]]+|\\[(?<number>)|\\](?<-number>))*(?(number)(?!))\\]");
            foreach (Match mrwi in rwi.Matches(Expression.Expression))
                Expression.Expression = Expression.Expression.Replace(mrwi.Value, mrwi.Value.Replace(" ", "@whitespace@"));

            Expression.Expression = Expression.Expression.Replace(" : ", " Range ");

            // Replace * for Cross join in case it is valid.-  } * {
            // Search for all sets in the SELECT STATEMENT AND THE SLICER
            Regex sets = new Regex("\\{(?>[^\\{\\}]+|\\{(?<number>)|\\}(?<-number>))*(?(number)(?!))\\}");
            List<string[]> crossjoins = new List<string[]>();
            foreach (Match msets in sets.Matches(Expression.Expression))
            {
                if (Expression.Expression.IndexOf(msets.Value + " * {") > -1)
                {
                    crossjoins.Add(new string[] { msets.Value, msets.NextMatch().Value });
                }
            }
            int nested = 1;
            for (int c = crossjoins.Count - 1; c >= 0; c--)
            {
                //for the last case.....
                if (c == crossjoins.Count - 1)
                {
                    Expression.Expression = Expression.Expression.Replace(((string[])crossjoins[c])[0] + " * " + ((string[])crossjoins[c])[1],
                                                                          "CROSSJOIN ( " + ((string[])crossjoins[c])[0] + " , " + ((string[])crossjoins[c])[1] + " ) ");
                }
                else
                {

                    //for a case of nested crossjoin
                    if (((string[])crossjoins[c])[1] == ((string[])crossjoins[c + 1])[0])
                    {
                        Expression.Expression = Expression.Expression.Replace(((string[])crossjoins[c])[0] + " * ",
                                                                              "CROSSJOIN ( " + ((string[])crossjoins[c])[0] + " , ");
                        string replacement = ((string[])crossjoins[c + 1])[1] + " ) ) ";
                        for (int r = 0; r < nested; r++)
                            replacement += ") ";

                        Expression.Expression = Expression.Expression.Replace(((string[])crossjoins[c + 1])[1] + " ) ", replacement);
                        nested++;
                    }
                    else
                    {
                        Expression.Expression = Expression.Expression.Replace(((string[])crossjoins[c])[0] + " * " + ((string[])crossjoins[c])[1],
                                                                          "CROSSJOIN ( " + ((string[])crossjoins[c])[0] + " , " + ((string[])crossjoins[c])[1] + " ) ");
                        nested = 1;
                    }
                }
            }
            return true;
        }

        /// <summary>
        /// Returns a list of command segments from a given expression
        /// </summary>
        /// <param name="Expression">The expression to return segments for</param>
        /// <returns>A string list containing all segments of the query</returns>
        public override List<string> Evaluate(QueryExpression Expression)
        {
            List<string> tmpList = new List<string>();
            // Splitting statements...
            string MDX = "";
            foreach (StatementToken st in MDXStatementTokens)
            {
                Match m = Regex.Match(Expression.Expression, st.TokenExpression);
                if (m.Value != null && m.Value != "")
                {
                    MDX += st.ProcessToken(Expression.Expression, m.Value);
                }
            }
            string[] results = MDX.Split(new string[] { " " } , StringSplitOptions.RemoveEmptyEntries);
            foreach (string result in results)
                tmpList.Add(result);
            return tmpList;
        }

        #region Internal Unparsing Functions

        /// <summary>
        /// Unparses an update query
        /// </summary>
        /// <param name="update">The update query</param>
        /// <returns>An MDX string</returns>
        private string UnparseUpdate(MDMUpdate update)
        {
            StringBuilder mdxBuilder = new StringBuilder();

            // Append the  mdx select keyword
            mdxBuilder.AppendLine("UPDATE");

            // Append the Cube name
            mdxBuilder.AppendLine(update.Cube);
            mdxBuilder.AppendLine("SET");
            unparseMDMTuple(update.Tuple, ref mdxBuilder);
            mdxBuilder.Remove(mdxBuilder.Length - 1, 1);
            mdxBuilder.AppendLine(" = ");
            mdxBuilder.AppendLine(update.NewValue);

            return mdxBuilder.ToString();
        }

        /// <summary>
        /// This method generates an MDX expression representing a given query
        /// </summary>
        /// <param name="query">The query to unparse.</param>
        /// <returns>A string that could generate the given query </returns>
        private String UnparseQuery(MDMQuery query)
        {
            // Create the mdxBuilder wich would built the output 
            StringBuilder mdxBuilder = new StringBuilder();

            // If has a Member Set...
            bool writeWith = true; // if has more than one Calculated Member or a CM and a Named Set, WITH is written only once

            // If has a Named Set ...
            if (query.NamedSets != null)
            {
                if (writeWith)
                {
                    mdxBuilder.AppendLine("WITH");
                    writeWith = false;
                }
                foreach (MDMNamedSet namedset in query.NamedSets)
                {
                    // Calling to the function that unparse a MDMNamedSet into an MDX expression
                    unparseMDMNamedSet(namedset, ref mdxBuilder);
                }
            }

            if (query.CalculatedMembers != null)
            {
                if (writeWith)
                {
                    mdxBuilder.AppendLine("WITH");
                    writeWith = false;
                }
                foreach (MDMCalculatedMember member in query.CalculatedMembers)
                {
                    // Calling to the function that unparse a MDMCalculatedMember into an MDX expression
                    unparseMDMCalculatedMember(member, ref mdxBuilder);
                }
            }
            
            // If has an Axis...
            if (query.Axes != null)
            {
                // Append the  mdx select keyword
                mdxBuilder.AppendLine("SELECT");
                // Iterate over the QueryAxes (in the Select statement)
                foreach (MDMQueryAxis axis in query.Axes)
                {
                    // Call to the function that unparse a MDMQueryAxes into an MDX expression
                    unparseMDMQueryAxis(axis, ref mdxBuilder);
                    // write the ON {axisname} function 
                    mdxBuilder.Append("ON " + axis.AxisName + ", ");
                }
                // Remove the last character from the output
                mdxBuilder.Remove(mdxBuilder.Length - 2, 2);

                // Insert a blank line
                mdxBuilder.AppendLine();

                // Append the mdx FROM keyword
                mdxBuilder.AppendLine("FROM");

                // Append the Cube name
                mdxBuilder.AppendLine(query.Cube);

                // Check if SlicerAxis is defined
                if (query.SlicerAxis != null)
                {
                    mdxBuilder.AppendLine("WHERE");
                
                    // Call to the function that unparse a MDMQueryAxes ( the slicer in this case) into an MDX expression
                    unparseMDMQueryAxis(query.SlicerAxis, ref mdxBuilder);
                }
            }
            // return the output
            return mdxBuilder.ToString();
        }

        /// <summary>
        /// This method generates an MDX expression that generates a given MDMCalculatedMember
        /// </summary>
        /// <param name="calculatedmember">The MDMCalculatedMember to unparse.</param>
        /// <param name="mdxBuilder">The StringBuilder that is constructing the output passed by reference</param>
        private void unparseMDMCalculatedMember(MDMCalculatedMember calculatedmember, ref StringBuilder mdxBuilder)
        {
            mdxBuilder.AppendLine("MEMBER ");
            unparseMDMMember(calculatedmember.Member, ref mdxBuilder, false);
            mdxBuilder.Remove(mdxBuilder.Length - 1, 1);
            mdxBuilder.AppendLine(" AS ");
            mdxBuilder.Append("'");
            if (calculatedmember.Operations != null && calculatedmember.Operations.Count > 0)
            {
                foreach (string operationmember in calculatedmember.Operations)
                {
                    if (operationmember.Length > 1)
                    {
                        MDMMember newmember = new MDMMember();
                        newmember.MemberName = operationmember;
                        unparseMDMMember(newmember, ref mdxBuilder, false);
                        mdxBuilder.Remove(mdxBuilder.Length - 1, 1);
                    }
                    else
                        mdxBuilder.Append(" " + operationmember + " ");
                }
            }
            else
            {
                if (calculatedmember.Function != null)
                {
                    unparseMDMSetFunction(calculatedmember.Function, ref mdxBuilder);
                }
            }
            mdxBuilder.Append("',");
            mdxBuilder.AppendLine(" SOLVE ORDER = " + calculatedmember.SolveOrder.ToString());
        }

        /// <summary>
        /// This method generates an MDX expression that generates a given MDMNamedSet
        /// </summary>
        /// <param name="namedset">The MDMNamedSet to unparse.</param>
        /// <param name="mdxBuilder">The StringBuilder that is constructing the output passed by reference</param>
        private void unparseMDMNamedSet(MDMNamedSet namedset, ref StringBuilder mdxBuilder)
        {
            mdxBuilder.AppendLine("SET ");
            mdxBuilder.AppendLine(" [" + namedset.Name + "] ");
            mdxBuilder.AppendLine("AS '");
            if (namedset.Function != null)
                unparseMDMSetFunction(namedset.Function, ref mdxBuilder);
            else
                unparseMDMSet(namedset.Set, ref mdxBuilder);
            mdxBuilder.AppendLine("'");
        }

        /// <summary>
        /// This method generates an MDX expression that generates a given MDMQueryAxes
        /// </summary>
        /// <param name="axis">The MDMQueryAxis to unparse.</param>
        /// <param name="mdxBuilder">The StringBuilder that is constructing the output passed by reference</param>
        private void unparseMDMQueryAxis(MDMQueryAxis axis, ref StringBuilder mdxBuilder)
        {
            // if the given axis has a Set it doesn't has a Set function.
            if (axis.Set != null)
            {
                // Calling the function that generates the MDX output for a given set
                unparseMDMSet(axis.Set, ref mdxBuilder);
            }
            else
            {
                unparseMDMSetFunction(axis.SetFunction, ref mdxBuilder);
            }

        }

        /// <summary>
        /// This method generates an MDX expression that generates a given MDMFunction
        /// </summary>
        /// <param name="function">The MDMFunction to unparse.</param>
        /// <param name="mdxBuilder">The StringBuilder that is constructing the output passed by reference</param>
        private void unparseMDMSetFunction(MDMFunction function, ref StringBuilder mdxBuilder)
        {
            mdxBuilder.AppendLine();
            // Appending the function name and the starting bracket.
            mdxBuilder.AppendLine(function.Name);
            mdxBuilder.AppendLine("(");

            bool firstprm = true;

            // checking for the parameters 
            if (function.Parameters != null)
            {
                // Iterating over the parameters of the function
                foreach (MDMFunctionParameter prm in function.Parameters)
                {
                    if (!firstprm)
                        mdxBuilder.Append(",");

                    // If is a MDMSet unparse the set
                    if (prm.Param is MDMSet)
                    {
                        unparseMDMSet((MDMSet)prm.Param, ref mdxBuilder);
                    }

                    // If is a MDMMember unparse the member
                    else if (prm.Param is MDMMember)
                    {
                        unparseMDMMember((MDMMember)prm.Param, ref mdxBuilder, false);
                    }

                    // If is a MDMTuple unparse the tuple
                    else if (prm.Param is MDMTuple)
                    {
                        unparseMDMTuple((MDMTuple)prm.Param, ref mdxBuilder);
                    }

                    // If is a MDMRange unparse the range
                    else if (prm.Param is MDMRange)
                    {
                        unparseMDMMember(((MDMRange)prm.Param).StartMember, ref mdxBuilder, false);
                        mdxBuilder.Remove(mdxBuilder.Length - 1, 1);
                        mdxBuilder.Append(" : ");
                        unparseMDMMember(((MDMRange)prm.Param).EndMember, ref mdxBuilder, false);
                    }

                    // If it is a Level, unparse the level
                    else if (prm.Param is MDMLevel)
                    {
                        unparseMDMLevel((MDMLevel)prm.Param, ref mdxBuilder);
                    }

                    // If it is a Function, call recursively this method
                    else if (prm.Param is MDMFunction)
                    {
                        unparseMDMSetFunction((MDMFunction)prm.Param, ref mdxBuilder);
                    }

                    else if (prm.Param is MdmOperationMember[])
                    {
                        foreach (MdmOperationMember om in (MdmOperationMember[])prm.Param)
                        {
                            if (om.Member is MDMMember)
                            {
                                unparseMDMMember((MDMMember)om.Member, ref mdxBuilder, false);
                                mdxBuilder.Remove(mdxBuilder.Length - 1, 1);
                            }
                            else
                                mdxBuilder.Append(" " + ((MdmOperationToken)om.Member).Token + " ");
                        }
                    }

                    /* CHECK IF IT IS AN ENUM, FLAG */
                    // The ckeck here is with the base type because the Type should be a defined function flag enum.
                    if (prm.ParamType != null)
                    {
                        if (prm.ParamType.BaseType == typeof(System.Enum))
                        {
                            // Appending the enum element name
                            mdxBuilder.AppendLine(Enum.GetName(prm.ParamType, prm.Param));
                        }
                    }
                    firstprm = false;
                }
            }
            mdxBuilder.AppendLine(")");
        }

        /// <summary>
        /// This method generates an MDX expression that generates a given MDMSet
        /// </summary>
        /// <param name="set">The MDMSet to unparse.</param>
        /// <param name="mdxBuilder">The StringBuilder that is constructing the output passed by reference</param>
        private void unparseMDMSet(MDMSet set, ref StringBuilder mdxBuilder)
        {
            if (set.Tuples != null)
            {
                mdxBuilder.AppendLine();
                mdxBuilder.AppendLine("{");
                foreach (MDMTuple tuple in set.Tuples)
                {
                    unparseMDMTuple(tuple, ref mdxBuilder);
                }
                mdxBuilder.Remove(mdxBuilder.Length - 1, 1);
                mdxBuilder.AppendLine("}");
            }
        }

        /// <summary>
        /// This method generates an MDX expression that generates a given MDMTuple
        /// </summary>
        /// <param name="tuple">The MDMTuple to unparse.</param>
        /// <param name="mdxBuilder">The StringBuilder that is constructing the output passed by reference</param>
        private void unparseMDMTuple(MDMTuple tuple, ref StringBuilder mdxBuilder)
        {
            mdxBuilder.Append("(");
            if (tuple.Members != null)
            {
                foreach (MDMMember mbr in tuple.Members)
                {
                    unparseMDMMember(mbr, ref mdxBuilder, false);
                }
            }
            if (tuple.Ranges != null)
            {
                foreach (MDMRange rng in tuple.Ranges)
                {
                    unparseMDMMember(rng.StartMember, ref mdxBuilder, false);
                    mdxBuilder.Remove(mdxBuilder.Length - 1, 1);
                    mdxBuilder.Append(" : ");
                    unparseMDMMember(rng.EndMember, ref mdxBuilder, false);
                }
                mdxBuilder.Remove(mdxBuilder.Length - 1, 1);
            }
            else
            {
                mdxBuilder.Remove(mdxBuilder.Length - 1, 1);
            }
            mdxBuilder.Append("),");

        }

        /// <summary>
        /// This method generates an MDX expression that generates a given MDMMember
        /// </summary>
        /// <param name="mbr">The MDMMember to unparse.</param>
        /// <param name="mdxBuilder">The StringBuilder that is constructing the output passed by reference</param>
        /// <param name="isAncestor">The First call of the method or a second recursive call...</param>
        private void unparseMDMMember(MDMMember mbr, ref StringBuilder mdxBuilder, bool isAncestor)
        {
            if (!isAncestor)
                mdxBuilder.Append("[" + mbr.MemberDimension.Replace("@whitespace@", " ") + "].");

            if (isAncestor)
            {
                if (mbr.IsId && mbr.MemberName.Substring(0, 1) != "&")
                    mdxBuilder.Append("&");
                mdxBuilder.Append("[" + mbr.MemberName.Replace("@whitespace@", " ") + "].");
            }

            if (mbr.Ancestor != null)
            {
                unparseMDMMember(mbr.Ancestor, ref mdxBuilder, true);
            }

            if (!isAncestor)
            {
                if (mbr.IsId && mbr.MemberName.Substring(0, 1) != "&")
                    mdxBuilder.Append("&");
                mdxBuilder.Append("[" + mbr.MemberName.Replace("@whitespace@", " ") + "]");
                if (mbr.Relationship != MemberRelationship.Self)
                {
                    mdxBuilder.Append("." + Enum.GetName(typeof(MemberRelationship), mbr.Relationship));
                }
                mdxBuilder.Append(",");
            }
        }

        /// <summary>
        /// Unparse a MDMMember and return the resultant MDX string for the member
        /// </summary>
        /// <param name="mbr">The member to unparse</param>
        /// <returns>An MDX string</returns>
        private string unparseMDMMember(MDMMember mbr)
        {
            StringBuilder b = new StringBuilder();
            unparseMDMMember(mbr, ref b, false);
            return b.ToString();
        }

        /// <summary>
        /// This method generates an MDX expression that generates a given MDMLevel
        /// </summary>
        /// <param name="lvl">The MDMLevel to unparse.</param>
        /// <param name="mdxBuilder">The StringBuilder that is constructing the output passed by reference</param>        
        private void unparseMDMLevel(MDMLevel lvl, ref StringBuilder mdxBuilder)
        {
            mdxBuilder.Append("[" + lvl.LevelDimension + "].");
            mdxBuilder.Append("[" + lvl.LevelName + "],");
        }

        #endregion

        #region Internal Parsing Functions

        /// <summary>
        /// Adds an error to the error collection
        /// </summary>
        /// <param name="error">The error to add</param>
        private void AddToErrorCollection(string error)
        {
            ErrorCollection.Add(error);
        }

        /// <summary>
        ///  Internal use for parsing an Operation
        /// </summary>
        /// <param name="expression">The expression to parse after the evaluation process</param>
        /// <param name="indexer">The counter of the expression. The current position being parsed</param>
        /// <param name="QueryInfo">The Query Info to store parsed values</param>
        private MdmOperationMember[] parseOperation(ref List<string> expression, ref int indexer, ref StatementSummarizedInfoHelper QueryInfo)
        {
            List<MdmOperationMember> tmpOperationMembers = new List<MdmOperationMember>();

            while ((string)expression[++indexer] != "MdmEndOperation")
            {
                MdmOperationMember tmpMember = new MdmOperationMember();
                if (((string)expression[indexer]).Trim() == "OperationToken" || ((string)expression[indexer]).Trim() == "OperationNumber")
                {
                    MdmOperationToken t = new MdmOperationToken();
                    t.Token = ((string)expression[++indexer]).Trim();
                    tmpMember.Member = t;
                }
                else
                {
                    tmpMember.Member = parseMember(ref expression, ref indexer, ref QueryInfo);
                }
                tmpOperationMembers.Add(tmpMember);

            }
            return (MdmOperationMember[])tmpOperationMembers.ToArray();
        }

        /// <summary>
        ///  Internal use for parsing the filter function
        /// </summary>
        /// <param name="expression">The expression to parse after the evaluation process</param>
        /// <param name="indexer">The counter of the expression. The current position being parsed</param>
        /// <param name="functionName">The name of the function</param>
        /// <param name="QueryInfo">The Query Info to store parsed values</param> 
        private MDMFunctionHelper parseFilterFunction(ref List<string> expression, ref int indexer, string functionName, ref StatementSummarizedInfoHelper QueryInfo)
        {
            MDMFunctionHelper f = parseSetsFunction(ref expression, ref indexer, functionName, ref QueryInfo);

            if ((string)expression[++indexer] == "MdmStartOperation")
            {
                f.addParameter(typeof(MdmOperationMember[])).Param = parseOperation(ref expression, ref indexer, ref QueryInfo);
            }

            indexer--;
            return f;
        }

        /// <summary>
        ///  Internal use for parsing the an expression
        /// </summary>
        /// <param name="expression">The expression to parse after the evaluation process</param>
        /// <param name="indexer">The counter of the expression. The current position being parsed</param>
        /// <param name="functionName">The name of the function</param>
        /// <param name="QueryInfo">The Query Info to store parsed values</param> 
        private MDMFunctionHelper parseSetsFunction(ref List<string> expression, ref int indexer, string functionName, ref StatementSummarizedInfoHelper QueryInfo)
        {
            MDMFunctionHelper f = new MDMFunctionHelper();
            f.Name = functionName;
            f.ReturnTypeName = "Structures.MDMSet";

            // The function can have two sets, one set and one function or two functions...
            indexer++;

            // An axis can have a MdmSet or a Function in wich case calls the function part of the if.
            while ((string)expression[indexer] == "MdmSet")
            {
                f.addParameter(typeof(MDMSet)).Param = parseSet(ref expression, ref indexer, ref QueryInfo);
                indexer++;

            }
            while ((string)expression[indexer] == "MdmFunction")
            {
                indexer++;
                foreach (FunctionToken ft in MDXFunctionTokens)
                {
                    Regex axisre = new Regex(ft.TokenExpression);
                    foreach (Match m in axisre.Matches((string)expression[indexer]))
                    {
                        MDMFunction tmpFunction = ft.ParseToken(ref expression, ref indexer, m.Value.ToUpper(), ref QueryInfo);
                        f.addParameter(typeof(MDMFunction)).Param = tmpFunction;
                        QueryInfo.addFunction(tmpFunction);
                    }
                }
                indexer++;
            }

            indexer--;
            return f;
        }

        /// <summary>
        /// Internal use for parsing the member part of a query. It is called when parsing axis,  slicer.
        /// </summary>
        /// <param name="expression">The expression to parse after the evaluation process</param>
        /// <param name="i">The counter of the expression. The current position being parsed</param>
        /// <param name="QueryInfo">The Query Info to store parsed values</param>
        private MDMMemberHelper parseMember(ref List<string> expression, ref int i, ref StatementSummarizedInfoHelper QueryInfo)
        {
            string dimension = "";
            string value = "";
            int depth = 0;

            MDMMemberHelper mh = new MDMMemberHelper();
            //int anc = 0;
            if ((string)expression[i + 1] == "MemberRelationship")
            {
                i++;
                string relationship = ((string)expression[++i]).Replace("@whitespace@", " ").Replace("@WHITESPACE@", " ").Trim().ToUpper();
                switch (relationship)
                {
                    case "DESCENDANTS":
                        {
                            mh.Relationship = MemberRelationship.Descendants;
                            break;
                        }
                    case "SIBLINGS":
                        {
                            mh.Relationship = MemberRelationship.Siblings;
                            break;
                        }
                    case "ANCESTORS":
                        {
                            mh.Relationship = MemberRelationship.Ancestors;
                            break;
                        }
                    case "CHILDREN":
                        {
                            mh.Relationship = MemberRelationship.Children;
                            break;
                        }
                    case "COUSINS":
                        {
                            mh.Relationship = MemberRelationship.Cousins;
                            break;
                        }
                    case "PARENTS":
                        {
                            mh.Relationship = MemberRelationship.Parents;
                            break;
                        }
                    case "CURRENTMEMBER":
                        {
                            mh.Relationship = MemberRelationship.CurrentMembers;
                            break;
                        }
                    case "CURRENTMEMBERS":
                        {
                            mh.Relationship = MemberRelationship.CurrentMembers;
                            break;
                        }
                    case "TRANSACTIONS":
                        {
                            mh.Relationship = MemberRelationship.Transactions;
                            break;
                        }
                    default:
                        {
                            mh.Relationship = MemberRelationship.Self;
                            break;
                        }
                }
            }           

            if ((string)expression[++i] == "MemberDimension")
                mh.MemberDimension = ((string)expression[++i]).Replace("@whitespace@", " ").Replace("@WHITESPACE@", " ");
            if ((string)expression[++i] == "MemberDepth")
                mh.MemberDepth = (int.Parse(expression[++i].ToString()));
            if ((string)expression[++i] == "MemberValue")
                mh.MemberName = ((string)expression[++i]).Replace("@whitespace@", " ").Replace("@WHITESPACE@", " ");

            if (mh.MemberName.Substring(0, 1) == "&")
            {
                mh.IsId = true;
                mh.MemberName = mh.MemberName.Substring(1);
            }

            // Check if it is an all Member Member.
            if (mh.MemberName.ToUpper().Trim() == "ALL MEMBERS" || mh.MemberName.ToUpper().Trim() == "ALLMEMBERS" || mh.MemberName.Trim() == "*"
                || mh.MemberName.ToUpper().Trim() == "ALL")
            {
                mh.IsAllMembers = true;
            }

            MDMMemberHelper tmpHelper = mh;
            if (i + 1 < expression.Count)
            {
                if ((string)expression[i + 1] == "Ancestor")
                {
                    i++;
                    //anc = 0;
                    int first = 0;
                    while (i + first < expression.Count && (string)expression[i + first] == "Ancestor")
                    {
                        i = i + first;

                        if ((string)expression[++i] == "MemberDimension")
                            dimension = ((string)expression[++i]).Replace("@whitespace@", " ").Replace("@WHITESPACE@", " ");
                        if ((string)expression[++i] == "MemberDepth")
                            depth = (int.Parse(expression[++i].ToString()));
                        if ((string)expression[++i] == "MemberValue")
                            value = ((string)expression[++i]).Replace("@whitespace@", " ").Replace("@WHITESPACE@", " ");
                        mh = mh.addAncestor(dimension, depth, value);

                        first = 1;
                    }
                }
            }

            QueryInfo.addMember(tmpHelper);
            return tmpHelper;
        }

        /// <summary>
        /// Parse a tuple
        /// </summary>
        /// <param name="expression">the expression wich contains the tuple</param>
        /// <param name="i">The current index of the expression</param>
        /// <param name="QueryInfo">The Query Info to store parsed values</param>
        /// <returns>The result Tuple</returns>
        private MDMTupleHelper parseTuple(ref List<string> expression, ref int i, ref StatementSummarizedInfoHelper QueryInfo)
        {
            //Tuples can have Ranges or Members
            MDMTupleHelper tmpTuple = new MDMTupleHelper();
            int rangecount = 0;
            if ((string)expression[i] == "MdmRange")
            {
                tmpTuple.addRange();

                if ((string)expression[++i] == "StartMember")
                {
                    if ((string)expression[++i] == "MdmMember")
                    {
                        tmpTuple.getRange(rangecount).addStartMember(parseMember(ref expression, ref i, ref QueryInfo));
                    }
                }
                if ((string)expression[++i] == "EndMember")
                {
                    if ((string)expression[++i] == "MdmMember")
                    {
                        tmpTuple.getRange(rangecount).addEndMember(parseMember(ref expression, ref i, ref QueryInfo));
                    }
                }
                rangecount++;
            }
            else
            {

                while ((string)expression[i] == "MdmMember")
                {
                    tmpTuple.addMember(parseMember(ref expression, ref i, ref QueryInfo));
                    if (i < expression.Count - 1)
                    {
                        i++;
                    }
                }
                i--;
            }
            return tmpTuple;
        }

        /// <summary>
        /// Parse a set
        /// </summary>
        /// <param name="expression">the expression wich contains the set</param>
        /// <param name="i">The current index of the expression</param>
        /// <param name="QueryInfo">The Query Info to store parsed values</param>
        /// <returns>The Set constructed from scratch</returns>
        private MDMSetHelper parseSet(ref List<string> expression, ref int i, ref StatementSummarizedInfoHelper QueryInfo)
        {
            MDMSetHelper tmpSet = new MDMSetHelper();
            i++;
            if (i < expression.Count)
            {
                while ((string)expression[i] == "MdmTuple")
                {
                    i++;
                    tmpSet.addTuple(parseTuple(ref expression, ref i, ref QueryInfo));
                    i++;
                    if (i == expression.Count) break;
                }
                i--;
            }
            return tmpSet;
        }

        /// <summary>
        /// Parse an axis
        /// </summary>
        /// <param name="expression">The expression wich contains the axis</param>
        /// <param name="i">The current index of the expression being parsed</param>
        /// <param name="QueryInfo">The Query Info to store parsed values</param>
        /// <returns>The result Axis</returns>
        private MDMQueryAxisHelper parseAxis(ref List<string> expression, ref int i, ref StatementSummarizedInfoHelper QueryInfo)
        {
            string name = (string)expression[++i];
            MDMQueryAxisHelper tmpAxis = new MDMQueryAxisHelper(name, name);
            i++;
            // An axis can have a MdmSet or a Function in wich case calls the function part of the if.
            if ((string)expression[i] == "MdmSet")
            {
                tmpAxis.addSet(parseSet(ref expression, ref i, ref QueryInfo));
            }
            if ((string)expression[i] == "MdmFunction")
            {
                i++;
                foreach (FunctionToken ft in MDXFunctionTokens)
                {
                    Regex axisre = new Regex(ft.TokenExpression);
                    foreach (Match m in axisre.Matches((string)expression[i]))
                    {
                        QueryInfo.addFunction(tmpAxis.addFunction(ft.ParseToken(ref expression, ref i, m.Value.ToUpper(), ref QueryInfo)));
                    }
                }
            }
            QueryInfo.addAxis(tmpAxis.AxisName);
            return tmpAxis;
        }

        /// <summary>
        /// Parse an calculated member
        /// </summary>
        /// <param name="expression">The expression wich contains the Calculated member</param>
        /// <param name="i">The current index of the expression being parsed</param>
        /// <param name="QueryInfo">The Query Info to store parsed values</param>
        /// <returns>The result Calculated Member</returns>
        private MDMCalculatedMemberHelper parseCalculatedMember(ref List<string> expression, ref int i, ref StatementSummarizedInfoHelper QueryInfo)
        {
            MDMCalculatedMemberHelper tmpCalculatedMember = new MDMCalculatedMemberHelper();
            i++;

            // An axis can have a MdmSet or a Function in wich case calls the function part of the if.
            if ((string)expression[i] == "MdmMember")
            {
                tmpCalculatedMember.addMember(parseMember(ref expression, ref i, ref QueryInfo));
                QueryInfo.addCalculatedMember(tmpCalculatedMember);
            }
            if ((string)expression[++i] == "MdmFunction")
            {
                i++;
                foreach (FunctionToken ft in MDXFunctionTokens)
                {
                    Regex axisre = new Regex(ft.TokenExpression);
                    foreach (Match m in axisre.Matches((string)expression[i]))
                    {
                        QueryInfo.addFunction(tmpCalculatedMember.addFunction(ft.ParseToken(ref expression, ref i, m.Value.ToUpper(), ref QueryInfo)));
                    }
                }
            }
            else if ((string)expression[i] == "MdmStartOperation")
            {
                while ((string)expression[++i] != "MdmEndOperation")
                {
                    if (((string)expression[i]).Trim() == "OperationToken" || ((string)expression[i]).Trim() == "OperationNumber")
                    {
                        tmpCalculatedMember.addOperationToken(new MDMOperationTokenHelper((string)expression[++i]));
                    }
                    else
                    {
                        tmpCalculatedMember.addOperationMember(parseMember(ref expression, ref i, ref QueryInfo));
                    }
                }
            }
            if ((string)expression[++i] == "SolveOrder")
            {
                tmpCalculatedMember.SolveOrder = int.Parse((string)expression[++i]);
            }
            return tmpCalculatedMember;
        }

        /// <summary>
        /// Parse an named set
        /// </summary>
        /// <param name="expression">The expression wich contains the axis</param>
        /// <param name="i">The current index of the expression being parsed</param>
        /// <param name="QueryInfo">The Query Info to store parsed values</param>
        /// <returns>The result Named Set</returns>
        private MDMNamedSetHelper parseNamedSet(ref List<string> expression, ref int i, ref StatementSummarizedInfoHelper QueryInfo)
        {
            i++;
            string name = ((string)expression[++i]).ToUpper().Replace("@WHITESPACE@", " ");
            MDMNamedSetHelper tmpNamedSet = new MDMNamedSetHelper(name);
            i++;

            // An axis can have a MdmSet or a Function in wich case calls the function part of the if.
            if ((string)expression[i] == "MdmSet")
            {
                tmpNamedSet.addSet(parseSet(ref expression, ref i, ref QueryInfo));
            }
            if ((string)expression[i] == "MdmFunction")
            {             
                i++;
                foreach (FunctionToken ft in MDXFunctionTokens)
                {
                    Regex axisre = new Regex(ft.TokenExpression);
                    foreach (Match m in axisre.Matches((string)expression[i]))
                    {
                        QueryInfo.addFunction(tmpNamedSet.addFunction(ft.ParseToken(ref expression, ref i, m.Value.ToUpper(), ref QueryInfo)));
                    }
                }
            }

            QueryInfo.addNamedSet(tmpNamedSet);
            return tmpNamedSet;
        }

        /// <summary>
        /// Parse the slicer axis 
        /// </summary>
        /// <param name="expression">The expression wich defines the slicer</param>
        /// <param name="i">The current index of the parsing procedure</param>
        /// <param name="QueryInfo">The Query Info to store parsed values</param>
        /// <returns>The Result new axis</returns>
        private MDMQueryAxisHelper parseSlicer(ref List<string> expression, ref int i, ref StatementSummarizedInfoHelper QueryInfo)
        {
            MDMQueryAxisHelper tmpAxis = new MDMQueryAxisHelper("SLICER", "SLICER");
            i++;

            // An axis can have a MdmSet or a Function in wich case calls the function part of the if.
            if ((string)expression[i] == "MdmSet")
            {
                tmpAxis.addSet(parseSet(ref expression, ref i, ref QueryInfo));
            }
            if ((string)expression[i] == "MdmFunction")
            {                
                i++;
                foreach (FunctionToken ft in MDXFunctionTokens)
                {
                    Regex axisre = new Regex(ft.TokenExpression);
                    foreach (Match m in axisre.Matches((string)expression[i]))
                    {
                        QueryInfo.addFunction(tmpAxis.addFunction(ft.ParseToken(ref expression, ref i, m.Value.ToUpper(), ref QueryInfo)));
                    }
                }
            }
            return tmpAxis;
        }

        /// <summary>
        /// Gets the type of statement
        /// </summary>
        /// <param name="Expression">The expression</param>
        /// <returns>The type of statement</returns>
        private MDMStatementTypes getStatementType(QueryExpression Expression)
        {
            if (Regex.IsMatch(Expression.Expression, "\\s[Uu][Pp][Dd][Aa][Tt][Ee]\\s|\\[Uu][Pp][Dd][Aa][Tt][Ee]\\s|[Uu][Pp][Dd][Aa][Tt][Ee]\\s"))
                return MDMStatementTypes.UPDATE;
            if (Regex.IsMatch(Expression.Expression, "\\s[Aa][Ll][Tt][Ee][Rr]\\s|\\[Aa][Ll][Tt][Ee][Rr]\\s|[Aa][Ll][Tt][Ee][Rr]\\s"))
                return MDMStatementTypes.ALTER;
            return MDMStatementTypes.SELECT;
        }

        /// <summary>
        /// This method parse a given expression with an Operation and return the MDMOperation in it
        /// </summary>
        /// <param name="Expression">This is an expression to parse.</param>
        /// <returns>A MDMOperation[] for the given expression </returns>
        private MdmOperationMember[] ParseOperation(QueryExpression Expression)
        {
            Normalize(ref Expression);
            string tmpOperation = " MdmStartOperation " + OperationSplitter(Expression.Expression) + " MdmEndOperation";
            int index = 0;
            List<string> operation = new List<string>(tmpOperation.Split(new string[] { " " } , StringSplitOptions.RemoveEmptyEntries));
            StatementSummarizedInfoHelper QueryInfo = new StatementSummarizedInfoHelper();
            return parseOperation(ref operation, ref index, ref QueryInfo);
        }

        /// <summary>
        /// This method parses a given Query Expression.
        /// </summary>
        /// <param name="expression">This is an expression to parse.</param>
        /// <returns>A MDMQuery for the given expression </returns>
        private MDMStatement parseUpdate(List<string> expression)
        {
            if (expression.Count > 0)
            {
                // an Update object to return
                UpdateBuilder updatestatement = new UpdateBuilder();

                // The QueryInfo stores the members, dimensions, levels and other structures organized in a way to do second level checks
                StatementSummarizedInfoHelper updateInfo = new StatementSummarizedInfoHelper();

                int i = 0;

                if ((string)expression[i] == "Cube")
                {
                    updatestatement.setCube(((string)expression[++i]).Replace("[", "").Replace("]", "").Replace("@whitespace@", " ").Replace("@WHITESPACE@", " "));
                    updateInfo.addCube(updatestatement.Cube);
                }
                if ((string)expression[++i] == "MdmTuple")
                {
                    i++;
                    updatestatement.addTuple(parseTuple(ref expression, ref i, ref updateInfo));
                }

                if ((string)expression[++i] == "NewValue")
                    updatestatement.setValue(expression[++i].ToString());

                updatestatement.StatementInfo = updateInfo;

                return updatestatement.Build();
            }
            return null;
        }

        /// <summary>
        /// This method parse a given Query Expression.
        /// </summary>
        /// <param name="expression">This is an expression to parse.</param>
        /// <returns>A MDMQuery for the given expression </returns>
        private MDMStatement parseQuery(List<string> expression)
        {
            // This query builder object is used to construct the returned query
            QueryBuilder b = new QueryBuilder();

            // The QueryInfo stores the members, dimensions, levels and other structures organized in a way to do second level checks
            StatementSummarizedInfoHelper QueryInfo = new StatementSummarizedInfoHelper();

            // Some internal variables

            for (int i = 0; i < expression.Count; i++)
            {
                if ((string)expression[i] == "MdmCalculatedMember")
                {                   
                    b.addCalculatedMember(parseCalculatedMember(ref expression, ref i, ref QueryInfo));
                }
                if ((string)expression[i] == "MdmNamedSet")
                {
                    b.addNamedSet(parseNamedSet(ref expression, ref i, ref QueryInfo));
                }

                if ((string)expression[i] == "Axis")
                {
                    // Parsing each axis of the expression
                    b.addAxis(parseAxis(ref expression, ref i, ref QueryInfo));
                }
                if ((string)expression[i] == "Cube")
                {
                    b.Cube = ((string)expression[++i]).Replace("[", "").Replace("]", "").Replace("@whitespace@", " ").Replace("@WHITESPACE@", " ");
                    QueryInfo.addCube(b.Cube);
                }
                if ((string)expression[i] == "Slicer")
                {
                    #region SLICER PARSING

                    b.addSlicer(parseSlicer(ref expression, ref i, ref QueryInfo));
                    QueryInfo.addAxis("SLICER");
                    #endregion
                }
            }
            b.StatementInfo = QueryInfo;            
            return b.Build();
        }

        #endregion

        #region Function Splitters

        /// <summary>
        /// This method splits the  update part of the mdx string, calling the Tuple splitter
        /// </summary>
        /// <param name="mdx">The string to split</param>
        /// <param name="token">The token to split by</param>
        /// <returns>the splitted and reorganized string</returns>
        public string UpdateSplitter(string mdx, string token)
        {
            string updateString = new System.Text.RegularExpressions.Regex("(?<=[Uu][Pp][Dd][Aa][Tt][Ee]\\s).*(?=\\s[Ss][Ee][Tt]\\s)").Match(mdx).Value;

            if (updateString != "")
            {
                string newValue = mdx.Substring(mdx.IndexOf("=")).Trim().Replace("=", "");

                updateString = " Cube " + updateString;

                string tupleString = new Regex("(?<=[Ss][Ee][Tt]\\s).*(?=\\=)").Match(mdx).Value.Trim();

                // NORMALIZE .VALUE sintax....
                if (tupleString.Substring(tupleString.Length - 6).ToUpper() == ".VALUE")
                {
                    tupleString = tupleString.Substring(0, tupleString.Length - 6).Trim();
                }

                return updateString + " MdmTuple " + TupleSplitter(tupleString) + " NewValue " + newValue;
            }
            else
            {
                AddToErrorCollection("Some portions of the update statement have been ignored");
                return "";
            }
        }

        /// <summary>
        /// This method splits the  select part of the mdx string, calling the Axissplitter each time that finds an Axis
        /// </summary>
        /// <param name="mdx">The string to split</param>
        /// <param name="token">The token to split by</param>
        /// <returns>the splitted and reorganized string</returns>
        public string SelectSplitter(string mdx, string token)
        {
            // Search starting with Select and ending with from discading prefix and suffix
            string SelectMdx = "";
            SelectMdx = new System.Text.RegularExpressions.Regex("(?<=[Ss][Ee][Ll][Ee][Cc][Tt]\\s).*(?=\\s[Ff][Rr][Oo][Mm]\\s)").Match(mdx).Value;

            // Search for different axis
            //SelectMdx = Regex.Replace(SelectMdx, "[Ss][Ee][Ll][Ee][Cc][Tt]", "");
            Regex axisre = new System.Text.RegularExpressions.Regex("[Oo][Nn]\\s\\w+|[Oo][Nn]\\s\\w+\\s\\,\\s");
            string tmpMdx = "";
            int lastIndex = 0;
            foreach (Match m in axisre.Matches(SelectMdx))
            {
                tmpMdx += "Axis ( " + m.Value.ToUpper().Replace("ON ", "") + " , ";
                tmpMdx += SelectMdx.Substring(lastIndex, m.Index - lastIndex) + " ) @newaxis@ ";
                lastIndex = m.Index + m.Length + 3;
            }
            // removing the last newaxis token
            tmpMdx = tmpMdx.Remove(tmpMdx.Length - 10, 10);
            // splitting the axis in an array 
            string[] axes = tmpMdx.Split(new string[] { "@newaxis@" }, StringSplitOptions.None);
            // setting temp string value to blank 
            tmpMdx = "";

            foreach (string axis in axes)
            {
                // Calling the Axis splitter method
                tmpMdx += AxisSplitter(axis);
            }
            return tmpMdx.Replace("(", "").Replace(")", "").Replace(" , ", " ").Replace("  ", " ");
            // inverting axiss Precedence.
        }

        /// <summary>
        /// Splits the from part of the mdx string, and gets the cube for the query
        /// </summary>
        /// <param name="mdx">the string to split</param>
        /// <param name="token">The token to split by</param>
        /// <returns>the splitted and reorganized string</returns>
        public string FromSplitter(string mdx, string token)
        {
            // Finding the "WHERE" word position
            int endPos = mdx.ToUpper().IndexOf("WHERE");
            if (endPos > 0)
                return mdx.Substring(mdx.ToUpper().IndexOf(token), endPos - mdx.ToUpper().IndexOf("FROM")).Replace("FROM", " Cube ");
            else
                return mdx.Substring(mdx.ToUpper().IndexOf(token)).Replace(token, " Cube ");
        }

        /// <summary>
        /// Splits the where part of the mdx string, and gets the Slicer axis of the query
        /// </summary>
        /// <param name="mdx">The MDX to split</param>
        /// <param name="token">The token to split by</param>
        /// <returns>A split string</returns>
        public string WhereSplitter(string mdx, string token)
        {
            // Finding the "WHERE" word position for use it as the starting point
            int startPos = mdx.ToUpper().IndexOf(token);
            if (startPos > 0)
            {
                mdx = mdx.Substring(startPos).Replace(token, "");
                // Calling the SlicerSplitter that works in a similar way that a common axis splitter.
                return " Slicer " + SlicerSplitter(mdx);
            }
            else
                return "";
        }

        /// <summary>
        /// Splits the with part of the mdx string, and gets the NamedSets and Calculated members!
        /// </summary>
        /// <param name="mdx">The MDX to split</param>
        /// <param name="token">The token to split by</param>
        /// <returns>The split string</returns>
        public string WithSplitter(string mdx, string token)
        {
            // Finding the "WHERE" word position for use it as the starting point
            int startPos = mdx.ToUpper().IndexOf(token);
            mdx = mdx.Substring(startPos + token.Length);

            int endPos = mdx.Length;
            if (Regex.IsMatch(mdx, "[Ss][Ee][Ll][Ee][Cc][Tt]\\s"))
                endPos = Regex.Match(mdx, "[Ss][Ee][Ll][Ee][Cc][Tt]\\s").Index;
            string tmpMdx = "";
            string rtrnMdx = "";
            foreach (Match m in Regex.Matches(mdx.Substring(0, endPos - 1), "[Ss][Ee][Tt]\\s|[Mm][Ee][Mm][Bb][Ee][Rr]\\s"))
            {
                if (m.NextMatch().Success)
                    tmpMdx = mdx.Substring(m.Index, m.NextMatch().Index);
                else
                    tmpMdx = mdx.Substring(m.Index, endPos - m.Index);
                if (tmpMdx.Substring(0, 3).ToUpper() == "SET")
                    rtrnMdx += NamedSetSplitter(tmpMdx);
                else
                    if (tmpMdx.Substring(0, 6).ToUpper() == "MEMBER")
                        rtrnMdx += CalculatedMemberSplitter(tmpMdx);
            }
            return rtrnMdx;
        }

        #endregion

        #region Object Splitters

        /// <summary>
        /// Splits axis and call set splitters
        /// </summary>
        /// <param name="mdx"></param>
        /// <returns></returns>
        private string AxisSplitter(string mdx)
        {
            string rtrnString = "";
            string axisDef = "";
            string axisContent = "";
            // searching for sets or set functions....
            //if (mdx.IndexOf('{') > -1)
            //{
            if (mdx.Substring(mdx.IndexOf(",") + 1).Trim().Substring(0, 1) == "{")
            {
                // Process the set inside the axis
                axisDef = mdx.Split(new char[] { '{' })[0].Replace("  ", " ");
                axisContent = mdx.Split(new char[] { '{' })[1].Replace("}  )", "");

                rtrnString += axisDef + " MdmSet " + SetSplitter(axisContent);
            }
            else
            {
                // Process the set functions inside the axis...
                axisDef = mdx.Substring(0, mdx.IndexOf(" , ")).Replace("  ", " ");
                axisContent = mdx.Substring(mdx.IndexOf(" , ") + 2).Replace("  ", " ");

                rtrnString += axisDef + FunctionSplitter(axisContent);
            }
            //}
            return rtrnString;
        }

        /// <summary>
        /// Splits the slicer axis
        /// </summary>
        /// <param name="mdx">The expression containing the slicer</param>
        /// <returns>The splitted string</returns>
        private string SlicerSplitter(string mdx)
        {
            string rtrnString = "";
            string axisContent = "";

            // Search if the { } is missed but is not a MdmFunction
            if (mdx.IndexOf("{") < 0 && mdx.Trim().IndexOf("(") == 0 || mdx.IndexOf("{") < 0 && mdx.Trim().IndexOf("[") == 0)
                mdx = " { " + mdx.Replace("(", "").Replace(")", "") + " } ";

            if (mdx.Trim().IndexOf('{') == 0)
            {
                // Process the set inside the axis
                axisContent = mdx.Split(new char[] { '{' })[1].Replace("}", "").Replace("(", "").Replace(")", "").Replace("}  )", "");
                rtrnString += " MdmSet " + SetSplitter(axisContent);
            }
            else
            {
                // Process the set functions inside the axis...
                axisContent = mdx.Split(new char[] { '{' })[1].Replace("}  )", "");
                rtrnString += FunctionSplitter(mdx);
            }
            return rtrnString;
        }

        /// <summary>
        /// Splits as Set
        /// </summary>
        /// <param name="mdx">The expression containing a set</param>
        /// <returns>the splitted set</returns>
        private string SetSplitter(string mdx)
        {
            string rtrnString = "";

            // I must split tuples and call a tuple splitter for each one...
            // A set could not have any tuple defined, so all the content is a tuple....
            // Searching for tuples...
            // when you have a ( following a ", " or "{ " then you have a Tuple....
            Regex tuples = new Regex(",\\s\\(|\\{\\s\\(|^\\w*\\(|^\\w*\\s\\(");
            bool findcontent = false;
            foreach (Match m in tuples.Matches(mdx))
            {
                findcontent = true;
                string tuple = mdx.Substring(m.Index + 2, mdx.IndexOf(")", m.Index) - (m.Index));
                rtrnString += " MdmTuple " + TupleSplitter(tuple);
            }
            // Did not find any tuple, proceed searching for Functions
            if (!findcontent)
            {
                //If none tuple is found, then must asume () symbols.
                string[] createdtuples = mdx.Split(',');
                foreach (string createdtuple in createdtuples)
                {
                    if (createdtuple != "")
                        rtrnString += " MdmTuple " + TupleSplitter(createdtuple);
                }
            }
            return rtrnString.Replace("{", "").Replace("}", "");
        }

        /// <summary>
        /// Splits a tuple
        /// </summary>
        /// <param name="mdx">The expression containing the tuple</param>
        /// <returns>The splitted tuple</returns>
        private string TupleSplitter(string mdx)
        {
            if (mdx.Trim().Substring(0, 1) == "(" && mdx.Trim().Substring(mdx.Trim().Length - 1, 1) == ")")
                mdx = mdx.Substring(1, mdx.Length - 2);

            string rtrnString = "";
            // tuple could have members or Range, at this point I am only intersted in Ranges....
            if (mdx.IndexOf("Range") > -1)
                return RangeSplitter(mdx);
            else
            {
                string[] members = mdx.Split(new string[] { " , " }, StringSplitOptions.None);
                foreach (string member in members)
                {
                    rtrnString += MemberSplitter(member);
                }
            }
            return rtrnString;
        }

        /// <summary>
        /// Splits a Member and his ancestors recursively
        /// </summary>
        /// <param name="mdx">the expression containing the member</param>
        /// <returns>The splitted string</returns>
        private string MemberSplitter(string mdx)
        {
            string rtrnStr = " MdmMember ";

            string[] levels = mdx.Split(new char[] { '.' });
            int memberlevels = levels.Length;
            int startancestor = 1;

            string memberdimension;
            string memberValue;
            string lastWord = levels[levels.Length - 1].ToUpper().Trim();

            memberdimension = " MemberDimension " + levels[0].Replace("[", "").Replace("]", "");

            if (lastWord == "DESCENDANTS" ||
                lastWord == "SIBILINGS" ||
                lastWord == "COUSINS" ||
                lastWord == "PARENTS" ||
                lastWord == "ANCESTORS" ||
                lastWord == "CHILDREN" ||
                lastWord == "SIBILING" ||
                lastWord == "CURRENTMEMBER" ||
                lastWord == "CURRENTMEMBERS" ||
                lastWord == "TRANSACTIONS")
            {
                startancestor = 2;
                memberValue = " MemberDepth " + ((int)(levels.Length - startancestor)).ToString() + " MemberValue " + levels[levels.Length - startancestor].Replace("[", "").Replace("]", "");
                rtrnStr += "MemberRelationship " + levels[levels.Length - 1].Replace("[", "").Replace("]", "");
            }
            else
                memberValue = " MemberDepth " + ((int)(levels.Length - 1)).ToString() + " MemberValue " + levels[levels.Length - 1].Replace("[", "").Replace("]", "");

            rtrnStr += memberdimension + memberValue;

            for (int i = 1; i < levels.Length - startancestor; i++)
            {
                rtrnStr += " Ancestor " + memberdimension + " MemberDepth " + i + " MemberValue " + levels[i].Replace("[", "").Replace("]", "");
            }
            return rtrnStr;
        }

        /// <summary>
        /// Splits a function
        /// </summary>
        /// <param name="mdx">The expression containing the function to split</param>
        /// <returns>The splitted function</returns>
        private string FunctionSplitter(string mdx)
        {
            string rtrnString = "";
            foreach (FunctionToken ft in MDXFunctionTokens)
            {
                Regex axisre = new Regex(ft.TokenExpression);
                Match m = axisre.Match(mdx.Trim().Substring(0, mdx.Trim().IndexOf(' ')));
                if (m.Value != null && m.Value != "")
                {
                    //normalizing function into uppercase
                    mdx = mdx.Replace(m.Value, m.Value.ToUpper());
                    rtrnString += ft.ProcessToken(mdx, m.Value.ToUpper());
                }
            }
            return rtrnString;
        }

        /// <summary>
        ///  Splitt a MDMRange
        /// </summary>
        /// <param name="mdx">the string containing the range to split</param>
        /// <returns>The splitted string</returns>
        private string RangeSplitter(string mdx)
        {
            // Remove ( and ) if there are any... 
            mdx = mdx.Replace("(", "").Replace(")", "");
            string[] tmpMdx = mdx.Split(new string[] { "Range" } , StringSplitOptions.None);
            return " MdmRange StartMember " + MemberSplitter(tmpMdx[0]) + " EndMember " + MemberSplitter(tmpMdx[1]) + "  ";
            // 
        }

        /// <summary>
        /// Splits a Function tha receives two set as a parameters
        /// </summary>
        /// <param name="mdx">the string containing the function to split</param>
        /// <param name="functionName">The name of the function being splitted</param>
        /// <returns>The splitted function</returns>
        private string TwoSetFunctionSplitter(string mdx, string functionName)
        {
            string rtrnString = "";

            string axisDef = mdx.Substring(0, mdx.IndexOf(functionName + " (")).Replace("  ", " ");
            string axisContent = mdx.Substring(mdx.IndexOf(functionName + " (") + functionName.Length + 2).Replace("  ", " ");

            rtrnString += axisDef + " MdmFunction " + functionName;
            int setdivisor = 0;
            // THE FIRST PARAMETER IS A SET OR A FUNCTION?
            if (axisContent.Trim().Substring(0, axisContent.Trim().IndexOf(' ')) == "{")
            {
                setdivisor = axisContent.IndexOf("} ,") + 2;
                rtrnString += " MdmSet " + SetSplitter(axisContent.Substring(0, setdivisor));
            }
            else
            {
                setdivisor = axisContent.IndexOf(") ,") + 2;
                rtrnString += FunctionSplitter(axisContent.Substring(setdivisor));
            }
            setdivisor++;
            // THE SECOND PARAMETER IS A SET OR A FUNCTION?
            if (axisContent.Trim().Substring(setdivisor, 1).Trim() == "{")
            {
                rtrnString += " MdmSet " + SetSplitter(axisContent.Substring(setdivisor));
            }
            else
            {
                rtrnString += FunctionSplitter(axisContent.Substring(setdivisor));
            }

            return rtrnString;
        }

        /// <summary>
        /// Splits a Function tha receives one set as a parameter
        /// </summary>
        /// <param name="mdx">the string containing the function to split</param>
        /// <param name="functionName">The name of the function being splitted</param>
        /// <returns>The splitted function</returns>
        private string SetFunctionSplitter(string mdx, string functionName)
        {
            string rtrnString = "";

            string axisDef = mdx.Substring(0, mdx.IndexOf(functionName + " (")).Replace("  ", " ");
            string axisContent = mdx.Substring(mdx.IndexOf(functionName + " (") + functionName.Length + 2).Replace("  ", " ");

            rtrnString += axisDef + " MdmFunction " + functionName;
            if (axisContent.Trim().Substring(0, axisContent.Trim().IndexOf(' ')) == "{")
            {
                rtrnString += " MdmSet " + SetSplitter(axisContent.Substring(0, axisContent.IndexOf("}")));
            }
            else
            {
                // Check if it is a function or are missing {}
                if (axisContent.Trim().Substring(0, 1) != "(")
                {
                    rtrnString += FunctionSplitter(axisContent.Substring(0, axisContent.IndexOf(")")));
                }
                else
                {
                    rtrnString += " MdmSet " + SetSplitter(axisContent.Substring(0, axisContent.IndexOf(")")));
                }
            }
            return rtrnString;
        }

        /// <summary>
        /// Splits the filter Function that receives a set and an operation as parameter
        /// </summary>
        /// <param name="mdx">the string containing the function to split</param>
        /// <param name="functionName">The name of the function being splitted</param>
        /// <returns>The splitted function</returns>
        private string FilterFunctionSplitter(string mdx, string functionName)
        {
            string rtrnString = "";

            string axisDef = mdx.Substring(0, mdx.IndexOf(functionName + " (")).Replace("  ", " ");
            string axisContent = mdx.Substring(mdx.IndexOf(functionName + " (") + functionName.Length + 2).Replace("  ", " ");

            rtrnString += axisDef + " MdmFunction " + functionName;
            int setdivisor = 0;
            // THE FIRST PARAMETER IS A SET OR A FUNCTION?
            if (axisContent.Trim().Substring(0, axisContent.Trim().IndexOf(' ')) == "{")
            {
                setdivisor = axisContent.IndexOf("} ,") + 3;
                rtrnString += " MdmSet " + SetSplitter(axisContent.Substring(0, setdivisor));
            }
            else
            {
                setdivisor = axisContent.IndexOf(") ,") + 3;
                rtrnString += FunctionSplitter(axisContent.Substring(0, setdivisor));
            }

            // THE SECOND PARAMETER IS AN OPERATION
            rtrnString += " MdmStartOperation " + OperationSplitter(axisContent.Substring(setdivisor, axisContent.IndexOf(')', setdivisor) - setdivisor)) + " MdmEndOperation";

            return rtrnString;
        }

        /// <summary>
        /// Splits an expression containing a Named Set
        /// </summary>
        /// <param name="mdx">The expression to split</param>
        /// <returns>The splitted expression</returns>
        private string NamedSetSplitter(string mdx)
        {
            int startPos = mdx.IndexOf("SET") + 3;
            int length = mdx.IndexOf(" AS") - startPos;
            Match operation = new System.Text.RegularExpressions.Regex("(?<=\\s\'\\s).*(?=\\s\'\\s)").Match(mdx);
            string rtrnString = " MdmNamedSet MdmNamedSetName " + mdx.Substring(startPos, length).Replace("[", "").Replace("]", "") + " ";
            bool isFunction = false;

            foreach (FunctionToken ft in MDXFunctionTokens)
            {
                Regex fnctre = new Regex(ft.TokenExpression);
                Match m = fnctre.Match(operation.Value.Trim());
                if (m.Value != null && m.Value != "")
                {
                    //normalizing function into uppercase
                    rtrnString += " MdmFunction " + ft.ProcessToken(operation.Value.Trim().ToUpper(), m.Value.Trim().ToUpper());
                    isFunction = true;
                    break;
                }
            }
            if (!isFunction)
                rtrnString += " MdmSet " + SetSplitter(operation.Value.Trim());

            return rtrnString;
        }

        /// <summary>
        /// Splits an expression having a Calculated member 
        /// </summary>
        /// <param name="mdx">the expression to split</param>
        /// <returns>the splited expression</returns>
        private string CalculatedMemberSplitter(string mdx)
        {
            Match member = new System.Text.RegularExpressions.Regex("(?<=[M][E][M][B][E][R]).*(?=[A][S]\\s)").Match(mdx.ToUpper().Trim());
            Match solveorder = new System.Text.RegularExpressions.Regex("(?<=\\s[S][O][L][V][E][_][O][R][D][E][R]\\s\\=).*(?=\\s)").Match(mdx.ToUpper());
            Match operation = new System.Text.RegularExpressions.Regex("(?<=\\s\'\\s).*(?=\\s\'\\s)").Match(mdx);
            int solve_order = -1;
            if (solveorder.Success)
            {
                int.TryParse(solveorder.Value.Trim(), out solve_order);
            }

            string rtrnString = " MdmCalculatedMember " + MemberSplitter(member.Value.Trim()) + " ";
            // Have to search for an operation or a function..                    
            bool isFunction = false;

            foreach (FunctionToken ft in MDXFunctionTokens)
            {
                Regex fnctre = new Regex(ft.TokenExpression);
                Match m = fnctre.Match(operation.Value.Trim());
                if (m.Value != null && m.Value != "")
                {
                    //normalizing function into uppercase
                    rtrnString += ft.ProcessToken(operation.Value.Trim().ToUpper(), m.Value.Trim().ToUpper()) + " ";
                    isFunction = true;
                    break;
                }
            }
            if (!isFunction)
                rtrnString += " MdmStartOperation " + OperationSplitter(operation.Value.Trim()) + " MdmEndOperation";

            rtrnString += " SolveOrder " + solve_order.ToString() + " ";
            return rtrnString;
        }

        /// <summary>
        /// Splits an operation
        /// </summary>
        /// <param name="operation">The operation to split</param>
        /// <returns>The split string</returns>
        private string OperationSplitter(string operation)
        {
            string rtnString = "";
            foreach (string token in TokenizeOperation(operation))
            {
                // Token could be from tree types:
                // a Member to operate.
                // a Number 
                // or a operantion or bracket
                if (Regex.IsMatch(token, "\\[[^\\s]*\\b\\]"))
                {
                    rtnString += MemberSplitter(token);
                }
                else
                {
                    if (Regex.IsMatch(token, "\\b[0-9]+\\.?[0-9]*"))
                        rtnString += " OperationNumber " + token;
                    else
                        rtnString += " OperationToken " + token;
                }
            }
            return rtnString;
        }

        /// <summary>
        /// Tokenises an operation
        /// </summary>
        /// <param name="operation">The operation to tokenise</param>
        /// <returns>A tokenised operation</returns>
        private string[] TokenizeOperation(string operation)
        {
            List<String> rtnr = new List<string>();
            // Select from 3 alternatives:
            // Any character in the class [/*-+]
            // Or [ follow by Any character other than whitespace followed by ]
            // Or any number expressed as a character between 0 and 9 at a beggining of word followed by any number
            Regex rwi = new Regex("[Oo][Rr]|[Aa][Nn][Dd]|[\\/\\*\\+\\-\\(\\)\\<\\>\\=]|\\[[^\\s]*\\b\\]|\\b[0-9]+\\.?[0-9]*");
            foreach (Match m in rwi.Matches(operation))
            {
                rtnr.Add(m.Value.Trim());
            }
            return (string[])rtnr.ToArray();
        }

        #endregion
    }
}