/*
 * This source file (CubedAdapter.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;

namespace Bamboo.Cubed.Connect
{
    /// <summary>
    /// This provides access to a data adapter to connect to a data command
    /// </summary>
    public class CubedAdapter : IDbDataAdapter
    {
        /// <summary>
        /// Initialises a new default Cubed Adapter
        /// </summary>
        public CubedAdapter()
        {
            _deletecommand = null;
            _selectcommand = null;
            _insertcommand = null;
            _updatecommand = null;
        }

        /// <summary>
        /// Initialises a new instance of the Cubed Adapter with the given SELECT command
        /// </summary>
        /// <param name="selectcommand">The SELECT command for this adapter</param>
        public CubedAdapter(CubedCommand selectcommand)
        {
            _deletecommand = null;
            _selectcommand = selectcommand;
            _insertcommand = null;
            _updatecommand = null;
        }

        private CubedCommand _deletecommand;
        /// <summary>
        /// This command deletes data from the cube
        /// </summary>
        public IDbCommand DeleteCommand
        {
            get { return _deletecommand; }
            set { _deletecommand = (CubedCommand)value; }
        }

        private CubedCommand _insertcommand;
        /// <summary>
        /// This command inserts new data into the cube
        /// </summary>
        public IDbCommand InsertCommand
        {
            get { return _insertcommand; }
            set { _insertcommand = (CubedCommand)value; }
        }

        private CubedCommand _selectcommand;
        /// <summary>
        /// This command queries the cube for data
        /// </summary>
        public IDbCommand SelectCommand
        {
            get { return _selectcommand; }
            set { _selectcommand = (CubedCommand)value; }
        }

        private CubedCommand _updatecommand;
        /// <summary>
        /// This command updates the cube with data
        /// </summary>
        public IDbCommand UpdateCommand
        {
            get { return _updatecommand; }
            set { _updatecommand = (CubedCommand)value; }
        }

        /// <summary>
        /// This fills a data set with the results of the select command, and returns the number of
        /// rows filled into the data set
        /// </summary>
        /// <param name="dataset">The dataset to fill</param>
        /// <returns>The number of rows returned</returns>
        public int Fill(DataSet dataset)
        {
            // Check whether our command exists and is initialised
            if (_selectcommand != null && _selectcommand.Connection != null &&
                _selectcommand.Connection.State == ConnectionState.Open)
            {
                // Our command is ready, so use it to return a data set
                DataSet results = _selectcommand.ExecuteDataSet();

                // Now, if a table already exists in the given data set with the same name, delete it
                if (results.Tables.Count > 0 && results.Tables[0].TableName != String.Empty &&
                    dataset != null && dataset.Tables.Count > 0 && dataset.Tables.Contains(results.Tables[0].TableName))
                {
                    dataset.Tables.Remove(results.Tables[0].TableName);
                }

                // Initialise the passed in data set if it is null
                if (dataset == null)
                    dataset = new DataSet();

                // Now add our returned table to the collection
                if (results.Tables.Count > 0)
                    foreach (DataTable thisresult in results.Tables)
                    {
                        // Create a copy of the table
                        DataTable thiscopiedresult = thisresult.Copy();

                        // Add the copy to the new collection
                        dataset.Tables.Add(thiscopiedresult);
                    }

                // Now, return the number of rows returned in the first table (if any)
                if (results.Tables.Count > 0)
                    return results.Tables[0].Rows.Count;
                else
                    return 0;
            }
            else if (_selectcommand == null)
                throw new CubeConnectionException("The SELECT command has not been initialised. Please initialise "
                    + "it before attempting to fill a data set.", null, String.Empty);
            else
                throw new CubeConnectionException("The connection for the SELECT command has not been initialised. "
                    + "Please initialise and open the connection before trying to fill a data set.", null, String.Empty);
        }

        /// <summary>
        /// This returns a data table array with each of the axes schema from the multi-dimensional data set
        /// returned by our SELECT command, and fills the given data set with an empty table containing the
        /// schema of the returned values
        /// </summary>
        /// <param name="dataset">The data set to fill with empty schema information</param>
        /// <param name="schematype">The type of schema to fill with</param>
        /// <returns>A data table array</returns>
        public DataTable[] FillSchema(DataSet dataset, SchemaType schematype)
        {
            // Check whether our command exists and is initialised
            if (_selectcommand != null && _selectcommand.Connection != null &&
                _selectcommand.Connection.State == ConnectionState.Open)
            {
                // Our command is ready, so modify the properties of the command to only return schema, and return a
                // tabular data set
                Structures.XMLA.PropertyList originalproperties = _selectcommand.Properties;
                _selectcommand.Properties.Content = Structures.XMLA.ContentTypes.Schema;
                DataSet results = _selectcommand.ExecuteDataSet();

                // Now, if a table already exists in the given data set with the same name, delete it
                if (results.Tables.Count > 0 && results.Tables[0].TableName != String.Empty &&
                    dataset != null && dataset.Tables.Count > 0 && dataset.Tables.Contains(results.Tables[0].TableName))
                {
                    dataset.Tables.Remove(results.Tables[0].TableName);
                }

                // Initialise the passed in data set if it is null
                if (dataset == null)
                    dataset = new DataSet();

                // Now add our returned table to the collection
                if (results.Tables.Count > 0)
                    foreach (DataTable thisresult in results.Tables)
                    {
                        // Create a copy of the table
                        DataTable thiscopiedresult = thisresult.Copy();

                        // Add the copy to the new collection
                        dataset.Tables.Add(thiscopiedresult);
                    }

                // Now grab a multi-dimensional dataset to use as a base for the returned data table array
                Structures.XMLA.MDDataSet mdresults = _selectcommand.ExecuteMDDataSet();

                // Now reset our properties on the select command to their original state
                _selectcommand.Properties = originalproperties;

                // Now create an array of data tables
                if (mdresults.Axes.Count > 0)
                {
                    List<DataTable> returnedaxes = new List<DataTable>();

                    // Now loop through and add the contents of each axis to the data table array
                    foreach (Structures.XMLA.Axis thisaxis in mdresults.Axes)
                    {
                        DataTable thisaxestable = new DataTable(thisaxis.Name);
                        thisaxestable.Columns.Add("MemberName");
                        thisaxestable.Columns.Add("MemberUniqueIdentifier");
                        thisaxestable.Columns.Add("Description");
                        thisaxestable.Columns.Add("Dimension");
                        foreach (Structures.XMLA.MemberType thismember in thisaxis.Members)
                        {
                            DataRow newmemberrow = thisaxestable.NewRow();
                            newmemberrow["MemberName"] = thismember.Caption;
                            newmemberrow["MemberUniqueIdentifier"] = thismember.UName;
                            newmemberrow["Dimension"] = thismember.Dimension;
                            newmemberrow["Description"] = thismember.LName;
                            thisaxestable.Rows.Add(newmemberrow);
                        }
                        returnedaxes.Add(thisaxestable);
                    }

                    // Finally return the results
                    return returnedaxes.ToArray();
                }
                else
                {
                    // No axes were returned, so return an empty data table array
                    return new DataTable[0];
                }
            }
            else if (_selectcommand == null)
                throw new CubeConnectionException("The SELECT command has not been initialised. Please initialise "
                    + "it before attempting to fill a data set.", null, String.Empty);
            else
                throw new CubeConnectionException("The connection for the SELECT command has not been initialised. "
                    + "Please initialise and open the connection before trying to fill a data set.", null, String.Empty);
        }

        /// <summary>
        /// This returns the parameters set on the SELECT command
        /// </summary>
        /// <returns>An array of the parameters on the SELECT command</returns>
        public IDataParameter[] GetFillParameters()
        {
            if (_selectcommand != null)
            {
                CubedParameter[] fillparameters = new CubedParameter[_selectcommand.Parameters.Count];
                _selectcommand.Parameters.CopyTo(fillparameters, 0);
                return fillparameters;
            }
            else
                throw new CubeConnectionException("The SELECT command has not been initialised. Please initialise "
                    + "the command before attempting to return the parameters.", null, String.Empty);
        }

        private MissingMappingAction _missingmappingaction;
        /// <summary>
        /// The action to take when mapping information is missing
        /// </summary>
        public MissingMappingAction MissingMappingAction
        {
            get { return _missingmappingaction; }
            set { _missingmappingaction = value; }
        }

        private MissingSchemaAction _missingschemaaction;
        /// <summary>
        /// The action to take when schema information is missing
        /// </summary>
        public MissingSchemaAction MissingSchemaAction
        {
            get { return _missingschemaaction; }
            set { _missingschemaaction = value; }
        }

        /// <summary>
        /// Contains a collection of table mappings for the Cubed Adapter
        /// </summary>
        public ITableMappingCollection TableMappings
        {
            get { return null; }
            set { throw new CubeConnectionException("Table Mappings are not supported.", null, String.Empty); }
        }

        /// <summary>
        /// This updates the selected cube given the data set provided
        /// </summary>
        /// <param name="dataset">A dataset containing data to update the cube with</param>
        /// <returns>The number of rows which have been affected by the query</returns>
        public int Update(DataSet dataset)
        {
            // Initially check to ensure a valid data set has been passed through
            if (dataset == null || dataset.Tables.Count < 1 || dataset.Tables[0].Rows.Count < 1)
            {
                // This data set is empty, so throw an exception
                throw new CubeConnectionException("The given data set supplied for update is empty. Please provide "
                    + "a valid data set with at least one table with at least one row for updating.", null,
                    String.Empty);
            }
            // Check whether an Update command has been initialised
            else if (_updatecommand != null && _updatecommand.CommandText != String.Empty
                && _updatecommand.Connection != null && _updatecommand.Connection.State == ConnectionState.Open)
            {
                // For each row, we should update the parameter collection with the given
                // values and execute the command. So we should save the initial parameter values to
                // return them later
                CubedParameterCollection originalparameters = (CubedParameterCollection)_updatecommand.Parameters;

                // Grab an integer to return with the total number of rows affected
                int rowsaffected = 0;

                // An update command has been provided, so use it by assuming the update command contains
                // parameters which represent values (columns) in the given data set
                foreach (DataRow thisupdaterow in dataset.Tables[0].Rows)
                {
                    // Now we cycle through each parameter and update its' value with the value from the given
                    // data set (if it exists)
                    foreach (CubedParameter thisparameter in _updatecommand.Parameters)
                        if (dataset.Tables[0].Columns.Contains(thisparameter.ParameterName.Trim('@')))
                        {
                            // This data set contains a column with the same name as this parameter, so assume 
                            // this is the requested value
                            thisparameter.Value = thisupdaterow[thisparameter.ParameterName.Trim('@')];
                        }

                    // Now we have updated the parameters for the command, we should it execute it
                    rowsaffected += _updatecommand.ExecuteNonQuery();
                }

                // Return the original parameters collection to the update command
                _updatecommand.Parameters = originalparameters;

                // Now we have finished executing each row, return the total number of rows affected
                return rowsaffected;
            }
            else if (_selectcommand != null && _selectcommand.Connection != null
                && _selectcommand.Connection.State == ConnectionState.Open)
            {
                // No update command has been provided, but a SELECT command has been, so try and automatically
                // generate an update command based on the given data set for each row. We assume each row
                // contains the names of members of a dimension
                int rowsaffected = 0;

                // Try and determine the value column, as well as the member columns from the dimensions
                string valuecolumn = String.Empty;
                foreach (DataColumn thisupdatecolumn in dataset.Tables[0].Columns)
                {
                    // First try the data type
                    if (thisupdatecolumn.DataType == typeof(double)
                        || thisupdatecolumn.DataType == typeof(decimal))
                    {
                        // This column contains either a decimal or double value, so we assume it is the correct
                        // column for the value
                        valuecolumn = thisupdatecolumn.ColumnName;
                        break;
                    }

                    // The data type is not set, but we can try to see what values are contained in the first
                    // row
                    double test = 0;
                    decimal dtest = 0;
                    if (double.TryParse(dataset.Tables[0].Rows[0][thisupdatecolumn.ColumnName].ToString(), out test))
                    {
                        // This value can be parsed to a double, so we assume it is the correct column for
                        // the value
                        valuecolumn = thisupdatecolumn.ColumnName;
                    }
                    else if (decimal.TryParse(dataset.Tables[0].Rows[0][thisupdatecolumn.ColumnName].ToString(), out dtest))
                    {
                        // This value can be parsed to a decimal, so we assume it is the correct column for
                        // the value
                        valuecolumn = thisupdatecolumn.ColumnName;
                    }
                }

                // Check to ensure we have found a value column - if not, raise an exception
                if (valuecolumn == String.Empty)
                    throw new CubeConnectionException("No column with a suitable value type was found in the "
                        + "given dataset. Please ensure there is at least one double or decimal column.", null,
                        String.Empty);

                // Grab a schema data set from the select command
                DataSet schema = new DataSet();
                DataTable[] schemainfo = FillSchema(schema, SchemaType.Mapped);                

                foreach (DataRow thisupdaterow in dataset.Tables[0].Rows)
                {
                    // For each column (except for the value column) we grab the value, and we loop through our 
                    // schema to see if we can find the member
                    List<string> membercolumns = new List<string>();
                    foreach (DataColumn thiscolumn in dataset.Tables[0].Columns)
                        if (thiscolumn.ColumnName != valuecolumn)
                        {
                            foreach (DataTable thisaxistable in schemainfo)
                            {
                                foreach (DataRow thismemberrow in thisaxistable.Rows)
                                {
                                    string membername = thismemberrow["MemberName"].ToString().ToUpper();
                                    string rowmembername = thisupdaterow[thiscolumn.ColumnName].ToString().ToUpper();
                                    if (membername == rowmembername)
                                    {
                                        // This member is being requested to be updated, so add it's name and 
                                        // it's dimensions name to the member columns list
                                        membercolumns.Add(thismemberrow["Dimension"].ToString() + "."
                                            + thismemberrow["MemberName"].ToString());                                        
                                    }
                                }
                            }
                        }

                    // We now have a list of members to be updated, so we need to grab the value for this
                    // update operation
                    double newvalue = 0;
                    decimal tempvalue = 0;
                    bool validvalue = false;
                    if (double.TryParse(thisupdaterow[valuecolumn].ToString(), out newvalue))
                        validvalue = true;
                    else if (decimal.TryParse(thisupdaterow[valuecolumn].ToString(), out tempvalue))
                    {
                        newvalue = Convert.ToDouble(tempvalue);
                        validvalue = true;
                    }

                    // Check if we have a valid value; if we do - write it back; if not, raise an exception
                    if (validvalue)
                    {
                        // First we generate the command text
                        string commandtext = "UPDATE " + _selectcommand.Connection.Database
                            + Environment.NewLine + "SET (";
                        foreach (string thismember in membercolumns)
                            commandtext += thismember + ",";
                        commandtext = commandtext.Trim(',');
                        commandtext += ").VALUE = " + newvalue.ToString();

                        // Then we copy the SELECT command to be our UPDATE command
                        _updatecommand = _selectcommand;

                        // Then we update the command text on the UPDATE command
                        _updatecommand.CommandText = commandtext;

                        // Finally we execute the command
                        rowsaffected += _updatecommand.ExecuteNonQuery();
                    }
                    else
                        throw new CubeConnectionException("The specified value for the row was invalid.", null,
                            String.Empty);
                }

                // Now we have finished looping through the given data set, return the number of rows
                // affected
                return rowsaffected;
            }
            else
                throw new CubeConnectionException("Neither the UPDATE nor the SELECT commands have been initialised. "
                    + "Please initialise on of these before attempting to update.", null, String.Empty);
        }
    }
}