/*
 * This source file (CubedAdministration.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.IO;
using System.Reflection;
using System.Security.Cryptography;
using System.Data;
using System.Collections.Specialized;
using Bamboo.Cubed.Structures;

namespace Bamboo.Cubed.Connect
{
    /// <summary>
    /// Represents a suite of administration options used to configure a server and publish cubes
    /// </summary>
    public class CubedAdministration
    {
        #region Private variables

        /// <summary>
        /// This is the connection which the administration class uses
        /// </summary>
        private CubedConnection _connection;       
                
        /// <summary>
        /// This is an internal collection of cubes which are being worked with through this connection
        /// </summary>
        private GenericKeyCollection<Guid, Cube> _cubes;        

        #endregion

        #region Initialisation

        /// <summary>
        /// Initialises a new instance of the CubedAdministration class based on the given Cubed Connection, and
        /// retrieves the current settings from the server
        /// </summary>
        /// <param name="connection">The Cubed Connection to use for accessing the Cubed Server</param>
        public CubedAdministration(CubedConnection connection)
        {
            _connection = connection;
            _settings = _connection.ReadSettings(connection.ConnectionString);
            _cubes = new GenericKeyCollection<Guid, Cube>();
            _cubeheaders = new GenericKeyCollection<Guid, Cube>();
        }

        #endregion

        #region Accessible Collections

        private ServerSettings _settings;
        /// <summary>
        /// The settings which are currently present on the server
        /// </summary>
        public ServerSettings Settings
        {
            get { return _settings; }
            set { _settings = value; }
        }

        /// <summary>
        /// This reads the cube with the given unique identifier from the collection
        /// </summary>
        /// <param name="cubeid">The unique identifier of the cube to retrieve</param>
        /// <returns>The requested cube</returns>
        public Cube GetCube(Guid cubeid)
        {        
            // Check if the collection contains the given Cube
            if (_cubes.Contains(cubeid))
                return _cubes[cubeid];
            else if (!CubeHeaders.Contains(cubeid))
                throw new CubeConnectionException("No cube exists with the specified identifier. ", null, String.Empty);
            _cubes.Add(_connection.ReadCube(_connection.ConnectionString, cubeid));
            return _cubes[cubeid];
        }

        /// <summary>
        /// This forces the connection to get the latest copy of the cube from the server, regardless of whether
        /// the local connection cache has a copy
        /// </summary>
        /// <param name="cubeid">The unique identifier of the cube to retrieve</param>
        /// <returns>The requested cube</returns>
        public Cube GetRefreshedCube(Guid cubeid)
        {
            // Check if the collection contains the given Cube, and remove it if it does
            if (_cubes.Contains(cubeid))
                _cubes.Remove(cubeid);

            // Check that the cube exists in the headers collection (to verify it is valid)
            if (!CubeHeaders.Contains(cubeid))
                throw new CubeConnectionException("No cube exists with the specified identifier. ", null, String.Empty);
            
            // Then finally (re)read the cube into our collection and return it
            _cubes.Add(_connection.ReadCube(_connection.ConnectionString, cubeid));
            return _cubes[cubeid];
        }

        /// <summary>
        /// This modifies the current cube if it already exists, or adds the given cube to the collection
        /// </summary>
        /// <param name="cube">The cube to modify or add</param>
        public void SetCube(Cube cube)
        {
            // Check if the collection contains the given Cube
            if (_cubes.Contains(cube.ID))
                _cubes.Remove(cube.ID);
            if (CubeHeaders.Contains(cube.ID))
                CubeHeaders.Remove(cube.ID);
            _cubes.Add(cube);
            _cubeheaders.Add(cube);
        }

        private GenericKeyCollection<Guid, Cube> _cubeheaders;
        /// <summary>
        /// This contains the collection of header information for all accessible Cubes on the server
        /// </summary>
        public GenericKeyCollection<Guid, Cube> CubeHeaders
        {
            get 
            {                
                if (_connection.State == ConnectionState.Open)
                {
                    if (_cubeheaders == null || _cubeheaders.Count < 1)
                    {
                        _cubeheaders = new GenericKeyCollection<Guid, Cube>();
                        List<Cube> _cubeslist = _connection.ReadCubeHeaders(_connection.ConnectionString);
                        if (_cubeslist != null)
                            foreach (Cube thiscube in _cubeslist)
                                _cubeheaders.Add(thiscube);
                    }
                    return _cubeheaders;
                }
                else
                    return new GenericKeyCollection<Guid, Cube>();
            }
        }

        /// <summary>
        /// This returns the node which represents the collection of members and measure given
        /// </summary>
        /// <param name="cubeid">The unique identifier of the cube to return data for</param>
        /// <param name="measure">The unique identifier of the measure to return data for</param>
        /// <param name="members">The list of members to return data for</param>
        /// <returns>A node with data</returns>
        public Node GetNode(Guid cubeid, Guid measure, List<Guid> members)
        {
            return _connection.GetNode(_connection.ConnectionString, cubeid, measure,
                members);
        }

        /// <summary>
        /// This gets a data matrix containing values for the selected members for each measure in the cube
        /// </summary>
        /// <param name="cubeid">The unique identifier of the cube</param>
        /// <param name="members">The selected members</param>
        /// <returns>A data matrix containing the values for each measure</returns>
        public DataMatrix GetMeasureMatrix(Guid cubeid, List<Guid> members)
        {
            return _connection.GetMeasureMatrix(_connection.ConnectionString, cubeid,
                members);
        }

        /// <summary>
        /// This gets a data matrix containing values for the selected measure and members pivoting on the
        /// remaining dimension
        /// </summary>
        /// <param name="cubeid">The unique identifier of the cube</param>
        /// <param name="measure">The measure to read</param>
        /// <param name="otherselectedmembers">The selected members on all dimensions except the pivot dimension</param>
        /// <returns>The given data matrix</returns>
        public DataMatrix GetMemberMatrix(Guid cubeid, Guid measure, List<Guid> otherselectedmembers)
        {
            return _connection.GetMemberMatrix(_connection.ConnectionString, cubeid,
                measure, otherselectedmembers);
        }

        #endregion

        #region Data Saving Functions

        /// <summary>
        /// This sets the node's value in the data matrix for the given cube
        /// </summary>
        /// <param name="cube">The cube to set the value in</param>
        /// <param name="node">The node to save</param>
        /// <returns>True if saved successfully, false if there was a problem</returns>
        public bool SetNode(Cube cube, Node node)
        {
            return _connection.SetNode(_connection.ConnectionString, cube, node);
        }

        /// <summary>
        /// Saves the server settings back to the Cube
        /// </summary>
        /// <param name="settings">The settings to save</param>
        /// <returns>True if successful, false if there was a problem</returns>
        public bool SaveSettings(ServerSettings settings)
        {
            return _connection.WriteSettings(settings, _connection.ConnectionString);
        }

        /// <summary>
        /// This saves any changes to the cube with the given unique identifier to the server
        /// </summary>
        /// <param name="cubeid">The unique identifier of the cube to save</param>
        /// <returns>True if the save was successful, false if not</returns>
        public bool SaveCube(Guid cubeid)
        {
            if (_cubes.Contains(cubeid))
            {
                // First we determine which parts of the cube have been changed 
                _connection.SaveCube(_connection.ConnectionString, _cubes[cubeid]);

                //if (_cubes[cubeid].IsDirty)
                //    _connection.ClientConnection.SaveCubeDetails(_cubes[cubeid], _connection.ConnectionString);
                
                //if (_cubes[cubeid].DatasourcesDirty)
                //    foreach (Datasource thisds in _cubes[cubeid].Datasources)
                //        if (thisds.IsDirty)
                //            _connection.ClientConnection.SaveDatasource(thisds, cubeid, _connection.ConnectionString);

                //if (_cubes[cubeid].MeasuresDirty)
                //    foreach (Measure thismeasure in _cubes[cubeid].Measures)
                //        if (thismeasure.IsDirty)
                //            _connection.ClientConnection.SaveMeasure(thismeasure, cubeid, _connection.ConnectionString);

                //if (_cubes[cubeid].DimensionsDirty)
                //    foreach (Dimension thisdimension in _cubes[cubeid].Dimensions)
                //    {
                //        if (thisdimension.IsDirty)
                //            _connection.ClientConnection.SaveDimensionDetails(thisdimension, cubeid, _connection.ConnectionString);

                //        if (thisdimension.MembersDirty)
                //            foreach (Member thismember in thisdimension.Members)
                //                if (thismember.IsDirty)
                //                    _connection.ClientConnection.SaveMember(thismember, _connection.ConnectionString);
                //    }
                
                // Then we return true
                return true;
            }
            else
                return false;
        }

        /// <summary>
        /// This saves any changes to any of the cubes in the collection to the server
        /// </summary>
        /// <returns>True if all cube saves were successful, false if not</returns>
        public bool SaveCubes()
        {
            bool allok = true;
            foreach (Cube thiscube in _cubes)
                if (!SaveCube(thiscube.ID))
                    allok = false;
            return allok;
        }      

        #endregion

        #region Deletion Functions

        /// <summary>
        /// Removes a cube, and all aggregated data associated with it, from the server
        /// </summary>
        /// <param name="cube">The Cube to remove from the server</param>
        /// <returns>True if the cube was removed, false if there was a problem</returns>
        public bool RemoveCube(Cube cube)
        {
            if (_cubes.Contains(cube.ID))
                _cubes.Remove(cube.ID);
            if (_cubeheaders.Contains(cube.ID))
                _cubeheaders.Remove(cube.ID);
            return _connection.RemoveCube(_connection.ConnectionString, cube.ID);
        }

        /// <summary>
        /// This removes the cube and all associated data from the server
        /// </summary>
        /// <param name="cubeid">The unique identifier of the cube to return</param>
        /// <returns>True if the cube was removed, false if there was a problem</returns>
        public bool RemoveCube(Guid cubeid)
        {
            if (_cubes.Contains(cubeid))
                _cubes.Remove(cubeid);
            if (_cubeheaders.Contains(cubeid))
                _cubeheaders.Remove(cubeid);
            return _connection.RemoveCube(_connection.ConnectionString, cubeid);
        }

        #endregion
        
        #region Consolidation Functions

        /// <summary>
        /// Runs a consolidation of all cubes on the server
        /// </summary>
        /// <returns>The newly started Consolidation object</returns>
        public void Consolidate()
        {
            _connection.StartFullConsolidation(_connection.ConnectionString, false);
        }

        /// <summary>
        /// Runs a consolidation of the specified cube on the server
        /// </summary>
        /// <param name="cube">The cube to consolidate</param>
        /// <param name="pointconsolidation">Whether to start a full or partial (point) consolidation</param>
        /// <returns>The newly started Consolidation object</returns>
        public void Consolidate(Cube cube, bool pointconsolidation)
        {
            _connection.StartSingleConsolidation(_connection.ConnectionString, cube.ID,
                pointconsolidation);
        }

        /// <summary>
        /// Runs a consolidation of the specified cube on the server
        /// </summary>
        /// <param name="cubeid">The unique identifier of the cube to consolidate</param>
        /// <param name="pointconsolidation">Whether to start a full or partial (point) consolidation</param>
        /// <returns>The newly started Consolidation object</returns>
        public void Consolidate(Guid cubeid, bool pointconsolidation)
        {
            _connection.StartSingleConsolidation(_connection.ConnectionString, cubeid,
                pointconsolidation);
        }

        /// <summary>
        /// This cancels the consolidation currently in progress (whether full or partial), if any are
        /// in progress
        /// </summary>
        public void CancelConsolidation()
        {
            _connection.CancelConsolidation(_connection.ConnectionString);
        }

        #endregion

        #region Desktop Management Functions

        /// <summary>
        /// Reads the dashboard with the given unique identifier
        /// </summary>
        /// <param name="dashboardid">The unique identifier of the dashboard to return</param>
        /// <returns>The dashboard if it has been found</returns>
        public DesktopDashboard ReadDashboard(Guid dashboardid)
        {
            return _connection.ReadDashboard(_connection.ConnectionString, dashboardid);
        }

        /// <summary>
        /// Deletes the search with the given unique identifier
        /// </summary>
        /// <param name="searchid">The unique identifier of the search to delete</param>
        /// <returns>True if deleted, otherwise an error</returns>
        public bool DeleteSearch(Guid searchid)
        {
            return _connection.DeleteSearch(_connection.ConnectionString, searchid);
        }
        
        /// <summary>
        /// Deletes the dashboard with the given unique identifier
        /// </summary>
        /// <param name="dashboardid">The unique identifier of the dashboard to delete</param>
        /// <returns>True if deleted, else false</returns>
        public bool DeleteDashboard(Guid dashboardid)
        {
            return _connection.DeleteDashboard(_connection.ConnectionString, dashboardid);
        }

        #endregion

        #region Security Functions

        /// <summary>
        /// Adds a new operating system user to the "root users" group of the server
        /// </summary>
        /// <param name="newrootuser">The Operating System account (with full domain) of the user to set 
        /// as an administrator of the server</param>
        /// <returns>True if the user was added, false if there was a problem</returns>           
        public bool AddRootUser(string newrootuser)
        {
            return _connection.AddRootUser(_connection.ConnectionString, newrootuser);
        }

        /// <summary>
        /// Removes a user from being a member of the "root users" group of the server
        /// </summary>
        /// <param name="rootuser">The Operating System account (with full domain) of the user to remove from the
        /// administrators group</param>
        /// <returns>True if the user was added, else an exception will be thrown</returns>
        public bool RemoveRootUser(string rootuser)
        {
            return _connection.RemoveRootUser(_connection.ConnectionString, rootuser);
        }

        /// <summary>
        /// Returns the current system roles in the system
        /// </summary>
        /// <returns>The current list of roles</returns>
        public List<Role> ReadRoles()
        {
            return _connection.ReadRoles(_connection.ConnectionString);
        }

        /// <summary>
        /// Returns the current users in the system
        /// </summary>
        /// <returns>The current list of system users</returns>
        public List<User> ReadUsers()
        {
            return _connection.ReadUsers(_connection.ConnectionString);
        }

        /// <summary>
        /// Saves the given roles collection back to the system
        /// </summary>
        /// <param name="roles">The roles to save</param>
        /// <returns>True if saved successfully, false if a problem occured</returns>
        public bool SaveRoles(List<Role> roles)
        {
            return _connection.WriteRoles(_connection.ConnectionString, roles);
        }

        /// <summary>
        /// Saves the given users collection back to the system
        /// </summary>
        /// <param name="users">The users to save</param>
        /// <returns>True if saved successfully, false if a problem occured</returns>
        public bool SaveUsers(List<User> users)
        {
            return _connection.WriteUsers(_connection.ConnectionString, users);
        }

        #endregion

        #region Import/Export Functions

        /// <summary>
        /// Exports all current users of the system to a byte array ready for saving
        /// </summary>
        /// <returns>A byte array ready for saving to disk</returns>
        public byte[] ExportUsers()
        {
            return _connection.ExportUsers(_connection.ConnectionString);
        }

        /// <summary>
        /// Imports all users contained in the given export into the system
        /// </summary>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>The number of records imported</returns>
        public int ImportUsers(byte[] export)
        {
            return _connection.ImportUsers(_connection.ConnectionString, export);
        }

        /// <summary>
        /// Exports all current roles of the system to a byte array ready for saving
        /// </summary>
        /// <returns>A byte array ready for saving to disk</returns>
        public byte[] ExportRoles()
        {
            return _connection.ExportRoles(_connection.ConnectionString);
        }

        /// <summary>
        /// Imports all roles contained in the given export into the system
        /// </summary>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>The number of records imported</returns>
        public int ImportRoles(byte[] export)
        {
            return _connection.ImportRoles(_connection.ConnectionString, export);
        }

        /// <summary>
        /// Exports all current searches of the system to a byte array ready for saving
        /// </summary>
        /// <returns>A byte array ready for saving to disk</returns>
        public byte[] ExportSearches()
        {
            return _connection.ExportSearches(_connection.ConnectionString);
        }

        /// <summary>
        /// Imports all searches contained in the given export into the system
        /// </summary>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>The number of records imported</returns>
        public int ImportSearches(byte[] export)
        {
            return _connection.ImportSearches(_connection.ConnectionString, export);
        }

        /// <summary>
        /// Exports all current dashboards of the system to a byte array ready for saving
        /// </summary>
        /// <returns>A byte array ready for saving to disk</returns>
        public byte[] ExportDashboards()
        {
            return _connection.ExportDashboards(_connection.ConnectionString);
        }

        /// <summary>
        /// Imports all dashboards contained in the given export into the system
        /// </summary>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>The number of records imported</returns>
        public int ImportDashboards(byte[] export)
        {
            return _connection.ImportDashboards(_connection.ConnectionString, export);
        }

        /// <summary>
        /// Exports the specified cube to a byte array ready for saving
        /// </summary>
        /// <param name="cubeid">The unique identifier of the cube to export</param>
        /// <returns>A bute array ready for saving to disk</returns>
        public byte[] ExportCube(Guid cubeid)
        {
            return _connection.ExportCube(_connection.ConnectionString, cubeid);
        }

        /// <summary>
        /// Imports the cube in the given export into the system
        /// </summary>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>True if imported successfully, false if not</returns>
        public bool ImportCube(byte[] export)
        {
            return _connection.ImportCube(_connection.ConnectionString, export);
        }

        #endregion

        #region Version Details

        /// <summary>
        /// This method returns the version of the Cubed Server
        /// </summary>
        /// <returns>Cubed Server version</returns>
        public string Version()
        {
            return _connection.Version();
        }

        /// <summary>
        /// This returns the true, full build version of the Cubed Server
        /// </summary>
        /// <returns>The full build version</returns>
        public string TrueVersion()
        {
            return _connection.TrueVersion();
        }

        /// <summary>
        /// This returns the full Release details of the Cubed Server
        /// </summary>
        /// <returns>The full release details</returns>
        public string Release()
        {
            return _connection.Release();
        }

        /// <summary>
        /// This returns the header of the current release of the Cubed Server
        /// </summary>
        /// <returns>The header of the current release</returns>
        public string ReleaseHeader()
        {
            return _connection.ReleaseHeader();
        }

        /// <summary>
        /// This returns the details of the current release of the Cubed Server
        /// </summary>
        /// <returns>The details of the current release</returns>
        public string ReleaseDetails()
        {
            return _connection.ReleaseDetails();
        }

        #endregion
    }
}