/*
 * This source file (CubedCommand.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;
using System.IO;
using Bamboo.Cubed.Utilities;

namespace Bamboo.Cubed.Connect
{
    /// <summary>
    /// This provides access to execute commands against a Cubed Server
    /// </summary>
    public class CubedCommand : IDbCommand
    {
        #region Properties and Constructors

        /// <summary>
        /// This initialises an empty, default Cubed Command
        /// </summary>
        public CubedCommand()
        {
            _command = new Structures.XMLA.Command();
            _commandtext = String.Empty;
            _connection = null;
            _commandtype = System.Data.CommandType.Text;
            _commandtimeout = 300;
            _parameters = new CubedParameterCollection();
            _properties = new Structures.XMLA.PropertyList();
            _restrictions = new Structures.XMLA.RestrictionList();
        }

        /// <summary>
        /// Initialises a new Cubed Cubed with the command text and connection given
        /// </summary>
        /// <param name="commandtext">The Command Text for this command</param>
        /// <param name="connection">The Cubed Connection to use for this command</param>
        public CubedCommand(string commandtext, CubedConnection connection)
        {
            _command = new Structures.XMLA.Command();
            _commandtext = commandtext;
            _connection = connection;
            _commandtype = System.Data.CommandType.Text;
            _commandtimeout = 300;
            _parameters = new CubedParameterCollection();
            _properties = new Structures.XMLA.PropertyList();
            _restrictions = new Structures.XMLA.RestrictionList();
        }

        /// <summary>
        /// This cancels our command preparation and destroys the current connection
        /// </summary>
        public void Cancel()
        {
            _command = null;
            _connection = null;
            _commandtext = String.Empty;
        }

        private Structures.XMLA.Command _command;
        /// <summary>
        /// The internally prepared command which contains parsed parameters
        /// </summary>
        public  Structures.XMLA.Command Command
        {
            get { return _command; }
            set { _command = value; }
        }

        private string _commandtext;
        /// <summary>
        /// This is the text of the command issued against Cubed
        /// </summary>
        public string CommandText
        {
            get { return _commandtext; }
            set { _commandtext = value; }
        }

        private int _commandtimeout;
        /// <summary>
        /// This is the timeout for command connections
        /// </summary>
        public int CommandTimeout
        {
            get { return _commandtimeout; }
            set { _commandtimeout = value; }
        }

        private CommandType _commandtype;
        /// <summary>
        /// This is the type of command being executed
        /// </summary>
        public CommandType CommandType
        {
            get { return _commandtype; }
            set 
            {
                // Only text commands are supported, so throw an exception if another type is specified
                _commandtype = value;
                if (_commandtype != CommandType.Text)
                {
                    _commandtype = CommandType.Text;
                    // Throw an exception
                    throw new CubeConnectionException("Only TEXT commands are supported.", null, String.Empty);
                }
            }
        }

        private CubedConnection _connection;
        /// <summary>
        /// This is the connection attached to this command
        /// </summary>
        public IDbConnection Connection
        {
            get { return _connection; }
            set { _connection = (CubedConnection)value; }
        }

        /// <summary>
        /// This returns an empty new Cubed Parameter
        /// </summary>
        /// <returns>The parameter to return</returns>
        public IDbDataParameter CreateParameter()
        {
            return new CubedParameter();
        }

        private Structures.XMLA.PropertyList _properties;
        /// <summary>
        /// These are the XML for Analysis (XMLA) properties which are set on the command, defining how
        /// the command will be executed against the cube
        /// </summary>
        public Structures.XMLA.PropertyList Properties
        {
            get { return _properties; }
            set { _properties = value; }
        }

        private Structures.XMLA.RestrictionList _restrictions;
        /// <summary>
        /// The restrictions on this command
        /// </summary>
        public Structures.XMLA.RestrictionList Restrictions
        {
            get { return _restrictions; }
            set { _restrictions = value; }
        }

        private CubedTransaction _transaction;
        /// <summary>
        /// This is the transaction connection to the command
        /// </summary>
        public IDbTransaction Transaction
        {
            get { return _transaction; }
            set { _transaction = (CubedTransaction)value; }
        }

        private UpdateRowSource _updatedrowsource;
        /// <summary>
        /// This is the updated row source setting for the command
        /// </summary>
        public UpdateRowSource UpdatedRowSource
        {
            get { return _updatedrowsource; }
            set { _updatedrowsource = value; }
        }

        /// <summary>
        /// This closes and destroys the command
        /// </summary>
        public void Dispose()
        {
            Cancel();
        }

        private CubedParameterCollection _parameters;
        /// <summary>
        /// The collection of parameters attached to the command
        /// </summary>
        public virtual IDataParameterCollection Parameters
        {
            get { return _parameters; }
            set { _parameters = (CubedParameterCollection)value; }
        }

        #endregion

        #region Multi-dimensional Query Execution

        /// <summary>
        /// This executes the command against the data source and returns the number of cells affected. It 
        /// should be used for update commands
        /// </summary>
        /// <returns>The number of records affected by the given command</returns>
        public int ExecuteNonQuery()
        {
            // First check to ensure that a command text has been specified
            if (_commandtext != null && _commandtext != String.Empty && _properties != null)
            {
                // The command text exists, so prepare our command object based on it
                Prepare();

                // Now, check whether our connection is initialised and open
                if (_connection != null && _connection.State == ConnectionState.Open)
                {
                    // Our connecton is prepared, so attempt to execute our command and return the result
                    return Convert.ToInt32(_connection.Execute(_command, _properties));
                }
                else
                    throw new CubeConnectionException("The connection has not been opened. Please open the connection "
                        + "before trying to execute the command.", null, String.Empty);
                
            }
            else if (_properties == null)
                throw new CubeConnectionException("The properties of this command have not been set. Please supply "
                    + "them before initiating the command.", null, String.Empty);
            else
                throw new CubeConnectionException("The command text is empty. Please supply a command text before "
                    + "attempting to execute a command.", null, String.Empty);
        }

        /// <summary>
        /// Executes this command as a SELECT query and returns a tabular data set
        /// </summary>
        /// <returns>A tabular data set of results from the query</returns>
        public DataSet ExecuteDataSet()
        {
            // First check to ensure that a command text has been specified
            if (_commandtext != null && _commandtext != String.Empty && _properties != null)
            {
                // The command text exists, so prepare our command object based on it
                Prepare();

                // Now, check whether our connection is initialised and open
                if (_connection != null && _connection.State == ConnectionState.Open)
                {
                    // Our connection is prepared, so lets make a copy of the properties for the command
                    // and explicitly specify a tabular data set
                    Structures.XMLA.PropertyList newproperties = _properties;
                    newproperties.Format = Structures.XMLA.FormatTypes.Tabular;
                    newproperties.Content = Structures.XMLA.ContentTypes.SchemaData;
                    _command.Statement = _commandtext;

                    // Now execute our command and return the result
                    string datasetxml = (string)_connection.Execute(_command, newproperties);
                    DataSet ds;
                    if (datasetxml == null || datasetxml == String.Empty)
                        ds = new DataSet();
                    else
                    {
                        ds = new DataSet();
                        StringReader sr = new StringReader(datasetxml);
                        ds.ReadXml(sr);
                        sr.Close();
                    }                                            
                    return ds;                 
                }
                else
                    throw new CubeConnectionException("The connection has not been opened. Please open the connection "
                        + "before trying to execute the command.", null, String.Empty);

            }
            else if (_properties == null)
                throw new CubeConnectionException("The properties of this command have not been set. Please supply "
                    + "them before initiating the command.", null, String.Empty);
            else
                throw new CubeConnectionException("The command text is empty. Please supply a command text before "
                    + "attempting to execute a command.", null, String.Empty);
        }
        
        /// <summary>
        /// Executes this command as a SELECT query and returns a multi-dimensional data set
        /// </summary>
        /// <returns>A multi-dimensional data set of results from the query</returns>
        public Structures.XMLA.MDDataSet ExecuteMDDataSet()
        {
            // First check to ensure that a command text has been specified
            if (_commandtext != null && _commandtext != String.Empty && _properties != null)
            {
                // The command text exists, so prepare our command object based on it
                Prepare();

                // Now, check whether our connection is initialised and open
                if (_connection != null && _connection.State == ConnectionState.Open)
                {
                    // Our connection is prepared, so lets make a copy of the properties for the command
                    // and explicitly specify a multi-dimensional data set
                    Structures.XMLA.PropertyList newproperties = _properties;
                    newproperties.Format = Structures.XMLA.FormatTypes.Multidimensional;
                    newproperties.Content = Structures.XMLA.ContentTypes.SchemaData;
                    _command.Statement = _commandtext;

                    // Now execute our command and return the result
                    string mddatasetxml = (string)_connection.Execute(_command, newproperties);
                    Structures.XMLA.MDDataSet mds;
                    if (mddatasetxml == null || mddatasetxml == String.Empty)
                        mds = new Structures.XMLA.MDDataSet();
                    else
                        mds = XMLManager<Structures.XMLA.MDDataSet>.DeserialiseXML(mddatasetxml);                    
                    return mds;                 
                }
                else
                    throw new CubeConnectionException("The connection has not been opened. Please open the connection "
                        + "before trying to execute the command.", null, String.Empty);

            }
            else if (_properties == null)
                throw new CubeConnectionException("The properties of this command have not been set. Please supply "
                    + "them before initiating the command.", null, String.Empty);
            else
                throw new CubeConnectionException("The command text is empty. Please supply a command text before "
                    + "attempting to execute a command.", null, String.Empty);
        }

        #endregion

        #region Relational Query Execution

        /// <summary>
        /// Executes this command as a SELECT query against a relational data source (set by data source info)
        /// against the given cube (set by cube)
        /// </summary>
        /// <returns>A tabular data set of results from the query</returns>
        public DataSet ExecuteRelationalDataSet()
        {
            // First check to ensure that a command text has been specified
            if (_commandtext != null && _commandtext != String.Empty && _properties != null)
            {
                // The command text exists, so prepare our command object based on it
                Prepare();

                // Now, check whether our connection is initialised and open
                if (_connection != null && _connection.State == ConnectionState.Open)
                {
                    // Our connection is prepared, so lets make a copy of the properties for the command
                    // and explicitly specify a tabular data set
                    Structures.XMLA.PropertyList newproperties = _properties;
                    newproperties.Format = Structures.XMLA.FormatTypes.Tabular;
                    newproperties.Content = Structures.XMLA.ContentTypes.SchemaData;
                    _command.Statement = _commandtext;

                    // Now execute our command and return the result                    
                    string datasetxml = (string)_connection.ExecuteRelational(_command, newproperties);
                    DataSet ds;
                    if (datasetxml == null || datasetxml == String.Empty)
                        ds = new DataSet();
                    else
                    {
                        ds = new DataSet();
                        StringReader sr = new StringReader(datasetxml);
                        ds.ReadXml(sr,XmlReadMode.InferTypedSchema);
                        sr.Close();
                    }
                    return ds;      
                }
                else
                    throw new CubeConnectionException("The connection has not been opened. Please open the connection "
                        + "before trying to execute the command.", null, String.Empty);
            }
            else if (_properties == null)
                throw new CubeConnectionException("The properties of this command have not been set. Please supply "
                    + "them before initiating the command.", null, String.Empty);
            else
                throw new CubeConnectionException("The command text is empty. Please supply a command text before "
                    + "attempting to execute a command.", null, String.Empty);
        }

        /// <summary>
        /// Executes this command as a SELECT query against a relational data source (set by data source info)
        /// against the given cube (set by cube)
        /// </summary>
        /// <returns>A multi-dimensional data set of results from the query</returns>
        public Structures.XMLA.MDDataSet ExecuteRelationalMDDataSet()
        {
            // First check to ensure that a command text has been specified
            if (_commandtext != null && _commandtext != String.Empty && _properties != null)
            {
                // The command text exists, so prepare our command object based on it
                Prepare();

                // Now, check whether our connection is initialised and open
                if (_connection != null && _connection.State == ConnectionState.Open)
                {
                    // Our connection is prepared, so lets make a copy of the properties for the command
                    // and explicitly specify a multi-dimensional data set
                    Structures.XMLA.PropertyList newproperties = _properties;
                    newproperties.Format = Structures.XMLA.FormatTypes.Multidimensional;
                    newproperties.Content = Structures.XMLA.ContentTypes.SchemaData;
                    _command.Statement = _commandtext;

                    // Now execute our command and return the result
                    string mddatasetxml = (string)_connection.ExecuteRelational(_command, newproperties);
                    Structures.XMLA.MDDataSet mds;
                    if (mddatasetxml == null || mddatasetxml == String.Empty)
                        mds = new Structures.XMLA.MDDataSet();
                    else
                        mds = XMLManager<Structures.XMLA.MDDataSet>.DeserialiseXML(mddatasetxml);
                    return mds;     
                }
                else
                    throw new CubeConnectionException("The connection has not been opened. Please open the connection "
                        + "before trying to execute the command.", null, String.Empty);

            }
            else if (_properties == null)
                throw new CubeConnectionException("The properties of this command have not been set. Please supply "
                    + "them before initiating the command.", null, String.Empty);
            else
                throw new CubeConnectionException("The command text is empty. Please supply a command text before "
                    + "attempting to execute a command.", null, String.Empty);
        }

        #endregion

        #region Information Execution Methods

        /// <summary>
        /// Executes the given schema data set command and returns a data set containing the requested
        /// schema
        /// </summary>
        /// <param name="schemarequest">The definition of the schema to return</param>
        /// <returns>A tabular data set of schema informaion</returns>        
        public DataSet GetSchemaDataSet(Structures.XMLA.RequestTypes schemarequest)
        {
           // Ensure that properties and restrictions have been supplied
            if (_properties != null && _restrictions != null)
            {
                if (_connection != null && _connection.State == ConnectionState.Open)
                {
                    string datasetxml = (string)_connection.Discover(schemarequest,
                        _restrictions, _properties);
                    DataSet ds;
                    if (datasetxml == null || datasetxml == String.Empty)
                        ds = new DataSet();
                    else
                    {
                        ds = new DataSet();
                        StringReader sr = new StringReader(datasetxml);
                        ds.ReadXml(sr);
                        sr.Close();
                    }
                    return ds;      
                }
                else
                    throw new CubeConnectionException("The connection has not been opened. Please open the connection "
                        + "before trying to execute the command.", null, String.Empty);
            }
            else
                throw new CubeConnectionException("Both the properties and restrictions needs to be specified.", null, String.Empty);                       
        }

        #endregion

        #region Other Execution Methods

        /// <summary>
        ///  This executes the command as a SELECT query and returns a fast-forward only data reader
        /// </summary>
        /// <param name="behavior">The behaviour of the reader with the command</param>
        /// <returns>A Data reader ready to loop through the resturned records</returns>
        public IDataReader ExecuteReader(CommandBehavior behavior)
        {
            // First we execute our command internally and grab the data set of results
            DataSet results = ExecuteDataSet();

            // If we have returned a correct set of results, we create a new data reader with those results
            CubedDataReader cdr = new CubedDataReader(results);           

            // Before returning the result set, check our behaviour to see what needs to happen
            if (behavior == CommandBehavior.CloseConnection)
                _connection.Close();           
            
            // Now return the data set
            return cdr;
        }

        /// <summary>
        /// This executes the command and returns a new data reader for fast-forward only access through the returned
        /// results
        /// </summary>
        /// <returns>A Data reader ready to loop through the returned records</returns>
        public IDataReader ExecuteReader()
        {
            // First we execute our command internally and grab the data set of results
            DataSet results = ExecuteDataSet();

            // If we have returned a correct set of results, we create a new data reader with those results
            CubedDataReader cdr = new CubedDataReader(results);

            // Now return the data set
            return cdr;
        }

        /// <summary>
        /// This executes the command and returns the first column of the first row returned in the data set
        /// </summary>
        /// <returns>The first column of the first row in the returned data set</returns>
        public object ExecuteScalar()
        {
            // First we execute our command internally and grab the data set of results
            DataSet results = ExecuteDataSet();

            // Then, if the command returned any results, return the first column of the first row
            if (results.Tables.Count > 0 && results.Tables[0].Rows.Count > 0)
            {
                // A data set was returned with at least one row, so return the first column of the first row
                return results.Tables[0].Rows[0][0];
            }
            else
                return null;
        }       

        /// <summary>
        /// This prepares the command internally by parsing parameters and checking the connection to the server, 
        /// as well as checking our command text has been set
        /// </summary>
        public void Prepare()
        {
            // To prepare our command, we first need to check it is valid
            if (_commandtext != null && _commandtext != String.Empty && _properties != null)
            {
                // Now, check whether our connection is initialised and open
                if (_connection != null && _connection.State == ConnectionState.Open)
                {
                    // The command is ready, so parse our parameters
                    _command.Statement = _command.Statement.ToUpper();
                    foreach (CubedParameter thisparameter in _parameters)
                    {
                        if (_command.Statement.Contains(thisparameter.ParameterName.ToUpper()))
                        {
                            // This parameter is present in our command text, so replace it with our value
                            _command.Statement.Replace(thisparameter.ParameterName.ToUpper(),
                                thisparameter.Value.ToString());
                        }
                    }
                }
                else
                    throw new CubeConnectionException("The connection has not been opened. Please open the connection "
                        + "before trying to execute the command.", null, String.Empty);

            }
            else if (_properties == null)
                throw new CubeConnectionException("The properties of this command have not been set. Please supply "
                    + "them before initiating the command.", null, String.Empty);
            else
                throw new CubeConnectionException("The command text is empty. Please supply a command text before "
                    + "attempting to execute a command.", null, String.Empty);
        }
        #endregion
    }
}
