/*
 * This source file (CubedConnection.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures;
using System.ServiceModel;
using Bamboo.Cubed.Engine;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel.Description;
using System.Security.Principal;

namespace Bamboo.Cubed.Connect
{
    /// <summary>
    /// This provides a connection to a Cubed Server
    /// </summary>
    public class CubedConnection : IDbConnection
    {
        #region Initialisation

        /// <summary>
        /// Initialises a default new Cubed Connection
        /// </summary>
        public CubedConnection()
        {
            _cube = null;
            _administration = null;
            _clientconnection = null;
            _connectiondetails = null;
            _transaction = null;
            _connectionstring = String.Empty;
            _state = ConnectionState.Closed;
            _connectiontimeout = 300;
            _connectionlastactivity = DateTime.Now;            
        }

        /// <summary>
        /// Initialises a new connection with the connection string given
        /// </summary>
        /// <param name="connectionstring">The connection string to connect to the Cubed server</param>
        public CubedConnection(string connectionstring)
        {
            _cube = null;
            _administration = null;
            _clientconnection = null;
            _connectiondetails = null;
            _transaction = null;
            _connectionstring = connectionstring;
            _state = ConnectionState.Closed;
            _connectiontimeout = 300;
            _connectionlastactivity = DateTime.Now;
        }

        #endregion

        #region Internal Properties

        private Cube _cube;
        /// <summary>
        /// The currently active cube (if this is not an Admin connection), otherwise it will return null
        /// </summary>
        public Cube Cube
        {
            get { return _cube; }
            set { _cube = value; }
        }        

        private CubedAdministration _administration;
        /// <summary>
        /// This is the Cubed Administration interface for the currently active connection. This will only be
        /// available if an "ADMIN" connection has been established to the server, otherwise it will return
        /// null
        /// </summary>
        public CubedAdministration Administration
        {
            get { return _administration; }
            set { _administration = value; }
        }

        /// <summary>
        /// This is the underlying web service reference connection. It should only be used internally
        /// </summary>
        private CubedClient _clientconnection;

        private ConnectionParameters _connectiondetails;
        /// <summary>
        /// The current details of the connection (passed back from the server when a valid connection string is
        /// passed)
        /// </summary>
        internal ConnectionParameters ConnectionDetails
        {
            get { return _connectiondetails; }
            set { _connectiondetails = value; }
        }

        /// <summary>
        /// If this connection has a transaction opened, this is the currently open transaction
        /// </summary>
        private CubedTransaction _transaction;

        /// <summary>
        /// This stores the last activity of the connection. If this was greater than the timeout period before
        /// now, any actions will be terminated
        /// </summary>
        private DateTime _connectionlastactivity;

        private string _connectionstring;
        /// <summary>
        /// The current connection string used to connect to the server
        /// </summary>
        public string ConnectionString
        {
            get { return _connectionstring; }
            set { _connectionstring = value; }
        }

        private int _connectiontimeout;
        /// <summary>
        /// The current timeout of the connection
        /// </summary>
        public int ConnectionTimeout
        {
            get { return _connectiontimeout; }
        }

        private string _database;
        /// <summary>
        /// The cube which has been connected to. This will be blank if this is an administration connection.
        /// </summary>
        public string Database
        {
            get { return _database; }
        }

        private ConnectionState _state;
        /// <summary>
        /// Returns the current state of the connection
        /// </summary>
        public ConnectionState State
        {
            get { return _state; }
        }

        #endregion

        #region DB Connection Methods

        /// <summary>
        /// This checks the current time out compared with the last connection activity. If the connection has
        /// elapsed, an exception will be thrown; otherwise the timer will be updated
        /// </summary>
        internal void CheckTimeout()
        {
            if (_connectiontimeout > 0 &&
                DateTime.Now.AddSeconds(_connectiontimeout * -1) > _connectionlastactivity)
            {
                // The connection has elapsed, so throw an exception
                throw new CubeConnectionException("The connection has elapsed. Please re-connect to the server.",
                    null, String.Empty);
            }
            else
            {
                // The time is fine, so update it
                _connectionlastactivity = DateTime.Now;
            }
        }

        /// <summary>
        /// Begin a new transaction with the given isolation level
        /// </summary>
        /// <param name="isolation">The isolation level to open the connection with</param>
        /// <returns>The new transaction object</returns>
        public IDbTransaction BeginTransaction(IsolationLevel isolation)
        {
            // Check the current time out period
            CheckTimeout();

            if (_transaction == null)
            {
                // There is no open transaction, so open one
                _transaction = new CubedTransaction(this);
                _transaction.IsolationLevel = isolation;
                return _transaction;
            }
            else
            {
                // There is a currently open transaction, so return an exception
                throw new CubeConnectionException("There is already an open transaction. Another cannot be opened until this is "
                    + "rolled back or committed.", null, String.Empty);
            }
        }

        /// <summary>
        /// Begin a new transaction with the default isolation level
        /// </summary>
        /// <returns>The new transaction object</returns>
        public IDbTransaction BeginTransaction()
        {
            // Check the current time out period
            CheckTimeout();
            
            if (_transaction == null)
            {
                // There is no open transaction, so open one
                _transaction = new CubedTransaction(this);
                _transaction.IsolationLevel = IsolationLevel.ReadCommitted;
                return _transaction;
            }
            else
            {
                // There is a currently open transaction, so return an exception
                throw new CubeConnectionException("There is already an open transaction. Another cannot be opened until this is "
                    + "rolled back or committed.", null, String.Empty);
            }
        }

        /// <summary>
        /// Changes the currently connected cube to the new one given
        /// </summary>
        /// <remarks>If there is a currently open transaction, this will automatically be committed</remarks>
        /// <param name="cube">The name of the new cube to open</param>
        /// <returns>True if the database was changed successfully</returns>
        public void ChangeDatabase(string cube)
        {
            // Check the current time out period
            CheckTimeout();
            
            // If there is a currently open transaction, automatically commit it
            if (_transaction != null)
            {
                _transaction.Commit();
                _transaction = null;
            }
            
            // Then check the connection state, if it's open - close it 
            if (_state == ConnectionState.Open)
               Close();            

            // Next, reconfigure the connection parameters for the new database
            _connectiondetails.Cube = cube;

            // Then try and re-connect to the server with the new connection
            _connectiondetails = _clientconnection.CheckAuthenticatedUser(_connectiondetails.ConnectionString);

            // Check whether the database was successfully opened
            if (_connectiondetails == null)
                throw new CubeConnectionException("A connection to the server could not be established.",
                    null, String.Empty);

            // Now that it was successfully opened, see if the cube exists
            List<Cube> cubes = _clientconnection.ReadCubes(_connectiondetails.ConnectionString);
            foreach (Cube thiscube in cubes)
                if (thiscube.Name.ToUpper().Equals(cube.ToUpper()))
                {
                    // We have found the cube, so we change the database
                    _database = thiscube.Name;
                    _cube = thiscube;
                }

            // The cube wasn't found, so raise an exception
            throw new CubeConnectionException("The specified Cube could not be found on the server.",
                null, String.Empty);
        }

        /// <summary>
        /// Closes the currently active connection. If any transactions are open they will be closed 
        /// and committed automatically.
        /// </summary>
        public void Close()
        {
            // First check whether there was an open transaction
            if (_transaction != null)
            {
                _transaction.Commit();
                _transaction = null;
            }
            
            _state = ConnectionState.Closed;
            _database = String.Empty;
            _administration = null;
            _clientconnection = null;
            _cube = null;
            _connectiontimeout = 0;
        }

        /// <summary>
        /// Creates a new command which belongs to this connection
        /// </summary>
        /// <returns>A CubedCommand ready for use</returns>
        public IDbCommand CreateCommand()
        {
            return new CubedCommand(String.Empty, this);
        }

        /// <summary>
        /// This attempts to open the connection to the server with the connection details given
        /// </summary>
        public void Open()
        {
            try
            {
                // Now try and authenticate to Cubed
                _connectiondetails = CheckAuthenticatedUser(_connectionstring);

                // Check whether we had a valid connection details returned
                if (_connectiondetails != null &&
                    (_connectiondetails.Admin == true || _connectiondetails.Cube != String.Empty) &&
                    _connectiondetails.UserName != String.Empty)
                {
                    // Our connection details were returned well so update our internal properties
                    if (_connectiondetails.Admin)
                    {
                        // This is an administration connection
                        _cube = null;
                        _database = String.Empty;
                        _administration = new CubedAdministration(this);
                        _transaction = null;
                        _state = ConnectionState.Open;
                        _connectiontimeout = _connectiondetails.Timeout;
                        _connectionlastactivity = DateTime.Now;
                    }
                    else
                    {
                        // This is not an administration connection, so find our cube
                        _administration = null;
                        _transaction = null;
                        List<Cube> cubes = _clientconnection.ReadCubes(_connectionstring);
                        foreach (Cube thiscube in cubes)
                            if (thiscube.Name.ToUpper().Equals(_connectiondetails.Cube.ToUpper()))
                            {
                                // We have found our cube
                                _database = thiscube.Name;
                                _cube = thiscube;
                            }
                        _state = ConnectionState.Open;
                        _connectiontimeout = _connectiondetails.Timeout;
                        _connectionlastactivity = DateTime.Now;

                        // The cube wasn't found, so raise an exception
                        if (_cube == null)
                            throw new CubeConnectionException("The specified Cube could not be found on the server.",
                                null, String.Empty);
                    }
                }
                else
                {
                    // The connection details were not received correctly, so throw an exception
                    throw new CubeConnectionException("The connection could not be established", null, String.Empty);
                }
            }
            catch (Exception e)
            {
                throw new CubeConnectionException("The connection could not be established due to an underlying protocol problem. Error: " + e.Message, e, e.ToString());
            }
        }

        /// <summary>
        /// Disposes the connection
        /// </summary>
        public void Dispose()
        {
            Close();
        }


        #endregion   

        #region WCF Client Management

        /// <summary>
        /// This opens the WCF client connection
        /// </summary>
        private void OpenClient()
        {
            // First try and create our connection details from the connection string
            _connectiondetails = new ConnectionParameters(_connectionstring);
            _connectiontimeout = _connectiondetails.Timeout;

            // Using the connection details, determine the methods used to connect to the server
            string address = String.Empty;
            switch (_connectiondetails.EndPoint.EndpointType)
            {
                case ServiceEndPointType.NetTcp:
                    // Establish the binding details
                    address = "net.tcp://" + _connectiondetails.EndPoint.Server + ":" + _connectiondetails.EndPoint.Port.ToString()
                        + "/CubedEngineService";
                    NetTcpBinding tcpbinding = new NetTcpBinding();
                    tcpbinding.MaxReceivedMessageSize = Int32.MaxValue;
                    tcpbinding.ReaderQuotas.MaxStringContentLength = Int32.MaxValue;
                    tcpbinding.ReaderQuotas.MaxArrayLength = Int32.MaxValue;
                    tcpbinding.ReaderQuotas.MaxDepth = Int32.MaxValue;
                    tcpbinding.ReceiveTimeout = new TimeSpan(0, 60, 0);
                    tcpbinding.SendTimeout = new TimeSpan(0, 60, 0);
                    tcpbinding.CloseTimeout = new TimeSpan(0, 60, 0);

                    // Establish our client connection end point
                    _clientconnection = new CubedClient(tcpbinding, new EndpointAddress(address));

                    // If not using integrated authentication, establish the credentials and certificate used for connection
                    if (!_connectiondetails.EndPoint.IntegratedAuthentication)
                    {
                        // If not using integrated authentication, we need to supply the credentials explicitly
                        _clientconnection.ClientCredentials.Windows.ClientCredential.Domain = _connectiondetails.Domain;
                        _clientconnection.ClientCredentials.Windows.ClientCredential.UserName = _connectiondetails.UserName;
                        _clientconnection.ClientCredentials.Windows.ClientCredential.Password = _connectiondetails.Password;
                        _clientconnection.ClientCredentials.Windows.AllowedImpersonationLevel = TokenImpersonationLevel.Impersonation;
                    }
                    break;
                case ServiceEndPointType.Soap:
                    // Establish the binding details
                    address = "http://" + _connectiondetails.EndPoint.Server + ":" + _connectiondetails.EndPoint.Port.ToString()
                        + "/CubedEngineService";
                    BasicHttpBinding httpbinding = new BasicHttpBinding();
                    httpbinding.MaxReceivedMessageSize = Int32.MaxValue;
                    httpbinding.ReaderQuotas.MaxDepth = Int32.MaxValue;
                    httpbinding.ReaderQuotas.MaxStringContentLength = Int32.MaxValue;
                    httpbinding.ReaderQuotas.MaxArrayLength = Int32.MaxValue;
                    httpbinding.ReceiveTimeout = new TimeSpan(0, 60, 0);
                    httpbinding.SendTimeout = new TimeSpan(0, 60, 0);
                    httpbinding.CloseTimeout = new TimeSpan(0, 60, 0);

                    // Establish our client connection end point
                    _clientconnection = new CubedClient(httpbinding, new EndpointAddress(address));

                    // Determine if we are using integrated authentication or whether we need to supply credentials
                    if (!_connectiondetails.EndPoint.IntegratedAuthentication)
                    {
                        // If not using integrated authentication, we need to supply the credentials explicitly
                        _clientconnection.ClientCredentials.Windows.ClientCredential.Domain = _connectiondetails.Domain;
                        _clientconnection.ClientCredentials.Windows.ClientCredential.UserName = _connectiondetails.UserName;
                        _clientconnection.ClientCredentials.Windows.ClientCredential.Password = _connectiondetails.Password;
                        _clientconnection.ClientCredentials.Windows.AllowedImpersonationLevel = TokenImpersonationLevel.Impersonation;
                    }
                    break;
            }

            // Before opening the connection, modify the behaviour of the endpoint to allow enough
            // object items in the serialization graph
            foreach (OperationDescription thisoperation in _clientconnection.Endpoint.Contract.Operations)
                thisoperation.Behaviors.Find<DataContractSerializerOperationBehavior>().MaxItemsInObjectGraph = Int32.MaxValue;

            // Finally open our client connection
            _clientconnection.Open();
        }

        /// <summary>
        /// This closes the WCF client connection
        /// </summary>
        private void CloseClient()
        {
            _clientconnection.Close();
        }

        #endregion           
        
        #region Client Methods

        /// <summary>
        /// This runs the AuthenticatedUser function on the Cubed Web Service to return the details of the 
        /// currently authenticated user
        /// </summary>
        /// <returns>A Cubed User object</returns>
        public User AuthenticatedUser()
        {
            OpenClient();
            User user = _clientconnection.AuthenticatedUser();
            CloseClient();
            return user;
        }

        /// <summary>
        /// This runs the CheckAuthenticatedUser function on the Cubed Web Service, given the connections string,
        /// to establish a connection with the server
        /// </summary>
        /// <param name="connectionstring">The connection string to connect with</param>
        /// <returns>The Connection Parameters</returns>
        public ConnectionParameters CheckAuthenticatedUser(string connectionstring)
        {
            OpenClient();
            ConnectionParameters parameters = _clientconnection.CheckAuthenticatedUser(connectionstring);
            CloseClient();
            return parameters;
        }

        /// <summary>
        /// This returns all current roles within the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed Server connection string</param>
        /// <returns>The current roles in the system</returns>
        public List<Role> ReadRoles(string connectionstring)
        {
            OpenClient();
            List<Role> roles = _clientconnection.ReadRoles(connectionstring);
            CloseClient();
            return roles;
        }

        /// <summary>
        /// This returns all current users within the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed Server connection string</param>
        /// <returns>The current users in the system</returns>
        public List<User> ReadUsers(string connectionstring)
        {
            OpenClient();
            List<User> users = _clientconnection.ReadUsers(connectionstring);
            CloseClient();
            return users;
        }

        /// <summary>
        /// This saves the given roles back into the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed Server connection string</param>
        /// <param name="roles">The roles to save in the system</param>
        /// <returns>True if saved correctly, False if a problem occured</returns>
        public bool WriteRoles(string connectionstring, List<Role> roles)
        {
            OpenClient();
            bool success = _clientconnection.WriteRoles(connectionstring, roles);
            CloseClient();
            return success;
        }

        /// <summary>
        /// This saves the given users back into the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed Server connection string</param>
        /// <param name="users">The collection of users to save into the system</param>
        /// <returns>True if saved successfully, false if a problem occured</returns>
        public bool WriteUsers(string connectionstring, List<User> users)
        {
            OpenClient();
            bool success = _clientconnection.WriteUsers(connectionstring, users);
            CloseClient();
            return success;
        }

        /// <summary>
        /// This runs the XMLA Discover function on the Cubed Web Service, given the properties, restrictions and
        /// request type, and returns a schema dataset 
        /// </summary>
        /// <param name="requesttype">The type of request</param>
        /// <param name="restrictions">The list of restrictions to apply to the request</param>
        /// <param name="properties">The list of properties for the request</param>
        /// <returns>A object representing the schema data as requested</returns>
        public string Discover(Bamboo.Cubed.Structures.XMLA.RequestTypes requesttype, Bamboo.Cubed.Structures.XMLA.RestrictionList restrictions, Bamboo.Cubed.Structures.XMLA.PropertyList properties)
        {
            OpenClient();
            string results = _clientconnection.Discover(requesttype, restrictions, properties);
            CloseClient();
            return results;
        }

        /// <summary>
        /// This runs the XMLA Execute function on the Cubed Web Service, and given the command and the
        /// properties passed through, it returns an object which will either be a System.DataSet or a Bamboo.Cubed.
        /// Structures.XMLA.MDDataSet, depending on the type of request
        /// </summary>
        /// <param name="command">The command to execute</param>
        /// <param name="properties">The list of properties to accompany the command</param>
        /// <returns>An object which contains either a System.DataSet or a Bamboo.Cubed.Structures.XMLA.MDDataSet</returns>
        public object Execute(Bamboo.Cubed.Structures.XMLA.Command command, Bamboo.Cubed.Structures.XMLA.PropertyList properties)
        {
            OpenClient();
            object results = _clientconnection.Execute(command, properties);
            CloseClient();
            return results;
        }

        /// <summary>
        /// This runs the ExecuteRelational function on the Cubed Web Service, and given the command and the
        /// properties passed through, it returns an object which will either be a System.DataSet or a Bamboo.Cubed.
        /// Structures.XMLA.MDDataSet, depending on the type of request
        /// </summary>
        /// <param name="command">The command to execute</param>
        /// <param name="properties">The list of properties to accompany the command</param>
        /// <returns>An object which contains either a System.DataSet or a Bamboo.Cubed.Structures.XMLA.MDDataSet</returns>
        public object ExecuteRelational(Bamboo.Cubed.Structures.XMLA.Command command, Bamboo.Cubed.Structures.XMLA.PropertyList properties)
        {
            OpenClient();
            object results = _clientconnection.ExecuteRelational(command, properties);
            CloseClient();
            return results;
        }

        /// <summary>
        /// This runs the ReadCube function on the Cubed Web Service, and given the connection string and the
        /// unique identifier of the cube passed through, it returns the Cube definition
        /// </summary>
        /// <param name="connectionstring">The connection string for the cube</param>
        /// <param name="cubeid">The unique identifier of the cube</param>
        /// <returns>The cube definition</returns>
        public Cube ReadCube(string connectionstring, Guid cubeid)
        {
            OpenClient();
            Cube cube = _clientconnection.ReadCube(connectionstring, cubeid);
            CloseClient();
            return cube;
        }

        /// <summary>
        /// This runs the ReadCubes function on the Cubed Web Service, and given the connection string
        /// provided it returns all Cube definitions on the server
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <returns>All cube definitions on the server</returns>
        public List<Cube> ReadCubes(string connectionstring)
        {
            OpenClient();
            List<Cube> cubes = _clientconnection.ReadCubes(connectionstring);
            CloseClient();
            return cubes;
        }

        /// <summary>
        /// This runs the ReadCubeHeaders function on the Cubed Web Service, and given the connection string
        /// provided it returns all the Cube Header definitions on the server
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <returns>All cube header definitions on the server</returns>
        public List<Cube> ReadCubeHeaders(string connectionstring)
        {
            OpenClient();
            List<Cube> cubeheaders = _clientconnection.ReadCubeHeaders(connectionstring);
            CloseClient();
            return cubeheaders;
        }

        /// <summary>
        /// This runs the ReadCubeDetails function on the Cubed Web Service, and given the connection string and the
        /// unique identifier of the cube passed through, it returns the Cube definition
        /// </summary>
        /// <param name="connectionstring">The connection string for the cube</param>
        /// <param name="cubeid">The unique identifier of the cube</param>
        /// <returns>The cube definition</returns>
        public Cube ReadCubeDetails(Guid cubeid, string connectionstring)
        {
            OpenClient();
            Cube cubedetails = _clientconnection.ReadCubeDetails(cubeid, connectionstring);
            CloseClient();
            return cubedetails;
        }

        /// <summary>
        /// This runs the ReadDatasource function on the Cubed Web Service, and given the connection string and the
        /// unique identifier of the datasource passed through, it returns the Datasource definition
        /// </summary>
        /// <param name="connectionstring">The connection string for the cube</param>
        /// <param name="datasourceid">The unique identifier of the datasource</param>
        /// <returns>The datasource definition</returns>
        public Datasource ReadDatasource(Guid datasourceid, string connectionstring)
        {
            OpenClient();
            Datasource datasource = _clientconnection.ReadDatasource(datasourceid, connectionstring);
            CloseClient();
            return datasource;
        }

        /// <summary>
        /// This runs the ReadMeasure function on the Cubed Web Service, and given the connection string and the
        /// unique identifier of the measure passed through, it returns the Measure definition
        /// </summary>
        /// <param name="connectionstring">The connection string for the cube</param>
        /// <param name="measureid">The unique identifier of the measure</param>
        /// <returns>The measure definition</returns>
        public Measure ReadMeasure(Guid measureid, string connectionstring)
        {
            OpenClient();
            Measure measure = _clientconnection.ReadMeasure(measureid, connectionstring);
            CloseClient();
            return measure;
        }

        /// <summary>
        /// This runs the ReadDimension function on the Cubed Web Service, and given the connection string and the
        /// unique identifier of the dimension passed through, it returns the Dimension definition
        /// </summary>
        /// <param name="connectionstring">The connection string for the cube</param>
        /// <param name="dimensionid">The unique identifier of the dimension</param>
        /// <returns>The dimension definition</returns>
        public Dimension ReadDimension(Guid dimensionid, string connectionstring)
        {
            OpenClient();
            Dimension dimension = _clientconnection.ReadDimension(dimensionid, connectionstring);
            CloseClient();
            return dimension;
        }

        /// <summary>
        /// This runs the ReadDimensionDetails function on the Cubed Web Service, and given the connection string and the
        /// unique identifier of the dimension passed through, it returns the Dimension's details definition
        /// </summary>
        /// <param name="connectionstring">The connection string for the cube</param>
        /// <param name="dimensionid">The unique identifier of the dimension</param>
        /// <returns>The dimension's details definition</returns>
        public Dimension ReadDimensionDetails(Guid dimensionid, string connectionstring)
        {
            OpenClient();
            Dimension dimension = _clientconnection.ReadDimensionDetails(dimensionid, connectionstring);
            CloseClient();
            return dimension;
        }

        /// <summary>
        /// This runs the PublishCubes function on the Cubed Web Service, and given the connection string and
        /// cubes provided, it saves their definitions to the server
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <param name="cubes">The cubes to publish</param>
        /// <returns>True if all were published, else false</returns>
        public bool PublishCubes(string connectionstring, List<Cube> cubes)
        {
            OpenClient();
            bool result = _clientconnection.PublishCubes(connectionstring, cubes);
            CloseClient();
            return result;
        }

        /// <summary>
        /// This runs the SaveCube function on the Cubed Web Service, and given the connection string and cube
        /// provided, it saves its definition to the server
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <param name="cube">The cube to publish</param>
        /// <returns>The unique identifier of the published cube</returns>
        public Guid SaveCube(string connectionstring, Cube cube)
        {
            OpenClient();
            Guid cubeid = _clientconnection.SaveCube(connectionstring, cube);
            CloseClient();
            return cubeid;
        }

        /// <summary>
        /// This runs the SaveCubeDetails function on the Cubed Web Service, and given the connection string and cube
        /// provided, it saves its definition (without datasources, measures and dimensions) to the server
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <param name="cube">The cube to save</param>
        /// <returns>True if saved successfully, false if not</returns>
        public bool SaveCubeDetails(Cube cube, string connectionstring)
        {
            OpenClient();
            bool result = _clientconnection.SaveCubeDetails(cube, connectionstring);
            CloseClient();
            return result;
        }

        /// <summary>
        /// This runs the SaveDatasource function on the Cubed Web Service, and given the connection string and 
        /// datasource/cube id provided, it saves its definition to the server
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <param name="datasource">The datasource to save</param>
        /// <param name="cubeid">The unique identifier of the cube this datasource belongs to</param>
        /// <returns>True if saved successfully, false if not</returns>
        public bool SaveDatasource(Datasource datasource, Guid cubeid, string connectionstring)
        {
            OpenClient();
            bool result = _clientconnection.SaveDatasource(datasource, cubeid, connectionstring);
            CloseClient();
            return result;
        }

        /// <summary>
        /// This runs the SaveMeasure function on the Cubed Web Service, and given the connection string and 
        /// measure/cube id provided, it saves its definition to the server
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <param name="measure">The measure to save</param>
        /// <param name="cubeid">The unique identifier of the cube this measure belongs to</param>
        /// <returns>True if saved successfully, false if not</returns>
        public bool SaveMeasure(Measure measure, Guid cubeid, string connectionstring)
        {
            OpenClient();
            bool result = _clientconnection.SaveMeasure(measure, cubeid, connectionstring);
            CloseClient();
            return result;
        }

        /// <summary>
        /// This runs the SaveDimension function on the Cubed Web Service, and given the connection string and 
        /// dimension/cube id provided, it saves its definition to the server with all members
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <param name="dimension">The dimension to save</param>
        /// <param name="cubeid">The unique identifier of the cube this dimension belongsd to</param>
        /// <returns>True if saved successfully, false if not</returns>
        public bool SaveDimension(Dimension dimension, Guid cubeid, string connectionstring)
        {
            OpenClient();
            bool result = _clientconnection.SaveDimension(dimension, cubeid, connectionstring);
            CloseClient();
            return result;
        }

        /// <summary>
        /// This runs the SaveDimensionDetails function on the Cubed Web Service, and given the connection string and 
        /// dimension/cube id provided, it saves its definition details to the server
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <param name="dimension">The dimension to save</param>
        /// <param name="cubeid">The unique identifier of the cube this dimension belongs to</param>
        /// <returns>True if saved successfully, false if not</returns>
        public bool SaveDimensionDetails(Dimension dimension, Guid cubeid, string connectionstring)
        {
            OpenClient();
            bool result = _clientconnection.SaveDimensionDetails(dimension, cubeid, connectionstring);
            CloseClient();
            return result;
        }

        /// <summary>
        /// This runs the SaveMember function on the Cubed Web Service, and given the connection string and 
        /// member provided, it saves its definition to the server
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <param name="member">The member to save</param>
        /// <returns>True if saved successfully, false if not</returns>
        public bool SaveMember(Member member, string connectionstring)
        {
            OpenClient();
            bool result = _clientconnection.SaveMember(member, connectionstring);
            CloseClient();
            return result;
        }

        /// <summary>
        /// This runs the RemoveCube function on the Cubed Web Service, and given the connection string and
        /// cube identifier provided, it removes the cube from the server
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <param name="cubeid">The unique identifier of the cube to remove</param>
        /// <returns>True if the cube was removed successfully, else false</returns>
        public bool RemoveCube(string connectionstring, Guid cubeid)
        {
            OpenClient();
            bool result = _clientconnection.RemoveCube(connectionstring, cubeid);
            CloseClient();
            return result;
        }

        /// <summary>
        /// This runs the AddRootUser function on the Cubed Web Service, and given the connection string and
        /// name of the new user, it adds them to the root users collection
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <param name="newusername">The name of the new user to add (including domain)</param>
        /// <returns>True if the user was added, false if not</returns>
        public bool AddRootUser(string connectionstring, string newusername)
        {
            OpenClient();
            bool result = _clientconnection.AddRootUser(connectionstring, newusername);
            CloseClient();
            return result;
        }

        /// <summary>
        /// This runs the RemoveRootUser function on the Cubed Web Service, and given the connection string and
        /// username, it removes them from the root users collection
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <param name="username">The name of the root user to remove</param>
        /// <returns>True if the user was removed, false if not</returns>
        public bool RemoveRootUser(string connectionstring, string username)
        {
            OpenClient();
            bool result = _clientconnection.RemoveRootUser(connectionstring, username);
            CloseClient();
            return result;
        }

        /// <summary>
        /// This runs the ReadSettings function on the Cubed Web Service, and given the connection string
        /// provided, it returns the complete settings collection for the server
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <returns>The complete server settings collection</returns>
        public ServerSettings ReadSettings(string connectionstring)
        {
            OpenClient();
            ServerSettings settings = _clientconnection.ReadSettings(connectionstring);
            CloseClient();
            return settings;
        }

        /// <summary>
        /// This runs the WriteSettings function on the Cubed Web Service, and given the connection string and 
        /// settings provided, it writes the new settings to the server
        /// </summary>
        /// <param name="newsettings">The settings to write</param>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <returns>True if the settings were written, false if not</returns>
        public bool WriteSettings(ServerSettings newsettings, string connectionstring)
        {
            OpenClient();
            bool result = _clientconnection.WriteSettings(newsettings, connectionstring);
            CloseClient();
            return result;
        }

        /// <summary>
        /// This cancels the currently active consolidation, if any are active
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        public void CancelConsolidation(string connectionstring)
        {
            OpenClient();
            _clientconnection.CancelConsolidation(connectionstring);
            CloseClient();
        }

        /// <summary>
        /// This runs the StartSingleConsolidation function on the Cubed Web Service, and given the connection
        /// string, cube identifier and parent consolidation value provided, it starts a single cube consolidation
        /// </summary>
        /// <param name="connectionstring">The connection string for the server</param>
        /// <param name="cubeid">The unique identifier of the cube</param>
        /// <param name="pointconsolidation">Whether to start a full or partial (point) consolidation</param>
        public void StartSingleConsolidation(string connectionstring, Guid cubeid, bool pointconsolidation)
        {
            OpenClient();
            _clientconnection.StartSingleConsolidation(connectionstring, cubeid, pointconsolidation);
            CloseClient();
        }

        /// <summary>
        /// This runs the StartFullConsolidation function on the Cubed Web Service, and given the connection
        /// string and parent consolidation value provided, it starts a consolidation of all cubes on the server
        /// </summary>
        /// <param name="pointconsolidation">Whether to start a full or partial (point) consolidation</param>
        /// <param name="connectionstring">The connection string for the server</param>
        public void StartFullConsolidation(string connectionstring, bool pointconsolidation)
        {
            OpenClient();
            _clientconnection.StartFullConsolidation(connectionstring, pointconsolidation);
            CloseClient();
        }

        /// <summary>
        /// This runs the Version function on the Cubed Web Service to return the current version details
        /// </summary>
        /// <returns>The current version of the Cubed Web Server</returns>
        public string Version()
        {
            OpenClient();
            string version = _clientconnection.Version();
            CloseClient();
            return version;
        }

        /// <summary>
        /// This runs the TrueVersion function on the Cubed Web Service to return the current true version details
        /// </summary>
        /// <returns>The current true version of the Cubed Web Server</returns>
        public string TrueVersion()
        {
            OpenClient();
            string version = _clientconnection.TrueVersion();
            CloseClient();
            return version;
        }

        /// <summary>
        /// This runs the Version function on the Cubed Web Service to return the current release name
        /// </summary>
        /// <returns>The current release of the Cubed Web Server</returns>
        public string Release()
        {
            OpenClient();
            string release = _clientconnection.Release();
            CloseClient();
            return release;
        }

        /// <summary>
        /// This runs the Version function on the Cubed Web Service to return the current release header
        /// </summary>
        /// <returns>The current release header of the Cubed Web Server</returns>
        public string ReleaseHeader()
        {
            OpenClient();
            string release = _clientconnection.ReleaseHeader();
            CloseClient();
            return release;
        }

        /// <summary>
        /// This runs the Version function on the Cubed Web Service to return the current release full details
        /// </summary>
        /// <returns>The current full release details of the Cubed Web Server</returns>
        public string ReleaseDetails()
        {
            OpenClient();
            string releasedetails = _clientconnection.ReleaseDetails();
            CloseClient();
            return releasedetails;
        }

        /// <summary>
        /// This returns the node which represents the collection of members and measure given
        /// </summary>
        /// <param name="connectionstring">A valid connection string to connect to the Cubed Server</param>
        /// <param name="cubeid">The unique identifier of the cube to return data for</param>
        /// <param name="measure">The unique identifier of the measure to return data for</param>
        /// <param name="members">The list of members to return data for</param>
        /// <returns>A node with data</returns>
        public Node GetNode(string connectionstring, Guid cubeid, Guid measure, List<Guid> members)
        {
            OpenClient();
            Node node = _clientconnection.GetNode(connectionstring, cubeid, measure, members);
            CloseClient();
            return node;
        }

        /// <summary>
        /// This gets a data matrix containing values for the selected members for each measure in the cube
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="cubeid">The unique identifier of the cube</param>
        /// <param name="members">The selected members</param>
        /// <returns>A data matrix containing the values for each measure</returns>
        public DataMatrix GetMeasureMatrix(string connectionstring, Guid cubeid, List<Guid> members)
        {
            OpenClient();
            DataMatrix measurematrix = _clientconnection.GetMeasureMatrix(connectionstring, cubeid, members);
            CloseClient();
            return measurematrix;
        }

        /// <summary>
        /// This gets a data matrix containing values for the selected measure and members pivoting on the
        /// remaining dimension
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="cubeid">The unique identifier of the cube</param>
        /// <param name="measure">The measure to read</param>
        /// <param name="otherselectedmembers">The selected members on all dimensions except the pivot dimension</param>
        /// <returns>The given data matrix</returns>
        public DataMatrix GetMemberMatrix(string connectionstring, Guid cubeid, Guid measure, List<Guid> otherselectedmembers)
        {
            OpenClient();
            DataMatrix membermatrix = _clientconnection.GetMemberMatrix(connectionstring, cubeid, measure, otherselectedmembers);
            CloseClient();
            return membermatrix;
        }

        /// <summary>
        /// This saves the given node back to the data matrix
        /// </summary>
        /// <param name="connectionstring">A valid connection string to connect to the Cubed Server</param>
        /// <param name="cube">The cube for which data is being written back to</param>
        /// <param name="node">The node to save</param>
        /// <returns>True if the data was successfully saved, False if not</returns>
        public bool SetNode(string connectionstring, Cube cube, Node node)
        {
            OpenClient();
            bool result = _clientconnection.SetNode(connectionstring, cube, node);
            CloseClient();
            return result;
        }

        /// <summary>
        /// Saves the new search with the given details to the database
        /// </summary>
        /// <param name="search">The search to save</param>
        /// <returns>True if saved, false if not</returns>
        public bool SaveSearch(DesktopSearch search)
        {
            OpenClient();
            bool result = _clientconnection.SaveSearch(_connectionstring, search);
            CloseClient();
            return result;
        }

        /// <summary>
        /// Saves the new dashboard with the given details to the database
        /// </summary>
        /// <param name="dashboard">The dashboard to save</param>
        /// <returns>True if saved, false if not</returns>
        public bool SaveDashboard(DesktopDashboard dashboard)
        {
            OpenClient();
            bool result = _clientconnection.SaveDashboard(_connectionstring, dashboard);
            CloseClient();
            return result;
        }

        /// <summary>
        /// Saves the new search category with the given details to the database
        /// </summary>
        /// <param name="category">The search category to save</param>
        /// <returns>True if saved, false if not</returns>
        public bool SaveSearchCategory(SearchCategory category)
        {
            OpenClient();
            bool result = _clientconnection.SaveSearchCategory(_connectionstring, category);
            CloseClient();
            return result;
        }

        /// <summary>
        /// This searches the desktop search collection for matches and returns them
        /// </summary>
        /// <param name="criterion">The criterion to search by</param>
        /// <returns>A collection of desktop searches matching the criterion</returns>
        public GenericKeyCollection<Guid, DesktopSearch> MatchSearches(List<string> criterion)
        {
            OpenClient();
            GenericKeyCollection<Guid, DesktopSearch> searches = _clientconnection.MatchSearches(_connectionstring, criterion);
            CloseClient();
            return searches;
        }

        /// <summary>
        /// This searches the desktop search collection for matches and returns them
        /// </summary>
        /// <param name="criterion">The criterion to search by</param>
        /// <param name="category">The Search Category to filter the results</param>
        /// <returns>A collection of desktop searches matching the criterion</returns>
        public GenericKeyCollection<Guid, DesktopSearch> MatchSearches(List<string> criterion, SearchCategory category)
        {
            OpenClient();
            GenericKeyCollection<Guid, DesktopSearch> searches = _clientconnection.MatchSearchesFilterCategory(_connectionstring, criterion, category);
            CloseClient();
            return searches;
        }

        /// <summary>
        /// This reads the given search and returns it
        /// </summary>
        /// <param name="searchid">The unique identifier of the search to look for</param>
        /// <returns>The search if it exists, else null</returns>
        public DesktopSearch ReadSearch(Guid searchid)
        {
            OpenClient();
            DesktopSearch search = _clientconnection.ReadSearch(_connectionstring, searchid);
            CloseClient();
            return search;
        }

        /// <summary>
        /// Reads the current users dashboard
        /// </summary>
        /// <returns>The dashboard for the current user</returns>
        public DesktopDashboard ReadDashboard()
        {
           OpenClient();
           DesktopDashboard dashboard = _clientconnection.ReadUserDashboard(_connectionstring, _connectiondetails.UserName);
           CloseClient();
           return dashboard;
        }

        /// <summary>
        /// This reads teh dashboard with the unique identifier given
        /// </summary>
        /// <param name="dashboardid">The unique identifier of the dashboard to return</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>The dashboard if it exists, else an exception will be thrown</returns>
        public DesktopDashboard ReadDashboard(string connectionstring, Guid dashboardid)
        {
            OpenClient();
            DesktopDashboard dashboard = _clientconnection.ReadDashboard(connectionstring, dashboardid);
            CloseClient();
            return dashboard;
        }

        /// <summary>
        /// This reads the dashboard for the given user and returns it
        /// </summary>
        /// <param name="username">The user to read the dashboard for</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>The dashboard for the given user</returns>
        public DesktopDashboard ReadUserDashboard(string connectionstring, string username)
        {
            OpenClient();
            DesktopDashboard dashboard = _clientconnection.ReadUserDashboard(connectionstring, username);
            CloseClient();
            return dashboard;
        }

        /// <summary>
        /// This reads the dashboards for the given user and returns them
        /// </summary>
        /// <param name="username">The user to read the dashboard for</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>The dashboards for the given user</returns>
        public DesktopDashboard[] ReadUserDashboards(string connectionstring, string username)
        {
            OpenClient();
            DesktopDashboard[] dashboards = _clientconnection.ReadUserDashboards(connectionstring, username);
            CloseClient();
            return dashboards;
        }

        /// <summary>
        /// This reads the given search category and returns it
        /// </summary>
        /// <param name="searchid">The unique identifier of the search category to look for</param>
        /// <returns>The search category if it exists, else null</returns>
        public SearchCategory ReadSearchCategory(Guid categoryid)
        {
            OpenClient();
            SearchCategory category = _clientconnection.ReadSearchCategory(_connectionstring, categoryid);
            CloseClient();
            return category;
        }

        /// <summary>
        /// This reads the search categories and returns them
        /// </summary>
        /// <returns>The search categories</returns>
        public SearchCategory[] ReadSearchCategories(string connectionstring)
        {
            OpenClient();
            SearchCategory[] category = _clientconnection.ReadSearchCategories(connectionstring);
            CloseClient();
            return category;
        }

        /// <summary>
        /// This creates a new dashboard for the given user 
        /// </summary>
        /// <param name="username">The user to read the dashboard for</param>
        /// <param name="dashboardDescription">The description for the new dashboard</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns></returns>
        public bool CreateDashboard(string connectionstring, string username, string dashboardDescription)
        {
            OpenClient();
            bool dashboard = _clientconnection.CreateDashboard(connectionstring, username, dashboardDescription);
            CloseClient();
            return dashboard;
        }
        /// <summary>
        /// This deletes the given search from the database
        /// </summary>
        /// <param name="searchid">The unique identifier of the search to delete</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>True if deleted, else an exception will be thrown</returns>
        public bool DeleteSearch(string connectionstring, Guid searchid)
        {
            OpenClient();
            bool result = _clientconnection.DeleteSearch(connectionstring, searchid);
            CloseClient();
            return result;
        }

        /// <summary>
        /// This deletes the given dashboard from the database
        /// </summary>
        /// <param name="dashboardid">The unique identifier of the dashboard to delete</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>True if deleted, else an exception will be thrown</returns>
        public bool DeleteDashboard(string connectionstring, Guid dashboardid)
        {
            OpenClient();
            bool result = _clientconnection.DeleteDashboard(connectionstring, dashboardid);
            CloseClient();
            return result;
        }

        /// <summary>
        /// This renames the given dashboard from the database
        /// </summary>
        /// <param name="dashboardid">The unique identifier of the dashboard to delete</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="newdashbaordname">The new name of the dashboard.</param>
        /// <returns>True if renamed, else an exception will be thrown</returns>
        public bool RenameDashboard(string connectionstring, Guid dashboardid, string newdashbaordname)
        {
            OpenClient();
            bool result = _clientconnection.RenameDashboard(connectionstring, dashboardid, newdashbaordname);
            CloseClient();
            return result;
        }

        /// <summary>
        /// Exports all current users of the system to a byte array ready for saving
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>A byte array ready for saving to disk</returns>
        public byte[] ExportUsers(string connectionstring)
        {
            OpenClient();
            byte[] result = _clientconnection.ExportUsers(connectionstring);
            CloseClient();
            return result;
        }

        /// <summary>
        /// Imports all users contained in the given export into the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>The number of records imported</returns>
        public int ImportUsers(string connectionstring, byte[] export)
        {
            OpenClient();
            int result = _clientconnection.ImportUsers(connectionstring, export);
            CloseClient();
            return result;
        }

        /// <summary>
        /// Exports all current roles of the system to a byte array ready for saving
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>A byte array ready for saving to disk</returns>
        public byte[] ExportRoles(string connectionstring)
        {
            OpenClient();
            byte[] result = _clientconnection.ExportRoles(connectionstring);
            CloseClient();
            return result;
        }

        /// <summary>
        /// Imports all roles contained in the given export into the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>The number of records imported</returns>
        public int ImportRoles(string connectionstring, byte[] export)
        {
            OpenClient();
            int result = _clientconnection.ImportRoles(connectionstring, export);
            CloseClient();
            return result;
        }

        /// <summary>
        /// Exports all current searches of the system to a byte array ready for saving
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>A byte array ready for saving to disk</returns>
        public byte[] ExportSearches(string connectionstring)
        {
            OpenClient();
            byte[] result = _clientconnection.ExportSearches(connectionstring);
            CloseClient();
            return result;
        }

        /// <summary>
        /// Imports all searches contained in the given export into the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>The number of records imported</returns>
        public int ImportSearches(string connectionstring, byte[] export)
        {
            OpenClient();
            int result = _clientconnection.ImportSearches(connectionstring, export);
            CloseClient();
            return result;
        }

        /// <summary>
        /// Exports all current dashboards of the system to a byte array ready for saving
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>A byte array ready for saving to disk</returns>
        public byte[] ExportDashboards(string connectionstring)
        {
            OpenClient();
            byte[] result = _clientconnection.ExportDashboards(connectionstring);
            CloseClient();
            return result;
        }

        /// <summary>
        /// Imports all dashboards contained in the given export into the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>The number of records imported</returns>
        public int ImportDashboards(string connectionstring, byte[] export)
        {
            OpenClient();
            int result = _clientconnection.ImportDashboards(connectionstring, export);
            CloseClient();
            return result;
        }

        /// <summary>
        /// Exports the specified cube to a byte array ready for saving
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="cubeid">The unique identifier of the cube to export</param>
        /// <returns>A bute array ready for saving to disk</returns>
        public byte[] ExportCube(string connectionstring, Guid cubeid)
        {
            OpenClient();
            byte[] result = _clientconnection.ExportCube(connectionstring, cubeid);
            CloseClient();
            return result;
        }

        /// <summary>
        /// Imports the cube in the given export into the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>True if imported successfully, false if not</returns>
        public bool ImportCube(string connectionstring, byte[] export)
        {
            OpenClient();
            bool result = _clientconnection.ImportCube(connectionstring, export);
            CloseClient();
            return result;
        }

        #endregion
    }
}
