/*
 * This source file (CubedDataReader.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;

namespace Bamboo.Cubed.Connect
{
    /// <summary>
    /// This provides a data reader for fast-forward only reading through a collection
    /// </summary>
    public class CubedDataReader : IDataReader
    {
        /// <summary>
        /// This is the internal data set used by the reader
        /// </summary>
        private DataSet _records;

        /// <summary>
        /// The current row in the collection
        /// </summary>
        private DataRow _currentrecord;

        /// <summary>
        /// The index of the current row in the entire collection
        /// </summary>
        private int _currentindex;

        /// <summary>
        /// Initialises a new instance of the Cubed Data reader with default properties
        /// </summary>
        public CubedDataReader()
        {
            _records = new DataSet();
            _currentrecord = null;
            _currentindex = -1;
        }

        /// <summary>
        /// /This initialises a new instance of the Cubed Data reader with the given data set to begin with
        /// </summary>
        /// <param name="records">The data set containing the records to base this data reader on</param>
        public CubedDataReader(DataSet records)
        {
            _records = records;
            _currentrecord = null;
            _currentindex = -1;
        }

        /// <summary>
        /// This closes the data reader and resets its' records
        /// </summary>
        public void Close()
        {
            _records = null;
            _currentrecord = null;
            _currentindex = -1;
        }

        /// <summary>
        /// Returns the total number of rows in the entire collection
        /// </summary>
        public int Depth
        {
            get 
            {
                if (_records != null && _records.Tables.Count > 0)
                {
                    return _records.Tables[0].Rows.Count;
                }
                else
                    throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            }
        }

        /// <summary>
        /// This returns the data table supporting the data reader without any data in it
        /// </summary>
        /// <returns>An empty schema-only version of the data table supporting this data reader</returns>
        public DataTable GetSchemaTable()
        {
            if (_records != null && _records.Tables.Count > 0)
            {
                DataTable schematable = _records.Tables[0].Copy();
                schematable.Rows.Clear();
                return schematable;
            }
            else
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
        }

        /// <summary>
        /// Returns a value indicating whether this Data Reader has been closed (either manually or automatically
        /// by reaching the end of the collection)
        /// </summary>
        public bool IsClosed
        {
            get 
            {
                if (_records == null || _records.Tables.Count < 1 || _currentrecord == null)
                    return true;
                else
                    return false;
            }
        }

        /// <summary>
        /// This moves the reader to the next result in the collection
        /// </summary>
        /// <returns>True if the next result is available, false if we have reached the end of the collection</returns>
        public bool NextResult()
        {
            if (_records != null && _records.Tables.Count > 0)
            {
                // Check whether we have already reached the end of our collection
                if (_records.Tables[0].Rows.Count > (_currentindex + 1))
                {
                    // We haven't yet reached the end of our collection, so move to the next record and return true
                    _currentindex++;
                    _currentrecord = _records.Tables[0].Rows[_currentindex];
                    return true;
                }
                else
                {
                    // We have reached the end of our collection, so set our current record to null
                    // and return false
                    _currentrecord = null;
                    return false;
                }
            }
            else
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
        }

        /// <summary>
        /// This moves the reader to the next result in the collection
        /// </summary>
        /// <returns>True if the next result is available, false if we have reached the end of the collection</returns>
        public bool Read()
        {
            return NextResult();
        }

        /// <summary>
        /// This is the number of records which have already been read through the collection
        /// </summary>
        public int RecordsAffected
        {
            get
            {
                if (_records != null && _records.Tables.Count > 0)
                {
                    // Check how many records we have already read through
                    return _currentindex;
                    
                }
                else
                    throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            }
        }

        /// <summary>
        /// This disposes the reader and releases any information it carries
        /// </summary>
        public void Dispose()
        {
            Close();
        }

        /// <summary>
        /// This returns the number of fields in the collection
        /// </summary>
        public int FieldCount
        {
            get 
            {
                if (_records != null && _records.Tables.Count > 0)
                {
                    return _records.Tables[0].Columns.Count;
                }
                else
                    throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            }
        }

        /// <summary>
        /// This returns the value of the column specified as a boolean
        /// </summary>
        /// <param name="index">The index of the column whose value should be retrieved</param>
        /// <returns>The value of the column as a boolean, or else an exception will be thrown</returns>
        public bool GetBoolean(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                return Convert.ToBoolean(_currentrecord[index]);
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// This returns the value of the column specified as a byte
        /// </summary>
        /// <param name="index">The index of the column whose value should be retrieved</param>
        /// <returns>The value of the column as a byte, or else an exception will be thrown</returns>
        public byte GetByte(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                return Convert.ToByte(_currentrecord[index]);
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// This reads the specified number of bytes from the field value into the byte array given
        /// </summary>
        /// <param name="index">The index of the field to read the value of</param>
        /// <param name="fieldoffset">The position to start reading from the field</param>
        /// <param name="buffer">A character array to fill with the results of the read</param>
        /// <param name="bufferoffset">The position to start writing the characters into the given array</param>
        /// <param name="length">The amount of characters to read</param>
        /// <returns>The number of characters returned</returns>
        public long GetBytes(int index, long fieldoffset, byte[] buffer, int bufferoffset, int length)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                // The given field exists, so create a new character list based on the given buffer and arrange it
                // to coincide with the given buffer offset
                List<byte> outputbuffer = new List<byte>(buffer);
                if (outputbuffer.Count > bufferoffset)
                    outputbuffer.RemoveRange(bufferoffset - 1, outputbuffer.Count - bufferoffset);
                else if (outputbuffer.Count < bufferoffset)
                    while (outputbuffer.Count < bufferoffset)
                        outputbuffer.Add(0);

                // Check that the field can be read as a string, and that it has enough characters to fulfill the
                // given requirements
                byte[] fieldvalue = (byte[])_currentrecord[index];
                if (fieldvalue.Length < ((fieldoffset - 1) + length))
                    throw new CubeConnectionException("The field does not contain enough characters to fulfill the "
                        + "field offset and length requirements given.", null, String.Empty);
                else
                {
                    // The field contains enough bytes, so read from it and return a value
                    long bytesread = length;
                    long i = fieldoffset;
                    while (i < fieldoffset + length)
                    {
                        outputbuffer.Add(fieldvalue[Convert.ToInt32(i)]);
                    }
                    buffer = outputbuffer.ToArray();
                    return bytesread;
                }
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// This returns the value of the column specified as a char
        /// </summary>
        /// <param name="index">The index of the column whose value should be retrieved</param>
        /// <returns>The value of the column as a char, or else an exception will be thrown</returns>
        public char GetChar(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                return Convert.ToChar(_currentrecord[index]);
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// This reads the specified number of characters from the field value into the character array given
        /// </summary>
        /// <param name="index">The index of the field to read the value of</param>
        /// <param name="fieldoffset">The position to start reading from the field</param>
        /// <param name="buffer">A character array to fill with the results of the read</param>
        /// <param name="bufferoffset">The position to start writing the characters into the given array</param>
        /// <param name="length">The amount of characters to read</param>
        /// <returns>The number of characters returned</returns>
        public long GetChars(int index, long fieldoffset, char[] buffer, int bufferoffset, int length)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                // The given field exists, so create a new character list based on the given buffer and arrange it
                // to coincide with the given buffer offset
                List<char> outputbuffer = new List<char>(buffer);
                if (outputbuffer.Count > bufferoffset)
                    outputbuffer.RemoveRange(bufferoffset - 1, outputbuffer.Count - bufferoffset);
                else if (outputbuffer.Count < bufferoffset)
                    while (outputbuffer.Count < bufferoffset)
                        outputbuffer.Add(' ');

                // Check that the field can be read as a string, and that it has enough characters to fulfill the
                // given requirements
                string fieldvalue = GetString(index);
                if (fieldvalue.Length < ((fieldoffset - 1) + length))
                    throw new CubeConnectionException("The field does not contain enough characters to fulfill the "
                        + "field offset and length requirements given.", null, String.Empty);
                else
                {
                    // The field contains enough characters, so read from it and return a value
                    long charactersread = length;
                    long i = fieldoffset;
                    while (i < fieldoffset + length)
                    {
                        outputbuffer.Add(fieldvalue[Convert.ToInt32(i)]);
                    }
                    buffer = outputbuffer.ToArray();
                    return charactersread;
                }
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// Returns a new data reader with only this columns values
        /// </summary>
        /// <param name="index">The index of the values to return</param>
        /// <returns>A new Cubed Data Reader with the current record set, with only the selected column</returns>
        public IDataReader GetData(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                // The index exists, so return a new data reader with just those values
                DataSet returnedresults = new DataSet();
                returnedresults.Tables.Add(_records.Tables[0].Copy());
                for (int i = returnedresults.Tables[0].Columns.Count - 1; i > -1; i--)
                    if (i != index)
                        returnedresults.Tables[0].Columns.RemoveAt(i);
                return new CubedDataReader(returnedresults);
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// Returns the data type of the field at the given index
        /// </summary>
        /// <param name="index">The index of the field to retrieve</param>
        /// <returns>The data type of the field at the given index</returns>
        public string GetDataTypeName(int index)
        {
            if (_records != null && _records.Tables.Count > 0 && _records.Tables[0].Columns.Count > index)
            {
                return _records.Tables[0].Columns[index].DataType.ToString();
            }
            else if (_records != null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no field at the given index.", null, String.Empty);            
        }

        /// <summary>
        /// This returns the value of the column specified as a datetime
        /// </summary>
        /// <param name="index">The index of the column whose value should be retrieved</param>
        /// <returns>The value of the column as a datetime, or else an exception will be thrown</returns>
        public DateTime GetDateTime(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                return Convert.ToDateTime(_currentrecord[index]);
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// This returns the value of the column specified as a decimal
        /// </summary>
        /// <param name="index">The index of the column whose value should be retrieved</param>
        /// <returns>The value of the column as a decimal, or else an exception will be thrown</returns>
        public decimal GetDecimal(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                return Convert.ToDecimal(_currentrecord[index]);
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// This returns the value of the column specified as a double
        /// </summary>
        /// <param name="index">The index of the column whose value should be retrieved</param>
        /// <returns>The value of the column as a double, or else an exception will be thrown</returns>
        public double GetDouble(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                return Convert.ToDouble(_currentrecord[index]);
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// Returns the data type of the field at the given index
        /// </summary>
        /// <param name="index">The index of the field to retrieve</param>
        /// <returns>The data type of the field at the given index</returns>
        public Type GetFieldType(int index)
        {
            if (_records != null && _records.Tables.Count > 0 && _records.Tables[0].Columns.Count > index)
            {
                return _records.Tables[0].Columns[index].DataType;
            }
            else if (_records != null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no field at the given index.", null, String.Empty);
        }

        /// <summary>
        /// This returns the value of the column specified as a float
        /// </summary>
        /// <param name="index">The index of the column whose value should be retrieved</param>
        /// <returns>The value of the column as a float, or else an exception will be thrown</returns>
        public float GetFloat(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                return Convert.ToSingle(_currentrecord[index]);
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// This returns the value of the column specified as aguid
        /// </summary>
        /// <param name="index">The index of the column whose value should be retrieved</param>
        /// <returns>The value of the column as a float, or else an exception will be thrown</returns>
        public Guid GetGuid(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                return new Guid(_currentrecord[index].ToString());
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// This returns the value of the column specified as a 16 bit integer
        /// </summary>
        /// <param name="index">The index of the column whose value should be retrieved</param>
        /// <returns>The value of the column as a 16 bit integer, or else an exception will be thrown</returns>
        public short GetInt16(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                return Convert.ToInt16(_currentrecord[index]);
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// This returns the value of the column specified as a 32 bit integer
        /// </summary>
        /// <param name="index">The index of the column whose value should be retrieved</param>
        /// <returns>The value of the column as a 32 bit integer, or else an exception will be thrown</returns>
        public int GetInt32(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                return Convert.ToInt32(_currentrecord[index]);
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// This returns the value of the column specified as a 64 bit integer
        /// </summary>
        /// <param name="index">The index of the column whose value should be retrieved</param>
        /// <returns>The value of the column as a 64 bit integer, or else an exception will be thrown</returns>
        public long GetInt64(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                return Convert.ToInt64(_currentrecord[index]);
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// This returns the name of the field at the specified index
        /// </summary>
        /// <param name="index">The index of the field to return</param>
        /// <returns>The name of the field</returns>
        public string GetName(int index)
        {
            if (_records != null && _records.Tables.Count > 0 && _records.Tables[0].Columns.Count > index)
            {
                // Return the name of the field
                return _records.Tables[0].Columns[index].ColumnName;
            }
            else
                throw new CubeConnectionException("There is no field with the index given.", null, String.Empty);
        }

        /// <summary>
        /// Returns the ordinal index of the field with the given name, if it exists
        /// </summary>
        /// <param name="name">The name of the field to find</param>
        /// <returns>The ordinal index of the field, or an exception will be thrown if the field doesn't exist</returns>
        public int GetOrdinal(string name)
        {
            if (_records != null && _records.Tables.Count > 0 && _records.Tables[0].Columns.Contains(name))
            {
                // We know the field exists, so return its index
                return _records.Tables[0].Columns.IndexOf(name);
            }
            else
                throw new CubeConnectionException("There is no field with the name given.", null, String.Empty);
        }

        /// <summary>
        /// This returns the value of the column specified as a string
        /// </summary>
        /// <param name="index">The index of the column whose value should be retrieved</param>
        /// <returns>The value of the column as a string, or else an exception will be thrown</returns>
        public string GetString(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                return Convert.ToString(_currentrecord[index]);
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// Returns the value of the column specified in its' original form
        /// </summary>
        /// <param name="index">The index of the column whose value should be retrieved</param>
        /// <returns>The value of the column in whatever format it exists</returns>
        public object GetValue(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                return _currentrecord[index];
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// This returns an array of values for the current record
        /// </summary>
        /// <param name="values">An array of values to update with the current record in the collection</param>
        /// <returns>The number of values returned</returns>
        public int GetValues(object[] values)
        {
            if (_records != null && _records.Tables.Count > 0 && _currentrecord != null)
            {
                values = _currentrecord.ItemArray;
                return values.Length;
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// Returns true if the value stored at the given index is null, otherwise returns false
        /// </summary>
        /// <param name="index">The index of the column to check the value of</param>
        /// <returns>True if the column value is null, false if not</returns>
        public bool IsDBNull(int index)
        {
            if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
            {
                if (_currentrecord[index] == DBNull.Value)
                    return true;
                else
                    return false;
            }
            else if (_records == null || _records.Tables.Count < 1)
                throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
            else
                throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                    + "to the next row in the collection", null, String.Empty);
        }

        /// <summary>
        /// Returns the value of the column in the active row with the name given
        /// </summary>
        /// <param name="name">The name of the column whose value you would like to retrieve</param>
        /// <returns>The value of the column specified</returns>
        public object this[string name]
        {
            get 
            {
                if (_records != null && _records.Tables.Count > 0 && _currentrecord != null)
                {
                    // We have a records collection and a current record, so check to see that a column with the
                    // specified name exists
                    if (_records.Tables[0].Columns.Contains(name))
                    {
                        return _currentrecord[name];
                    }
                    else
                        throw new CubeConnectionException("No column exists with the name specified.", null,
                            String.Empty);
                }
                else if (_records == null || _records.Tables.Count < 1)
                    throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
                else
                    throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                        + "to the next row in the collection", null, String.Empty);
            }
        }

        /// <summary>
        /// Returns the value of the column in the active row with the index given
        /// </summary>
        /// <param name="index">The index of the column whose value you would like to retrieve</param>
        /// <returns>The value of the column specified</returns>
        public object this[int index]
        {
            get 
            {
                if (_records != null && _records.Tables.Count > 0 &&
                _currentrecord != null && _records.Tables[0].Columns.Count > index)
                {
                    // Return the value of the specified column
                    return _currentrecord[index];
                }
                else if (index >= _records.Tables[0].Columns.Count)
                    throw new CubeConnectionException("There are no columns at the given index.", null,
                        String.Empty);
                else if (_records == null || _records.Tables.Count < 1)
                    throw new CubeConnectionException("There are no records in the collection.", null, String.Empty);
                else
                    throw new CubeConnectionException("There is no current row in the collection. Use Read() to advance "
                        + "to the next row in the collection", null, String.Empty);
            }
        }
    }
}