/*
 * This source file (CubedParameterCollection.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;

namespace Bamboo.Cubed.Connect
{
    /// <summary>
    /// This contains a collection of Cubed Parameters
    /// </summary>
    public class CubedParameterCollection : IDataParameterCollection
    {
        /// <summary>
        /// Initialises a default instance of the Cubed Parameter Collection
        /// </summary>
        public CubedParameterCollection()
        {
            _parameters = new Dictionary<string, CubedParameter>();
            _parameterslist = new List<CubedParameter>();
        }
        
        /// <summary>
        /// The internal list of parameters
        /// </summary>
        private Dictionary<string, CubedParameter> _parameters;

        /// <summary>
        /// An indexed list of the parameters
        /// </summary>
        private List<CubedParameter> _parameterslist;

        /// <summary>
        /// Returns true if the collection contains the parameter specified, false if not
        /// </summary>
        /// <param name="parametername">The name of the parameter to check</param>
        /// <returns>True if the collection contains the parameter, false if not</returns>
        public bool Contains(string parametername)
        {
            return _parameters.ContainsKey(parametername);
        }

        /// <summary>
        /// Returns the index of the parameter in the underlying list collection
        /// </summary>
        /// <param name="parametername">The name of the parameter to search for</param>
        /// <returns>The index of the item, or -1 if the item was not found</returns>
        public int IndexOf(string parametername)
        {
            for (int i = 0; i < _parameterslist.Count; i++)
            {
                if (_parameterslist[i].ParameterName.ToUpper().Equals(parametername.ToUpper()))
                    return i;
            }
            return -1;
        }

        /// <summary>
        /// Removes the given parameter from the collection
        /// </summary>
        /// <param name="parametername">The name of the parameter to remove</param>
        public void RemoveAt(string parametername)
        {
            for (int i = _parameterslist.Count - 1; i > -1; i--)
                if (_parameterslist[i].ParameterName.ToUpper().Equals(parametername.ToUpper()))
                    _parameterslist.RemoveAt(i);
            _parameters.Remove(parametername);               
        }

        /// <summary>
        /// Returns the parameter with the name given
        /// </summary>
        /// <param name="parametername">The name of the parameter to return</param>
        /// <returns>The parameter requested</returns>
        public object this[string parametername]
        {
            get { return _parameters[parametername]; }
            set { _parameters[parametername] = (CubedParameter)value; }
        }

        /// <summary>
        /// Adds a new Cubed Parameter to the collection
        /// </summary>
        /// <param name="value">The parameter to add</param>
        /// <returns>The added parameter</returns>
        public int Add(object value)
        {
            if (value is CubedParameter)
            {
                CubedParameter newparameter = (CubedParameter)value;
                _parameters.Add(newparameter.ParameterName, newparameter);
                return IndexOf(newparameter.ParameterName);
            }
            else
                throw new CubeConnectionException("Only Cubed Parameters can be added to this collection.",
                    null, String.Empty);
        }

        /// <summary>
        /// Clears all parameters in the collection
        /// </summary>
        public void Clear()
        {
            _parameterslist.Clear();
            _parameters.Clear();
        }

        /// <summary>
        /// Returns true if the parameter collection contains the given value
        /// </summary>
        /// <param name="value">The value to search for</param>
        /// <returns>True if the collection contains the given value, false if not</returns>
        public bool Contains(object value)
        {
            foreach (CubedParameter thisparameter in _parameterslist)
                if (thisparameter.Value == value)
                    return true;
            return false;
        }

        /// <summary>
        /// Returns the index of the given value in the collection
        /// </summary>
        /// <param name="value">The value to look for</param>
        /// <returns>The index of the value, or -1 if it's not found</returns>
        public int IndexOf(object value)
        {
            for (int i = 0; i < _parameterslist.Count; i++)
                if (_parameterslist[i].Value == value)
                    return i;
            return -1;
        }

        /// <summary>
        /// Inserts a new parameter in the collection
        /// </summary>
        /// <param name="index">The index to insert the parameter at</param>
        /// <param name="value">The parameter to add</param>
        public void Insert(int index, object value)
        {
            if (value is CubedParameter)
            {
                CubedParameter newparameter = (CubedParameter)value;
                _parameterslist.Insert(index, newparameter);
                _parameters.Add(newparameter.ParameterName, newparameter);
            }
            else
                throw new CubeConnectionException("Only Cubed Parameters can be added to this collection.",
                    null, String.Empty);
        }

        /// <summary>
        /// Always returns false as this collection is never of fixed size
        /// </summary>
        public bool IsFixedSize
        {
            get { return false; }
        }

        /// <summary>
        /// Always returns false as this collection is never read only
        /// </summary>
        public bool IsReadOnly
        {
            get { return false; }
        }

        /// <summary>
        /// Removes the parameter with the given value from the collection
        /// </summary>
        /// <param name="value">The parameter to remove</param>
        public void Remove(object value)
        {
            for (int i = _parameterslist.Count; i > -1; i--)
                if (_parameterslist[i].Value == value)
                {                    
                    // Remove the parameter
                    _parameters.Remove(_parameterslist[i].ParameterName);
                    _parameterslist.RemoveAt(i);                    
                }
        }

        /// <summary>
        /// Removes the parameter at the index given
        /// </summary>
        /// <param name="index">The index of the parameter to remove</param>
        public void RemoveAt(int index)
        {
            if (_parameterslist.Count > index)
            {
                _parameters.Remove(_parameterslist[index].ParameterName);
                _parameterslist.RemoveAt(index);
            }
        }

        /// <summary>
        /// Returns the parameter at the given index
        /// </summary>
        /// <param name="index">The index of the parameter to return</param>
        /// <returns>The parameter stored at the given index</returns>
        public object this[int index]
        {
            get { return _parameterslist[index]; }
            set { _parameterslist[index] = (CubedParameter)value; }
        }

        /// <summary>
        /// Copys the contents of this collection to an array
        /// </summary>
        /// <param name="array">The array to copy to</param>
        /// <param name="index">The index to begin the copy process at</param>
        public void CopyTo(Array array, int index)
        {
            CubedParameter[] outarray = new CubedParameter[_parameterslist.Count - index];
            _parameterslist.CopyTo(outarray, index);
        }

        /// <summary>
        /// Returns the number of Cubed Parameters in the collection
        /// </summary>
        public int Count
        {
            get { return _parameterslist.Count; }
        }
        
        /// <summary>
        /// Returns false as this is not synchronised
        /// </summary>
        public bool IsSynchronized
        {
            get { return false; }
        }
        
        /// <summary>
        /// Returns null as this is not synchronised
        /// </summary>
        public object SyncRoot
        {
            get { return null;}
        }

        /// <summary>
        /// Returns the enumerator for the collection
        /// </summary>
        /// <returns>The collection enumerator</returns>
        public IEnumerator GetEnumerator()
        {
            return _parameterslist.GetEnumerator();
        }
    }
}
