/*
 * This source file (CubedTransaction.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures;

namespace Bamboo.Cubed.Connect
{
    /// <summary>
    /// This provides an interface for executing multiple transactions with rollback capability
    /// </summary>
    public class CubedTransaction : IDbTransaction
    {
        /// <summary>
        /// This "shadow" data matrix is used in circumstances where a transaction is opened and needs to be 
        /// rolled back. This shadow takes a copy of the data matrix locally to use which can be passed back to
        /// the server if it needs to be
        /// </summary>
        private DataMatrix _shadow;

        /// <summary>
        /// This is the connection underlying this transaction
        /// </summary>
        private CubedConnection _connection;

        /// <summary>
        /// This marks whether the transaction has been successfully opened
        /// </summary>
        internal bool _active;

        /// <summary>
        /// The isolation level for this transaction
        /// </summary>
        private IsolationLevel _isolationlevel;

        /// <summary>
        /// Initialises a new instance of the Cubed Transaction for the given connection
        /// </summary>
        /// <param name="connection">The Cubed Connection this transaction will be opened for</param>
        public CubedTransaction(CubedConnection connection)
        {
            _connection = connection;
            if (_connection != null && _connection.State == ConnectionState.Open
                && !_connection.ConnectionDetails.Admin && _connection.Cube != null)
            {
                // If the snapshot was taken successfully, mark our transaction as active
                _active = true;
            }
            else if (_connection.State != ConnectionState.Open || _connection == null)
            {
                // This connection is not open, throw an exception and set our active value to false
                Dispose();
                throw new CubeConnectionException("The connection has not been opened.", null, String.Empty);
            }
            else if (_connection.ConnectionDetails.Admin)
            {
                // This connection is in administrative mode, so throw an exception
                Dispose();
                throw new CubeConnectionException("This connection is open in administrative mode. It does not support transactions.", null, String.Empty);
            }
            else
            {
                // An unknown error occured
                Dispose();
                throw new CubeConnectionException("An unknown error has occured.", null, String.Empty);
            }
        }
        
        /// <summary>
        /// Commits any write back transactions to the server for the open transaction
        /// </summary>
        public void Commit()
        {
            _connection.CheckTimeout();
            
            if (_connection != null && Connection.State == ConnectionState.Open && _shadow != null
                && _active)
            {
                // The connection is open, so we simply need to close the transaction as the records will
                // have already been committed
                Dispose();

            }
            else if (_connection.State != ConnectionState.Open || _connection == null)
            {
                // The underlying connection is not open, so throw an exception
                Dispose();
                throw new CubeConnectionException("The underlying connection has not been opened.", null, String.Empty);
            }
            else
            {
                // An unknown error has occured, so throw an exception
                Dispose();
                throw new CubeConnectionException("An unknown error has occured.", null, String.Empty);
            }
        }

        /// <summary>
        /// This is the currently active Connection for this transaction
        /// </summary>
        public IDbConnection Connection
        {
            get { return _connection; }
            set { _connection = (CubedConnection)value; }
        }

        /// <summary>
        /// The isolation level of this transaction
        /// </summary>
        public IsolationLevel IsolationLevel
        {
            get { return _isolationlevel; }
            set { _isolationlevel = value; }
        }

        /// <summary>
        /// This rolls back the active transaction and restores the Cube's data to the state prior to 
        /// any changes which were made
        /// </summary>
        public void Rollback()
        {
            _connection.CheckTimeout();
            
            // This transaction has been requested to be rolled back. Check whether the connection is open
            // and that we have a snapshot of the data matrix.
            if (_connection != null && Connection.State == ConnectionState.Open && _shadow != null
               && _active)
            {
                // The transaction is valid, so roll back the data
                bool allok = true;
                foreach (Node thisnode in _shadow.Nodes)
                    if (!_connection.SetNode(_connection.ConnectionString, _connection.Cube,
                        thisnode))
                        allok = false;
                if (!allok)
                {
                    // The save did not work, so throw an error stating that the transaction could not be rolled
                    // back
                    throw new CubeConnectionException("The transaction could not be rolled back.", null, String.Empty);
                }
            }
            else if (_connection.State != ConnectionState.Open || _connection == null)
            {
                // The underlying connection is not open, so throw an exception
                Dispose();
                throw new CubeConnectionException("The underlying connection has not been opened.", null, String.Empty);
            }
            else
            {
                // An unknown error has occured, so throw an exception
                Dispose();
                throw new CubeConnectionException("An unknown error has occured.", null, String.Empty);
            }
        }

        /// <summary>
        /// This closes the transaction
        /// </summary>
        public void Dispose()
        {
            _connection = null;
            _shadow = null;
            _active = false;
        }
    }
}
