/*
 * This source file (AddNewDimension.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Structures;

namespace Bamboo.Cubed.Designer
{
    /// <summary>
    /// This control allows for adding a new dimension
    /// </summary>
    public partial class AddNewDimension : UserControl
    {
        /// <summary>
        /// This initialises a new instance of the Add New Dimension control
        /// </summary>
        /// <param name="editingcube">The cube this dimension is being added to</param>
        public AddNewDimension(Cube editingcube)
        {
            InitializeComponent();
            isloading = true;
            _editingcube = editingcube;
            HeaderLabel.Text = HeaderLabel.Text + " " + editingcube.Name;
            
            // Set up our combo boxes
            DimensionMemberType.Items.Clear();
            foreach (string thismembertype in Enum.GetNames(typeof(MemberTypes)))
                DimensionMemberType.Items.Add(thismembertype);
            if (DimensionMemberType.Items.Count > 0)
                DimensionMemberType.SelectedIndex = 0;
            UsedDatasource.Items.Clear();
            UsedDatasource.DataSource = AvailableDatasources();
            UsedDatasource.DisplayMember = "Name";
            UsedDatasource.ValueMember = "ID";            
            if (editingcube.Datasources.Count > 0)
            {
                // The datasource has not been chosen, but there is one available, so select it
                UsedDatasource.SelectedIndex = 0;
            }
            isloading = false;
        }

        private bool isloading;

        private Cube _editingcube;
        /// <summary>
        /// The cube this dimension is being added to
        /// </summary>
        public Cube EditingCube
        {
            get { return _editingcube; }
            set { _editingcube = value; }
        }

        /// <summary>
        /// This adds the dimension with the details given to the cube, returning TRUE if it is added successfully
        /// </summary>
        public bool AddDimensionToCube()
        {
            // Look for the new dimension in the collection
            Dimension newdimension = new Dimension();
            newdimension.Name = DimensionName.Text;
            newdimension.Description = DimensionDescription.Text;
            newdimension.Enabled = DimensionEnabled.Checked;
            newdimension.TimeDimension = TimeDimension.Checked;
            newdimension.AddTotalMember = IncludeTotalMember.Checked;
            if (UsedDatasource.SelectedValue != null && UsedDatasource.SelectedValue.ToString() != String.Empty)
                newdimension.Datasource = _editingcube.Datasources[new Guid(UsedDatasource.SelectedValue.ToString())].ID;
            if (DimensionMemberType.Text != null && DimensionMemberType.Text != String.Empty)
                newdimension.MemberType = (MemberTypes)Enum.Parse(typeof(MemberTypes),DimensionMemberType.Text);

            // Check that the dimension is valid
            if (newdimension.Name == String.Empty || newdimension.Description == String.Empty)
                return false;

            // Check whether the dimension already exists
            foreach (Dimension thisdimension in _editingcube.Dimensions)
                if (thisdimension.Name.ToUpper().Equals(newdimension.Name.ToUpper()))
                {
                    // This dimension already exists, we can't add it
                    return false;
                }

            // The new dimension is valid, so add it
            _editingcube.Dimensions.Add(newdimension);
            return true;
        }

        /// <summary>
        /// This returns a data table of all available datasources on this cube
        /// </summary>
        /// <returns>A data table of available datasources from the cube</returns>
        private DataTable AvailableDatasources()
        {
            DataTable availabledatasources = new DataTable();
            availabledatasources.Columns.Add("Name");
            availabledatasources.Columns.Add("Description");
            availabledatasources.Columns.Add("ID");

            foreach (Datasource thisdatasource in _editingcube.Datasources)
            {
                DataRow newdatasource = availabledatasources.NewRow();
                newdatasource["Name"] = thisdatasource.Name;
                newdatasource["Description"] = thisdatasource.Description;
                newdatasource["ID"] = thisdatasource.ID;
                availabledatasources.Rows.Add(newdatasource);
            }

            return availabledatasources;
        }

        private void UsedDatasource_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!isloading)
            {
                // The "Used Datasource" has changed, so update our label
                UpdateScreen();
            }
        }

        private void UpdateScreen()
        {
            // Update the description label
            if (UsedDatasource.SelectedIndex > -1)
            {
                DatasourceDescriptionLabel.Text = _editingcube.Datasources[new Guid(UsedDatasource.SelectedValue.ToString())].Description;
            }
        }
    }
}
