/*
 * This source file (AddNewDimension.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Structures;

namespace Bamboo.Cubed.Designer
{
    /// <summary>
    /// This control allows for adding a new dimension
    /// </summary>
    public partial class AddNewMember : UserControl
    {
        /// <summary>
        /// This initialises a new instance of the Add New Dimension control
        /// </summary>
        /// <param name="editingcube">The cube this dimension is being added to</param>
        /// <param name="editingdimensionid">The unique identifier of the dimension this member is being added to</param>
        /// <param name="parentmemberid">The unique identifier of the parent of this member (or empty if at the
        /// root level)</param>
        public AddNewMember(Cube editingcube, Guid editingdimensionid, Guid parentmemberid)
        {
            InitializeComponent();
            _editingcube = editingcube;
            _editingdimensionid = editingdimensionid;
            _parentmemberid = parentmemberid;
            HeaderLabel.Text = HeaderLabel.Text + " " + editingcube.Dimensions[editingdimensionid].Name;

            // Prepare our drop downs
            MemberSignage.Items.Clear();
            foreach (string thissignage in Enum.GetNames(typeof(Signage)))
                MemberSignage.Items.Add(thissignage);
            if (MemberSignage.Items.Count > 0)
                MemberSignage.SelectedIndex = 0;

            MemberAggregationType.Items.Clear();
            foreach (string thisaggregationtype in Enum.GetNames(typeof(AggregationTypes)))
                MemberAggregationType.Items.Add(thisaggregationtype);
            if (MemberAggregationType.Items.Count > 0)
                MemberAggregationType.SelectedIndex = 0;

            MemberConsolidationType.Items.Clear();
            foreach (string thisconsolidationtype in Enum.GetNames(typeof(ConsolidationTypes)))
                MemberConsolidationType.Items.Add(thisconsolidationtype);
            if (MemberConsolidationType.Items.Count > 0)
                MemberConsolidationType.SelectedIndex = 0;
        }

        private Cube _editingcube;
        /// <summary>
        /// The cube this dimension is being added to
        /// </summary>
        public Cube EditingCube
        {
            get { return _editingcube; }
            set { _editingcube = value; }
        }

        private Guid _editingdimensionid;
        /// <summary>
        /// The unique identifier of the dimension this member is being added to
        /// </summary>
        public Guid EditingDimensionID
        {
            get { return _editingdimensionid; }
            set { _editingdimensionid = value; }
        }    

        /// <summary>
        /// The unique identifier of the parent of this member (or empty if this member is being add to the 
        /// root level)
        /// </summary>
        private Guid _parentmemberid;        

        /// <summary>
        /// This adds the member with the details given to the selected dimension in the cube
        /// </summary>
        public bool AddMemberToCubeDimension()
        {
            // First we check that our member is valid
            if (MemberName.Text != String.Empty || MemberDescription.Text != String.Empty)
            {
                // Now we determine whether this member is being added at the root level or as a child
                // of another member
                Member newmember = null;
                if (_parentmemberid == new Guid() ||
                    !_editingcube.Dimensions[_editingdimensionid].Members.Contains(_parentmemberid))
                {
                    // This member is being added at the root level
                    newmember = new Member(MemberName.Text, null, _editingcube.Dimensions[_editingdimensionid]);
                    newmember.Description = MemberDescription.Text;
                    newmember.Aggregation = (AggregationTypes)Enum.Parse(typeof(AggregationTypes),
                        MemberAggregationType.Text);
                    newmember.Consolidation = (ConsolidationTypes)Enum.Parse(typeof(ConsolidationTypes),
                        MemberConsolidationType.Text);
                    newmember.RollUp = MemberRollup.Checked;
                    newmember.Sign = (Signage)Enum.Parse(typeof(Signage), MemberSignage.Text);                    
                }
                else
                {
                    // This member has a parent
                    newmember = new Member(MemberName.Text, _editingcube.Dimensions[_editingdimensionid].Members[_parentmemberid],
                        _editingcube.Dimensions[_editingdimensionid]);
                    newmember.Description = MemberDescription.Text;
                    newmember.Aggregation = (AggregationTypes)Enum.Parse(typeof(AggregationTypes),
                        MemberAggregationType.Text);
                    newmember.Consolidation = (ConsolidationTypes)Enum.Parse(typeof(ConsolidationTypes),
                        MemberConsolidationType.Text);
                    newmember.RollUp = MemberRollup.Checked;
                    newmember.Sign = (Signage)Enum.Parse(typeof(Signage), MemberSignage.Text);      
                }

                // Check to ensure the member doesn't already exist
                foreach (Member thismember in _editingcube.Dimensions[_editingdimensionid].Members)
                    if (thismember.Name.ToUpper().Equals(newmember.Name.ToUpper()))
                        return false;

                // The member is OK, so add it to the collection
                _editingcube.Dimensions[_editingdimensionid].Members.Add(newmember);
                return true;
            }
            else
            {
                // The member is not valid             
                return false;
            }
        }
    }
}
