/*
 * This source file (CubedDesigner.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Linq;
using System.Windows.Forms;
using Bamboo.Cubed.Engine;
using Bamboo.Cubed.Engine.Persistence;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Designer.Properties;
using Bamboo.Cubed.Structures;
using System.IO;

namespace Bamboo.Cubed.Designer
{
    public partial class CubedDesigner : Form
    {
        #region Sub Control Instances

        AddNewServer newservercontrol;
        ManageServerSettings serversettings;
        ManageSystemRoles managesystemroles;
        ManageSystemUsers managesystemusers;
        StorageProviderSettings storageprovidersettings;
        FunctionSettings functionsettings;
        ParserSettings parsersettings;
        ActiveConsolidations activeconsolidations;
        AddNewCube addnewcube;
        AddNewDimension addnewdimension;
        AddNewMember addnewmember;
        ManageCubeSettings managecubesettings;
        ManageDimensionSettings managedimensionsettings;
        ManageMemberSettings managemembersettings;
        CubeDatasourceSettings cubedatasourcesettings;
        DimensionMemberGenerators dimensionmembergenerators;
        ManageAccessRoles manageaccessroles;
        CubeMeasures cubemeasures;
        DimensionMeasureLinks dimensionmeasurelinks;
        ManageLevelSettings managelevelsettings;
        MemberFormulaSettings memberformulasettings;
        DataSetQueryAnalysis datasetqueryanalysis;
        DimensionAttributes dimensionattributes;
        DimensionAttributeValues dimensionattributevalues;
        DoConsolidation doconsolidation;
        CubedConnection _connection = null;

        #endregion

        #region Accessible Properties

        private string _currentuser;
        /// <summary>
        /// The currently logged in, authorised user
        /// </summary>
        public string CurrentUser
        {
            get { return _currentuser; }
            set { _currentuser = value; }
        }

        /// <summary>
        /// The text currently displayed on the status bar
        /// </summary>
        public string ActionStatusText
        {
            get { return ActionStatusLabel.Text; }
            set { ActionStatusLabel.Text = value; }
        }

        /// <summary>
        /// The current maximum of the progress on the status bar
        /// </summary>
        public int ActionProgressMax
        {
            get { return ActionStatusProgress.Maximum; }
            set { ActionStatusProgress.Maximum = value; }
        }

        /// <summary>
        /// The current minimum of the progress on the status bar
        /// </summary>
        public int ActionProgressMin
        {
            get { return ActionStatusProgress.Minimum; }
            set { ActionStatusProgress.Minimum = value; }
        }

        /// <summary>
        /// The current value of the progress on the status bar
        /// </summary>
        public int ActionProgressCurrent
        {
            get { return ActionStatusProgress.Value; }
            set { ActionStatusProgress.Value = value; }
        }

        /// <summary>
        /// Hides the text on the status bar
        /// </summary>
        public void HideStatusText()
        {
            ActionStatusLabel.Visible = false;
        }

        /// <summary>
        /// Shows the text on the status bar
        /// </summary>
        public void ShowStatusText()
        {
            ActionStatusLabel.Visible = true;
        }

        /// <summary>
        /// This hides the "waiting" animation on the toolbar
        /// </summary>
        public void HideWaitingAnimation()
        {
            ActionStatus.Items[WaitingAnimationIndex].Visible = false;
            ActionStatusProgress.Visible = false;
            Cursor = Cursors.Default;
        }

        /// <summary>
        /// This shows the "waiting" animation on the toolbar
        /// </summary>
        public void ShowWaitingAnimation()
        {
            Cursor = Cursors.WaitCursor;
        }

        /// <summary>
        /// Hides the progress on the status bar
        /// </summary>
        public void HideStatusProgress()
        {
            ActionStatusProgress.Visible = false;
        }

        /// <summary>
        /// Shows the status on the progress bar
        /// </summary>
        public void ShowStatusProgress()
        {
            ActionStatusProgress.Visible = true;
        }

        private CurrentActions _currentaction = CurrentActions.None;
        /// <summary>
        /// The current action in the Action Panel
        /// </summary>
        public CurrentActions CurrentAction
        {
            get { return _currentaction; }
            set { _currentaction = value; }
        }

        private string _titletext;

        private ConnectedServer _connectedserver;

        #endregion

        #region Internal Methods

        /// <summary>
        /// This shows a standard error dialog
        /// </summary>
        /// <param name="errormessage">The error message to display</param>
        private void ShowError(string errormessage)
        {
            MessageBox.Show(errormessage, "An error has occured", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        /// <summary>
        /// This shows a standard information dialog
        /// </summary>
        /// <param name="infomessage">The information to display</param>
        private void ShowInformation(string infomessage)
        {
            MessageBox.Show(infomessage, "For your information", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        /// <summary>
        /// This shows a standard warning dialog
        /// </summary>
        /// <param name="warningmessage">The warning to display</param>
        private void ShowWarning(string warningmessage)
        {
            MessageBox.Show(warningmessage, "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }

        /// <summary>
        /// This checks to ensure the current connection is open and accessible
        /// </summary>
        /// <param name="server">The server to connect to</param>
        /// <param name="forcenew">Whether to force the new connection to be re-created and opened</param>
        private void CheckConnection(ConnectedServer server, bool forcenew)
        {
            if (_connection == null || _connection.State != ConnectionState.Open || forcenew)
            {
                // Check whether we are using integrated authentication or username/password credentials
                if (server.EndPoint.IntegratedAuthentication)
                {
                    // We are using integrated security
                    string connectionstring = "Server=" + server.EndPoint.Server + ";" +
                                "Port=" + server.EndPoint.Port.ToString() + ";Protocol=" +
                                server.EndPoint.EndpointType.ToString() + ";Timeout=" + server.TimeOut.ToString()
                                + ";Admin=True;User ID=" + Environment.UserDomainName + "\\" + Environment.UserName 
                                + ";Integrated Security=true;"                                ;
                    _connection = new CubedConnection(connectionstring);
                    _connection.Open();
                }
                else
                {
                    // Attempt to pass through the username/password specified rather than the integrated username
                    string connectionstring = "Server=" + server.EndPoint.Server + ";" +
                                "Port=" + server.EndPoint.Port.ToString() + ";Protocol=" +
                                server.EndPoint.EndpointType.ToString() + ";Timeout=" + server.TimeOut.ToString()
                                + ";Admin=True;User ID=" + server.Username
                                + ";Password=" + server.Password.ToString() + ";Integrated Security=false;";
                               
                    _connection = new CubedConnection(connectionstring);
                    _connection.Open();
                }                
                
                // Then read the settings from the server and save them locally (in case a local consolidation is
                // started)
                SettingsManager.Settings = _connection.Administration.Settings;
            }
        }

        /// <summary>
        /// Closes the currently open connection
        /// </summary>
        private void CloseConnection()
        {
            if (_connection != null && _connection.State == ConnectionState.Open)
            {
                _connection.Close();
            }
        }

        /// <summary>
        /// This customises the current screen for the particular current action
        /// </summary>
        private void CustomiseScreen()
        {
            switch (_currentaction)
            {
                case CurrentActions.None:
                    ActionStatus.Text = "Welcome to Cubed.";
                    HideStatusProgress();
                    ShowStatusText();
                    HideWaitingAnimation();
                    ActionToolStrip.Visible = false;
                    break;
                case CurrentActions.AddNewServer:
                    ActionStatus.Text = "Adding new Server";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Save and add the new server.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "This cancels without adding the new server.";
                    break;
                case CurrentActions.ConfigureServer:
                    ActionStatus.Text = "Managing Server Settings";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Save the general settings for this server.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "This cancels without saving the settings.";
                    break;
                case CurrentActions.ManageSystemRoles:
                    ActionStatus.Text = "Managing System Roles";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Save the roles which have been edited.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "This cancels without saving the updated roles.";
                    AddButton.Visible = false;
                    RemoveButton.Visible = false;
                    break;
                case CurrentActions.ManageSystemUsers:
                    ActionStatus.Text = "Managing System Users";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Save the users which have been edited.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "This cancels without saving the updated users.";
                    AddButton.Visible = false;
                    RemoveButton.Visible = false;
                    break;
                case CurrentActions.StorageProviders:
                    ActionStatus.Text = "Managing Storage Provider";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Save the storage provider settings for this server.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "This cancels without saving the storage provider settings.";
                    break;
                case CurrentActions.Functions:
                    ActionStatus.Text = "Managing Available Functions";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Save the list of available functions for this server.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "This cancels without saving the available function settings.";
                    AddButton.Visible = true;
                    AddButton.ToolTipText = "This adds the currently selected function Add In to the available collection.";
                    RemoveButton.Visible = functionsettings.FunctionSelected();
                    RemoveButton.ToolTipText = "This removes the currently selected available function.";
                    break;
                case CurrentActions.QueryParsers:
                    ActionStatus.Text = "Managing Available Query Parsers";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Save the list of available parsers for this server.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "This cancels without saving the available parser settings.";
                    AddButton.Visible = true;
                    AddButton.ToolTipText = "This adds the currently selected Parser Add In to the available collection.";
                    RemoveButton.Visible = parsersettings.ParserSelected();
                    RemoveButton.ToolTipText = "This removes the currently selected available parser.";
                    break;
                case CurrentActions.Consolidations:
                    ActionStatus.Text = "Managing Consolidations";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    RefreshButton.Visible = true;
                    RefreshButton.ToolTipText = "This refreshes the current consolidations statuses from the server.";
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "This saves any changes made to the Consolidation Intervals.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "This leaves the consolidations screen.";
                    break;
                case CurrentActions.NewCube:
                    ActionStatus.Text = "Adding a new Cube";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to add the new cube to the server collection.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to cancel adding a new cube.";
                    break;
                case CurrentActions.NewDimension:
                    ActionStatus.Text = "Adding a new Dimension";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to add the new dimension to the selected cube.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to cancel adding a new dimension.";
                    break;
                case CurrentActions.NewMember:
                    ActionStatus.Text = "Adding a new Member";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to add the new member to the selected dimension.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to cancel adding a new member.";
                    break;
                case CurrentActions.CubeSettings:
                    ActionStatus.Text = "Managing Cube Settings";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to save the settings of the selected cube.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to cancel your changes.";
                    break;
                case CurrentActions.DimensionSettings:
                    ActionStatus.Text = "Managing Dimension Settings";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to save the settings of the selected dimension.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to cancel your changes.";
                    break;
                case CurrentActions.MemberSettings:
                    ActionStatus.Text = "Managing Member Settings";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to save the settings of the selected member.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to cancel your changes.";
                    break;
                case CurrentActions.Datasources:
                    ActionStatus.Text = "Managing Cube Datasources";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Save the list of available datasources for this cube.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "This cancels without saving the available datasources.";
                    AddButton.Visible = true;
                    AddButton.ToolTipText = "This adds the currently selected Database Add In as a new data source.";
                    RemoveButton.Visible = cubedatasourcesettings.DatasourceSelected();
                    RemoveButton.ToolTipText = "This removes the currently selected available datasource.";
                    break;
                case CurrentActions.DimensionMemberGenerators:
                    ActionStatus.Text = "Managing Dimension Member Generators";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to save the member generators of the selected dimension.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to cancel your changes.";
                    AddButton.Visible = true;
                    AddButton.ToolTipText = "This adds a new member generator to the collection.";
                    RemoveButton.Visible = dimensionmembergenerators.GeneratorSelected();
                    RemoveButton.ToolTipText = "This removes the currently selected member generator.";
                    break;
                case CurrentActions.ManageAccessRoles:
                    ActionStatus.Text = "Managing Security Roles";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to save the security roles.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to cancel your changes.";
                    break;
                case CurrentActions.Measures:
                    ActionStatus.Text = "Managing Cube Measures";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to save your measures.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to cancel your changes.";
                    AddButton.Visible = true;
                    AddButton.ToolTipText = "Click here to add a new measure to the cube.";
                    RemoveButton.Visible = cubemeasures.MeasureSelected();
                    RemoveButton.ToolTipText = "Click here to remove the selected measure.";
                    break;
                case CurrentActions.MeasureLinks:
                    ActionStatus.Text = "Managing Dimension Measure Links";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to save the dimension measure links.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to cancel your changes.";
                    break;
                case CurrentActions.Levels:
                    ActionStatus.Text = "Managing Dimension Levels";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to save the dimension levels.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to cancel your changes.";
                    break;
                case CurrentActions.MemberFormula:
                    ActionStatus.Text = "Managing Member Formula";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to save the formula configuration for this member.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to cancel your changes.";
                    RemoveButton.Visible = true;
                    RemoveButton.ToolTipText = "Click here to remove the member formula on this member.";
                    break;
                case CurrentActions.Analysis:
                    ActionStatus.Text = "Running Analysis";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to save the query.";
                    OpenButton.Visible = true;
                    OpenButton.ToolTipText = "Click here to load a previously saved query.";
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to close this query.";
                    break;
                case CurrentActions.Attributes:
                    ActionStatus.Text = "Managing Dimension Attributes";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to save the attributes for this dimension.";                    
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to cancel any changes to the dimension attributes.";
                    AddButton.Visible = true;
                    AddButton.ToolTipText = "Click here to add a new attribute to the dimension.";
                    RemoveButton.Visible = dimensionattributes.AttributeSelected();
                    RemoveButton.ToolTipText = "Click here to remove the selected attribute.";
                    break;
                case CurrentActions.MemberAttributes:
                    ActionStatus.Text = "Managing Member Attribute Values";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    SaveButton.Visible = true;
                    SaveButton.ToolTipText = "Click here to save the attribute values for this member.";                    
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to cancel any changes to the member attribute values.";
                    break;
                case CurrentActions.DoConsolidation:
                    ActionStatus.Text = "Consolidating Current Cube";
                    HideStatusProgress();
                    ShowStatusText();
                    ActionToolStrip.Visible = true;
                    HideAllChildren(ActionToolStrip);
                    CancelButton.Visible = true;
                    CancelButton.ToolTipText = "Click here to close the consolidation window (and cancel the current consolidation).";
                    break;
            }
        }

        /// <summary>
        /// This hides all child controls of a control
        /// </summary>
        /// <param name="control">The referenced control</param>
        private void HideAllChildren(ToolStrip toolstrip)
        {
            foreach (ToolStripItem thisitem in toolstrip.Items)
                thisitem.Visible = false;
        }

        /// <summary>
        /// This shows all child controls of a control
        /// </summary>
        /// <param name="control">The referenced control</param>
        private void ShowAllChildren(ToolStrip toolstrip)
        {
            foreach (ToolStripItem thisitem in toolstrip.Items)
                thisitem.Visible = true;
        }

        #endregion

        #region Constructors

        public CubedDesigner()
        {
            InitializeComponent();
        }

        private void CubedDesigner_Load(object sender, EventArgs e)
        {
            // Grab our default details
            this.Hide();
            this.WindowState = FormWindowState.Normal;
            _currentuser = Environment.UserDomainName + "\\" + Environment.UserName;
            _titletext = "Cubed Designer " + Information.ReleaseHeader + " (" + Information.ReleaseDetails + ") ["
                + _currentuser + "]";
            this.Text = _titletext;
            VersionHeader.Text = Information.ReleaseHeader;
            VersionDetails.Text = Information.ReleaseDetails + " (" + Information.TrueVersion + ")";
            for (int i = 0; i < ServerActionsStrip.Items.Count; i++)
            {
                ServerActionsStrip.Items[i].Enabled = false;
            }
            CustomiseScreen();
            AddNewServerButton.Enabled = true;

            // Set our first tab to active
            CubedRibbon.CurrentTabPage = ServerTabPage;

            // Display the splash screen
            DesignerSplash splash = new DesignerSplash();
            splash.Show();
            splash.Refresh();
            this.Refresh();
            DateTime loadtime = DateTime.Now;

            // Re-load previous settings
            LoadFromSettings();

            // Hide the splash screen (only if it's taken at least 3 seconds to load Cubed)
            while (DateTime.Now < loadtime.AddSeconds(3))
            { }
            splash.Close();

            // Customise the screen with default settings            
            CustomiseScreen();
            WaitingAnimiation = ActionStatus.Items.Add(Resources.WaitingAnimation_16);
            WaitingAnimationIndex = ActionStatus.Items.IndexOf(WaitingAnimiation);
            HideWaitingAnimation();
            ServerList_SelectedIndexChanged(this, new EventArgs());
            this.Show();
            this.WindowState = FormWindowState.Normal;
            if (DesignerSettingsManager.Settings.WindowMaximised)
                this.WindowState = FormWindowState.Maximized;
            else
                this.WindowState = FormWindowState.Normal;
        }

        private void LoadFromSettings()
        {
            // First we check whether the designer should be maximised or not
            Cursor = Cursors.WaitCursor;
            if (DesignerSettingsManager.Settings.WindowMaximised)
                this.WindowState = FormWindowState.Maximized;
            else
                this.WindowState = FormWindowState.Normal;

            // Then we loop through each previously loaded server connection and re-initialise it
            int servererrors = 0;
            for (int i = DesignerSettingsManager.Settings.ConnectedServers.Count - 1; i > -1; i-- )
            {
                // Try and connect to the server to check it exists
                try
                {
                    CheckConnection(DesignerSettingsManager.Settings.ConnectedServers[i], false);
                    ListViewItem newserver = new ListViewItem(DesignerSettingsManager.Settings.ConnectedServers[i].EndPoint.Server, 5);
                    if (!ServerList.Items.Contains(newserver))
                        ServerList.Items.Add(newserver);
                }
                catch
                {
                    servererrors++;

                    // Remove the faulty server connection
                    DesignerSettingsManager.Settings.ConnectedServers.RemoveAt(i);
                }
            }
            Cursor = Cursors.Default;
            if (servererrors > 0)
            {
                ShowError(servererrors.ToString() + " of your previous server connections were unable to be restored.");
            }
        }

        #endregion

        #region Launcher Buttons

        private void AddNewServerButton_Click(object sender, EventArgs e)
        {
            // Add the new server window into the main panel
            CurrentAction = CurrentActions.AddNewServer;
            newservercontrol = new AddNewServer(new ConnectedServer());
            newservercontrol.Dock = DockStyle.Fill;
            ActionPanel.Controls.Add(newservercontrol);
            CustomiseScreen();
        }

        private void RemoveServerButton_Click(object sender, EventArgs e)
        {
            if (ServerList.SelectedItems.Count > 0)
            {
                // Remove the server from the designers settings
                if (DesignerSettingsManager.Settings.ConnectedServers.Exists(thisserver =>
                thisserver.EndPoint.Server.ToUpper() == ServerList.SelectedItems[0].Text.ToUpper()))
                    DesignerSettingsManager.Settings.ConnectedServers.RemoveAt(DesignerSettingsManager.Settings.ConnectedServers.FindIndex(thisserver =>
                        thisserver.EndPoint.Server.ToUpper() == ServerList.SelectedItems[0].Text.ToUpper()));

                // Remove the server from the list
                ServerList.Items.Remove(ServerList.SelectedItems[0]);
            }
        }

        private void GeneralServerSettingsButton_Click(object sender, EventArgs e)
        {
            if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving general settings from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.ConfigureServer;
                    if (_connection.Administration.Settings.EndPoints.Count < 1)
                        _connection.Administration.Settings.EndPoints.Add(new ServiceEndPoint());
                    serversettings = new ManageServerSettings(ServerList.SelectedItems[0].Text,
                        _connection.Administration.Settings.StructuresTableName,
                        _connection.Administration.Settings.DataTablePrefix,
                        _connection.Administration.Settings.LogLevel,
                        _connection.Administration.Settings.SMTPServer,
                        _connection.Administration.Settings.SMTPPort,
                        _connection.Administration.Settings.SMTPSSL,
                        _connection.Administration.Settings.SMTPAuthenticated,
                        _connection.Administration.Settings.SMTPUser,
                        _connection.Administration.Settings.SMTPPassword,
                        _connection.Administration.Settings.EndPoints[0].EndpointType,
                        _connection.Administration.Settings.EndPoints[0].Server,
                        _connection.Administration.Settings.EndPoints[0].Port,
                        _connection.Administration.Settings.MaxMemory,
                        _connection.Administration.Settings.AddInsDirectory);
                    serversettings.Dock = DockStyle.Fill;
                    ActionPanel.Controls.Add(serversettings);
                    ActionStatusText = "Settings Loaded.";
                    HideWaitingAnimation();
                    CustomiseScreen();
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving general settings failed.";
                }
            }
        }

        private void RolesButton_Click(object sender, EventArgs e)
        {
            if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving roles from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.ManageSystemRoles;
                    managesystemroles = new ManageSystemRoles(_connection.Administration.ReadRoles(),
                        _connection.Administration.ReadUsers(), _connection);
                    managesystemroles.Dock = DockStyle.Fill;
                    ActionPanel.Controls.Add(managesystemroles);
                    ActionStatusText = "System Roles Loaded.";
                    HideWaitingAnimation();
                    CustomiseScreen();
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving system roles failed.";
                }
            }
        }

        private void UsersButton_Click(object sender, EventArgs e)
        {
            if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving users from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.ManageSystemUsers;

                    // Load the current users
                    managesystemusers = new ManageSystemUsers(_connection.Administration.ReadUsers(),
                        _connection);

                    // Update the users which are administrators
                    foreach (User thisuser in managesystemusers.Users)
                        if (_connection.Administration.Settings.RootUsers.Exists(testroot =>
                            testroot.ToUpper() == thisuser.DomainUserName.ToUpper()))
                            thisuser.IsAdmin = true;
                        else
                            thisuser.IsAdmin = false;

                    managesystemusers.Dock = DockStyle.Fill;
                    ActionPanel.Controls.Add(managesystemusers);
                    ActionStatusText = "System Users Loaded.";
                    HideWaitingAnimation();
                    CustomiseScreen();
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving system users failed.";
                }
            }
        }

        private void StorageProviderButton_Click(object sender, EventArgs e)
        {
            if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving current storage provider settings from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.StorageProviders;
                    storageprovidersettings = new StorageProviderSettings(_connection.Administration.Settings.StorageProvider);
                    storageprovidersettings.Dock = DockStyle.Fill;
                    ActionPanel.Controls.Add(storageprovidersettings);
                    ActionStatusText = "Current Storage Provider Settings Loaded.";
                    HideWaitingAnimation();
                    CustomiseScreen();
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving current storage provider settings failed.";
                }
            }
        }

        private void FunctionsButton_Click(object sender, EventArgs e)
        {
            if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving current available function settings from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.Functions;
                    functionsettings = new FunctionSettings(_connection.Administration.Settings.Functions);
                    functionsettings.CurrentFunctionsGrid.SelectionChanged += new EventHandler(CurrentFunctionsGrid_SelectionChanged);
                    functionsettings.Dock = DockStyle.Fill;
                    ActionPanel.Controls.Add(functionsettings);
                    ActionStatusText = "Current Available Function Settings Loaded.";
                    HideWaitingAnimation();
                    CustomiseScreen();
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving current available function settings failed.";
                }
            }
        }

        private void ParsersButton_Click(object sender, EventArgs e)
        {
            if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving current available parser settings from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.QueryParsers;
                    parsersettings = new ParserSettings(_connection.Administration.Settings.Parsers);
                    parsersettings.CurrentParsersGrid.SelectionChanged += new EventHandler(CurrentParsersGrid_SelectionChanged);
                    parsersettings.Dock = DockStyle.Fill;
                    ActionPanel.Controls.Add(parsersettings);
                    ActionStatusText = "Current Available Parser Settings Loaded.";
                    HideWaitingAnimation();
                    CustomiseScreen();
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving current available parser settings failed.";
                }
            }
        }

        private void ConsolidationsButton_Click(object sender, EventArgs e)
        {
            if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving current consolidation information from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.Consolidations;
                    activeconsolidations = new ActiveConsolidations(_connection.Administration.CubeHeaders, _connection.Administration.Settings,
                        _connection);
                    activeconsolidations.Dock = DockStyle.Fill;
                    ActionPanel.Controls.Add(activeconsolidations);
                    ActionStatusText = "Current Available Consolidation Information Loaded.";
                    HideWaitingAnimation();
                    CustomiseScreen();
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving current available consolidation information failed.";
                }
            }
        }

        private void DeleteCubeButton_Click(object sender, EventArgs e)
        {
            if (CubesList.SelectedItems.Count > 0 && CubesList.SelectedItems[0].Text != String.Empty)
            {
                if (MessageBox.Show("Are you sure you wish to delete this cube? This cannot be undone.",
                    "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                {
                    // The user has selected yes, delete the cube
                    ActionStatusText = "Attempting to remove the selected cube...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to remove the selected cube from the server
                    try
                    {
                        // First find the cube
                        Cube removecube = null;
                        foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                            if (thiscube.Name.ToUpper().Equals(CubesList.SelectedItems[0].Text.ToUpper()))
                            {
                                // This is the cube we want to delete
                                removecube = _connection.Administration.GetCube(thiscube.ID);
                                break;
                            }

                        // Now attempt to delete the cube                        
                        _connection.Administration.RemoveCube(removecube);

                        // Now re-load the cubes list
                        CubesList.Items.Clear();
                        foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                            CubesList.Items.Add(new ListViewItem(thiscube.Name, 0));

                        // Report back success
                        ActionStatusText = "Cube deleted.";
                        HideWaitingAnimation();
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not delete the new cube. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Deletion of the cube failed.";
                    }
                }
            }
        }

        private void NewCubeButton_Click(object sender, EventArgs e)
        {
            // Add the new cube window into the main panel
            CurrentAction = CurrentActions.NewCube;
            addnewcube = new AddNewCube();
            addnewcube.Dock = DockStyle.Fill;
            ActionPanel.Controls.Add(addnewcube);
            CustomiseScreen();
        }

        private void AddDimensionButton_Click(object sender, EventArgs e)
        {
            // First grab the selected cube
            ActionStatusText = "Preparing to add a new dimension...";
            ShowStatusText();
            CustomiseScreen();
            ShowWaitingAnimation();
            this.Refresh();

            // Attempt to connect to the server
            CheckConnection(_connectedserver, false);

            // Attempt to grab our new connection's list of cubes
            try
            {
                // First find the cube
                Cube dimensioncube = null;
                foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                    if (thiscube.Name.ToUpper().Equals(CubesList.SelectedItems[0].Text.ToUpper()))
                    {
                        // This is the cube we want to add a dimension to
                        dimensioncube = _connection.Administration.GetCube(thiscube.ID);
                        break;
                    }

                // Add the new dimension window into the main panel
                CurrentAction = CurrentActions.NewDimension;
                addnewdimension = new AddNewDimension(dimensioncube);
                addnewdimension.Dock = DockStyle.Fill;
                ActionPanel.Controls.Add(addnewdimension);
                CustomiseScreen();
                HideWaitingAnimation();
            }
            catch (CubeConnectionException c1)
            {
                ShowError("Could not add a new dimension to the cube cube. " + c1.ToString());
                HideWaitingAnimation();
                ActionStatusText = "Adding of new dimension failed.";
            }
        }

        private void RemoveDimensionsButton_Click(object sender, EventArgs e)
        {
            if (DimensionsList.SelectedItems.Count > 0 && DimensionsList.SelectedItems[0].Text != String.Empty)
            {
                if (MessageBox.Show("Are you sure you wish to delete this dimension? This cannot be undone.",
                    "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                {
                    // The user has selected yes, delete the dimension
                    ActionStatusText = "Attempting to remove the selected dimension...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to remove the selected dimension from the server
                    try
                    {
                        // First find the the dimension belongs in
                        Cube removecube = null;
                        foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                            if (thiscube.Name.ToUpper().Equals(CubesList.SelectedItems[0].Text.ToUpper()))
                            {
                                // This is the cube we want to delete a dimension from
                                removecube = _connection.Administration.GetCube(thiscube.ID);
                                break;
                            }

                        // Now attempt to find the dimension in the cube to remove
                        for (int i = removecube.Dimensions.Count - 1; i > -1; i--)
                            if (removecube.Dimensions[i].Name.ToUpper().
                                Equals(DimensionsList.SelectedItems[0].Text.ToUpper()))
                            {
                                removecube.Dimensions.RemoveAt(i);
                                _connection.Administration.SetCube(removecube);
                                _connection.Administration.SaveCube(removecube.ID);
                                break;
                            }

                        // Report back success
                        ActionStatusText = "Cube deleted.";
                        HideWaitingAnimation();

                        // Now re-load the dimensions list
                        DimensionsList.Items.Clear();
                        foreach (Dimension thisdimension in removecube.Dimensions)
                            DimensionsList.Items.Add(new ListViewItem(thisdimension.Name, 6));
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not delete the new dimension. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Deletion of the dimension failed.";
                    }
                }
            }
        }

        private void ExportCubeButton_Click(object sender, EventArgs e)
        {
            if (CubesList.SelectedItems.Count > 0 && CubesList.SelectedItems[0].Text != String.Empty)
            {
                // The user has selected a cube, try and find it in our collection
                ActionStatusText = "Preparing to export cube...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                // Attempt to connect to the server
                CheckConnection(_connectedserver, false);

                // Attempt to grab our new connection's list of cubes
                try
                {                    
                    // Grab the export cube definition from the server
                    byte[] exportedcube = _connection.Administration.ExportCube(_connection.Administration.CubeHeaders.FirstOrDefault(testcube =>
                        testcube.Name == CubesList.SelectedItems[0].Text).ID);

                    // Get the default name
                    ExportCubeLocation.FileName = CubesList.SelectedItems[0].Text + ".cz";

                    // Now prompt the user for an export location
                    if (ExportCubeLocation.ShowDialog() == DialogResult.OK)
                    {
                        // The user has selected a location, so export the file
                        if (File.Exists(ExportCubeLocation.FileName))
                            File.Delete(ExportCubeLocation.FileName);

                        // Save the file contents
                        File.WriteAllBytes(ExportCubeLocation.FileName, exportedcube);

                        // Show a success message
                        ActionStatusText = "Cube exported successfully to: " + ExportCubeLocation.FileName + ".";
                        HideWaitingAnimation();
                    }
                    else
                    {
                        // The user cancelled, so reset our status
                        ActionStatusText = "Cube export cancelled.";
                        HideWaitingAnimation();

                    }
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not export the selected cube. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Export of cubes failed.";
                }
            }
        }

        private void ImportCubeButton_Click(object sender, EventArgs e)
        {
            // First ask the user to choose the file
            try
            {
                if (ImportCubeLocation.ShowDialog() == DialogResult.OK &&
                    ImportCubeLocation.FileName != String.Empty && File.Exists(ImportCubeLocation.FileName))
                {
                    // Show our status
                    ActionStatusText = "Importing selected cube...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Grab our current cube selection to see if the cube already exists
                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // The user has chosen a valid file, try and import from it
                    byte[] exportfile = File.ReadAllBytes(ImportCubeLocation.FileName);

                    // Now try and deserialise the original cube
                    bool importsuccess = _connection.Administration.ImportCube(exportfile);

                    if (importsuccess)
                    {
                        // As the cube imported successfully, update the status to reflect that
                        CheckConnection(_connectedserver, true);
                        CubesList.Items.Clear();
                        foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                            CubesList.Items.Add(new ListViewItem(thiscube.Name, 0));

                        // Report success
                        ActionStatusText = "Import successful.";
                        HideWaitingAnimation();
                    }
                    else
                    {
                        // As the cube didn't import, report the error
                        ActionStatusText = "The import failed.";
                        HideWaitingAnimation();
                    }
                }
            }
            catch (CubeConnectionException c1)
            {
                ShowError("Could not import the selected cube. " + c1.ToString());
                HideWaitingAnimation();
                ActionStatusText = "Import of cube failed.";
            }
        }

        private void AddMemberButton_Click(object sender, EventArgs e)
        {
            // First grab the selected cube
            ActionStatusText = "Preparing to add a new member...";
            ShowStatusText();
            CustomiseScreen();
            ShowWaitingAnimation();
            this.Refresh();

            // Attempt to connect to the server
            CheckConnection(_connectedserver, false);

            // Attempt to grab our new connection's list of cubes
            try
            {
                // First find the cube
                Cube dimensioncube = null;
                foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                    if (thiscube.Name.ToUpper().Equals(CubesList.SelectedItems[0].Text.ToUpper()))
                    {
                        // This is the cube we want to add a member to
                        dimensioncube = _connection.Administration.GetCube(thiscube.ID);
                        break;
                    }

                // Now find our dimension
                Dimension memberdimension = null;
                foreach (Dimension thisdimension in dimensioncube.Dimensions)
                    if (thisdimension.Name.ToUpper().Equals(DimensionsList.SelectedItems[0].Text.ToUpper()))
                    {
                        // This is the dimension we want to add a member to
                        memberdimension = thisdimension;
                        break;
                    }

                // Now we determine if this member is being added to the root of the dimension or under a parent
                if ((MembersList.SelectedNode != null && MembersList.SelectedNode.Text ==
                    memberdimension.Name) || MembersList.SelectedNode == null)
                {
                    // This member is being added to the dimension at the root level
                    addnewmember = new AddNewMember(dimensioncube, memberdimension.ID, new Guid());
                }
                else if (MembersList.SelectedNode != null && MembersList.SelectedNode.Text != String.Empty)
                {
                    // A member has been chosen which isn't the dimension, so we assume the new member is being
                    // added under that member. First we need to find the member
                    Member parentmember = null;
                    foreach (Member thismember in memberdimension.Members)
                        if (thismember.Name.ToUpper().Equals(MembersList.SelectedNode.Text.ToUpper()))
                        {
                            // This is the parent member we want to add this member under
                            parentmember = thismember;
                            break;
                        }

                    // Now we initialise the member
                    addnewmember = new AddNewMember(dimensioncube, memberdimension.ID, parentmember.ID);
                }

                // Add the new member window into the main panel
                CurrentAction = CurrentActions.NewMember;
                addnewmember.Dock = DockStyle.Fill;
                ActionPanel.Controls.Add(addnewmember);
                CustomiseScreen();
                HideWaitingAnimation();
            }
            catch (CubeConnectionException c1)
            {
                ShowError("Could not add a new member to the cube. " + c1.ToString());
                HideWaitingAnimation();
                ActionStatusText = "Adding of new member failed.";
            }
        }

        private void DeleteMemberButton_Click(object sender, EventArgs e)
        {
            if (MembersList.SelectedNode != null && MembersList.SelectedNode.Text != String.Empty &&
                MembersList.SelectedNode.Text != DimensionsList.SelectedItems[0].Text)
            {
                if (MessageBox.Show("Are you sure you wish to delete this member? This cannot be undone.",
                    "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                {
                    // The user has selected yes, delete the member
                    ActionStatusText = "Attempting to remove the selected member...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to remove the selected member from the server
                    try
                    {
                        // First find the the member belongs in
                        Cube removecube = null;
                        foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                            if (thiscube.Name.ToUpper().Equals(CubesList.SelectedItems[0].Text.ToUpper()))
                            {
                                // This is the cube we want to delete a dimension from
                                removecube = _connection.Administration.GetCube(thiscube.ID);
                                break;
                            }

                        // Now attempt to find the dimension in the member exists in
                        Dimension removedimension = null;
                        foreach (Dimension thisdimension in removecube.Dimensions)
                            if (thisdimension.Name.ToUpper().
                                Equals(DimensionsList.SelectedItems[0].Text.ToUpper()))
                            {
                                // This is the dimension it belongs in
                                removedimension = thisdimension;
                                break;
                            }

                        // Now try and find the member
                        List<Member> memberchildren = new List<Member>();
                        foreach (Member thismember in removedimension.Members)
                            if (thismember.Name.ToUpper().Equals(MembersList.SelectedNode.Text.ToUpper()))
                            {
                                // This is our selected member, so we try and delete it and any of its' children                                
                                GatherAllDescendants(ref memberchildren, thismember, removedimension);
                                break;
                            }

                        // Now we have the complete list of the member and all its' children, remove them all
                        // from the dimensions member list
                        foreach (Member thisremovedmember in memberchildren)
                            removecube.Dimensions[removedimension.ID].Members.Remove(thisremovedmember.ID);

                        // Now publish the cube back to the server
                        _connection.Administration.SetCube(removecube);
                        _connection.Administration.SaveCube(removecube.ID);

                        // Now re-load the members list
                        Cube newcube = null;
                        foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                            if (thiscube.Name == removecube.Name)
                                newcube = _connection.Administration.GetCube(thiscube.ID);
                        MembersList.Nodes.Clear();
                        TreeNode rootnode = new TreeNode(newcube.Dimensions[removedimension.ID].Name, 6, 6);
                        BuildMembersTree(ref rootnode, newcube.Dimensions[removedimension.ID], null, true);
                        rootnode.Expand();
                        MembersList.Nodes.Add(rootnode);

                        // Report back success
                        ActionStatusText = "Member deleted.";
                        HideWaitingAnimation();
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not delete the member. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Deletion of the member failed.";
                    }
                }
            }
        }

        private void CubeSettings_Click(object sender, EventArgs e)
        {
            if (CubesList.SelectedItems.Count > 0 && CubesList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving cube settings from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.CubeSettings;

                    // Find our cube
                    Cube editingcube = null;
                    foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                        if (thiscube.Name == CubesList.SelectedItems[0].Text)
                            editingcube = _connection.Administration.GetCube(thiscube.ID);
                    if (editingcube != null)
                    {
                        managecubesettings = new ManageCubeSettings(editingcube, 
                            _connection.Administration.ReadUsers());
                        managecubesettings.Dock = DockStyle.Fill;
                        ActionPanel.Controls.Add(managecubesettings);
                        ActionStatusText = "Cube Loaded.";
                        HideWaitingAnimation();
                        CustomiseScreen();
                    }
                    else
                    {
                        // The cube wasn't found so show an error
                        ShowError("The selected cube cannot be edited.");
                        HideWaitingAnimation();
                        ActionStatusText = "Cube wasn't found.";
                    }
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving cube settings failed.";
                }
            }
        }

        private void DimensionSettingsButton_Click(object sender, EventArgs e)
        {
            if (DimensionsList.SelectedItems.Count > 0 && DimensionsList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving dimension settings from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.DimensionSettings;

                    // Find our cube and dimension
                    Cube editingcube = null;
                    foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                        if (thiscube.Name == CubesList.SelectedItems[0].Text)
                            editingcube = _connection.Administration.GetCube(thiscube.ID);
                    Dimension editingdimension = null;
                    if (editingcube != null)
                        foreach (Dimension thisdimension in editingcube.Dimensions)
                            if (thisdimension.Name == DimensionsList.SelectedItems[0].Text)
                                editingdimension = thisdimension;
                    if (editingcube != null && editingdimension != null)
                    {
                        managedimensionsettings = new ManageDimensionSettings(editingcube, editingdimension.ID);
                        managedimensionsettings.Dock = DockStyle.Fill;
                        ActionPanel.Controls.Add(managedimensionsettings);
                        ActionStatusText = "Dimension Loaded.";
                        HideWaitingAnimation();
                        CustomiseScreen();
                    }
                    else
                    {
                        // The dimension wasn't found so show an error
                        ShowError("The selected dimension cannot be edited.");
                        HideWaitingAnimation();
                        ActionStatusText = "Dimension wasn't found.";
                    }
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving dimension settings failed.";
                }
            }
        }

        private void MemberSettingsButton_Click(object sender, EventArgs e)
        {
            if (MembersList.SelectedNode != null && MembersList.SelectedNode.Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving member settings from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.MemberSettings;

                    // Find our cube, dimension and member
                    Cube editingcube = null;
                    foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                        if (thiscube.Name == CubesList.SelectedItems[0].Text)
                            editingcube = _connection.Administration.GetCube(thiscube.ID);
                    Dimension editingdimension = null;
                    if (editingcube != null)
                        foreach (Dimension thisdimension in editingcube.Dimensions)
                            if (thisdimension.Name == DimensionsList.SelectedItems[0].Text)
                                editingdimension = thisdimension;
                    Member editingmember = null;
                    if (editingdimension != null)
                        foreach (Member thismember in editingdimension.Members)
                            if (thismember.Name == MembersList.SelectedNode.Text)
                                editingmember = thismember;
                    if (editingcube != null && editingdimension != null && editingmember != null)
                    {
                        managemembersettings = new ManageMemberSettings(editingcube, editingdimension.ID,
                            editingmember.ID);
                        managemembersettings.Dock = DockStyle.Fill;
                        ActionPanel.Controls.Add(managemembersettings);
                        ActionStatusText = "Member Loaded.";
                        HideWaitingAnimation();
                        CustomiseScreen();
                    }
                    else
                    {
                        // The member wasn't found so show an error
                        ShowError("The selected member cannot be edited.");
                        HideWaitingAnimation();
                        ActionStatusText = "Member wasn't found.";
                    }
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving member settings failed.";
                }
            }
        }

        private void DatasourcesButton_Click(object sender, EventArgs e)
        {
            if (CubesList.SelectedItems.Count > 0 && CubesList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving cube datasources from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.Datasources;

                    // Find our cube
                    Cube editingcube = null;
                    foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                        if (thiscube.Name == CubesList.SelectedItems[0].Text)
                            editingcube = _connection.Administration.GetCube(thiscube.ID);
                    if (editingcube != null)
                    {
                        cubedatasourcesettings = new CubeDatasourceSettings(editingcube);
                        cubedatasourcesettings.Dock = DockStyle.Fill;
                        cubedatasourcesettings.CurrentDatasourcesGrid.SelectionChanged += new EventHandler(CurrentDatasourcesGrid_SelectionChanged);
                        ActionPanel.Controls.Add(cubedatasourcesettings);
                        ActionStatusText = "Datasources Loaded.";
                        HideWaitingAnimation();
                        CustomiseScreen();
                    }
                    else
                    {
                        // The cube wasn't found so show an error
                        ShowError("The selected cubes datasources cannot be edited.");
                        HideWaitingAnimation();
                        ActionStatusText = "Cube wasn't found.";
                    }
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving cube datasources failed.";
                }
            }
        }

        private void MemberGeneratorsButton_Click(object sender, EventArgs e)
        {
            if (DimensionsList.SelectedItems.Count > 0 && DimensionsList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving dimension member generators from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.DimensionMemberGenerators;

                    // Find our cube
                    Cube editingcube = null;
                    foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                        if (thiscube.Name == CubesList.SelectedItems[0].Text)
                            editingcube = _connection.Administration.GetCube(thiscube.ID);

                    // Find our dimension
                    Dimension editingdimension = null;
                    if (editingcube != null)
                        foreach (Dimension thisdimension in editingcube.Dimensions)
                            if (thisdimension.Name == DimensionsList.SelectedItems[0].Text)
                                editingdimension = thisdimension;

                    if (editingcube != null && editingdimension != null)
                    {
                        dimensionmembergenerators = new DimensionMemberGenerators(editingcube, editingdimension.ID);
                        dimensionmembergenerators.Dock = DockStyle.Fill;
                        ActionPanel.Controls.Add(dimensionmembergenerators);
                        ActionStatusText = "Dimension Datasource Settings Loaded.";
                        HideWaitingAnimation();
                        CustomiseScreen();
                    }
                    else
                    {
                        // The cube wasn't found so show an error
                        ShowError("The selected dimensions datasource cannot be edited.");
                        HideWaitingAnimation();
                        ActionStatusText = "Cube wasn't found.";
                    }
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving dimension member generators failed.";
                }
            }
        }

        private void CubeRolesButton_Click(object sender, EventArgs e)
        {
            if (CubesList.SelectedItems.Count > 0 && CubesList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving cube roles from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.ManageAccessRoles;

                    // Find our cube
                    Cube editingcube = null;
                    foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                        if (thiscube.Name == CubesList.SelectedItems[0].Text)
                            editingcube = _connection.Administration.GetCube(thiscube.ID);
                    if (editingcube != null)
                    {
                        manageaccessroles = new ManageAccessRoles(editingcube, new Guid(),
                            _connection.Administration.ReadRoles());
                        manageaccessroles.Dock = DockStyle.Fill;
                        ActionPanel.Controls.Add(manageaccessroles);
                        ActionStatusText = "Roles Loaded.";
                        HideWaitingAnimation();
                        CustomiseScreen();
                    }
                    else
                    {
                        // The cube wasn't found so show an error
                        ShowError("The selected cubes roles cannot be edited.");
                        HideWaitingAnimation();
                        ActionStatusText = "Cube wasn't found.";
                    }
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving cube roles failed.";
                }
            }
        }

        private void MemberRolesButton_Click(object sender, EventArgs e)
        {
            if (MembersList.SelectedNode != null && MembersList.SelectedNode.Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving member roles from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.ManageAccessRoles;

                    // Find our cube, dimension and member
                    Cube editingcube = null;
                    foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                        if (thiscube.Name == CubesList.SelectedItems[0].Text)
                            editingcube = _connection.Administration.GetCube(thiscube.ID);
                    Dimension editingdimension = null;
                    if (editingcube != null)
                        foreach (Dimension thisdimension in editingcube.Dimensions)
                            if (thisdimension.Name == DimensionsList.SelectedItems[0].Text)
                                editingdimension = thisdimension;
                    Member editingmember = null;
                    if (editingdimension != null)
                        foreach (Member thismember in editingdimension.Members)
                            if (thismember.Name == MembersList.SelectedNode.Text)
                                editingmember = thismember;
                    if (editingcube != null && editingdimension != null && editingmember != null)
                    {
                        manageaccessroles = new ManageAccessRoles(editingcube, editingmember.ID,
                            _connection.Administration.ReadRoles());
                        manageaccessroles.Dock = DockStyle.Fill;
                        ActionPanel.Controls.Add(manageaccessroles);
                        ActionStatusText = "Member Roles Loaded.";
                        HideWaitingAnimation();
                        CustomiseScreen();
                    }
                    else
                    {
                        // The member wasn't found so show an error
                        ShowError("The selected members roles cannot be edited.");
                        HideWaitingAnimation();
                        ActionStatusText = "Member wasn't found.";
                    }
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving member roles failed.";
                }
            }
        }

        private void MeasuresButton_Click(object sender, EventArgs e)
        {
            if (CubesList.SelectedItems.Count > 0 && CubesList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving cube measures from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.Measures;

                    // Find our cube
                    Cube editingcube = null;
                    foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                        if (thiscube.Name == CubesList.SelectedItems[0].Text)
                            editingcube = _connection.Administration.GetCube(thiscube.ID);
                    if (editingcube != null)
                    {
                        cubemeasures = new CubeMeasures(editingcube);
                        cubemeasures.Dock = DockStyle.Fill;
                        cubemeasures.CurrentMeasuresGrid.SelectionChanged += new EventHandler(CurrentMeasuresGrid_SelectionChanged);
                        ActionPanel.Controls.Add(cubemeasures);
                        ActionStatusText = "Measures Loaded.";
                        HideWaitingAnimation();
                        CustomiseScreen();
                    }
                    else
                    {
                        // The cube wasn't found so show an error
                        ShowError("The selected cubes measures cannot be edited.");
                        HideWaitingAnimation();
                        ActionStatusText = "Cube wasn't found.";
                    }
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving cube measures failed.";
                }
            }
        }

        private void MeasureLinksButton_Click(object sender, EventArgs e)
        {
            if (DimensionsList.SelectedItems.Count > 0 && DimensionsList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving dimension measure links from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.MeasureLinks;

                    // Find our cube
                    Cube editingcube = null;
                    foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                        if (thiscube.Name == CubesList.SelectedItems[0].Text)
                            editingcube = _connection.Administration.GetCube(thiscube.ID);

                    // Find our dimension
                    Dimension editingdimension = null;
                    if (editingcube != null)
                        foreach (Dimension thisdimension in editingcube.Dimensions)
                            if (thisdimension.Name == DimensionsList.SelectedItems[0].Text)
                                editingdimension = thisdimension;

                    if (editingcube != null && editingdimension != null)
                    {
                        dimensionmeasurelinks = new DimensionMeasureLinks(editingcube, editingdimension.ID);
                        dimensionmeasurelinks.Dock = DockStyle.Fill;
                        ActionPanel.Controls.Add(dimensionmeasurelinks);
                        ActionStatusText = "Dimension Measure Links Loaded.";
                        HideWaitingAnimation();
                        CustomiseScreen();
                    }
                    else
                    {
                        // The cube wasn't found so show an error
                        ShowError("The selected dimensions measure links cannot be edited.");
                        HideWaitingAnimation();
                        ActionStatusText = "Cube wasn't found.";
                    }
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving dimension measure links failed.";
                }
            }
        }

        private void LevelsButton_Click(object sender, EventArgs e)
        {
            if (DimensionsList.SelectedItems.Count > 0 && DimensionsList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving dimension levels from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.Levels;

                    // Find our cube
                    Cube editingcube = null;
                    foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                        if (thiscube.Name == CubesList.SelectedItems[0].Text)
                            editingcube = _connection.Administration.GetCube(thiscube.ID);

                    // Find our dimension
                    Dimension editingdimension = null;
                    if (editingcube != null)
                        foreach (Dimension thisdimension in editingcube.Dimensions)
                            if (thisdimension.Name == DimensionsList.SelectedItems[0].Text)
                                editingdimension = thisdimension;

                    if (editingcube != null && editingdimension != null)
                    {
                        managelevelsettings = new ManageLevelSettings(editingcube, editingdimension.ID);
                        managelevelsettings.Dock = DockStyle.Fill;
                        ActionPanel.Controls.Add(managelevelsettings);
                        ActionStatusText = "Dimension Levels Loaded.";
                        HideWaitingAnimation();
                        CustomiseScreen();
                    }
                    else
                    {
                        // The cube wasn't found so show an error
                        ShowError("The selected dimensions levels cannot be edited.");
                        HideWaitingAnimation();
                        ActionStatusText = "Cube wasn't found.";
                    }
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving dimension levels failed.";
                }
            }
        }

        private void MemberFormulaButton_Click(object sender, EventArgs e)
        {
            if (MembersList.SelectedNode != null && MembersList.SelectedNode.Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving member formula configuration from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.MemberFormula;

                    // Find our cube, dimension and member
                    Cube editingcube = null;
                    foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                        if (thiscube.Name == CubesList.SelectedItems[0].Text)
                            editingcube = _connection.Administration.GetCube(thiscube.ID);
                    Dimension editingdimension = null;
                    if (editingcube != null)
                        foreach (Dimension thisdimension in editingcube.Dimensions)
                            if (thisdimension.Name == DimensionsList.SelectedItems[0].Text)
                                editingdimension = thisdimension;
                    Member editingmember = null;
                    if (editingdimension != null)
                        foreach (Member thismember in editingdimension.Members)
                            if (thismember.Name == MembersList.SelectedNode.Text)
                                editingmember = thismember;
                    if (editingcube != null && editingdimension != null && editingmember != null)
                    {
                        memberformulasettings = new MemberFormulaSettings(editingcube, editingdimension.ID,
                            editingmember.ID);
                        memberformulasettings.Dock = DockStyle.Fill;
                        ActionPanel.Controls.Add(memberformulasettings);
                        ActionStatusText = "Member Formula Configuration Loaded.";
                        HideWaitingAnimation();
                        CustomiseScreen();
                    }
                    else
                    {
                        // The member wasn't found so show an error
                        ShowError("The selected members formula configuration cannot be edited.");
                        HideWaitingAnimation();
                        ActionStatusText = "Member wasn't found.";
                    }
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving member formula configuration failed.";
                }
            }
        }

        private void RemoveCurrentQueryButton_Click(object sender, EventArgs e)
        {
            if (OpenAnalysis.SelectedItems.Count > 0 && OpenAnalysis.SelectedItems[0].Text != String.Empty)
            {                
                // Try and find the analysis window in our controls collection and close it if it exists
                for (int i = ActionPanel.Controls.Count - 1; i > -1; i--)                
                    if (ActionPanel.Controls[i] is DataSetQueryAnalysis)
                        if (((DataSetQueryAnalysis)ActionPanel.Controls[i]).QueryID == OpenAnalysis.SelectedItems[0].Text)
                            ActionPanel.Controls.RemoveAt(i);

                // Remove the selected item
                OpenAnalysis.Items.RemoveAt(OpenAnalysis.SelectedIndices[0]);

                // Customise the screen
                CurrentAction = CurrentActions.None;
                CustomiseScreen();
            }
        }

        private void AddNewQueryButton_Click(object sender, EventArgs e)
        {
            CheckConnection(_connectedserver, false);

            try
            {
                CurrentAction = CurrentActions.Analysis;
                datasetqueryanalysis = new DataSetQueryAnalysis(_connection, Guid.NewGuid().ToString());
                datasetqueryanalysis.Dock = DockStyle.Fill;
                ActionPanel.Controls.Add(datasetqueryanalysis);
                
                OpenAnalysis.Items.Add(new ListViewItem(datasetqueryanalysis.QueryID, 2));
                CustomiseScreen();

            }
            catch (CubeConnectionException c1)
            {
                ShowError("Could not open the specified query: " + c1.ToString());
                HideWaitingAnimation();
                ActionStatusText = "Retrieving query failed.";
            }
        }

        private void AttributesButton_Click(object sender, EventArgs e)
        {
            if (DimensionsList.SelectedItems.Count > 0 && DimensionsList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving dimension attributes from server...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.Attributes;

                    // Find our cube
                    Cube editingcube = null;
                    foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                        if (thiscube.Name == CubesList.SelectedItems[0].Text)
                            editingcube = _connection.Administration.GetCube(thiscube.ID);

                    // Find our dimension
                    Dimension editingdimension = null;
                    if (editingcube != null)
                        foreach (Dimension thisdimension in editingcube.Dimensions)
                            if (thisdimension.Name == DimensionsList.SelectedItems[0].Text)
                                editingdimension = thisdimension;

                    if (editingcube != null && editingdimension != null)
                    {
                        dimensionattributes = new DimensionAttributes(editingcube, editingdimension.ID);
                        dimensionattributes.Dock = DockStyle.Fill;
                        ActionPanel.Controls.Add(dimensionattributes);
                        ActionStatusText = "Dimension Attributes Loaded.";
                        HideWaitingAnimation();
                        CustomiseScreen();
                    }
                    else
                    {
                        // The cube wasn't found so show an error
                        ShowError("The selected dimensions attributes cannot be edited.");
                        HideWaitingAnimation();
                        ActionStatusText = "Cube wasn't found.";
                    }
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving dimension attributes failed.";
                }
            }
        }

        private void MemberAttributeValuesButton_Click(object sender, EventArgs e)
        {
            // First try and connect to the server
            ActionStatusText = "Retrieving member attribute values from server...";
            ShowStatusText();
            CustomiseScreen();
            ShowWaitingAnimation();
            this.Refresh();

            CheckConnection(_connectedserver, false);

            try
            {
                CurrentAction = CurrentActions.MemberAttributes;

                // Find our cube, dimension and member
                Cube editingcube = null;
                foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                    if (thiscube.Name == CubesList.SelectedItems[0].Text)
                        editingcube = _connection.Administration.GetCube(thiscube.ID);
                Dimension editingdimension = null;
                if (editingcube != null)
                    foreach (Dimension thisdimension in editingcube.Dimensions)
                        if (thisdimension.Name == DimensionsList.SelectedItems[0].Text)
                            editingdimension = thisdimension;
                Member editingmember = null;
                if (editingdimension != null)
                    foreach (Member thismember in editingdimension.Members)
                        if (thismember.Name == MembersList.SelectedNode.Text)
                            editingmember = thismember;
                if (editingcube != null && editingdimension != null && editingmember != null)
                {
                    dimensionattributevalues = new DimensionAttributeValues(editingcube, editingdimension.ID,
                        editingmember.ID);
                    dimensionattributevalues.Dock = DockStyle.Fill;
                    ActionPanel.Controls.Add(dimensionattributevalues);
                    ActionStatusText = "Member Attribute Values Loaded.";
                    HideWaitingAnimation();
                    CustomiseScreen();
                }
                else
                {
                    // The member wasn't found so show an error
                    ShowError("The selected members attribute values cannot be edited.");
                    HideWaitingAnimation();
                    ActionStatusText = "Member wasn't found.";
                }
            }
            catch (CubeConnectionException c1)
            {
                ShowError("Could not connect to the specified server. " + c1.ToString());
                HideWaitingAnimation();
                ActionStatusText = "Retrieving member attribute values failed.";
            }
        }

        private void BeginConsolidationButton_Click(object sender, EventArgs e)
        {
            if (CubesList.SelectedItems.Count > 0 && CubesList.SelectedItems[0].Text != String.Empty)
            {
                // First try and connect to the server
                ActionStatusText = "Retrieving cube details for consolidation...";
                ShowStatusText();
                CustomiseScreen();
                ShowWaitingAnimation();
                this.Refresh();

                CheckConnection(_connectedserver, false);

                try
                {
                    CurrentAction = CurrentActions.DoConsolidation;

                    // Find our cube
                    Cube editingcube = null;
                    foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                        if (thiscube.Name == CubesList.SelectedItems[0].Text)
                            editingcube = _connection.Administration.GetCube(thiscube.ID);
                    if (editingcube != null)
                    {
                        doconsolidation = new DoConsolidation(editingcube, _connection);
                        doconsolidation.Dock = DockStyle.Fill;
                        ActionPanel.Controls.Add(doconsolidation);
                        ActionStatusText = "Consolidation Prepared.";
                        HideWaitingAnimation();
                        CustomiseScreen();
                    }
                    else
                    {
                        // The cube wasn't found so show an error
                        ShowError("The selected cube cannot be consolidated.");
                        HideWaitingAnimation();
                        ActionStatusText = "Cube wasn't found.";
                    }
                }
                catch (CubeConnectionException c1)
                {
                    ShowError("Could not connect to the specified server. " + c1.ToString());
                    HideWaitingAnimation();
                    ActionStatusText = "Retrieving cube for consolidation failed.";
                }
            }
        }

        #endregion

        #region Save Button

        private void SaveButton_Click(object sender, EventArgs e)
        {
            switch (_currentaction)
            {
                case CurrentActions.AddNewServer:
                    // Check whether the given server already exists
                    foreach (ListViewItem thisitem in ServerList.Items)
                        if (thisitem.Text.ToUpper().Equals(newservercontrol.Server.ToUpper()))
                        {
                            ShowError("This server already exists in the connection list on the left.");
                            return;
                        }

                    // Update the screen with the progress
                    ActionStatusText = "Attempting to connect to new server...";
                    ShowStatusText();
                    ShowWaitingAnimation();
                    CustomiseScreen();
                    this.Refresh();                    

                    // Attempt to connect to the new server
                    try
                    {
                        // Attempt to connect to the new server
                        _connectedserver = new ConnectedServer();
                        _connectedserver.EndPoint.Server = newservercontrol.Server;
                        _connectedserver.EndPoint.Port = newservercontrol.PortNumber;
                        _connectedserver.EndPoint.EndpointType = newservercontrol.ProtocolType;
                        _connectedserver.TimeOut = newservercontrol.TimeOut;
                        _connectedserver.EndPoint.IntegratedAuthentication = newservercontrol.IntegratedSecurity;
                        _connectedserver.Username = newservercontrol.Username;
                        _connectedserver.Password = newservercontrol.Password;
                        CheckConnection(_connectedserver, true);

                        CancelButton_Click(this, new EventArgs());
                        ActionStatusText = "New Connection opened to server " + newservercontrol.Server;

                        // Now that the server has been connected, add it to the list on the left
                        ListViewItem newserver = new ListViewItem(newservercontrol.Server, 5);
                        if (!ServerList.Items.Contains(newserver))
                            ServerList.Items.Add(newserver);
                        DesignerSettingsManager.Settings.ConnectedServers.Add(_connectedserver);
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not connect to the new server. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Last connection failed.";
                    }
                    break;
                case CurrentActions.ConfigureServer:
                    ActionStatusText = "Attempting to save general settings for this server...";
                    ShowStatusText();
                    ShowWaitingAnimation();
                    CustomiseScreen();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the settings to the server
                    try
                    {
                        // Save the modified settings to the server
                        ServerSettings currentsettings = _connection.Administration.Settings;
                        if (currentsettings.EndPoints.Count < 1)
                            currentsettings.EndPoints.Add(new ServiceEndPoint());
                        currentsettings.StructuresTableName = serversettings.CubeStructuresTable;
                        currentsettings.DataTablePrefix = serversettings.CubeDataTablePrefix;
                        currentsettings.LogLevel = serversettings.CubeLoggingLevel;
                        currentsettings.SMTPServer = serversettings.SMTPServerLocation;
                        currentsettings.SMTPPort = serversettings.SMTPPortNumber;
                        currentsettings.SMTPSSL = serversettings.UseServerSSL;
                        currentsettings.SMTPAuthenticated = serversettings.UseServerAuthentication;
                        currentsettings.SMTPUser = serversettings.SMTPServerUsername;
                        currentsettings.SMTPPassword = serversettings.SMTPServerPassword;
                        currentsettings.EndPoints[0].Server = serversettings.ServerName;
                        currentsettings.EndPoints[0].EndpointType = serversettings.ServerEndPointType;
                        currentsettings.EndPoints[0].Port = serversettings.ServerPort;
                        currentsettings.MaxMemory = serversettings.MaxMemoryUsage;
                        currentsettings.AddInsDirectory = serversettings.CubeAddInsDirectory;
                        _connection.Administration.SaveSettings(currentsettings);
                        CancelButton_Click(this, new EventArgs());
                        ActionStatusText = "Settings saved. You will need to restart the Cubed Service on the server before any changes will take effect.";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the settings. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of settings failed.";
                    }
                    break;
                case CurrentActions.ManageSystemRoles:
                    ActionStatusText = "Attempting to save system roles for this server...";
                    ShowStatusText();
                    ShowWaitingAnimation();
                    CustomiseScreen();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the root users to the server
                    try
                    {
                        // Then save the roles back
                        _connection.Administration.SaveRoles(managesystemroles.Roles);

                        // Update the system status
                        CancelButton_Click(this, new EventArgs());
                        ActionStatusText = "System Roles saved.";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the system roles. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of system roles failed.";
                    }
                    break;
                case CurrentActions.ManageSystemUsers:
                    ActionStatusText = "Attempting to save system users for this server...";
                    ShowStatusText();
                    ShowWaitingAnimation();
                    CustomiseScreen();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the users to the server
                    try
                    {
                        // Then save the users back
                        _connection.Administration.SaveUsers(managesystemusers.Users);

                        // Now try and grab all users which are root users, and update the system settings
                        ServerSettings currentsettings = _connection.Administration.Settings;
                        currentsettings.RootUsers.Clear();
                        foreach (User thisuser in managesystemusers.Users.Where(testuser => testuser.IsAdmin))
                            currentsettings.RootUsers.Add(thisuser.DomainUserName.ToUpper());
                        _connection.Administration.SaveSettings(currentsettings);

                        // Update the system status
                        CancelButton_Click(this, new EventArgs());
                        ActionStatusText = "System Users saved.";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the system users. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of system users failed.";
                    }
                    break;
                case CurrentActions.StorageProviders:
                    ActionStatusText = "Attempting to save storage provider settings for this server...";
                    ShowStatusText();
                    ShowWaitingAnimation();
                    CustomiseScreen();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the settings to the server
                    try
                    {
                        // Save the modified settings to the server
                        ServerSettings currentsettings = _connection.Administration.Settings;
                        currentsettings.StorageProvider = storageprovidersettings.CurrentConnector;
                        currentsettings.StorageProvider.ConnectorConfiguration = storageprovidersettings.ConfigurationXML;
                        _connection.Administration.SaveSettings(currentsettings);
                        CancelButton_Click(this, new EventArgs());
                        ActionStatusText = "Storage Provider Settings saved.";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the settings. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of storage provider settings failed.";
                    }
                    break;
                case CurrentActions.Functions:
                    ActionStatusText = "Attempting to save available function settings for this server...";
                    ShowStatusText();
                    ShowWaitingAnimation();
                    CustomiseScreen();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the settings to the server
                    try
                    {
                        // Save the modified settings to the server
                        ServerSettings currentsettings = _connection.Administration.Settings;
                        currentsettings.Functions = functionsettings.CurrentFunctions;
                        _connection.Administration.SaveSettings(currentsettings);
                        CancelButton_Click(this, new EventArgs());
                        ActionStatusText = "Available Function Settings saved.";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the settings. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of available function settings failed.";
                    }
                    break;
                case CurrentActions.QueryParsers:
                    ActionStatusText = "Attempting to save available parser settings for this server...";
                    ShowStatusText();
                    ShowWaitingAnimation();
                    CustomiseScreen();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the settings to the server
                    try
                    {
                        // Save the modified settings to the server
                        ServerSettings currentsettings = _connection.Administration.Settings;
                        currentsettings.Parsers = parsersettings.CurrentParsers;
                        _connection.Administration.SaveSettings(currentsettings);
                        CancelButton_Click(this, new EventArgs());
                        ActionStatusText = "Available Parser Settings saved.";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the settings. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of available parser settings failed.";
                    }
                    break;
                case CurrentActions.Consolidations:
                    ActionStatusText = "Attempting to save consolidation settings for this server...";
                    ShowStatusText();
                    ShowWaitingAnimation();
                    CustomiseScreen();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the settings to the server
                    try
                    {
                        // Save the modified settings to the server
                        ServerSettings currentsettings = _connection.Administration.Settings;
                        _connection.Administration.SaveSettings(currentsettings);

                        // Now save any modified cubes
                        foreach (Guid thiscubeid in activeconsolidations.ModifiedCubeIDs)
                        {
                            Cube modifiedcube = _connection.Administration.GetCube(thiscubeid);
                            if (modifiedcube != null)
                            {
                                modifiedcube.ConsolidationInterval = activeconsolidations.Cubes[thiscubeid].ConsolidationInterval;
                                modifiedcube.PointConsolidation = activeconsolidations.Cubes[thiscubeid].PointConsolidation;
                                _connection.Administration.SetCube(modifiedcube);
                                _connection.Administration.SaveCube(modifiedcube.ID);
                            }
                        }

                        CancelButton_Click(this, new EventArgs());
                        ActionStatusText = "Available Parser Settings saved.";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the settings. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of available parser settings failed.";
                    }
                    break;
                case CurrentActions.NewCube:
                    ActionStatusText = "Attempting to add the new cube to the collection...";
                    ShowStatusText();
                    ShowWaitingAnimation();
                    CustomiseScreen();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the new cube to the server
                    try
                    {
                        // First check the cube doesn't already exist
                        foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                            if (thiscube.Name.ToUpper().Equals(addnewcube.NewCubeName.ToUpper()))
                            {
                                ShowError("A cube with this name already exists.");
                                ActionStatusText = "Cube not added.";
                                break;
                            }

                        // The cube doesn't exist, so create and add it
                        Cube newcube = new Cube(addnewcube.NewCubeName);
                        newcube.Description = addnewcube.NewCubeDescription;
                        _connection.Administration.SetCube(newcube);
                        _connection.Administration.SaveCube(newcube.ID);

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        ActionStatusText = "New Cube Added.";

                        // Now re-load the cubes list
                        CubesList.Items.Clear();
                        foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                            CubesList.Items.Add(new ListViewItem(thiscube.Name, 0));
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the new cube. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of new cube failed.";
                    }
                    break;
                case CurrentActions.NewDimension:
                    ActionStatusText = "Attempting to add the new dimension to the chosen cube...";
                    ShowStatusText();
                    ShowWaitingAnimation();
                    CustomiseScreen();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the new dimension to the server
                    try
                    {
                        // First grab the updated dimension from the dimension add control
                        if (!addnewdimension.AddDimensionToCube())
                        {
                            // The dimension already exists - so raise an error
                            ShowError("A dimension with this name already exists. Please select a new name.");
                            HideWaitingAnimation();
                            CustomiseScreen();
                            this.Refresh();
                            ActionStatusText = "Adding of new dimension failed.";
                            break;
                        }
                        Cube editedcube = addnewdimension.EditingCube;

                        // Try and publish the cube back to the server
                        _connection.Administration.SetCube(editedcube);
                        _connection.Administration.SaveCube(editedcube.ID);

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        ActionStatusText = "New Dimension Added.";

                        // Now re-load the dimensions list
                        DimensionsList.Items.Clear();
                        foreach (Dimension thisdimension in editedcube.Dimensions)
                            DimensionsList.Items.Add(new ListViewItem(thisdimension.Name, 6));
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the new dimension. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of the new dimension failed.";
                    }
                    break;
                case CurrentActions.NewMember:
                    ActionStatusText = "Attempting to add the new member to the chosen cube...";
                    ShowStatusText();
                    ShowWaitingAnimation();
                    CustomiseScreen();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the new member to the dimension
                    try
                    {
                        // First grab the updated member from the member add control
                        if (!addnewmember.AddMemberToCubeDimension())
                        {
                            // The member already exists - so raise an error
                            ShowError("A member with this name already exists. Please select a new name.");
                            HideWaitingAnimation();
                            CustomiseScreen();
                            this.Refresh();
                            ActionStatusText = "Adding of new member failed.";
                            break;
                        }
                        Cube editedcube = addnewmember.EditingCube;

                        // Try and publish the cube back to the server
                        _connection.Administration.SetCube(editedcube);
                        _connection.Administration.SaveCube(editedcube.ID);

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        ActionStatusText = "New Member Added.";

                        // Now re-load the members list
                        Cube newcube = _connection.Administration.GetCube(editedcube.ID);
                        MembersList.Nodes.Clear();
                        TreeNode rootnode = new TreeNode(newcube.Dimensions[addnewmember.EditingDimensionID].Name, 6, 6);
                        BuildMembersTree(ref rootnode, newcube.Dimensions[addnewmember.EditingDimensionID], null, true);
                        rootnode.Expand();
                        MembersList.Nodes.Add(rootnode);
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the new member. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of the new member failed.";
                    }
                    break;
                case CurrentActions.CubeSettings:
                    ActionStatusText = "Attempting to save cube settings...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the update cube back to the server
                    try
                    {
                        // First grab the updated member from the member add control
                        if (!managecubesettings.PrepareSave())
                        {
                            // The settings can't be saved
                            ShowError("The settings cannot be saved, please ensure you have filled out all neccessary fields.");
                            HideWaitingAnimation();
                            CustomiseScreen();
                            this.Refresh();
                            ActionStatusText = "Saving of cube settings failed.";
                            break;
                        }

                        Cube editedcube = managecubesettings.EditingCube;

                        // Try and publish the cube back to the server
                        _connection.Administration.SetCube(editedcube);
                        _connection.Administration.SaveCube(editedcube.ID);

                        // Now re-load the cube list
                        CubesList.Items.Clear();
                        foreach (Cube thiscube in _connection.Administration.CubeHeaders)
                            CubesList.Items.Add(new ListViewItem(thiscube.Name, 0));

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        ActionStatusText = "Cube Saved";
                        HideWaitingAnimation();
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the cube settings. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of cube settings failed.";
                    }
                    break;
                case CurrentActions.DimensionSettings:
                    ActionStatusText = "Attempting to save dimension settings...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the update cube back to the server
                    try
                    {
                        // First grab the updated dimension and cube from the settings
                        if (!managedimensionsettings.PrepareSave())
                        {
                            // The settings can't be saved
                            ShowError("The settings cannot be saved, please ensure you have filled out all neccessary fields.");
                            HideWaitingAnimation();
                            CustomiseScreen();
                            this.Refresh();
                            ActionStatusText = "Saving of dimension settings failed.";
                            break;
                        }

                        Cube editedcube = managedimensionsettings.EditingCube;

                        // Try and publish the cube back to the server
                        _connection.Administration.SetCube(editedcube);
                        _connection.Administration.SaveCube(editedcube.ID);

                        // Now re-load the dimensions list
                        DimensionsList.Items.Clear();
                        foreach (Dimension thisdimension in editedcube.Dimensions)
                            DimensionsList.Items.Add(new ListViewItem(thisdimension.Name, 6));

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        HideWaitingAnimation();
                        ActionStatusText = "Dimension Saved";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the dimension settings. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of dimension settings failed.";
                    }
                    break;
                case CurrentActions.MemberSettings:
                    ActionStatusText = "Attempting to save member settings...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the update cube back to the server
                    try
                    {
                        // First grab the updated member, dimension and cube from the settings
                        if (!managemembersettings.PrepareSave())
                        {
                            // The settings can't be saved
                            ShowError("The settings cannot be saved, please ensure you have filled out all neccessary fields.");
                            HideWaitingAnimation();
                            CustomiseScreen();
                            this.Refresh();
                            ActionStatusText = "Saving of member settings failed.";
                            break;
                        }

                        Cube editedcube = managemembersettings.EditingCube;

                        // Try and publish the cube back to the server
                        _connection.Administration.SetCube(editedcube);
                        _connection.Administration.SaveCube(editedcube.ID);

                        // Now re-load the members list
                        Cube newcube = _connection.Administration.GetCube(editedcube.ID);
                        MembersList.Nodes.Clear();
                        TreeNode rootnode = new TreeNode(newcube.Dimensions[managemembersettings.DimensionID].Name, 6, 6);
                        BuildMembersTree(ref rootnode, newcube.Dimensions[managemembersettings.DimensionID], null, true);
                        rootnode.Expand();
                        MembersList.Nodes.Add(rootnode);

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        HideWaitingAnimation();
                        ActionStatusText = "Member Saved";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the member settings. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of member settings failed.";
                    }
                    break;
                case CurrentActions.Datasources:
                    ActionStatusText = "Attempting to save cube datasources...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the update cube back to the server
                    try
                    {
                        Cube editedcube = cubedatasourcesettings.EditingCube;

                        // Try and publish the cube back to the server
                        _connection.Administration.SetCube(editedcube);
                        _connection.Administration.SaveCube(editedcube.ID);

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        ActionStatusText = "Cube Datasources Saved";
                        HideWaitingAnimation();
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the cube datasources. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of cube datasources failed.";
                    }
                    break;
                case CurrentActions.DimensionMemberGenerators:
                    ActionStatusText = "Attempting to save dimension member generators...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the update cube back to the server
                    try
                    {
                        Cube editedcube = dimensionmembergenerators.EditingCube;

                        // Try and publish the cube back to the server
                        _connection.Administration.SetCube(editedcube);
                        _connection.Administration.SaveCube(editedcube.ID);

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        HideWaitingAnimation();
                        ActionStatusText = "Dimension Member Generators Saved";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the dimension member generators. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of dimension member generators failed.";
                    }
                    break;
                case CurrentActions.ManageAccessRoles:
                    ActionStatusText = "Attempting to save roles...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the update cube back to the server
                    try
                    {
                        // Grab the cube that was being edited
                        Cube editedcube = manageaccessroles.EditingCube;

                        // Check whether a member or cube was being edited
                        if (manageaccessroles.EditingMember != null)
                        {
                            // A member was being edited, so we need to replace the member instance
                            // in the given cube with the edited member
                            editedcube.Dimensions[manageaccessroles.EditingMember.Dimension].Members.Remove(manageaccessroles.EditingMember.ID);
                            editedcube.Dimensions[manageaccessroles.EditingMember.Dimension].Members.Add(manageaccessroles.EditingMember);
                        }                        

                        // Try and publish the cube back to the server
                        _connection.Administration.SetCube(editedcube);
                        _connection.Administration.SaveCube(editedcube.ID);

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        HideWaitingAnimation();
                        ActionStatusText = "Roles Saved";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the roles. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of roles failed.";
                    }
                    break;
                case CurrentActions.Measures:
                    ActionStatusText = "Attempting to save cube measures...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the update cube back to the server
                    try
                    {
                        Cube editedcube = cubemeasures.EditingCube;

                        // Try and publish the cube back to the server
                        _connection.Administration.SetCube(editedcube);
                        _connection.Administration.SaveCube(editedcube.ID);

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        ActionStatusText = "Cube Measures Saved";
                        HideWaitingAnimation();
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the cube measures. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of cube measures failed.";
                    }
                    break;
                case CurrentActions.MeasureLinks:
                    ActionStatusText = "Attempting to save dimension measure links...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the update cube back to the server
                    try
                    {
                        Cube editedcube = dimensionmeasurelinks.EditingCube;

                        // Call save changes to ensure the latest changes have been saved
                        dimensionmeasurelinks.SaveChanges();

                        // Try and publish the cube back to the server
                        _connection.Administration.SetCube(editedcube);
                        _connection.Administration.SaveCube(editedcube.ID);

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        HideWaitingAnimation();
                        ActionStatusText = "Dimension Measure Links Saved";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the dimension measure links. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of dimension measure links failed.";
                    }
                    break;
                case CurrentActions.Levels:
                    ActionStatusText = "Attempting to save dimension levels...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the update cube back to the server
                    try
                    {
                        Cube editedcube = managelevelsettings.EditingCube;
                                             
                        // Try and publish the cube back to the server
                        _connection.Administration.SetCube(editedcube);
                        _connection.Administration.SaveCube(editedcube.ID);

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        HideWaitingAnimation();
                        ActionStatusText = "Dimension Levels Saved";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the dimension levels. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of dimension levels failed.";
                    }
                    break;
                case CurrentActions.MemberFormula:
                    ActionStatusText = "Attempting to save member formula...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the update cube back to the server
                    try
                    {
                        // Save our changes
                        memberformulasettings.SaveChanges();

                        // Grab our changed cube
                        Cube editedcube = memberformulasettings.EditingCube;

                        // Try and publish the cube back to the server
                        _connection.Administration.SetCube(editedcube);
                        _connection.Administration.SaveCube(editedcube.ID);

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        HideWaitingAnimation();
                        ActionStatusText = "Member Formula Configuration Saved";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the member formula configuration. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of member formula configuration failed.";
                    }
                    break;
                case CurrentActions.Analysis:
                    datasetqueryanalysis.SaveToFile();
                    break;
                case CurrentActions.Attributes:
                    ActionStatusText = "Attempting to save dimension attributes...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the update cube back to the server
                    try
                    {
                        Cube editedcube = dimensionattributes.EditingCube;

                        // Try and publish the cube back to the server
                        _connection.Administration.SetCube(editedcube);
                        _connection.Administration.SaveCube(editedcube.ID);

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        HideWaitingAnimation();
                        ActionStatusText = "Dimension Attributes Saved";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the dimension attributes. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of dimension levels attributes.";
                    }
                    break;
                case CurrentActions.MemberAttributes:
                    ActionStatusText = "Attempting to save member attribute values...";
                    ShowStatusText();
                    CustomiseScreen();
                    ShowWaitingAnimation();
                    this.Refresh();

                    // Attempt to connect to the server
                    CheckConnection(_connectedserver, false);

                    // Attempt to save the update cube back to the server
                    try
                    {
                        // Grab our changed cube
                        Cube editedcube = dimensionattributevalues.EditingCube;

                        // Try and publish the cube back to the server
                        _connection.Administration.SetCube(editedcube);
                        _connection.Administration.SaveCube(editedcube.ID);

                        // Report back success
                        CancelButton_Click(this, new EventArgs());
                        HideWaitingAnimation();
                        ActionStatusText = "Member Attribute Values Saved";
                    }
                    catch (CubeConnectionException c1)
                    {
                        ShowError("Could not save the member attribute values. " + c1.ToString());
                        HideWaitingAnimation();
                        ActionStatusText = "Saving of member attribute values failed.";
                    }
                    break;
            }
        }

        #endregion

        #region Child Event Handling

        private void ServerList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty)
            {
                this.Text = _titletext + " @ " + ServerList.SelectedItems[0].Text;
                for (int i = 0; i < ServerActionsStrip.Items.Count; i++)
                {
                    ServerActionsStrip.Items[i].Enabled = true;
                }
                foreach (ConnectedServer thisserver in DesignerSettingsManager.Settings.ConnectedServers)
                    if (thisserver.EndPoint.Server == ServerList.SelectedItems[0].Text)
                        _connectedserver = thisserver;
            }
            else
            {
                this.Text = _titletext;
                for (int i = 0; i < ServerActionsStrip.Items.Count; i++)
                {
                    ServerActionsStrip.Items[i].Enabled = false;
                }
                AddNewServerButton.Enabled = true;
            }
        }

        void CurrentRootUsers_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        void CurrentFunctionsGrid_SelectionChanged(object sender, EventArgs e)
        {
            RemoveButton.Visible = functionsettings.FunctionSelected();
        }

        void CurrentParsersGrid_SelectionChanged(object sender, EventArgs e)
        {
            RemoveButton.Visible = parsersettings.ParserSelected();
        }

        private void CubedDesigner_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Save our designer settings for next time
            DesignerSettings currentsettings = DesignerSettingsManager.Settings;
            if (this.WindowState == FormWindowState.Maximized)
                currentsettings.WindowMaximised = true;
            else
                currentsettings.WindowMaximised = false;
            DesignerSettingsManager.Settings = currentsettings;
        }

        private void CubesList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (CubesList.SelectedItems.Count > 0 && CubesList.SelectedItems[0].Text != String.Empty)
            {
                this.Text = _titletext + " @ " + ServerList.SelectedItems[0].Text + " : " +
                    CubesList.SelectedItems[0].Text;
                for (int i = 0; i < CubeActionsStrip.Items.Count; i++)
                {
                    CubeActionsStrip.Items[i].Enabled = true;
                }
            }
            else
            {
                if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty)
                    this.Text = _titletext + " @ " + ServerList.SelectedItems[0].Text;
                else
                    this.Text = _titletext;

                for (int i = 0; i < CubeActionsStrip.Items.Count; i++)
                {
                    CubeActionsStrip.Items[i].Enabled = false;
                }
                NewCubeButton.Enabled = true;
                ImportCubeButton.Enabled = true;
            }
        }

        private void DimensionsList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (DimensionsList.SelectedItems.Count > 0 && DimensionsList.SelectedItems[0].Text != String.Empty)
            {
                this.Text = _titletext + " @ " + ServerList.SelectedItems[0].Text + " : " +
                    CubesList.SelectedItems[0].Text + " > " + DimensionsList.SelectedItems[0].Text;
                for (int i = 0; i < DimensionActionsStrip.Items.Count; i++)
                {
                    DimensionActionsStrip.Items[i].Enabled = true;
                }
            }
            else
            {
                if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty
                    && CubesList.SelectedItems.Count > 0 && CubesList.SelectedItems[0].Text != String.Empty)
                    this.Text = _titletext + " @ " + ServerList.SelectedItems[0].Text + " : " +
                    CubesList.SelectedItems[0].Text;
                else if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty)
                    this.Text = _titletext + " @ " + ServerList.SelectedItems[0].Text;
                else
                    this.Text = _titletext;

                for (int i = 0; i < DimensionActionsStrip.Items.Count; i++)
                {
                    DimensionActionsStrip.Items[i].Enabled = false;
                }
                AddDimensionButton.Enabled = true;
            }
        }

        private void MembersList_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (MembersList.SelectedNode != null && MembersList.SelectedNode.Text != String.Empty
                && MembersList.SelectedNode.Text != DimensionsList.SelectedItems[0].Text)
            {
                this.Text = _titletext + " @ " + ServerList.SelectedItems[0].Text + " : " +
                    CubesList.SelectedItems[0].Text + " > " + DimensionsList.SelectedItems[0].Text + " >> " +
                    MembersList.SelectedNode.Text;
                for (int i = 0; i < MembersActionsStrip.Items.Count; i++)
                {
                    MembersActionsStrip.Items[i].Enabled = true;
                }
            }
            else
            {
                if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty
                    && CubesList.SelectedItems.Count > 0 && CubesList.SelectedItems[0].Text != String.Empty &&
                    DimensionsList.SelectedItems.Count > 0 && DimensionsList.SelectedItems[0].Text != String.Empty)
                    this.Text = _titletext + " @ " + ServerList.SelectedItems[0].Text + " : " +
                    CubesList.SelectedItems[0].Text + " > " + DimensionsList.SelectedItems[0].Text;
                else if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty
                    && CubesList.SelectedItems.Count > 0 && CubesList.SelectedItems[0].Text != String.Empty)
                    this.Text = _titletext + " @ " + ServerList.SelectedItems[0].Text + " : " +
                    CubesList.SelectedItems[0].Text;
                else if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty)
                    this.Text = _titletext + " @ " + ServerList.SelectedItems[0].Text;
                else
                    this.Text = _titletext;

                for (int i = 0; i < MembersActionsStrip.Items.Count; i++)
                {
                    MembersActionsStrip.Items[i].Enabled = false;
                }
                AddMemberButton.Enabled = true;
            }
        }

        void CurrentDatasourcesGrid_SelectionChanged(object sender, EventArgs e)
        {
            RemoveButton.Visible = cubedatasourcesettings.DatasourceSelected();
        }

        void CurrentMeasuresGrid_SelectionChanged(object sender, EventArgs e)
        {
            RemoveButton.Visible = cubemeasures.MeasureSelected();
        }

        #endregion

        #region Generic Buttons

        private void CancelButton_Click(object sender, EventArgs e)
        {
            // Remove the top control
            ActionPanel.Controls.RemoveAt(ActionPanel.Controls.Count - 1);
            CurrentAction = CurrentActions.None;

            // Then discover the next control type and set the current action if one exists
            if (ActionPanel.Controls.Count > 0)
            {
                if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is AddNewServer)
                    CurrentAction = CurrentActions.AddNewServer;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is ManageServerSettings)
                    CurrentAction = CurrentActions.ConfigureServer;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is ManageSystemRoles)
                    CurrentAction = CurrentActions.ManageSystemRoles;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is StorageProviderSettings)
                    CurrentAction = CurrentActions.StorageProviders;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is FunctionSettings)
                    CurrentAction = CurrentActions.Functions;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is ParserSettings)
                    CurrentAction = CurrentActions.QueryParsers;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is ActiveConsolidations)
                    CurrentAction = CurrentActions.Consolidations;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is AddNewCube)
                    CurrentAction = CurrentActions.NewCube;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is AddNewDimension)
                    CurrentAction = CurrentActions.NewDimension;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is AddNewMember)
                    CurrentAction = CurrentActions.NewMember;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is ManageCubeSettings)
                    CurrentAction = CurrentActions.CubeSettings;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is ManageDimensionSettings)
                    CurrentAction = CurrentActions.DimensionSettings;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is ManageMemberSettings)
                    CurrentAction = CurrentActions.MemberSettings;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is CubeDatasourceSettings)
                    CurrentAction = CurrentActions.Datasources;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is DimensionMemberGenerators)
                    CurrentAction = CurrentActions.DimensionMemberGenerators;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is ManageAccessRoles)
                    CurrentAction = CurrentActions.ManageAccessRoles;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is CubeMeasures)
                    CurrentAction = CurrentActions.Measures;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is DimensionMeasureLinks)
                    CurrentAction = CurrentActions.MeasureLinks;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is ManageLevelSettings)
                    CurrentAction = CurrentActions.Levels;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is MemberFormulaSettings)
                    CurrentAction = CurrentActions.MemberFormula;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is DataSetQueryAnalysis)
                    CurrentAction = CurrentActions.Analysis;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is DimensionAttributes)
                    CurrentAction = CurrentActions.Attributes;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is DimensionAttributeValues)
                    CurrentAction = CurrentActions.MemberAttributes;
                else if (ActionPanel.Controls[ActionPanel.Controls.Count - 1] is DoConsolidation)
                    CurrentAction = CurrentActions.DoConsolidation;
            }
            
            // Finally customise our screen based on the selection
            CustomiseScreen();
        }

        private void AddButton_Click(object sender, EventArgs e)
        {
            switch (CurrentAction)
            {
                case CurrentActions.Functions:
                    // Add a new function
                    functionsettings.AddCurrentFunction();
                    RemoveButton.Visible = functionsettings.FunctionSelected();
                    break;
                case CurrentActions.QueryParsers:
                    // Add a new parser
                    parsersettings.AddCurrentParser();
                    RemoveButton.Visible = parsersettings.ParserSelected();
                    break;
                case CurrentActions.Datasources:
                    // Add a new data source
                    cubedatasourcesettings.AddCurrentDatasource();
                    RemoveButton.Visible = cubedatasourcesettings.DatasourceSelected();
                    break;
                case CurrentActions.Measures:
                    // Add a new default measure
                    cubemeasures.AddNewEmptyMeasure();
                    RemoveButton.Visible = cubemeasures.MeasureSelected();
                    break;                    
                case CurrentActions.DimensionMemberGenerators:
                    // Add a default new member generator
                    dimensionmembergenerators.AddNewEmptyGenerator();
                    RemoveButton.Visible = dimensionmembergenerators.GeneratorSelected();
                    break;
                case CurrentActions.Attributes:
                    // Add a default new attribute
                    dimensionattributes.AddNewAttribute();
                    RemoveButton.Visible = dimensionattributes.AttributeSelected();
                    break;
            }
        }

        private void RemoveButton_Click(object sender, EventArgs e)
        {
            switch (CurrentAction)
            {
                case CurrentActions.Functions:
                    // Remove the current row if seleced
                    RemoveButton.Visible = functionsettings.RemoveSelectedFunction();
                    break;
                case CurrentActions.QueryParsers:
                    // Remove the current row if seleced
                    RemoveButton.Visible = parsersettings.RemoveSelectedParser();
                    break;
                case CurrentActions.Datasources:
                    // Remove the current row if selected
                    RemoveButton.Visible = cubedatasourcesettings.RemoveSelectedDatasource();
                    break;
                case CurrentActions.Measures:
                    // Remove the current row if selected
                    RemoveButton.Visible = cubemeasures.RemoveSelectedMeasure();
                    break;
                case CurrentActions.MemberFormula:
                    // Remove the formula if the user has chosen to
                    if (memberformulasettings.ClearEvaluator())
                    {
                        // The user has chosen to remove the settings, so clear the screen
                        SaveButton_Click(this, new EventArgs());                          
                    }
                    break;
                case CurrentActions.DimensionMemberGenerators:
                    // Remove the current row if selected
                    RemoveButton.Visible = dimensionmembergenerators.RemoveSelectedGenerator();
                    break;
                case CurrentActions.Attributes:
                    // Remove the current attribute if selected
                    RemoveButton.Visible = dimensionattributes.RemoveSelectedAttribute();
                    break;
            }
        }

        private void RefreshButton_Click(object sender, EventArgs e)
        {
            switch (CurrentAction)
            {
                case CurrentActions.Consolidations:
                    if (MessageBox.Show("Refreshing the consolidation list will cause you to lose any changes to "
                        + "automatic consolidation intervals which you haven't saved. Continue?", "Confirm Refresh",
                        MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                    {
                        // Cancel and close our consolidation window
                        CancelButton_Click(this, new EventArgs());

                        // Then Re-load the Consolidations screen
                        ConsolidationsButton_Click(this, new EventArgs());
                    }
                    break;
            }
        }

        private void OpenButton_Click(object sender, EventArgs e)
        {
            switch (CurrentAction)
            {
                case CurrentActions.Analysis:
                    datasetqueryanalysis.LoadFromFile();
                    break;
            }
        }

        /// <summary>
        /// Exits the application
        /// </summary>
        private void RibbonExitButton_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        #endregion
           
        #region Navigation and Tab Panel

        /// <summary>
        /// When the current tab page changes, attempt to update the navigation panel
        /// </summary>
        private void CubedRibbon_CurrentTabPageChanged(object sender, EventArgs e)
        {
            // Depending on the tab which was selected, change the navigation panel, using the 
            // navigation panel selected event to ensure that only valid changes are made
            switch (CubedRibbon.CurrentTabPage.Name.ToUpper())
            {
                case "SERVERTABPAGE":
                    Ascend.Windows.Forms.NavigationPaneEventArgs sae = new Ascend.Windows.Forms.NavigationPaneEventArgs(null,
                        -1, "SERVERS", Ascend.Windows.Forms.NavigationPaneAction.Selected);
                    NavigationPanel_Selected(null, sae);
                    break;
                case "CUBESTABPAGE":
                    Ascend.Windows.Forms.NavigationPaneEventArgs cae = new Ascend.Windows.Forms.NavigationPaneEventArgs(null,
                        -1, "CUBESPANEL", Ascend.Windows.Forms.NavigationPaneAction.Selected);
                    NavigationPanel_Selected(null, cae);
                    break;
                case "ANALYSISTABPAGE":
                    Ascend.Windows.Forms.NavigationPaneEventArgs aae = new Ascend.Windows.Forms.NavigationPaneEventArgs(null,
                        -1, "ANALYSISPANEL", Ascend.Windows.Forms.NavigationPaneAction.Selected);
                    NavigationPanel_Selected(null, aae);
                    break;
                case "DIMENSIONSTABPAGE":
                    Ascend.Windows.Forms.NavigationPaneEventArgs dae = new Ascend.Windows.Forms.NavigationPaneEventArgs(null,
                        -1, "DIMENSIONSPANEL", Ascend.Windows.Forms.NavigationPaneAction.Selected);
                    NavigationPanel_Selected(null, dae);
                    break;
                case "MEMBERSTABPAGE":
                    Ascend.Windows.Forms.NavigationPaneEventArgs mae = new Ascend.Windows.Forms.NavigationPaneEventArgs(null,
                        -1, "MEMBERSPANEL", Ascend.Windows.Forms.NavigationPaneAction.Selected);
                    NavigationPanel_Selected(null, mae);
                    break;
            }
        }

        /// <summary>
        /// When the navigation panel changes, check to ensure the selection is valid, initialise
        /// the screen, and update the tab panel
        /// </summary>
        private void NavigationPanel_Selected(object sender, Ascend.Windows.Forms.NavigationPaneEventArgs e)
        {
            switch (e.NavigationPanePageKey.ToUpper())
            {
                case "SERVERS":
                    // Change the ribbon tab group to the servers menu
                    CubedRibbon.CurrentTabPage = ServerTabPage;
                    break;
                case "CUBESPANEL":
                    // The user has selected the cubes panel, so check to ensure that a valid server has been
                    // selected
                    if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty)
                    {
                        // A valid server has been chosen, so initialise the cube screen
                        // Update the screen with the progress
                        ActionStatusText = "Attempting to retrieve cubes from server...";
                        ShowStatusText();
                        ShowWaitingAnimation();
                        CustomiseScreen();
                        this.Refresh();

                        // Attempt to connect to the new server
                        CheckConnection(_connectedserver, false);

                        // Attempt to connect to the new server
                        try
                        {
                            // Try and grab the list of cubes on the server
                            GenericKeyCollection<Guid, Cube> cubes = new GenericKeyCollection<Guid, Cube>();
                            cubes = _connection.Administration.CubeHeaders;

                            // Update the list on the screen with the cubes
                            CubesList.Items.Clear();

                            foreach (Cube thiscube in cubes)
                            {
                                ListViewItem newcube = new ListViewItem(thiscube.Name, 0);
                                CubesList.Items.Add(newcube);
                            }

                            // Update the screen
                            ActionStatusText = "All current cubes have been retrieved.";

                            // Change the ribbon tab group to the cubes menu
                            CubedRibbon.CurrentTabPage = CubesTabPage;
                        }
                        catch (CubeConnectionException c1)
                        {
                            NavigationPanel.SelectNavigationPage(0);
                            CubedRibbon.CurrentTabPage = ServerTabPage;
                            ShowError("Could not retrieve cubes from server. " + c1.ToString());
                            HideWaitingAnimation();
                            ActionStatusText = "Cubes could not be retrieved.";
                        }

                        // Update the selected item screen
                        CubesList_SelectedIndexChanged(this, new EventArgs());
                    }
                    else
                    {
                        // No server is available, so show a warning and return to the server panel
                        NavigationPanel.SelectNavigationPage(0);
                        CubedRibbon.CurrentTabPage = ServerTabPage;
                        ShowWarning("No server has been selected. Please connect to and select a server "
                            + "before trying to edit cubes.");
                    }
                    break;
                case "ANALYSISPANEL":
                    // The user has selected the analysis panel, so check to ensure that a valid server
                    // has been selected
                    if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty)
                    {                        
                        // Change the ribbon tab group to the analysis menu
                        CubedRibbon.CurrentTabPage = AnalysisTabPage;
                    }
                    else
                    {
                        // No server is available, so show a warning and return to the server panel
                        NavigationPanel.SelectNavigationPage(0);
                        CubedRibbon.CurrentTabPage = ServerTabPage;
                        ShowWarning("No server has been selected. Please connect to a server "
                            + "before trying to perform analysis.");
                    }
                    break;
                case "DIMENSIONSPANEL":
                    // The user has selected the dimensions panel, so check to ensure that a valid server and cube have been
                    // selected
                    if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty
                        && CubesList.SelectedItems.Count > 0 && CubesList.SelectedItems[0].Text != String.Empty)
                    {
                        // A valid server and cube have been chosen, so initialise the dimensions screen
                        ActionStatusText = "Attempting to retrieve dimensions for cube...";
                        ShowStatusText();
                        ShowWaitingAnimation();
                        CustomiseScreen();
                        this.Refresh();

                        // Attempt to connect to the new server
                        CheckConnection(_connectedserver, false);

                        // Attempt to connect to the new server
                        try
                        {
                            // Try and grab the list of cubes on the server
                            GenericKeyCollection<Guid, Cube> cubes = new GenericKeyCollection<Guid, Cube>();
                            cubes = _connection.Administration.CubeHeaders;

                            // Try and find our given cube
                            Cube dimensioncube = null;
                            foreach (Cube thiscube in cubes)
                                if (thiscube.Name.ToUpper().Equals(CubesList.SelectedItems[0].Text.ToUpper()))
                                    dimensioncube = _connection.Administration.GetCube(thiscube.ID);

                            // Now we have found our screen, pull back the dimensions
                            DimensionsList.Items.Clear();
                            foreach (Dimension thisdimension in dimensioncube.Dimensions)
                            {
                                ListViewItem newdimension = new ListViewItem(thisdimension.Name, 6);
                                DimensionsList.Items.Add(newdimension);
                            }

                            // Update the screen
                            ActionStatusText = "All current dimensions have been retrieved.";
                            HideWaitingAnimation();

                            // Update the selected item screen
                            DimensionsList_SelectedIndexChanged(this, new EventArgs());

                            // Change the ribbon tab group to the dimensions menu
                            CubedRibbon.CurrentTabPage = DimensionsTabPage;
                        }
                        catch (CubeConnectionException c1)
                        {
                            NavigationPanel.SelectNavigationPage(0);
                            CubedRibbon.CurrentTabPage = ServerTabPage;
                            ShowError("Could not retrieve dimensions from cube. " + c1.ToString());
                            HideWaitingAnimation();
                            ActionStatusText = "Dimensions could not be retrieved.";
                        }
                    }
                    else
                    {
                        // No server is available, so show a warning and return to the server panel
                        NavigationPanel.SelectNavigationPage(0);
                        CubedRibbon.CurrentTabPage = ServerTabPage;
                        ShowWarning("No cube has been selected. Please connect to a server and select a cube "
                            + "before trying to manage dimensions.");
                    }
                    break;
                case "MEMBERSPANEL":
                    // The user has selected the members panel, so check to ensure that a valid dimension has been
                    // selected
                    if (ServerList.SelectedItems.Count > 0 && ServerList.SelectedItems[0].Text != String.Empty
                        && CubesList.SelectedItems.Count > 0 && CubesList.SelectedItems[0].Text != String.Empty
                        && DimensionsList.SelectedItems.Count > 0 && DimensionsList.SelectedItems[0].Text !=
                        String.Empty)
                    {
                        // A valid dimension has been chosen, so initialise the members screen
                        ActionStatusText = "Attempting to retrieve members for dimension...";
                        ShowStatusText();
                        ShowWaitingAnimation();
                        CustomiseScreen();
                        this.Refresh();

                        // Attempt to connect to the new server
                        CheckConnection(_connectedserver, false);

                        // Attempt to connect to the new server
                        try
                        {
                            // Try and grab the list of cubes on the server
                            GenericKeyCollection<Guid, Cube> cubes = new GenericKeyCollection<Guid, Cube>();
                            cubes = _connection.Administration.CubeHeaders;

                            // Try and find our given cube
                            Cube dimensioncube = null;
                            foreach (Cube thiscube in cubes)
                                if (thiscube.Name.ToUpper().Equals(CubesList.SelectedItems[0].Text.ToUpper()))
                                    dimensioncube = _connection.Administration.GetCube(thiscube.ID);

                            // Try and find our dimension in the cube
                            Dimension memberdimension = null;
                            foreach (Dimension thisdimension in dimensioncube.Dimensions)
                                if (thisdimension.Name.ToUpper().Equals(DimensionsList.SelectedItems[0].Text.ToUpper()))
                                    memberdimension = thisdimension;

                            // Now we have found our dimension, pull back the members
                            MembersList.Nodes.Clear();
                            TreeNode rootnode = new TreeNode(memberdimension.Name, 6, 6);
                            BuildMembersTree(ref rootnode, memberdimension, null, true);
                            rootnode.Expand();
                            MembersList.Nodes.Add(rootnode);

                            // Update the screen
                            ActionStatusText = "All current members have been retrieved.";

                            // Update the selected item screen
                            MembersList_AfterSelect(this, new TreeViewEventArgs(new TreeNode()));

                            // Update the screen
                            ActionStatusText = "All current members have been retrieved.";
                            HideWaitingAnimation();

                            // Change the ribbon tab group to the members menu
                            CubedRibbon.CurrentTabPage = MembersTabPage;
                        }
                        catch (CubeConnectionException c1)
                        {
                            NavigationPanel.SelectNavigationPage(0);
                            CubedRibbon.CurrentTabPage = ServerTabPage;
                            ShowError("Could not retrieve members from dimension. " + c1.ToString());
                            HideWaitingAnimation();
                            ActionStatusText = "Members could not be retrieved.";
                        }
                    }
                    else
                    {
                        // No dimension is available, so show a warning and return to the server panel
                        NavigationPanel.SelectNavigationPage(0);
                        CubedRibbon.CurrentTabPage = ServerTabPage;
                        ShowWarning("No dimension has been selected. Please connect to a server and select a cube "
                             + "and dimension before trying to manage dimensions.");
                    }
                    break;
            }
        }

        #endregion

        #region Internal Functions

        /// <summary>
        /// This adds all members of a dimension to a root treenode
        /// </summary>
        /// <param name="currentnode">The node to add child nodes to</param>
        /// <param name="membersdimension">The dimension (only used on the first pass)</param>
        /// <param name="currentmember">The current member (only used on subsequent passes)</param>
        private void BuildMembersTree(ref TreeNode currentnode, Dimension membersdimension,
            Member currentmember, bool firstpass)
        {
            // Determine if this is the top level or a child level
            if (firstpass)
            {
                // We are at the top level, so add all the dimension members with no parent
                foreach (Member thismember in membersdimension.Members)
                    if (thismember.Parent == new Guid())
                    {
                        // This is a top level member, so add it to the node
                        TreeNode newnode = new TreeNode(thismember.Name, 7, 7);
                        currentnode.Nodes.Add(newnode);

                        // Now recursively call this function for all children
                        BuildMembersTree(ref newnode, membersdimension, thismember, false);
                    }
            }
            else if (currentmember != null)
            {
                // We are at a member level, so add all the members children
                foreach (Member thismember in membersdimension.Members.Where(testmember => currentmember.Children.Contains(testmember.ID)))
                {
                    TreeNode newnode = new TreeNode(thismember.Name, 7, 7);
                    currentnode.Nodes.Add(newnode);

                    // Now recursively call this function for all children
                    BuildMembersTree(ref newnode, membersdimension, thismember, false);
                }
            }
        }

        /// <summary>
        /// This gathers all descendants of a member, as well as the member itself, into a complete, flattened
        /// list
        /// </summary>
        /// <param name="completelist">The list to add all descendants, and the member itself, to</param>
        /// <param name="currentmember">The current member being processed</param>
        /// <param name="memberdimension">The dimension of the current member</param>
        private void GatherAllDescendants(ref List<Member> completelist, Member currentmember,
            Dimension memberdimension)
        {
            // Add the current member to the complete list
            completelist.Add(currentmember);

            // Now cycle through and add all children to the complete list
            foreach (Member thischild in memberdimension.Members.Where(testmember => currentmember.Children.Contains(testmember.ID)))
                GatherAllDescendants(ref completelist, thischild, memberdimension);
        }

        #endregion                                                        

        #region Ribbon Menu Button States

        /// <summary>
        /// The ribbon menu buttons enabled status is linked to the visiblity of it's 
        /// parent button
        /// </summary>
        private void SaveButton_VisibleChanged(object sender, EventArgs e)
        {
            RibbonSaveButton.Enabled = SaveButton.Visible;
        }

        /// <summary>
        /// The ribbon menu buttons enabled status is linked to the visiblity of it's 
        /// parent button
        /// </summary>
        private void OpenButton_VisibleChanged(object sender, EventArgs e)
        {
            RibbonOpenButton.Enabled = OpenButton.Visible;
        }

        /// <summary>
        /// The ribbon menu buttons enabled status is linked to the visiblity of it's 
        /// parent button
        /// </summary>
        private void CancelButton_VisibleChanged(object sender, EventArgs e)
        {
            RibbonCancelButton.Enabled = CancelButton.Visible;
        }

        /// <summary>
        /// The ribbon menu buttons enabled status is linked to the visiblity of it's 
        /// parent button
        /// </summary>
        private void AddButton_VisibleChanged(object sender, EventArgs e)
        {
            RibbonAddButton.Enabled = AddButton.Visible;
        }

        /// <summary>
        /// The ribbon menu buttons enabled status is linked to the visiblity of it's 
        /// parent button
        /// </summary>
        private void RemoveButton_VisibleChanged(object sender, EventArgs e)
        {
            RibbonRemoveButton.Enabled = RemoveButton.Visible;
        }

        /// <summary>
        /// The ribbon menu buttons enabled status is linked to the visiblity of it's 
        /// parent button
        /// </summary>
        private void RefreshButton_VisibleChanged(object sender, EventArgs e)
        {
            RibbonRefreshButton.Enabled = RefreshButton.Visible;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void AddNewServerButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonAddServerButton.Enabled = AddNewServerButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void RemoveServerButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonRemoveServerButton.Enabled = RemoveServerButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void GeneralServerSettingsButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonServerSettingsButton.Enabled = GeneralServerSettingsButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void RolesButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonSecurityRolesButton.Enabled = RolesButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void StorageProviderButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonStorageSettingsButton.Enabled = StorageProviderButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void FunctionsButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonFunctionsButton.Enabled = FunctionsButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void ParsersButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonQueryParsersButton.Enabled = ParsersButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void ConsolidationsButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonConsolidationsButton.Enabled = ConsolidationsButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void NewCubeButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonNewCubeButton.Enabled = NewCubeButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void DeleteCubeButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonRemoveCubeButton.Enabled = DeleteCubeButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void ImportCubeButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonImportCubeButton.Enabled = ImportCubeButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void ExportCubeButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonExportCubeButton.Enabled = ExportCubeButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void CubeSettings_EnabledChanged(object sender, EventArgs e)
        {
            RibbonCubeSettingsButton.Enabled = CubeSettings.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void DatasourcesButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonDatasourcesButton.Enabled = DatasourcesButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void CubeRolesButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonCubeRolesButton.Enabled = CubeRolesButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void UsersButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonUsersButton.Enabled = UsersButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void MeasuresButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonCubeMeasuresButton.Enabled = MeasuresButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void BeginConsolidationButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonConsolidateCubeButton.Enabled = BeginConsolidationButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void AddDimensionButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonAddDimensionButton.Enabled = AddDimensionButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void RemoveDimensionsButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonRemoveDimensionButton.Enabled = RemoveDimensionsButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void DimensionSettingsButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonDimensionSettingsButton.Enabled = DimensionSettingsButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void MemberGeneratorsButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonMemberGeneratorsButton.Enabled = MemberGeneratorsButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void MeasureLinksButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonMeasureLinksButton.Enabled = MeasureLinksButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void LevelsButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonLevelsButton.Enabled = LevelsButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void AttributesButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonAttributesButton.Enabled = AttributesButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void KPIsButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonKPIsButton.Enabled = KPIsButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void AddMemberButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonAddMemberButton.Enabled = AddMemberButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void DeleteMemberButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonRemoveMemberButton.Enabled = DeleteMemberButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void MemberSettingsButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonMemberSettingsButton.Enabled = MemberSettingsButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void MemberRolesButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonMemberRolesButton.Enabled = MemberRolesButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void MemberAttributeValuesButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonMemberAttributeValuesButton.Enabled = MemberAttributeValuesButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void MemberFormulaButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonMemberFormulasButton.Enabled = MemberFormulaButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void AddNewQueryButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonAddQueryButton.Enabled = AddNewQueryButton.Enabled;
        }

        /// <summary>
        /// The ribbon button's enabled state changes with the main navigation panel button
        /// </summary>
        private void RemoveCurrentQueryButton_EnabledChanged(object sender, EventArgs e)
        {
            RibbonRemoveQueryButton.Enabled = RemoveCurrentQueryButton.Enabled;
        }

        #endregion            
    }
}