/*
 * This source file (DesignerSettingsManager.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures;
using System.Data;
using System.IO;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.AddIns.DatabaseConnectors;
using Bamboo.Cubed.Utilities;

namespace Bamboo.Cubed.Designer
{
    /// <summary>
    /// This class provides support for reading and writing the Cubed Designer settings file
    /// </summary>
    public static class DesignerSettingsManager
    {
        private static DesignerSettings _settings;
        /// <summary>
        /// This reads and returns the current designer settings, or updates the current settings on disk
        /// </summary>
        [XmlElement("Settings")]
        public static DesignerSettings Settings
        {
            get 
            { 
                if (_settings == null)
                    ReadSettings(Information.MyDirectory + "Designer.xml");
                return _settings;
            }
            set 
            {
                _settings = value;
                SaveSettings(Information.MyDirectory + "Designer.xml"); 
            } 
        }

        /// <summary>
        /// Saves the Cubed Designer settings to disk
        /// </summary>
        /// <param name="filename">The filename to save the settings to</param>
        /// <returns>True if successful</returns>
        public static bool SaveSettings(string filename)
        {   
            // Save our settings to disk
            try
            {
                // First encrypt any passwords stored
                foreach (ConnectedServer thisserver in _settings.ConnectedServers)
                    if (thisserver.Username != null || thisserver.Password != null)
                    {
                        thisserver.Username = EncryptionManager.EncryptString(thisserver.Username);
                        thisserver.Password = EncryptionManager.EncryptString(thisserver.Password);
                    }

                // Serialise to disk
                XMLManager<DesignerSettings>.SerialiseToFile(filename, _settings);

                // Then decrypt the passwords again before returning
                foreach (ConnectedServer thisserver in _settings.ConnectedServers)
                    if (thisserver.Username != null || thisserver.Password != null)
                    {
                        thisserver.Username = EncryptionManager.DecryptString(thisserver.Username);
                        thisserver.Password = EncryptionManager.DecryptString(thisserver.Password);
                    }

                // Return true to indicate success
                return true;
            }
            catch
            {
                return false;
            }
        }

        /// <summary>
        /// Reads the Cubed Designer Settings from a file
        /// </summary>
        /// <param name="filename">The filename to read from</param>
        public static void ReadSettings(string filename)
        {
            _settings = new DesignerSettings();

            try
            {
                // Try and deserialise our settings from disk
                _settings = XMLManager<DesignerSettings>.DeserialiseFromFile(filename);

                // Then decrypt any passwords stored
                foreach (ConnectedServer thisserver in _settings.ConnectedServers)
                    if (thisserver.Username != null || thisserver.Password != null)
                    {
                        thisserver.Username = EncryptionManager.DecryptString(thisserver.Username);
                        thisserver.Password = EncryptionManager.DecryptString(thisserver.Password);
                    }
            }
            catch
            {
                // Even though an error occured writing the settings, return a new default settings
                _settings = new DesignerSettings();

                // Attempt to save the new settings
                SaveSettings(filename);
            }
        }        
    }
}
