/*
 * This source file (DimensionAttributeValues.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;

namespace Bamboo.Cubed.Designer
{
    /// <summary>
    /// This control allows for the setting of values of attributes for a member
    /// </summary>
    public partial class DimensionAttributeValues : UserControl
    {
        /// <summary>
        /// This initialises a new instance of the Attribute values editor
        /// </summary>     
        /// <param name="editingcube">The cube being edited</param>
        /// <param name="dimensionid">The unique identifier of the dimension being edited</param>
        /// <param name="memberid">The unique identifier of the member being edited</param>
        public DimensionAttributeValues(Cube editingcube, Guid dimensionid, Guid memberid)
        {
            InitializeComponent();
            _editingcube = editingcube;
            _dimensionid = dimensionid;
            _memberid = memberid;
            HeaderLabel.Text = "Manage attribute values for: " + _editingcube.Dimensions[dimensionid].Members[_memberid].Name;
            
            // Initialise our screen
            AttributeValuesPanel.Dock = DockStyle.Fill;
            AttributeValuesPanel.AutoSize = true;
            AttributeValuesPanel.AutoSizeMode = AutoSizeMode.GrowAndShrink;
            AttributeValuesPanel.Visible = false;
            AttributesGrid.Visible = true;
            BindGrids();
        }        

        private Cube _editingcube;
        /// <summary>
        /// The cube being edited
        /// </summary>
        public Cube EditingCube
        {
            get { return _editingcube; }
            set { _editingcube = value; }
        }

        private Guid _dimensionid;
        /// <summary>
        /// This is the unique identifier of the dimension being edited
        /// </summary>
        public Guid DimensionID
        {
            get { return _dimensionid; }
            set { _dimensionid = value; }
        }

        private Guid _memberid;
        /// <summary>
        /// This is the unique identifier of the member being edited
        /// </summary>
        public Guid MemberID
        {
            get { return _memberid; }
            set { _memberid = value; }
        }

        /// <summary>
        /// The unique identifier of the attribute being edited
        /// </summary>
        private Guid _editingattributeid;

        private void BindGrids()
        {
            try
            {
                AttributesGrid.AutoGenerateColumns = false;
                AttributesGrid.DataSource = null;
                AttributesGrid.DataSource = CurrentAttributes();
            }
            catch
            { }
        }

        /// <summary>
        /// This returns a data set with the attributes
        /// </summary>
        /// <returns>The current attributes</returns>
        private DataTable CurrentAttributes()
        {
            DataTable currentattributes = new DataTable();
            currentattributes.Columns.Add("AttributeID");
            currentattributes.Columns.Add("AttributeName");            
            currentattributes.Columns.Add("AttributeValues");
            currentattributes.Columns.Add("EditAttributeValues");

            // First go through the dimension for this member and grab all attributes and add them if they
            // don't already exist
            foreach (Bamboo.Cubed.Structures.Attribute thisattribute in _editingcube.Dimensions[_dimensionid].Attributes)
            {
                bool alreadyexists = false;
                foreach (AttributeValue thisvalue in _editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues)
                    if (thisvalue.Attribute == thisattribute.ID)
                        alreadyexists = true;
                if (!alreadyexists)
                {
                    AttributeValue newvalue = new AttributeValue(thisattribute, new List<string>());
                    _editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues.Add(newvalue);
                }
            }

            // Now go through the attribute values and remove any of which the attributes have been deleted
            List<Guid> removedattributes = new List<Guid>();
            foreach (AttributeValue thisvalue in _editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues)
            {
                bool stillexists = false;
                foreach (Bamboo.Cubed.Structures.Attribute thisattribute in _editingcube.Dimensions[_dimensionid].Attributes)
                    if (thisattribute.ID == thisvalue.Attribute)
                        stillexists = true;
                if (!stillexists)
                {
                    // The attribute has been removed from the dimension, so mark the attribute value for removal
                    removedattributes.Add(thisvalue.ID);
                }
            }
            foreach (Guid removedattributeid in removedattributes)
                _editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues.Remove(removedattributeid);

            // Add all attribute values
            foreach (AttributeValue thisattributevalue in _editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues)
            {
                DataRow newattributerow = currentattributes.NewRow();
                newattributerow["AttributeID"] = thisattributevalue.ID;
                newattributerow["AttributeName"] = _editingcube.Dimensions[_dimensionid].Attributes[thisattributevalue.Attribute].Name;
                string allvalues = String.Empty;
                foreach (string thisvalue in thisattributevalue.Values)
                    allvalues += thisvalue + ", ";
                allvalues = allvalues.TrimEnd(' ');
                allvalues = allvalues.TrimEnd(',');
                newattributerow["AttributeValues"] = allvalues;
                newattributerow["EditAttributeValues"] = "Edit Attribute Values...";
                currentattributes.Rows.Add(newattributerow);
            }

            return currentattributes;
        }

        private void CurrentAttributesGrid_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.ColumnIndex > -1 && e.RowIndex > -1)
            {
                // Check to see if the user has clicked on the "edit attribute values" column
                if (AttributesGrid.Columns[e.ColumnIndex].Name == "EditAttributeValues")
                {
                    // Show the editing attribute values panel
                    AttributeValuesPanel.Visible = true;
                    AttributesGrid.Visible = false;
                    _editingattributeid = new Guid(AttributesGrid.Rows[e.RowIndex].Cells["AttributeID"].Value.ToString());

                    // Determine the type of value being edited and show the appropriate editor
                    AttributeValueTypes valuetype = _editingcube.Dimensions[_dimensionid].Attributes[_editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues[_editingattributeid].Attribute].ValueType;
                    string[] values = _editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues[_editingattributeid].Values.ToArray();
                    switch (valuetype)
                    {
                        case AttributeValueTypes.Boolean:
                            // Show the boolean editing panel and set the value of the control
                            BooleanAttributeValuePanel.Visible = true;
                            bool booleanattributevalue = false;
                            if (values.Length > 0)
                                Boolean.TryParse(values[0], out booleanattributevalue);
                            TrueAttributeValue.Checked = booleanattributevalue;
                            FalseAttributeValue.Checked = !booleanattributevalue;

                            // Hide all other editing panels
                            TextAttributeValuePanel.Visible = false;
                            ListAttributeValuesPanel.Visible = false;
                            NumberAttributeValuePanel.Visible = false;
                            DateTimeAttributeValuePanel.Visible = false;
                            MultiSelectListAttributeValuePanel.Visible = false;                            
                            break;
                        case AttributeValueTypes.DateTime:
                            // Show the DateTime editing panel and set the value of the control
                            DateTimeAttributeValuePanel.Visible = true;
                            DateTime datetimeattributevalue = DateTime.Now;
                            if (values.Length > 0)
                            DateTime.TryParse(values[0], out datetimeattributevalue);
                            DateTimeAttributeValue.Value = datetimeattributevalue;

                            // Hide all other editing panels
                            TextAttributeValuePanel.Visible = false;
                            ListAttributeValuesPanel.Visible = false;
                            NumberAttributeValuePanel.Visible = false;
                            BooleanAttributeValuePanel.Visible = false;
                            MultiSelectListAttributeValuePanel.Visible = false;    
                            break;
                        case AttributeValueTypes.List:
                            // Show the List editing panel, fill its items collection, 
                            // and set the value of the control
                            ListAttributeValuesPanel.Visible = true;
                            ListAttributeValue.Items.Clear();
                            foreach (string thisvalue in _editingcube.Dimensions[_dimensionid].Attributes[_editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues[_editingattributeid].Attribute].Values)
                                ListAttributeValue.Items.Add(thisvalue);
                            string selectedlistvalue = String.Empty;
                            if (values.Length > 0)
                                selectedlistvalue = values[0];
                            if (ListAttributeValue.Items.Contains(selectedlistvalue))
                                ListAttributeValue.Text = selectedlistvalue;
                            else if (ListAttributeValue.Items.Count > 0)
                                ListAttributeValue.SelectedIndex = 0;                            

                            // Hide all other editing panels
                            TextAttributeValuePanel.Visible = false;
                            DateTimeAttributeValuePanel.Visible = false;
                            NumberAttributeValuePanel.Visible = false;
                            BooleanAttributeValuePanel.Visible = false;
                            MultiSelectListAttributeValuePanel.Visible = false;    
                            break;
                        case AttributeValueTypes.MultiSelectList:
                            // Show the Multi-Select List editing panel, fill its items collection, 
                            // and set the value of the control
                            MultiSelectListAttributeValuePanel.Visible = true;
                            MultiSelectListAttributeValue.Items.Clear();
                            foreach (string thisvalue in _editingcube.Dimensions[_dimensionid].Attributes[_editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues[_editingattributeid].Attribute].Values)
                                MultiSelectListAttributeValue.Items.Add(thisvalue);
                            foreach (ListViewItem thisitem in MultiSelectListAttributeValue.Items)
                                thisitem.Checked = false;
                            foreach (string thisselectedvalue in values)
                                foreach (ListViewItem thisitem in MultiSelectListAttributeValue.Items)
                                    if (thisitem.Text.ToUpper().Equals(thisselectedvalue.ToUpper()))
                                        thisitem.Checked = true;

                            // Hide all other editing panels
                            TextAttributeValuePanel.Visible = false;
                            DateTimeAttributeValuePanel.Visible = false;
                            NumberAttributeValuePanel.Visible = false;
                            BooleanAttributeValuePanel.Visible = false;
                            ListAttributeValuesPanel.Visible = false;    
                            break;
                        case AttributeValueTypes.Number:
                            // Show the Number editing panel and set the value of the control
                            NumberAttributeValuePanel.Visible = true;
                            int intattributevalue = 0;
                            if (values.Length > 0)
                                Int32.TryParse(values[0], out intattributevalue);
                            NumberAttributeValue.Value = Convert.ToInt32(intattributevalue);

                            // Hide all other editing panels
                            TextAttributeValuePanel.Visible = false;
                            ListAttributeValuesPanel.Visible = false;
                            DateTimeAttributeValuePanel.Visible = false;
                            BooleanAttributeValuePanel.Visible = false;
                            MultiSelectListAttributeValuePanel.Visible = false;  
                            break;
                        case AttributeValueTypes.Text:
                            // Show the Text editing panel and set the value of the control
                            TextAttributeValuePanel.Visible = true;
                            if (values.Length > 0)
                                TextAttributeValue.Text = values[0];
                            else
                                TextAttributeValue.Text = String.Empty;

                            // Hide all other editing panels
                            NumberAttributeValuePanel.Visible = false;
                            ListAttributeValuesPanel.Visible = false;
                            DateTimeAttributeValuePanel.Visible = false;
                            BooleanAttributeValuePanel.Visible = false;
                            MultiSelectListAttributeValuePanel.Visible = false;  
                            break;
                    }
                }
            }
        }

        private void CancelAttributeValuesButton_Click(object sender, EventArgs e)
        {
            // Hide the attribute values panel
            AttributeValuesPanel.Visible = false;
            AttributesGrid.Visible = true;
        }

        private void SaveAttributeValuesButton_Click(object sender, EventArgs e)
        {
            // Hide the attribute values panel
            AttributeValuesPanel.Visible = false;
            AttributesGrid.Visible = true;

            // Save the attribute values to the selected attribute
            _editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues[_editingattributeid].Values.Clear();
            AttributeValueTypes valuetype = _editingcube.Dimensions[_dimensionid].Attributes[_editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues[_editingattributeid].Attribute].ValueType;

            switch (valuetype)
            {
                case AttributeValueTypes.Boolean:
                    if (TrueAttributeValue.Checked)
                        _editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues[_editingattributeid].Values.Add("true");
                    else
                        _editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues[_editingattributeid].Values.Add("false");
                    break;
                case AttributeValueTypes.DateTime:
                    _editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues[_editingattributeid].Values.Add(DateTimeAttributeValue.Value.ToString());
                    break;
                case AttributeValueTypes.List:
                    _editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues[_editingattributeid].Values.Add(ListAttributeValue.Text);
                    break;
                case AttributeValueTypes.MultiSelectList:
                    foreach (ListViewItem thisitem in MultiSelectListAttributeValue.Items)
                        if (thisitem.Checked)
                            _editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues[_editingattributeid].Values.Add(thisitem.Text);
                    break;
                case AttributeValueTypes.Number:
                    _editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues[_editingattributeid].Values.Add(Convert.ToInt32(NumberAttributeValue.Value).ToString());
                    break;
                case AttributeValueTypes.Text:
                    _editingcube.Dimensions[_dimensionid].Members[_memberid].AttributeValues[_editingattributeid].Values.Add(TextAttributeValue.Text);
                    break;
            }

            // Re-bind our grid
            BindGrids();
        }
    }
}
