/*
 * This source file (DimensionDatasourceSettings.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.AddIns;

namespace Bamboo.Cubed.Designer
{
    /// <summary>
    /// This control allows for configuring the datasource for a dimension
    /// </summary>
    public partial class DimensionMemberGenerators : UserControl
    {
        /// <summary>
        /// Whether the current control is loading or not
        /// </summary>
        private bool loading = false;

        private Cube _editingcube;
        /// <summary>
        /// The cube whose Dimension Member Generator's are being managed
        /// </summary>
        public Cube EditingCube
        {
            get { return _editingcube; }
            set { _editingcube = value; }
        }

        private Guid _dimensionid;
        /// <summary>
        /// The unique identifier of the Dimension whose Member Generator's are being managed
        /// </summary>
        public Guid DimensionID
        {
            get { return _dimensionid; }
            set { _dimensionid = value; }
        }

        /// <summary>
        /// The unique identifier of the generator currently being edited
        /// </summary>
        private Guid _editinggeneratorid;

        /// <summary>
        /// The unique identifier of the member table details being edited
        /// </summary>
        private Guid _editingtabledetailsid;

        /// <summary>
        /// This initialises a new instance of the Dimension Datasource Settings
        /// </summary>
        /// <param name="editingcube">The cube which is being edited</param>
        /// <param name="dimensionid">The unique identifier of the dimension being edited</param>
        public DimensionMemberGenerators(Cube editingcube, Guid dimensionid)
        {
            InitializeComponent();
            loading = true;
            _editingcube = editingcube;
            _dimensionid = dimensionid;
            HeaderLabel.Text = "Dimension Member Generators for: " + editingcube.Dimensions[dimensionid].Name;

            // Prepare our grid drop downs
            PrepareComboColumns();

            // Prepare our other drop downs
            UpdateMemberTables();

            // Update our grid data
            BindGrid();

            // Set controls to fill when they are in use
            RangesPanel.Dock = DockStyle.Fill;
            TablesPanel.Dock = DockStyle.Fill;
            _editinggeneratorid = new Guid();

            loading = false;
        }             

        /// <summary>
        /// This updates the visibility on the range panel corresponding to the selected range type
        /// </summary>
        private void UpdateRangePanel()
        {
            if (RangeType.Items.Count > 0 && RangeType.Text != null && RangeType.Text != String.Empty)
            {
                // First grab the range type
                RangeTypes currentrangetype = (RangeTypes)Enum.Parse(typeof(RangeTypes), RangeType.Text);
            
                // Then determine the type and what fields should be visible
                if (currentrangetype == RangeTypes.UniqueValues)
                { 
                    // Enable the unique value fields
                    MemberNameField.Enabled = true;
                    MemberDescriptionField.Enabled = true;

                    // Disable the date fields
                    DateRangeLowerLimit.Enabled = false;
                    DateRangeUpperLimit.Enabled = false;
                    YearStartMonth.Enabled = false;
                    DateFormatString.Enabled = false;

                    // Disable the number fields
                    NumberLowerRange.Enabled = false;
                    NumberUpperRange.Enabled = false;
                    NumberSeed.Enabled = false;
                    NumberFormatString.Enabled = false;

                    // Set default values
                    if (MemberNameField.Items.Count > 0)
                        MemberNameField.SelectedIndex = 0;
                    if (MemberDescriptionField.Items.Count > 0)
                        MemberDescriptionField.SelectedIndex = 0;
                    DateRangeLowerLimit.Value = DateTime.Now.AddYears(-3);
                    DateRangeUpperLimit.Value = DateTime.Now.AddYears(1);
                    YearStartMonth.Value = 7;
                    DateFormatString.Text = new RangeGenerator().DateFormatString;
                    NumberLowerRange.Value = 1;
                    NumberUpperRange.Value = 1000;
                    NumberSeed.Value = 1;
                    NumberFormatString.Text = new RangeGenerator().NumberFormatString;
                }
                else if (currentrangetype == RangeTypes.SeededNumber)
                {
                    // Disable the unique value fields
                    MemberNameField.Enabled = false;
                    MemberDescriptionField.Enabled = false;

                    // Disable the date fields
                    DateRangeLowerLimit.Enabled = false;
                    DateRangeUpperLimit.Enabled = false;
                    YearStartMonth.Enabled = false;
                    DateFormatString.Enabled = false;

                    // Enable the number fields
                    NumberLowerRange.Enabled = true;
                    NumberUpperRange.Enabled = true;
                    NumberSeed.Enabled = true;
                    NumberFormatString.Enabled = true;

                    // Set default values
                    if (MemberNameField.Items.Count > 0)
                        MemberNameField.SelectedIndex = 0;
                    if (MemberDescriptionField.Items.Count > 0)
                        MemberDescriptionField.SelectedIndex = 0;
                    DateRangeLowerLimit.Value = DateTime.Now.AddYears(-3);
                    DateRangeUpperLimit.Value = DateTime.Now.AddYears(1);
                    YearStartMonth.Value = 7;
                    DateFormatString.Text = new RangeGenerator().DateFormatString;
                    NumberLowerRange.Value = 1;
                    NumberUpperRange.Value = 1000;
                    NumberSeed.Value = 1;
                    NumberFormatString.Text = new RangeGenerator().NumberFormatString;
                }
                else
                {
                    // Disable the unique value fields
                    MemberNameField.Enabled = false;
                    MemberDescriptionField.Enabled = false;

                    // Enable the date fields
                    DateRangeLowerLimit.Enabled = true;
                    DateRangeUpperLimit.Enabled = true;
                    YearStartMonth.Enabled = true;
                    DateFormatString.Enabled = true;

                    // Disable the number fields
                    NumberLowerRange.Enabled = false;
                    NumberUpperRange.Enabled = false;
                    NumberSeed.Enabled = false;
                    NumberFormatString.Enabled = false;

                    // Set default values
                    if (MemberNameField.Items.Count > 0)
                        MemberNameField.SelectedIndex = 0;
                    if (MemberDescriptionField.Items.Count > 0)
                        MemberDescriptionField.SelectedIndex = 0;
                    DateRangeLowerLimit.Value = DateTime.Now.AddYears(-3);
                    DateRangeUpperLimit.Value = DateTime.Now.AddYears(1);
                    YearStartMonth.Value = 7;
                    DateFormatString.Text = new RangeGenerator().DateFormatString;
                    NumberLowerRange.Value = 1;
                    NumberUpperRange.Value = 1000;
                    NumberSeed.Value = 1;
                    NumberFormatString.Text = new RangeGenerator().NumberFormatString;
                }
            }
        }

        private DataTable CurrentGenerators()
        {
            DataTable currentgenerators = new DataTable();
            currentgenerators.Columns.Add("GeneratorID");
            currentgenerators.Columns.Add("GeneratorName");
            currentgenerators.Columns.Add("GeneratorSignage");
            currentgenerators.Columns.Add("GeneratorOverwrite");
            currentgenerators.Columns.Add("GeneratorAggregation");
            currentgenerators.Columns.Add("GeneratorConfiguration");

            foreach (MemberGenerator thisgenerator in _editingcube.Dimensions[_dimensionid].MemberGenerators)
            {
                DataRow newgenerator = currentgenerators.NewRow();
                newgenerator["GeneratorID"] = thisgenerator.ID;
                newgenerator["GeneratorName"] = thisgenerator.Name;
                newgenerator["GeneratorSignage"] = thisgenerator.Sign.ToString();
                newgenerator["GeneratorOverwrite"] = thisgenerator.ReplaceExisting;
                newgenerator["GeneratorAggregation"] = thisgenerator.Aggregation.ToString();
                newgenerator["GeneratorConfiguration"] = "Configure...";
                currentgenerators.Rows.Add(newgenerator);
            }

            return currentgenerators;
        }

        private DataTable CurrentGeneratorTables()
        {
            DataTable currentgeneratortables = new DataTable();
            currentgeneratortables.Columns.Add("TableName");
            currentgeneratortables.Columns.Add("TableID");
            currentgeneratortables.Columns.Add("PrimaryKeyField");
            currentgeneratortables.Columns.Add("NameField");
            currentgeneratortables.Columns.Add("DescriptionField");
            currentgeneratortables.Columns.Add("RelatedTable");
            currentgeneratortables.Columns.Add("RelatedKeyField");
            currentgeneratortables.Columns.Add("MemberParentTable");
            currentgeneratortables.Columns.Add("MemberFilters");

            foreach (MemberTableDetails thisgeneratortable in _editingcube.Dimensions[_dimensionid].MemberGenerators
                [_editinggeneratorid].MemberTables)
            {
                DataRow newgeneratortable = currentgeneratortables.NewRow();
                newgeneratortable["TableName"] = thisgeneratortable.Table;
                newgeneratortable["TableID"] = thisgeneratortable.ID;
                newgeneratortable["PrimaryKeyField"] = thisgeneratortable.PrimaryKey;
                newgeneratortable["NameField"] = thisgeneratortable.Name;
                newgeneratortable["DescriptionField"] = thisgeneratortable.Description;
                newgeneratortable["RelatedTable"] = thisgeneratortable.RelatedTable;
                newgeneratortable["RelatedKeyField"] = thisgeneratortable.RelatedKey;
                newgeneratortable["MemberParentTable"] = "Parent Links...";
                newgeneratortable["MemberFilters"] = "Filters...";
                currentgeneratortables.Rows.Add(newgeneratortable);
            }

            return currentgeneratortables;
        }

        /// <summary>
        /// Returns whether a member generator has been selected or not
        /// </summary>
        /// <returns>True if selected, false if not</returns>
        public bool GeneratorSelected()
        {
            // Return whether another measure is selected or now
            if (CurrentGeneratorsGrid.SelectedRows.Count > 0)
                return true;
            else
                return false;
        }

        /// <summary>
        /// This adds a new empty generator to the collection ready for configuring
        /// </summary>
        /// <returns>True if a generator is selected, false if not</returns>
        public bool AddNewEmptyGenerator()
        {
            // Generate a unique new name for the generator
            int counter = 0;
            string basegeneratorname = "New Generator";
            string generatorname = basegeneratorname;
            bool exists = true;
            while (exists)
            {
                exists = false;
                foreach (MemberGenerator thisgenerator in _editingcube.Dimensions[_dimensionid].MemberGenerators)
                    if (thisgenerator.Name.ToUpper().Equals(generatorname.ToUpper()))
                    {
                        exists = true;
                        counter++;
                        generatorname = basegeneratorname + " " + counter.ToString();
                        break;
                    }
            }

            // Create a default generator
            MemberGenerator newgenerator = new MemberGenerator(generatorname, _editingcube.Dimensions[_dimensionid]);

            // Add the generator to our collection and rebind our grid            
            _editingcube.Dimensions[_dimensionid].MemberGenerators.Add(newgenerator);
            BindGrid();
            if (CurrentGeneratorsGrid.SelectedRows.Count > 0)
                return true;
            else
                return false;
        }

        /// <summary>
        /// This removes the currently selected member generator from the collection
        /// </summary>
        /// <returns>True if another member generator is selected</returns>
        public bool RemoveSelectedGenerator()
        {
            // Check whether a generator is selected on the data grid
            if (CurrentGeneratorsGrid.SelectedRows.Count > 0)
            {
                // Find the ID of the selected generator
                Guid thisgeneratorid = new Guid(CurrentGeneratorsGrid.SelectedRows[0].Cells["GeneratorID"].Value.ToString());

                // Remove the row from the collection
                _editingcube.Dimensions[_dimensionid].MemberGenerators.Remove(thisgeneratorid);

                // Update our grid data
                BindGrid();

                // Return whether another measure is selected or now
                if (CurrentGeneratorsGrid.SelectedRows.Count > 0)
                    return true;
                else
                    return false;
            }
            else
                return false;
        }

        private void PrepareComboColumns()
        {
            // First we prepare the signage
            DataTable signagetypes = new DataTable();
            signagetypes.Columns.Add("ID");
            signagetypes.Columns.Add("Value");
            foreach (string thissignagetype in Enum.GetNames(typeof(Signage)))
            {
                DataRow newsignagetype = signagetypes.NewRow();
                newsignagetype["ID"] = thissignagetype;
                newsignagetype["Value"] = thissignagetype;
                signagetypes.Rows.Add(newsignagetype);
            }
            GeneratorSignage.DataSource = signagetypes;
            GeneratorSignage.ValueMember = "ID";
            GeneratorSignage.DisplayMember = "Value";

            // Next the aggregation types
            DataTable aggregationtypes = new DataTable();
            aggregationtypes.Columns.Add("ID");
            aggregationtypes.Columns.Add("Value");
            foreach (string thisaggregationtype in Enum.GetNames(typeof(AggregationTypes)))
            {
                DataRow newaggregationtype = aggregationtypes.NewRow();
                newaggregationtype["ID"] = thisaggregationtype;
                newaggregationtype["Value"] = thisaggregationtype;
                aggregationtypes.Rows.Add(newaggregationtype);
            }
            GeneratorAggregation.DataSource = aggregationtypes;
            GeneratorAggregation.ValueMember = "ID";
            GeneratorAggregation.DisplayMember = "Value";

            // Now the the Tables available for the member generator tables and related tables
            CubedDatabaseConnector dbprovider = AddInManager.Connectors[_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource]
                    .Provider.ConnectorID].ConnectorInstance;
                    dbprovider.SetConfiguration(_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource].Provider.ConnectorConfiguration);
                    dbprovider.Open();

            DataTable generatortables = new DataTable();
            generatortables.Columns.Add("ID");
            generatortables.Columns.Add("Value");
            List<string> availabletables = dbprovider.DatabaseTables();
            foreach (string thisgeneratortable in availabletables)
            {
                DataRow newgeneratortable = generatortables.NewRow();
                newgeneratortable["ID"] = thisgeneratortable;
                newgeneratortable["Value"] = thisgeneratortable;
                generatortables.Rows.Add(newgeneratortable);
            }
            NewTable.DataSource = generatortables;
            NewTable.ValueMember = "ID";
            NewTable.DisplayMember = "Value";

            DataTable parenttables = new DataTable();
            parenttables.Columns.Add("ID");
            parenttables.Columns.Add("Value");
            List<string> availableparenttables = dbprovider.DatabaseTables();
            foreach (string thisparenttable in availableparenttables)
            {
                DataRow newparenttable = parenttables.NewRow();
                newparenttable["ID"] = thisparenttable;
                newparenttable["Value"] = thisparenttable;
                parenttables.Rows.Add(newparenttable);
            }
            ParentTableName.DataSource = parenttables;
            ParentTableName.ValueMember = "ID";
            ParentTableName.DisplayMember = "Value";           

            // Then we prepare the range generator types
            RangeType.Items.Clear();
            foreach (string thisrangetype in Enum.GetNames(typeof(RangeTypes)))            
                RangeType.Items.Add(thisrangetype);
            if (RangeType.Items.Count > 0)
                RangeType.SelectedIndex = 0;
        }

        /// <summary>
        /// This binds our data grid
        /// </summary>
        private void BindGrid()
        {
            try
            {
                CurrentGeneratorsGrid.AutoGenerateColumns = false;
                CurrentGeneratorsGrid.DataSource = null;
                CurrentGeneratorsGrid.DataSource = CurrentGenerators();
                CurrentGeneratorsGrid.Sort(CurrentGeneratorsGrid.Columns[0], ListSortDirection.Ascending);
            }
            catch
            { }
        }

        /// <summary>
        /// This updates the list of available tables based on the selected dimension
        /// </summary>
        private void UpdateMemberTables()
        {
            // Connect to our data provider and update our member tables collection
            try
            {
                // Now update our selected database table
                CubedDatabaseConnector dbprovider = AddInManager.Connectors[_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource]
                    .Provider.ConnectorID].ConnectorInstance;
                dbprovider.SetConfiguration(_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource].Provider.ConnectorConfiguration);
                dbprovider.Open();
                MemberTable.DataSource = dbprovider.DatabaseTables();
                
                if (MemberTable.Items.Count > 0)
                    MemberTable.SelectedIndex = 0;
                else
                    MemberTable.SelectedIndex = -1;

                // And then update our database fields
                UpdateMemberTableFields(dbprovider);

                // Close our database connection
                dbprovider.Rollback();
            }
            catch
            {
                MemberTable.DataSource = null;
                MemberNameField.DataSource = null;
                MemberDescriptionField.DataSource = null;
            }
        }

        /// <summary>
        /// This updates the "related tables" drop down to reflect all tables currently added in the collection
        /// </summary>
        private void UpdateRelatedTables()
        {
            NewRelatedTable.DataSource = null;
            List<string> currenttables = new List<string>();
            foreach (MemberTableDetails thiscurrenttable in _editingcube.Dimensions[_dimensionid].MemberGenerators
                [_editinggeneratorid].MemberTables)
                currenttables.Add(thiscurrenttable.Table);
            NewRelatedTable.DataSource = currenttables;
            
            // Update related table fields
            if (NewRelatedTable.Items.Count > 0)
            {
                NewRelatedTable.SelectedIndex = 0;
                UpdateRelatedTableFields(NewRelatedTable.Text);
            }
            else
            {
                NewRelatedKeyField.DataSource = null;
            }
        }

        private void UpdateRelatedTableFields(string selectedtable)
        {
            CubedDatabaseConnector dbprovider = AddInManager.Connectors[_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource]
                    .Provider.ConnectorID].ConnectorInstance;
            dbprovider.SetConfiguration(_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource].Provider.ConnectorConfiguration);
            dbprovider.Open();
            List<string> relatedfields = dbprovider.DatabaseTableFields(selectedtable);
            NewRelatedKeyField.DataSource = relatedfields;
            if (NewRelatedKeyField.Items.Count > 0)
                NewRelatedKeyField.SelectedIndex = 0;
            dbprovider.Rollback();
        }

        /// <summary>
        /// This updates the fields drop downs based on the dimensions data source and selected table
        /// </summary>
        /// <param name="dbprovider">The database provider to connect to the source database with</param>
        private void UpdateMemberTableFields(CubedDatabaseConnector dbprovider)
        {
            // This updates all table fields when a new database table has been chosen
            if (MemberTable.SelectedIndex > -1 && MemberTable.Text != String.Empty)
            {
                // Get our list of available fields from the database provider                
                List<string> availablenamefields = dbprovider.DatabaseTableFields(MemberTable.Text);
                List<string> availabledescriptionfields = dbprovider.DatabaseTableFields(MemberTable.Text);

                // Set our drop down values
                if (availablenamefields.Count > 0)
                {
                    // Check to see if we already have values                    
                    MemberNameField.DataSource = availablenamefields;                    
                    MemberDescriptionField.DataSource = availabledescriptionfields;                    
                }
                else
                {
                    // There are no database fields                    
                    MemberNameField.DataSource = null;
                    MemberDescriptionField.DataSource = null;
                }
            }
            else
            {
                MemberNameField.DataSource = null;
                MemberDescriptionField.DataSource = null;
            }
        }

        /// <summary>
        /// This updates the field lists for the combo boxes in the Member Generator Table Fields
        /// </summary>
        /// <param name="dbprovider">The database connector for use in getting table fields</param>
        private void UpdateMemberGeneratorTableFields(CubedDatabaseConnector dbprovider, string selectedtable)
        {            
            DataTable generatorprimarykeyfields = new DataTable();
            generatorprimarykeyfields.Columns.Add("ID");
            generatorprimarykeyfields.Columns.Add("Value");
            List<string> availablekeyfields = dbprovider.DatabaseTableFields(selectedtable);
            foreach (string thisgeneratorfield in availablekeyfields)
            {
                DataRow newgeneratorfield = generatorprimarykeyfields.NewRow();
                newgeneratorfield["ID"] = thisgeneratorfield;
                newgeneratorfield["Value"] = thisgeneratorfield;
                generatorprimarykeyfields.Rows.Add(newgeneratorfield);
            }
            NewPrimaryKeyField.DataSource = generatorprimarykeyfields;
            NewPrimaryKeyField.ValueMember = "ID";
            NewPrimaryKeyField.DisplayMember = "Value";

            DataTable generatornamefields = new DataTable();
            generatornamefields.Columns.Add("ID");
            generatornamefields.Columns.Add("Value");
            List<string> availablenamefields = dbprovider.DatabaseTableFields(selectedtable);
            foreach (string thisgeneratorfield in availablenamefields)
            {
                DataRow newgeneratorfield = generatornamefields.NewRow();
                newgeneratorfield["ID"] = thisgeneratorfield;
                newgeneratorfield["Value"] = thisgeneratorfield;
                generatornamefields.Rows.Add(newgeneratorfield);
            }
            NewNameField.DataSource = generatornamefields;
            NewNameField.ValueMember = "ID";
            NewNameField.DisplayMember = "Value";

            DataTable generatordescriptionfields = new DataTable();
            generatordescriptionfields.Columns.Add("ID");
            generatordescriptionfields.Columns.Add("Value");
            List<string> availabledescriptionfields = dbprovider.DatabaseTableFields(selectedtable);
            foreach (string thisgeneratorfield in availabledescriptionfields)
            {
                DataRow newgeneratorfield = generatordescriptionfields.NewRow();
                newgeneratorfield["ID"] = thisgeneratorfield;
                newgeneratorfield["Value"] = thisgeneratorfield;
                generatordescriptionfields.Rows.Add(newgeneratorfield);
            }
            NewDescriptionField.DataSource = generatordescriptionfields;
            NewDescriptionField.ValueMember = "ID";
            NewDescriptionField.DisplayMember = "Value";
        }

        private void UpdateAttributesTable(string selectedtable)
        {
            // First we update the combo box values with the available database table fields
            CubedDatabaseConnector dbprovider = AddInManager.Connectors[_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource]
                   .Provider.ConnectorID].ConnectorInstance;
            dbprovider.SetConfiguration(_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource].Provider.ConnectorConfiguration);
            dbprovider.Open();
            List<string> availablefields = dbprovider.DatabaseTableFields(selectedtable);
            AttributeValueField.DataSource = availablefields;            
            dbprovider.Rollback();

            // Then we add each of the attribute values from the selected member attributes table, but
            // ONLY if they exist on the dimensions attributes collection
            DataTable AttributeValueFields = new DataTable();
            AttributeValueFields.Columns.Add("AttributeName");
            AttributeValueFields.Columns.Add("AttributeValueFieldID");
            AttributeValueFields.Columns.Add("AttributeValueField");
            
            // First we add any other attributes which exist on this dimension which haven't already been
            // catered for
            foreach (Bamboo.Cubed.Structures.Attribute thisattribute in _editingcube.Dimensions[_dimensionid].Attributes)
            {
                bool alreadyexists = false;
                foreach (AttributeValueField thisfield in _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid]
                .MemberTables[_editingtabledetailsid].AttributeValueFields)
                {
                    if (thisfield.Attribute == thisattribute.ID)
                        alreadyexists = true;
                }
                if (!alreadyexists)
                {
                    // The attribute hasn't already been added to the attribute/value list, so add it
                    AttributeValueField newvaluefield = new AttributeValueField();
                    newvaluefield.Attribute = thisattribute.ID;
                    newvaluefield.FieldName = String.Empty;
                    _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid]
                    .AttributeValueFields.Add(newvaluefield);
                }
            }
            
            // Then we remove any that don't exist
            foreach (AttributeValueField thisfield in _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid]
                .MemberTables[_editingtabledetailsid].AttributeValueFields)
            {
                bool stillexists = false;
                foreach (Bamboo.Cubed.Structures.Attribute thisattribute in _editingcube.Dimensions[_dimensionid].Attributes)
                {
                    if (thisattribute.ID == thisfield.Attribute)
                        stillexists = true;
                }

                if (stillexists)
                {
                    // The attribute still exists, so we add the attribute value/field
                    DataRow newfield = AttributeValueFields.NewRow();
                    newfield["AttributeName"] = _editingcube.Dimensions[_dimensionid].Attributes[thisfield.Attribute].Name;
                    newfield["AttributeValueFieldID"] = thisfield.ID;
                    newfield["AttributeValueField"] = thisfield.FieldName;
                    AttributeValueFields.Rows.Add(newfield);
                }
            }


            CurrentAttributeGenerators.AutoGenerateColumns = false;
            CurrentAttributeGenerators.DataSource = null;
            CurrentAttributeGenerators.DataSource = AttributeValueFields;                
        }

        private void SaveAttributeGenerators()
        {
            // Save all the attribute generator field names back to the collection
            _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].AttributeValueFields.Clear();
            foreach (DataGridViewRow thisrow in CurrentAttributeGenerators.Rows)
            {
                AttributeValueField newfield = new AttributeValueField();
                
                // Find the attribute
                foreach (Bamboo.Cubed.Structures.Attribute thisattribute in _editingcube.Dimensions[_dimensionid].Attributes)
                {
                    if (thisattribute.Name.ToUpper().Equals(thisrow.Cells["AttributeName"].Value.ToString().ToUpper()))
                    {
                        newfield.Attribute = thisattribute.ID;
                    }
                }

                // Set the field if it's not null
                if (thisrow.Cells["AttributeValueField"].Value != null && thisrow.Cells["AttributeValueField"].Value.ToString() !=
                    String.Empty)
                {
                    newfield.FieldName = thisrow.Cells["AttributeValueField"].Value.ToString();
                }

                // Add it to the collection
                if (newfield.Attribute != null)
                {
                    _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].AttributeValueFields.Add(newfield);
                }
            }

            // Hide the Attribute Generators panel
            AttributeGeneratorsPanel.Visible = false;
            TablesPanel.Visible = true;

            // Enable the Add/Remove buttons
            AddNewTableButton.Enabled = true;
            RemoveSelectedTableButton.Enabled = true;
        }

        private void MemberTable_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!loading)
            {
                try
                {
                    CubedDatabaseConnector dbprovider = AddInManager.Connectors[_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource]
                    .Provider.ConnectorID].ConnectorInstance;
                    dbprovider.SetConfiguration(_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource].Provider.ConnectorConfiguration);
                    dbprovider.Open();
                    UpdateMemberTableFields(dbprovider);
                    dbprovider.Rollback();
                }
                catch
                {
                    MemberNameField.DataSource = null;
                    MemberDescriptionField.DataSource = null;
                }
            }
        }

        private void UpdateRangesPanel()
        {
            // First set the range text
            RangeType.Text = _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.RangeType.ToString();

            // Now set the appropriate settings to enabled
            RangeType_SelectedIndexChanged(this, null);

            // Now set the existing values (if they exist)
            RangeGenerator selectedrange = _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges;
            if (selectedrange.Table != String.Empty && MemberTable.Items.Contains(selectedrange.Table))
                MemberTable.Text = selectedrange.Table; 
            
            // Update the selected tables' fields
            loading = false;
            MemberTable_SelectedIndexChanged(this, null);
            loading = true;

            if (selectedrange.NameField != String.Empty && MemberNameField.Items.Contains(selectedrange.NameField))
                MemberNameField.Text = selectedrange.NameField;
            if (selectedrange.DescriptionField != String.Empty && MemberDescriptionField.Items.Contains(selectedrange.DescriptionField))
                MemberDescriptionField.Text = selectedrange.DescriptionField;
            if (selectedrange.DateLowerRange != null)
                DateRangeLowerLimit.Value = selectedrange.DateLowerRange;
            if (selectedrange.DateUpperRange != null)
                DateRangeUpperLimit.Value = selectedrange.DateUpperRange;
            YearStartMonth.Value = Convert.ToDecimal(selectedrange.YearStartMonth);
            if (selectedrange.DateFormatString != String.Empty)
                DateFormatString.Text = selectedrange.DateFormatString;
            NumberLowerRange.Value = Convert.ToDecimal(selectedrange.NumberLowerRange);
            NumberUpperRange.Value = Convert.ToDecimal(selectedrange.NumberUpperRange);
            NumberSeed.Value = Convert.ToDecimal(selectedrange.NumberSeed);
            if (selectedrange.NumberFormatString != String.Empty)
                NumberFormatString.Text = selectedrange.NumberFormatString;
        }

        private void CurrentGeneratorsGrid_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {
            // Check which button was pressed, if any
            if (e.ColumnIndex > -1 && CurrentGeneratorsGrid.Columns[e.ColumnIndex].Name == "GeneratorConfiguration")
            {
                // The user wants to configure the generator, so we need to determine what type of member
                // generator this is                
                loading = true;
                _editinggeneratorid = new Guid(CurrentGeneratorsGrid.Rows[e.RowIndex].Cells["GeneratorID"].Value.ToString()); ;
                CurrentMemberGeneratorsPanel.Visible = false;

                if (_editingcube.Dimensions[_dimensionid].MemberType == MemberTypes.Filter)
                {
                    // This is a filter generator, so we need to configure a range generator 
                    RangesPanel.Visible = true;
                    TablesPanel.Visible = false;
                    
                    // Update the ranges panel based on the current configuration
                    UpdateRangesPanel();
                }
                else
                {
                    // This is an identifier based generator, so we need to configure the member tables
                    RangesPanel.Visible = false;
                    TablesPanel.Visible = true;
                    CurrentTablesPanel.Visible = true;
                    CurrentTablesPanel.Dock = DockStyle.Fill;
                    AddNewTablePanel.Visible = false;
                    AttributeGeneratorsPanel.Visible = false;

                    // Update the tables panel based on the current configuration
                    CurrentTablesGrid.DataSource = CurrentGeneratorTables();
                }
                                
                loading = false;
            }
        }

        private void AddNewTableButton_Click(object sender, EventArgs e)
        {
            // Enable the "Add New Table" panel
            AddNewTablePanel.Visible = true;            
            AddNewTableButton.Enabled = false;
            RemoveSelectedTableButton.Enabled = false;
            SaveNewTableButton.Enabled = true;
            CancelNewTableButton.Enabled = true;

            // Prepare the main tables and fields list
            if (NewTable.Items.Count > 0)
            {
                NewTable.SelectedIndex = 0;
                CubedDatabaseConnector dbprovider = AddInManager.Connectors[_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource]
                   .Provider.ConnectorID].ConnectorInstance;
                dbprovider.SetConfiguration(_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource].Provider.ConnectorConfiguration);
                dbprovider.Open();                
                UpdateMemberGeneratorTableFields(dbprovider, NewTable.Text);
                dbprovider.Rollback();
            }
            else
            {
                NewPrimaryKeyField.DataSource = null;
                NewNameField.DataSource = null;
                NewDescriptionField.DataSource = null;
            }

            if (CurrentGeneratorTables().Rows.Count > 0)
            {
                // There are already existing tables, so make the "related table" settings enabled
                NewRelatedTable.Enabled = true;
                NewRelatedKeyField.Enabled = true;
                UpdateRelatedTables();
            }
            else
            {
                // There are no existing tables, so make the "related table" settings disabled
                NewRelatedTable.Enabled = false;
                NewRelatedKeyField.Enabled = false;
                NewRelatedTable.DataSource = null;
                NewRelatedKeyField.DataSource = null;
            }
        }

        private void CancelNewTableButton_Click(object sender, EventArgs e)
        {
            CurrentTablesPanel.Dock = DockStyle.Fill;
            if (AttributeGeneratorsPanel.Visible)
            {
                // Hide the Attribute Generators panel
                AttributeGeneratorsPanel.Visible = false;
                TablesPanel.Visible = true;

                // Enable the Add/Remove buttons
                AddNewTableButton.Enabled = true;
                RemoveSelectedTableButton.Enabled = true;
            }
            else if (AddNewTablePanel.Visible)
            {
                // Disable the "Add New Table" panel
                AddNewTablePanel.Visible = false;
                AddNewTableButton.Enabled = true;
                AttributeGeneratorsPanel.Visible = false;
                RemoveSelectedTableButton.Enabled = true;
            }
            else if (MemberParentTablePanel.Visible)
            {
                // Disable the "Member Parent Table" panel
                MemberParentTablePanel.Visible = false;
                TablesPanel.Visible = true;

                // Enable the Add/Remove buttons
                AddNewTableButton.Enabled = true;
                RemoveSelectedTableButton.Enabled = true;
            }
            else if (MemberTableFilterPanel.Visible)
            {
                // Hide the Member Table Filter panel
                MemberTableFilterPanel.Visible = false;
                TablesPanel.Visible = true;

                // Enable the Add/Remove buttons
                AddNewTableButton.Enabled = true;
                RemoveSelectedTableButton.Enabled = true;
            }
            else
            {
                // Hide the entire member tables panel and re-load the current Member Generators panel                
                TablesPanel.Visible = false;
                CurrentMemberGeneratorsPanel.Visible = true;
            }
        }

        private void SaveNewTableButton_Click(object sender, EventArgs e)
        {
            // Determine what is being saved
            CurrentTablesPanel.Dock = DockStyle.Fill;
            if (AttributeGeneratorsPanel.Visible)
            {
                // The "attribute generators" are being saved, so save the grid back to the selected table
                SaveAttributeGenerators();
            }
            else if (AddNewTablePanel.Visible)
            {
                // The "New Table" is being saved, so sheck that the user has selected all required fields
                if (NewTable.SelectedIndex > -1 && NewTable.Text != String.Empty &&
                    NewPrimaryKeyField.SelectedIndex > -1 && NewPrimaryKeyField.Text != String.Empty &&
                    NewNameField.SelectedIndex > -1 && NewNameField.Text != String.Empty &&
                    (CurrentGeneratorTables().Rows.Count < 1 ||
                    (NewRelatedTable.SelectedIndex > -1 && NewRelatedTable.Text != String.Empty &&
                    NewRelatedKeyField.SelectedIndex > -1 && NewRelatedKeyField.Text != String.Empty)))
                {
                    // All fields have been selected correctly, so continue. First, we disable the 
                    // "Add New Table" panel
                    AddNewTablePanel.Visible = false;
                    AddNewTableButton.Enabled = true;
                    RemoveSelectedTableButton.Enabled = true;

                    // Then we add the selected record into the collection
                    if (CurrentGeneratorTables().Rows.Count < 1)
                    {
                        // This is the first record
                        MemberTableDetails newgeneratortable = new MemberTableDetails();
                        newgeneratortable.Table = NewTable.Text;
                        newgeneratortable.PrimaryKey = NewPrimaryKeyField.Text;
                        newgeneratortable.Name = NewNameField.Text;
                        if (NewDescriptionField.SelectedIndex > -1 && NewDescriptionField.Text != String.Empty)
                            newgeneratortable.Description = NewDescriptionField.Text;
                        _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables.Add(newgeneratortable);
                        BindTablesGrid();
                    }
                    else
                    {
                        // This is a subsequent record
                        MemberTableDetails newgeneratortable = new MemberTableDetails();
                        newgeneratortable.Table = NewTable.Text;
                        newgeneratortable.PrimaryKey = NewPrimaryKeyField.Text;
                        newgeneratortable.Name = NewNameField.Text;
                        newgeneratortable.RelatedTable = NewRelatedTable.Text;
                        newgeneratortable.RelatedKey = NewRelatedKeyField.Text;
                        if (NewDescriptionField.SelectedIndex > -1 && NewDescriptionField.Text != String.Empty)
                            newgeneratortable.Description = NewDescriptionField.Text;
                        _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables.Add(newgeneratortable);
                        BindTablesGrid();
                    }
                }
                else
                {
                    // Not every required field has been selected, so raise an error
                    MessageBox.Show("Not all fields required for a new table have been selected. Please select the fields "
                        + "before saving.", "Not all fields selected", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else if (MemberParentTablePanel.Visible)
            {
                // The member parent tables are being saved
                if (ParentTableName.Text != String.Empty)
                {
                    Bamboo.Cubed.Structures.MemberTableParent parentlink = new MemberTableParent();
                    parentlink.ParentTable = ParentTableName.Text;
                    parentlink.MemberIDField = ParentTableMemberID.Text;
                    parentlink.ParentIDField = ParentTableParentID.Text;
                    _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].ParentLink = parentlink;
                }
                else
                    _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].ParentLink = null;

                // Hide the Member Table Parents panel
                MemberParentTablePanel.Visible = false;
                TablesPanel.Visible = true;

                // Enable the Add/Remove buttons
                AddNewTableButton.Enabled = true;
                RemoveSelectedTableButton.Enabled = true;
            }
            else if (MemberTableFilterPanel.Visible)
            {
                // The member table filter is being saved                
                if (MemberFilterField.Text != String.Empty)
                {
                    ValueFilter filter = new ValueFilter();
                    filter.FieldName = MemberFilterField.Text;
                    if (FilterListCheckBox.Checked)
                    {
                        // This is a list filter
                        filter.FilterType = FilterTypes.List;
                        foreach (string thisvalue in FilterValuesList.Items)
                            filter.AcceptableValues.Add(thisvalue);
                    }
                    else
                    {
                        // This is a range filter
                        filter.FilterType = FilterTypes.Range;
                        filter.UpperValue = UpperLimit.Text;
                        filter.LowerValue = LowerLimit.Text;
                    }
                    _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].Filter = filter;
                }
                else
                    _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].Filter = null;

                // Hide the Member Table Filter panel
                MemberTableFilterPanel.Visible = false;
                TablesPanel.Visible = true;

                // Enable the Add/Remove buttons
                AddNewTableButton.Enabled = true;
                RemoveSelectedTableButton.Enabled = true;
            }
            else
            {
                // Save Member tables are saved automatically as we go, so simply close the screen                
                TablesPanel.Visible = false;
                CurrentMemberGeneratorsPanel.Visible = true;
            }
        }

        private void BindTablesGrid()
        {
            CurrentTablesGrid.AutoGenerateColumns = false;
            CurrentTablesGrid.DataSource = null;
            CurrentTablesGrid.DataSource = CurrentGeneratorTables();
        }

        private void NewRelatedTable_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (NewRelatedTable.Items.Count > 0)
            {
                UpdateRelatedTableFields(NewRelatedTable.Text);
            }
            else
            {
                NewRelatedKeyField.DataSource = null;
            }
        }

        private void NewTable_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (NewTable.SelectedIndex > -1 && NewTable.Text != String.Empty)
            {
                CubedDatabaseConnector dbprovider = AddInManager.Connectors[_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource]
                    .Provider.ConnectorID].ConnectorInstance;
                dbprovider.SetConfiguration(_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource].Provider.ConnectorConfiguration);
                dbprovider.Open();
                UpdateMemberGeneratorTableFields(dbprovider, NewTable.Text);
            }
            else
            {
                NewPrimaryKeyField.DataSource = null;
                NewNameField.DataSource = null;
                NewDescriptionField.DataSource = null;
            }
        }

        private void RemoveSelectedTableButton_Click(object sender, EventArgs e)
        {
            int failcount = 0;
            for (int i = CurrentTablesGrid.SelectedRows.Count - 1; i > -1; i--)
            {
                // First we check whether this selected row can be deleted
                string tablename = CurrentTablesGrid.SelectedRows[i].Cells["TableName"].Value.ToString();
                bool relatedtables = false;
                foreach (DataGridViewRow thistablerow in CurrentTablesGrid.Rows)
                    if (thistablerow.Cells["RelatedTable"].Value != null &&
                        thistablerow.Cells["RelatedTable"].Value.ToString().ToUpper() ==
                        tablename.ToUpper())
                        relatedtables = true;
                if (relatedtables)
                {
                    // This table cannot be deleted as other tables are related to it
                    failcount++;
                }
                else
                {
                    // It is fine to delete the current table, so delete it from our underlying cube
                    _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables.Remove(new Guid(CurrentTablesGrid.SelectedRows[i].Cells["TableID"].Value.ToString()));
                }
            }

            BindTablesGrid();

            if (failcount > 0)
            {
                // Some tables could not be deleted as they were related to other tables
                MessageBox.Show("Some tables could not be removed as other tables are related to them. Please remove the related tables first.",
                    "Related tables found", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void CurrentTablesGrid_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {
            // Check which button was pressed, if any
            if (e.ColumnIndex > -1 && CurrentTablesGrid.Columns[e.ColumnIndex].Name == "AttributesGenerator")
            {
                // The user wants to configure the attributes generator
                _editingtabledetailsid = new Guid(CurrentTablesGrid.Rows[e.RowIndex].Cells["TableID"].Value.ToString());
                CurrentTablesPanel.Dock = DockStyle.Top;
                AttributeGeneratorsPanel.Visible = true;
                AttributeGeneratorsPanel.Dock = DockStyle.Fill;
                AddNewTableButton.Enabled = false;
                RemoveSelectedTableButton.Enabled = false;
                SaveNewTableButton.Enabled = true;
                CancelNewTableButton.Enabled = true;

                // Update the attributes table
                UpdateAttributesTable(CurrentTablesGrid.Rows[e.RowIndex].Cells["TableName"].Value.ToString());
            }
            else if (e.ColumnIndex > -1 && CurrentTablesGrid.Columns[e.ColumnIndex].Name == "MemberParentTable")
            {
                // The user wants to configure the members parents
                _editingtabledetailsid = new Guid(CurrentTablesGrid.Rows[e.RowIndex].Cells["TableID"].Value.ToString());
                CurrentTablesPanel.Dock = DockStyle.Top;
                MemberParentTablePanel.Visible = true;
                MemberParentTablePanel.Dock = DockStyle.Fill;
                AddNewTableButton.Enabled = false;
                RemoveSelectedTableButton.Enabled = false;
                SaveNewTableButton.Enabled = true;
                CancelNewTableButton.Enabled = true;

                // Attempt to load the previously configured settings
                if (_editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].ParentLink != null)
                {
                    ParentTableName.Text = _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].ParentLink.ParentTable;
                    ParentTableMemberID.Text = _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].ParentLink.MemberIDField;
                    ParentTableParentID.Text = _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].ParentLink.ParentIDField;                    
                }
                else
                {
                    ParentTableName.Text = String.Empty;
                    ParentTableMemberID.Text = String.Empty;
                    ParentTableParentID.Text = String.Empty;
                }
            }
            else if (e.ColumnIndex > -1 && CurrentTablesGrid.Columns[e.ColumnIndex].Name == "MemberFilters")
            {
                // The user wants to configure the members filters
                CurrentTablesPanel.Dock = DockStyle.Top;
                _editingtabledetailsid = new Guid(CurrentTablesGrid.Rows[e.RowIndex].Cells["TableID"].Value.ToString());
                MemberTableFilterPanel.Visible = true;
                MemberTableFilterPanel.Dock = DockStyle.Fill;
                AddNewTableButton.Enabled = false;
                RemoveSelectedTableButton.Enabled = false;
                SaveNewTableButton.Enabled = true;
                CancelNewTableButton.Enabled = true;

                // Load the Member Table fields list
                LoadMemberFilterTableFields();

                // Enable the current filter (if one exists)
                if (_editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].Filter != null &&
                    _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].Filter.ID != null)
                {
                    if (_editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].Filter.FilterType == FilterTypes.Range)
                    {
                        // This is a range filter
                        FilterValuesList.Items.Clear();
                        AddNewValue.Text = String.Empty;
                        FilterRangeCheckBox.Checked = true;
                        FilterRangeCheckBox_CheckedChanged(null, null);
                        LowerLimit.Text = _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].Filter.LowerValue;
                        UpperLimit.Text = _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].Filter.UpperValue;
                    }
                    else
                    {
                        // This is a list filter
                        UpperLimit.Text = String.Empty;
                        LowerLimit.Text = String.Empty;
                        FilterListCheckBox.Checked = true;
                        FilterListCheckBox_CheckedChanged(null, null);
                        AddNewValue.Text = String.Empty;
                        FilterValuesList.Items.Clear();
                        foreach (string thisvalue in
                            _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].Filter.AcceptableValues)
                            FilterValuesList.Items.Add(thisvalue);
                    }
                }
                else
                {
                    // Default to a list filter
                    UpperLimit.Text = String.Empty;
                    LowerLimit.Text = String.Empty;
                    FilterListCheckBox.Checked = true;
                    FilterListCheckBox_CheckedChanged(null, null);
                    AddNewValue.Text = String.Empty;
                    FilterValuesList.Items.Clear();
                }
            }
        }

        private void RangeType_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (RangeType.Text == RangeTypes.UniqueValues.ToString())
            {
                // Set the unique values definitions to being enabled and set their defaults
                MemberTable.Enabled = true;
                if (MemberTable.Items.Count > 0)
                    MemberTable.SelectedIndex = 0;
                MemberNameField.Enabled = true;
                if (MemberNameField.Items.Count > 0)
                    MemberNameField.SelectedIndex = 0;
                MemberDescriptionField.Enabled = true;
                if (MemberDescriptionField.Items.Count > 0)
                    MemberDescriptionField.SelectedIndex = 0;

                // Disable the others
                DateRangeLowerLimit.Enabled = false;
                DateRangeUpperLimit.Enabled = false;
                YearStartMonth.Enabled = false;
                DateFormatString.Enabled = false;
                NumberLowerRange.Enabled = false;
                NumberUpperRange.Enabled = false;
                NumberSeed.Enabled = false;
                NumberFormatString.Enabled = false;
            }
            else if (RangeType.Text == RangeTypes.SeededNumber.ToString())
            {
                // Set the number range definitions to being enabled and set their defaults
                NumberLowerRange.Enabled = true;
                NumberLowerRange.Value = 1;
                NumberUpperRange.Enabled = true;
                NumberUpperRange.Value = 1000;
                NumberSeed.Enabled = true;
                NumberSeed.Value = 1;
                NumberFormatString.Enabled = true;
                NumberFormatString.Text = new RangeGenerator().NumberFormatString;

                // Disable the others
                DateRangeLowerLimit.Enabled = false;
                DateRangeUpperLimit.Enabled = false;
                YearStartMonth.Enabled = false;
                DateFormatString.Enabled = false;
                MemberTable.Enabled = false;
                MemberNameField.Enabled = false;
                MemberDescriptionField.Enabled = false;
            }
            else
            {
                // Set the date range definitions to being enabled and set their defaults
                DateRangeLowerLimit.Value = DateTime.Now.AddYears(-3);
                DateRangeLowerLimit.Enabled = true;
                DateRangeUpperLimit.Value = DateTime.Now.AddYears(1);
                DateRangeUpperLimit.Enabled = true;
                YearStartMonth.Value = 7;
                YearStartMonth.Enabled = true;
                DateFormatString.Text = new RangeGenerator(((RangeTypes)Enum.Parse(typeof(RangeTypes),RangeType.Text))).DateFormatString;
                DateFormatString.Enabled = true;

                // Disable the others                
                MemberTable.Enabled = false;
                MemberNameField.Enabled = false;
                MemberDescriptionField.Enabled = false;
                NumberLowerRange.Enabled = false;
                NumberUpperRange.Enabled = false;
                NumberSeed.Enabled = false;
                NumberFormatString.Enabled = false;
            }
        }

        private void CancelRangeGeneratorButton_Click(object sender, EventArgs e)
        {
            // Changes have been cancelled, so simply hide the range generator panel and re-show the main panel
            RangesPanel.Visible = false;
            CurrentMemberGeneratorsPanel.Visible = true;
        }

        private void SaveRangeGeneratorButton_Click(object sender, EventArgs e)
        {
            // First close up the panel
            RangesPanel.Visible = false;
            CurrentMemberGeneratorsPanel.Visible = true;

            // Then save the appropriate settings
            _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.RangeType
                = (RangeTypes)Enum.Parse(typeof(RangeTypes), RangeType.Text);
            if (_editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.RangeType
                == RangeTypes.UniqueValues)
            {
                // A unique values generator has been selected, so save those values
                _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.Table =
                    MemberTable.Text;
                _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.NameField =
                    MemberNameField.Text;
                _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.DescriptionField =
                    MemberDescriptionField.Text;
            }
            else if (_editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.RangeType
                == RangeTypes.SeededNumber)
            {
                // A seeded number generator has been selected, so save those values
                _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.NumberLowerRange =
                    Convert.ToInt32(NumberLowerRange.Value);
                _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.NumberUpperRange =
                    Convert.ToInt32(NumberUpperRange.Value);
                _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.NumberSeed =
                    Convert.ToInt32(NumberSeed.Value);
                _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.NumberFormatString =
                    NumberFormatString.Text;
            }
            else
            {
                // A date generator has been selected, so save those values
                _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.DateLowerRange =
                    DateRangeLowerLimit.Value;
                _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.DateUpperRange =
                    DateRangeUpperLimit.Value;
                _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.YearStartMonth =
                    Convert.ToInt32(YearStartMonth.Value);
                _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].Ranges.DateFormatString =
                    DateFormatString.Text;
            }
        }

        private void CurrentGeneratorsGrid_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            // Determine which cell was edited, if any
            if (e.ColumnIndex > -1 && CurrentGeneratorsGrid.Columns[e.ColumnIndex].Name == "GeneratorName")
            {
                // The user changed the name of a measure
                _editingcube.Dimensions[_dimensionid].MemberGenerators[new Guid(CurrentGeneratorsGrid.Rows[e.RowIndex].Cells["GeneratorID"].Value.ToString())].Name =
                    CurrentGeneratorsGrid.Rows[e.RowIndex].Cells["GeneratorName"].Value.ToString();
            }
            else if (e.ColumnIndex > -1 && CurrentGeneratorsGrid.Columns[e.ColumnIndex].Name == "GeneratorSignage")
            {
                // The user changed the type of a measure
                _editingcube.Dimensions[_dimensionid].MemberGenerators[new Guid(CurrentGeneratorsGrid.Rows[e.RowIndex].Cells["GeneratorID"].Value.ToString())].Sign =
                    (Signage)Enum.Parse(typeof(Signage), CurrentGeneratorsGrid.Rows[e.RowIndex].Cells["GeneratorSignage"].Value.ToString());
            }
            else if (e.ColumnIndex > -1 && CurrentGeneratorsGrid.Columns[e.ColumnIndex].Name == "GeneratorOverwrite")
            {
                // The user changed the measure consolidation type
                _editingcube.Dimensions[_dimensionid].MemberGenerators[new Guid(CurrentGeneratorsGrid.Rows[e.RowIndex].Cells["GeneratorID"].Value.ToString())].ReplaceExisting =
                    Boolean.Parse(CurrentGeneratorsGrid.Rows[e.RowIndex].Cells["GeneratorOverwrite"].Value.ToString());
            }
            else if (e.ColumnIndex > -1 && CurrentGeneratorsGrid.Columns[e.ColumnIndex].Name == "GeneratorAggregation")
            {
                // The user changed the measure aggregation type
                _editingcube.Dimensions[_dimensionid].MemberGenerators[new Guid(CurrentGeneratorsGrid.Rows[e.RowIndex].Cells["GeneratorID"].Value.ToString())].Aggregation =
                    (AggregationTypes)Enum.Parse(typeof(AggregationTypes), CurrentGeneratorsGrid.Rows[e.RowIndex].Cells["GeneratorAggregation"].Value.ToString());
            }

            // Rebind our grid
            BindGrid();
        }

        private void ParentTableName_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (ParentTableName.SelectedIndex > -1 && ParentTableName.Text != String.Empty)
            {
                CubedDatabaseConnector dbprovider = AddInManager.Connectors[_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource]
                    .Provider.ConnectorID].ConnectorInstance;
                dbprovider.SetConfiguration(_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource].Provider.ConnectorConfiguration);
                dbprovider.Open();
                UpdateParentTableFields(dbprovider, ParentTableName.Text);
            }
            else
            {
                ParentTableParentID.DataSource = null;
                ParentTableMemberID.DataSource = null;
            }
        }

        private void LoadMemberFilterTableFields()
        {
            // Grab the name of the current member table
            string selectedtable = _editingcube.Dimensions[_dimensionid].MemberGenerators[_editinggeneratorid].MemberTables[_editingtabledetailsid].Table;
            CubedDatabaseConnector dbprovider = AddInManager.Connectors[_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource]
                    .Provider.ConnectorID].ConnectorInstance;
            dbprovider.SetConfiguration(_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource].Provider.ConnectorConfiguration);
            dbprovider.Open();

            DataTable memberfiltertablefields = new DataTable();
            memberfiltertablefields.Columns.Add("ID");
            memberfiltertablefields.Columns.Add("Value");
            List<string> availablememberfilterfields = dbprovider.DatabaseTableFields(selectedtable);
            foreach (string thismemberfilterfield in availablememberfilterfields)
            {
                DataRow newmemberfilterfield = memberfiltertablefields.NewRow();
                newmemberfilterfield["ID"] = thismemberfilterfield;
                newmemberfilterfield["Value"] = thismemberfilterfield;
                memberfiltertablefields.Rows.Add(newmemberfilterfield);
            }
            MemberFilterField.DataSource = memberfiltertablefields;
            MemberFilterField.ValueMember = "ID";
            MemberFilterField.DisplayMember = "Value";

            dbprovider.Rollback();
        }

        private void UpdateParentTableFields(CubedDatabaseConnector dbprovider, string selectedtable)
        {
            DataTable parenttablememberidfields = new DataTable();
            parenttablememberidfields.Columns.Add("ID");
            parenttablememberidfields.Columns.Add("Value");
            List<string> availablememberidfields = dbprovider.DatabaseTableFields(selectedtable);
            foreach (string thismemberidfield in availablememberidfields)
            {
                DataRow newmemberidfield = parenttablememberidfields.NewRow();
                newmemberidfield["ID"] = thismemberidfield;
                newmemberidfield["Value"] = thismemberidfield;
                parenttablememberidfields.Rows.Add(newmemberidfield);
            }
            ParentTableMemberID.DataSource = parenttablememberidfields;
            ParentTableMemberID.ValueMember = "ID";
            ParentTableMemberID.DisplayMember = "Value";

            DataTable parenttableparentidfields = new DataTable();
            parenttableparentidfields.Columns.Add("ID");
            parenttableparentidfields.Columns.Add("Value");
            List<string> availableparentidfields = dbprovider.DatabaseTableFields(selectedtable);
            foreach (string thisparentidfield in availableparentidfields)
            {
                DataRow newgeneratorfield = parenttableparentidfields.NewRow();
                newgeneratorfield["ID"] = thisparentidfield;
                newgeneratorfield["Value"] = thisparentidfield;
                parenttableparentidfields.Rows.Add(newgeneratorfield);
            }
            ParentTableParentID.DataSource = parenttableparentidfields;
            ParentTableParentID.ValueMember = "ID";
            ParentTableParentID.DisplayMember = "Value";
        }

        private void FilterListCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (FilterListCheckBox.Checked)
            {
                // Enable our values list
                FilterValuesList.Enabled = true;
                FilterValuesList.Items.Clear();
                AddFilterValueButton.Enabled = true;
                RemoveValueButton.Enabled = true;
                AddNewValue.Text = String.Empty;
                AddNewValue.Enabled = true;

                // Disable our range selections
                UpperLimit.Enabled = false;
                UpperLimitLabel.Enabled = false;
                UpperLimit.Text = String.Empty;
                LowerLimit.Enabled = false;
                LowerLimitLabel.Enabled = false;
                LowerLimit.Text = String.Empty;
            }
            else
            {
                // Disable our values list
                FilterValuesList.Enabled = false;
                FilterValuesList.Items.Clear();
                AddFilterValueButton.Enabled = false;
                RemoveValueButton.Enabled = false;
                AddNewValue.Text = String.Empty;
                AddNewValue.Enabled = false;

                // Enable our range selections
                UpperLimit.Enabled = true;
                UpperLimitLabel.Enabled = true;
                UpperLimit.Text = String.Empty;
                LowerLimit.Enabled = true;
                LowerLimitLabel.Enabled = true;
                LowerLimit.Text = String.Empty;
            }
        }

        private void FilterRangeCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (FilterListCheckBox.Checked)
            {
                // Enable our values list
                FilterValuesList.Enabled = true;
                FilterValuesList.Items.Clear();
                AddFilterValueButton.Enabled = true;
                RemoveValueButton.Enabled = true;
                AddNewValue.Text = String.Empty;
                AddNewValue.Enabled = true;

                // Disable our range selections
                UpperLimit.Enabled = false;
                UpperLimitLabel.Enabled = false;
                UpperLimit.Text = String.Empty;
                LowerLimit.Enabled = false;
                LowerLimitLabel.Enabled = false;
                LowerLimit.Text = String.Empty;
            }
            else
            {
                // Disable our values list
                FilterValuesList.Enabled = false;
                FilterValuesList.Items.Clear();
                AddFilterValueButton.Enabled = false;
                RemoveValueButton.Enabled = false;
                AddNewValue.Text = String.Empty;
                AddNewValue.Enabled = false;

                // Enable our range selections
                UpperLimit.Enabled = true;
                UpperLimitLabel.Enabled = true;
                UpperLimit.Text = String.Empty;
                LowerLimit.Enabled = true;
                LowerLimitLabel.Enabled = true;
                LowerLimit.Text = String.Empty;
            }
        }

        private void AddFilterValueButton_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (AddNewValue.Text != String.Empty)
            {
                if (!FilterValuesList.Items.Contains(AddNewValue.Text))
                    FilterValuesList.Items.Add(AddNewValue.Text);
                else
                    MessageBox.Show("This value has already been added to the list",
                    "Value already added", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                AddNewValue.Text = String.Empty;
            }
            else
                MessageBox.Show("Please enter the new value to add to the acceptable list in the box to the left",
                    "Enter a new value to add", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }

        private void RemoveValueButton_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (FilterValuesList.SelectedIndex > -1)
            {
                FilterValuesList.Items.RemoveAt(FilterValuesList.SelectedIndex);
            }
            else
                MessageBox.Show("Please click on the value in the list you want to remove",
                    "Select value to remove", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }
    }
}
