/*
 * This source file (FunctionSettings.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.AddIns;

namespace Bamboo.Cubed.Designer
{
    /// <summary>
    /// This control allows for configuring available Functions
    /// </summary>
    public partial class FunctionSettings : UserControl
    {
        /// <summary>
        /// This is the current configuration control 
        /// </summary>
        private CubedFunctionConfiguration _configurationcontrol;

        /// <summary>
        /// Whether the current control is loading or not
        /// </summary>
        private bool loading = false;

        /// <summary>
        /// The alias of the record being editor, or blank if none
        /// </summary>
        private string editingalias = String.Empty;

        /// <summary>
        /// The current alias being edited in the grid
        /// </summary>
        private string currentalias = String.Empty;

        /// <summary>
        /// This initialises a new instance of the Function Providers Add Ins
        /// </summary>
        /// <param name="currentfunctions">The current function Add Ins available on the server</param>
        public FunctionSettings(GenericKeyCollection<string, Function> currentfunctions)
        {
            InitializeComponent();
            loading = true;
            _currentfunctions = currentfunctions;

            // Create the combo box
            NewFunctionProvider.Items.Clear();
            NewFunctionProvider.DataSource = AvailableFunctionAddins();
            NewFunctionProvider.DisplayMember = "Name";
            NewFunctionProvider.ValueMember = "ID";            

            // Select the first function
            if (AddInManager.Functions.Count > 0)
                NewFunctionProvider.SelectedIndex = 0;
            _currentnewfunction = new Function(String.Empty, new Guid(NewFunctionProvider.SelectedValue.ToString()),
                    String.Empty, AddInManager.Functions[new Guid(NewFunctionProvider.SelectedValue.ToString())].CurrentVersion);            

            // Update our grid data
            CurrentFunctionsGrid.AutoGenerateColumns = false;
            CurrentFunctionsGrid.DataSource = null;
            CurrentFunctionsGrid.DataSource = CurrentFunctionAddIns();

            // Fill the screen with our grid
            CurrentFunctionListPanel.Dock = DockStyle.Fill;

            UpdateScreen();            
            loading = false;
        }

        /// <summary>
        /// This updates the screen with the chosen Add-Ins configuration
        /// </summary>
        private void UpdateScreen()
        {
            // Initialise our data grid with the currently configured functions, as well as
            // set our label with the description of the currently selected function
            if (_currentnewfunction != null)
                NewDescriptionLabel.Text = AddInManager.Functions[_currentnewfunction.FunctionID].Description;
        }

        private GenericKeyCollection<string, Function> _currentfunctions;
        /// <summary>
        /// The current list of Cubed Functions available on the server
        /// </summary>
        public GenericKeyCollection<string, Function> CurrentFunctions
        {
            get { return _currentfunctions; }
            set { _currentfunctions = value; }
        }       

        private void NewFunctionProvider_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!loading)
            {
                // The "New Function" provider has changed, so update our label
                _currentnewfunction = new Function(String.Empty, new Guid(NewFunctionProvider.SelectedValue.ToString()),
                    String.Empty, AddInManager.Functions[new Guid(NewFunctionProvider.SelectedValue.ToString())].CurrentVersion);
                UpdateScreen();
            }
        }

        /// <summary>
        /// The currently selected new function
        /// </summary>
        private Function _currentnewfunction;

        /// <summary>
        /// This returns a data table of all available providers
        /// </summary>
        /// <returns>A data table of available providers from the Add In Manager</returns>
        private DataTable AvailableFunctionAddins()
        {
            DataTable AvailableFunctionAddins = new DataTable();
            AvailableFunctionAddins.Columns.Add("Name");
            AvailableFunctionAddins.Columns.Add("Description");
            AvailableFunctionAddins.Columns.Add("ID");

            foreach (CubedFunctionDescription thisfunction in AddInManager.Functions)
            {
                DataRow newfunction = AvailableFunctionAddins.NewRow();
                newfunction["Name"] = thisfunction.Name;
                newfunction["Description"] = thisfunction.Description;
                newfunction["ID"] = thisfunction.ID;
                AvailableFunctionAddins.Rows.Add(newfunction);
            }

            return AvailableFunctionAddins;
        }

        /// <summary>
        /// This returns a data table with the current function add ins
        /// </summary>
        /// <returns>A data table with the current function add ins</returns>
        private DataTable CurrentFunctionAddIns()
        {
            DataTable currentfunctionaddins = new DataTable();
            currentfunctionaddins.Columns.Add("Alias");
            currentfunctionaddins.Columns.Add("Name");
            currentfunctionaddins.Columns.Add("ID");
            currentfunctionaddins.Columns.Add("Version");
            currentfunctionaddins.Columns.Add("Configure");

            foreach (Function thisfunction in _currentfunctions)
            {
                DataRow newfunction = currentfunctionaddins.NewRow();
                newfunction["Alias"] = thisfunction.Alias;
                newfunction["Name"] = AddInManager.Functions[thisfunction.FunctionID].Name;
                newfunction["ID"] = AddInManager.Functions[thisfunction.FunctionID].ID;
                newfunction["Version"] = thisfunction.FunctionVersion;
                newfunction["Configure"] = "Configure...";
                currentfunctionaddins.Rows.Add(newfunction);
            }

            return currentfunctionaddins;
        }

        /// <summary>
        /// This adds the currently selected function into the collection with a default alias
        /// </summary>
        public void AddCurrentFunction()
        {
            // Check whether a function is currently selected
            if (_currentnewfunction != null && AddInManager.Functions.Contains(_currentnewfunction.FunctionID))
            {
                // Automatically create a default alias for the function
                editingalias = String.Empty;
                bool alreadyexists = true;
                int newcounter = 0;
                string newalias = AddInManager.Functions[_currentnewfunction.FunctionID].Name;
                string basealias = newalias;

                while (alreadyexists)
                {
                    alreadyexists = false;
                    foreach (Function thisfunction in CurrentFunctions)
                        if (thisfunction.Alias.ToUpper().Equals(newalias.ToUpper()))
                            alreadyexists = true;
                    if (alreadyexists)
                    {
                        newcounter++;
                        newalias = basealias + newcounter.ToString();
                    }
                }

                // Now we have an alias, update the current new function to use it
                _currentnewfunction.Alias = newalias;
                _currentnewfunction.ID = Guid.NewGuid();

                // Now that we have an alias for the new function, allow the user to configure it
                AddInConfigurationPanel.Visible = true;
                FunctionConfigurationPanel.Controls.Clear();
                _configurationcontrol = AddInManager.Functions[_currentnewfunction.FunctionID].ConfigurationInstance;
                _configurationcontrol.SetConfiguration(_currentnewfunction.FunctionConfiguration);
                FunctionConfigurationPanel.Controls.Add(_configurationcontrol);
                CurrentFunctionListPanel.Visible = false;
                NewFunctionTopPanel.Visible = false;
            }
        }

        /// <summary>
        /// This removes the currently selected function from the collection
        /// </summary>
        /// <returns>True if another function is selected</returns>
        public bool RemoveSelectedFunction()
        {
            // Check whether a function is selected on the data grid
            if (CurrentFunctionsGrid.SelectedRows.Count > 0)
            {
                // Find the Alias of the selected function
                string thisalias = CurrentFunctionsGrid.SelectedRows[0].Cells["Alias"].Value.ToString();

                // Remove the row from the collection
                CurrentFunctions.Remove(thisalias);

                // Update our grid data
                CurrentFunctionsGrid.DataSource = null;
                CurrentFunctionsGrid.DataSource = CurrentFunctionAddIns();
                CurrentFunctionsGrid.Sort(CurrentFunctionsGrid.Columns[0], ListSortDirection.Ascending);

                // Return whether another function is selected or now
                if (CurrentFunctionsGrid.SelectedRows.Count > 0)
                    return true;
                else
                    return false;
            }
            else
                return false;
        }
        
        /// <summary>
        /// Returns whether a function has been selected or not
        /// </summary>
        /// <returns>True if selected, false if not</returns>
        public bool FunctionSelected()
        {
            // Return whether another function is selected or now
            if (CurrentFunctionsGrid.SelectedRows.Count > 0)
                return true;
            else
                return false;
        }

        private void AddNewConfiguredFunction()
        {
            if (_currentnewfunction != null && _configurationcontrol != null)
            {
                // Grab the function configuration
                _currentnewfunction.FunctionConfiguration = _configurationcontrol.GetConfiguration();
                
                // Add our newly configured function
                _currentfunctions.Add(_currentnewfunction);

                // Update our grid data
                CurrentFunctionsGrid.DataSource = null;
                CurrentFunctionsGrid.DataSource = CurrentFunctionAddIns();
                CurrentFunctionsGrid.Sort(CurrentFunctionsGrid.Columns[0], ListSortDirection.Ascending);
            }
        }

        private void CancelButton_Click(object sender, EventArgs e)
        {
            // Clear the current panel
            AddInConfigurationPanel.Visible = false;
            FunctionConfigurationPanel.Controls.Clear();
            CurrentFunctionListPanel.Visible = true;
            NewFunctionTopPanel.Visible = true;
        }

        private void AddNewButton_Click(object sender, EventArgs e)
        {
            // Check if this is an add or configure
            if (editingalias == String.Empty)
            {
                // This is a new function, so add the newly configured function
                AddNewConfiguredFunction();
            }
            else
            {
                // This is a configuration, so update an existing function
                _currentfunctions[editingalias].FunctionConfiguration = _configurationcontrol.GetConfiguration();

                // Update our grid data
                CurrentFunctionsGrid.DataSource = null;
                CurrentFunctionsGrid.DataSource = CurrentFunctionAddIns();
                CurrentFunctionsGrid.Sort(CurrentFunctionsGrid.Columns[0], ListSortDirection.Ascending);
            }

            // Clear the current panel
            AddInConfigurationPanel.Visible = false;
            FunctionConfigurationPanel.Controls.Clear();
            CurrentFunctionListPanel.Visible = true;
            NewFunctionTopPanel.Visible = true;
        }

        private void CurrentFunctionsGrid_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {            
            if (CurrentFunctionsGrid.Columns[e.ColumnIndex].Name == "Alias" && currentalias != String.Empty)
            {
                Function editedfunction = _currentfunctions[currentalias];
                _currentfunctions.Remove(currentalias);
                editedfunction.Alias = CurrentFunctionsGrid.Rows[e.RowIndex].Cells[0].Value.ToString();
                _currentfunctions.Add(editedfunction);
                currentalias = String.Empty;
            }
        }

        private void CurrentFunctionsGrid_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.ColumnIndex > -1 &&
                CurrentFunctionsGrid.Columns[e.ColumnIndex].Name == "Configure")
            {
                // The user has clicked the configure button, so configure the selected row
                editingalias = CurrentFunctionsGrid.Rows[e.RowIndex].Cells["Alias"].Value.ToString();

                // Add the configuration control
                AddInConfigurationPanel.Visible = true;
                FunctionConfigurationPanel.Controls.Clear();
                _configurationcontrol = AddInManager.Functions[_currentfunctions[editingalias].FunctionID].ConfigurationInstance;
                _configurationcontrol.SetConfiguration(_currentfunctions[editingalias].FunctionConfiguration);                
                FunctionConfigurationPanel.Controls.Add(_configurationcontrol);
                CurrentFunctionListPanel.Visible = false;
                NewFunctionTopPanel.Visible = false;                
            }
        }

        private void CurrentFunctionsGrid_CellBeginEdit(object sender, DataGridViewCellCancelEventArgs e)
        {
            if (CurrentFunctionsGrid.Columns[e.ColumnIndex].Name == "Alias")
            {
                currentalias = CurrentFunctionsGrid.Rows[e.RowIndex].Cells[0].Value.ToString();
            }
        }
    }
}
