/*
 * This source file (ManageCubeSettings.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;
using System.Linq;

namespace Bamboo.Cubed.Designer
{
    /// <summary>
    /// This control allows for modified the settings of a cube
    /// </summary>
    public partial class ManageCubeSettings : UserControl
    {
        /// <summary>
        /// This initialises a new instance of the Manage Cube Settings control
        /// </summary>     
        /// <param name="editingcube">The cube being edited</param>
        /// <param name="users">The list of users available in the system</param>
        public ManageCubeSettings(Cube editingcube, List<User> users)
        {
            InitializeComponent();
            _editingcube = editingcube;
            HeaderLabel.Text = "Manage cube settings for: " + _editingcube.Name;
            _users = users;

            // Prepare our storage modes
            CubeStorageMode.Items.Clear();
            foreach (string thismode in Enum.GetNames(typeof(CubeStorageModes)))
                CubeStorageMode.Items.Add(thismode);
            
            // Initialise our screen
            CubeName.Text = _editingcube.Name;
            CubeDescription.Text = _editingcube.Description;
            CubeStorageMode.Text = _editingcube.StorageMode.ToString();
            AddNotificationUser.Items.Clear();
            foreach (User thisuser in _users)
                AddNotificationUser.Items.Add(thisuser.FullName);
            NotificationUsers.Items.Clear();
            foreach (User thisuser in _editingcube.NotificationUsers)
                NotificationUsers.Items.Add(thisuser.FullName);
        }

        /// <summary>
        /// The collection of users available in the system
        /// </summary>
        private List<User> _users;

        /// <summary>
        /// This prepares the cube for being saved back to the database
        /// </summary>
        /// <returns>True if the cube was prepared, false if not</returns>
        public bool PrepareSave()
        {
            // Check field lengths
            if (CubeName.Text != String.Empty && CubeDescription.Text != String.Empty)
            {
                _editingcube.Name = CubeName.Text;
                _editingcube.Description = CubeDescription.Text;
                _editingcube.StorageMode = (CubeStorageModes)Enum.Parse(typeof(CubeStorageModes),
                    CubeStorageMode.Text);
                _editingcube.NotificationUsers.Clear();
                foreach (string thisuser in NotificationUsers.Items)
                    _editingcube.NotificationUsers.Add(_users.FirstOrDefault(testuser =>
                        testuser.FullName == thisuser));
                return true;
            }
            else 
                return false;
        }

        private Cube _editingcube;
        /// <summary>
        /// The cube being edited
        /// </summary>
        public Cube EditingCube
        {
            get { return _editingcube; }
            set { _editingcube = value; }
        }

        private void AddUserButton_Click(object sender, EventArgs e)
        {
            if (AddNotificationUser.Text != String.Empty && !NotificationUsers.Items.Contains(AddNotificationUser.Text))
            {
                NotificationUsers.Items.Add(AddNotificationUser.Text);
            }
            else
                MessageBox.Show("Please select a user to add.", "No user selected", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
        }

        private void RemoveUserButton_Click(object sender, EventArgs e)
        {
            if (NotificationUsers.SelectedItems.Count > 0)
                NotificationUsers.Items.RemoveAt(NotificationUsers.SelectedIndices[0]);
        }
    }
}
