/*
 * This source file (ManageMemberSettings.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.AddIns;

namespace Bamboo.Cubed.Designer
{
    /// <summary>
    /// This control allows for modifying the settings of a member
    /// </summary>
    public partial class ManageMemberSettings : UserControl
    {
        /// <summary>
        /// This initialises a new instance of the Manage Member Settings control
        /// </summary>     
        /// <param name="editingcube">The cube being edited</param>
        /// <param name="dimensionid">The unique identifier of the dimension being edited</param>
        /// <param name="memberid">The unique identifier of the member being edited</param>
        public ManageMemberSettings(Cube editingcube, Guid dimensionid, Guid memberid)
        {
            InitializeComponent();
            _isloading = true;
            _editingcube = editingcube;
            _dimensionid = dimensionid;
            _memberid = memberid;
            HeaderLabel.Text = "Manage member settings for: " + _editingcube.Dimensions[dimensionid].Members[memberid].Name;

            // Prepare our drop downs
            MemberSignage.Items.Clear();
            foreach (string thissignage in Enum.GetNames(typeof(Signage)))
                MemberSignage.Items.Add(thissignage);
            if (MemberSignage.Items.Count > 0)
                MemberSignage.SelectedIndex = 0;

            MemberAggregationType.Items.Clear();
            foreach (string thisaggregationtype in Enum.GetNames(typeof(AggregationTypes)))
                MemberAggregationType.Items.Add(thisaggregationtype);
            if (MemberAggregationType.Items.Count > 0)
                MemberAggregationType.SelectedIndex = 0;

            MemberConsolidationType.Items.Clear();
            foreach (string thisconsolidationtype in Enum.GetNames(typeof(ConsolidationTypes)))
                MemberConsolidationType.Items.Add(thisconsolidationtype);
            if (MemberConsolidationType.Items.Count > 0)
                MemberConsolidationType.SelectedIndex = 0;
            
            // Initialise our screen
            MemberName.Text = _editingcube.Dimensions[dimensionid].Members[memberid].Name;
            MemberDescription.Text = _editingcube.Dimensions[dimensionid].Members[memberid].Description;
            MemberRollup.Checked = _editingcube.Dimensions[dimensionid].Members[memberid].RollUp;
            MemberSignage.Text = _editingcube.Dimensions[dimensionid].Members[memberid].Sign.ToString();
            MemberConsolidationType.Text = _editingcube.Dimensions[dimensionid].Members[memberid].Consolidation.ToString();
            MemberAggregationType.Text = _editingcube.Dimensions[dimensionid].Members[memberid].Aggregation.ToString();
            MemberIsCurrent.Checked = _editingcube.Dimensions[dimensionid].Members[memberid].IsCurrent;
            MemberDisplayOrder.Value = Convert.ToDecimal(_editingcube.Dimensions[dimensionid].Members[memberid].DisplayOrder);
            MemberDisplayOrder.Enabled = !_editingcube.Dimensions[dimensionid].AutoOrder;

            if (_editingcube.Dimensions[_dimensionid].MemberType == MemberTypes.Filter)
            {
                // Enable the filter based options
                MemberFilterField.Enabled = true;
                MemberUniqueKey.Enabled = false;
                MemberUniqueKey.Text = String.Empty;
                
                // Determine the type of filter
                if (_editingcube.Dimensions[_dimensionid].Members[_memberid].Filter.FilterType == FilterTypes.List)
                {
                    // Enable the list filter options
                    FilterListCheckBox.Enabled = true;
                    FilterRangeCheckBox.Enabled = true;
                    FilterListCheckBox.Checked = true;

                    AddNewValue.Enabled = true;
                    AddNewValue.Text = String.Empty;
                    FilterValuesList.Enabled = true;
                    FilterValuesList.Items.Clear();
                    foreach (string thisfiltervalue in _editingcube.Dimensions[_dimensionid].Members[_memberid].Filter.AcceptableValues)
                        FilterValuesList.Items.Add(thisfiltervalue);                    
                    AddFilterValueButton.Enabled = true;
                    RemoveValueButton.Enabled = true;

                    // Disable the range filter options                    
                    UpperLimit.Enabled = false;
                    UpperLimit.Text = String.Empty;
                    LowerLimit.Enabled = false;
                    LowerLimit.Text = String.Empty;
                }
                else
                {
                    // Enable the range filter options
                    FilterListCheckBox.Enabled = true;
                    FilterRangeCheckBox.Enabled = true;
                    FilterRangeCheckBox.Checked = true;

                    UpperLimit.Enabled = true;
                    UpperLimit.Text = _editingcube.Dimensions[_dimensionid].Members[_memberid].Filter.UpperValue;
                    LowerLimit.Enabled = true;
                    LowerLimit.Text = _editingcube.Dimensions[_dimensionid].Members[_memberid].Filter.LowerValue;

                    // Disable the list filter options
                    AddNewValue.Enabled = false;
                    AddNewValue.Text = String.Empty;
                    FilterValuesList.Enabled = false;
                    FilterValuesList.Items.Clear();                                        
                    AddFilterValueButton.Enabled = false;
                    RemoveValueButton.Enabled = false;
                }                
            }
            else
            {
                // Enable the identifier based options
                MemberTable.Enabled = true;
                MemberUniqueKey.Text = String.Join(";", _editingcube.Dimensions[_dimensionid].Members[_memberid].PrimaryKey.ToArray());

                // Disable the filter based options
                MemberFilterField.Enabled = false;
                MemberFilterField.Text = String.Empty;
                AddNewValue.Enabled = false;
                AddNewValue.Text = String.Empty;
                FilterValuesList.Enabled = false;
                FilterValuesList.Items.Clear();
                UpperLimit.Enabled = false;
                UpperLimit.Text = String.Empty;
                LowerLimit.Enabled = false;
                LowerLimit.Text = String.Empty;
                FilterRangeCheckBox.Enabled = false;
                FilterListCheckBox.Enabled = false;
                AddFilterValueButton.Enabled = false;
                RemoveValueButton.Enabled = false;
            }

            // Prepare our filter and combo boxes
            UpdateCombos();
            
            _isloading = false;
        }

        /// <summary>
        /// This prepares the member for being saved back to the database
        /// </summary>
        /// <returns>True if the member was prepared, false if not</returns>
        public bool PrepareSave()
        {
            // Check field lengths
            if (MemberName.Text != String.Empty && MemberDescription.Text != String.Empty)
            {
                _editingcube.Dimensions[_dimensionid].Members[_memberid].Name = MemberName.Text;
                _editingcube.Dimensions[_dimensionid].Members[_memberid].Description = MemberDescription.Text;
                _editingcube.Dimensions[_dimensionid].Members[_memberid].RollUp = MemberRollup.Checked;
                _editingcube.Dimensions[_dimensionid].Members[_memberid].Sign = (Signage)Enum.Parse(typeof(Signage), MemberSignage.Text);
                _editingcube.Dimensions[_dimensionid].Members[_memberid].Consolidation = (ConsolidationTypes)Enum.Parse(typeof(ConsolidationTypes), MemberConsolidationType.Text);
                _editingcube.Dimensions[_dimensionid].Members[_memberid].Aggregation = (AggregationTypes)Enum.Parse(typeof(AggregationTypes), MemberAggregationType.Text);
                _editingcube.Dimensions[_dimensionid].Members[_memberid].IsCurrent = MemberIsCurrent.Checked;
                _editingcube.Dimensions[_dimensionid].Members[_memberid].DisplayOrder = Convert.ToInt32(MemberDisplayOrder.Value);
                if (_editingcube.Dimensions[_dimensionid].MemberType == MemberTypes.Filter)
                {
                    // This is a filter type member
                    _editingcube.Dimensions[_dimensionid].Members[_memberid].Tablename = MemberTable.Text;
                    _editingcube.Dimensions[_dimensionid].Members[_memberid].Filter.FieldName = MemberFilterField.Text;
                    if (FilterListCheckBox.Checked)
                    {
                        // This is a list filter
                        _editingcube.Dimensions[_dimensionid].Members[_memberid].Filter.FilterType = FilterTypes.List;
                        _editingcube.Dimensions[_dimensionid].Members[_memberid].Filter.AcceptableValues.Clear();
                        foreach (string thisfiltervalue in FilterValuesList.Items)
                            _editingcube.Dimensions[_dimensionid].Members[_memberid].Filter.AcceptableValues.Add(thisfiltervalue);
                    }
                    else
                    {
                        // This is a range filter
                        _editingcube.Dimensions[_dimensionid].Members[_memberid].Filter.FilterType = FilterTypes.Range;
                        _editingcube.Dimensions[_dimensionid].Members[_memberid].Filter.LowerValue = LowerLimit.Text;
                        _editingcube.Dimensions[_dimensionid].Members[_memberid].Filter.UpperValue = UpperLimit.Text;
                    }
                }
                else
                {
                    // This is an identifier type member
                    _editingcube.Dimensions[_dimensionid].Members[_memberid].Tablename = MemberTable.Text;
                    _editingcube.Dimensions[_dimensionid].Members[_memberid].PrimaryKey = new List<string>(MemberUniqueKey.Text.Split(';'));
                }
                return true;
            }
            else 
                return false;
        }

        private bool _isloading;

        private Cube _editingcube;
        /// <summary>
        /// The cube being edited
        /// </summary>
        public Cube EditingCube
        {
            get { return _editingcube; }
            set { _editingcube = value; }
        }

        private Guid _dimensionid;
        /// <summary>
        /// This is the unique identifier of the dimension being edited
        /// </summary>
        public Guid DimensionID
        {
            get { return _dimensionid; }
            set { _dimensionid = value; }
        }

        private Guid _memberid;
        /// <summary>
        /// This is the unique identifier of the member being edited
        /// </summary>
        public Guid MemberID
        {
            get { return _memberid; }
            set { _memberid = value; }
        }

        private void MemberTable_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!_isloading)
            {
                // Our table has changed, so update our fields
                if (MemberTable.SelectedIndex > 0)
                    _editingcube.Dimensions[_dimensionid].Members[_memberid].Tablename = MemberTable.Text;

                // Grab our database provider to select our fields lists
                CubedDatabaseConnector dbprovider = AddInManager.Connectors[_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource].Provider.ConnectorID].ConnectorInstance;
                dbprovider.SetConfiguration(_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource].Provider.ConnectorConfiguration);
                dbprovider.Open();

                // Update our fields lists
                UpdateTableFields(dbprovider);

                // Close our provider
                dbprovider.Rollback();
            }
        }

        /// <summary>
        /// This updates our combo boxes used
        /// </summary>
        private void UpdateCombos()
        {
            // Connect to our data provider and update our field list
            try
            {
                // Now update our selected database table
                CubedDatabaseConnector dbprovider = AddInManager.Connectors[_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource].Provider.ConnectorID].ConnectorInstance;
                dbprovider.SetConfiguration(_editingcube.Datasources[_editingcube.Dimensions[_dimensionid].Datasource].Provider.ConnectorConfiguration);
                dbprovider.Open();
                MemberTable.DataSource = dbprovider.DatabaseTables();

                if (_editingcube.Dimensions[_dimensionid].Members[_memberid].Tablename != String.Empty &&
                    MemberTable.Items.Contains(_editingcube.Dimensions[_dimensionid].Members[_memberid].Tablename))
                    MemberTable.Text = _editingcube.Dimensions[_dimensionid].Members[_memberid].Tablename;
                else if (MemberTable.Items.Count > 0)
                    MemberTable.SelectedIndex = 0;
                else
                    MemberTable.SelectedIndex = -1;

                // And then update our database fields
                UpdateTableFields(dbprovider);

                // Close our database connection
                dbprovider.Rollback();
            }
            catch
            {
                MemberTable.DataSource = null;
                MemberFilterField.DataSource = null;
            }
        }

        /// <summary>
        /// This updates the fields drop downs based on the selected table
        /// </summary>
        /// <param name="dbprovider">The database provider to connect to the source database with</param>
        private void UpdateTableFields(CubedDatabaseConnector dbprovider)
        {
            // This updates all table fields when a new database table has been chosen
            if (MemberTable.SelectedIndex > -1 && MemberTable.Text != String.Empty)
            {
                // Get our list of available fields from the database provider                
                List<string> availablefilterfields = dbprovider.DatabaseTableFields(MemberTable.Text);

                // Set our drop down values
                if (availablefilterfields.Count > 0)
                {
                    // Check to see if we already have values                    
                    MemberFilterField.DataSource = availablefilterfields;
                    if (_editingcube.Dimensions[_dimensionid].Members[_memberid].Filter.FieldName != String.Empty &&
                            availablefilterfields.Contains(_editingcube.Dimensions[_dimensionid].Members[_memberid].Filter.FieldName))
                        MemberFilterField.Text = _editingcube.Dimensions[_dimensionid].Members[_memberid].Filter.FieldName;
                    else
                        MemberFilterField.Text = String.Empty;
                }
                else
                {
                    // There are no database fields                    
                    MemberFilterField.DataSource = null;
                }
            }
            else
            {
                MemberFilterField.DataSource = null;
            }
        }

        private void FilterListCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (FilterListCheckBox.Checked)
            {
                // Enable our values list
                FilterValuesList.Enabled = true;
                FilterValuesList.Items.Clear();
                AddFilterValueButton.Enabled = true;
                RemoveValueButton.Enabled = true;
                AddNewValue.Text = String.Empty;
                AddNewValue.Enabled = true;

                // Disable our range selections
                UpperLimit.Enabled = false;
                UpperLimitLabel.Enabled = false;
                UpperLimit.Text = String.Empty;
                LowerLimit.Enabled = false;
                LowerLimitLabel.Enabled = false;
                LowerLimit.Text = String.Empty;
            }
            else
            {
                // Disable our values list
                FilterValuesList.Enabled = false;
                FilterValuesList.Items.Clear();
                AddFilterValueButton.Enabled = false;
                RemoveValueButton.Enabled = false;
                AddNewValue.Text = String.Empty;
                AddNewValue.Enabled = false;

                // Enable our range selections
                UpperLimit.Enabled = true;
                UpperLimitLabel.Enabled = true;
                UpperLimit.Text = String.Empty;
                LowerLimit.Enabled = true;
                LowerLimitLabel.Enabled = true;
                LowerLimit.Text = String.Empty;
            }
        }

        private void FilterRangeCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (FilterListCheckBox.Checked)
            {
                // Enable our values list
                FilterValuesList.Enabled = true;
                FilterValuesList.Items.Clear();
                AddFilterValueButton.Enabled = true;
                RemoveValueButton.Enabled = true;
                AddNewValue.Text = String.Empty;
                AddNewValue.Enabled = true;

                // Disable our range selections
                UpperLimit.Enabled = false;
                UpperLimitLabel.Enabled = false;
                UpperLimit.Text = String.Empty;
                LowerLimit.Enabled = false;
                LowerLimitLabel.Enabled = false;
                LowerLimit.Text = String.Empty;
            }
            else
            {
                // Disable our values list
                FilterValuesList.Enabled = false;
                FilterValuesList.Items.Clear();
                AddFilterValueButton.Enabled = false;
                RemoveValueButton.Enabled = false;
                AddNewValue.Text = String.Empty;
                AddNewValue.Enabled = false;

                // Enable our range selections
                UpperLimit.Enabled = true;
                UpperLimitLabel.Enabled = true;
                UpperLimit.Text = String.Empty;
                LowerLimit.Enabled = true;
                LowerLimitLabel.Enabled = true;
                LowerLimit.Text = String.Empty;
            }
        }

        private void AddFilterValueButton_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (AddNewValue.Text != String.Empty)
            {
                if (!FilterValuesList.Items.Contains(AddNewValue.Text))
                    FilterValuesList.Items.Add(AddNewValue.Text);
                else
                    MessageBox.Show("This value has already been added to the list",
                    "Value already added", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                AddNewValue.Text = String.Empty;
            }
            else
                MessageBox.Show("Please enter the new value to add to the acceptable list in the box to the left",
                    "Enter a new value to add", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }

        private void RemoveValueButton_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (FilterValuesList.SelectedIndex > -1)
            {
                FilterValuesList.Items.RemoveAt(FilterValuesList.SelectedIndex);
            }
            else
                MessageBox.Show("Please click on the value in the list you want to remove",
                    "Select value to remove", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }
    }
}
