/*
 * This source file (ManageServerSettings.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;

namespace Bamboo.Cubed.Designer
{
    /// <summary>
    /// This control allows for adding a new server to the Cubed Server list
    /// </summary>
    public partial class ManageServerSettings : UserControl
    {
        /// <summary>
        /// This initialises a new instance of the Add New Server control
        /// </summary>
        /// <param name="server">The address of the server to edit the settings of</param>
        /// <param name="structurestable">The structures table in the storage media</param>
        /// <param name="datatableprefix">The prefix applied to all data tables in the media</param>
        /// <param name="logginglevel">The level of logging used by Cubed</param>
        /// <param name="smtpserverlocation">The SMTP Server Address</param>
        /// <param name="smtpportnumber">The SMTP Port Number</param>
        /// <param name="useauthentication">Whether to use SMTP Authentication or not</param>
        /// <param name="usessl">Whether to use SSL or not</param>
        /// <param name="smtpuser">The SMTP Username (if authenticated)</param>
        /// <param name="smtppassword">The SMTP Password (if authenticated)</param>
        /// <param name="serverendpointtype">The type of the primary server binding</param>
        /// <param name="servername">The name of the primary server binding</param>
        /// <param name="serverport">The port number for the primary server binding</param>
        /// <param name="maxmemory">The maximum memory to use in megabytes</param>
        /// <param name="addinsdirectory">The directory where Cubed Add Ins are located</param>
        public ManageServerSettings(string server, string structurestable, string datatableprefix,
            LoggingLevels logginglevel, string smtpserverlocation,
            int smtpportnumber, bool usessl, bool useauthentication, string smtpuser,
            string smtppassword, ServiceEndPointType serverendpointtype, string servername,
            int serverport, int maxmemory, string addinsdirectory)
        {
            InitializeComponent();
            _loading = true;
            _server = server;
            HeaderLabel.Text = "Manage server settings for: " + _server;

            // Initialise the Logging Levels
            LoggingLevel.Items.Clear();
            foreach (string thislevel in Enum.GetNames(typeof(LoggingLevels)))
            {
                LoggingLevel.Items.Add(thislevel);
            }
            CubeLoggingLevel = logginglevel;

            // Initialise the default tables
            CubeStructuresTable = structurestable;
            CubeDataTablePrefix = datatableprefix;

            // Initialise the email settings
            SMTPServerLocation = smtpserverlocation;
            SMTPPortNumber = smtpportnumber;
            UseServerAuthentication = useauthentication;
            UseServerSSL = usessl;
            SMTPServerUsername = smtpuser;
            SMTPServerPassword = smtppassword;

            // Initialise the primary binding settings
            EndPointType.Items.Clear();
            foreach (string thistype in Enum.GetNames(typeof(ServiceEndPointType)))
            {
                EndPointType.Items.Add(thistype);
            }
            ServerEndPointType = serverendpointtype;
            ServerName = servername;
            ServerPort = serverport;

            // Initialise the server settings
            MaxMemoryUsage = maxmemory;
            CubeAddInsDirectory = addinsdirectory;
            _loading = false; 
        }

        private bool _loading;

        private string _server;
        /// <summary>
        /// This is the address of the Cubed server
        /// </summary>
        public string Server
        {
            get { return _server; }
            set { _server = value; }
        }        

        /// <summary>
        /// This is the table in the storage media to store Cube Structures
        /// </summary>
        public string CubeStructuresTable
        {
            get { return StructuresTable.Text; }
            set { StructuresTable.Text = value; }
        }

        /// <summary>
        /// This is the prefix of all tables in the storage media storing data
        /// </summary>
        public string CubeDataTablePrefix
        {
            get { return DataTablePrefix.Text; }
            set { DataTablePrefix.Text = value; }
        }

        /// <summary>
        /// The directory where Add Ins are located
        /// </summary>
        public string CubeAddInsDirectory
        {
            get { return AddInsDirectory.Text; }
            set { AddInsDirectory.Text = value; }
        }

        /// <summary>
        /// This is the level of logging used by Cubed
        /// </summary>
        public LoggingLevels CubeLoggingLevel
        {
            get { return (LoggingLevels)Enum.Parse(typeof(LoggingLevels), LoggingLevel.Text); }
            set 
            {
                LoggingLevels _value = value;
                LoggingLevel.Text = _value.ToString(); 
            }
        }

        /// <summary>
        /// This is the address of the SMTP Server on the system
        /// </summary>
        public string SMTPServerLocation
        {
            get { return SMTPServer.Text; }
            set { SMTPServer.Text = value; }
        }

        /// <summary>
        /// The SMTP Port Number of the server
        /// </summary>
        public int SMTPPortNumber
        {
            get { return Convert.ToInt32(SMTPPort.Value); }
            set { SMTPPort.Value = Convert.ToDecimal(value); }
        }

        /// <summary>
        /// This is the type of end point used by the primary service binding
        /// </summary>
        public ServiceEndPointType ServerEndPointType
        {
            get { return (ServiceEndPointType)Enum.Parse(typeof(ServiceEndPointType), EndPointType.Text); }
            set { EndPointType.Text = value.ToString(); }
        }

        /// <summary>
        /// This is the name of the primary binding server
        /// </summary>
        public string ServerName
        {
            get { return EndPointServer.Text; }
            set { EndPointServer.Text = value; }
        }

        /// <summary>
        /// This is the port of the primary binding server
        /// </summary>
        public int ServerPort
        {
            get { return Convert.ToInt32(EndPointPort.Value); }
            set { EndPointPort.Value = Convert.ToDecimal(value); }
        }

        /// <summary>
        /// Whether to use authentication to connect to the SMTP server or not
        /// </summary>
        public bool UseServerAuthentication
        {
            get { return UseAuthentication.Checked; }
            set { UseAuthentication.Checked = value; }
        }

        /// <summary>
        /// Whether to use SSL to connect to the server or not
        /// </summary>
        public bool UseServerSSL
        {
            get { return UseSSL.Checked; }
            set { UseSSL.Checked = true; }
        }

        /// <summary>
        /// The username to use to connect to the SMTP Server (if authenticated)
        /// </summary>
        public string SMTPServerUsername
        {
            get { return SMTPUser.Text; }
            set { SMTPUser.Text = value; }
        }
        /// <summary>
        /// The password to use to connect to the SMTP Server (if authenticated)
        /// </summary>
        public string SMTPServerPassword
        {
            get { return SMTPPassword.Text; }
            set { SMTPPassword.Text = value; }
        }

        /// <summary>
        /// This is the maximum memory to use (in megabytes)
        /// </summary>
        public int MaxMemoryUsage
        {
            get { return Convert.ToInt32(MaxMemory.Value); }
            set { MaxMemory.Value = Convert.ToDecimal(value); }
        }

        private void ChooseLogPathButton_MouseEnter(object sender, EventArgs e)
        {
            Cursor = Cursors.Hand;
        }

        private void ChooseLogPathButton_MouseHover(object sender, EventArgs e)
        {
            Cursor = Cursors.Hand;
        }

        private void ChooseLogPathButton_MouseLeave(object sender, EventArgs e)
        {
            Cursor = Cursors.Default;
        }

        private void UseSSL_CheckedChanged(object sender, EventArgs e)
        {
            if (!_loading)
            {
                if (UseSSL.Checked)
                    SMTPPortNumber = 465;
                else
                    SMTPPortNumber = 25;
            }
        }

        private void FindCertificateButton_MouseEnter(object sender, EventArgs e)
        {
            this.Cursor = Cursors.Hand;
        }

        private void FindCertificateButton_MouseHover(object sender, EventArgs e)
        {
            this.Cursor = Cursors.Hand;
        }

        private void FindCertificateButton_MouseLeave(object sender, EventArgs e)
        {
            this.Cursor = Cursors.Default;
        }

        private void FindAddInsButton_Click(object sender, EventArgs e)
        {
            if (AddInsDirectoryFinder.ShowDialog() == DialogResult.OK)
                AddInsDirectory.Text = AddInsDirectoryFinder.SelectedPath + "\\";
        }
    }
}
