/*
 * This source file (MemberFormulaSettings.Designer.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.AddIns;

namespace Bamboo.Cubed.Designer
{
    /// <summary>
    /// This control allows for configuring the chosen Member Formula Provider Add In
    /// </summary>
    public partial class MemberFormulaSettings : UserControl
    {
        /// <summary>
        /// This is the current configuration control 
        /// </summary>
        private CubedFormulaConfiguration _configurationcontrol;

        /// <summary>
        /// Whether the current control is loading or not
        /// </summary>
        private bool loading = false;

        /// <summary>
        /// This initialises a new instance of the Member Formula Evaluator Provider Add In
        /// </summary>
        /// <param name="currentconnector">The current storage provider</param>
        public MemberFormulaSettings(Cube editingcube, Guid dimensionid, Guid memberid)
        {
            InitializeComponent();
            loading = true;
            _editingcube = editingcube;
            _dimensionid = dimensionid;
            _memberid = memberid;
            HeaderLabel.Text = _editingcube.Dimensions[_dimensionid].Members[_memberid].Name;
            if (AddInManager.Formulas.Contains(_editingcube.Dimensions[dimensionid].Members[memberid].Calculation.EvaluatorID))
            {
                _currentevaluator = _editingcube.Dimensions[_dimensionid].Members[_memberid].Calculation;
            }
            else if (AddInManager.Formulas.Count > 0)
            {
                _currentevaluator = new FormulaEvaluator(AddInManager.Formulas[0].ID, String.Empty,
                    AddInManager.Formulas[0].CurrentVersion);
            }

            // Create the combo box
            FormulaEvaluatorSelector.Items.Clear();
            FormulaEvaluatorSelector.DataSource = AvailableProviders();
            FormulaEvaluatorSelector.DisplayMember = "Name";
            FormulaEvaluatorSelector.ValueMember = "ID";

            // Try and select the current connector
            FormulaEvaluatorSelector.SelectedValue = _currentevaluator.EvaluatorID;
            UpdateScreen();            
            loading = false;
        }

        private Cube _editingcube;
        /// <summary>
        /// The current editing cube
        /// </summary>
        public Cube EditingCube
        {
            get { return _editingcube; }
            set { _editingcube = value; }
        }

        private Guid _dimensionid;
        /// <summary>
        /// The current unique identifier of the dimension
        /// </summary>
        public Guid DimensionID
        {
            get { return _dimensionid; }
            set { _dimensionid = value; }
        }

        private Guid _memberid;
        /// <summary>
        /// The unique identifier of the member being edited
        /// </summary>
        public Guid MemberID
        {
            get { return _memberid; }
            set { _memberid = value; }
        }

        /// <summary>
        /// This updates the screen with the chosen Add-Ins configuration
        /// </summary>
        private void UpdateScreen()
        {
            // If all has loaded well, open up the configuration for the current control
            AddInPanel.Controls.Clear();
            _configurationcontrol = AddInManager.Formulas[_currentevaluator.EvaluatorID].ConfigurationInstance;
            _configurationcontrol.SetConfiguration(_currentevaluator.EvaluatorConfiguration);
            AddInPanel.Controls.Add(_configurationcontrol);
        }

        private FormulaEvaluator _currentevaluator;        
        
        private void FormulaEvaluatorSelector_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!loading)
            {
                // The database connector has changed, so update the screen
                _currentevaluator = new FormulaEvaluator(new Guid(FormulaEvaluatorSelector.SelectedValue.ToString()),
                    String.Empty, AddInManager.Formulas[new Guid(FormulaEvaluatorSelector.SelectedValue.ToString())].CurrentVersion);
                UpdateScreen();
            }
        }

        /// <summary>
        /// This saves the formula evaluator settings to the member
        /// </summary>
        public void SaveChanges()
        {
            _currentevaluator.EvaluatorConfiguration = _configurationcontrol.GetConfiguration();           
            _editingcube.Dimensions[_dimensionid].Members[_memberid].Calculation = _currentevaluator;
        }

        /// <summary>
        /// This clears the current formula evaluator settings for this member
        /// </summary>
        public bool ClearEvaluator()
        {
            if (MessageBox.Show("Are you sure you want to clear the formula settings?", "Clear settings",
                MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _editingcube.Dimensions[_dimensionid].Members[_memberid].Calculation = new FormulaEvaluator();
                return true;
            }
            return false;
        }

        /// <summary>
        /// This returns a data table of all available providers
        /// </summary>
        /// <returns>A data table of available providers from the Add In Manager</returns>
        private DataTable AvailableProviders()
        {
            DataTable AvailableAddIns = new DataTable();
            AvailableAddIns.Columns.Add("Name");
            AvailableAddIns.Columns.Add("Description");
            AvailableAddIns.Columns.Add("ID");

            foreach (CubedFormulaDescription thisevaluator in AddInManager.Formulas)
            {
                DataRow newevaluator = AvailableAddIns.NewRow();
                newevaluator["Name"] = thisevaluator.Name;
                newevaluator["Description"] = thisevaluator.Description;
                newevaluator["ID"] = thisevaluator.ID;
                AvailableAddIns.Rows.Add(newevaluator);
            }

            return AvailableAddIns;
        }
    }
}
