/*
 * This source file (StorageProviderSettings.Designer.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.AddIns;

namespace Bamboo.Cubed.Designer
{
    /// <summary>
    /// This control allows for configuring the chosen Storage Provider Add In
    /// </summary>
    public partial class StorageProviderSettings : UserControl
    {
        /// <summary>
        /// This is the current configuration control 
        /// </summary>
        private CubedDatabaseConnectorConfiguration _configurationcontrol;

        /// <summary>
        /// Whether the current control is loading or not
        /// </summary>
        private bool loading = false;

        /// <summary>
        /// This initialises a new instance of the Storage Provider Add In
        /// </summary>
        /// <param name="currentconnector">The current storage provider</param>
        public StorageProviderSettings(DatabaseConnector currentconnector)
        {
            InitializeComponent();
            loading = true;
            _currentconnector = currentconnector;

            // Create the combo box
            StorageProvider.Items.Clear();
            StorageProvider.DataSource = AvailableProviders();
            StorageProvider.DisplayMember = "Name";
            StorageProvider.ValueMember = "ID";

            // Try and select the current connector
            StorageProvider.SelectedValue = _currentconnector.ConnectorID;
            UpdateScreen();            
            loading = false;
        }

        /// <summary>
        /// This updates the screen with the chosen Add-Ins configuration
        /// </summary>
        private void UpdateScreen()
        {
            // If all has loaded well, open up the configuration for the current control
            AddInPanel.Controls.Clear();
            _configurationcontrol = AddInManager.Connectors[_currentconnector.ConnectorID].ConfigurationInstance;
            _configurationcontrol.SetConfiguration(_currentconnector.ConnectorConfiguration);
            AddInPanel.Controls.Add(_configurationcontrol);
        }

        private DatabaseConnector _currentconnector;
        /// <summary>
        /// This is the chosen Storage Provider
        /// </summary>
        public DatabaseConnector CurrentConnector
        {
            get { return _currentconnector; }
            set { _currentconnector = value; }
        }

        /// <summary>
        /// This gets the current Base64 encoded configuration of the selected Database Connector Add In
        /// </summary>
        public string ConfigurationXML
        {
            get { return _configurationcontrol.GetConfiguration(); } 
        }

        private void StorageProvider_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!loading)
            {
                // The database connector has changed, so update the screen
                _currentconnector = new DatabaseConnector(new Guid(StorageProvider.SelectedValue.ToString()),
                    String.Empty, AddInManager.Connectors[new Guid(StorageProvider.SelectedValue.ToString())].CurrentVersion);
                UpdateScreen();
            }
        }

        /// <summary>
        /// This returns a data table of all available providers
        /// </summary>
        /// <returns>A data table of available providers from the Add In Manager</returns>
        private DataTable AvailableProviders()
        {
            DataTable AvailableAddIns = new DataTable();
            AvailableAddIns.Columns.Add("Name");
            AvailableAddIns.Columns.Add("Description");
            AvailableAddIns.Columns.Add("ID");

            foreach (CubedDatabaseConnectorDescription thisconnector in AddInManager.Connectors)
            {
                DataRow newconnector = AvailableAddIns.NewRow();
                newconnector["Name"] = thisconnector.Name;
                newconnector["Description"] = thisconnector.Description;
                newconnector["ID"] = thisconnector.ID;
                AvailableAddIns.Rows.Add(newconnector);
            }

            return AvailableAddIns;
        }
    }
}
