﻿/*
 * This source file (SearchItemControl.ascx.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Engine.Desktop;
using System.Xml.Linq;
using Bamboo.Cubed.Utilities.Logging;
using Bamboo.Cubed.Utilities;

namespace Bamboo.Cubed.Desktop
{
    /// <summary>
    /// This control contains a single search item, allowing for choosing of different result viewers and
    /// adding to a users dashboard
    /// </summary>
    public partial class SearchItemControl : System.Web.UI.UserControl
    {
        #region Internal Properties

        /// <summary>
        /// Whether this is the first time this item has been loaded (so restore the saved web display
        /// settings) - or if not then the users changes (if any) are preserved
        /// </summary>
        public bool FirstLoad
        {
            get
            {
                if (Session[this.ID + "_FirstLoad"] == null)
                    Session[this.ID + "_FirstLoad"] = true;
                return (bool)Session[this.ID + "_FirstLoad"];
            }

            set
            {
                Session[this.ID + "_FirstLoad"] = value;
            }
        }

        /// <summary>
        /// The current desktop search
        /// </summary>
        public DesktopSearch CurrentSearch
        {
            get
            {
                if (Session[this.ID + "_CurrentSearch"] == null)
                    Session[this.ID + "_CurrentSearch"] = new DesktopSearch();
                return (DesktopSearch)Session[this.ID + "_CurrentSearch"];
            }

            set
            {
                Session[this.ID + "_CurrentSearch"] = value;
            }
        }

        /// <summary>
        /// The current search results
        /// </summary>
        public object SearchResults
        {
            get
            {
                return (object)Session[this.ID + "_CurrentSearchResults"];
            }
            set
            {
                Session[this.ID + "_CurrentSearchResults"] = value;
            }
        }

        /// <summary>
        /// The currently active search control
        /// </summary>
        private CubedSearch searchcontrol;

        /// <summary>
        /// The currently active result control
        /// </summary>
        private CubedResult resultcontrol;        

        #endregion

        #region Events

        /// <summary>
        /// Creates the child controls - including the dynamic search control
        /// </summary>
        protected override void OnInit(EventArgs e)
        {
            // Add the base controls
            base.OnInit(e);

            try
            {
                // Then find our search control so that we can use it to execute the results 
                if (AddInManager.SearchControls.Contains(CurrentSearch.SearchID))
                {
                    // Our search control exists so add it
                    searchcontrol = (CubedSearch)LoadControl("./bin/AddIns/" +
                        AddInManager.SearchControls[CurrentSearch.SearchID].SearchInstance.UserControlName);
                    searchcontrol.SearchID = CurrentSearch.ID;
                    searchcontrol.ID = this.ID + "_SearchControlInstance";
                    searchcontrol.ConnectionString = ((string)Session["ConnectionString"]);
                }

                // Then add our currently selected results control
                if (AddInManager.ResultControls.Contains(CurrentSearch.DefaultResult))
                {
                    // Our result control exists so add it
                    resultcontrol = (CubedResult)LoadControl("./bin/AddIns/" +
                        AddInManager.ResultControls[CurrentSearch.DefaultResult].ResultInstance.UserControlName);
                    resultcontrol.ID = this.ID + "_ResultControlInstance";
                    resultcontrol.SearchID = CurrentSearch.ID;
                    resultcontrol.DashboardMode = false;
                    ResultControlPanel.Controls.Add(resultcontrol);
                }

                // Now display the list of options of viewers available for this search
                PopulateViewerList();

                // Then, if this is the first time the control has been loaded - restore the saved defaults.
                // Otherwise don't do this as the user is permitted to change them
                if (FirstLoad)
                {
                    FirstLoad = false;
                    resultcontrol.LoadWebDisplaySettings(CurrentSearch.AvailableViewers[CurrentSearch.AvailableViewers.FirstOrDefault(thisviewer =>
                        thisviewer.ResultID == CurrentSearch.DefaultResult).ID].DisplaySettings);
                    ResultViewerList.SelectedValue = CurrentSearch.AvailableViewers[CurrentSearch.AvailableViewers.FirstOrDefault(thisviewer =>
                        thisviewer.ResultID == CurrentSearch.DefaultResult).ID].ResultID.ToString();
                }

                //Load Dashboards
                bl_dashboards.DataSource = WebDesktopManager.ReadDashboards(HttpContext.Current.User.Identity.Name).OrderBy(dash => dash.Description);
                bl_dashboards.DataBind();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        }

        /// <summary>
        /// Bind our results to our results control if they exist
        /// </summary>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            try
            {
                // Now load the search control settings from the current search
                searchcontrol.LoadWebSearch(CurrentSearch.SearchDetails);

                // Execute the results of the search and retrieve them (as this is an existing search)
                searchcontrol.SearchID = CurrentSearch.ID;
                SearchResults = searchcontrol.GetWebResults();

                // Bind our results to our search control
                resultcontrol.Results = SearchResults;

                // Now save the currently selected result control's settings in case the user wants
                // to add it to their dashboard
                CurrentSearch.AvailableViewers[CurrentSearch.AvailableViewers.FirstOrDefault(thisviewer =>
                thisviewer.ResultID == CurrentSearch.DefaultResult).ID].DisplaySettings = resultcontrol.SaveWebDisplaySettings();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        }

        /// <summary>
        /// Check whether a viewer has been selected and grab its' ID
        /// </summary>
        protected void ResultViewerList_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (ResultViewerList.SelectedIndex > -1)
                {
                    // Add the selected result to the open searches & results variables
                    WebDesktopManager.OpenSearch(HttpContext.Current.User.Identity.Name,
                        CurrentSearch.ID,
                        new Guid(ResultViewerList.SelectedValue),false);
                    //Save the item as the active view
                    Session["activeTabId"] = CurrentSearch.ID + "_" + new Guid(ResultViewerList.SelectedValue) + "_ItemTab";
                    //Reset the Selected Value so that the next time we get here it will show correctly
                    ResultViewerList.SelectedValue = CurrentSearch.AvailableViewers[CurrentSearch.AvailableViewers.FirstOrDefault(thisviewer =>
                        thisviewer.ResultID == CurrentSearch.DefaultResult).ID].ResultID.ToString();

                }
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        } 


        /// <summary>
        /// This removes the current open item from the collection of open searches/results - thereby 
        /// closing its tab
        /// </summary>
        protected void CloseItemButton_Click(object sender, EventArgs e)
        {
            try
            {
                // Grab the original item's search and result ID
                Guid itemsearchid = new Guid(this.ID.Substring(0, 36));
                Guid itemresultid = new Guid(this.ID.Substring(37, 36));

                // Then, if they exist in the open collection, remove them
                WebDesktopManager.CloseSearch(HttpContext.Current.User.Identity.Name,
                    itemsearchid, itemresultid);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        }

        /// <summary>
        /// Adds the current (saved) search item as a dashboard item to the users dashboard
        /// </summary>
        protected void bl_dashboards_Click(object sender, BulletedListEventArgs e)
        {
            try
            {
                // Create a new empty dashboard item and save it
                DesktopDashboardItem newitem = new DesktopDashboardItem(CurrentSearch.ID, CurrentSearch.DefaultResult, WebDesktopManager.ReadDashboard(new Guid(bl_dashboards.Items[e.Index].Value)), 0);
                WebDesktopManager.AddNewDashboardItem(newitem, new Guid(bl_dashboards.Items[e.Index].Value));
                AddDashboardButton.Enabled = false;
                DashboardAddedLabel.Visible = true;
                pceDashBoards.Cancel();

                // Then refresh the page to include the item on the user's dashboard
                //Response.Redirect("Default.aspx");
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        }

        /// <summary>
        /// This removes the current open item from the collection of open searches/results - thereby 
        /// closing its tab
        /// </summary>
        protected void BookmarkItemButton_Click(object sender, EventArgs e)
        {
            try
            {
                CurrentSearch.Bookmarked = !CurrentSearch.Bookmarked;
      
                // Save the search Details (AddNewSearch checks if it exists, and removes it and add it again in order to update an existing search).
                WebDesktopManager.AddNewSearch(CurrentSearch);  
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        }

        #endregion

        #region Support Functions

        /// <summary>
        /// This adds all supported viewers to the screen
        /// </summary>
        private void PopulateViewerList()
        {
            // Check whether the viewers list has already been populated and load it if it hasn't
            if (ResultViewerList.Items.Count < 1)
            {
                // It hasn't been loaded so add all supported viewers to it
                foreach (DesktopSearchResult thisresult in CurrentSearch.AvailableViewers)
                {
                    // Add the viewer to our list
                    ResultViewerList.Items.Add(new ListItem(AddInManager.ResultControls[thisresult.ResultID].Name,
                        AddInManager.ResultControls[thisresult.ResultID].ID.ToString()));
                }
            }

            // If no item has been selected, select the first
            if (ResultViewerList.SelectedIndex < 0)
                ResultViewerList.SelectedIndex = 0;
        }

        #endregion                   
    }
}