﻿/*
 * This source file (Main.aspx.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Engine;
using Bamboo.Cubed.Engine.Desktop;
using Bamboo.Cubed.AddIns;
using AjaxControlToolkit;

namespace Bamboo.Cubed.Desktop
{
    /// <summary>
    /// This is the primary web page used by the desktop
    /// </summary>
    public partial class Main : System.Web.UI.Page, INamingContainer
    {
        #region Internal Properties

        /// <summary>
        /// This contains the current Cubed connection string
        /// </summary>
        public string ConnectionString
        {
            get
            {
                if (Session["ConnectionString"] == null || Session["ConnectionString"].ToString() == String.Empty)
                {
                    // Initialise our connection
                    if (ConfigurationManager.AppSettings.AllKeys.Contains("Server") &&
                        ConfigurationManager.AppSettings.AllKeys.Contains("Port") &
                        ConfigurationManager.AppSettings.AllKeys.Contains("Protocol"))
                    {
                        Session["ConnectionString"] = "Server=" + ConfigurationManager.AppSettings["Server"] +
                            ";Protocol=" + ConfigurationManager.AppSettings["Protocol"] + ";Port=" +
                            ConfigurationManager.AppSettings["Port"] + ";User ID=" +
                            HttpContext.Current.User.Identity.Name;
                        return (string)Session["ConnectionString"];
                    }
                    else
                        return null;
                }
                else
                    return (string)Session["ConnectionString"];
            }
            set
            {
                Session["ConnectionString"] = value;
            }
        }

        #endregion

        #region Events

        /// <summary>
        /// Initialises the default settings on page load
        /// </summary>
        protected override void OnInit(EventArgs e)
        {
            //base.OnInit(e);

            // Check if we have an available, legitimate connection string - and if not, create one
            if (ConnectionString == null)
                Response.Redirect("AccessDenied.aspx");
            else
            {
                // We have a valid connection string so create a connection and assign it to our active connection
                // manager
                if (WebDesktopManager.ConnectionString == null || WebDesktopManager.ConnectionString == String.Empty)
                    WebDesktopManager.ConnectionString = ConnectionString;
            }
        }

        /// <summary>
        /// Creates all child controls ready for display
        /// </summary>
        protected override void CreateChildControls()
        {
            base.CreateChildControls();

            // Create our search controls
            foreach (CubedSearchDescription thissearch in WebDesktopManager.GetSearchCreators())
            {
                // Initialise a search control for this search
                SearchCreatorControl searcher = (SearchCreatorControl)LoadControl("SearchCreatorControl.ascx");
                searcher.ID = thissearch.ID + "_SearchControl";
                searcher.CurrentSearch = new DesktopSearch();
                searcher.CurrentSearch.SearchID = thissearch.ID;
                searcher.CurrentSearch.SearchVersion = thissearch.CurrentVersion;
                searcher.SearchHeaderLabel.Text = thissearch.Name;
                searcher.SearchDescriptionLabel.Text = thissearch.Description;

                // Now create a new tab for this search creator
                TabPanel newcreatortab = new TabPanel();
                newcreatortab.HeaderText = thissearch.Name;
                newcreatortab.ID = thissearch.ID + "_Tab";
                newcreatortab.Controls.Add(searcher);
                DesktopTabs.Tabs.AddAt(1, newcreatortab);
            }

            // Create our result viewer controls as new tabs
            LoadItemTabs();
        }

        /// <summary>
        /// This runs to check whether any new item tabs have been loaded and adds them to the collection
        /// </summary>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            // Add any extra result viewer tabs which have been loaded
            LoadItemTabs();
        }

        /// <summary>
        /// This initiates a new search
        /// </summary>
        protected void SearchButton_Click(object sender, EventArgs e)
        {
            // Grab the results from the server
            GenericKeyCollection<Guid, DesktopSearch> results = WebDesktopManager.MatchSearches(new List<string>(SearchBox.Text.Split(' ')));
            Session["SearchResults"] = results;           
            SearchResults.Results = results;
            SearchResults.DataBind();

            // Update the labels
            if (results.Count > 0)
            {
                ResultsHeaderPanel.Visible = true;
                SearchResultsCountLabel.Text = "(" + results.Count + " results found for <b>" + SearchBox.Text + "</b>.)";
            }
            else
            {
                ResultsHeaderPanel.Visible = false;
            }
        }

        #endregion

        #region Support Functions

        /// <summary>
        /// This creates the loaded results items as new tabs
        /// </summary>
        private void LoadItemTabs()
        {
            List<Guid> opensearches = new List<Guid>();
            List<Guid> openresults = new List<Guid>();
            if (Session["OpenSearches"] != null && Session["OpenSearches"] is List<Guid> &&
                Session["OpenResults"] != null && Session["OpenResults"] is List<Guid>)
            {
                // Grab the searches & results
                opensearches = (List<Guid>)Session["OpenSearches"];
                openresults = (List<Guid>)Session["OpenResults"];

                // We have one or more open searches & results, so add them
                for (int i = 0; i < opensearches.Count; i++)
                {
                    // First get the requested search
                    DesktopSearch search = WebDesktopManager.ReadSearch(opensearches[i]);

                    // Then grab the requested result
                    DesktopSearchResult result = search.AvailableViewers[openresults[i]];

                    // Then check whether it is already open (don't open it twice)
                    if (DesktopTabs.Tabs.Cast<TabPanel>().FirstOrDefault(thistab => thistab.ID == search.ID + "_ItemTab") == null)
                    {
                        // The tab collection doesn't contain this control, so add it
                        ResultViewerControl viewer = (ResultViewerControl)LoadControl("ResultViewerControl.ascx");
                        viewer.CurrentSearch = search;
                        viewer.CurrentResult = result;

                        // Set the name and description
                        viewer.ResultHeaderLabel.Text = WebDesktopManager.GetResultViewers()[result.ResultID].Name;
                        viewer.ResultDescriptionLabel.Text = WebDesktopManager.GetResultViewers()[result.ResultID].Description;

                        // Finally add a new tab to the end of the current list and add this search result into it
                        TabPanel newitemtab = new TabPanel();
                        newitemtab.HeaderText = search.Name;
                        newitemtab.ID = search.ID + "_ItemTab";
                        newitemtab.Controls.Add(viewer);
                        DesktopTabs.Tabs.Add(newitemtab);
                        DesktopTabs.ActiveTab = newitemtab;
                        MainUpdatePanel.Update();
                    }
                }
            }
            else
            {
                // At present the open searches & results session variables are not initialised so initialise them
                Session["OpenSearches"] = opensearches;
                Session["OpenResults"] = openresults;
            }
        }

        #endregion
    }
}
