﻿/*
 * This source file (CubedEngineService.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Structures.Query;
using Bamboo.Cubed.Engine;
using Bamboo.Cubed.Engine.Persistence;
using Bamboo.Cubed.Engine.XMLA;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.Utilities.Logging;
using Bamboo.Cubed.Engine.Security;
using System.IO;
using System.Data;
using System.Security.Principal;
using System.Threading;

namespace Bamboo.Cubed.Service
{
    /// <summary>
    /// This implements the Cubed Engine Service to provide functionality of Cubed
    /// </summary>
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
    public class CubedEngineService : ICubedEngineService
    {
        #region Security Methods

        /// <summary>
        /// This method returns the currently authenticated user of the system
        /// </summary>
        /// <returns>A user object with the currently authenticated user</returns>
        public User AuthenticatedUser()
        {
            return SecurityManager.AuthenticatedUser;
        }

        /// <summary>
        /// This method returns a connection parameters object which ensures the user currently connected
        /// to the web server is the same as the user passed through the connection string.
        /// </summary>
        /// <param name="connectionstring">A Cubed connection string</param>
        /// <returns>A Cubed Connection Parameters object with the users details, or - if the user
        /// is different to the user who is connected to the web server, it returns a null record.</returns>
        public ConnectionParameters CheckAuthenticatedUser(string connectionstring)
        {
            ConnectionParameters connection = new ConnectionParameters(connectionstring);
            if (connection.UserName.ToUpper().Equals(SecurityManager.AuthenticatedUser.UserName.ToUpper())
                && connection.Domain.ToUpper().Equals(SecurityManager.AuthenticatedUser.Domain.ToUpper()))
            {
                // Check if the user wants admin rights, if they do - check that they are the system
                // administrator, or they are using the service account
                if (!connection.Admin)
                {
                    // Just return the connection as the user has not requested administration rights
                    return connection;
                }
                else
                {
                    // Log the fact that an administrator has attempted to access the system
                    LoggingManager.Write("Administrator Access", "The user: " + SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " accessed the system as "
                        + "an administrator. Their connection string was: " + connectionstring, LoggingTypes.Event);

                    // Then continue to validate that the user is actually an administrator
                    if (SecurityManager.AuthenticatedUser.IsAdmin)
                        return connection;
                    else
                    {
                        // The user wants admin rights, but they are not the administrator, so return a 
                        // null object to indicate an error
                        LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                        connection = null;
                        return connection;
                    }
                }
            }
            else
            {
                // The user connected to the web server is a different user than specified in the
                // connection string, so return a null object to indicate an error
                LoggingManager.Write("Security Access Violation", "User attempted to connect to Cubed as a different identity than that which was "
                    + "authenticated. User: " + SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " was refused access, as "
                    + "they tried to connect as user: " + connection.Domain + "\\" + connection.UserName,
                    LoggingTypes.MajorError);
                connection = null;
                return connection;
            }
        }

        /// <summary>
        /// This returns all current roles within the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed Server connection string</param>
        /// <returns>The current roles in the system</returns>
        public List<Role> ReadRoles(string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return new List<Role>();
                }

                // Return the current list of roles
                return SecurityManager.Roles.List;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This returns all current users within the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed Server connection string</param>
        /// <returns>The current users in the system</returns>
        public List<User> ReadUsers(string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return new List<User>();
                }

                // Return the current list of users
                return SecurityManager.Users.List;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This saves the given roles back into the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed Server connection string</param>
        /// <param name="roles">The roles to save in the system</param>
        /// <returns>True if saved correctly, False if a problem occured</returns>
        public bool WriteRoles(string connectionstring, List<Role> roles)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return false;
                }

                GenericKeyCollection<Guid, Role> saveroles = new GenericKeyCollection<Guid,Role>();
                foreach (Role thisrole in roles)
                    saveroles.Add(thisrole);
                if (SecurityManager.SaveRoles(saveroles))
                    return true;
                else
                    return false;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This saves the given users back into the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed Server connection string</param>
        /// <param name="users">The collection of users to save into the system</param>
        /// <returns>True if saved successfully, false if a problem occured</returns>
        public bool WriteUsers(string connectionstring, List<User> users)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return false;
                }

                GenericKeyCollection<Guid, User> saveusers = new GenericKeyCollection<Guid, User>();
                foreach (User thisuser in users)
                    saveusers.Add(thisuser);
                if (SecurityManager.SaveUsers(saveusers))
                    return true;
                else
                    return false;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        #endregion

        #region XMLA Support Functions

        /// <summary>
        /// This returns the results of an XMLA Discover function in a dataset
        /// </summary>
        /// <param name="RequestType">The type of request</param>
        /// <param name="Restrictions">A set of restrictions to filter the returned data with</param>
        /// <param name="Properties">The set of properties to use</param>
        /// <returns>A DataSet containing the results of the query</returns>
        public string Discover(Structures.XMLA.RequestTypes RequestType,
            Structures.XMLA.RestrictionList Restrictions,
            Structures.XMLA.PropertyList Properties)
        {
            try
            {
                // First we read the connection string from the properties
                string connectionstring = String.Empty;
                if (Properties.ConnectionString != null)
                    connectionstring = Properties.ConnectionString;

                // Check to see if the user authentication worked. If not, return a null array to indicate an error.
                if (CheckAuthenticatedUser(connectionstring) == null) return null;

                // Read the server settings (including the URL of the Cubed Web Service)
                string AppPath = Directory.GetCurrentDirectory();

                // Return the results of the Engine's Discover method
                DataSet discoverresults = DiscoveryManager.Discover(RequestType, Restrictions, Properties, SettingsManager.Settings,
                    SecurityManager.AuthenticatedUser.DomainUserName, AppPath, new ConnectionParameters(connectionstring));

                // As the results are a tabular dataset, we need to manually serialize before returning
                StringBuilder sb = new StringBuilder();
                StringWriter sw = new StringWriter(sb);
                discoverresults.WriteXml(sw);
                sw.Close();
                return sb.ToString();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This returns the results of an XMLA Execute function to return data from a server
        /// </summary>
        /// <param name="Command">The mdXML command to run against the server</param>
        /// <param name="Properties">The set of properties to use</param>
        /// <returns>An object containing either a multi-dimensional dataset or simple rowset
        /// with the results of the query</returns>
        public object Execute(Structures.XMLA.Command Command,
            Structures.XMLA.PropertyList Properties)
        {
            try
            {
                // First we read the connection string from the properties
                string connectionstring = String.Empty;
                if (Properties.ConnectionString != null)
                    connectionstring = Properties.ConnectionString;

                // Check to see if the user authentication worked. If not, return a null array to indicate an error.
                if (CheckAuthenticatedUser(connectionstring) == null) return null;

                // Return the results of the Engine's Execute method            
                object o = ExecutionManager.Execute(Command, Properties, SettingsManager.Settings, SecurityManager.AuthenticatedUser.DomainUserName,
                    CubeManager.ReadCubes(), true);

                // Determine if the result is a flattened, tabular data set or a multi-dimensional dataset. If it
                // is a flattened data set we need to manually build the XML to return as we cannot pass the
                // object through the XML serializer
                if (o is DataSet)
                {
                    // The returned result is a DataSet so we must serialise it to a string
                    StringBuilder sb = new StringBuilder();
                    StringWriter sw = new StringWriter(sb);
                    ((DataSet)o).WriteXml(sw);
                    sw.Close();
                    return sb.ToString();
                }
                else
                    return XMLManager<Structures.XMLA.MDDataSet>.SerialiseXML(((Structures.XMLA.MDDataSet)o));
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        #endregion

        #region Relational Support Functions

        /// <summary>
        /// This executes the given command against a relational data source used by a cube
        /// </summary>
        /// <param name="Command">The SQL (relational) command to execute</param>
        /// <param name="Properties">The properties of the command, which must include the name of the cube
        /// and the name of the data source info (datasource) to use on the cube</param>
        /// <returns>Either an MDDataSet or DataSet depending on the properties set</returns>
        public object ExecuteRelational(Structures.XMLA.Command Command,
            Structures.XMLA.PropertyList Properties)
        {
            try
            {
                // First we read the connection string from the properties
                string connectionstring = String.Empty;
                if (Properties.ConnectionString != null)
                    connectionstring = Properties.ConnectionString;

                // Check to see if the user authentication worked. If not, return a null array to indicate an error.
                if (CheckAuthenticatedUser(connectionstring) == null) return null;

                // Check to ensure the command specified is not an UPDATE, DELETE or SELECT command
                if (Command.Statement.ToUpper().Contains("INSERT") ||
                    Command.Statement.ToUpper().Contains("UPDATE") ||
                    Command.Statement.ToUpper().Contains("DELETE"))
                    return null;

                // Return the results of the Engine's Execute method                                         
                object o = ExecutionManager.Execute(Command, Properties, SettingsManager.Settings, SecurityManager.AuthenticatedUser.DomainUserName,
                    CubeManager.ReadCubes(), false);

                // Determine if the result is a flattened, tabular data set or a multi-dimensional dataset. If it
                // is a flattened data set we need to manually build the XML to return as we cannot pass the
                // object through the XML serializer
                if (o is DataSet)
                {
                    // The returned result is a DataSet so we must serialise it to a string
                    StringBuilder sb = new StringBuilder();
                    StringWriter sw = new StringWriter(sb);
                    ((DataSet)o).WriteXml(sw);
                    sw.Close();
                    return sb.ToString();
                }
                else
                    return XMLManager<Structures.XMLA.MDDataSet>.SerialiseXML(((Structures.XMLA.MDDataSet)o));
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }


        #endregion

        #region Data Retrieval Functions

        /// <summary>
        /// This returns the node which represents the collection of members and measure given
        /// </summary>
        /// <param name="connectionstring">A valid connection string to connect to the Cubed Server</param>
        /// <param name="cubeid">The unique identifier of the cube to return data for</param>
        /// <param name="measure">The unique identifier of the measure to return data for</param>
        /// <param name="members">The list of members to return data for</param>
        /// <returns>A node with data</returns>
        public Node GetNode(string connectionstring, Guid cubeid, Guid measure, List<Guid> members)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                return MatrixManager.GetNode(cubeid, measure, members, false);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This gets a data matrix containing values for the selected members for each measure in the cube
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="cubeid">The unique identifier of the cube</param>
        /// <param name="members">The selected members</param>
        /// <returns>A data matrix containing the values for each measure</returns>
        public DataMatrix GetMeasureMatrix(string connectionstring, Guid cubeid, List<Guid> members)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                return MatrixManager.GetMeasureMatrix(cubeid, members, false);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This gets a data matrix containing values for the selected measure and members pivoting on the
        /// remaining dimension
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="cubeid">The unique identifier of the cube</param>
        /// <param name="measure">The measure to read</param>
        /// <param name="otherselectedmembers">The selected members on all dimensions except the pivot dimension</param>
        /// <returns>The given data matrix</returns>
        public DataMatrix GetMemberMatrix(string connectionstring, Guid cubeid, Guid measure, List<Guid> otherselectedmembers)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                return MatrixManager.GetMemberMatrix(cubeid, measure, otherselectedmembers, false);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This returns a single cube from the server, based on the ID specified
        /// </summary>
        /// <param name="connectionstring">A valid connection string to connect to the Cubed Server</param>
        /// <param name="cubeid">The unique identifier of the cube to return</param>
        /// <returns>The requested cube if it can be found, or null if it can't</returns>
        public Cube ReadCube(string connectionstring, Guid cubeid)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                // Check whether the user has access to this cube
                Cube cube = CubeManager.ReadCube(cubeid);
                if (SecurityManager.CanReadCube(cube))
                    return cube;
                else
                    return null;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This returns an array of all the cubes currently published
        /// to the Cubed server, which the authenticated user has access to
        /// </summary>
        /// <param name="connectionstring"> A valid connection string to connect to the Cubed Server</param>
        /// <returns>An array of Cubed Cubes</returns>
        public List<Cube> ReadCubes(string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                // Read the list of cubes from the server
                List<Cube> readcubes = CubeManager.ReadCubes();

                // Then check which cubes this user has permission to access
                List<Cube> permittedcubes = new List<Cube>();
                foreach (Cube thiscube in readcubes)
                    if (SecurityManager.CanReadCube(thiscube))
                        permittedcubes.Add(thiscube);

                return permittedcubes;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This returns an array of all the cube headers currently published
        /// to the Cubed server, which the authenticated user has access to
        /// </summary>
        /// <param name="connectionstring"> A valid connection string to connect to the Cubed Server</param>
        /// <returns>An array of Cubed Cubes</returns>
        public List<Cube> ReadCubeHeaders(string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                // Return our cubes
                List<Cube> readcubes = CubeManager.ReadCubes();

                // Then check which cubes this user has permission to access
                List<Cube> permittedcubes = new List<Cube>();
                foreach (Cube thiscube in readcubes)
                    if (SecurityManager.CanReadCube(thiscube))
                        permittedcubes.Add(thiscube);

                return permittedcubes;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        #endregion

        #region Data Detailed Retrieval Functions

        /// <summary>
        /// This reads the "shell" of the given cube and returns it's details, without measures, dimensions or
        /// datasources
        /// </summary>
        /// <param name="cubeid">The unique identifier of the cube to retrieve</param>
        /// <param name="connectionstring">The Cubed connection string to use</param>
        /// <returns>The cube requested, or null if an error occured</returns>
        public Cube ReadCubeDetails(Guid cubeid, string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                // Check whether the user has access to the cube
                Cube cube = CubeManager.ReadCubeDetails(cubeid);
                if (cube != null && cube.ID.Equals(cubeid) && (SecurityManager.CanReadCube(cube)))
                    return cube;
                else
                    return null;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This returns the datasource with the unique identifier given
        /// </summary>
        /// <param name="datasourceid">The unique identifier of the datasource to return</param>
        /// <param name="connectionstring">The connection string to connect to Cubedwith</param>
        /// <returns>The datasource if valid, else null</returns>
        public Datasource ReadDatasource(Guid datasourceid, string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                Datasource datasource = CubeManager.ReadDatasource(datasourceid);
                if (datasource != null && datasource.ID.Equals(datasourceid))
                    return datasource;
                else
                    return null;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This returns the measure with the given identifier
        /// </summary>
        /// <param name="measureid">The unique identifier of the measure to return</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>The measure if valid, else null</returns>
        public Measure ReadMeasure(Guid measureid, string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                Measure measure = CubeManager.ReadMeasure(measureid);
                if (measureid != null && measure.ID.Equals(measureid))
                    return measure;
                else
                    return null;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This returns the full dimension, with all members, requested
        /// </summary>
        /// <param name="dimensionid">The unique identifier of the dimension to return</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>The dimension if valid, else null</returns>
        public Dimension ReadDimension(Guid dimensionid, string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                Dimension dimension = CubeManager.ReadDimension(dimensionid);
                if (dimension != null && dimension.ID.Equals(dimensionid))
                    return dimension;
                else
                    return null;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// The "shell" of the dimension requested, without any members
        /// </summary>
        /// <param name="dimensionid">The unique identifier of the dimension to return</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>The dimension without members if valid, else null</returns>
        public Dimension ReadDimensionDetails(Guid dimensionid, string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                Dimension dimension = CubeManager.ReadDimensionDetails(dimensionid);
                if (dimensionid != null && dimension.ID.Equals(dimensionid))
                    return dimension;
                else
                    return null;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        #endregion

        #region Data Saving Functions

        /// <summary>
        /// This saves the given node back to the data matrix
        /// </summary>
        /// <param name="connectionstring">A valid connection string to connect to the Cubed Server</param>
        /// <param name="cube">The cube for which data is being written back to</param>
        /// <param name="node">The node to save</param>
        /// <returns>True if the data was successfully saved, False if not</returns>
        public bool SetNode(string connectionstring, Cube cube, Node node)
        {
            // Check to see if the user authentication worked. If not, return false to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // Save the node
                if (!MatrixManager.SetNode(node, cube.ID, false))
                    // Return false as the node was not set correctly
                    return false;
                else
                    // Otherwise return true as a node ID was successfully written
                    return true;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This publishes an array of cubes to the server. Any cubes which currently exist will be updated to reflect the changes,
        /// and any cubes which don't currently exist will be added.
        /// </summary>
        /// <param name="connectionstring"> A valid connection string to connect to the Cubed Server</param>
        /// <param name="cubes"> An array of Cubed cubes to publish to the server</param>        
        /// <returns> True if the cubes were published successfully, False if there was a problem</returns>
        public bool PublishCubes(string connectionstring, List<Cube> cubes)
        {
            // Check to see if the user authentication worked. If not, return false to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return false;
                }

                // Save the cubes
                return CubeManager.SaveCubes(cubes);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This saves a cube to the server. If the cube already exists, it will be updated; if it does not exist it will
        /// be added.
        /// </summary>
        /// <param name="connectionstring"> A valid connection string to connect to the Cubed Server</param>
        /// <param name="cube">A cube object to add to the Server</param>        
        /// <returns>The unique identifier of the new cube (or the cube which was modified)</returns>
        public Guid SaveCube(string connectionstring, Cube cube)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return new Guid();

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return new Guid();
                }

                if (CubeManager.SaveCube(cube))
                    return cube.ID;
                else
                    return new Guid();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return new Guid();
            }
        }

        #endregion

        #region Data Detailed Saving Functions

        /// <summary>
        /// This saves the details of the given cube, without datasources, measures or dimensions, to the 
        /// database
        /// </summary>
        /// <param name="cube">The cube to save</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>True if saved, else false</returns>
        public bool SaveCubeDetails(Cube cube, string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return false to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return false;
                }

                return CubeManager.SaveCubeDetails(cube);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This saves the given datasource to the database
        /// </summary>
        /// <param name="datasource">The datasource to save</param>
        /// <param name="cubeid">The unique identifier of the cube this datasource belongs in</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>True if saved, else false</returns>
        public bool SaveDatasource(Datasource datasource, Guid cubeid, string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return false to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return false;
                }

                return CubeManager.SaveDatasource(datasource, cubeid);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This saves the given measure to the database
        /// </summary>
        /// <param name="measure">The measure to save</param>
        /// <param name="cubeid">The unique identifier of the cube this measure belongs to</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>True if saved, else false</returns>
        public bool SaveMeasure(Measure measure, Guid cubeid, string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return false to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return false;
                }

                return CubeManager.SaveMeasure(measure, cubeid);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This saves the given dimension, and all of it's members, to the database
        /// </summary>
        /// <param name="dimension">The dimension to save</param>
        /// <param name="cubeid">The unique identifier of the cube this dimension belongs to</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>True if saved, else false</returns>
        public bool SaveDimension(Dimension dimension, Guid cubeid, string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return false to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return false;
                }

                return CubeManager.SaveDimension(dimension, cubeid);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This saves the given dimensions "shell", without its' members, to the database
        /// </summary>
        /// <param name="dimension">The dimension to save</param>
        /// <param name="cubeid">The unique identifier of the cube to save</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>True if saved, else false</returns>
        public bool SaveDimensionDetails(Dimension dimension, Guid cubeid, string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return false to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return false;
                }

                return CubeManager.SaveDimensionDetails(dimension, cubeid);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This saves the given member to the database
        /// </summary>
        /// <param name="member">The member to save</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>True if saved, false if not</returns>
        public bool SaveMember(Member member, string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return false to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return false;
                }

                return CubeManager.SaveMember(member);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        #endregion

        #region Deletion Functions

        /// <summary>
        /// Removes the specified cube from the server (deleting all files and/or databases tables with structures and data associated
        /// with this cube)
        /// </summary>
        /// <param name="connectionstring">A valid Cubed Server connection string</param>
        /// <param name="cubeid">The unique identifier of the cube to remove</param>
        /// <returns>True if the cube was removed, False if there was a problem</returns>
        public bool RemoveCube(string connectionstring, Guid cubeid)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return false;
                }

                Cube cube = CubeManager.ReadCube(cubeid);
                if (cube != null)
                    return CubeManager.DeleteCube(cube);
                else
                    return false;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        #endregion

        #region Other Administrative Functions

        /// <summary>
        /// Adds an Operating System user to the root priviledges group on this server
        /// </summary>
        /// <param name="connectionstring">A valid Cubed Server connection string</param>
        /// <param name="newusername">The full Operating System user name (including domain) of the user to add</param>
        /// <returns>True if the user was added successfully, False if there was a problem</returns>
        public bool AddRootUser(string connectionstring, string newusername)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return false;
                }

                // Add the root user
                SettingsManager.Settings.RootUsers.Add(newusername);

                return true;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// Removes a user from the root users group on this Cubed server
        /// </summary>
        /// <param name="connectionstring">A valid Cubed Server connection string</param>
        /// <param name="username">The full Operating System account (including domain) of the user to remove from the root account</param>
        /// <returns>True if the removal was successful, False if there was a problem</returns>
        public bool RemoveRootUser(string connectionstring, string username)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return false;
                }

                // Remove the specified user
                SettingsManager.Settings.RootUsers.Remove(username);

                return true;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This reads the current settings from the server and returns them to the user
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>The current server settings</returns>
        public ServerSettings ReadSettings(string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return null to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                // return the current settings
                return SettingsManager.Settings;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// Writes the given settings back to the Cubed Server
        /// </summary>
        /// <param name="newsettings">The new settings to write</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>True if the write was successful, false if there was a problem</returns>
        public bool WriteSettings(ServerSettings newsettings, string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return false;
                }

                // return the current settings
                SettingsManager.Settings = newsettings;
                return true;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        #endregion

        #region Consolidation Functions

        /// <summary>
        /// Cancels the currently active consolidation (if any are active)
        /// </summary>
        /// <param name="connectionstring">The connection string to use to connect to the cube</param>
        public void CancelConsolidation(string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return;
                }

                // Cancel the currently active consolidation (if any are active)
                ConsolidationManager.CancelConsolidation();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return;
            }
        }

        /// <summary>
        /// Starts a background consolidation of a particular cube on the server
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="cubeid">The unique identifier of the cube to run the background consolidation for</param>
        /// <param name="pointconsolidation">Whether to perform a full or partial (point) consolidation</param>
        /// <returns>The consolidation record for the new consolidation, or null if there was a problem</returns>
        public void StartSingleConsolidation(string connectionstring, Guid cubeid, bool pointconsolidation)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return;
                }

                Cube cube = CubeManager.ReadCube(cubeid);
                if (cube != null)
                {
                    // Start the consolidation and subscribe to its' events
                    ConsolidationManager.OnConsolidationCompleted += new ConsolidationCompleted(ConsolidationManager_OnConsolidationCompleted);
                    ConsolidationManager.StartConsolidation(cubeid, pointconsolidation);
                }
                else
                    return;
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return;
            }
        }

        /// <summary>
        /// Starts a background consolidation of all cubes on the server
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>        
        /// <param name="pointconsolidation">Whether to perform a full or partial (point) consolidation</param>
        /// <returns>The consolidation record for the new consolidation, or null if there was a problem</returns>
        public void StartFullConsolidation(string connectionstring, bool pointconsolidation)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return;

            try
            {
                // This is an administrative function; if the user is not an administrator of the server, return false
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                {
                    LoggingManager.Write("Failed Administrator Access Attempt Detected.", "User: " +
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() + " attempted to access as an Administrator.",
                            LoggingTypes.MajorError);
                    return;
                }

                // Start a full consolidation, and subscribe to its' events
                ConsolidationManager.OnConsolidationCompleted += new ConsolidationCompleted(ConsolidationManager_OnConsolidationCompleted);
                ConsolidationManager.StartConsolidation(pointconsolidation);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return;
            }
        }



        /// <summary>
        /// This subscribes to the consolidation managers "completed" event and updates the given cube when
        /// a consolidation has been completed
        /// </summary>
        /// <param name="results">The results of the consolidation which has been completed</param>
        private void ConsolidationManager_OnConsolidationCompleted(ConsolidationCompletedEventArgs results)
        {
            // Check if there were any cubes processed and update their status to reflect the consolidation
            // result
            if (results.CubesProcessed)
            {
                // Now update the cube to reflect the latest consolidation
                Cube thiscube = CubeManager.ReadCube(results.Cube.ID);
                thiscube.ConsolidatingUser = results.Cube.ConsolidatingUser;
                thiscube.LastConsolidation = DateTime.Now;
                thiscube.StartConsolidation = false;
                thiscube.ConsolidationInProgress = false;
                thiscube.ConsolidationNodesProcessed = 0;
                if (results.Exception == null)
                    thiscube.ConsolidationResult = "Success";
                else if (results.WasCancelled)
                    thiscube.ConsolidationResult = "Cancelled";
                else
                    thiscube.ConsolidationResult = "Error: " + results.Exception.ToString();

                try
                {
                    // Save the cube back to the server (if possible - it may have already been saved by the
                    // active thread)
                    CubeManager.SaveCube(thiscube);

                    // Then try and send out a broadcast notification
                    if (thiscube.NotificationUsers.Count > 0)
                    {
                        List<string> useraddresses = new List<string>();
                        foreach (User thisuser in thiscube.NotificationUsers)
                            useraddresses.Add(thisuser.Email);
                        EmailManager.BroadcastEmail(useraddresses, "Consolidation of " +
                            thiscube.Name, "The consolidation of this cube was completed at " +
                            thiscube.LastConsolidation.ToString() + "." + Environment.NewLine +
                            "The result was: " + Environment.NewLine + thiscube.ConsolidationResult + "." +
                            Environment.NewLine + "The consolidation was run by: " + thiscube.ConsolidatingUser + ".",
                            SettingsManager.Settings.SMTPServer, SettingsManager.Settings.SMTPPort,
                            SettingsManager.Settings.SMTPAuthenticated, SettingsManager.Settings.SMTPSSL,
                            SettingsManager.Settings.SMTPUser, SettingsManager.Settings.SMTPPassword);
                    }
                }
                catch (Exception e1)
                {
                    // An error occurred sending the e-mail so log it as a minor error
                    LoggingManager.Write(e1, LoggingTypes.MinorError);
                }
            }
        }

        #endregion

        #region Support Functions

        /// <summary>
        /// This method returns the version of the Cubed Server
        /// </summary>
        /// <returns>Cubed Server version</returns>
        public string Version()
        {
            return Information.Version;
        }

        /// <summary>
        /// This returns the true, full build version of the Cubed Server
        /// </summary>
        /// <returns>The full build version</returns>
        public string TrueVersion()
        {
            return Information.Version;
        }

        /// <summary>
        /// This returns the full Release details of the Cubed Server
        /// </summary>
        /// <returns>The full release details</returns>
        public string Release()
        {
            return Information.Release;
        }

        /// <summary>
        /// This returns the header of the current release of the Cubed Server
        /// </summary>
        /// <returns>The header of the current release</returns>
        public string ReleaseHeader()
        {
            return Information.ReleaseHeader;
        }

        /// <summary>
        /// This returns the details of the current release of the Cubed Server
        /// </summary>
        /// <returns>The details of the current release</returns>
        public string ReleaseDetails()
        {
            return Information.ReleaseDetails;
        }

        #endregion

        #region Desktop Functionality

        /// <summary>
        /// This saves the given desktop search to the database, either overwriting the exist search if
        /// it is an edit, or creating a new search. It will attempt to save the search as a public search
        /// if the user has requested to do so, but if their rights are not high enough or if they choose
        /// to save it privately, it will save it as their private search
        /// </summary>
        /// <param name="search">The search to save</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>True if saved successfully, false if not</returns>
        public bool SaveSearch(string connectionstring, DesktopSearch search)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // Check whether the current user has chosen to save this search as private, or whether
                // they are permitted to save searches publicly or at all
                if (search.IsPublic)
                {
                    // Now check whether the search already exists
                    DesktopSearch currentsearch = ReadSearch(connectionstring, search.ID);
                    if (currentsearch != null && currentsearch.ID == search.ID)
                    {
                        // The search already exists, so we are editing it. Check whether this user has
                        // edit rights on the search
                        if (SecurityManager.AuthenticatedUser.PublicSearchRights == AccessTypes.Edit ||
                            SecurityManager.AuthenticatedUser.PublicSearchRights == AccessTypes.Delete)
                        {
                            // The user has permissed to edit the search, so save it
                            return DesktopManager.SaveSearch(search);
                        }
                        else
                            // The user has no permissed to edit searches, so return false without saving
                            return false;
                    }
                    else
                    {
                        // The search doesn't exist, so we are creating it. Check whether the current user
                        // has enough rights to create new public searches
                        if (SecurityManager.AuthenticatedUser.PublicSearchRights == AccessTypes.Create ||
                            SecurityManager.AuthenticatedUser.PublicSearchRights == AccessTypes.Edit ||
                            SecurityManager.AuthenticatedUser.PublicSearchRights == AccessTypes.Delete)
                        {
                            // The user has enough permission to create new public searches, so save it
                            return DesktopManager.SaveSearch(search);
                        }
                        else
                        {
                            // The user cannot save public searches, so check whether this user can 
                            // save it as a private search
                            if (SecurityManager.AuthenticatedUser.PrivateSearchRights == AccessTypes.Create ||
                                SecurityManager.AuthenticatedUser.PrivateSearchRights == AccessTypes.Edit ||
                                SecurityManager.AuthenticatedUser.PrivateSearchRights == AccessTypes.Delete)
                            {
                                // The user has enough permission to save it as a private search, so amend 
                                // it and save it
                                search.IsPublic = false;
                                search.PrivateUser = SecurityManager.AuthenticatedUser.DomainUserName.ToUpper();
                                return DesktopManager.SaveSearch(search);
                            }
                            else
                                // The user cannot save public or private searches, so return false without saving
                                return false;
                        }
                    }
                }
                else
                {
                    // Check whether the search already exists
                    DesktopSearch currentsearch = ReadSearch(connectionstring, search.ID);
                    if (currentsearch != null && currentsearch.ID == search.ID)
                    {
                        // The search already exists, so check to ensure it is private, and that the 
                        // current user has permissions to edit it
                        if ((!currentsearch.IsPublic && currentsearch.PrivateUser.ToUpper() ==
                            SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() &&
                            (SecurityManager.AuthenticatedUser.PrivateSearchRights == AccessTypes.Delete ||
                            SecurityManager.AuthenticatedUser.PrivateSearchRights == AccessTypes.Edit)) ||
                            SecurityManager.AuthenticatedUser.IsAdmin)
                        {
                            // The user has permission to edit the search, so update it
                            search.IsPublic = false;
                            search.PrivateUser = SecurityManager.AuthenticatedUser.DomainUserName.ToUpper();
                            return DesktopManager.SaveSearch(search);
                        }
                        else
                            // The current user either does not own the existing search, or it is a public
                            // search, so return false without saving
                            return false;
                    }
                    else
                    {
                        // This search doesn't exist, so update it with the users name and attempt to create it
                        search.IsPublic = false;
                        search.PrivateUser = SecurityManager.AuthenticatedUser.DomainUserName.ToUpper();
                        if (SecurityManager.AuthenticatedUser.PrivateSearchRights == AccessTypes.Create ||
                            SecurityManager.AuthenticatedUser.PrivateSearchRights == AccessTypes.Edit ||
                            SecurityManager.AuthenticatedUser.PrivateSearchRights == AccessTypes.Delete)
                        {
                            // The user has enough permission to save, so return the saved search
                            return DesktopManager.SaveSearch(search);
                        }
                        else
                            // The user is not permitted to save searches so return false without saving
                            return false;
                    }
                }                
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This saves the given desktop dashboard to the database
        /// </summary>
        /// <param name="dashboard">The dashboard to save</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>True if saved successfully, false if not</returns>
        public bool SaveDashboard(string connectionstring, DesktopDashboard dashboard)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                return DesktopManager.SaveDashboard(dashboard);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This saves the given search category to the database, either overwriting the existing category if
        /// it is an edit, or creating a new one.
        /// </summary>
        /// <param name="category">The search category to save</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>True if saved successfully, false if not</returns>
        public bool SaveSearchCategory(string connectionstring, SearchCategory category)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // The category already exists, so we are editing it.
                return DesktopManager.SaveSearchCategory(category);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This matches searches based on the given criteria and returns the searches
        /// </summary>
        /// <param name="criteria">The criteria to match</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>A collection of searches matched</returns>
        public GenericKeyCollection<Guid, DesktopSearch> MatchSearches(string connectionstring, List<string> criteria)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                // First we need return a list of all relevant matches (whether public or private, this 
                // users or others)
                GenericKeyCollection<Guid, DesktopSearch> searches = DesktopManager.MatchSearches(criteria);

                // Then we need to filter the list based on the current users rights
                if (SecurityManager.AuthenticatedUser.PublicSearchRights != AccessTypes.None)
                {
                    // The user can view public searches, so return all public searches and any private
                    // searches for this user
                    GenericKeyCollection<Guid, DesktopSearch> filteredsearches = new GenericKeyCollection<Guid, DesktopSearch>();
                    foreach (DesktopSearch thissearch in searches.Where(testsearch => testsearch.IsPublic ||
                        (!testsearch.IsPublic && testsearch.PrivateUser.ToUpper() ==
                        SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() &&
                        SecurityManager.AuthenticatedUser.PrivateSearchRights != AccessTypes.None)))
                        filteredsearches.Add(thissearch);
                    return filteredsearches;
                }
                else if (SecurityManager.AuthenticatedUser.PrivateSearchRights != AccessTypes.None)
                {
                    // The user has no public search rights, but they can return private searches, so
                    // return any searches which are private and owned by this user
                    GenericKeyCollection<Guid, DesktopSearch> filteredsearches = new GenericKeyCollection<Guid, DesktopSearch>();
                    foreach (DesktopSearch thissearch in searches.Where(testsearch => !testsearch.IsPublic &&
                        testsearch.PrivateUser.ToUpper() == SecurityManager.AuthenticatedUser.DomainUserName.ToUpper()))
                        filteredsearches.Add(thissearch);
                    return filteredsearches;
                }
                else
                    // This user has no public OR private search rights, so return an empty collection
                    return new GenericKeyCollection<Guid, DesktopSearch>();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This matches searches based on the given criteria and returns the searches
        /// </summary>
        /// <param name="criteria">The criteria to match</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="category">The category to filter the results.</param>
        /// <returns>A collection of searches matched</returns>
        public GenericKeyCollection<Guid, DesktopSearch> MatchSearchesFilterCategory(string connectionstring, List<string> criteria, SearchCategory category)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                // First we need return a list of all relevant matches (whether public or private, this 
                // users or others)
                GenericKeyCollection<Guid, DesktopSearch> searches = DesktopManager.MatchSearches(criteria, category);

                // Then we need to filter the list based on the current users rights
                if (SecurityManager.AuthenticatedUser.PublicSearchRights != AccessTypes.None)
                {
                    // The user can view public searches, so return all public searches and any private
                    // searches for this user
                    GenericKeyCollection<Guid, DesktopSearch> filteredsearches = new GenericKeyCollection<Guid, DesktopSearch>();
                    foreach (DesktopSearch thissearch in searches.Where(testsearch => testsearch.IsPublic ||
                        (!testsearch.IsPublic && testsearch.PrivateUser.ToUpper() ==
                        SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() &&
                        SecurityManager.AuthenticatedUser.PrivateSearchRights != AccessTypes.None)))
                        filteredsearches.Add(thissearch);
                    return filteredsearches;
                }
                else if (SecurityManager.AuthenticatedUser.PrivateSearchRights != AccessTypes.None)
                {
                    // The user has no public search rights, but they can return private searches, so
                    // return any searches which are private and owned by this user
                    GenericKeyCollection<Guid, DesktopSearch> filteredsearches = new GenericKeyCollection<Guid, DesktopSearch>();
                    foreach (DesktopSearch thissearch in searches.Where(testsearch => !testsearch.IsPublic &&
                        testsearch.PrivateUser.ToUpper() == SecurityManager.AuthenticatedUser.DomainUserName.ToUpper()))
                        filteredsearches.Add(thissearch);
                    return filteredsearches;
                }
                else
                    // This user has no public OR private search rights, so return an empty collection
                    return new GenericKeyCollection<Guid, DesktopSearch>();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This reads the search with the given identifier from the database
        /// </summary>
        /// <param name="searchid">The unique identifier of the search to read</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>The desktop search (if it exists) or an exception will be thrown</returns>
        public DesktopSearch ReadSearch(string connectionstring, Guid searchid)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                return DesktopManager.ReadSearch(searchid);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This reads the dashboard for the given user and returns it
        /// </summary>
        /// <param name="username">The user to read the dashboard for</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>The dashboard for the given user</returns>
        public DesktopDashboard ReadUserDashboard(string connectionstring, string username)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                return DesktopManager.ReadDashboard(username);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This reads all the dashboards for the given user and returns them
        /// </summary>
        /// <param name="username">The user to read the dashboard for</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>The dashboards for the given user</returns>
        public DesktopDashboard[] ReadUserDashboards(string connectionstring, string username)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                return DesktopManager.ReadDashboards(username);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This reads teh dashboard with the unique identifier given
        /// </summary>
        /// <param name="dashboardid">The unique identifier of the dashboard to return</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>The dashboard if it exists, else an exception will be thrown</returns>
        public DesktopDashboard ReadDashboard(string connectionstring, Guid dashboardid)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                return DesktopManager.ReadDashboard(dashboardid);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This reads the search with the given identifier from the database
        /// </summary>
        /// <param name="searchid">The unique identifier of the search to read</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>The desktop search (if it exists) or an exception will be thrown</returns>
        public SearchCategory ReadSearchCategory(string connectionstring, Guid categoryid)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                return DesktopManager.ReadSearchCategory(categoryid);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This reads all the search categories and returns them
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>The search categories</returns>
        public SearchCategory[] ReadSearchCategories(string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return null;

            try
            {
                return DesktopManager.ReadSearchCategories();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return null;
            }
        }

        /// <summary>
        /// This creates a new dashboard for the user
        /// </summary>
        /// <param name="dashboardid">The unique identifier of the dashboard to return</param>
        /// <param name="dashboardDescription">The description for the new dashboard</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns></returns>
        public bool CreateDashboard(string connectionstring, string username, string dashboardDescription)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                return DesktopManager.CreateDashboard(username, dashboardDescription);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This deletes the given search from the database if the user has permission to
        /// </summary>
        /// <param name="searchid">The unique identifier of the search to delete</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>True if deleted, else an exception will be thrown</returns>
        public bool DeleteSearch(string connectionstring, Guid searchid)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                // Check whether this is a private or public search by first reading it
                DesktopSearch search = ReadSearch(connectionstring, searchid);
                if (search.IsPublic)
                {
                    // Check whether the user has permissions to delete public searches
                    if (SecurityManager.AuthenticatedUser.PublicSearchRights == AccessTypes.Delete)
                        return DesktopManager.DeleteSearch(searchid);
                    else
                        // The user cannot delete public searches so return false without deleting
                        return false;
                }
                else
                {
                    // Check whether the user has permission to delete private searches
                    if (SecurityManager.AuthenticatedUser.PrivateSearchRights == AccessTypes.Delete)
                    {
                        // The user has sufficient priviledges, but we need to check to ensure it is 
                        // their search or they are an administrator
                        if (search.PrivateUser.ToUpper() == SecurityManager.AuthenticatedUser.DomainUserName.ToUpper() ||
                            SecurityManager.AuthenticatedUser.IsAdmin)
                            return DesktopManager.DeleteSearch(searchid);
                        else
                            // The user is not permitted to delete another users search, so return false without saving
                            return false;
                    }
                    else
                        // The user has no permission to delete searches so return false without deleting
                        return false;
                }                
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This deletes the given dashboard from the database
        /// </summary>
        /// <param name="dashboardid">The unique identifier of the dashboard to delete</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>True if deleted, else an exception will be thrown</returns>
        public bool DeleteDashboard(string connectionstring, Guid dashboardid)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                return DesktopManager.DeleteDashboard(dashboardid);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        /// <summary>
        /// This renames the given dashboard from the database
        /// </summary>
        /// <param name="dashboardid">The unique identifier of the dashboard to rename</param>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="newdashboardname">The new name of the dashboard</param>
        /// <returns>True if renamed, else an exception will be thrown</returns>
        public bool RenameDashboard(string connectionstring, Guid dashboardid, string newdashboardname)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                return DesktopManager.RenameDashboard(dashboardid, newdashboardname);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        #endregion       

        #region Import / Export Functionality

        /// <summary>
        /// Exports all current users of the system to a byte array ready for saving
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>A byte array ready for saving to disk</returns>
        public byte[] ExportUsers(string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return new byte[0];

            try
            {
                return SecurityManager.ExportUsers();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return new byte[0];
            }
        }

        /// <summary>
        /// Imports all users contained in the given export into the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>The number of records imported</returns>
        public int ImportUsers(string connectionstring, byte[] export)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return 0;

            try
            {
                return SecurityManager.ImportUsers(export);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return 0;
            }
        }

        /// <summary>
        /// Exports all current roles of the system to a byte array ready for saving
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>A byte array ready for saving to disk</returns>
        public byte[] ExportRoles(string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return new byte[0];

            try
            {
                return SecurityManager.ExportRoles();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return new byte[0];
            }
        }

        /// <summary>
        /// Imports all roles contained in the given export into the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>The number of records imported</returns>
        public int ImportRoles(string connectionstring, byte[] export)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return 0;

            try
            {
                return SecurityManager.ImportRoles(export);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return 0;
            }
        }

        /// <summary>
        /// Exports all current searches of the system to a byte array ready for saving
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>A byte array ready for saving to disk</returns>
        public byte[] ExportSearches(string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return new byte[0];

            try
            {
                return DesktopManager.ExportSearches();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return new byte[0];
            }
        }

        /// <summary>
        /// Imports all searches contained in the given export into the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>The number of records imported</returns>
        public int ImportSearches(string connectionstring, byte[] export)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return 0;

            try
            {
                return DesktopManager.ImportSearches(export);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return 0;
            }
        }

        /// <summary>
        /// Exports all current dashboards of the system to a byte array ready for saving
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <returns>A byte array ready for saving to disk</returns>
        public byte[] ExportDashboards(string connectionstring)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return new byte[0];

            try
            {
                return DesktopManager.ExportDashboards();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return new byte[0];
            }
        }

        /// <summary>
        /// Imports all dashboards contained in the given export into the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>The number of records imported</returns>
        public int ImportDashboards(string connectionstring, byte[] export)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return 0;

            try
            {
                return DesktopManager.ImportDashboards(export);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return 0;
            }
        }

        /// <summary>
        /// Exports the specified cube to a byte array ready for saving
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="cubeid">The unique identifier of the cube to export</param>
        /// <returns>A bute array ready for saving to disk</returns>
        public byte[] ExportCube(string connectionstring, Guid cubeid)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return new byte[0];

            try
            {
                return CubeManager.ExportCube(cubeid);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return new byte[0];
            }
        }

        /// <summary>
        /// Imports the cube in the given export into the system
        /// </summary>
        /// <param name="connectionstring">A valid Cubed connection string</param>
        /// <param name="export">A byte array containing the export</param>
        /// <returns>True if imported successfully, false if not</returns>
        public bool ImportCube(string connectionstring, byte[] export)
        {
            // Check to see if the user authentication worked. If not, return a null array to indicate an error.
            if (CheckAuthenticatedUser(connectionstring) == null) return false;

            try
            {
                return CubeManager.ImportCube(export);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
                return false;
            }
        }

        #endregion
    }
}
