﻿/*
 * This source file (Program.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceProcess;
using System.Text;
using System.Configuration.Install;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Engine.Persistence;
using Bamboo.Cubed.Utilities;
using System.IO;
using System.Threading;

namespace Bamboo.Cubed.Service
{
    /// <summary>
    /// This executes the Cubed Service and determines what needs to be run
    /// </summary>
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        static void Main(string[] args)
        {
            // Set out active component and logging level properties for logging purposes
            Information.ActiveComponent = "Cubed Service";
            Information.LoggingLevel = SettingsManager.Settings.LogLevel;

            if (args.Length > 0)
            {
                bool activeargument = false;
                foreach (string thisarg in args)
                {
                    if (thisarg.Contains("help"))
                    {
                        // Write out the help to the command line
                        Console.WriteLine("Bamboo Software's Cubed Service - Console Mode Help");
                        Console.WriteLine("This application can be installed / uninstalled / run in console mode by doing the following:");
                        Console.WriteLine("/install=\"Cubed Service\"");
                        Console.WriteLine(">> This will install the service with the name \"CubedService\".");
                        Console.WriteLine("/uninstall=\"Cubed Service\"");
                        Console.WriteLine(">> This will uninstall an existing service with the name given.");
                        Console.WriteLine("/console");
                        Console.WriteLine(">> This will run the application in console mode");
                        Console.WriteLine("/help");
                        Console.WriteLine(">> This will show the help commands (this output)");
                        activeargument = true;
                    }
                    else if (thisarg.Contains("settingslocation"))
                    {
                        // Check to see that we can split the argument in two
                        if (thisarg.Split('=').Length == 2)
                        {
                            // Then grab the second parameter (minus any quotation marks)
                            string settingslocation = thisarg.Split('=')[1].Replace("\"", "");
                            if (File.Exists(settingslocation))
                            {
                                // A valid settings file has been specified so load it
                                SettingsManager.SettingsLocation = settingslocation;
                            }
                        }
                    }
                    else if (thisarg.Contains("/install"))
                    {
                        // First see if we can split the install parameter to grab the name of the service
                        TransactedInstaller ti = new TransactedInstaller();                       
                        ProjectInstaller pi = new ProjectInstaller();                        
                        if (thisarg.Split('=').Length == 2)
                        {
                            string servicename = thisarg.Split('=')[1];
                            ((ServiceInstaller)pi.Installers[1]).ServiceName = servicename;
                            ((ServiceInstaller)pi.Installers[1]).DisplayName = servicename;
                        }
                        ti.Installers.Add(pi);
                        string path = String.Format("/assemblypath={0}",
                            System.Reflection.Assembly.GetExecutingAssembly().Location);
                        string[] cmdline = { path };
                        InstallContext ctx = new InstallContext("", cmdline);
                        ti.Context = ctx;
                        ti.Install(new System.Collections.Hashtable());
                        activeargument = true;
                    }
                    else if (thisarg.Contains("/uninstall"))
                    {
                        // First see if we can split the install parameter to grab the name of the service
                        TransactedInstaller ti = new TransactedInstaller();
                        ProjectInstaller pi = new ProjectInstaller();
                        if (thisarg.Split('=').Length == 2)
                        {
                            string servicename = thisarg.Split('=')[1];
                            ((ServiceInstaller)pi.Installers[1]).ServiceName = servicename;
                            ((ServiceInstaller)pi.Installers[1]).DisplayName = servicename;
                        }
                        ti.Installers.Add(pi);
                        string path = String.Format("/assemblypath={0}",
                            System.Reflection.Assembly.GetExecutingAssembly().Location);
                        string[] cmdline = { path };
                        InstallContext ctx = new InstallContext("", cmdline);
                        ti.Context = ctx;
                        ti.Uninstall(null);
                        activeargument = true;
                    }
                    else if (thisarg.Contains("/console"))
                    {
                        // Read the service configuration and start the service
                        Console.WriteLine("Cubed Service starting in console mode...");
                        Console.WriteLine("------------------------------------------------------------------------");
                        Console.WriteLine("The service is being provided on the following ports:");
                        foreach (ServiceEndPoint thisendpoint in SettingsManager.Settings.EndPoints)
                            switch (thisendpoint.EndpointType)
                            {
                                case ServiceEndPointType.NetTcp:
                                    Console.WriteLine("Net.Tcp @ net.tcp://" + Environment.MachineName + ":" + thisendpoint.Port.ToString() + "/CubedEngineService");
                                    break;
                                case ServiceEndPointType.Soap:
                                    Console.WriteLine("SOAP @ http://" + Environment.MachineName + ":" + thisendpoint.Port.ToString() + "/CubedEngineService");
                                    break;
                            }
                        if (SettingsManager.Settings.ExposeMetaData)
                            Console.WriteLine("Metadata is exposed via all available end points at /MetadataService");

                        // Indicate that the service is active
                        activeargument = true;

                        // Start the service
                        Console.WriteLine("Starting the service and caching data structures. This may take a few moments.");
                        List<string> errors = ServiceManager.StartService();
                        if (errors.Count > 0)
                        {
                            Console.WriteLine("------------------------------------------------------------------------");
                            Console.WriteLine("Some errors occured while trying to start the service. They were: ");
                            foreach (string thiserror in errors)
                                Console.WriteLine(thiserror);
                            Console.WriteLine("Press the ESCAPE key to exit.");
                        }
                        else
                        {
                            Console.WriteLine("------------------------------------------------------------------------");
                            Console.WriteLine("Note: Automatic consolidations are disabled when running in console mode.");
                            Console.WriteLine("The service is now operational. To close the service, press the ESCAPE key.");                            
                        }

                        // Wait for the user to cancel the service
                        ConsoleKey keypressed = ConsoleKey.A;
                        while (keypressed != ConsoleKey.Escape)
                        {
                            keypressed = Console.ReadKey().Key;
                            Thread.Sleep(new TimeSpan(0, 0, 5));
                        }

                        // Close the service
                        Console.WriteLine();
                        Console.WriteLine("Stopping the service and saving data structures. This may take a few moments.");
                        ServiceManager.StopService();
                    }
                }

                // Check to see if the service was active, otherwise start the base service
                if (!activeargument)
                {
                    ServiceBase[] ServicesToRun;
                    ServicesToRun = new ServiceBase[] 
			    { 
				    new CubedService() 
			    };
                    ServiceBase.Run(ServicesToRun);
                }
            }
            else
            {
                // No arguments were supplied so start the service
                ServiceBase[] ServicesToRun;
                ServicesToRun = new ServiceBase[] 
			    { 
				    new CubedService() 
			    };
                ServiceBase.Run(ServicesToRun);
            }
        }

        /// <summary>
        /// This checks to see if any automatic consolidations need to be run
        /// </summary>
        private static void ConsolidationScheduler_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            //ServiceManager.CheckAutomaticConsolidations();
        }
    }
}
