/*
 * This source file (DefaultCubeBuilder.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.AddIns;

namespace Bamboo.Cubed.Engine.Builders
{
    /// <summary>
    /// This class contains code to help in building a default cube with basic dimensions and members
    /// </summary>
    public static class DefaultCubeBuilder
    {
        /// <summary>
        /// Create a new, default Cube
        /// </summary>
        /// <param name="name"> The name of the new cube</param>
        /// <returns> A blank cube with default dimensions and measures</returns>
        public static Cube BuildCube(string name)
        {
            Cube newcube = new Cube(name);

            // First we add a default data source and it's properties
            if (AddInManager.Connectors.Count < 0)
                throw new EngineBuilderException("Error: no storage connectors have been defined. A cube cannot be " +
                    "build without having somewhere to store the contents.", null, String.Empty);
            DatabaseConnector defaultconnector = new DatabaseConnector(AddInManager.Connectors[0].ID,
                String.Empty, AddInManager.Connectors[0].CurrentVersion);
            Datasource ds = newcube.AddDataSource("Default", defaultconnector);

            // Now we add a default role
            Role role = newcube.AddRole("Administrator");
            newcube.ReadAccessRoles[0].Enabled = true;

            // Now we add 2 default measures and their properties
            newcube.AddMeasure("Amount", ds, DataTypes.Decimal);
            newcube.Measures[0].MeasureValueField = "Amount";            
            newcube.Measures[0].TableName = "Transaction";
            newcube.AddMeasure("Year to Date", ds, DataTypes.Decimal);
            newcube.Measures[1].MeasureValueField = "YTDAmount";
            newcube.Measures[1].TableName = "Transaction";

            // Now we add 5 standard dimensions and their properties, default role and default members

            #region Period Dimension

            newcube.AddDimension("Period", ds);
            newcube.Dimensions[0].Datasource = ds.ID;
            newcube.Dimensions[0].Description = "Period";
            newcube.Dimensions[0].Enabled = true;
            Member previousyear = newcube.Dimensions[0].AddMember(DateTime.Now.AddYears(-1).Year.ToString() + "-" + DateTime.Now.Year.ToString(), null);
            Member q1 = newcube.Dimensions[0].AddMember("1st Quarter", previousyear);
            newcube.Dimensions[0].AddMember("July", q1);
            newcube.Dimensions[0].AddMember("August", q1);
            newcube.Dimensions[0].AddMember("September", q1);
            Member q2 = newcube.Dimensions[0].AddMember("2nd Quarter", previousyear);
            newcube.Dimensions[0].AddMember("October", q2);
            newcube.Dimensions[0].AddMember("November", q2);
            newcube.Dimensions[0].AddMember("December", q2);
            Member q3 = newcube.Dimensions[0].AddMember("3rd Quarter", previousyear);
            newcube.Dimensions[0].AddMember("January", q3);
            newcube.Dimensions[0].AddMember("February", q3);
            newcube.Dimensions[0].AddMember("March", q3);
            Member q4 = newcube.Dimensions[0].AddMember("1st Quarter", previousyear);
            newcube.Dimensions[0].AddMember("April", q4);
            newcube.Dimensions[0].AddMember("May", q4);
            newcube.Dimensions[0].AddMember("June", q4);

            Member currentyear = newcube.Dimensions[0].AddMember(DateTime.Now.Year.ToString() + "-" + DateTime.Now.AddYears(1).Year.ToString(), null);
            q1 = newcube.Dimensions[0].AddMember("1st Quarter", currentyear);
            newcube.Dimensions[0].AddMember("July", q1);
            newcube.Dimensions[0].AddMember("August", q1);
            newcube.Dimensions[0].AddMember("September", q1);
            q2 = newcube.Dimensions[0].AddMember("2nd Quarter", currentyear);
            newcube.Dimensions[0].AddMember("October", q2);
            newcube.Dimensions[0].AddMember("November", q2);
            newcube.Dimensions[0].AddMember("December", q2);
            q3 = newcube.Dimensions[0].AddMember("3rd Quarter", currentyear);
            newcube.Dimensions[0].AddMember("January", q3);
            newcube.Dimensions[0].AddMember("February", q3);
            newcube.Dimensions[0].AddMember("March", q3);
            q4 = newcube.Dimensions[0].AddMember("1st Quarter", currentyear);
            newcube.Dimensions[0].AddMember("April", q4);
            newcube.Dimensions[0].AddMember("May", q4);
            newcube.Dimensions[0].AddMember("June", q4);

            Member fy1 = newcube.Dimensions[0].AddMember(DateTime.Now.AddYears(1).Year.ToString() + "-" + DateTime.Now.AddYears(2).Year.ToString(), null);
            q1 = newcube.Dimensions[0].AddMember("1st Quarter", fy1);
            newcube.Dimensions[0].AddMember("July", q1);
            newcube.Dimensions[0].AddMember("August", q1);
            newcube.Dimensions[0].AddMember("September", q1);
            q2 = newcube.Dimensions[0].AddMember("2nd Quarter", fy1);
            newcube.Dimensions[0].AddMember("October", q2);
            newcube.Dimensions[0].AddMember("November", q2);
            newcube.Dimensions[0].AddMember("December", q2);
            q3 = newcube.Dimensions[0].AddMember("3rd Quarter", fy1);
            newcube.Dimensions[0].AddMember("January", q3);
            newcube.Dimensions[0].AddMember("February", q3);
            newcube.Dimensions[0].AddMember("March", q3);
            q4 = newcube.Dimensions[0].AddMember("1st Quarter", fy1);
            newcube.Dimensions[0].AddMember("April", q4);
            newcube.Dimensions[0].AddMember("May", q4);
            newcube.Dimensions[0].AddMember("June", q4);

            Member fy2 = newcube.Dimensions[0].AddMember(DateTime.Now.AddYears(2).Year.ToString() + "-" + DateTime.Now.AddYears(3).Year.ToString(), null);
            q1 = newcube.Dimensions[0].AddMember("1st Quarter", fy2);
            newcube.Dimensions[0].AddMember("July", q1);
            newcube.Dimensions[0].AddMember("August", q1);
            newcube.Dimensions[0].AddMember("September", q1);
            q2 = newcube.Dimensions[0].AddMember("2nd Quarter", fy2);
            newcube.Dimensions[0].AddMember("October", q2);
            newcube.Dimensions[0].AddMember("November", q2);
            newcube.Dimensions[0].AddMember("December", q2);
            q3 = newcube.Dimensions[0].AddMember("3rd Quarter", fy2);
            newcube.Dimensions[0].AddMember("January", q3);
            newcube.Dimensions[0].AddMember("February", q3);
            newcube.Dimensions[0].AddMember("March", q3);
            q4 = newcube.Dimensions[0].AddMember("1st Quarter", fy2);
            newcube.Dimensions[0].AddMember("April", q4);
            newcube.Dimensions[0].AddMember("May", q4);
            newcube.Dimensions[0].AddMember("June", q4);

            Member fy3 = newcube.Dimensions[0].AddMember(DateTime.Now.AddYears(3).Year.ToString(), null);
            q1 = newcube.Dimensions[0].AddMember("1st Quarter", fy3);
            newcube.Dimensions[0].AddMember("July", q1);
            newcube.Dimensions[0].AddMember("August", q1);
            newcube.Dimensions[0].AddMember("September", q1);
            q2 = newcube.Dimensions[0].AddMember("2nd Quarter", fy3);
            newcube.Dimensions[0].AddMember("October", q2);
            newcube.Dimensions[0].AddMember("November", q2);
            newcube.Dimensions[0].AddMember("December", q2);
            q3 = newcube.Dimensions[0].AddMember("3rd Quarter", fy3);
            newcube.Dimensions[0].AddMember("January", q3);
            newcube.Dimensions[0].AddMember("February", q3);
            newcube.Dimensions[0].AddMember("March", q3);
            q4 = newcube.Dimensions[0].AddMember("1st Quarter", fy3);
            newcube.Dimensions[0].AddMember("April", q4);
            newcube.Dimensions[0].AddMember("May", q4);
            newcube.Dimensions[0].AddMember("June", q4);

            #endregion

            #region Organisational and Geographical Hierarchy Dimensions

            newcube.AddDimension("Organisational Hierarchy", ds);
            Member finmember = newcube.Dimensions[1].AddMember("Finance", null);
            newcube.Dimensions[1].AddMember("Accounting", finmember);
            newcube.Dimensions[1].AddMember("Budgeting", finmember);
            newcube.Dimensions[1].AddMember("Statutory Authorities", finmember);
            Member marketmember = newcube.Dimensions[1].AddMember("Marketing", null);
            newcube.Dimensions[1].AddMember("Advertising", marketmember);
            newcube.Dimensions[1].AddMember("Direct Sales", marketmember);
            newcube.Dimensions[1].AddMember("Market Sales", marketmember);
            newcube.Dimensions[1].AddMember("Trend Analysis", marketmember);
            Member adminmember = newcube.Dimensions[1].AddMember("Administration", null);
            newcube.Dimensions[1].AddMember("Legal", adminmember);
            newcube.Dimensions[1].AddMember("Human Resources", adminmember);
            Member devmember = newcube.Dimensions[1].AddMember("Development", null);
            newcube.Dimensions[1].AddMember("Research & Development", devmember);
            newcube.Dimensions[1].AddMember("New Projects", devmember);

            newcube.AddDimension("Geographical Hierarchy", ds);
            Member namember = newcube.Dimensions[2].AddMember("North America", null);
            Member usmember = newcube.Dimensions[2].AddMember("United States", null);
            newcube.Dimensions[2].AddMember("New York", usmember);
            newcube.Dimensions[2].AddMember("Los Angeles", usmember);
            newcube.Dimensions[2].AddMember("Boston", usmember);
            Member camember = newcube.Dimensions[2].AddMember("Canada", namember);
            newcube.Dimensions[2].AddMember("Vancouver", camember);
            newcube.Dimensions[2].AddMember("Montreal", camember);
            Member memember = newcube.Dimensions[2].AddMember("Mexico", namember);
            newcube.Dimensions[2].AddMember("Mexico City", memember);
            Member samember = newcube.Dimensions[2].AddMember("South America", null);
            Member armember = newcube.Dimensions[2].AddMember("Argentina", samember);
            newcube.Dimensions[2].AddMember("Buenos Aires", armember);
            Member brmember = newcube.Dimensions[2].AddMember("Brazil", samember);
            newcube.Dimensions[2].AddMember("Brasilia", brmember);
            newcube.Dimensions[2].AddMember("Rio De Janiro", brmember);
            Member vemember = newcube.Dimensions[2].AddMember("Venezuela", samember);
            Member eumember = newcube.Dimensions[2].AddMember("Europe", null);
            Member ukmember = newcube.Dimensions[2].AddMember("United Kingdom", eumember);
            newcube.Dimensions[2].AddMember("London", ukmember);
            newcube.Dimensions[2].AddMember("Manchester", ukmember);
            newcube.Dimensions[2].AddMember("Scotland", ukmember);
            newcube.Dimensions[2].AddMember("Ireland", ukmember);
            newcube.Dimensions[2].AddMember("Wales", ukmember);
            Member frmember = newcube.Dimensions[2].AddMember("France", eumember);
            newcube.Dimensions[2].AddMember("Paris", frmember);
            newcube.Dimensions[2].AddMember("Lyon", frmember);
            Member demember = newcube.Dimensions[2].AddMember("Germany", eumember);
            newcube.Dimensions[2].AddMember("Hamburg", demember);
            newcube.Dimensions[2].AddMember("Berlin", demember);
            Member asmember = newcube.Dimensions[2].AddMember("Asia", null);
            Member ozmember = newcube.Dimensions[2].AddMember("Australia", asmember);
            newcube.Dimensions[2].AddMember("Sydney", ozmember);
            newcube.Dimensions[2].AddMember("Melbourne", ozmember);
            Member chmember = newcube.Dimensions[2].AddMember("China", asmember);
            newcube.Dimensions[2].AddMember("Beijing", chmember);
            newcube.Dimensions[2].AddMember("Hong Kong", chmember);
            Member inmember = newcube.Dimensions[2].AddMember("Indonesia", asmember);
            newcube.Dimensions[2].AddMember("Java", inmember);

            #endregion

            #region Schedule and Version Dimensions

            newcube.AddDimension("Schedule", ds);
            Member minmember = newcube.Dimensions[3].AddMember("Income Statement", null);
            Member totalexp = newcube.Dimensions[3].AddMember("Total Expenditure", minmember);
            newcube.Dimensions[3].AddMember("Depreciation", totalexp);
            newcube.Dimensions[3].AddMember("Wages & Salaries", totalexp);
            newcube.Dimensions[3].AddMember("Land & Buildings", totalexp);
            Member totalin = newcube.Dimensions[3].AddMember("Total Income", minmember);
            newcube.Dimensions[3].AddMember("Sales", totalin);
            newcube.Dimensions[3].AddMember("Interest", totalin);
            Member bsmember = newcube.Dimensions[3].AddMember("Balance Sheet", null);
            Member totala = newcube.Dimensions[3].AddMember("Total Assets", bsmember);
            newcube.Dimensions[3].AddMember("Buildings", totala);
            newcube.Dimensions[3].AddMember("Vehicles", totala);
            Member totall = newcube.Dimensions[3].AddMember("Total Liabilities", bsmember);
            newcube.Dimensions[3].AddMember("Loans", totall);
            Member kpimember = newcube.Dimensions[3].AddMember("Key Performance Indicators", null);
            Member targets = newcube.Dimensions[3].AddMember("Strategic Targets", kpimember);
            newcube.Dimensions[3].AddMember("Sales", targets);
            newcube.Dimensions[3].AddMember("Performance", targets);

            newcube.AddDimension("Version", ds);
            newcube.Dimensions[4].AddMember("Approved Estimates", null);
            newcube.Dimensions[4].AddMember("Working Estimates", null);
            newcube.Dimensions[4].AddMember("Actuals", null);
            newcube.Dimensions[4].AddMember("Forecast", null);

            #endregion

            // Now we create default links between all measures and all dimensions in the cube
            foreach (Measure measure in newcube.Measures)
            {
                foreach (Dimension dimension in newcube.Dimensions)
                {
                    dimension.AddMeasureLink(measure, String.Empty, String.Empty, String.Empty, String.Empty);
                }
            }

            // Finally we return the new cube            
            return newcube;
        }
    }
}
