/*
 * This source file (FormulaManager.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Utilities.Logging;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.Engine.Persistence;
using System.Xml;
using System.Xml.Serialization;

namespace Bamboo.Cubed.Engine.Formulas
{
    /// <summary>
    /// The Formula Manager class provides formula evaluation functionality for the calculation of members and
    /// measures values
    /// </summary>
    public class FormulaManager
    {
        /// <summary>
        /// Initialises the Formula Manager with the given evaluator
        /// </summary>
        /// <param name="evaluator">The Formula Evaluator to use for this formula evaluation</param>
        public FormulaManager(FormulaEvaluator evaluator)
        {
            // Based on the details of the given evaluator, attempt to initialise our calculator
            if (AddInManager.Formulas.Contains(evaluator.EvaluatorID))
                _calculator = AddInManager.Formulas[evaluator.EvaluatorID];
            else
                throw new FormulaManagerException("Requested formula evaluator does not exist in the Add Ins collection",
                    null, "Requested Evaluator ID: " + evaluator.EvaluatorID + "; Version: " 
                    + evaluator.EvaluatorVersion.ToString());  
 
            // Set our calculators internal configuration properties
            _calculator.FormulaInstance.InitialiseProperties(evaluator.EvaluatorConfiguration,
                evaluator.EvaluatorVersion);
            _calculator.ConfigurationInstance.InitialiseProperties(evaluator.EvaluatorConfiguration,
                evaluator.EvaluatorVersion);
        }

        private CubedFormulaDescription _calculator;
        /// <summary>
        /// The Calculator which has been initialised to evaluate the given member or measures value
        /// </summary>
        public CubedFormulaDescription Calculator
        {
            get { return _calculator; }
            set { _calculator = value; }
        }

        /// <summary>
        /// Returns a new node with the value for the given calculated measure
        /// </summary>
        /// <param name="cube">The cube being consolidated</param>
        /// <param name="calculatedmeasure">The measure to calculate</param>
        /// <param name="valuescollection">A collection of other measures for the same member collection
        /// and their values</param>
        /// <returns>The value of the calculated measure</returns>
        public double CalculateMeasure(Cube cube, Measure calculatedmeasure, 
            Dictionary<string, double> valuescollection)
        {
            try
            {
                return _calculator.FormulaInstance.EvaluateMeasure(cube, calculatedmeasure, valuescollection);
            }
            catch (Exception e1)
            {
                // Log the error and return zero
                LoggingManager.Write(e1, LoggingTypes.MinorError);
                return 0;
            }
        }

        /// <summary>
        /// Returns a new node with the value for the given calculated member
        /// </summary>
        /// <param name="cube">The cube being consolidated</param>
        /// <param name="calculatedmember">The member to calculate</param>
        /// <param name="valuescollection">A collection of other members for the same dimension
        /// and their values</param>
        /// <returns>The value of the calculated member</returns>
        public double CalculateMember(Cube cube, Member calculatedmember,
            Dictionary<string, double> valuescollection)
        {
            try
            {
                return _calculator.FormulaInstance.EvaluateMember(cube, calculatedmember, valuescollection);
            }
            catch (Exception e1)
            {
                // Log the error and return zero
                LoggingManager.Write(e1, LoggingTypes.MinorError);
                return 0;
            }
        }
    }
}
