/*
 * This source file (SettingsManager.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures;
using System.Data;
using System.IO;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.AddIns.DatabaseConnectors;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.Utilities.Logging;
using System.Threading;

namespace Bamboo.Cubed.Engine.Persistence
{
    /// <summary>
    /// This class provides support for reading and writing the Cubed server settings to/from disk
    /// </summary>
    public static class SettingsManager
    {
        private static string settingslocation = Information.MyDirectory + "Server.xml";
        /// <summary>
        /// This is the location of the settings file to load
        /// </summary>
        public static string SettingsLocation
        {
            get
            {
                if (settingslocation == null || settingslocation == String.Empty)
                    settingslocation = Information.MyDirectory + "Server.xml";
                return settingslocation;
            }
            set { settingslocation = value; }
        }

        /// <summary>
        /// This reads and returns the current system settings from the server, or updates the settings on
        /// the server (automatically saving)
        /// </summary>
        [XmlElement("Settings")]
        public static ServerSettings Settings
        {
            get { return ReadSettings(SettingsLocation); }
            set { SaveSettings(((ServerSettings)value), SettingsLocation); } 
        }

        /// <summary>
        /// Saves the CubedServer settings to disk
        /// </summary>
        /// <param name="settings"> The settings to save</param>
        /// <param name="filename"> The filename to save the settings to</param>
        /// <returns> True if successful</returns>
        public static bool SaveSettings(ServerSettings settings, string filename)
        {            
            // First delete the existing settings
            try
            {
                Utilities.FileAccess.DeleteFile(filename);
            }
            catch (FileAccessException fe)
            {
                throw new SettingsAccessException("Error saving settings to disk - cannot delete current settings.", fe, String.Empty);
            }

            // Check if the root users collection is empty, and if it is - add the currently
            // connected user
            if (settings.RootUsers.Count < 1)
            {
                if (Thread.CurrentPrincipal.Identity.Name.Contains("\\"))
                    settings.RootUsers.Add(Thread.CurrentPrincipal.Identity.Name);
                else
                    settings.RootUsers.Add(Environment.MachineName + "\\" +
                        Thread.CurrentPrincipal.Identity.Name);
            }

            // Then, encrypted all of the root users names before saving
            for (int i = 0; i < settings.RootUsers.Count; i++)
                settings.RootUsers[i] = EncryptionManager.EncryptString(settings.RootUsers[i]);

            // Also encrypt SMTP server information
            settings.SMTPServer = EncryptionManager.EncryptString(settings.SMTPServer);
            settings.SMTPUser = EncryptionManager.EncryptString(settings.SMTPUser);
            settings.SMTPPassword = EncryptionManager.EncryptString(settings.SMTPPassword);

            // Then save our settings to disk
            try
            {
                XMLManager<ServerSettings>.SerialiseToFile(filename, settings);              
            }
            catch (Exception e1)
            {
                throw new SettingsAccessException("Error saving settings to disk - cannot write to settings file.", e1, String.Empty);
            }

            // Before returning, decrypt all the root users so the session can continue
            for (int i = 0; i < settings.RootUsers.Count; i++)
                settings.RootUsers[i] = EncryptionManager.DecryptString(settings.RootUsers[i]);

            // Then decrypt SMTP server information
            settings.SMTPServer = EncryptionManager.DecryptString(settings.SMTPServer);
            settings.SMTPUser = EncryptionManager.DecryptString(settings.SMTPUser);
            settings.SMTPPassword = EncryptionManager.DecryptString(settings.SMTPPassword);
            
            // Return true to indicate success
            return true;
        }

        /// <summary>
        /// Reads the Cubed Server Settings from a file
        /// </summary>
        /// <param name="filename"> The filename to read from</param>
        /// <returns> The settings contained in the file</returns>
        public static ServerSettings ReadSettings(string filename)
        {
            ServerSettings settings;

            try
            {
                // First see if the file exists
                if (File.Exists(filename))
                {
                    settings = XMLManager<ServerSettings>.DeserialiseFromFile(filename);

                    // Before returning, decrypt all the root users names before returning
                    for (int i = 0; i < settings.RootUsers.Count; i++)
                        settings.RootUsers[i] = EncryptionManager.DecryptString(settings.RootUsers[i]);

                    // Then decrypt SMTP server information
                    settings.SMTPServer = EncryptionManager.DecryptString(settings.SMTPServer);
                    settings.SMTPUser = EncryptionManager.DecryptString(settings.SMTPUser);
                    settings.SMTPPassword = EncryptionManager.DecryptString(settings.SMTPPassword);

                    // Return the settings if they were found successfully
                    return settings;
                }
                else
                    throw new SettingsAccessException("Error accessing settings: file not found.", null, String.Empty);
            }
            catch (SettingsAccessException se)
            {   
                // Even though an error occured reading the settings, return a new default settings
                settings = new ServerSettings();
                settings.EndPoints.Add(new ServiceEndPoint());

                // Then, if there is a valid connected principal, finish creating and save the settings -
                // otherwise return whats already been created
                if (!Thread.CurrentPrincipal.Identity.IsAuthenticated ||
                    Thread.CurrentPrincipal.Identity.Name == String.Empty)
                    return settings;

                // Log the failure to find the settings
                LoggingManager.Write(se, LoggingTypes.MinorError);

                // First add the current user as the new root user
                if (Thread.CurrentPrincipal.Identity.Name.Contains("\\"))
                    settings.RootUsers.Add(Thread.CurrentPrincipal.Identity.Name);
                else
                    settings.RootUsers.Add(Environment.MachineName + "\\" + 
                        Thread.CurrentPrincipal.Identity.Name);

                // Then Initialise all Add In providers found with default settings
                foreach (CubedFunctionDescription thisfunction in AddInManager.Functions)
                {
                    Function newfunction = new Function(thisfunction.Name, thisfunction.ID, String.Empty,
                        thisfunction.CurrentVersion);
                    settings.Functions.Add(newfunction);
                }
                foreach (CubedQueryParserDescription thisparser in AddInManager.Parsers)
                {
                    QueryParser newparser = new QueryParser(thisparser.Name, thisparser.ID, String.Empty,
                        thisparser.CurrentVersion);
                    settings.Parsers.Add(newparser);
                }

                // Then, provided a default storage provider is found, add the default storage provider
                if (AddInManager.Connectors.Count > 0)
                {
                    DatabaseConnector newconnector = new DatabaseConnector(AddInManager.Connectors[0].ID,
                        String.Empty, AddInManager.Connectors[0].CurrentVersion);
                    settings.StorageProvider = newconnector;
                }

                // Set the default directories for logging
                settings.LogLevel = LoggingLevels.Medium;

                // Set the default table names for structures and data
                settings.StructuresTableName = "Cubed_Structures";
                settings.DataTablePrefix = "Cubed_Data";

                // Attempt to save the new settings
                SaveSettings(settings, filename);

                // Finally return the settings
                return settings;
            }
        }        
    }
}
