/*
 * This source file (StorageCreator.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures;
using System.Data;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.AddIns.DatabaseConnectors;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.Engine.Databases;
using System.Text.RegularExpressions;

namespace Bamboo.Cubed.Engine.Persistence
{
    /// <summary>
    /// This class provides support verifying and creating (if necessary) storage media
    /// </summary>
    public static class StorageCreator
    {
        /// <summary>
        /// Creates the storage manager functionality to store the data and structures for the Cubed Server
        /// </summary>
        /// <param name="settings">The Cubed Server settings file</param>
        /// <returns>True if successful, False if there was a problem</returns>
        public static bool PrepareStorage(ServerSettings settings)
        {
            try
            {
                // First we need to prepare the Storage Provider
                DatabaseManager datamanagerprovider = new DatabaseManager(settings.StorageProvider);
                CubedDatabaseConnector datamanager = datamanagerprovider.Provider.ConnectorInstance;

                // Then attempt to open the connection
                datamanager.Open();

                // Now we need to create the storage database, but only if it doesn't already exist
                if (!datamanager.CreateDatabase(true))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed Storage Database.");
                }

                // We need to create the various Cube definitions tables to be save our Cubes to, beginning with
                // our cubes table                
                if (!datamanager.CreateTable(settings.StructuresTableName + "_Cubes",
                    DataDefinitions.CubeFieldNames, DataDefinitions.CubeFieldTypes, true, DataDefinitions.CubeFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our data sources table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_Datasources",
                    DataDefinitions.DatasourceFieldNames, DataDefinitions.DatasourceFieldTypes, true, DataDefinitions.DatasourceFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our data providers table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_DatasourceProviders",
                    DataDefinitions.DataProviderFieldNames, DataDefinitions.DataProviderFieldTypes, true, DataDefinitions.DataProviderFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our dimensions table
                if (!datamanager.CreateIndexedTable(settings.StructuresTableName + "_Dimensions",
                    DataDefinitions.DimensionFieldNames, DataDefinitions.DimensionFieldTypes, true, new string[] { DataDefinitions.DimensionFieldNames[0] },
                    DataDefinitions.DimensionIndexes))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our attributes table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_Attributes",
                    DataDefinitions.AttributeFieldNames, DataDefinitions.AttributeFieldTypes, true, DataDefinitions.AttributeFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our KPIs table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_KPIs",
                    DataDefinitions.KPIFieldNames, DataDefinitions.KPIFieldTypes, true, DataDefinitions.KPIFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Member Generators table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_MemberGenerators",
                    DataDefinitions.MemberGeneratorFieldNames, DataDefinitions.MemberGeneratorFieldTypes, true, DataDefinitions.MemberGeneratorFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Range Generators table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_RangeGenerators",
                    DataDefinitions.RangeGeneratorFieldNames, DataDefinitions.RangeGeneratorFieldTypes, true, DataDefinitions.RangeGeneratorFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Member Table Details table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_MemberTables",
                    DataDefinitions.MemberTableFieldNames, DataDefinitions.MemberTableFieldTypes, true, DataDefinitions.MemberTableFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Member Table Parents table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_MemberTableParents",
                    DataDefinitions.MemberTableParentFieldNames, DataDefinitions.MemberTableParentFieldTypes,
                    true, DataDefinitions.MemberTableParentFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Member Table Parents table.");
                }

                // Then our Attribute Value Fields table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_AttributeValueFields", 
                    DataDefinitions.AttributeValueFieldFieldNames, DataDefinitions.AttributeValueFieldFieldTypes, true,
                    DataDefinitions.AttributeValueFieldFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Roles table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_Roles",
                    DataDefinitions.RoleFieldNames, DataDefinitions.RoleFieldTypes, true, DataDefinitions.RoleFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Users table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_Users",
                    DataDefinitions.UserFieldNames, DataDefinitions.UserFieldTypes, true, DataDefinitions.UserFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Members table
                if (!datamanager.CreateIndexedTable(settings.StructuresTableName + "_Members",
                    DataDefinitions.MemberFieldNames, DataDefinitions.MemberFieldTypes, true, new string[] { DataDefinitions.MemberFieldNames[0] }, 
                    DataDefinitions.MemberIndexes))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Value Filters table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_ValueFilters",
                    DataDefinitions.ValueFilterFieldNames, DataDefinitions.ValueFilterFieldTypes, true, DataDefinitions.ValueFilterFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Attribute Values table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_AttributeValues",
                    DataDefinitions.AttributeValueFieldNames, DataDefinitions.AttributeValueFieldTypes, true, DataDefinitions.AttributeValueFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Formula Evaluators table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_FormulaEvaluators",
                    DataDefinitions.FormulaEvaluatorFieldNames, DataDefinitions.FormulaEvaluatorFieldTypes, true, DataDefinitions.FormulaEvaluatorFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our KPI Calculators table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_KPICalculators",
                    DataDefinitions.KPICalculatorFieldNames, DataDefinitions.KPICalculatorFieldTypes, true, DataDefinitions.KPICalculatorFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Levels table
                if (!datamanager.CreateIndexedTable(settings.StructuresTableName + "_Levels",
                    DataDefinitions.LevelFieldNames, DataDefinitions.LevelFieldTypes, true, new string[] { DataDefinitions.LevelFieldNames[0] },
                    DataDefinitions.LevelIndexes))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Measure Links table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_MeasureLinks",
                    DataDefinitions.MeasureLinkFieldNames, DataDefinitions.MeasureLinkFieldTypes, true, DataDefinitions.MeasureLinkFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Measures table
                if (!datamanager.CreateIndexedTable(settings.StructuresTableName + "_Measures",
                    DataDefinitions.MeasureFieldNames, DataDefinitions.MeasureFieldTypes, true, new string[] { DataDefinitions.MeasureFieldNames[0] },
                    DataDefinitions.MeasureIndexes))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Descendants table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_Descendants",
                    DataDefinitions.DescendantsFieldNames, DataDefinitions.DescendantsFieldTypes, true,
                    DataDefinitions.DescendantsFieldNames))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables.");
                }

                // Then our Desktop Searches table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_DesktopSearches",
                    DataDefinitions.DesktopSearchFieldNames, DataDefinitions.DesktopSearchFieldTypes,
                    true, DataDefinitions.DesktopSearchFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables: Desktop Search table");
                }

                // Then our Desktop Search Results table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_DesktopSearchResults",
                    DataDefinitions.DesktopSearchResultFieldNames, DataDefinitions.DesktopSearchResultFieldTypes,
                    true, DataDefinitions.DesktopSearchResultFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables: Desktop Search Results table");
                }

                // Then our Desktop Search Indexes table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_DesktopSearchIndexes",
                    DataDefinitions.DesktopSearchIndexFieldNames, DataDefinitions.DesktopSearchIndexFieldTypes,
                    true, DataDefinitions.DesktopSearchIndexFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables: Desktop Search Indexes table");
                }

                // Then our Desktop Searches table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_DesktopDashboards",
                    DataDefinitions.DesktopDashboardFieldNames, DataDefinitions.DesktopDashboardFieldTypes,
                    true, DataDefinitions.DesktopDashboardFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables: Desktop Dashboards table");
                }

                // Then our Desktop Searches table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_DesktopDashboardItems",
                    DataDefinitions.DesktopDashboardItemFieldNames, DataDefinitions.DesktopDashboardItemFieldTypes,
                    true, DataDefinitions.DesktopDashboardItemFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables: Desktop Dashboard Items table");
                }

                // Then our Search Categories table
                if (!datamanager.CreateTable(settings.StructuresTableName + "_SearchCategories",
                    DataDefinitions.SearchCategoriesFieldNames, DataDefinitions.SearchCategoriesFieldTypes,
                    true, DataDefinitions.SearchCategoriesFieldNames[0]))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error creating Cubed storage tables: Search Categories table");
                }

                // Then create our Get Member Ancestor function
                if (!datamanager.CreateFunction(settings.StructuresTableName + "_GetMemberAncestor", SQLFunctionTypes.Scalar,
                    DataDefinitions.GetMemberAncestorParameterNames, DataDefinitions.GetMemberAncestorParameterTypes,
                    Regex.Replace(DataDefinitions.GetMemberAncestorCode, "Members", 
                        settings.StructuresTableName + "_Members", RegexOptions.IgnoreCase),
                     "[", "]", null, DataCreationTypes.UniqueIdentifier, true))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new ApplicationException("Error created Cubed storage functions: GetMemberAncestor function");
                }

                // Finally create our Get Member Descendants function
                if (!datamanager.CreateFunction(settings.StructuresTableName + "_GetMemberDescendants", SQLFunctionTypes.InlineTable,
                    DataDefinitions.GetMemberDescendantsParameterNames, DataDefinitions.GetMemberDescendantsParameterTypes,
                    Regex.Replace(DataDefinitions.GetMemberDescendantsCode, "Members",
                        settings.StructuresTableName + "_Members", RegexOptions.IgnoreCase),
                     "[", "]", null, DataCreationTypes.UniqueIdentifier, true))
                {
                    // Roll back the transaction due to the error
                    datamanager.Rollback();
                    throw new ApplicationException("Error created Cubed storage functions: GetMemberDescendants function");
                }

                // Try to commit or rollback the transaction
                if (!datamanager.Commit())
                {
                    datamanager.Rollback();
                    throw new System.ApplicationException("Error saving records to Database.");
                }

                // No errors occured so return true to indicate that the storage media was prepared successfully
                return true;
            }
            catch (Exception e1)
            {
                throw new StorageAccessException("Error preparing storage media." + e1.ToString(), e1, String.Empty);
            }
        }
    }
}
