/*
 * This source file (QueryManager.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Structures.Query;
using Bamboo.Cubed.AddIns;
using System.Data;
using Bamboo.Cubed.Engine.Persistence;
using Bamboo.Cubed.Utilities.Logging;
using Bamboo.Cubed.Engine.Security;
using Bamboo.Cubed.Utilities;

namespace Bamboo.Cubed.Engine.Query
{
    /// <summary>
    /// The Query Manager provides the ability to query values from the Cube using the given query parser
    /// </summary>
    public class QueryManager
    {
        /// <summary>
        /// Initialises the Query Manager with the given Query Parser
        /// </summary>
        /// <param name="parser">The Formula Evaluator to use for this formula evaluation</param>
        public QueryManager(QueryParser parser)
        {
            // Based on the details of the given parser, attempt to initialise our query parser
            if (AddInManager.Parsers.Contains(parser.ParserID))
                Parser = AddInManager.Parsers[parser.ParserID];
            else
                throw new QueryManagerException("Requested query parser does not exist in the Add Ins collection",
                    null, "Requested Parser ID: " + parser.ParserID + "; Version: " 
                    + parser.ParserVersion.ToString());  
 
            // Set our parsers internal configuration properties
            Parser.ParserInstance.InitialiseProperties(parser.ParserConfiguration,
                parser.ParserVersion);
            Parser.ConfigurationInstance.InitialiseProperties(parser.ParserConfiguration,
                parser.ParserVersion);
        }

        /// <summary>
        /// Initialises the Query Manager ready to return results based on the full set of details provided
        /// </summary>
        /// <param name="properties">The properties of the query</param>
        /// <param name="command">The command text of the query</param>
        /// <param name="format">The format of the query result set (if a SELECT query)</param>
        /// <param name="content">The content of the query result set (if a SELECT query)</param>
        /// <param name="settings">The settings of the server</param>
        /// <param name="cubes">The cubes in the database</param>
        /// <param name="username">The user executing the query</param>
        public QueryManager(Structures.XMLA.PropertyList properties, string command,
            Structures.XMLA.FormatTypes format, Structures.XMLA.ContentTypes content,
            ServerSettings settings, List<Cube> cubes, string username)
        { 
            // First we initialise our query parser based on the property "command type"
            foreach (QueryParser thisparser in settings.Parsers)
                if (thisparser.TypeAlias.ToUpper() == properties.CommandType.ToUpper())
                {
                    Parser = AddInManager.Parsers[thisparser.ParserID];
                    Parser.ConfigurationInstance.InitialiseProperties(thisparser.ParserConfiguration,
                        thisparser.ParserVersion);
                    Parser.ParserInstance.InitialiseProperties(thisparser.ParserConfiguration,
                        thisparser.ParserVersion);
                }

            // Now check whether the parser for this type alias was found
            if (_parser == null)
                throw new QueryManagerException("No parser is configured with the type alias: " + properties.CommandType
                    + " specified by this query.", null, String.Empty); 
            
            // The parser required was found, so now initialise our Query Manager default settings
            _settings = settings;
            _cubes = cubes;
            _username = username;

            // Now attempt to parse the given query and determine what type of query it is            
            MDMStatement _query = Parser.ParserInstance.Parse(command);
            if (_query is MDMUpdate)
            {
                _updatequery = (MDMUpdate)_query;
                _querytype = MDMStatementTypes.UPDATE;
            }
            else if (_query is MDMQuery)
            {
                _selectquery = (MDMQuery)_query;
                _querytype = MDMStatementTypes.SELECT;
            }
            else
                throw new QueryManagerException("This query is not recognised as either an UPDATE or a SELECT query,"
                        + " and ALTER queries are not supported.", null, String.Empty);
 
            // Now we have successfully parsed our query, we can initialise our content and format values
            _format = format;
            _content = content;
            _properties = properties;
        }

        private CubedQueryParserDescription _parser;
        /// <summary>
        /// The Parser which has been initialised to parse the given query and return a result
        /// </summary>
        public CubedQueryParserDescription Parser
        {
            get { return _parser; }
            set { _parser = value; }
        }

        private ServerSettings _settings;
        /// <summary>
        /// The settings for Cubed
        /// </summary>
        public ServerSettings Settings
        {
            get { return _settings; }
            set { _settings = value; }
        }

        private List<Cube> _cubes;
        /// <summary>
        /// The cubes in the catalog
        /// </summary>
        public List<Cube> Cubes
        {
            get { return _cubes; }
            set { _cubes = value; }
        }

        private Cube _targetcube;
        /// <summary>
        /// This is the name of the cube which is the target or focus of the query. It will only be set once
        /// the cube has been validated
        /// </summary>
        public Cube TargetCube
        {
            get 
            {
                if (IsValid)
                    return _targetcube;
                else
                    throw new QueryManagerException("This query cannot be validated, so no target cube can be "
                        + "identified.", null, String.Empty);
            }
        }

        private string _username;
        /// <summary>
        /// The name of the user executing this query
        /// </summary>
        public string Username
        {
            get { return _username; }
            set { _username = value; }
        }

        private MDMQuery _selectquery;
        /// <summary>
        /// The Multi Dimensional Select Query (if this is a SELECT query)
        /// </summary>
        public MDMQuery SelectQuery
        {
            get { return _selectquery; }
            set { _selectquery = value; }
        }

        private MDMUpdate _updatequery;
        /// <summary>
        /// The Multi Dimensional Update Query (if this is an UPDATE query)
        /// </summary>
        public MDMUpdate UpdateQuery
        {
            get { return _updatequery; }
            set { _updatequery = value; }
        }

        private MDMStatementTypes _querytype;
        /// <summary>
        /// The type of Multi Dimensional Query this is (either a SELECT or an UPDATE)
        /// </summary>
        public MDMStatementTypes QueryType
        {
            get { return _querytype; }
            set { _querytype = value; }
        }

        private Structures.XMLA.ContentTypes _content;
        /// <summary>
        /// The type of content requested to be returned (if this is a SELECT query)
        /// </summary>
        public Structures.XMLA.ContentTypes Content
        {
            get { return _content; }
            set { _content = value; }
        }

        private Structures.XMLA.FormatTypes _format;
        /// <summary>
        /// The format of the content requested to be returned (if this is a SELECT query)
        /// </summary>
        public Structures.XMLA.FormatTypes Format
        {
            get { return _format; }
            set { _format = value; }
        }

        private Structures.XMLA.PropertyList _properties;
        /// <summary>
        /// The properties sent with the command
        /// </summary>
        public Structures.XMLA.PropertyList Properties
        {
            get { return _properties; }
            set { _properties = value; }
        }

        /// <summary>
        /// Contains the collection of errors which have occured during the parsing of this query
        /// </summary>
        public List<string> Errors
        {
            get 
            {
                if (_parser != null)
                    return Parser.ParserInstance.ErrorCollection;
                else
                    return new List<string>(new string[] { "Parser not initialised." });
            }
        }

        private bool? _isvalid;
        /// <summary>
        /// Returns a value indicating whether the given query is valid or not
        /// </summary>
        public bool IsValid
        {
            get
            {
                if (_isvalid.HasValue)
                    return _isvalid.Value;
                else
                {
                    _isvalid = Validate();
                    return _isvalid.Value;
                }
            }
        }

        private bool? _executed;
        private Structures.XMLA.MDDataSet _mddataset;
        /// <summary>
        /// The Multi Dimensional DataSet containing the results of the query
        /// </summary>
        /// <remarks>
        /// If this query has not been executed, an empty multi-dimensional data set will be returned. To execute
        /// the query, call the Execute method
        /// </remarks>
        public Structures.XMLA.MDDataSet MDDataSet
        {
            get
            {
                if (_querytype == MDMStatementTypes.SELECT)
                {
                    if (_format == Bamboo.Cubed.Structures.XMLA.FormatTypes.Multidimensional
                        || ((_content == Bamboo.Cubed.Structures.XMLA.ContentTypes.SchemaData
                        || _content == Bamboo.Cubed.Structures.XMLA.ContentTypes.Data)
                        && _format == Bamboo.Cubed.Structures.XMLA.FormatTypes.Native))
                    {
                        if (_executed.HasValue && _executed == true)                            
                                return _mddataset;
                        else
                            return new Bamboo.Cubed.Structures.XMLA.MDDataSet();                        
                    }
                    else
                        throw new QueryManagerException("The defined format type is either Tabular, or the requested "
                            + "content is schema only, in which case a multi dimensional data set is not returned.",
                            null, String.Empty);
                }
                else
                    throw new QueryManagerException("No results are returned from UPDATE or ALTER queries.",
                        null, String.Empty);
            }
        }

        private DataSet _dataset;
        /// <summary>
        /// The Tabular DataSet containing the results of the query
        /// </summary>
        /// <remarks>
        /// If this query has not been executed, an empty data set will be returned. To execute
        /// the query, call the Execute method
        /// </remarks>
        public DataSet DataSet
        {
            get
            {
                if (_querytype == MDMStatementTypes.SELECT)
                {
                    if (_format == Bamboo.Cubed.Structures.XMLA.FormatTypes.Tabular || 
                        _selectquery.Transactional 
                        || (_content == Bamboo.Cubed.Structures.XMLA.ContentTypes.Schema                        
                        && _format == Bamboo.Cubed.Structures.XMLA.FormatTypes.Native))
                    {
                        if (_executed.HasValue && _executed == true)                            
                                return _dataset;
                        else
                            return new DataSet();
                    }
                    else
                        throw new QueryManagerException("The defined format type is either Multi Dimensional, or the requested "
                            + "content is data only or schema and data, in which case a standard data set is not returned.",
                            null, String.Empty);
                }
                else
                    throw new QueryManagerException("No results are returned from UPDATE or ALTER queries.",
                        null, String.Empty);
            }
        }

        private long _cellsupdated;
        /// <summary>
        /// Returns how many cells have been updated by an UPDATE query
        /// </summary>
        /// <remarks>
        /// If this query has not been executed, 0 will be returned as the cells updated value. To execute
        /// the query, call the Execute method
        /// </remarks>
        public long CellsUpdated
        {
            get
            {
                if (_querytype == MDMStatementTypes.UPDATE)
                {
                    if (_executed.HasValue && _executed == true)
                            return _cellsupdated;
                    else
                        return 0;
                }
                else
                    throw new QueryManagerException("SELECT and ALTER queries do not update cell values.",
                        null, String.Empty);
            }
        }

        /// <summary>
        /// This determines if the query is valid against the Cubed data structures, by determining that all 
        /// specified members, dimensions, etc. exist. 
        /// </summary>
        /// <returns>True if the query is valid, false if not</returns>
        public bool Validate()
        {
            if (_querytype == MDMStatementTypes.UPDATE)
            {
                // This is an UPDATE query
                return Validate(_updatequery);
            }
            else if (_querytype == MDMStatementTypes.SELECT)
            {
                // This is a SELECT query
                return Validate(_selectquery);
            }
            else
                throw new QueryManagerException("ALTER queries are not supported.", null, String.Empty);            
        }

        /// <summary>
        /// This validates an UPDATE query to ensure it will successfully run against the Cube
        /// </summary>
        /// <param name="updatequery">The UPDATE query to validate</param>
        /// <returns>True if the query is valid, false if not</returns>
        public bool Validate(MDMUpdate updatequery)
        {
            if (updatequery != null && updatequery.StatementInfo.QueryAxis.Length > 0)
            {
                // This query has been initialised so we proceed to validate it
                Errors.Clear();

                // The first step in validation is to check that the cube exists
                // If this step in validation fails, quit execution as we cannot continue validating against an
                // unknown cube
                int cubeindex = -1;                
                for (int i = 0; i < _cubes.Count; i++)
                    if (_cubes[i].Name.ToUpper().Equals(updatequery.Cube.ToUpper()))
                        cubeindex = i;
                if (cubeindex < 0)
                {
                    // The cube doesn't exist, so raise an error
                    Errors.Add("Cube " + updatequery.Cube + " does not exist.");
                    return false;
                }
                else
                    _targetcube = _cubes[cubeindex];
                
                // Try and ensure that the value given to update is valid
                double updatevalue = 0;
                if (!double.TryParse(updatequery.NewValue, out updatevalue))
                {
                    // The value passed to update is not valid
                    Errors.Add("Value passed for update " + updatequery.NewValue + " is not valid.");
                }                

                // The next step is to ensure that all the dimensions exist
                // If any dimensions don't exist, add to the error collection but don't return yet
                foreach (string thisquerydimension in updatequery.StatementInfo.QueryDimensions)
                {
                    bool dimensionexists = false;
                    foreach (Dimension thisdimension in _targetcube.Dimensions)
                        if (thisdimension.Name.ToUpper().Equals(thisquerydimension.ToUpper()))
                            dimensionexists = true;
                    if (!dimensionexists)
                    {
                        // This dimension does not exist, so raise an error
                        Errors.Add("Specified dimension: " + thisquerydimension + " does not exist in the cube "
                            + updatequery.Cube + ".");
                    }
                }

                // Now we ensure that all measures exist in the cube
                foreach (string thisquerymeasure in updatequery.StatementInfo.QueryMeasures)
                {
                    bool measureexists = false;
                    foreach (Measure thismeasure in _targetcube.Measures)
                        if (thismeasure.Name.ToUpper().Equals(thisquerymeasure.ToUpper()))
                            measureexists = true;
                    if (!measureexists)
                    {
                        // This measure does not exist, so raise an error
                        Errors.Add("Specified measure: " + thisquerymeasure + " does not exist in the cube "
                            + updatequery.Cube + ".");
                    }
                }

                // Now we ensure that all passed through levels exist in their particular dimension
                foreach (MDMLevel thisquerylevel in updatequery.StatementInfo.QueryLevels)
                {
                    bool levelexists = false;
                    foreach (Dimension thisdimension in _targetcube.Dimensions)
                        if (thisdimension.Name.ToUpper().Equals(thisquerylevel.LevelDimension.ToUpper()))
                        {
                            // This is the dimension specified for the level, check the level exists
                            foreach (Level thislevel in thisdimension.Levels)
                            {
                                if (thislevel.Name.ToUpper().Equals(thisquerylevel.LevelName.ToUpper()))
                                    levelexists = true;
                            }
                        }

                    if (!levelexists)
                    {
                        // This level does not exist, so raise an error
                        Errors.Add("Specified level: " + thisquerylevel.LevelName + " does not exist in the "
                            + "dimension " + thisquerylevel.LevelDimension + ".");
                    }
                }

                // Now check that all functions specified in the query are supported
                foreach (MDMFunction thisqueryfunction in updatequery.StatementInfo.QueryFunctions)
                {
                    bool functionexists = false;
                    foreach (Function thisfunction in _settings.Functions)
                        if (thisfunction.Alias.ToString().Equals(thisqueryfunction.Name.ToUpper()))
                            functionexists = true;
                    if (!functionexists)
                    {
                        // This function does not exist in our Add Ins collection, so raise an error
                        Errors.Add("Specified function: " + thisqueryfunction.Name + " is not supported by Cubed."
                            + " Consider using a third party Add In or creating your own.");
                    }
                }

                // Now we have validated the query against the cube, we check if any errors were raised. If not, 
                // we pass the query as valid - otherwise we return the errors
                if (Errors.Count > 0)
                {
                    // Errors were raised, this query is not valid
                    return false;
                }
                else
                {
                    // No errors were raised, so we assume the query has passed all validation checks
                    return true;
                }
            }
            else if (updatequery != null && updatequery.StatementInfo.QueryAxis.Length < 1)
            {
                // The query has been initialised but contains no axes so raise an error
                Errors.Add("This query contains no Axes.");
                return false;
            }
            else
            {
                // The query has not been initialised
                Errors.Add("Internal error parsing query. Please check your syntax.");
                return false;
            }
        }
        
        /// <summary>
        /// This validates a SELECT query to ensure it will successfully run against the cube
        /// </summary>
        /// <param name="selectquery">The SELECT query to validate</param>
        /// <returns>True if teh query is valid, false if not</returns>
        public bool Validate(MDMQuery selectquery)
        {
            if (selectquery != null && selectquery.StatementInfo.QueryAxis.Length > 0)
            {
                // This query has been initialised so we proceed to validate it
                Errors.Clear();

                // The first step in validation is to check that the cube exists
                // If this step in validation fails, quit execution as we cannot continue validating against an
                // unknown cube
                int cubeindex = _cubes.FindIndex(thiscube => thiscube.Name.ToUpper() == selectquery.Cube.ToUpper());                
                if (cubeindex < 0)
                {
                    // The cube doesn't exist, so raise an error
                    Errors.Add("Cube " + selectquery.Cube + " does not exist.");
                    return false;
                }
                else
                    _targetcube = _cubes[cubeindex];

                // The next step is to ensure that all the dimensions exist
                // If any dimensions don't exist, add to the error collection but don't return yet
                foreach (string thisquerydimension in selectquery.StatementInfo.QueryDimensions)
                {
                    if (_targetcube.Dimensions.FirstOrDefault(thisdim => thisdim.Name.ToUpper() ==
                        thisquerydimension.ToUpper()) == null)
                    {
                        // This dimension does not exist, so raise an error
                        Errors.Add("Specified dimension: " + thisquerydimension + " does not exist in the cube "
                            + selectquery.Cube + ".");
                    }
                }

                // Now we ensure that all measures exist in the cube                
                foreach (string thisquerymeasure in selectquery.StatementInfo.QueryMeasures)
                {
                    if (_targetcube.Measures.FirstOrDefault(thismeasure => thismeasure.Name.ToUpper() ==
                        thisquerymeasure.ToUpper()) == null)
                    {
                        // This measure does not exist, so raise an error
                        Errors.Add("Specified measure: " + thisquerymeasure + " does not exist in the cube "
                            + selectquery.Cube + ".");
                    }
                }

                // Now we ensure that all passed through levels exist in their particular dimension
                foreach (MDMLevel thisquerylevel in selectquery.StatementInfo.QueryLevels)
                {                   
                    if (_targetcube.Dimensions.FirstOrDefault(thisdim => thisdim.Name.ToUpper() ==
                        thisquerylevel.LevelDimension.ToUpper()) == null ||
                        _targetcube.Dimensions.FirstOrDefault(thisdim => thisdim.Name.ToUpper() ==
                        thisquerylevel.LevelDimension.ToUpper()).Levels.FirstOrDefault(thislevel => thislevel.Name.ToUpper() ==
                        thisquerylevel.LevelName.ToUpper()) == null)
                    {
                        // This level does not exist, so raise an error
                        Errors.Add("Specified level: " + thisquerylevel.LevelName + " does not exist in the "
                            + "dimension " + thisquerylevel.LevelDimension + ".");
                    }
                }

                // Now check that all functions specified in the query are supported
                foreach (MDMFunction thisqueryfunction in selectquery.StatementInfo.QueryFunctions)
                {
                    if (_settings.Functions.FirstOrDefault(thisfunction => thisfunction.Alias.ToUpper() ==
                        thisqueryfunction.Name.ToUpper()) == null)
                    {
                        // This function does not exist in our Add Ins collection, so raise an error
                        Errors.Add("Specified function: " + thisqueryfunction.Name + " is not supported by Cubed.");
                    }
                }

                // Now we have validated the query against the cube, we check if any errors were raised. If not, 
                // we pass the query as valid - otherwise we return the errors
                if (Errors.Count > 0)
                {
                    // Errors were raised, this query is not valid
                    return false;
                }
                else
                {
                    // No errors were raised, so we assume the query has passed all validation checks
                    return true;
                }
            }
            else if (selectquery != null && selectquery.StatementInfo.QueryAxis.Length < 1)
            {
                // The query has been initialised but contains no axes so raise an error
                Errors.Add("This query contains no Axes.");
                return false;
            }
            else
            {
                // The query has not been initialised
                Errors.Add("Internal error parsing query. Please check your syntax.");
                return false;
            }
        }
        
        /// <summary>
        /// This executes the query, if it is valid, and fills the appropriate data set depending on the type
        /// of query. It returns a boolean to determine if the query executed successfully
        /// </summary>
        /// <returns>True if the query executed successfully, false if not</returns>
        public bool Execute()
        {
            if (IsValid)
            {
                if (_querytype == MDMStatementTypes.UPDATE)
                {
                    // The query is an UPDATE query, so execute it and update the results of how many
                    // cells were updated
                    return Execute(ref _cellsupdated);
                }
                else if (_querytype == MDMStatementTypes.SELECT &&
                    _selectquery.Transactional)
                {
                    // Since this query contains a "transactional" nature we need to execute it and return the
                    // transactions associated with any of the cells requested
                    return ExecuteTransactional(ref _dataset);
                }
                else if (_querytype == MDMStatementTypes.SELECT
                            && _format == Bamboo.Cubed.Structures.XMLA.FormatTypes.Tabular
                            || (_content == Bamboo.Cubed.Structures.XMLA.ContentTypes.Schema
                            && _format == Bamboo.Cubed.Structures.XMLA.FormatTypes.Native))
                {
                    // Due to either the format or the content specified, we need to return a standard tabular
                    // dataset
                    return Execute(ref _dataset);
                }
                else if (_querytype == MDMStatementTypes.SELECT)
                {
                    // A tabular data set is not required, so we return a multi dimensional dataset
                    return Execute(ref _mddataset);
                }
                else
                {
                    // An ALTER query was requested so throw an exception
                    throw new QueryManagerException("ALTER queries are not supported.", null, String.Empty);
                }
            }
            else
                throw new QueryManagerException("The query could not be executed because it was invalid.",
                    null, String.Empty);
        }

        /// <summary>
        /// This executes the query and returns TRANSACTIONAL data for the specified cells
        /// </summary>
        /// <param name="dataset">A tabular dataset to fill with the results of the query at a transactional level</param>
        /// <returns>True if the query executed correctly, false if not</returns>
        private bool ExecuteTransactional(ref DataSet dataset)
        {
            // Check the query has been validated
            if (IsValid)
            {
                // This is a valid query, so first we execute the Multi-dimensional data set's execute method
                // to get the values returned from the cube
                Structures.XMLA.MDDataSet mddataset = new Bamboo.Cubed.Structures.XMLA.MDDataSet();
                if (Execute(ref mddataset))
                {
                    // The execution was successful, so now generate a standard DataSet with the transactional
                    // data relating to the multi-dimensional data
                    if (DataGenerator.GetTransactions(mddataset, ref dataset, TargetCube))
                    {
                        _executed = true;
                        if (_selectquery != null && dataset.Tables.Count > 0)
                        {
                            LoggingManager.Write("Multi-dimensional query", "A query was executed on the cube: " + Environment.NewLine +
                                _selectquery.Cube + Environment.NewLine + "The results returned were: " + Environment.NewLine +
                                dataset.Tables[0].Rows.Count.ToString(), LoggingTypes.Event);
                        }
                        return true;
                    }
                    else
                    {
                        // There was a problem executing the multi dimensional data set, so return an error
                        Errors.Add("Transactional data for the expression could not be retrieved. A data set can not be returned.");
                        dataset = new DataSet();
                        _executed = false;
                        return false;
                    }
                }
                else
                {
                    // There was a problem executing the multi dimensional data set, so return an error
                    Errors.Add("The multi dimensional expression could not be executed. A data set can not be returned.");
                    dataset = new DataSet();
                    _executed = false;
                    return false;
                }
            }
            else
            {
                Errors.Add("The query could not be executed because it was invalid.");
                _executed = false;
                return false;
            }
        }

        /// <summary>
        /// This executes the query, and fills the given tabular dataset with the results
        /// </summary>
        /// <param name="dataset">A tabular dataset to fill with the results of the query</param>
        /// <returns>True if the query executed correctly, false if not</returns>
        private bool Execute(ref DataSet dataset)
        {
            // Check the query has been validated
            if (IsValid)
            {
                // This is a valid query, so first we execute the Multi-dimensional data set's execute method
                // to get the values returned from the cube
                Structures.XMLA.MDDataSet mddataset = new Bamboo.Cubed.Structures.XMLA.MDDataSet();
                if (Execute(ref mddataset))
                {
                    // The multi dimensional dataset was returned successfully, so convert it to a standard flat
                    // data set
                    dataset = mddataset.ToDataSet();
                    _executed = true;
                    if (_selectquery != null && dataset.Tables.Count > 0)
                    {
                        LoggingManager.Write("Multi-dimensional query", "A query was executed on the cube: " + Environment.NewLine +
                            _selectquery.Cube + Environment.NewLine + "The results returned were: " + Environment.NewLine +
                            dataset.Tables[0].Rows.Count.ToString(), LoggingTypes.Event);
                    }
                    return true;
                }
                else
                {
                    // There was a problem executing the multi dimensional data set, so return an error
                    Errors.Add("The multi dimensional expression could not be executed. A data set can not be returned.");
                    dataset = new DataSet();
                    _executed = false;
                    return false;
                }
            }
            else
            {
                Errors.Add("The query could not be executed because it was invalid.");
                _executed = false;
                return false;
            }
        }

        /// <summary>
        /// This executes the query, and fills the given multi dimensional dataset with the results
        /// </summary>
        /// <param name="mddataset">A multi dimensional dataset to fill with the results of the query</param>
        /// <returns>True if the query executed correctly, false if not</returns>
        private bool Execute(ref Structures.XMLA.MDDataSet mddataset)
        {
            // Check the query has been validated
            if (IsValid)
            {
                // Before starting any in-depth query analysis, check if this user has access to this cube
                if (!SecurityManager.CanReadCube(_targetcube))
                {
                    Errors.Add("Security Violation: User " + SecurityManager.AuthenticatedUser.UserName + " is not permitted READ access to the cube ("
                        + _targetcube.Name + "). This query cannot be executed.");
                    return false;
                }

                // Initialise our new Multi dimensional data set
                mddataset = new Structures.XMLA.MDDataSet();                
                                                
                // Check whether any axes have been specified with special keywords, in which case we need
                // to order them appropriately. First we look for a COLUMN axis
                foreach (MDMQueryAxis thisaxis in _selectquery.Axes)
                    if (thisaxis.AxisType == MDMQueryAxisType.COLUMNS)
                    {
                        // Call the axis creator to add the column axis first
                        if (!AxisCreator.CreateAxis(ref mddataset, thisaxis, true, _selectquery.CalculatedMembers,
                            Errors, _targetcube, false, _username))
                        {
                            _executed = false;
                            return false;
                        }
                    }

                // Then we look for a ROW axis
                foreach (MDMQueryAxis thisaxis in _selectquery.Axes)
                    if (thisaxis.AxisType == MDMQueryAxisType.ROWS)
                    {
                        // Call the axis creator to add the row axis next
                        if (!AxisCreator.CreateAxis(ref mddataset, thisaxis, true, _selectquery.CalculatedMembers,
                            Errors, _targetcube, false, _username))
                        {
                            _executed = false;
                            return false;
                        }
                    }

                // Next we look for a PAGE axis
                foreach (MDMQueryAxis thisaxis in _selectquery.Axes)
                    if (thisaxis.AxisType == MDMQueryAxisType.PAGES)
                    {
                        // Call the axis creator to add the page axis next
                        if (!AxisCreator.CreateAxis(ref mddataset, thisaxis, true, _selectquery.CalculatedMembers,
                            Errors, _targetcube, false, _username))
                        {
                            _executed = false;
                            return false;
                        }
                    }

                // Then we look for a CHAPTER axis
                foreach (MDMQueryAxis thisaxis in _selectquery.Axes)
                    if (thisaxis.AxisType == MDMQueryAxisType.CHAPTERS)
                    {
                        // Call the axis creator to add the chapter axis next
                        if (!AxisCreator.CreateAxis(ref mddataset, thisaxis, true, _selectquery.CalculatedMembers,
                            Errors, _targetcube, false, _username))
                        {
                            _executed = false;
                            return false;
                        }
                    }

                // Next we look for a SECTION axis
                foreach (MDMQueryAxis thisaxis in _selectquery.Axes)
                    if (thisaxis.AxisType == MDMQueryAxisType.SECTIONS)
                    {
                        // Call the axis creator to add the section axis next
                        if (!AxisCreator.CreateAxis(ref mddataset, thisaxis, true, _selectquery.CalculatedMembers,
                            Errors, _targetcube, false, _username))
                        {
                            _executed = false;
                            return false;
                        }
                    }

                // Finally we look for any remaining axes which are not SLICERS
                foreach (MDMQueryAxis thisaxis in _selectquery.Axes)
                    if (thisaxis.AxisType != MDMQueryAxisType.SLICER)
                    {
                        // Check if the axis was added already (if it was the first specified special named axis
                        bool alreadyexists = false;
                        foreach (Structures.XMLA.Axis thisqueryaxis in mddataset.Axes)
                            if (thisqueryaxis.Name == thisaxis.AxisName)
                                alreadyexists = true;

                        // If it hasn't already been added, we add it specifically
                        if (!alreadyexists)
                        {
                            if (!AxisCreator.CreateAxis(ref mddataset, thisaxis, true, _selectquery.CalculatedMembers,
                                Errors, _targetcube, false, _username))
                            {
                                _executed = false;
                                return false;
                            }
                        }
                    }                

                // Then, if a slicer axis has been specified, set it in the data set
                if (_selectquery.SlicerAxis != null && _selectquery.SlicerAxis.AxisName.Length > 0
                    && _selectquery.SlicerAxis.AxisType == MDMQueryAxisType.SLICER)
                {
                    // A slicer axis has been specified, so update our data set to reflect it
                    if (!AxisCreator.CreateAxis(ref mddataset, _selectquery.SlicerAxis, false,
                        _selectquery.CalculatedMembers, Errors, _targetcube, true, _username))
                    {
                        _executed = false;
                        return false;
                    }                        
                }
                else
                    mddataset.FilterAxis = null;

                // We have now created our axes collection in our data set, lets add the info collections
                mddataset.OlapInfo.CubeInfo.MDDataSetCube.CubeName = _targetcube.Name;
                
                // Now the information about each axis
                foreach (Structures.XMLA.Axis thisaxis in mddataset.Axes)
                {
                    // First we create the new axis info and name it (based on the name of the axis
                    Structures.XMLA.AxisInfo newaxisinfo = new Structures.XMLA.AxisInfo();
                    newaxisinfo.Name = thisaxis.Name;

                    // Then we add a hierarchy info collection for each unique dimension represented on this
                    // axis                    
                    foreach (Structures.XMLA.MemberType thismember in thisaxis.Members)
                    {
                        // Check whether this dimension already exists in the hierarchy info for this axis
                        bool alreadyrepresented = false;
                        foreach (Structures.XMLA.DimensionInfo thisdimension in newaxisinfo.Dimensions)
                            if (thisdimension.Name.ToUpper().Equals(thismember.Dimension.ToUpper()))
                                alreadyrepresented = true;

                        // If it isn't already represented, we add it
                        if (!alreadyrepresented)
                        {                            
                            Structures.XMLA.DimensionInfo newdimension = new Structures.XMLA.DimensionInfo();
                            
                            // First look for the dimension in the cube dimensions collection
                            Dimension thisdimension = _targetcube.Dimensions.List.Find(testdim =>
                                testdim.Name.ToUpper() == thismember.Dimension.ToUpper());
                            if (thisdimension != null)
                            {
                                // This is our dimension so lets add it
                                newdimension.Caption = thisdimension.Caption;
                                newdimension.DisplayInfo = true;
                                newdimension.LName = thisdimension.Description;
                                newdimension.Name = thisdimension.Name;
                                newdimension.UName = thisdimension.ID;
                                newaxisinfo.Dimensions.Add(newdimension);
                            }
                            else
                            {
                                // As we couldn't find the dimension in the dimensions collection, look
                                // for it in the measures collection
                                Measure thismeasure = _targetcube.Measures.List.Find(testmeasure =>
                                    testmeasure.Name.ToUpper() == thismember.LName.ToUpper());
                                if (thismeasure != null)
                                {
                                    newdimension.Caption = thismeasure.Name;
                                    newdimension.DisplayInfo = true;
                                    newdimension.LName = thismeasure.Name;
                                    newdimension.Name = thismeasure.Name;
                                    newdimension.UName = thismeasure.ID;
                                    newaxisinfo.Dimensions.Add(newdimension);
                                }
                            }
                        }
                    }

                    // Now our axis info is prepared we add it to the collection
                    mddataset.OlapInfo.AxesInfo.Add(newaxisinfo);
                }                

                // Now we have updated our basic cube info, and found our axis, we check to see if there
                // other features (such as named sets and calculated members) to consider
                if (_selectquery.NamedSets != null)
                    foreach (MDMNamedSet thisset in _selectquery.NamedSets)
                    {
                        // We assume that we need to add this named set as a new axis or its own, so first we
                        // create the axis
                        MDMQueryAxis namedaxis = new MDMQueryAxis();
                        namedaxis.AxisName = thisset.Name;
                        namedaxis.AxisType = MDMQueryAxisType.COLUMNS;
                        namedaxis.Set = thisset.Set;
                        namedaxis.SetFunction = thisset.Function;

                        // Now we pass the axis to the Axis Creator to return a dataset axis
                        AxisCreator.CreateAxis(ref mddataset, namedaxis, true, _selectquery.CalculatedMembers,
                            Errors, _targetcube, false, _username);
                    }
                
                // The final step before processing the data to fill the multi-dimensional data set is to
                // ensure that an axis exists with at least one measure in it
                if (_selectquery.StatementInfo.QueryMeasures.Count() < 1)
                {
                    // No measures have been specified, so add a new axis as the measure axis
                    if (_targetcube.Measures.Count > 0)
                    {
                        MDMQueryAxis measureaxis = new MDMQueryAxis();

                        // Determine how many axes there already are and name the axis appropriately
                        if (_selectquery.Axes.Count < 2)
                        {
                            measureaxis.AxisName = "ROWS";
                            measureaxis.AxisType = MDMQueryAxisType.ROWS;
                        }
                        else if (_selectquery.Axes.Count < 3)
                        {
                            measureaxis.AxisName = "PAGES";
                            measureaxis.AxisType = MDMQueryAxisType.PAGES;
                        }
                        else if (_selectquery.Axes.Count < 4)
                        {
                            measureaxis.AxisName = "CHAPTERS";
                            measureaxis.AxisType = MDMQueryAxisType.CHAPTERS;
                        }
                        else
                        {
                            measureaxis.AxisName = "SECTIONS";
                            measureaxis.AxisType = MDMQueryAxisType.SECTIONS;
                        }                        
                        
                        // Add a single member to a single tuple of a single set for the measure
                        measureaxis.Set = new MDMSet();
                        MDMTuple measuretuple = new MDMTuple();
                        measuretuple.Dimensionality = 1;
                        MDMMember measuremember = new MDMMember();
                        measuremember.MemberDimension = "MEASURES";
                        measuremember.Relationship = MemberRelationship.Self;
                        measuremember.MemberName = _targetcube.Measures[0].Name;
                        measuretuple.Members = new MDMMember[] { measuremember };
                        measureaxis.Set.Tuples = new MDMTuple[] { measuretuple };
                        _selectquery.Axes.Add(measureaxis);
                        _selectquery.StatementInfo.QueryMeasures = new string[] { _targetcube.Measures[0].Name };

                        // Then add the new measure to the measure axis on the MDDataSet
                        Bamboo.Cubed.Structures.XMLA.Axis MeasureAxis = new Bamboo.Cubed.Structures.XMLA.Axis();
                        MeasureAxis.Name = measureaxis.AxisName;
                        Bamboo.Cubed.Structures.XMLA.TupleType MeasureTuple = new Bamboo.Cubed.Structures.XMLA.TupleType();
                        Bamboo.Cubed.Structures.XMLA.MemberType MeasureMember = new Bamboo.Cubed.Structures.XMLA.MemberType();
                        MeasureMember.Caption = _targetcube.Measures[0].Name;
                        MeasureMember.Dimension = "MEASURES";
                        MeasureMember.Hierarchy = "MEASURES";
                        MeasureMember.LName = _targetcube.Measures[0].Name;
                        MeasureMember.UName = _targetcube.Measures[0].ID;
                        MeasureTuple.Members.Add(MeasureMember);
                        MeasureAxis.Tuples.Add(MeasureTuple);
                        MeasureAxis.Members.Add(MeasureMember);
                        mddataset.Axes.Add(MeasureAxis);
                        
                        // And add an axis info for this measure axis
                        Bamboo.Cubed.Structures.XMLA.AxisInfo MeasureAxisInfo = new Bamboo.Cubed.Structures.XMLA.AxisInfo();
                        Bamboo.Cubed.Structures.XMLA.DimensionInfo MeasureDimensionInfo = new Bamboo.Cubed.Structures.XMLA.DimensionInfo();
                        MeasureDimensionInfo.Caption = "MEASURES";
                        MeasureDimensionInfo.LName = "MEASURES";
                        MeasureDimensionInfo.Name = "MEASURES";
                        MeasureDimensionInfo.UName = new Guid();
                        MeasureAxisInfo.Dimensions.Add(MeasureDimensionInfo);
                        MeasureAxisInfo.Name = "MEASURES";
                        mddataset.OlapInfo.AxesInfo.Add(MeasureAxisInfo);                        
                    }
                    else
                    {
                        // The cube contains no measures, so raise an error and exit
                        Errors.Add("The query could not be executed as the source cube contains no measures.");
                        _executed = false;
                        return false;
                    }
                }

                // Now we need to go through the target cube, and add any remaining dimensions, which aren't
                // represented explicitly, onto the filter axis 
                foreach (Dimension thisdimension in _targetcube.Dimensions)
                {
                    if (!_selectquery.Axes.Exists(testaxis => testaxis.Set.Tuples.FirstOrDefault(testtuple =>
                        testtuple.Members.FirstOrDefault(testmember => testmember.MemberDimension.ToUpper() ==
                            thisdimension.Name.ToUpper()) != null) != null) &&
                        (_selectquery.SlicerAxis == null ||
                        _selectquery.SlicerAxis.Set.Tuples.FirstOrDefault(testtuple =>
                        testtuple.Members.FirstOrDefault(testmember => testmember.MemberDimension.ToUpper() ==
                            thisdimension.Name.ToUpper()) != null) == null))
                    {
                        // This dimension is not represented anywhere in this query so add it's 
                        // default member to the slicer axis of the select query and the 
                        // MD Data Set
                        Member defaultmember = thisdimension.DefaultMember;
                        if (defaultmember == null && thisdimension.Members.Count > 0)
                            defaultmember = thisdimension.Members[0];

                        if (defaultmember != null)
                        {
                            // There is a default member for this dimension so add it to the select query
                            // first
                            if (_selectquery.SlicerAxis == null)
                                _selectquery.SlicerAxis = new MDMQueryAxis();
                            if (_selectquery.SlicerAxis.Set == null)
                                _selectquery.SlicerAxis.Set = new MDMSet();
                            List<MDMTuple> tuples = new List<MDMTuple>();
                            if (_selectquery.SlicerAxis.Set.Tuples != null)
                                tuples.AddRange(_selectquery.SlicerAxis.Set.Tuples);
                            MDMTuple newtuple = new MDMTuple();
                            newtuple.Dimensionality = 1;
                            MDMMember newmember = new MDMMember();
                            newmember.MemberDimension = thisdimension.Name;
                            newmember.MemberDepth = 0;
                            newmember.MemberName = defaultmember.Name;                            
                            newmember.Relationship = MemberRelationship.Self;
                            newtuple.Members = new MDMMember[] { newmember };
                            tuples.Add(newtuple);
                            _selectquery.SlicerAxis.Set.Tuples = tuples.ToArray();

                            // Then add it to the MDDataSet
                            if (mddataset.FilterAxis == null)
                                mddataset.FilterAxis = new Bamboo.Cubed.Structures.XMLA.Axis();
                            if (mddataset.FilterAxis.Members == null)
                                mddataset.FilterAxis.Members = new Bamboo.Cubed.Structures.XMLA.MembersType();
                            if (mddataset.FilterAxis.Tuples == null)
                                mddataset.FilterAxis.Tuples = new List<Bamboo.Cubed.Structures.XMLA.TupleType>();
                            Bamboo.Cubed.Structures.XMLA.MemberType newquerymember = new Bamboo.Cubed.Structures.XMLA.MemberType();
                            newquerymember.Caption = defaultmember.Name;
                            newquerymember.Dimension = thisdimension.Name;
                            newquerymember.Hierarchy = thisdimension.Name;                            
                            newquerymember.UName = defaultmember.ID;
                            newquerymember.LNum = Convert.ToUInt32(defaultmember.Level);
                            newquerymember.LNumSpecified = true;
                            newquerymember.LName = thisdimension.Levels[defaultmember.Level].Name;
                            Bamboo.Cubed.Structures.XMLA.TupleType newquerytuple = new Bamboo.Cubed.Structures.XMLA.TupleType();                           
                            newquerytuple.Members.Add(newquerymember);
                            mddataset.FilterAxis.Tuples.Add(newquerytuple);
                            mddataset.FilterAxis.Members.Add(newquerymember);
                        }                        
                    }
                }               

                // Finally we need to find the measure axis in the MDDataSet and normalise it by
                // adding the members of the measure axis to the measures collection
                Bamboo.Cubed.Structures.XMLA.Axis querymeasureaxis = mddataset.Axes.AxisCollection.Find(thisaxis =>
                    thisaxis.Tuples.Exists(thistuple => thistuple.Members.MemberCollection.Exists(thismember =>
                        thismember.Dimension.ToUpper() == "MEASURES")));
                if (querymeasureaxis != null)
                {
                    // This is the measure axis, so add each of its' members as a fully qualified 
                    // measure
                    foreach (Bamboo.Cubed.Structures.XMLA.MemberType thismember in querymeasureaxis.Members.MemberCollection)
                    {
                        Bamboo.Cubed.Structures.XMLA.MeasureType newmeasure = new Bamboo.Cubed.Structures.XMLA.MeasureType();
                        newmeasure.Name = thismember.Caption;
                        newmeasure.LName = thismember.LName;
                        newmeasure.UName = thismember.UName;
                        querymeasureaxis.Measures.Add(newmeasure);
                    }
                }

                // We now have a fully constructed, multi-dimensional data set without any data. Now we 
                // can pass this data set to our query data generator to return the data
                if (DataGenerator.Fill(ref mddataset, _targetcube, Errors, _settings, _username, 
                    Properties.IncludeZeros))
                {
                    // We were successfully able to fill the data set with data, so return true to indicate
                    // this query has been executed successfully
                    _executed = true;
                    if (_selectquery != null && mddataset.Cells != null && mddataset.Axes != null)
                    {
                        LoggingManager.Write("Multi-dimensional query", "A query was executed on the cube: " + Environment.NewLine +
                            _selectquery.Cube + Environment.NewLine + "The results returned were: " + Environment.NewLine +
                            "Axes: " + mddataset.Axes.Count.ToString() + ", Total Cells: " + mddataset.Cells.Count.ToString(), LoggingTypes.Event);
                    }
                    return true;
                }
                else
                {
                    // An error occured while trying to fill the data set with data, so return false to indicate
                    // the query did not execute successfully
                    _executed = false;
                    return false;
                }
            }
            else
            {
                Errors.Add("The query could not be executed because it was invalid.");
                _executed = false;
                return false;
            }
        }

        /// <summary>
        /// This executes the query, and fills the cells updated to reflect how many cells have been updated
        /// </summary>
        /// <param name="cellsupdated">The number of cells updated</param>
        /// <returns>True if the query was successful, false if there was a problem</returns>
        private bool Execute(ref long cellsupdated)
        {
            // Check the query has been validated
            if (IsValid)
            {
                // Before starting any in-depth query analysis, check if this user has access to this cube
                if (!SecurityManager.CanWriteCube(_targetcube))
                {
                    Errors.Add("Security Violation: User " + SecurityManager.AuthenticatedUser.UserName + " is not permitted WRITE access to the cube "
                        + _targetcube.Name + ". This query cannot be executed.");
                    return false;
                }
                
                // Everything has been validated, so update the value as requested in the data matrix
                double newvalue = double.Parse(_updatequery.NewValue);
                cellsupdated = 0;

                // Now we have the new value to update this cell to, we need to find the every cell referenced
                // by the update query
                foreach (string _thismeasure in _updatequery.StatementInfo.QueryMeasures)
                    foreach (Measure thismeasure in _targetcube.Measures)
                        if (thismeasure.Name.ToUpper().Equals(_thismeasure.ToUpper()))
                        {
                            // We have found one measure of the update statement, so now we need to cycle through
                            // our cube, and try and find the selected members for each dimension of the cube, or
                            // if the query doesn't specify them, we need to select the defaults
                            Node updatenode = new Node();
                            updatenode.Measure = thismeasure;
                            updatenode.Value = newvalue;                            
                            List<Dimension> explicitdimensions = new List<Dimension>();

                            foreach (Dimension thisdimension in _targetcube.Dimensions)
                                foreach (MDMMember _thismember in _updatequery.StatementInfo.QueryMembers)
                                    if (_thismember.MemberDimension.ToUpper().Equals(thisdimension.Name.ToUpper()))
                                    {
                                        // The current query member belongs to this dimension, so lets find the
                                        // member in the dimension's member collection
                                        foreach (Member thismember in thisdimension.Members)
                                            if (thismember.Name.ToUpper().Equals(_thismember.MemberName.ToUpper()))
                                            {
                                                // Check that the user has access to write back to this member
                                                if (!SecurityManager.CanWriteMember(_targetcube, thismember))
                                                {
                                                    Errors.Add("Security Violation: User " + SecurityManager.AuthenticatedUser.UserName +
                                                        " is not permitted to write back to the member (" +
                                                        thismember.Name + "). This query cannot be " +
                                                        "executed.");
                                                    return false;
                                                }
                                                
                                                // The member has been found; lets add it to the collection
                                                updatenode.Members.Add(thismember);

                                                // Because this member has been found, it means its' dimension
                                                // has been explicitly matched, so we need to add it to the explicit
                                                // collection
                                                if (!explicitdimensions.Contains(thisdimension))
                                                    explicitdimensions.Add(thisdimension);
                                            }
                                    }

                            // Now that we have been through all members in the query and matched them to the
                            // target cube, we can look through the list of explicit dimensions to see if there
                            // are any dimensions which have not been represented by at least one member in
                            // this query
                            foreach (Dimension thisdimension in _targetcube.Dimensions)
                                if (!explicitdimensions.Contains(thisdimension))
                                {
                                    // This dimension has not been explicitly mentioned by this query, so because
                                    // we need all dimensions represented, we need to select the default member -
                                    // which is the first one
                                    if (thisdimension.Members.Count > 0)
                                        updatenode.Members.Add(thisdimension.Members[0]);
                                }

                            // Now that we have all members set on the update node, we need to check to
                            // see whether any dimensions have more than one member specified; if they do, we 
                            // need to split the update into multiple updates
                            List<Node> allupdatenodes = UpdateQueryHelper.NodeSplitter(updatenode, _targetcube);                            

                            // Now we have split the update nodes, we need to go through and split them again
                            // based on whether they are leaf members or not
                            List<Node> fullupdatenodes = UpdateQueryHelper.UpdateValueSplitter(allupdatenodes,
                                TargetCube);

                            // Finally, since we have got a list of nodes which all need to be updated for the 
                            // current measure with the specified value, we need to update the value of the 
                            // data matrix
                            foreach (Node thisupdatenode in fullupdatenodes)                                
                            {
                                // Prior to updating this value, check that no members have any calculated members
                                // If they do - raise an error and don't run the update
                                if (UpdateQueryHelper.CheckCalculations(thisupdatenode))
                                {
                                    // The node passed the calculation check, so update it
                                    MatrixManager.SetNode(thisupdatenode, TargetCube.ID, false);

                                    // Now that we've updated the value in the Data Matrix, we increment the cells 
                                    // updated value to reflect another cell update
                                    cellsupdated++;
                                }
                                else
                                {
                                    // The node has either calculated members or a calculated measure - so raise
                                    // an error and don't update the value
                                    Errors.Add("One or more of the members, or the measure, could not be written "
                                        + "because they contained calculated members.");
                                }
                            }
                        }

                // Since we have successfully parsed the query and updated required nodes, we return true
                _executed = true;
                return true;
            }
            else
            {
                Errors.Add("The query could not be executed because it was invalid.");
                return false;
            }
        }           
    }
}