/*
 * This source file (SecurityManager.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures;
using System.Data;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.Utilities.Logging;
using Bamboo.Cubed.Engine.Persistence;
using System.Security.Principal;
using System.Threading;

namespace Bamboo.Cubed.Engine.Security
{
    /// <summary>
    /// This class provides support for accessing the currently authenticated users and roles and 
    /// their access to the Cubed server
    /// </summary>
    public static class SecurityManager
    {
        #region Internal Variables

        /// <summary>
        /// This contains all roles available in the system
        /// </summary>
        private static GenericKeyCollection<Guid, Role> _roles;

        /// <summary>
        /// This contains all users with access to the system
        /// </summary>
        private static GenericKeyCollection<Guid, User> _users;

        /// <summary>
        /// This contains all users who have already accessed Cubed during the current live session and
        /// their security details
        /// </summary>
        private static GenericKeyCollection<Guid, User> _authenticatedusers;

        #endregion

        #region In-Memory Saving Functions

        /// <summary>
        /// This updates the cached (in-memory) collection of users
        /// </summary>
        /// <param name="users">The users to update</param>
        /// <returns>True if updated, else an exception will be returned</returns>
        public static bool SaveUsers(GenericKeyCollection<Guid, User> users)
        {
            // Update the internal users collection with the one given
            _users = users;

            // Return true to indicate success
            return true;
        }

        /// <summary>
        /// This updates the cached (in-memory) collection of roles
        /// </summary>
        /// <param name="roles">The roles to update</param>
        /// <returns>True if updated, else an exception will be returned</returns>
        public static bool SaveRoles(GenericKeyCollection<Guid, Role> roles)
        {
            // Update the internal roles collection with the one given
            _roles = roles;

            // Return true to indicate success
            return true;
        }

        #endregion

        #region Database Saving Functions

        /// <summary>
        /// This saves the current (cached), in-memory collection of users to the database
        /// </summary>
        /// <returns>True if the collection has been saved, else an exception will be thrown</returns>
        public static bool SaveUsersToDatabase()
        {
            // Save the users collection to the database
            return StorageManager.SaveUsersToDatabase(_users);
        }

        /// <summary>
        /// This saves the current (cached), in-memory collection of roles to the database
        /// </summary>
        /// <returns>True if the collection has been saved, else an exception will be thrown</returns>
        public static bool SaveRolesToDatabase()
        {
            // Save the roles collection to the database
            return StorageManager.SaveRolesToDatabase(_roles);
        }

        #endregion

        #region Database Retrieval Functions

        /// <summary>
        /// This updates the current in memory cache of users from the database
        /// </summary>
        /// <returns>True if updated, else an exception will be thrown</returns>
        public static bool ReadUsersFromDatabase()
        {
            try
            {
                _users = StorageManager.ReadUsersFromDatabase();
            }
            catch
            {
                _users = new GenericKeyCollection<Guid, User>();
            }
            _authenticatedusers = new GenericKeyCollection<Guid, User>();
            return true;
        }

        /// <summary>
        /// This updates the current in memory cache of roles from the database
        /// </summary>
        /// <returns>True if updated, else an exception will be thrown</returns>
        public static bool ReadRolesFromDatabase()
        {
            try
            {
                _roles = StorageManager.ReadRolesFromDatabase();
            }
            catch
            {
                _roles = new GenericKeyCollection<Guid, Role>();
            }
            return true;
        }

        #endregion

        #region In-Memory Retrieval Functions

        /// <summary>
        /// This returns the collection of currently authenticated users in the system
        /// </summary>
        public static GenericKeyCollection<Guid, User> AuthenticatedUsers
        {
            get
            {
                if (_authenticatedusers == null)
                    _authenticatedusers = new GenericKeyCollection<Guid, User>();
                return _authenticatedusers;
            }
            set
            {
                _authenticatedusers = value;
            }
        }

        /// <summary>
        /// This returns the currently cached collection of users, reading from the database if
        /// neccessary
        /// </summary>
        public static GenericKeyCollection<Guid, User> Users
        {
            get
            {
                // If the users collection is null or empty, attempt to read it from the database
                if (_users == null || _users.Count < 1)
                {
                    _users = new GenericKeyCollection<Guid, User>();
                    ReadUsersFromDatabase();
                }

                // Then return the collection
                return _users;
            }
        }

        /// <summary>
        /// This returns the currently cached collection of roles, reading from the database if
        /// neccessary
        /// </summary>
        public static GenericKeyCollection<Guid, Role> Roles
        {
            get
            {
                // If the roles collection is null or empty, attempt to read it from the database
                if (_roles == null || _roles.Count < 1)
                {
                    _roles = new GenericKeyCollection<Guid, Role>();
                    ReadRolesFromDatabase();
                }

                // Then return the collection
                return _roles;
            }
        }

        #endregion

        #region Authentication Functions
                
        /// <summary>
        /// This authenticates the current user, ensuring that they have access to Cubed and determining
        /// what level of access to grant them
        /// </summary>
        public static User AuthenticatedUser
        {
            get
            {
                // Create our empty user record                
                User currentuser = null;

                // Attempt to authenticate the current user
                try
                {
                    // Initially check to ensure the user hasn't been authenticated before, and if
                    // they have - return their previously authenticated token
                    User preauthenticateduser = AuthenticatedUsers.FirstOrDefault(testuser =>
                    testuser.GetHashCode() == Thread.CurrentPrincipal.Identity.Name.GetHashCode() &&
                    testuser.AuthenticatedAt.AddMinutes(5) > DateTime.Now);
                    if (preauthenticateduser != null)
                        return preauthenticateduser;

                    // Check if we can obtain valid user credentials from the current thread
                    WindowsIdentity identity = (WindowsIdentity)Thread.CurrentPrincipal.Identity;
                    if (identity != null && identity.IsAuthenticated && identity.Name != String.Empty)
                    {
                        // Check if the current identity has a domain associated with it and set the user
                        // properties accordingly
                        currentuser = new User();
                        if (identity.Name.Contains("\\"))
                        {
                            // A domain has been supplied so use it
                            currentuser.Domain = identity.Name.Split('\\')[0];
                            currentuser.UserName = identity.Name.Split('\\')[1];
                        }
                        else
                        {
                            // No domain has been supplied so assume the current machine
                            currentuser.Domain = Environment.MachineName;
                            currentuser.UserName = identity.Name;
                        }                        

                        // This is the first time the user has been authenticated to the system, so check
                        // to ensure they are a valid Cubed user
                        if (SettingsManager.Settings.RootUsers.Exists(testuser => testuser.ToUpper() ==
                            currentuser.DomainUserName.ToUpper()))
                        {
                            // This user is an administrator. Test whether they have a full user record 
                            // (and use this to grab their e-mail/mobile/etc) 
                            string rootusername = currentuser.UserName;
                            string rootuserdomain = currentuser.Domain;
                            currentuser = Users.FirstOrDefault(testuser => testuser.UserName.ToUpper() ==
                                currentuser.UserName.ToUpper() && testuser.Domain.ToUpper() ==
                                currentuser.Domain.ToUpper());
                            if (currentuser != null)
                            {
                                // This administrator has a full user record, so just update it with 
                                // full admin rights and access to each cube
                                currentuser.IsAdmin = true;
                                foreach (Cube thiscube in CubeManager.ReadCubes())
                                {
                                    currentuser.ReadAccessCubes.Add(thiscube.ID);
                                    currentuser.WriteAccessCubes.Add(thiscube.ID);                                    
                                }
                                currentuser.DeniedWriteAccessMembers.Clear();
                                currentuser.DeniedReadAccessMembers.Clear();
                                currentuser.PrivateSearchRights = AccessTypes.Delete;
                                currentuser.PublicSearchRights = AccessTypes.Delete;
                            }
                            else
                            {
                                // This user has no full user record, so just create an empty default
                                // record
                                currentuser = new User();
                                currentuser.UserName = rootusername;
                                currentuser.Domain = rootuserdomain;
                                currentuser.Email = rootuserdomain + "@" + rootusername;
                                currentuser.IsAdmin = true;
                                currentuser.FullName = currentuser.DomainUserName;
                                foreach (Cube thiscube in CubeManager.ReadCubes())
                                {
                                    currentuser.ReadAccessCubes.Add(thiscube.ID);
                                    currentuser.WriteAccessCubes.Add(thiscube.ID);
                                }
                                currentuser.DeniedWriteAccessMembers.Clear();
                                currentuser.DeniedReadAccessMembers.Clear();
                                currentuser.PrivateSearchRights = AccessTypes.Delete;
                                currentuser.PublicSearchRights = AccessTypes.Delete;
                            }
                        }
                        else
                        {
                            // As this user is not an administrator, check whether they are a user in the
                            // Cubed system
                            currentuser = Users.FirstOrDefault(testuser => testuser.UserName.ToUpper() ==
                                currentuser.UserName.ToUpper() && testuser.Domain.ToUpper() ==
                                currentuser.Domain.ToUpper());
                            if (currentuser != null)
                            {
                                // The user exists, so now we need to determine their access levels
                                currentuser = CheckUserAccess(currentuser, identity);                                
                            }
                            else
                                // The user does not exist in the Cubed user collection so throw an exception
                                throw new PersistenceSecurityException("Security Violation: The authenticated user (" + identity.Name + ") does not have access to Cubed. Please ensure they have been added as a user of the system.",
                                     null, String.Empty);
                        }

                        // Now create a log entry of the user accessing
                        LoggingManager.Write("Login Record", "The user: " + currentuser.DomainUserName + 
                            " logged in successfully.", LoggingTypes.Event);

                        // Now we have a valid user from the operating system, set the hash code
                        // of the user for fast retrieval and set the authenticated at date/time
                        // to ensure user records are only cached for 5 minutes
                        currentuser.Hashcode = Thread.CurrentPrincipal.Identity.Name.GetHashCode();
                        currentuser.AuthenticatedAt = DateTime.Now;

                        // Then add the newly authenticated user record to the cached collection to
                        // avoid having to re-authenticate again
                        if (!AuthenticatedUsers.Contains(currentuser.ID))
                            AuthenticatedUsers.Add(currentuser);
                        else
                        {
                            AuthenticatedUsers.Remove(currentuser.ID);
                            AuthenticatedUsers.Add(currentuser);
                        }

                        // And finally return the user record
                        return currentuser;
                    }
                    else
                    {
                        // We can't obtain a username so throw an exception
                        throw new PersistenceSecurityException("Security Violation: No valid user credentials could be obtained. Check the user exists on the domain or server.",
                             null, String.Empty);
                    }
                }
                catch (Exception e1)
                {                    
                    // Check if we have any information about the current user
                    if (currentuser != null && currentuser.DomainUserName != String.Empty)
                        // As we know the current user, throw a specific error relating to that user
                        throw new PersistenceSecurityException("Security Violation: The authenticated user (" + currentuser.DomainUserName + ") does not have access to Cubed. Please ensure they have been added as a user of the system.",
                                 null, String.Empty);
                    else
                        // We do not know the current user credentials so throw that error
                        throw new PersistenceSecurityException("Security Violation: No valid user credentials could be obtained. Check the user exists on the domain or server." + Environment.NewLine + e1.ToString(),
                             null, String.Empty);
                }
            }
        }

        /// <summary>
        /// This determines the appropriate level of access for the user, based on their roles and any 
        /// system groups they are associated with, and returns an updated version of their record with 
        /// those details
        /// </summary>
        /// <param name="currentuser">The currently authenticated user</param>
        /// <param name="useridentity">Their Windows Identity record used to obtain their groups</param>
        /// <returns>An updated user record</returns>
        public static User CheckUserAccess(User currentuser, WindowsIdentity useridentity)
        {
            // First, we need to update the groups the user belongs to
            for (int i = 0; i < useridentity.Groups.Count; i++)
            {
                // Check whether this group has a domain associated with it
                if (useridentity.Groups[i].Value.Contains('\\'))
                {
                    // Add the full group name as it contains the domain it exists in
                    currentuser.Groups.Add(useridentity.Groups[i].Value);
                }
                else
                {
                    // As this record does not contain a domain, assume it belongs to the current system
                    currentuser.Groups.Add(Environment.MachineName + "\\" + useridentity.Groups[i].Value);
                }
            }

            // Then we need to obtain all roles which the current user, or any groups the user belongs to,
            // holds in the Cubed system
            GenericKeyCollection<Guid, Role> userroles = new GenericKeyCollection<Guid, Role>();
            foreach (Role thisrole in Roles.Where(testrole => testrole.Members.Exists(testuser =>
                testuser.DomainUserName.ToUpper() == currentuser.DomainUserName.ToUpper() ||
                currentuser.Groups.Exists(testgroup => testgroup.ToUpper() == 
                testuser.DomainUserName.ToUpper()))))
                if (!userroles.Contains(thisrole.ID))
                    userroles.Add(thisrole);

            // Now that we have a definitive collection of roles of which this user is a member (directly
            // or indirectly), we can determine the appropriate level of access. This is done in two parts,
            // for Cubed Desktop rights, the HIGHEST granted permission is taken
            foreach (Role thisrole in userroles)
            {
                // Check the private search rights
                if (Convert.ToInt32(thisrole.PrivateSearchRights) >
                    Convert.ToInt32(currentuser.PrivateSearchRights))
                {
                    // As the rights are ordered numerically, this role has a higher access level than 
                    // the current user, so update the users access
                    currentuser.PrivateSearchRights = thisrole.PrivateSearchRights;
                }

                // Check the public search rights
                if (Convert.ToInt32(thisrole.PublicSearchRights) >
                    Convert.ToInt32(currentuser.PublicSearchRights))
                {
                    // As the rights are ordered numerically, this role has a higher access level than 
                    // the current user, so update the users access
                    currentuser.PublicSearchRights = thisrole.PublicSearchRights;
                }
            }

            // To determine the access to Cubes (and individual members of Cubes), we need to cycle
            // through each cube in the system, first adding any CUBE level rights (upgrading if 
            // necessary), and then checking any DENY rights on individual members and adding them
            foreach (Cube thiscube in CubeManager.ReadCubes())
            {
                // Check to determine if the user has READ rights to this cube
                foreach (Role thisrole in userroles)
                    if (thiscube.ReadAccessRoles.Contains(thisrole.ID))
                    {
                        // Add the current cube to the users read access collection
                        if (!currentuser.ReadAccessCubes.Contains(thiscube.ID))
                            currentuser.ReadAccessCubes.Add(thiscube.ID);
                        break;
                    }

                // Check to determine if the user has WRITE rights to this cube
                foreach (Role thisrole in userroles)
                    if (thiscube.WriteAccessRoles.Contains(thisrole.ID))
                    {
                        // Add the current cube to the users write access collection
                        if (!currentuser.WriteAccessCubes.Contains(thiscube.ID))
                            currentuser.WriteAccessCubes.Add(thiscube.ID);
                        break;
                    }

                // Now determine any GRANT read access on members of the cube
                foreach (Role thisrole in userroles)
                    foreach (Dimension thisdimension in thiscube.Dimensions)
                        foreach (Member thisgrantmember in thisdimension.Members.Where(testmember =>
                            testmember.GrantedReadRoles.Contains(thisrole.ID)))
                        {
                            // Add the member to the users granted read access collection
                            if (!currentuser.GrantedReadAccessMembers.Contains(thisgrantmember.ID))
                                currentuser.GrantedReadAccessMembers.Add(thisgrantmember.ID);
                        }

                // Now determine any GRANT write access on members of the cube
                foreach (Role thisrole in userroles)
                    foreach (Dimension thisdimension in thiscube.Dimensions)
                        foreach (Member thisgrantmember in thisdimension.Members.Where(testmember =>
                            testmember.GrantedWriteRoles.Contains(thisrole.ID)))
                        {
                            // Add the member to the users granted write access collection
                            if (!currentuser.GrantedWriteAccessMembers.Contains(thisgrantmember.ID))
                                currentuser.GrantedWriteAccessMembers.Add(thisgrantmember.ID);
                        }

                // Now determine any DENY read access on members of the cube
                foreach (Role thisrole in userroles)
                    foreach (Dimension thisdimension in thiscube.Dimensions)
                        foreach (Member thisdenymember in thisdimension.Members.Where(testmember =>
                            testmember.DeniedReadRoles.Contains(thisrole.ID)))
                        {
                            // Add the member to the users denied read access collection
                            if (!currentuser.DeniedReadAccessMembers.Contains(thisdenymember.ID))
                                currentuser.DeniedReadAccessMembers.Add(thisdenymember.ID);

                            // Also add the member to the users denied write access collection (as they 
                            // cannot write if they can't read)
                            if (!currentuser.DeniedWriteAccessMembers.Contains(thisdenymember.ID))
                                currentuser.DeniedWriteAccessMembers.Add(thisdenymember.ID);

                            // Then, if this user has a matching GRANT permission, remove it - as DENY
                            // permissions take higher priority
                            if (!currentuser.GrantedReadAccessMembers.Contains(thisdenymember.ID))
                                currentuser.GrantedReadAccessMembers.Remove(thisdenymember.ID);
                        }

                // Now determine any DENY write access on members of the cube
                foreach (Role thisrole in userroles)
                    foreach (Dimension thisdimension in thiscube.Dimensions)
                        foreach (Member thisdenymember in thisdimension.Members.Where(testmember =>
                            testmember.DeniedWriteRoles.Contains(thisrole.ID)))
                        {
                            // Add the member to the users denied write access collection
                            if (!currentuser.DeniedWriteAccessMembers.Contains(thisdenymember.ID))
                                currentuser.DeniedWriteAccessMembers.Add(thisdenymember.ID);

                            // Then, if this user has a matching GRANT permission, remove it - as DENY
                            // permissions take higher priority
                            if (!currentuser.GrantedWriteAccessMembers.Contains(thisdenymember.ID))
                                currentuser.GrantedWriteAccessMembers.Remove(thisdenymember.ID);
                        }
            }

            // Now the user record has been updated, return it
            return currentuser;
        }

        /// <summary>
        /// This determines whether the currently authenticated user has access to read the given cube
        /// </summary>
        /// <param name="cube">The cube the user is trying to access</param>
        /// <returns>True if the user is permitted access, false if not</returns>
        public static bool CanReadCube(Cube cube)
        {
            if (AuthenticatedUser.ReadAccessCubes.Contains(cube.ID))
                return true;
            else
                return false;
        }

        /// <summary>
        /// This determines whether the currently authenticated user has access to writeback data to 
        /// the given cube
        /// </summary>
        /// <param name="cube">The cube the user is trying to access for writeback</param>
        /// <returns>True if the user is permitted access, false if not</returns>
        public static bool CanWriteCube(Cube cube)
        {
            // First check whether this user can read this cube, if they cannot, then they are not 
            // permitted to write either
            if (!CanReadCube(cube))
                return false;

            // Then check whether the user has permission to write to this cube
            if (AuthenticatedUser.WriteAccessCubes.Contains(cube.ID))
                return true;
            else
                return false;
        }

        /// <summary>
        /// This determines whether the currently authenticated user has access to read the given member of
        /// the given cube
        /// </summary>
        /// <param name="cube">The cube the user is trying to access</param>
        /// <param name="member">The member the user is trying to access</param>
        /// <returns>True if the user is permitted to read the member, false if not</returns>
        public static bool CanReadMember(Cube cube, Member member)
        {
            // First check whether the user can read the cube
            if (!CanReadCube(cube))
                return false;

            // Now determine if the user has any DENY read roles, if they do, then check whether this member
            // is one of those denied roles
            if (AuthenticatedUser.DeniedReadAccessMembers.Contains(member.ID))
                return false;
            else
            {
                // The user doesn't have any denied read permissions on this member, so check whether 
                // the user has any GRANT permissions. If they do, then they must contain an explicit 
                // grant for this member - if they don't, then their CUBE read access takes effect and
                // they are automatically granted
                if (AuthenticatedUser.GrantedReadAccessMembers.Count > 0 &&
                    !AuthenticatedUser.GrantedReadAccessMembers.Contains(member.ID))
                    // The user's roles have got explicit GRANTS on one or more members, which means they
                    // need to contain this member's ID as a GRANT to read it, which they don't - so return
                    // false
                    return false;
                else
                    // Either the user's roles have no explicit GRANTS, so therefore they are granted access to
                    // all members through their CUBE GRANT access, OR the user's roles have been explictly
                    // granted READ access to this member, so return true
                    return true;
            }
        }

        /// <summary>
        /// This determines whether the currently authenticated user has access to writeback data to
        /// the given member of the given cube
        /// </summary>
        /// <param name="cube">The cube the user is trying to access</param>
        /// <param name="member">The member the user is trying to access</param>
        /// <returns>True if the user is permitted to writeback to the member, false if not</returns>
        public static bool CanWriteMember(Cube cube, Member member)
        {
            // First check whether the user can writeback to the given cube
            if (!CanWriteCube(cube))
                return false;

            // Then check whether the user can READ this member, if not - then they are not permitted
            // to write
            if (!CanReadMember(cube, member))
                return false;

            // Now determine if the user has any DENY write roles, if they do, then check whether this member
            // is one of those denied roles
            if (AuthenticatedUser.DeniedWriteAccessMembers.Contains(member.ID))
                return false;
            else
            {
                // The user doesn't have any denied write permissions on this member, so check whether 
                // the user has any GRANT permissions. If they do, then they must contain an explicit 
                // grant for this member - if they don't, then their CUBE write access takes effect and
                // they are automatically granted
                if (AuthenticatedUser.GrantedWriteAccessMembers.Count > 0 &&
                    !AuthenticatedUser.GrantedWriteAccessMembers.Contains(member.ID))
                    // The user's roles have got explicit GRANTS on one or more members, which means they
                    // need to contain this member's ID as a GRANT to write it, which they don't - so return
                    // false
                    return false;
                else
                    // Either the user's roles have no explicit GRANTS, so therefore they are granted access to
                    // all members through their CUBE GRANT access, OR the user's roles have been explictly
                    // granted WRITE access to this member, so return true
                    return true;
            }
        }

        /// <summary>
        /// This determines whether the currently authenticated user has access to read the given node
        /// </summary>
        /// <param name="cube">The cube the user is trying to read</param>
        /// <param name="node">The node the user is attempting to read</param>
        /// <returns>True if the user is permitted to read the node, false if not</returns>
        public static bool CanReadNode(Cube cube, Node node)
        {
            // Cycle through each member of the given node and check the user has access. If they do not
            // have access to any member, return false
            foreach (Member thismember in node.Members)
                if (!CanReadMember(cube, thismember))
                    return false;

            // Return true as they have been granted read rights on each member
            return true;
        }

        /// <summary>
        /// This determines whether the currently authenticated user has access to writeback the given node
        /// </summary>
        /// <param name="cube">The cube the user is trying to write</param>
        /// <param name="node">The node the user is attempting to write</param>
        /// <returns>True if the user is permitted to writeback the node, false if not</returns>
        public static bool CanWriteNode(Cube cube, Node node)
        {
            // Cycle through each member of the given node and check the user has access. If they do not
            // have access to any member, return false
            foreach (Member thismember in node.Members)
                if (!CanWriteMember(cube, thismember))
                    return false;

            // Return true as they have been granted write access on each member
            return true;
        }

        #endregion

        #region Import/Export functions

        /// <summary>
        /// Exports all users from the system to a byte array ready to be saved to disk
        /// </summary>
        /// <returns>A byte array containing an export of all users in the system</returns>
        public static byte[] ExportUsers()
        {
            // First grab the serialised version of the users collection
            string users = XMLManager<List<User>>.SerialiseEncrypted(Users.List);

            // Then write the contents of the export to a byte array
            return CompressionManager.Compress(UnicodeEncoding.Unicode.GetBytes(users));
        }

        /// <summary>
        /// Imports all users from the given export into the system
        /// </summary>
        /// <param name="export">A byte array containing the exported users collection</param>
        /// <returns>The number of users which were imported (added OR changed)</returns>
        public static int ImportUsers(byte[] export)
        {
            // First grab the string representing the serialised export
            string exportedusers = UnicodeEncoding.Unicode.GetString(CompressionManager.Decompress(export));

            // Then deserialise the string into a list of users from the export
            List<User> importusers = XMLManager<List<User>>.DeserialiseEncrypted(exportedusers);

            // Now import each user
            int importcount = 0;
            foreach (User thisuser in importusers)
            {
                // Remove the current record if it already exists
                if (Users.Contains(thisuser.ID))
                    Users.Remove(thisuser.ID);

                // Then add the record
                Users.Add(thisuser);
                importcount++;
            }

            // Return the number of records imported
            return importcount;
        }

        /// <summary>
        /// Exports all roles from the system to a byte array ready to be saved to disk
        /// </summary>
        /// <returns>A byte array containing an export of all roles in the system</returns>
        public static byte[] ExportRoles()
        {
            // First grab the serialised version of the users collection
            string roles = XMLManager<List<Role>>.SerialiseEncrypted(Roles.List);

            // Then write the contents of the export to a byte array
            return CompressionManager.Compress(UnicodeEncoding.Unicode.GetBytes(roles));
        }

        /// <summary>
        /// Imports all roles from the given export into the system
        /// </summary>
        /// <param name="export">A byte array containing the roles which were exported</param>
        /// <returns>The number of roles which were imported (added OR changed)</returns>
        public static int ImportRoles(byte[] export)
        {
            // First grab the string representing the serialised export
            string exportedroles = UnicodeEncoding.Unicode.GetString(CompressionManager.Decompress(export));

            // Then deserialise the string into a list of roles from the export
            List<Role> importroles = XMLManager<List<Role>>.DeserialiseEncrypted(exportedroles);

            // Now import each role
            int importcount = 0;
            foreach (Role thisrole in importroles)
            {
                // Remove the current record if it already exists
                if (Roles.Contains(thisrole.ID))
                    Roles.Remove(thisrole.ID);

                // Then check to ensure that each user of the role exists, and if it doesn't - remove
                // it from the role
                foreach (User thisuser in thisrole.Members.Where(testuser =>
                    !Users.Contains(testuser.ID)))
                    thisrole.Members.Remove(thisuser);

                // Then add the record
                Roles.Add(thisrole);
                importcount++;
            }

            // Return the number of records imported
            return importcount;
        }

        #endregion
    }
}
