/*
 * This source file (DatasourceManager.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures;
using System.Data;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Engine.Persistence;

namespace Bamboo.Cubed.Engine.XMLA
{
    /// <summary>
    /// This class provides support for returning XMLA data sources and catalogs supported
    /// </summary>
    public static class DatasourceManager
    {
        /// <summary>
        /// Returns a list of all data sources (cubes) available in the catalog
        /// </summary>
        /// <param name="Restrictions">The restrictions used to limit the returned list</param>
        /// <param name="Properties">The properties used to return the list</param>
        /// <param name="MySettings">The Server settings file</param>
        /// <param name="UserName">The name of the user making the request</param>
        /// <param name="AppPath">The full path of the Cubed Web Service calling this function</param>        
        /// <returns>A dataset with the required data sources</returns>
        public static DataSet GetDatasources(Structures.XMLA.RestrictionList Restrictions,
            Structures.XMLA.PropertyList Properties, Structures.ServerSettings MySettings, string UserName,
            string AppPath)
        {
            // Create a new dataset with the required structure
            DataSet returnds = new DataSet();
            DataTable datasources = new DataTable("DataSources");
            datasources.Columns.Add("DataSourceName", typeof(string));
            datasources.Columns.Add("DataSourceDescription", typeof(string));
            datasources.Columns.Add("URL", typeof(string));
            datasources.Columns.Add("DataSourceInfo", typeof(string));
            datasources.Columns.Add("ProviderName", typeof(string));
            datasources.Columns.Add("ProviderType", typeof(List<Structures.XMLA.ProviderTypes>));
            datasources.Columns.Add("AuthenticationMode", typeof(Structures.XMLA.AuthenticationModeTypes));

            // Set the default data provider types
            List<Structures.XMLA.ProviderTypes> SupportedProviders = new List<Structures.XMLA.ProviderTypes>();
            SupportedProviders.Add(Structures.XMLA.ProviderTypes.MDP);
            SupportedProviders.Add(Structures.XMLA.ProviderTypes.TDP);

            // Now add available datasources (catalogs) based on the user and what they have permission to access
            List<Cube> cubes = CubeManager.ReadCubes();
            foreach (Cube thiscube in cubes)
            {
                // First determine if this cube matches the restriction list
                bool isrestricted = false;
                if (Restrictions.DataSourceName != null && Restrictions.DataSourceName != String.Empty
                    && !thiscube.Name.Contains(Restrictions.DataSourceName))
                    isrestricted = true;
                if (Restrictions.DataSourceDescription != null && Restrictions.DataSourceDescription != String.Empty
                    && !thiscube.Description.Contains(Restrictions.DataSourceName))
                    isrestricted = true;
                foreach (Structures.XMLA.ProviderTypes thisprovider in Restrictions.ProviderType)
                {
                    if (thisprovider == Structures.XMLA.ProviderTypes.DMP)
                        isrestricted = true;
                }
                if (Restrictions.AuthenticationMode != Structures.XMLA.AuthenticationModeTypes.Integrated)
                    isrestricted = true;
                if (Restrictions.URL != null && Restrictions.URL != String.Empty && Restrictions.URL.ToUpper()
                    != AppPath.ToUpper())
                    isrestricted = true;

                // If it has matched all restrictions so far, then we can continue; if it has been restricted and
                // ruled out, then we ignore it
                if (!isrestricted)
                {
                    // This cube hasn't been restricted by the settings passed through so we should return it
                    DataRow cuberow = datasources.NewRow();
                    cuberow["DataSourceName"] = thiscube.Name;
                    cuberow["DataSourceDescription"] = thiscube.Description;
                    cuberow["URL"] = AppPath;
                    cuberow["DataSourceInfo"] = "Initial Catalog=" + thiscube.Name + ";";
                    cuberow["ProviderName"] = AddInManager.Connectors[MySettings.StorageProvider.ConnectorID].Name;
                    cuberow["ProviderType"] = SupportedProviders;
                    cuberow["AuthenticationMode"] = Structures.XMLA.AuthenticationModeTypes.Integrated;
                    datasources.Rows.Add(cuberow);
                }
            }

            // Return our data set
            returnds.Tables.Add(datasources);
            return returnds;
        }

        /// <summary>
        /// Returns a list of all catalogs (cubes) available to the user making the request
        /// </summary>
        /// <param name="Restrictions">The restrictions used to limit the returned list</param>
        /// <param name="Properties">The properties used to return the list</param>
        /// <param name="MySettings">The Server settings file</param>
        /// <param name="UserName">The name of the user making the request</param>
        /// <param name="AppPath">The full path of the Cubed Web Service calling this function</param>        
        /// <returns>A dataset with the required data sources</returns>
        public static DataSet GetCatalogs(Structures.XMLA.RestrictionList Restrictions,
            Structures.XMLA.PropertyList Properties, Structures.ServerSettings MySettings, string UserName,
            string AppPath)
        {
            // Create a new dataset with the required structure
            DataSet returnds = new DataSet();
            DataTable catalogs = new DataTable("Catalogs");
            catalogs.Columns.Add("CatalogName", typeof(string));

            // Now add available datasources (catalogs) based on the user and what they have permission to access
            List<Cube> cubes = CubeManager.ReadCubes();
            foreach (Cube thiscube in cubes)
            {
                // First determine if this cube matches the restriction list
                bool isrestricted = false;
                if (Restrictions.CatalogName != null && Restrictions.CatalogName != String.Empty
                    && !thiscube.Name.Contains(Restrictions.CatalogName))
                    isrestricted = true;

                // If it has matched all restrictions so far, then we can continue; if it has been restricted and
                // ruled out, then we ignore it
                if (!isrestricted)
                {
                    // This cube hasn't been restricted by the settings passed through so we should return it
                    DataRow cuberow = catalogs.NewRow();
                    cuberow["Catalog_Name"] = thiscube.Name;
                    catalogs.Rows.Add(cuberow);
                }
            }

            // Return our data set
            returnds.Tables.Add(catalogs);
            return returnds;
        }
    }
}
