/*
 * This source file (DiscoveryManager.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Engine.Query;
using System.Data;
using Bamboo.Cubed.AddIns;

namespace Bamboo.Cubed.Engine.XMLA
{
    /// <summary>
    /// This class provides support for the XMLA Discovery method of querying a server
    /// </summary>
    public static class DiscoveryManager
    {
        /// <summary>
        /// This returns the results of an XMLA Discover function in a dataset
        /// </summary>
        /// <param name="RequestType">The type of request</param>
        /// <param name="Restrictions">A set of restrictions to filter the returned data with</param>
        /// <param name="Properties">The set of properties to use</param>
        /// <param name="MySettings">The server settings</param>
        /// <param name="UserName">The name of the user executing the query</param>
        /// <param name="AppPath">The full path of the Cubed Web Service making this request</param>
        /// <param name="MyConnection">The connection parameters of this connection</param>
        /// <returns>A DataSet containing the results of the query</returns>
        public static DataSet Discover(Structures.XMLA.RequestTypes RequestType, Structures.XMLA.RestrictionList Restrictions,
            Structures.XMLA.PropertyList Properties, Structures.ServerSettings MySettings, string UserName,
            string AppPath, ConnectionParameters MyConnection)
        {
            // The user should already be authenticated so we just need to return the correct data set
            // based on the request type
            switch (RequestType)
            {
                case Structures.XMLA.RequestTypes.DISCOVER_DATASOURCES:
                    // Returns a list of the data sources (cubes) which exist in the catalog
                    return DatasourceManager.GetDatasources(Restrictions, Properties, MySettings, UserName, AppPath);
                case Structures.XMLA.RequestTypes.DISCOVER_ENUMERATORS:
                    // Returns a list of the enumerators used by Cubed
                    return EnumerationManager.GetEnumerators(Restrictions, Properties, MySettings, UserName, AppPath);
                case Structures.XMLA.RequestTypes.DISCOVER_KEYWORDS:
                    break;
                case Structures.XMLA.RequestTypes.DISCOVER_LITERALS:
                    break;
                case Structures.XMLA.RequestTypes.DISCOVER_PROPERTIES:
                    // Returns a list of the properties which are supported by Cubed
                    return PropertyManager.GetProperties(Restrictions, Properties, MySettings, UserName, AppPath, MyConnection);
                case Structures.XMLA.RequestTypes.DISCOVER_SCHEMA_ROWSETS:
                    // Returns a list of the schema rowsets (from this enumeration) which are supported
                    return SchemaManager.GetSchemaRowsets(Restrictions, Properties, MySettings, UserName, AppPath);
                case Structures.XMLA.RequestTypes.DBSCHEMA_CATALOGS:
                    // Returns a list of the catalogs (cubes) which are present on the Cubed server
                    return DatasourceManager.GetCatalogs(Restrictions, Properties, MySettings, UserName, AppPath);
                case Structures.XMLA.RequestTypes.MDSCHEMA_ACTIONS:
                    break;
                case Structures.XMLA.RequestTypes.MDSCHEMA_CUBES:
                    break;
                case Structures.XMLA.RequestTypes.MDSCHEMA_DIMENSIONS:
                    break;
                case Structures.XMLA.RequestTypes.MDSCHEMA_FUNCTIONS:
                    break;
                case Structures.XMLA.RequestTypes.MDSCHEMA_HIERARCHIES:
                    break;
                case Structures.XMLA.RequestTypes.MDSCHEMA_MEASURES:
                    break;
                case Structures.XMLA.RequestTypes.MDSCHEMA_MEMBERS:
                    break;
                case Structures.XMLA.RequestTypes.MDSCHEMA_PROPERTIES:
                    break;
                case Structures.XMLA.RequestTypes.MDSCHEMA_SETS:
                    break;
                case Structures.XMLA.RequestTypes.MDSCHEMA_KPIS:
                    break;
            }

            // If no valid request type has been specified, return an empty dataset
            return new DataSet();
        }
    }
}
