/*
 * This source file (PropertyManager.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures;
using System.Data;
using Bamboo.Cubed.AddIns;

namespace Bamboo.Cubed.Engine.XMLA
{
    /// <summary>
    /// This class provides support for returning XMLA properties supported
    /// </summary>
    public static class PropertyManager
    {
        /// <summary>
        /// Returns a list of all properties available to this provider
        /// </summary>
        /// <param name="Restrictions">The restrictions used to limit the returned list</param>
        /// <param name="Properties">The properties used to return the list</param>
        /// <param name="MySettings">The Server settings file</param>
        /// <param name="UserName">The name of the user making the request</param>
        /// <param name="AppPath">The full path of the Cubed Web Service calling this function</param>
        /// <param name="MyConnection">The connection parameters of the current connection</param>
        /// <returns>A dataset with the required data sources</returns>
        public static DataSet GetProperties(Structures.XMLA.RestrictionList Restrictions,
            Structures.XMLA.PropertyList Properties, Structures.ServerSettings MySettings, string UserName,
            string AppPath, ConnectionParameters MyConnection)
        {
            // Create a new dataset with the required structure
            DataSet returnds = new DataSet();
            DataTable properties = new DataTable("Properties");
            properties.Columns.Add("PropertyName", typeof(string));
            properties.Columns.Add("PropertyDescription", typeof(string));
            properties.Columns.Add("PropertyType", typeof(string));
            properties.Columns.Add("PropertyAccessType", typeof(Structures.XMLA.PropertyAccessTypes));
            properties.Columns.Add("IsRequired", typeof(Boolean));
            properties.Columns.Add("Value", typeof(string));

            // Check whether the properties to return have been restricted
            List<string> restrictedproperties = new List<string>();
            foreach (string thisproperty in Restrictions.PropertyName)
                restrictedproperties.Add(thisproperty.ToUpper());

            // Add requested, supported properties to the data set
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("ADMIN"))
                properties.Rows.Add(new object[] { "Admin", "Defines whether a user is connector as an administator or not",
                "Boolean", Structures.XMLA.PropertyAccessTypes.Read, false, MyConnection.Admin.ToString() });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("AXISFORMAT"))
                properties.Rows.Add(new object[] { "AxisFormat", "Defines the axis format to return query results in",
                "EnumString", Structures.XMLA.PropertyAccessTypes.Write, true, Properties.AxisFormat.ToString() });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("BEGINRANGE"))
                properties.Rows.Add(new object[] { "BeginRange", "Defines the start of the cell range to return",
                "Integer", Structures.XMLA.PropertyAccessTypes.Write, false, Properties.BeginRange.ToString() });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("CATALOG"))
                properties.Rows.Add(new object[] { "Catalog", "The name of the cube which is currently connected",
                "String", Structures.XMLA.PropertyAccessTypes.ReadWrite, false, MyConnection.Cube });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("CONNECTIONSTRING"))
                properties.Rows.Add(new object[] { "ConnectionString", "The connection string used to establish this connection",
                "String", Structures.XMLA.PropertyAccessTypes.ReadWrite, false, MyConnection.ConnectionString });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("CONTENT"))
                properties.Rows.Add(new object[] { "Content", "The type of content requested",
                "EnumString", Structures.XMLA.PropertyAccessTypes.Write, false, Properties.Content.ToString() });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("CUBE"))
                properties.Rows.Add(new object[] { "Cube", "The cube which has been connected to",
                "String", Structures.XMLA.PropertyAccessTypes.ReadWrite, false, MyConnection.Cube });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("DATASOURCEINFO"))
                properties.Rows.Add(new object[] { "DataSourceInfo", "The specific data source info for this cube",
                "String", Structures.XMLA.PropertyAccessTypes.ReadWrite, false, Properties.DataSourceInfo.ToString() });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("ENDRANGE"))
                properties.Rows.Add(new object[] { "EndRange", "Defines the end of the cell range to return",
                "Integer", Structures.XMLA.PropertyAccessTypes.Write, false, Properties.EndRange.ToString() });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("FORMAT"))
                properties.Rows.Add(new object[] { "Format", "The format of the returned result set",
                "EnumString", Structures.XMLA.PropertyAccessTypes.Write, false, Properties.Format.ToString() });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("LOCALEIDENTIFIER"))
                properties.Rows.Add(new object[] { "LocaleIdentifier", "The identifier of the current locale",
                "Integer", Structures.XMLA.PropertyAccessTypes.ReadWrite, false, Properties.LocaleIdentifier.ToString() });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("MDXSUPPORT"))
                properties.Rows.Add(new object[] { "MDXSupport", "The extent of MDX Support provided by this provider",
                "EnumString", Structures.XMLA.PropertyAccessTypes.Read, false, Properties.MDXSupport.ToString() });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("PROVIDERNAME"))
                properties.Rows.Add(new object[] { "ProviderName", "The name of the provider",
                "String", Structures.XMLA.PropertyAccessTypes.Read, false, Properties.ProviderName });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("PROVIDERVERSION"))
                properties.Rows.Add(new object[] { "ProviderVersion", "The version of the provider",
                "String", Structures.XMLA.PropertyAccessTypes.Write, false, Properties.ProviderVersion });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("STATESUPPORT"))
                properties.Rows.Add(new object[] { "StateSupport", "The level of state support provided",
                "EnumString", Structures.XMLA.PropertyAccessTypes.Read, false, Properties.StateSupport.ToString() });
            if (restrictedproperties.Count < 0 || restrictedproperties.Contains("TIMEOUT"))
                properties.Rows.Add(new object[] { "Timeout", "The number of seconds before a connection is ended",
                "Integer", Structures.XMLA.PropertyAccessTypes.ReadWrite, false, MyConnection.Timeout.ToString() });

            // Return our data set
            returnds.Tables.Add(properties);
            return returnds;
        }
    }
}
