/*
 * This source file (MemberGenerator.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Data;
using System.Linq;
using System.IO;
using System.Reflection;
using System.Security.Cryptography;
using System.Collections.Specialized;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.Engine.Persistence;
using Bamboo.Cubed.AddIns.FormulaEvaluators;
using System.Threading;
using Bamboo.Cubed.Engine.Builders;
using Bamboo.Cubed.Engine.Databases;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Engine.Formulas;
using Bamboo.Cubed.Engine.Security;

namespace Bamboo.Cubed.Engine
{
    /// <summary>
    /// This class generators members for a cube from a relational data source
    /// </summary>
    public static class MemberGenerator
    {
        /// <summary>
        /// This will process any dimension updates for the specified cube. This essentially searches through the relational data source, 
        /// and finds any new members for dimensions, or updates existing members
        /// </summary>
        /// <param name="cube">A Cubed Cube to process</param>
        public static void Generate(Cube cube)
        {
            DatabaseManager datamanagerprovider = null;
            CubedDatabaseConnector datamanager = null;

            try
            {
                // First we check that the user requesting the consolidation is an administrator, or that 
                // the requesting user is our local system account
                if (!SecurityManager.AuthenticatedUser.IsAdmin)
                    throw new ConsolidationManagerException("Error running consolidation: Only root users can run consolidations.",
                        null, String.Empty);           

                // Now that we are sure we can run the consolidation, we begin by cycling through all dimensions
                foreach (Dimension thisdimension in cube.Dimensions)
                {
                    // Now connect to our data source for this dimension                    
                    datamanagerprovider = new DatabaseManager(cube.Datasources[thisdimension.Datasource].Provider);
                    datamanager = datamanagerprovider.Provider.ConnectorInstance;
                    if (!datamanager.Open())
                        throw new ConsolidationManagerException("Error running consolidation: Cannot connect to data source for " +
                            "dimension " + thisdimension.Name + ".",
                        null, String.Empty);

                    // Check whether we need to add a total member to this dimension
                    if (thisdimension.AddTotalMember)
                    {
                        if (thisdimension.Members.FirstOrDefault(testmember => testmember.Name.ToUpper() ==
                            "TOTAL " + thisdimension.Name.ToUpper()) == null)
                        {
                            // No total member currently exists so add it now
                            Member totalmember = new Member("Total " + thisdimension.Name,
                                null, thisdimension);
                            totalmember.Aggregation = AggregationTypes.Sum;
                            totalmember.Consolidation = ConsolidationTypes.AggregateOnly;
                            totalmember.Description = "Total " + thisdimension.Name;
                            totalmember.Parent = new Guid();
                            totalmember.PrimaryKey = new List<string>();
                            totalmember.RollUp = true;
                            totalmember.Sign = Signage.Positive;
                            totalmember.Tablename = String.Empty;
                            thisdimension.Members.Add(totalmember);
                        }
                    }

                    // For each dimension, we process any member generators on the dimension to grab their 
                    // latest members from the database
                    foreach (Bamboo.Cubed.Structures.MemberGenerator thisgenerator in thisdimension.MemberGenerators)
                    {
                        // Determine what type of members this dimension should generate
                        if (thisdimension.MemberType == Bamboo.Cubed.Structures.MemberTypes.Filter)
                        {
                            // Generate filter members, based on ranges
                            DataSet rangevalues = new DataSet();
                            GenericKeyCollection<Guid, Member> generatedmembers;
                            if (thisgenerator.Ranges.RangeType == RangeTypes.UniqueValues)
                            {
                                // As this is a unique values range, we need to connect to the database and 
                                // grab all values from the range table to provide the unique values
                                SQLQueryGenerator sqlgen = new SQLQueryGenerator();
                                sqlgen.Aggregate = false;
                                sqlgen.UniqueValues = true;
                                sqlgen.AddPrimaryTable(thisgenerator.Ranges.Table, thisgenerator.Ranges.NameField);
                                sqlgen.AddField(thisgenerator.Ranges.Table, thisgenerator.Ranges.NameField,
                                    thisgenerator.Ranges.NameField, SQLFieldFunctionTypes.GROUP);
                                sqlgen.AddField(thisgenerator.Ranges.Table, thisgenerator.Ranges.DescriptionField,
                                    thisgenerator.Ranges.DescriptionField, SQLFieldFunctionTypes.GROUP);
                                rangevalues = datamanager.SelectRecords(sqlgen);

                                // Now we have our unique values, generate the members from it
                                generatedmembers = thisgenerator.Ranges.GenerateMembers(thisdimension,
                                rangevalues.Tables[0]);
                            }
                            else
                            {
                                // This isn't a unique values range, so generate the members without the
                                // attachment of a data table
                                generatedmembers = thisgenerator.Ranges.GenerateMembers(thisdimension, null);
                            }

                            // Now loop through the members and add them to the dimension
                            foreach (Member thisgeneratedmember in generatedmembers)
                            {
                                // Determine if this member already exists in our collection
                                bool memberexists = false;
                                for (int i = thisdimension.Members.Count - 1; i > -1; i--)
                                {
                                    if (thisdimension.Members[i].Name.ToUpper() == thisgeneratedmember.Name.ToUpper())
                                    {
                                        // This member already exists, so return that and either update or ignore
                                        // the generated member
                                        memberexists = true;
                                        if (thisgenerator.ReplaceExisting)
                                        {
                                            // This generator is configured to replace existing members, so
                                            // replace them
                                            thisdimension.Members[i] = thisgeneratedmember;
                                            thisdimension.Members[i].Aggregation = thisgenerator.Aggregation;
                                            thisdimension.Members[i].Sign = thisgenerator.Sign;
                                        }
                                    }
                                }

                                // Add the newly generated member if it doesn't already exist
                                if (!memberexists)
                                {
                                    thisgeneratedmember.Aggregation = thisgenerator.Aggregation;
                                    thisgeneratedmember.Sign = thisgenerator.Sign;
                                    thisdimension.Members.Add(thisgeneratedmember);
                                }
                            }
                        }
                        else
                        {
                            // Generate table, identifier based members                            
                            SQLQueryGenerator sqlgen = new SQLQueryGenerator();
                            sqlgen.Aggregate = false;
                            sqlgen.UniqueValues = true;
                            
                            // Add our primary table to the generator
                            foreach (MemberTableDetails thismembertable in thisgenerator.MemberTables)
                                if (thismembertable.RelatedTable == null ||
                                    thismembertable.RelatedTable == String.Empty)
                                {
                                    // This is our primary table
                                    sqlgen.AddPrimaryTable(thismembertable.Table, thismembertable.PrimaryKey);
                                    break;
                                }

                            // Now add our other related tables
                            foreach (MemberTableDetails thismembertable in thisgenerator.MemberTables)
                                if (thismembertable.RelatedTable != null &&
                                    thismembertable.RelatedTable != String.Empty)
                                {
                                    // This is a related table
                                    sqlgen.AddJoinedTable(thismembertable.Table, thismembertable.PrimaryKey,
                                        thismembertable.RelatedTable, thismembertable.RelatedKey, SQLJoinTypes.Inner);
                                }

                            // Now add our fields
                            foreach (MemberTableDetails thismembertable in thisgenerator.MemberTables)
                            {
                                sqlgen.AddField(thismembertable.Table, thismembertable.PrimaryKey,
                                    thismembertable.Table + "#" + thismembertable.PrimaryKey, 
                                    SQLFieldFunctionTypes.GROUP);
                                sqlgen.AddField(thismembertable.Table, thismembertable.Name,
                                    thismembertable.Table + "#" + thismembertable.Name, 
                                    SQLFieldFunctionTypes.GROUP);
                                if (thismembertable.Description != null && thismembertable.Description !=
                                    String.Empty)
                                    sqlgen.AddField(thismembertable.Table, thismembertable.Description,
                                        thismembertable.Table + "#" + thismembertable.Description, 
                                        SQLFieldFunctionTypes.GROUP);
                                if (thismembertable.ParentLink != null && thismembertable.ParentLink.ParentTable != String.Empty)                                
                                {
                                    // There is a parent/link for this collection
                                    if (!sqlgen.Tables.Contains(thismembertable.ParentLink.ParentTable))
                                        sqlgen.AddJoinedTable(thismembertable.ParentLink.ParentTable,
                                            thismembertable.ParentLink.MemberIDField, thismembertable.Table,
                                            thismembertable.PrimaryKey, SQLJoinTypes.LeftOuter);
                                    sqlgen.AddField(thismembertable.ParentLink.ParentTable,
                                        thismembertable.ParentLink.ParentIDField, thismembertable.ParentLink.ParentTable + "#" + thismembertable.ParentLink.ParentIDField,
                                        SQLFieldFunctionTypes.GROUP);
                                }

                                if (thismembertable.Filter != null && thismembertable.Filter.FieldName != String.Empty)
                                {
                                    // Determine what type of filter
                                    if (thismembertable.Filter.FilterType == FilterTypes.Range)
                                    {
                                        // This is a range filter
                                        sqlgen.AddRangeFilter(thismembertable.Table, thismembertable.Filter.FieldName,
                                            thismembertable.Filter.LowerValue, thismembertable.Filter.UpperValue);
                                    }
                                    else
                                    {
                                        // This is a list filter
                                        List<object> filtervalues = new List<object>();
                                        foreach (string thisvalue in thismembertable.Filter.AcceptableValues)
                                            if (thisvalue != String.Empty)
                                                filtervalues.Add(thisvalue);
                                        sqlgen.AddListFilter(thismembertable.Table, thismembertable.Filter.FieldName,
                                            filtervalues);
                                    }
                                }
                            }                            

                            // Grab the list of our members for this generator
                            DataSet results = datamanager.SelectRecords(sqlgen);
                            DataTable members = new DataTable();
                            if (results.Tables.Count > 0)
                                members = results.Tables[0];

                            // Now, cycle through the list and add all members into our dimension 
                            foreach (DataRow thismemberrow in members.Rows)
                            {
                                // Now cycle through each member table to grab the value
                                for (int i = 0; i < thisgenerator.MemberTables.Count; i++)
                                {
                                    // Check whether a valid value has been provided for this 
                                    // member table definition
                                    if (thismemberrow[thisgenerator.MemberTables[i].Table + "#" +
                                                thisgenerator.MemberTables[i].PrimaryKey] != null &&
                                                thismemberrow[thisgenerator.MemberTables[i].Table + "#" +
                                                thisgenerator.MemberTables[i].Name] != null)
                                    {
                                        Member existingmember = thisdimension.Members.FirstOrDefault(testmember =>
                                                testmember.Name.ToUpper() == thismemberrow[thisgenerator.MemberTables[i].Table + "#" +
                                                thisgenerator.MemberTables[i].Name].ToString().ToUpper());
                                        if (existingmember != null)
                                        {
                                            // There is already an existing member, so just update it and add
                                            // the member IDs into the primary key collection
                                            if (thisgenerator.ReplaceExisting)
                                            {
                                                // We need to update the member, so grab it's
                                                // basic details
                                                existingmember.Tablename =
                                                thisgenerator.MemberTables[i].Table;
                                                if (!existingmember.PrimaryKey.Exists(testkey =>
                                                    testkey.ToUpper() == thismemberrow[thisgenerator.MemberTables[i].Table + "#" +
                                                    thisgenerator.MemberTables[i].PrimaryKey].ToString().ToUpper()))
                                                {
                                                    existingmember.PrimaryKey.Add(thismemberrow[thisgenerator.MemberTables[i].Table + "#" +
                                                    thisgenerator.MemberTables[i].PrimaryKey].ToString());
                                                }
                                                existingmember.Name =
                                                    thismemberrow[thisgenerator.MemberTables[i].Table + "#" +
                                                    thisgenerator.MemberTables[i].Name].ToString();
                                                if (thisgenerator.MemberTables[i].Description != null
                                                    && thisgenerator.MemberTables[i].Description != String.Empty
                                                    && thismemberrow[thisgenerator.MemberTables[i].Table + "#" +
                                                    thisgenerator.MemberTables[i].Description] != null)
                                                    existingmember.Description =
                                                        thismemberrow[thisgenerator.MemberTables[i].Table + "#" +
                                                        thisgenerator.MemberTables[i].Description].ToString();
                                                existingmember.Aggregation = thisgenerator.Aggregation;
                                                existingmember.Sign = thisgenerator.Sign;

                                                // If the member currently has a parent, remove it or
                                                // set it to total members
                                                if (existingmember.Parent != null &&
                                                    existingmember.Parent != new Guid())
                                                {
                                                    Member parentmember = thisdimension.Members.FirstOrDefault(testmember =>
                                                        testmember.ID == existingmember.Parent);
                                                    if (parentmember != null)
                                                    {
                                                        parentmember.Children.Remove(existingmember.ID);
                                                    }
                                                    existingmember.Parent = new Guid();
                                                }
                                            }
                                        }
                                        else
                                        {
                                            // This member doesn't exist so add it new
                                            Member newmember = new Member(thismemberrow[thisgenerator.MemberTables[i].Table +
                                                "#" + thisgenerator.MemberTables[i].Name].ToString(), null,
                                                thisdimension);
                                            newmember.Aggregation = thisgenerator.Aggregation;
                                            newmember.Sign = thisgenerator.Sign;
                                            newmember.Tablename = thisgenerator.MemberTables[i].Table;
                                            newmember.PrimaryKey.Add(thismemberrow[thisgenerator.MemberTables[i].Table +
                                                "#" + thisgenerator.MemberTables[i].PrimaryKey].ToString());
                                            if (thisgenerator.MemberTables[i].Description != null
                                                        && thisgenerator.MemberTables[i].Description != String.Empty
                                                        && thismemberrow[thisgenerator.MemberTables[i].Table + "#" +
                                                        thisgenerator.MemberTables[i].Description] != null)
                                                newmember.Description = thismemberrow[thisgenerator.MemberTables[i].Table +
                                                    "#" + thisgenerator.MemberTables[i].Description].ToString();
                                            thisdimension.Members.Add(newmember);
                                        }
                                    }
                                }
                            }

                            // Now we have all members generated, cycle through each member in the collection
                            // and grab it's details to check if it has a parent
                            foreach (Member thismember in thisdimension.Members)
                            {
                                // Cycle through each member table to query this member's details and
                                // check if it has a parent
                                for (int i = 0; i < thisgenerator.MemberTables.Count; i++)
                                {
                                    // First check for specific parent links
                                    if (thisgenerator.MemberTables[i].ParentLink != null &&
                                        thisgenerator.MemberTables[i].ParentLink.ParentTable != String.Empty)
                                    {
                                        // This generator has specified member / parent links, so cycle
                                        // through them
                                        foreach (DataRow thismemberrow in members.Rows)
                                        {
                                            if (thismemberrow[thisgenerator.MemberTables[i].ParentLink.ParentTable + "#" +
                                                            thisgenerator.MemberTables[i].ParentLink.ParentIDField].ToString() !=
                                                            String.Empty && thismember.PrimaryKey.Exists(testkey => testkey != String.Empty && 
                                                                testkey.ToUpper() == 
                                                                thismemberrow[thisgenerator.MemberTables[i].Table + "#" + 
                                                            thisgenerator.MemberTables[i].PrimaryKey].ToString().ToUpper()))
                                            {
                                                // This member has a parent specified so try and find it
                                                Member parentmember = thisdimension.Members.FirstOrDefault(member => member.PrimaryKey.Exists(testmember =>
                                                    testmember != String.Empty && testmember.ToUpper() == thismemberrow[thisgenerator.MemberTables[i].ParentLink.ParentTable + "#" +
                                                    thisgenerator.MemberTables[i].ParentLink.ParentIDField].ToString().ToUpper()));
                                                if (parentmember != null)
                                                {
                                                    // We have found this members parent, so add the
                                                    // member to the child collection of the member
                                                    if (!parentmember.Children.Contains(thismember.ID))
                                                        parentmember.Children.Add(thismember.ID);
                                                    thismember.Parent = parentmember.ID;
                                                }
                                            }
                                        }
                                    }

                                    // Then check for relationship based links
                                    foreach (MemberTableDetails thisparenttable in thisgenerator.MemberTables)
                                        if (thisparenttable.RelatedTable.ToUpper() == thisgenerator.MemberTables[i].Table.ToUpper() &&
                                            thisparenttable.Table != thismember.Tablename)
                                        {
                                            // This table, which may potentially contain parents for the
                                            // current member, so let's grab the details of the current
                                            // member from their table and find the row in the members
                                            // collection which matches it
                                            string memberprimarykeycolumn = 
                                                thisparenttable.RelatedTable + "#" +
                                                thisgenerator.MemberTables.FirstOrDefault(testtable =>
                                                    testtable.Table.ToUpper() == thisparenttable.RelatedTable.ToUpper()).PrimaryKey;
                                            DataRow parentrow = members.Rows.Cast<DataRow>().FirstOrDefault(testrow =>
                                                thismember.PrimaryKey.Exists(testparent => testparent.ToUpper() ==
                                                testrow[memberprimarykeycolumn].ToString().ToUpper()));
                                            if (parentrow != null)
                                            {
                                                // We have found a row for this relationship which matches this member, so now
                                                // we need to find it's parent member in our collection
                                                Member parentmember = thisdimension.Members.FirstOrDefault(testmember => testmember.ID != 
                                                    thismember.ID && (testmember.Tablename == thisparenttable.Table) &&
                                                    testmember.PrimaryKey.Exists(testparent => testparent.ToUpper() == parentrow[thisparenttable.Table +
                                                    "#" + thisparenttable.PrimaryKey].ToString().ToUpper()));
                                                if (parentmember != null)
                                                {
                                                    // We have found the members parent so update it
                                                    thismember.Parent = parentmember.ID;
                                                }
                                            }
                                        }
                                }
                            }

                            // Then - if the dimension requires a total member, add it to all the 
                            // members without parents now
                            if (thisdimension.AddTotalMember)
                            {
                                foreach (Member thismember in thisdimension.Members.Where(testmember =>
                                    (testmember.Parent == null || testmember.Parent == new Guid()) &&
                                    testmember.Name.ToUpper() != ("TOTAL " + thisdimension.Name.ToUpper())))
                                {
                                    // This member has no parent so make the total member it's parent
                                    Member totalmember = thisdimension.Members.FirstOrDefault(testmember =>
                                        testmember.Name.ToUpper() == "TOTAL " + thisdimension.Name.ToUpper());
                                    thismember.Parent = totalmember.ID;
                                }
                            }

                            // Finally, check for any duplicate primary key members at the same level of
                            // the cube. If any exist, remove the oldest one assuming that the latest has
                            // had a name change
                            Dictionary<Guid, Guid> markedfordeletionmembers = new Dictionary<Guid, Guid>();
                            foreach (Level thislevel in thisdimension.Levels)
                                foreach (Guid thismemberid in thislevel.Members)
                                {
                                    // Grab the member details
                                    Member thismember = thisdimension.Members[thismemberid];

                                    // Now check whether any other members of this level have the same 
                                    // primary key
                                    foreach (Member thissamemember in thisdimension.Members.Where(testmember => testmember.ID != 
                                        thismember.ID && 
                                        testmember.Level == thismember.Level && testmember.PrimaryKey.Exists(testkey =>
                                            thismember.PrimaryKey.Exists(testprimarykey => testprimarykey.ToUpper() == testkey.ToUpper()))))
                                    {
                                        // This member has the same primary key at the same level as our
                                        // test member, so determine which is newer, and mark the other for
                                        // deletion
                                        if (thissamemember.Discovered > thismember.Discovered && 
                                            !markedfordeletionmembers.ContainsKey(thismember.ID))
                                            markedfordeletionmembers.Add(thismember.ID, thissamemember.ID);
                                        else if (!markedfordeletionmembers.ContainsKey(thissamemember.ID))
                                            markedfordeletionmembers.Add(thissamemember.ID, thismember.ID);
                                    }
                                }

                            // Then remove any members marked for deletion
                            foreach (Guid thisdeletionmember in markedfordeletionmembers.Keys)
                            {
                                // First, if this member has any children, re-parent them under the
                                // replacement member
                                foreach (Guid thischildmember in thisdimension.Members[thisdeletionmember].Children)
                                {
                                    thisdimension.Members[thischildmember].Parent = markedfordeletionmembers[thisdeletionmember];
                                }

                                // Then delete the member
                                thisdimension.Members.Remove(thisdeletionmember);
                            }
                        }
                    }

                    // Close our datasource connection
                    datamanager.Rollback();
                }

                // Since we have generated new members dynamically, execute the repair tree function on 
                // each dimension to ensure parents and levels are correct, as well as the order function if
                // required to order correctly
                foreach (Dimension thisdimension in cube.Dimensions)
                {
                    thisdimension.RepairTree();
                    if (thisdimension.AutoOrder)
                        thisdimension.OrderMembers();
                }

                // Now we have finished all the dimension updates, we can save the cube
                CubeManager.SaveCube(cube);

                // Now that all of our members and relationships have been generated, pre-cache our "descendants"
                // collection to ensure fast query retrieval
                GenerateDescendantsCache(cube);
            }
            catch (Exception e1)
            {           
                // Attempt to close the data manager if it is open
                if (datamanager != null && datamanager.State == ConnectionState.Open)
                    datamanager.Rollback();

                // Throw our error
                throw new ConsolidationManagerException("Error generating members list: " + e1.ToString(), e1, String.Empty);
            }
        }

        /// <summary>
        /// This generates a cache of all descendants of each member in the given cube to ensure fast query retrieval
        /// </summary>
        /// <param name="cube">The cube to generate the cache for</param>
        public static void GenerateDescendantsCache(Cube cube)
        {
            DatabaseManager datamanagerprovider = null;
            CubedDatabaseConnector datamanager = null;

            try
            {
                // Grab a data manager based on the core system data source
                datamanagerprovider = new DatabaseManager(SettingsManager.Settings.StorageProvider);
                datamanager = datamanagerprovider.Provider.ConnectorInstance;

                // Open our connection
                datamanager.Open();

                // Now cycle through each member of the cube
                foreach (Dimension thisdimension in cube.Dimensions)
                    foreach (Member thismember in thisdimension.Members)
                    {
                        // Delete the current "descendants" of this member
                        string[] deletefield = new string[1] { "id" };
                        object[] deletevalues = new object[1] { thismember.ID };
                        datamanager.DeleteRecord(SettingsManager.Settings.StructuresTableName + "_Descendants",
                            deletefield, deletevalues);

                        // Now grab a list of all of the descendant's "IDs" for this member
                        foreach (Member thisdescendant in thisdimension.GetMemberDescendants(thismember))
                        {
                            string[] insertfields = new string[2] { "id", "descendantid" };
                            object[] insertvalues = new object[2] { thismember.ID, thisdescendant.ID };
                            DataCreationTypes[] inserttypes = new DataCreationTypes[2] { DataCreationTypes.UniqueIdentifier,
                                DataCreationTypes.UniqueIdentifier };
                            string[] insertkeys = insertfields;
                            datamanager.InsertRecord(SettingsManager.Settings.StructuresTableName + "_Descendants",
                                insertfields, insertvalues, inserttypes, false, insertkeys);
                        }

                    }

                // Now commit our changes
                datamanager.Commit();             
            }
            catch (Exception e1)
            {
                // Attempt to close the data manager if it is open
                if (datamanager != null && datamanager.State == ConnectionState.Open)
                    datamanager.Rollback();

                // Throw our error
                throw new ConsolidationManagerException("Error generating query descendant cache: " + e1.ToString(), e1, String.Empty);
            }
        }
    }
}
