/*
 * This source file (CellData.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;
using System.Security.Cryptography;
using System.IO;
using System.Reflection;
using Bamboo.Cubed.Utilities;
using System.Xml.Schema;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures.XMLA
{
    /// <summary>
    /// Contains a collection of Cell Data
    /// </summary>
    [Serializable]
    [DataContract]
    public class CellData
    {
        GenericKeyCollection<string, CellIndex> _cellindices;
        /// <summary>
        /// This dictionary stores the string pattern of the index of each Axis represented by each
        /// cell in this query, alongside the index in the cell collection of where to find that cell
        /// </summary>
        [DataMember]
        public GenericKeyCollection<string, CellIndex> CellIndices
        {
            get
            {
                if (_cellindices == null) _cellindices = new GenericKeyCollection<string, CellIndex>();
                return _cellindices;
            }
            set { _cellindices = value; } 
        }

        /// <summary>
        /// Returns the enumerator
        /// </summary>
        /// <returns>The enumerator</returns>
        public IEnumerator GetEnumerator()
        {
            return CellCollection.GetEnumerator();
        }

        /// <summary>
        /// Adds a new cell
        /// </summary>
        /// <param name="cell">The cell to add</param>
        /// <param name="indices">The index of each member of each axis which this cell represents</param>
        public void Add(Cell cell, params int[] indices)
        {               
            // Add our cell to the collection
            CellCollection.Add(cell);

            // Then we determine the index of the new cell
            int newindex = CellCollection.Count - 1;
            for (int index = CellCollection.Count - 1; index > -1; index--)
                if (CellCollection[index] == cell)
                {
                    newindex = index;
                    break;
                }

            // Finally add the cells index and index string to the Cell Indices collection
            CellIndices.Add(new CellIndex(newindex, indices));
        }

        /// <summary>
        /// Returns the specified cell
        /// </summary>
        /// <param name="index">The cell to return</param>
        /// <returns>The specified cell</returns>
        public Cell this[int index]
        {
            get { return (Cell)CellCollection[index]; }
            set { CellCollection[index] = value; }
        }

        /// <summary>
        /// Returns the Cell stored at the intersection of the passed in Axes indexes
        /// </summary>
        /// <param name="indices">A collection of indices which represent every axes of the data set</param>
        /// <returns>The cell specified</returns>
        public Cell this[params int[] indices]
        {
            get 
            {
                // First we create a string which represents the indices collection
                string indexstring = String.Empty;
                foreach (int thisindex in indices)
                    indexstring += thisindex.ToString() + "-";
                indexstring = indexstring.TrimEnd('-');

                // Next, we check whether ALL axes have been represented, and if not - we assume the remaining
                // axis (to the right) have an index of 0 (the first member of the axis)                
                string[] exampleaxisindex = CellIndices.GetKey(0).Split('-');
                int axiscount = exampleaxisindex.Length;
                int specifiedaxiscount = indices.Length;
                while (specifiedaxiscount < axiscount)
                {
                    specifiedaxiscount++;
                    indexstring = indexstring + "-0";
                }

                // Then we try and find the cell index by using this string
                int cellindex = CellIndices[indexstring].Index;

                // Then we try and retrieve the cell
                if (cellindex > -1 && cellindex < CellCollection.Count)
                    return CellCollection[cellindex];

                // If we can't find it, we return null
                return null;
            }
            set 
            {
                // First we create a string which represents the indices collection
                string indexstring = String.Empty;
                foreach (int thisindex in indices)
                    indexstring += thisindex.ToString() + "-";
                indexstring = indexstring.TrimEnd('-');

                // Next, we check whether ALL axes have been represented, and if not - we assume the remaining
                // axis (to the right) have an index of 0 (the first member of the axis)               
                string[] exampleaxisindex = CellIndices.GetKey(0).Split('-');
                int axiscount = exampleaxisindex.Length;
                int specifiedaxiscount = indices.Length;
                while (specifiedaxiscount < axiscount)
                {
                    specifiedaxiscount++;
                    indexstring = indexstring + "-0";
                }

                // Then we try and find the cell index by using this string
                int cellindex = CellIndices[indexstring].Index;

                // If the cell index is valid, update the cell in the collection
                if (cellindex > -1 && cellindex < CellCollection.Count)
                    CellCollection[cellindex] = value;
            } 
        }

        /// <summary>
        /// Counts the number of cells in the collection
        /// </summary>
        public int Count
        {
            get { return CellCollection.Count; }
        }

        /// <summary>
        /// Clears all cells and indexes
        /// </summary>
        public void Clear()
        {
            CellCollection.Clear();
            CellIndices.Clear();
        }

        /// <summary>
        /// Removes the specified cell
        /// </summary>
        /// <param name="index">The index of the cell to remove</param>
        public void Remove(int index)
        {
            if (index < CellCollection.Count)
            {
                // First we remove the index of the cell from the Cell Indices collection                
                CellIndices.Remove(CellIndices.GetKey(index));

                // Then we remove the Cell itself
                CellCollection.RemoveAt(index);
            }
        }

        /// <summary>
        /// Removes the specified cell
        /// </summary>
        /// <param name="cell">The cell to remove</param>
        /// <param name="indices">The collection of indices representing the member on each axis which
        /// this cell represents</param>
        public void Remove(Cell cell, params int[] indices)
        {
            // First we create a string representing the indices of this cell
            string indexstring = String.Empty;
            foreach (int thisindex in indices)
                indexstring += thisindex.ToString() + "-";
            indexstring = indexstring.TrimEnd('-');

            // Next, we check whether ALL axes have been represented, and if not - we assume the remaining
            // axis (to the right) have an index of 0 (the first member of the axis)            
            string[] exampleaxisindex = CellIndices.GetKey(0).Split('-');
            int axiscount = exampleaxisindex.Length;
            int specifiedaxiscount = indices.Length;
            while (specifiedaxiscount < axiscount)
            {
                specifiedaxiscount++;
                indexstring = indexstring + "-0";
            }

            // Now we remove this cells index from the Cell Indices collection
            CellIndices.Remove(indexstring);

            // Then we remove the cell itself            
            CellCollection.Remove(cell);
        }

        private List<Cell> _cellcollection;
        /// <summary>
        /// The list of cells
        /// </summary>
        [DataMember]
        public List<Cell> CellCollection
        {
            get
            {
                if (_cellcollection == null) _cellcollection = new List<Cell>();
                return _cellcollection;
            }
            set { _cellcollection = value; }
        }
    }
}
