/*
 * This source file (MDDataSet.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;
using System.Linq;
using System.Security.Cryptography;
using System.IO;
using System.Reflection;
using Bamboo.Cubed.Utilities;
using System.Xml.Schema;
using System.Runtime.Serialization;
using System.Data;

namespace Bamboo.Cubed.Structures.XMLA
{
    /// <summary>
    /// Represents a Multi-Dimensional DataSet
    /// </summary>
    [Serializable]
    [DataContract]
    public class MDDataSet
    {
        private OlapInfo _olapinfo;
        /// <summary>
        /// The OlapInfo for the data set
        /// </summary>
        [DataMember]
        public OlapInfo OlapInfo
        {
            get
            {
                if (_olapinfo == null) _olapinfo = new OlapInfo();
                return _olapinfo;
            }
            set { _olapinfo = value; }
        }

        private Axes _axes;
        /// <summary>
        /// The axes of the data set
        /// </summary>
        [DataMember]
        public Axes Axes
        {
            get
            {
                if (_axes == null) _axes = new Axes();
                return _axes;
            }
            set { _axes = value; }
        }

        private Axis _filteraxis;
        /// <summary>
        /// Returns the axis used as a filter for this data set
        /// </summary>
        [DataMember]
        public Axis FilterAxis
        {
            get
            {
                if (_filteraxis == null) _filteraxis = new Axis();
                return _filteraxis;
            }
            set { _filteraxis = value; }
        }

        private CellData _cells;
        /// <summary>
        /// The cell data of the data set
        /// </summary>
        [DataMember]
        public CellData Cells
        {
            get
            {
                if (_cells == null) _cells = new CellData();
                return _cells;
            }
            set { _cells = value; }
        }

        /// <summary>
        /// This converts the current multi-dimensional dataset into a flat data set representation
        /// </summary>
        /// <returns>The converted flat data set</returns>
        public DataSet ToDataSet()
        {
            // Create a new empty, flat data set
            DataSet flatdataset = new DataSet();

            // If there are less than two axes, return the empty data set 
            if (this.Axes.Count < 2)
            {
                flatdataset.Tables.Add(new DataTable());
                return flatdataset;
            }

            // Now populate this dataset with the results of the passed in MDDataSet          
            flatdataset.Tables.Add(this.OlapInfo.CubeInfo.MDDataSetCube.CubeName);
            flatdataset.Tables[0].Columns.Add(this.OlapInfo.CubeInfo.MDDataSetCube.CubeName, typeof(string));

            // Create all the data columns as type of "Double", and place the first axis on the columns
            for (int i = 0; i < this.Axes[0].Members.Count; i++)
            {
                if (!flatdataset.Tables[0].Columns.Contains(this.Axes[0].Members[i].Caption))
                    flatdataset.Tables[0].Columns.Add(this.Axes[0].Members[i].Caption, typeof(double));
                else
                {
                    // As a column already exists with this name, apply a numbered suffix
                    int suffixno = 1;
                    string columnname = this.Axes[0].Members[i].Caption + suffixno.ToString();
                    while (flatdataset.Tables[0].Columns.Contains(columnname))
                    {
                        suffixno++;
                        columnname = this.Axes[0].Members[i].Caption + suffixno.ToString();
                    }

                    // Now add the column
                    flatdataset.Tables[0].Columns.Add(columnname, typeof(double));
                }
            }

            if (this.Axes.Count == 2)
            {
                // There is only 2 axis, so place the 2nd axis on the rows
                for (int y = 0; y < this.Axes[1].Members.Count; y++)
                {
                    DataRow rowvalues = flatdataset.Tables[0].NewRow();
                    rowvalues[0] = this.Axes[1].Members[y].Caption;
                    for (int x = 0; x < this.Axes[0].Members.Count; x++)
                    {
                        rowvalues[x + 1] = (double)this.Cells[x, y].Value;
                    }
                    flatdataset.Tables[0].Rows.Add(rowvalues);
                }
            }
            else if (this.Axes.Count > 2)
            {
                // There are more than 2 axes, so place the 2nd and 3rd on the rows (and ignore the rest)
                for (int y = 0; y < this.Axes[1].Members.Count; y++)
                {
                    // Loop through the 3rd (Z) axis
                    for (int z = 0; z < this.Axes[2].Members.Count; z++)
                    {
                        DataRow rowvalues = flatdataset.Tables[0].NewRow();
                        rowvalues[0] = this.Axes[1].Members[y].Caption;
                        rowvalues[1] = this.Axes[2].Members[z].Caption;
                        for (int x = 0; x < this.Axes[0].Members.Count; x++)
                        {
                            rowvalues[x + 2] = (double)this.Cells[x, y, z].Value;
                        }
                        flatdataset.Tables[0].Rows.Add(rowvalues);
                    }
                }
            }

            // Finally return the converted data set
            return flatdataset;
        }

        /// <summary>
        /// Converts this multi-dimensional data set into a 2 dimensional data set, representing
        /// the two axis specified
        /// </summary>
        /// <param name="xaxis">The axis to display on the X-Axis of the data set (rows)</param>
        /// <param name="yaxis">The axis to display on the Y-Axis of the data set (columns)</param>
        /// <returns>A 2 dimensional data set with the specified X and Y axis represented</returns>
        public DataSet ToDataSet(string xaxis, string yaxis)
        {
            // Create a new empty, flat data set
            DataSet flatdataset = new DataSet();

            // Now populate this dataset with the results of the passed in MDDataSet          
            flatdataset.Tables.Add(this.OlapInfo.CubeInfo.MDDataSetCube.CubeName);
            flatdataset.Tables[0].Columns.Add(this.OlapInfo.CubeInfo.MDDataSetCube.CubeName, typeof(string));

            // Now determine the X and Y axis indexes
            int x = -1;
            int y = -1;
            
            if (this.Axes.AxisCollection.Exists(textaxis => textaxis.Name.ToUpper() == xaxis.ToUpper()))
            {
                // The X-Axis exists so use it
                x = this.Axes.AxisCollection.FindIndex(textaxis => textaxis.Name.ToUpper() == xaxis.ToUpper());
            }

            if (this.Axes.AxisCollection.Exists(textaxis => textaxis.Name.ToUpper() == yaxis.ToUpper()) &&
                xaxis.ToUpper() != yaxis.ToUpper())
            {
                // The Y-Axis exists so use it
                y = this.Axes.AxisCollection.FindIndex(textaxis => textaxis.Name.ToUpper() == yaxis.ToUpper());
            }

            if (x == -1 && y == 0)
                x = 1;
            else if (x == -1)
                x = 0;

            if (y == -1 && x == 0)
                y = 1;
            else if (y == -1)
                y = 0;

            // Now check to ensure we have at least 2 axis to return
            if (this.Axes.Count > x && this.Axes.Count > y)
            {
                // We have all the axes we need so now generate our flat data set

                // Create all the data columns as type of "Double", and place the first axis on the columns
                for (int i = 0; i < this.Axes[y].Members.Count; i++)
                {
                    if (!flatdataset.Tables[0].Columns.Contains(this.Axes[y].Members[i].Caption))
                        flatdataset.Tables[0].Columns.Add(this.Axes[y].Members[i].Caption, typeof(double));
                    else
                    {
                        // There is another member with the same name already added to this list so 
                        // increment the name
                        int nameinc = 1;
                        string newname = this.Axes[y].Members[i].Caption + " " + nameinc.ToString();
                        while (flatdataset.Tables[0].Columns.Contains(newname))
                        {
                            nameinc++;
                            newname = this.Axes[y].Members[i].Caption + " " + nameinc.ToString();
                        }
                        flatdataset.Tables[0].Columns.Add(newname, typeof(double));
                    }
                }

                // Now return the axis requested
                for (int xvalue = 0; xvalue < this.Axes[x].Members.Count; xvalue++)
                {
                    // Create our new row and set the X caption as the first column
                    DataRow rowvalues = flatdataset.Tables[0].NewRow();
                    rowvalues[0] = this.Axes[x].Members[xvalue].Caption;

                    // Now add all the values from the Y axis for the columns
                    for (int yvalue = 0; yvalue < this.Axes[y].Members.Count; yvalue++)
                    {                        
                        // Grab our data array
                        List<int> arrayvalue = new List<int>();
                        for (int i = 0; i < this.Axes.Count; i++)
                            arrayvalue.Add(0);
                        arrayvalue[x] = xvalue;
                        arrayvalue[y] = yvalue;
                        rowvalues[yvalue + 1] = (double)this.Cells[arrayvalue.ToArray()].Value;                        
                    }

                    // Add our new row
                    flatdataset.Tables[0].Rows.Add(rowvalues);
                }
            }

            // Return the data set
            return flatdataset;
        }

        /// <summary>
        /// Converts this multi-dimensional data set into a 2 dimensional data set, representing
        /// the two axis specified
        /// </summary>
        /// <param name="xaxis">The axis to display on the X-Axis of the data set (rows)</param>
        /// <param name="yaxis">The axis to display on the Y-Axis of the data set (columns)</param>
        /// <param name="xaxislevel">The level of the x-axis to load (only members from this level
        /// will be returned)</param>
        /// <returns>A 2 dimensional data set with the specified X and Y axis represented</returns>
        public DataSet ToDataSet(string xaxis, string yaxis, int xaxislevel)
        {
            // Create a new empty, flat data set
            DataSet flatdataset = new DataSet();

            // Now populate this dataset with the results of the passed in MDDataSet          
            flatdataset.Tables.Add(this.OlapInfo.CubeInfo.MDDataSetCube.CubeName);
            flatdataset.Tables[0].Columns.Add(this.OlapInfo.CubeInfo.MDDataSetCube.CubeName, typeof(string));

            // Now determine the X and Y axis indexes
            int x = -1;
            int y = -1;

            if (this.Axes.AxisCollection.Exists(textaxis => textaxis.Name.ToUpper() == xaxis.ToUpper()))
            {
                // The X-Axis exists so use it
                x = this.Axes.AxisCollection.FindIndex(textaxis => textaxis.Name.ToUpper() == xaxis.ToUpper());
            }

            if (this.Axes.AxisCollection.Exists(textaxis => textaxis.Name.ToUpper() == yaxis.ToUpper()) &&
                xaxis.ToUpper() != yaxis.ToUpper())
            {
                // The Y-Axis exists so use it
                y = this.Axes.AxisCollection.FindIndex(textaxis => textaxis.Name.ToUpper() == yaxis.ToUpper());
            }

            if (x == -1 && y == 0)
                x = 1;
            else if (x == -1)
                x = 0;

            if (y == -1 && x == 0)
                y = 1;
            else if (y == -1)
                y = 0;

            // Now check to ensure we have at least 2 axis to return
            if (this.Axes.Count > x && this.Axes.Count > y)
            {
                // We have all the axes we need so now generate our flat data set

                // Create all the data columns as type of "Double", and place the first axis on the columns
                for (int i = 0; i < this.Axes[y].Members.Count; i++)
                {
                    if (!flatdataset.Tables[0].Columns.Contains(this.Axes[y].Members[i].Caption))
                        flatdataset.Tables[0].Columns.Add(this.Axes[y].Members[i].Caption, typeof(double));
                    else
                    {
                        // There is another member with the same name already added to this list so 
                        // increment the name
                        int nameinc = 1;
                        string newname = this.Axes[y].Members[i].Caption + " " + nameinc.ToString();
                        while (flatdataset.Tables[0].Columns.Contains(newname))
                        {
                            nameinc++;
                            newname = this.Axes[y].Members[i].Caption + " " + nameinc.ToString();
                        }
                        flatdataset.Tables[0].Columns.Add(newname, typeof(double));
                    }
                }

                // Now return the axis requested
                for (int xvalue = 0; xvalue < this.Axes[x].Members.Count; xvalue++)
                {
                    // Determine if this X-axis member is from the specified level, and only return
                    // it if it is
                    if (this.Axes[x].Members[xvalue].LNum == xaxislevel)
                    {
                        // Create our new row and set the X caption as the first column
                        DataRow rowvalues = flatdataset.Tables[0].NewRow();
                        rowvalues[0] = this.Axes[x].Members[xvalue].Caption;

                        // Now add all the values from the Y axis for the columns
                        for (int yvalue = 0; yvalue < this.Axes[y].Members.Count; yvalue++)
                        {
                            // Grab our data array
                            List<int> arrayvalue = new List<int>();
                            for (int i = 0; i < this.Axes.Count; i++)
                                arrayvalue.Add(0);
                            arrayvalue[x] = xvalue;
                            arrayvalue[y] = yvalue;
                            rowvalues[yvalue + 1] = (double)this.Cells[arrayvalue.ToArray()].Value;
                        }

                        // Add our new row
                        flatdataset.Tables[0].Rows.Add(rowvalues);
                    }
                }
            }

            // Return the data set
            return flatdataset;
        }

        /// <summary>
        /// This converts the given flat dataset into a multi-dimensional dataset
        /// </summary>
        /// <param name="source">The source dataset</param>
        public void FromDataSet(DataSet source)
        {
            // Clear the current data set details
            this.Axes = new Axes();
            this.Cells = new CellData();
            this.FilterAxis = new Axis();
            this.OlapInfo = new OlapInfo();

            // Create the different elements of the multi-dimensional data set to return
            Axis _firstaxis = new Axis();
            Axis _secondaxis = new Axis();
            CellData _celldata = new CellData();
            _celldata = new CellData();

            // Loop through the columns of the DataSet to find the correct elements to create the first axis
            foreach (DataColumn c in source.Tables[0].Columns)
            {
                if (c.Ordinal > 0)
                {
                    MemberType _colmember = new MemberType();
                    _colmember.Caption = c.ColumnName;
                    TupleType _tuple = new TupleType();
                    _tuple.Members.Add(_colmember);
                    _firstaxis.Members.Add(_colmember);
                    _firstaxis.Tuples.Add(_tuple);
                }
            }

            // Loop through the rows of the axis to create the second axis and populate the axes with data
            for (int y = 0; y < source.Tables[0].Rows.Count; y++)
            {
                int x = 0;
                foreach (DataColumn c in source.Tables[0].Columns)
                {
                    if (c.Ordinal == 0)
                    {
                        MemberType _member = new MemberType();
                        _member.Caption = source.Tables[0].Rows[y][c.Ordinal].ToString();
                        _secondaxis.Members.Add(_member);
                        TupleType _tuple2 = new TupleType();
                        _tuple2.Members.Add(_member);
                        _secondaxis.Tuples.Add(_tuple2);
                    }
                    else
                    {
                        Cell _newcell = new Cell();
                        _newcell.Value = double.Parse(source.Tables[0].Rows[y][c.Ordinal].ToString());
                        _celldata.Add(_newcell, x, y);
                        x++;
                    }
                }

            }

            // Give generic names to the return axis
            _firstaxis.Name = "Columns";
            this.Axes.Add(_firstaxis);
            _secondaxis.Name = "Rows";
            this.Axes.Add(_secondaxis);
            this.Cells = _celldata;            
        }
    }
}
