/*
 * This source file (PropertyList.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;
using System.Security.Cryptography;
using System.IO;
using System.Reflection;
using Bamboo.Cubed.Utilities;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures.XMLA
{
    /// <summary>
    /// Specifies the properties for connection to the XMLA data source on an Execute or Discover method
    /// </summary>
    [Serializable]
    [DataContract]
    public class PropertyList
    {
        /// <summary>
        /// Initialises a new default instance of the PropertyList
        /// </summary>
        public PropertyList()
        {
            axisformat = AxisFormatTypes.TupleFormat;
            beginrange = -1;
            catalog = String.Empty;
            content = ContentTypes.SchemaData;
            commandtype = "MDX";
            cube = String.Empty;
            datasourceinfo = String.Empty;
            endrange = -1;
            format = FormatTypes.Native;
            localeidenfitier = System.Globalization.CultureInfo.CurrentUICulture.LCID;
            mdxsupport = MDXSupportTypes.Core;
            password = String.Empty;
            providername = "CubedConnect";
            providerversion = Information.Version;
            statesupport = StateSupportTypes.None;
            timeout = 0;
            username = Environment.UserDomainName + "\\" + Environment.UserName;
            admin = false;
            server = System.Environment.MachineName;
            _includezeros = true;
        }

        /// <summary>
        /// Initialises a new instance of Property List based on the passed in connection parameter object
        /// </summary>
        /// <param name="_connectionparameters">The Connection Parameters object to use to build this property list</param>
        public PropertyList(ConnectionParameters _connectionparameters)
        {
            axisformat = AxisFormatTypes.TupleFormat;
            beginrange = -1;
            catalog = String.Empty;
            content = ContentTypes.SchemaData;
            commandtype = "MDX";
            cube = _connectionparameters.Cube;
            datasourceinfo = String.Empty;
            endrange = -1;
            format = FormatTypes.Native;
            localeidenfitier = System.Globalization.CultureInfo.CurrentUICulture.LCID;
            mdxsupport = MDXSupportTypes.Core;
            password = String.Empty;
            providername = "CubedConnect";
            providerversion = Information.Version;
            statesupport = StateSupportTypes.None;
            timeout = _connectionparameters.Timeout;
            username = _connectionparameters.UserName;
            admin = _connectionparameters.Admin;
            server = _connectionparameters.EndPoint.Server;
            _includezeros = true;
        }

        /// <summary>
        /// Initialises a new instance of Property List based on the passed in connection parameters
        /// </summary>
        /// <param name="_server">The Cubed Server URL</param>
        /// <param name="_commandtype">The type of command to execute (if this is an execute method)</param>
        /// <param name="_cube">The name of the cube to connect to</param>
        /// <param name="_username">The full username of the current user</param>
        /// <param name="_admin">Whether the user wants administration rights</param>
        /// <param name="_timeout">The connection timeout</param>
        public PropertyList(string _server, string _commandtype, string _cube, string _username,
            bool _admin, int _timeout)
        {
            axisformat = AxisFormatTypes.TupleFormat;
            beginrange = -1;
            catalog = String.Empty;
            content = ContentTypes.SchemaData;
            commandtype = _commandtype;
            cube = _cube;
            datasourceinfo = String.Empty;
            endrange = -1;
            format = FormatTypes.Native;
            localeidenfitier = System.Globalization.CultureInfo.CurrentUICulture.LCID;
            mdxsupport = MDXSupportTypes.Core;
            password = String.Empty;
            providername = "CubedConnect";
            providerversion = Information.Version;
            statesupport = StateSupportTypes.None;
            timeout = _timeout;
            username = _username;
            admin = _admin;
            server = _server;
            _includezeros = true;
        }

        private AxisFormatTypes axisformat;
        /// <summary>
        /// The format that the axis follows
        /// </summary>
        [DataMember]
        public AxisFormatTypes AxisFormat
        {
            get { return axisformat; }
            set { axisformat = value; }
        }

        private int beginrange;
        /// <summary>
        /// The beginning of the cell range to return
        /// </summary>
        [DataMember]
        public int BeginRange
        {
            get { return beginrange; }
            set { beginrange = value; }
        }

        private string catalog;
        /// <summary>
        /// The Catalog to load
        /// </summary>
        [DataMember]
        public string Catalog
        {
            get { return catalog; }
            set { catalog = value; }
        }

        private ContentTypes content;
        /// <summary>
        /// The type of content to return
        /// </summary>
        [DataMember]
        public ContentTypes Content
        {
            get { return content; }
            set { content = value; }
        }

        private string commandtype;
        /// <summary>
        /// The type of command which is being executed by the Execute method. The default is "MDX"
        /// </summary>
        [DataMember]
        public string CommandType
        {
            get { return commandtype; }
            set { commandtype = value; }
        }

        private string cube;
        /// <summary>
        /// The cube to return
        /// </summary>
        [DataMember]
        public string Cube
        {
            get { return cube; }
            set { cube = value; }
        }

        private string datasourceinfo;
        /// <summary>
        /// Provider specific Datasource connection info
        /// </summary>
        [DataMember]
        public string DataSourceInfo
        {
            get { return datasourceinfo; }
            set { datasourceinfo = value; }
        }

        private int endrange;
        /// <summary>
        /// The end of the cell range to return
        /// </summary>
        [DataMember]
        public int EndRange
        {
            get { return endrange; }
            set { endrange = value; }
        }

        private FormatTypes format;
        /// <summary>
        /// The format of the returned result set
        /// </summary>
        [DataMember]
        public FormatTypes Format
        {
            get { return format; }
            set { format = value; }
        }

        private int localeidenfitier;
        /// <summary>
        /// The language identifier to use for the connection
        /// </summary>
        [DataMember]
        public int LocaleIdentifier
        {
            get { return localeidenfitier; }
            set { localeidenfitier = value; }
        }

        private MDXSupportTypes mdxsupport;
        /// <summary>
        /// Indicates the level of MDX Support required
        /// </summary>
        [DataMember]
        public MDXSupportTypes MDXSupport
        {
            get { return mdxsupport; }
            set { mdxsupport = value; }
        }

        private string password;
        /// <summary>
        /// This value is deprecated and ignored as no direct passing of password is supported
        /// </summary>
        [DataMember]
        public string Password
        {
            get { return password; }
            set { password = value; }
        }

        private string providername;
        /// <summary>
        /// Returns the provider name (that is, Cubed)
        /// </summary>
        [DataMember]
        public string ProviderName
        {
            get { return providername; }
            set { providername = value; }
        }

        private string providerversion;
        /// <summary>
        /// Returns the current version of this provider
        /// </summary>
        [DataMember]
        public string ProviderVersion
        {
            get { return providerversion; }
            set { providerversion = value; }
        }

        private StateSupportTypes statesupport;
        /// <summary>
        /// Returns the level of state support provided by Cubed
        /// </summary>
        [DataMember]
        public StateSupportTypes StateSupport
        {
            get { return statesupport; }
            set { statesupport = value; }
        }

        private int timeout;
        /// <summary>
        /// Indicates the timeout period for inactive connections
        /// </summary>
        [DataMember]
        public int Timeout
        {
            get { return timeout; }
            set { timeout = value; }
        }

        private string username;
        /// <summary>
        /// The name of the currently authenticated user (this is determined automatically by the system)
        /// </summary>
        [DataMember]
        public string UserName
        {
            get { return username; }
            set { username = value; }
        }

        private bool admin;
        /// <summary>
        /// Sets whether the currently logged on user has requested administration rights
        /// </summary>
        [DataMember]
        public bool Admin
        {
            get { return admin; }
            set { admin = value; }
        }

        private string server;
        /// <summary>
        /// The Cubed Server which is connected to
        /// </summary>
        [DataMember]
        public string Server
        {
            get { return server; }
            set { server = value; }
        }

        /// <summary>
        /// This returns a valid connection string based on the passed through XMLA parameters
        /// </summary>
        public string ConnectionString
        {
            get
            {
                return "Server=" + server + ";Username=" + username + ";Cube=" + cube
                    + ";Timeout=" + Convert.ToInt32(timeout).ToString() + ";Admin=" + admin.ToString();
            }
        }

        private bool _includezeros;
        /// <summary>
        /// Whether to exclude explicitly zero values in the returned search
        /// </summary>
        [DataMember]
        public bool IncludeZeros
        {
            get { return _includezeros; }
            set { _includezeros = value; }
        }
    }    
}
