/*
 * This source file (ConnectionParameters.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// An object containing parameters used for connecting to a Cubed Server
    /// </summary>
    [Serializable]
    [DataContract]
    public class ConnectionParameters
    {
        /// <summary>
        /// Creates a blank new connection parameters object
        /// </summary>
        public ConnectionParameters()
        {
            _endpoint = new ServiceEndPoint();
            _timeout = 300;
            _username = String.Empty;
            _domain = String.Empty;
            _cube = "Default";
            _admin = false;
            _password = String.Empty;
            _connectionstring = String.Empty;
        }

        /// <summary>
        /// Initialises a new instance of connectionparameters based on the connection string given to parse
        /// </summary>
        /// <param name="connectionstring">The connection string to use to parse the connection parameters</param>
        public ConnectionParameters(string connectionstring)
        {
            _endpoint = new ServiceEndPoint();
            _domain = String.Empty;
            _cube = String.Empty;
            _admin = false;
            _timeout = 300;
            _password = String.Empty;
            _connectionstring = connectionstring;           
            ParseString(connectionstring);
        }

        /// <summary>
        /// This parses the given connections string and updates the internal components
        /// </summary>
        /// <param name="connectionstring">The connection string to parse</param>
        private void ParseString(string connectionstring)
        {
            string[] parameters = connectionstring.Split(';');
            foreach (string s in parameters)
            {
                string[] parameterdetails = s.Split('=');
                if (parameterdetails.Length > 1)
                {
                    string parametername = parameterdetails[0].ToUpper().Trim();
                    string parametervalue = parameterdetails[1].Trim();
                    if (parametername.Equals("SERVER") || parametername.Equals("LOCATION") || parametername.Equals("DATA SOURCE"))
                    {
                        _endpoint.Server = parametervalue;
                    }
                    else if (parametername.Equals("PORT"))
                    {
                        int port = 10000;
                        int.TryParse(parametervalue, out port);
                        _endpoint.Port = port;
                    }
                    else if (parametername.Equals("PROTOCOL") || parametername.Equals("METHOD"))
                    {
                        try
                        {
                            _endpoint.EndpointType = (ServiceEndPointType)Enum.Parse(typeof(ServiceEndPointType), parametervalue);                            
                        }
                        catch
                        {
                            // This was not an acceptable end point method
                        }
                    }
                    else if (parametername.Equals("USERNAME") || parametername.Equals("USER") || parametername.Equals("USER ID")
                        || parametername.Equals("USER NAME") || parametername.Equals("USER ID") || parametername.Equals("USERID"))
                    {
                        string[] domainuser = parametervalue.Split('\\');
                        if (domainuser.Length == 2)
                        {
                            // This user has a domain
                            _domain = domainuser[0];
                            _username = domainuser[1];
                        }
                        else
                        {
                            // This user has no domain so set it to nothing
                            _domain = String.Empty;
                            _username = parametervalue;
                        }
                    }
                    else if (parametername.Equals("PASSWORD") || parametername.Equals("PWD"))
                    {
                        _password = parametervalue;
                    }
                    else if (parametername.Equals("INTEGRATEDSECURITY") || parametername.Equals("INTEGRATED SECURITY")
                        || parametername.Equals("INTEGRATEDAUTHENTICATION") || parametername.Equals("INTEGRATED AUTHENTICATION"))
                    {
                        bool integrated = true;
                        bool.TryParse(parametervalue, out integrated);
                        _endpoint.IntegratedAuthentication = integrated;
                    }
                    else if (parametername.Equals("INITIAL CATALOG") || parametername.Equals("CATALOG") || parametername.Equals("CUBE"))
                    {
                        _cube = parametervalue;
                    }
                    else if (parametername.Equals("TIMEOUT") || parametername.Equals("TIME OUT") || parametername.Equals("CONNECT TIMEOUT"))
                    {
                        try
                        {
                            _timeout = Convert.ToInt16(parametervalue);
                        }
                        catch
                        {
                            _timeout = 0;
                        }
                    }
                    else if (parametername.Equals("ADMIN") || parametername.Equals("ADMINSTRATION") || parametername.Equals("ADMINMODE")
                         || parametername.Equals("ADMIN MODE"))
                    {
                        try
                        {
                            _admin = Convert.ToBoolean(parametervalue);
                        }
                        catch
                        {
                            _admin = false;
                        }
                    }
                }
            }
        }

        private string _connectionstring;
        /// <summary>
        /// The Connection String used to connect to the Cubed server
        /// </summary>
        [DataMember]
        public string ConnectionString
        {
            get 
            { 
                // First create a new connection string based on our object details
                string connectionstring = "SERVER=" + _endpoint.Server + ";PORT=" + _endpoint.Port.ToString() + ";";
                connectionstring += "USER=" + _domain + "\\" + _username + ";PROTOCOL=" + _endpoint.EndpointType.ToString() + ";";                
                if (_endpoint.IntegratedAuthentication)
                {
                    connectionstring += "INTEGRATED SECURITY=true;";
                }
                else
                {
                    connectionstring += "PASSWORD=" + _password + ";";

                }
                if (_admin)
                {
                    // This is an admin connection
                    connectionstring += "ADMIN=true;";
                }
                else
                {
                    // This isn't an admin connection
                    connectionstring += "CUBE=" + _cube + ";";
                }

                connectionstring += "TIMEOUT=" + _timeout.ToString();

                // Return the encoded string
                return connectionstring;
            }
            set 
            { 
                // First update the internal connection string with the new value
                _connectionstring = value; 

                // Then reset all our internal values to their defaults (before parsing the 
                // connections string)
                _endpoint = new ServiceEndPoint();
                _domain = String.Empty;
                _cube = String.Empty;
                _admin = true;
                _timeout = 300;
                _password = String.Empty;

                // Then parse the connection string into the object
                ParseString(_connectionstring);
            }
        }

        private ServiceEndPoint _endpoint;
        /// <summary>
        /// The details of the end point this client is connecting to
        /// </summary>
        [DataMember]
        public ServiceEndPoint EndPoint
        {
            get { return _endpoint; }
            set { _endpoint = value; }
        }

        private string _username;
        /// <summary>
        /// The Username used to connect to the Cubed Server (always required)
        /// </summary>
        [DataMember]
        public string UserName
        {
            get { return _username; }
            set { _username = value; }
        }

        private string _domain;
        /// <summary>
        /// The domain of the current user (if using domain authentication)
        /// </summary>
        [DataMember]
        public string Domain
        {
            get { return _domain; }
            set { _domain = value; }
        }

        /// <summary>
        /// This returns the user name and domain if a valid domain has been supplied, otherwise just the
        /// username
        /// </summary>
        public string DomainUser
        {
            get 
            {
                if (_domain != String.Empty)
                    return _domain + "\\" + _username;
                else
                    return _username;
            } 
        }

        private string _password;
        /// <summary>
        /// The users password (only required if NOT using domain authentication, or if the user's computer is 
        /// not on the same domain as the server) 
        /// </summary>
        [DataMember]
        public string Password
        {
            get { return _password; }
            set { _password = value; }
        }

        private string _cube;
        /// <summary>
        /// The name of the Cube to connect to
        /// </summary>
        [DataMember]
        public string Cube
        {
            get { return _cube; }
            set { _cube = value; }
        }

        private bool _admin;
        /// <summary>
        /// Returns true if this is an administration connection, false if not
        /// </summary>
        [DataMember]
        public bool Admin
        {
            get { return _admin; }
            set { _admin = value; }
        }

        private int _timeout;
        /// <summary>
        /// Sets the time (in seconds) that will be used to automatically disconnect from the server if no
        /// activity is detected
        /// </summary>
        [DataMember]
        public int Timeout
        {
            get { return _timeout; }
            set { _timeout = value; }
        }
    }
}
