/*
 * This source file (Cube.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.ServiceModel;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// This is the primary cube structure containing all child elements.
    /// </summary> 
    [Serializable]
    [DataContract]
    public class Cube
    {
        /// <summary>
        /// Creates a blank new cube
        /// </summary>
        public Cube()
        {
            _id = Guid.NewGuid();
            _name = "New Cube";
            _description = "New Cube";
            _storagemode = CubeStorageModes.LeafOnly;
            _datasources = new GenericKeyCollection<Guid, Datasource>();
            _dimensions = new GenericKeyCollection<Guid, Dimension>();
            _measures = new GenericKeyCollection<Guid, Measure>();
            _readaccessroles = new GenericKeyCollection<Guid, Role>();
            _writeaccessroles = new GenericKeyCollection<Guid, Role>();
            _consolidationinterval = 0;
            _lastconsolidation = new DateTime(1901, 1, 1, 0, 0, 0);
            _consolidatinguser = "N/A";
            _consolidationresult = String.Empty;
            _nextconsolidation = new DateTime(1901, 1, 1, 0, 0, 0);
            _pointconsolidation = false;
            _startconsolidation = false;
            _consolidationstarted = new DateTime(1901, 1, 1, 0, 0, 0);
            _consolidationinprogress  = false;
            _consolidationnodesprocessed = 0;
            _notificationusers = new List<User>();
        }

        /// <summary>
        /// Creates a new cube
        /// </summary>
        /// <param name="name">The name of the new cube to create</param>
        public Cube(string name)
        {
            _id = Guid.NewGuid();
            _name = name;
            _description = name;
            _storagemode = CubeStorageModes.LeafOnly;
            _datasources = new GenericKeyCollection<Guid, Datasource>();
            _dimensions = new GenericKeyCollection<Guid, Dimension>();
            _measures = new GenericKeyCollection<Guid, Measure>();
            _readaccessroles = new GenericKeyCollection<Guid, Role>();
            _writeaccessroles = new GenericKeyCollection<Guid, Role>();
            _consolidationinterval = 0;
            _lastconsolidation = new DateTime(1901, 1, 1, 0, 0, 0);
            _consolidatinguser = "N/A";
            _consolidationresult = String.Empty;
            _nextconsolidation = new DateTime(1901, 1, 1, 0, 0, 0);
            _pointconsolidation = false;
            _startconsolidation = false;
            _consolidationstarted = new DateTime(1901, 1, 1, 0, 0, 0);
            _consolidationinprogress  = false;
            _consolidationnodesprocessed = 0;
            _notificationusers = new List<User>();
        }

        private Guid _id;
        /// <summary>
        /// This is the unique identifier of the cube
        /// </summary>
        [DataMember]
        public Guid ID
        {
            get { return _id; }
            set { _id = value; }
        }

        private string _name;
        /// <summary>
        /// This is the name of the cube
        /// </summary>
        [DataMember]
        public string Name
        {
            get { return _name; }
            set {  _name = value; }
        }

        private string _description;
        /// <summary>
        /// The description of the cube
        /// </summary>
        [DataMember]
        public string Description
        {
            get { return _description; }
            set { _description = value; }
        }

        private CubeStorageModes _storagemode;
        /// <summary>
        /// The storage mode employed by the cube
        /// </summary>
        [DataMember]
        public CubeStorageModes StorageMode
        {
            get { return _storagemode; }
            set { _storagemode = value; }
        }

        private List<User> _notificationusers;
        /// <summary>
        /// The list of users who receive notifications about this cube
        /// </summary>
        [DataMember]
        public List<User> NotificationUsers
        {
            get { return _notificationusers; }
            set { _notificationusers = value; }
        }
        
        private GenericKeyCollection<Guid, Datasource> _datasources;
        /// <summary>
        /// The Data Sources available for use by this cube
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, Datasource> Datasources
        {
            get  { return _datasources;  }
            set  {  _datasources = value; }
        }
                
        /// <summary>
        /// Adds a new datasource to this Cubed Cube
        /// </summary>
        /// <param name="name">The name of the new datasource</param>
        /// <param name="provider">The provider type of this datasource</param>
        /// <returns>The new datasource</returns>
        public Datasource AddDataSource(string name, DatabaseConnector provider)
        {
            Datasource newds = new Datasource(name, provider);
            _datasources.Add(newds);
            return newds;
        }
                
        private GenericKeyCollection<Guid, Dimension> _dimensions;
        /// <summary>
        /// The Dimension Collection of the cube
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, Dimension> Dimensions
        {
            get { return _dimensions;  }
            set  { _dimensions = value; }
        }

        /// <summary>
        /// Adds a new dimension to this Cubed Cube
        /// </summary>
        /// <param name="name">The name of the new Dimension</param>
        /// <param name="datasource">The datasource used by this dimension</param>
        /// <returns>The new Dimension</returns>
        public Dimension AddDimension(string name, Datasource datasource)
        {
            Dimension newdim = new Dimension(name, datasource);
            _dimensions.Add(newdim);
            return newdim;
        }

        private GenericKeyCollection<Guid, Role> _readaccessroles;
        /// <summary>
        /// The Roles who are permitted READ access this cube
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, Role> ReadAccessRoles
        {
            get { return _readaccessroles; }
            set { _readaccessroles = value; }
        }

        private GenericKeyCollection<Guid, Role> _writeaccessroles;
        /// <summary>
        /// The Roles who are permitted WRITE access this cube
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, Role> WriteAccessRoles
        {
            get { return _writeaccessroles; }
            set { _writeaccessroles = value; }
        }
        
        /// <summary>
        /// Adds a new role to this Cubed Cube
        /// </summary>
        /// <param name="name">The name of the new Role</param>
        /// <returns>The new Role</returns>
        public Role AddRole(string name)
        {
            Role newrole = new Role(name);
            _readaccessroles.Add(newrole);
            return newrole;
        }

        private GenericKeyCollection<Guid, Measure> _measures;
        /// <summary>
        /// The MeasureCollection of the cube
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, Measure> Measures
        {
            get { return _measures;  }
            set { _measures = value; }
        }

        /// <summary>
        /// Adds a new measure to this Cubed Cube
        /// </summary>
        /// <param name="name">The name of the new Measure</param>
        /// <param name="datasource">The unique identifier of the datasource used by this measure</param>
        /// <param name="type">The provider type of the new measure</param>
        /// <returns>The new Measure</returns>
        public Measure AddMeasure(string name, Datasource datasource, DataTypes type)
        {
            Measure newmeasure = new Measure(name, datasource, type);
            _measures.Add(newmeasure);
            return newmeasure;
        }

        private bool _startconsolidation;
        /// <summary>
        /// Whether a consolidation should be started on this cube when the consolidation engine next checks it
        /// </summary>
        [DataMember]
	    public bool StartConsolidation
	    {
		    get { return _startconsolidation; }
		    set { _startconsolidation = value;}
	    }

        private bool _consolidationinprogress;
        /// <summary>
        /// Whether a consolidation should be started on this cube when the consolidation engine next checks it
        /// </summary>
        [DataMember]
	    public bool ConsolidationInProgress
	    {
		    get { return _consolidationinprogress;}
		    set { _consolidationinprogress = value;}
	    }

        private DateTime _consolidationstarted;
        /// <summary>
        /// The time the current consolidation was started (if one is active)
        /// </summary>
        [DataMember]
        public DateTime ConsolidationStarted
        {
            get { return _consolidationstarted; }
            set { _consolidationstarted = value; }
        }

        private int _consolidationinterval;
        /// <summary>
        /// The interval (in minutes) between automatic consolidations
        /// </summary>
        [DataMember]
        public int ConsolidationInterval
        {
            get { return _consolidationinterval; }
            set { _consolidationinterval = value; }
        }

        private bool _pointconsolidation;
        /// <summary>
        /// Whether the next consolidation for this cube should be a point consolidation rather than
        /// a full consolidation
        /// </summary>
        [DataMember]
        public bool PointConsolidation
        {
            get { return _pointconsolidation; }
            set { _pointconsolidation = value; }
        }

        private DateTime _nextconsolidation;
        /// <summary>
        /// The time to run the next automatic consolidation
        /// </summary>
        [DataMember]
        public DateTime NextConsolidation
        {
            get { return _nextconsolidation; }
            set { _nextconsolidation = value; }
        }

        private DateTime _lastconsolidation;
        /// <summary>
        /// The time the last consolidation was run
        /// </summary>
        [DataMember]
        public DateTime LastConsolidation
        {
            get { return _lastconsolidation; }
            set { _lastconsolidation = value; }
        }

        private string _consolidatinguser;
        /// <summary>
        /// The name of the user who last run the consolidation (or "SYSTEM" if it was an automatic consolidation)
        /// </summary>
        [DataMember]
        public string ConsolidatingUser
        {
            get { return _consolidatinguser; }
            set { _consolidatinguser = value; }
        }

        private string _consolidationresult;
        /// <summary>
        /// This contains the result of the last consolidation
        /// </summary>
        [DataMember]
        public string ConsolidationResult
        {
            get { return _consolidationresult; }
            set { _consolidationresult = value; }
        }

        /// <summary>
        /// This returns the total number of nodes of data which this cube contains
        /// </summary>   
        public long TotalNodes
        {
            get
            {
                List<int> dimensionmembers = new List<int>();
                foreach (Dimension thisdimension in _dimensions)
                    dimensionmembers.Add(thisdimension.Members.Count);
                dimensionmembers.Add(_measures.Count);
                if (dimensionmembers.Count > 0)
                {
                    long totalnodes = Convert.ToInt64(dimensionmembers[0]);
                    if (dimensionmembers.Count > 1)
                    {
                        for (int i = 1; i < dimensionmembers.Count; i++)
                        {
                            totalnodes = totalnodes * dimensionmembers[i];
                        }
                    }
                    return totalnodes;
                }
                else
                    return 0;
            }
        }

        private long _consolidationnodesprocessed;
        /// <summary>
        /// The total number of nodes already processed in the current consolidation
        /// </summary>
        [DataMember]
        public long ConsolidationNodesProcessed
        {
            get { return _consolidationnodesprocessed; }
            set { _consolidationnodesprocessed = value; }
        }

        /// <summary>
        /// The percentage of the total consolidation completed
        /// </summary>
        public double Complete
        {
            get
            {
                try
                {
                    return (_consolidationnodesprocessed / TotalNodes) * 100;
                }
                catch
                {
                    return 0;
                }
            }
        }
    }
}
