/*
 * This source file (CubedFunctionParameter.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// This provides the ability to add parameters to Cubed Functions
    /// </summary>
    [Serializable]
    [DataContract]
    public class CubedFunctionParameter
    {
        /// <summary>
        /// Initialises a default instance of a new Cubed Function Parameter
        /// </summary>
        public CubedFunctionParameter()
        {
            name = String.Empty;
            required = false;
            parametervaluetype = typeof(string);
            parametervalue = null;
        }

        /// <summary>
        /// Initialises a new instance of the CubedFunctionParameter based on the passed in details
        /// </summary>
        /// <param name="_name">The name of the new parameter</param>
        /// <param name="_required">Whether the parameter is required or not</param>
        /// <param name="_valuetype">The type of value this parameter expects</param>
        public CubedFunctionParameter(string _name, bool _required, Type _valuetype)
        {
            name = _name;
            required = _required;
            parametervaluetype = _valuetype;
            parametervalue = null;
        }

        private string name;
        /// <summary>
        /// The name of the parameter
        /// </summary>
        [DataMember]
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        private bool required;
        /// <summary>
        /// Whether the parameter is required or not
        /// </summary>
        [DataMember]
        public bool Required
        {
            get { return required; }
            set { required = value; }
        }

        private Type parametervaluetype;
        /// <summary>
        /// The type of value this parameter expects
        /// </summary>
        [DataMember]
        public Type ParameterValueType
        {
            get { return parametervaluetype; }
            set { parametervaluetype = value; }
        }

        /// <summary>
        /// This private object contains the parameters value in a neutral form
        /// </summary>
        [DataMember]
        private object parametervalue;

        /// <summary>
        /// This sets the passed in object (which should be the same type as the value type that this
        /// parameter expects) based on the value of the parameter, if its been set
        /// </summary>
        /// <param name="returnvalue">The object which will be set to contain the parameters value</param>
        public void GetParameterValue(out object returnvalue)
        {
            try
            {
                returnvalue = Activator.CreateInstance(ParameterValueType);                
                if (parametervalue != null && parametervalue.GetType() == ParameterValueType)
                {
                    returnvalue = parametervalue;
                }
            }
            catch
            {
                returnvalue = null;
            }
        }

        /// <summary>
        /// This sets the value of the parameter internally based on the passed in object (which should be
        /// the same type as the value type that this parameter expects)
        /// </summary>
        /// <param name="setvalue"></param>
        /// <returns>True if the value was successfully set, False if the passed in value was not valid for
        /// this parameter</returns>
        public bool SetParameterValue(object setvalue)
        {
            if (setvalue != null && setvalue.GetType() == ParameterValueType)
            {
                parametervalue = setvalue;
                return true;
            }
            else
                return false;
        }
    }
}
