/*
 * This source file (DesktopDashboardItem.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Utilities;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// Contains the details of a single dashboard item
    /// </summary>
    [Serializable]
    [DataContract]
    public class DesktopDashboardItem
    {
        /// <summary>
        /// Initialises a new, blank default dashboard item
        /// </summary>
        public DesktopDashboardItem()
        {
            _id = Guid.NewGuid();
            _search = new Guid();
            _result = new Guid();
            _column = 0;
            _row = 0;
            _width = 1;
            _height = 1;
            _refreshinterval = 0;
        }

        /// <summary>
        /// Initialises a new dashboard item with the details given
        /// </summary>
        /// <param name="search">The unique identifier of the search used to generate this dashboard</param>
        /// <param name="result">The unique identifier of the result being used to view this dashboard</param>
        /// <param name="dashboard">The dashboard in which this item will be placed</param>
        /// <param name="refreshinterval">The time (in minutes) to refresh the dashboard. 0 is for "manual refresh"</param>
        public DesktopDashboardItem(Guid search, Guid result, DesktopDashboard dashboard, int refreshinterval)
        {
            // First we assign all fields which have been passed in
            _id = Guid.NewGuid();
            _search = search;
            _result = result;
            _width = 1;
            _height = 1;
            _refreshinterval = refreshinterval;

            //Get the last item in the dashboard
            short lastRow = dashboard.Items.Count > 0 ? dashboard.Items.Max(it => it.Row) : Convert.ToInt16(0);
           
            //Set the row to the next available value
            _row = ++lastRow;
            _column = 1;

        }

        private Guid _id;
        /// <summary>
        /// The unique identifier of this dashboard item
        /// </summary>
        [DataMember]
        public Guid ID
        {
            get { return _id; }
            set { _id = value; }
        }

        private Guid _search;
        /// <summary>
        /// The desktop search this dashboard corresponds to
        /// </summary>
        [DataMember]
        public Guid Search
        {
            get { return _search; }
            set { _search = value; }
        }

        private Guid _result;
        /// <summary>
        /// The result this dashboard is using as a viewer
        /// </summary>
        [DataMember]
        public Guid Result
        {
            get { return _result; }
            set { _result = value; }
        }

        private short _column;
        /// <summary>
        /// The column index of this dashboard item
        /// </summary>
        [DataMember]
        public short Column
        {
            get { return _column; }
            set { _column = value; }
        }

        private short _row;
        /// <summary>
        /// The row index of this dashboard item
        /// </summary>
        [DataMember]
        public short Row
        {
            get { return _row; }
            set { _row = value; }
        }

        private short _width;
        /// <summary>
        /// The width of this dashboard item
        /// </summary>
        [DataMember]
        public short Width
        {
            get { return _width; }
            set { _width = value; }
        }

        private short _height;
        /// <summary>
        /// The height of this dashboard item
        /// </summary>
        [DataMember]
        public short Height
        {
            get { return _height; }
            set { _height = value; }
        }

        private int _refreshinterval;
        /// <summary>
        /// The interval (in minutes) to refresh this dashboard item. Enter 0 for "manual refresh".
        /// </summary>
        [DataMember]
        public int RefreshInterval
        {
            get { return _refreshinterval; }
            set { _refreshinterval = value; }
        }
    }
}