/*
 * This source file (DesktopSearch.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Utilities;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// Contains the details of a single saved search for the Cubed Desktop
    /// </summary>
    [Serializable]
    [DataContract]
    public class DesktopSearch
    {
        /// <summary>
        /// Initialises a new, blank default search
        /// </summary>
        public DesktopSearch()
        {
            _id = Guid.NewGuid();
            _name = "New Search";
            _description = String.Empty;
            _keywords = new List<string>();
            _availableviewers = new GenericKeyCollection<Guid, DesktopSearchResult>();
            _defaultresult = new Guid();
            _searchid = new Guid();
            _searchversion = 1.0;
            _searchdetails = String.Empty;
            _index = new List<DesktopSearchIndex>();
            _supportsweb = true;
            _supportswindows = false;
            _ispublic = true;
            _privateuser = String.Empty;
            _bookmarked = false;
            _category = Guid.Empty;
        }

        /// <summary>
        /// Initialises a new search based on the details given
        /// </summary>
        /// <param name="name">The name of the new search</param>
        /// <param name="keywords">The list of keywords to match for this search</param>
        /// <param name="availableviewers">The available viewers for this search</param>
        /// <param name="defaultresult">The default result viewer for this search</param>
        /// <param name="searchid">The unique identifier of the Search Add In used for this search</param>
        /// <param name="searchversion">The version of the Search Add In used for this search</param>
        /// <param name="supportsweb">Whether this search supports web viewing</param>
        /// <param name="supportswindows">Whether this search supports windows viewing</param>
        /// <param name="searchdetails">The details of the search</param>
        /// <param name="ispublic">Whether this search is available to all users publicly</param>
        /// <param name="privateuser">If this search is not public, this is the name of the user who owns
        /// <param name="bookmarked">Whether this search is bookmarked.</param>
        /// the search</param>
        public DesktopSearch(string name, List<string> keywords, GenericKeyCollection<Guid, DesktopSearchResult> availableviewers, Guid defaultresult,
            Guid searchid, double searchversion, string searchdetails, bool supportsweb, bool supportswindows,
            bool ispublic, string privateuser, bool bookmarked, Guid category)
        {
            // First we assign all fields
            _id = Guid.NewGuid();
            _name = name;
            _availableviewers = availableviewers;
            _defaultresult = defaultresult;
            _searchid = searchid;
            _searchversion = searchversion;
            _searchdetails = searchdetails;
            _index = new List<DesktopSearchIndex>();
            _supportsweb = supportsweb;
            _supportswindows = supportswindows;
            _ispublic = ispublic;
            _privateuser = privateuser;
            _bookmarked = bookmarked;
            _category = category;
        }

        private Guid _id;
        /// <summary>
        /// The unique identifier of this new search
        /// </summary>
        [DataMember]
        public Guid ID
        {
            get { return _id; }
            set { _id = value; }
        }

        private string _name;
        /// <summary>
        /// The name of the search
        /// </summary>
        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        private string _description;
        /// <summary>
        /// The description of the search
        /// </summary>
        [DataMember]
        public string Description
        {
            get { return _description; }
            set { _description = value; }
        }

        private List<string> _keywords;
        /// <summary>
        /// The list of key words which match this search
        /// </summary>
        [DataMember]
        public List<string> Keywords
        {
            get { return _keywords; }
            set { _keywords = value; }
        }

        private GenericKeyCollection<Guid, DesktopSearchResult> _availableviewers;
        /// <summary>
        /// This is the list Desktop Results of the viewers which are enabled for this search
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, DesktopSearchResult> AvailableViewers
        {
            get { return _availableviewers; }
            set { _availableviewers = value; }
        }

        private Guid _defaultresult;
        /// <summary>
        /// This is the unique identifier of the default result to display for this search
        /// </summary>
        [DataMember]
        public Guid DefaultResult
        {
            get { return _defaultresult; }
            set { _defaultresult = value; }
        }

        private Guid _searchid;
        /// <summary>
        /// This is the unique identifier of the Search Add In used to generate this search
        /// </summary>
        [DataMember]
        public Guid SearchID
        {
            get { return _searchid; }
            set { _searchid = value; }
        }

        private double _searchversion;
        /// <summary>
        /// This is the version of the Search Add In used to generatethis search
        /// </summary>
        [DataMember]
        public double SearchVersion
        {
            get { return _searchversion; }
            set { _searchversion = value; }
        }

        private string _searchdetails;
        /// <summary>
        /// This is the search details configuration for this search
        /// </summary>
        [DataMember]
        public string SearchDetails
        {
            get { return _searchdetails; }
            set { _searchdetails = value; }
        }

        private List<DesktopSearchIndex> _index;
        /// <summary>
        /// The index for the search
        /// </summary>
        [DataMember]
        public List<DesktopSearchIndex> Index
        {
            get { return _index; }
            set { _index = value; }
        }

        private bool _supportsweb;
        /// <summary>
        /// Whether this search supports viewing in the web desktop
        /// </summary>
        [DataMember]
        public bool SupportsWeb
        {
            get { return _supportsweb; }
            set { _supportsweb = value; }
        }

        private bool _supportswindows;
        /// <summary>
        /// Whether this search supports viewing in the windows desktop
        /// </summary>
        [DataMember]
        public bool SupportsWindows
        {
            get { return _supportswindows; }
            set { _supportswindows = value; }
        }

        private bool _ispublic;
        /// <summary>
        /// Whether this search is available publicly for all users of the desktop
        /// </summary>
        [DataMember]
        public bool IsPublic
        {
            get { return _ispublic; }
            set { _ispublic = value; }
        }

        private string _privateuser;
        /// <summary>
        /// If this is a private view, this is the name of the user who the view belongs to
        /// </summary>
        [DataMember]
        public string PrivateUser
        {
            get { return _privateuser; }
            set { _privateuser = value; }
        }

        private bool _bookmarked;
        /// <summary>
        /// Whether this search is bookmarked.
        /// </summary>
        [DataMember]
        public bool Bookmarked
        {
            get { return _bookmarked; }
            set { _bookmarked = value; }
        }

        private Guid _category;
        /// <summary>
        /// The unique identifier for the category assigned
        /// </summary>
        [DataMember]
        public Guid Category
        {
            get { return _category; }
            set { if(_category != value) _category = value; }
        }
    }
}