/*
 * This source file (Dimension.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Utilities;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// The dimensions provide the segments for the data to be summarized in
    /// </summary>
    [Serializable]
    [DataContract]
    public class Dimension
    {
        /// <summary>
        /// Creates a blank new Dimension
        /// </summary>
        public Dimension()
        {
            _id = Guid.NewGuid();
            _name = "New Dimension";
            _description = "New Dimension";
            _enabled = true;
            _timedimension = false;
            _timeperiod = TimePeriodTypes.MonthYear;
            _membertype = MemberTypes.Identifier;
            _attributes = new GenericKeyCollection<Guid, Attribute>();
            _kpis = new GenericKeyCollection<Guid, KPI>();
            _membergenerators = new GenericKeyCollection<Guid, MemberGenerator>();
            _datasource = new Guid();
            _members = new GenericKeyCollection<Guid, Member>();
            _levels = new GenericKeyCollection<Guid, Level>();
            _measurelinks = new GenericKeyCollection<Guid, MeasureLink>();
            _addtotalmember = true;
            _autoorder = true;
        }

        /// <summary>
        /// Create a new dimension to partition data into
        /// </summary>
        /// <param name="name"> This is the name of the new dimension to create</param>
        /// <param name="datasource">This is the datasource used by the new dimension</param>
        public Dimension(string name, Datasource datasource)
        {
            _id = Guid.NewGuid();
            _name = name;
            _description = name;
            _enabled = true;
            _timedimension = false;
            _timeperiod = TimePeriodTypes.MonthYear;
            _membertype = MemberTypes.Identifier;
            _attributes = new GenericKeyCollection<Guid, Attribute>();
            _kpis = new GenericKeyCollection<Guid, KPI>();
            _membergenerators = new GenericKeyCollection<Guid, MemberGenerator>();
            _datasource = datasource.ID;
            _members = new GenericKeyCollection<Guid, Member>();
            _levels = new GenericKeyCollection<Guid, Level>();
            _measurelinks = new GenericKeyCollection<Guid, MeasureLink>();
            _addtotalmember = true;
            _autoorder = true;
        }

        private Guid _id;
        /// <summary>
        /// The unique identifier of this new dimension
        /// </summary>
        [DataMember]
        public Guid ID
        {
            get { return _id; }
            set { _id = value; }
        }

        private string _name;
        /// <summary>
        /// The name of the dimension
        /// </summary>
        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        /// <summary>
        /// The caption of the dimension. This is provided for compatibility purposes with SQL Server Analysis Services. It is
        /// the same as the Name property.
        /// </summary>
        public string Caption
        {
            get { return _name; }
        }

        private string _description;
        /// <summary>
        /// The description of the dimension
        /// </summary>
        [DataMember]
        public string Description
        {
            get { return _description; }
            set { _description = value; }
        }

        /// <summary>
        /// The unique name of the dimension. This is provided for compatibility purposes with SQL Server Analysis Services. It is
        /// the same as the ID property.
        /// </summary>
        public Guid UniqueName
        {
            get { return _id; }
        }

        private bool _enabled;
        /// <summary>
        /// Dimensions which are disabled are not aggregated or available for questioning
        /// </summary>
        [DataMember]
        public bool Enabled
        {
            get { return _enabled; }
            set { _enabled = value; }
        }

        private bool _timedimension;
        /// <summary>
        /// Marks that this dimension is based on a date/time field
        /// </summary>
        [DataMember]
        public bool TimeDimension
        {
            get { return _timedimension; }
            set { _timedimension = value; }
        }

        private TimePeriodTypes _timeperiod;
        /// <summary>
        /// If this dimension is a time dimension, what is the format of the time period to use for it
        /// </summary>
        [DataMember]
        public TimePeriodTypes TimePeriod
        {
            get { return _timeperiod; }
            set { _timeperiod = value; }
        }

        private bool _autoorder;
        /// <summary>
        /// If this dimension is automatically ordered or whether the order is set manually by the designer
        /// </summary>
        [DataMember]
        public bool AutoOrder
        {
            get { return _autoorder; }
            set { _autoorder = value; }
        }

        private MemberTypes _membertype;
        /// <summary>
        /// The type of members which are used on this dimension - either identifier or filter based
        /// </summary>
        [DataMember]
        public MemberTypes MemberType
        {
            get { return _membertype; }
            set { _membertype = value; }
        }

        private GenericKeyCollection<Guid, Attribute> _attributes;
        /// <summary>
        /// The attributes available to use with members on this collection
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, Attribute> Attributes
        {
            get { return _attributes; }
            set { _attributes = value;}
        }

        private GenericKeyCollection<Guid, KPI> _kpis;
        /// <summary>
        /// The collection of KPIs which are calculated on this dimension
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, KPI> KPIs
        {
            get { return _kpis; }
            set { _kpis = value; }
        }

        private GenericKeyCollection<Guid, MemberGenerator> _membergenerators;
        /// <summary>
        /// This is the collection of Member Generators used to automatically generate this dimensions'
        /// members at consolidation time
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, MemberGenerator> MemberGenerators
        {
            get { return _membergenerators; }
            set { _membergenerators = value; }
        }

        private Guid _datasource;
        /// <summary>
        /// The datasource for this dimension
        /// </summary>
        [DataMember]
        public Guid Datasource
        {
            get { return _datasource; }
            set { _datasource = value; }
        }

        private GenericKeyCollection<Guid, Level> _levels;
        /// <summary>
        /// The levels in this dimension
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, Level> Levels
        {
            get { return _levels; }
            set { _levels = value; }
        }

        /// <summary>
        /// This returns the default member for the dimension. This is the first appearing highest-level
        /// member in the collection. It returns null if no members exist at the base (0) level.
        /// </summary>
        public Member DefaultMember
        {
            get
            {
                return _members.FirstOrDefault(testmember => testmember.Level == 0);
            }
        }

        private GenericKeyCollection<Guid, Member> _members;
        /// <summary>
        /// The members of this dimension
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, Member> Members
        {
            get { return _members; }
            set 
            {
                // When we set the members collection, check to see if a new member has been added; if
                // it has, check the levels to see if we need to add a new one
                _members = value;
                CheckLevels();
            }
        }

        private GenericKeyCollection<Guid, MeasureLink> _measurelinks;
        /// <summary>
        /// The links between this dimension and the cube measures
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, MeasureLink> MeasureLinks
        {
            get { return _measurelinks; }
            set { _measurelinks = value; }
        }

        /// <summary>
        /// This function checks all members in the collection to ensure their level exists properly
        /// </summary>
        public void CheckLevels()
        {
            // Check for each member in the collection that it has an appropriate level available
            foreach (Member thismember in _members)
            {
                // First check to see if the level exists
                bool levelexists = false;
                foreach (Level thislevel in _levels)
                    if (thislevel.Depth == thismember.Level)
                    {
                        // This level exists already
                        levelexists = true;

                        // Add the member if it the level doesn't already contain it
                        if (!thislevel.Members.Contains(thismember.ID))
                            thislevel.Members.Add(thismember.ID);
                        break;
                    }
                if (!levelexists)
                {
                    // Create the new level as it doesn't already exist
                    Level newlevel = new Level("Level " + thismember.Level.ToString(),
                    this, thismember.Level);

                    // Then we add the member to the level
                    newlevel.Members.Add(thismember.ID);

                    // Then we add the level into the collection
                    _levels.Add(newlevel);                    
                }
            }
        }

        /// <summary>
        /// Adds a new parent member to this dimension (a member at the root level)
        /// </summary>
        /// <param name="newname">The name of the new member to add</param>
        /// <returns>The new member</returns>
        public Member AddMember(string newname)
        {
            Member newmember = new Member(newname, null, this);
            newmember.Name = newname;

            // Set the level of this member to "0" to indicate that they are the root level
            newmember.Level = 0;

            // Add the new member to the collection
            _members.Add(newmember);

            // Check to ensure the level exists for the member
            CheckLevels();

            return newmember;
        }

        /// <summary>
        /// Adds a new member to this dimension
        /// </summary>
        /// <param name="newname">The name of the new member to add</param>
        /// <param name="newparent">The parent of the new member</param>
        /// <returns>The new member</returns>
        public Member AddMember(string newname, Member newparent)
        {
            Member newmember = new Member(newname, newparent, this);

            // Set this member's level based on it's parent
            newmember.Level = _members[newparent.ID].Level + 1;

            // Add the new member to the collection
            _members.Add(newmember);

            // Add the member to the parents children collection
            _members[newparent.ID].Children.Add(newmember.ID);

            // Check to ensure the level exists for the member
            CheckLevels();

            return newmember;
        }

        /// <summary>
        /// Adds a new link between this dimension and a measure in the cube
        /// </summary>
        /// <param name="measure">The measure to link to</param>
        /// <param name="measureidcolumn">The column in the measure table containing the measures identifier,
        /// if it isn't directly connected to the members</param>
        /// <param name="linkingtable">The name of the table in the datasource where the link is stored (if used, else
        /// a direct link is presumed0</param>
        /// <param name="linkingtablemeasureidcolumn">The name of the column in the linking table representing the ID of the measure</param>
        /// <param name="linkingtablememberidcolumn">The name of the column in the linking table representing the ID of the member</param>        
        public void AddMeasureLink(Measure measure, string measureidcolumn, string linkingtable, 
            string linkingtablemeasureidcolumn, string linkingtablememberidcolumn)
        {
            MeasureLink newmeasurelink = new MeasureLink(measure, measureidcolumn, linkingtable, 
                linkingtablemeasureidcolumn, linkingtablememberidcolumn);
            _measurelinks.Add(newmeasurelink);
        }

        private bool _addtotalmember;
        /// <summary>
        /// Whether to include a total member on the dimension when generating at consolidation time
        /// </summary>
        [DataMember]
        public bool AddTotalMember
        {
            get { return _addtotalmember; }
            set { _addtotalmember = value; }
        }

        /// <summary>
        /// This function repairs the hierarchy of the dimension by ensuring all levels exist as they should
        /// and members are correctly parented
        /// </summary>
        public void RepairTree()
        {
            // First check members know about their children
            foreach (Member thismember in _members)
            {
                // First find any children of this member and ensure the member knows about them
                foreach (Member thischild in _members.Where(testmember => testmember.Parent == thismember.ID))
                {
                    // This member is a child of the current member so check the member knows about it
                    if (!thismember.Children.Contains(thischild.ID))
                        thismember.Children.Add(thischild.ID);
                }                
            }

            // Then check that all members without parents exist at the root level, and update their children to
            // exist at a lower level
            foreach (Member thismember in _members.Where(testmember => testmember.Parent == new Guid()))
            {
                thismember.Level = 0;
                SetChildLevels(thismember.Children, 0);
            }

            // Finally check all levels to ensure they exist and the members know about them
            CheckLevels();
        }

        /// <summary>
        /// This method sets the level property on all children listed
        /// </summary>
        /// <param name="children">A collection of unique identifiers of children to set the level of</param>
        /// <param name="parentlevel">The level of the parent member</param>
        public void SetChildLevels(List<Guid> children, int parentlevel)
        {
            for (int i = children.Count -1; i > -1; i--)            
            {
                // Grab the child member (if it exists)
                if (_members.Contains(children[i]))
                {
                    Member thischild = _members[children[i]];
                    thischild.Level = parentlevel + 1;

                    // Now that it has the correct level, set the level on all of it's children
                    SetChildLevels(thischild.Children, thischild.Level);
                }
                else
                {
                    // This child no longer exists in the collection, so remove it from the parent
                    // list
                    children.RemoveAt(i);
                }
            }
        }

        /// <summary>
        /// Returns a list of the unique identifiers of the given members ascendants
        /// </summary>
        /// <param name="member">The member to return the ascendants of</param>
        /// <param name="includeself">Whether to include the member itself in the returned list</param>
        /// <returns>A list of unique identifiers of the members ascendants</returns>
        public List<Guid> GetMemberAscendantIDs(Member member, bool includeself)
        {
            List<Guid> memberids = new List<Guid>();
            if (includeself)
                memberids.Add(member.ID);
            foreach (Member thismember in GetMemberAscendants(member))
                memberids.Add(thismember.ID);
            return memberids;
        }

        /// <summary>
        /// Returns a list of the unique identifiers of the given members descendants
        /// </summary>
        /// <param name="member">The member to return the descendants of</param>
        /// <param name="includeself">Whether to include the member itself in the returned list</param>
        /// <returns>A list of unique identifiers of the members descendants</returns>
        public List<Guid> GetMemberDescendantIDs(Member member, bool includeself)
        {
            List<Guid> memberids = new List<Guid>();
            if (includeself)
                memberids.Add(member.ID);
            foreach (Member thismember in GetMemberDescendants(member))
                memberids.Add(thismember.ID);
            return memberids;
        }

        /// <summary>
        /// This returns a list of the ascendants of a member
        /// </summary>
        /// <param name="member">The member to check</param>
        /// <returns>Either a list of members (the ascendants) or null</returns>
        public List<Member> GetMemberAscendants(Member member)
        {
            List<Member> members = new List<Member>();           
            GetMemberAscendantList(member, members);
            return members;
        }

        /// <summary>
        /// This returns a list of the descendants of a given member
        /// </summary>
        /// <param name="member">The member to return the descendants of</param>
        /// <returns>A list of members who are descendants of the given member</returns>
        public List<Member> GetMemberDescendants(Member member)
        {
            List<Member> members = new List<Member>();
            GetMemberDescendantList(member, members);
            return members;
        }

        /// <summary>
        /// Updates the given member collection recursively with the members parents
        /// </summary>
        /// <param name="member">The member to search for</param>
        /// <param name="members">The members returned</param>
        private void GetMemberAscendantList(Member member, List<Member> members)
        {
            if (member.ID != null && member.Parent != null && member.Parent != new Guid())
            {
                members.Add(member);
                GetMemberAscendantList(_members[member.Parent], members);
            }
        }

        /// <summary>
        /// Updates the given member collection recursively with the members children
        /// </summary>
        /// <param name="member">The member to check</param>
        /// <param name="members">The list of members to update</param>
        private void GetMemberDescendantList(Member member, List<Member> members)
        {
            foreach (Member thismember in _members.Where(testmember => testmember.Parent ==
                member.ID))
            {
                members.Add(thismember);
                if (thismember.Children.Count > 0)
                    GetMemberDescendantList(thismember, members);
            }
        }

        /// <summary>
        /// Re-orders the member collection either alphabetically or based on the time period if this is a 
        /// time based dimension
        /// </summary>
        public void OrderMembers()
        {
            // First set each member's "display order" property to -1
            foreach (Member thismember in _members)
                thismember.DisplayOrder = -1;

            // Now cycle through each root level member, ordered, and then order each of its' children
            int currentorder = 1;
            List<Member> rootmembers = new List<Member>();
            foreach (Member thismember in _members.Where(testmember => testmember.Parent == null ||
                testmember.Parent == Guid.Empty))
                rootmembers.Add(thismember);
            OrderMemberList(ref rootmembers, ref currentorder);

            // Finally, now the display order property is set, physically re-index the members collection to 
            // their corresponding order so queries, lists, etc. are all ordered correctly
            GenericKeyCollection<Guid, Member> orderedlist = new GenericKeyCollection<Guid, Member>();
            foreach (Member thisorderedmember in _members.OrderBy(testmember => testmember.DisplayOrder))
                orderedlist.Add(thisorderedmember);
            _members = orderedlist;
        }

        /// <summary>
        /// Orders the given list of members by updating their display order property, and then ordering their
        /// children underneath them
        /// </summary>
        /// <param name="memberlist">The list of members to order</param>
        /// <param name="currentorder">The current (starting) order number to use</param>
        private void OrderMemberList(ref List<Member> memberlist, ref int currentorder)
        {
            // Order this collection of members
            if (_timedimension)
            {
                foreach (Member thismember in OrderTimeMembers(memberlist))
                {
                    // First order this member
                    thismember.DisplayOrder = currentorder;
                    currentorder++;

                    // Now order this members children if it has any
                    List<Member> memberchildren = new List<Member>();
                    foreach (Guid thischild in thismember.Children)
                        memberchildren.Add(_members[thischild]);
                    OrderMemberList(ref memberchildren, ref currentorder);
                }
            }
            else
            {
                foreach (Member thismember in memberlist.OrderBy(testmember => testmember.Name))
                {
                    // First order this member
                    thismember.DisplayOrder = currentorder;
                    currentorder++;

                    // Now order this members children if it has any
                    List<Member> memberchildren = new List<Member>();
                    foreach (Guid thischild in thismember.Children)
                        memberchildren.Add(_members[thischild]);
                    OrderMemberList(ref memberchildren, ref currentorder);
                }
            }
        }

        /// <summary>
        /// Returns a list of members ordered assuming they are time based 
        /// </summary>
        /// <param name="members">The members to return ordered</param>
        /// <returns>Returns the list of members ordered by their time value weighting</returns>
        private List<Member> OrderTimeMembers(List<Member> members)
        {
            // Generate a list of members identifiers and their date value weightings
            Dictionary<Guid, DateTime> memberdatevalues = new Dictionary<Guid,DateTime>();

            foreach (Member thismember in members)
            {
                // Search for appropriate time markers and create a date value appropriate
                int year = 1900;
                int quarter = -1;
                int month = -1;
                int day = 1;

                // First attempt to find a year value if one exists
                string[] timemarkers = thismember.Name.Split(' ');
                foreach (string thismarker in timemarkers.Where(teststring => teststring.Length == 4))
                {
                    int yearvalue = -1;
                    if (int.TryParse(thismarker, out yearvalue))
                        year = yearvalue;
                }

                // Now attempt to find a quarter value if one exists
                foreach (string thismarker in timemarkers.Where(teststring => teststring.Length == 2))
                    if (thismarker[0].ToString().ToUpper() == "Q")
                    {
                        // This is a quarter value, so attempt to grab the value of the quarter
                        int quartervalue = -1;
                        if (int.TryParse(thismarker[1].ToString(), out quartervalue))
                            quarter = quartervalue;
                    }

                // Now attempt to find a month value if one exists
                foreach (string thismarker in timemarkers)
                {
                    switch (thismarker.ToUpper())
                    {
                        case "JANUARY":
                            month = 1;
                            break;
                        case "FEBRUARY":
                            month = 2;
                            break;
                        case "MARCH":
                            month = 3;
                            break;
                        case "APRIL":
                            month = 4;
                            break;
                        case "MAY":
                            month = 5;
                            break;
                        case "JUNE":
                            month = 6;
                            break;
                        case "JULY":
                            month = 7;
                            break;
                        case "AUGUST":
                            month = 8;
                            break;
                        case "SEPTEMBER":
                            month = 9;
                            break;
                        case "OCTOBER":
                            month = 10;
                            break;
                        case "NOVEMBER":
                            month = 11;
                            break;
                        case "DECEMBER":
                            month = 12;
                            break;
                    }
                }

                // Finally attempt to find the day if it exists
                foreach (string thismarker in timemarkers.Where(teststring => teststring.Length == 1 || 
                    teststring.Length == 2))
                {
                    int dayvalue = -1;
                    if (int.TryParse(thismarker, out dayvalue))
                        day = dayvalue;
                }

                // Finally, if no month value was specified but a quarter was, then set the value of the 
                // month to the first month of the quarter
                if (month == -1 && (quarter > 0 && quarter < 5))
                {
                    switch (quarter)
                    {
                        case 1:
                            month = 1;
                            break;
                        case 2:
                            month = 4;
                            break;
                        case 3:
                            month = 7;
                            break;
                        case 4:
                            month = 10;
                            break;
                    }
                }
                else if (month == -1)
                {
                    // Set month to January as it wasn't found
                    month = 1;
                }

                // Now add this this value to the collection
                DateTime thisdatetimevalue = new DateTime(year, month, day);
                memberdatevalues.Add(thismember.ID, thisdatetimevalue);
            }

            // Now we have our collection, order it by the date time value obtained into our new member 
            // list
            List<Member> orderedlist = new List<Member>();
            foreach (KeyValuePair<Guid, DateTime> thismember in memberdatevalues.OrderBy(testvalue => testvalue.Value))
            {
                orderedlist.Add(members.FirstOrDefault(testmember => testmember.ID == thismember.Key));
            }

            // Finally return our time ordered list
            return orderedlist;
        }
    }
}
