/*
 * This source file (Member.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Utilities;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// This is the class used for each individual member of a dimension
    /// </summary>
    [Serializable]
    [DataContract]
    public class Member
    {
        /// <summary>
        /// Creates a new blank Member object
        /// </summary>
        public Member()
        {
            _id = Guid.NewGuid();
            _dimension = new Guid();
            _name = "New Member";
            _discovered = DateTime.Now;
            _description = "New Member";
            _filter = new ValueFilter();
            _tablename = String.Empty;
            _primarykey = new List<string>();
            _attributevalues = new GenericKeyCollection<Guid, AttributeValue>();
            _grantedreadroles = new GenericKeyCollection<Guid, Role>();
            _grantedwriteroles = new GenericKeyCollection<Guid, Role>();
            _deniedreadroles = new GenericKeyCollection<Guid, Role>();
            _deniedwriteroles = new GenericKeyCollection<Guid, Role>();
            _parent = new Guid();
            _children = new List<Guid>();
            _level = 0;
            _rollup = true;
            _sign = Signage.Positive;
            _aggregation = AggregationTypes.Sum;
            _consolidation = ConsolidationTypes.AggregateOnly;
            _calculation = new FormulaEvaluator();
            _iscurrent = false;
            _displayorder = -1;
        }

        /// <summary>
        /// Creates a new member record for adding to a dimension
        /// </summary>
        /// <param name="name">This is the name of the new member to create</param>
        /// <param name="parent">This the members parent, or null if this is a root level member</param>
        /// <param name="dimension">This is the dimension to which this member belongs</param>
        public Member(string name, Member parent, Dimension dimension)
        {
            _id = Guid.NewGuid();
            _dimension = dimension.ID;
            _name = name;
            _discovered = DateTime.Now;
            _description = name;
            _filter = new ValueFilter();
            _tablename = String.Empty;
            _primarykey = new List<string>();
            _attributevalues = new GenericKeyCollection<Guid, AttributeValue>();
            _grantedreadroles = new GenericKeyCollection<Guid, Role>();
            _grantedwriteroles = new GenericKeyCollection<Guid, Role>();
            _deniedreadroles = new GenericKeyCollection<Guid, Role>();
            _deniedwriteroles = new GenericKeyCollection<Guid, Role>();
            if (parent != null)
            {
                Parent = parent.ID;
                _level = parent.Level + 1;
            }
            else
            {
                _parent = new Guid();
                _level = 0;
            }
            _children = new List<Guid>();
            _rollup = true;
            _sign = Signage.Positive;
            _aggregation = AggregationTypes.Sum;
            _consolidation = ConsolidationTypes.AggregateOnly;
            _calculation = new FormulaEvaluator();
            _iscurrent = false;
            _displayorder = -1;
        }        

        private Guid _id;
        /// <summary>
        /// The unique identifier of the member
        /// </summary>
        [DataMember]
        public Guid ID
        {
            get { return _id; }
            set { _id = value; }
        }

        private Guid _dimension;
        /// <summary>
        /// The dimension that this member belongs to
        /// </summary>
        [DataMember]
        public Guid Dimension
        {
            get { return _dimension; }
            set { _dimension = value; }
        }

        /// <summary>
        /// The unique name of the member. This is provided for compatibility purposes with SQL Server Analysis Services. It is
        /// the same as the ID property.
        /// </summary>
        public Guid UniqueName
        {
            get { return _id; }
            set { _id = value; }
        }

        private string _name;
        /// <summary>
        /// The name of the member
        /// </summary>
        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value;}
        }

        private DateTime _discovered;
        /// <summary>
        /// The date and time this member was first discovered in the source database (or created if
        /// built manually)
        /// </summary>
        [DataMember]
        public DateTime Discovered
        {
            get { return _discovered; }
            set { _discovered = value; }
        }

        /// <summary>
        /// The caption of the member. This is provided for compatibility purposes with SQL Server Analysis Services. It is
        /// the same as the Name property.                    
        /// </summary>                    
        public string Caption
        {
            get { return _name; }
        }

        private string _description;
        /// <summary>
        /// The description of the member
        /// </summary>
        [DataMember]
        public string Description
        {
            get { return _description; }
            set { _description = value; }
        }

        private ValueFilter _filter;
        /// <summary>
        /// The filter applied to values which make up this member (if this is a RANGE based member)
        /// </summary>
        [DataMember]
        public ValueFilter Filter
        {
            get { return _filter; }
            set { _filter = value; }
        }

        private string _tablename;
        /// <summary>
        /// If this member is an identifier or filter member, this is the name of the table where the member's
        /// identifiers are stored
        /// </summary>
        [DataMember]
        public string Tablename
        {
            get { return _tablename; }
            set { _tablename = value; }
        }

        private List<string> _primarykey;
        /// <summary>
        /// The unique identifier of this member in the underlying data source (if this is an identifier
        /// based member)
        /// </summary>
        [DataMember]
        public List<string> PrimaryKey
        {
            get { return _primarykey; }
            set { _primarykey = value; }
        }

        private GenericKeyCollection<Guid, AttributeValue> _attributevalues;
        /// <summary>
        /// The collection of attribute values associated with this member
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, AttributeValue> AttributeValues
        {
            get { return _attributevalues; }
            set { _attributevalues = value; }
        }

        private GenericKeyCollection<Guid, Role> _grantedreadroles;
        /// <summary>
        /// Roles with access to read this member
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, Role> GrantedReadRoles
        {
            get { return _grantedreadroles; }
            set { _grantedreadroles = value; }
        }

        private GenericKeyCollection<Guid, Role> _grantedwriteroles;
        /// <summary>
        /// Roles listed are granted writeback access to this member
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, Role> GrantedWriteRoles
        {
            get { return _grantedwriteroles; }
            set { _grantedwriteroles = value; }
        }

        private GenericKeyCollection<Guid, Role> _deniedreadroles;
        /// <summary>
        /// Roles which have been denied access to this member
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, Role> DeniedReadRoles
        {
            get { return _deniedreadroles; }
            set { _deniedreadroles = value; }
        }

        private GenericKeyCollection<Guid, Role> _deniedwriteroles;
        /// <summary>
        /// Roles listed are denied writeback access to this member
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, Role> DeniedWriteRoles
        {
            get { return _deniedwriteroles; }
            set { _deniedwriteroles = value; }
        }

        private Guid _parent;
        /// <summary>
        /// The parent of this member (This will be null if the member is at the top level of the dimension)
        /// </summary>
        [DataMember]
        public Guid Parent
        {
            get { return _parent; }
            set { _parent = value; }
        }

        private List<Guid> _children;
        /// <summary>
        /// This returns all children of this member (this will contain no values if the member is a leaf node)
        /// </summary>
        [DataMember]
        public List<Guid> Children
        {
            get { return _children; }
            set { _children = value; }
        }

        private int _level;
        /// <summary>
        /// Gets a number returning the level of the current member within the cube
        /// </summary>
        [DataMember]
        public int Level
        {
            get { return _level; }
            set { _level = value; }
        }

        private bool _rollup;
        /// <summary>
        /// Whether this member is rolled up to it's parents figure
        /// </summary>
        [DataMember]
        public bool RollUp
        {
            get { return _rollup; }
            set { _rollup = value; }
        }

        private Signage _sign;
        /// <summary>
        /// The natural signage of this member
        /// </summary>
        [DataMember]
        public Signage Sign
        {
            get { return _sign; }
            set { _sign = value; }
        }

        private AggregationTypes _aggregation;
        /// <summary>
        /// The method used to aggregate this member
        /// </summary>
        [DataMember]
        public AggregationTypes Aggregation
        {
            get { return _aggregation; }
            set { _aggregation = value; }
        }

        private ConsolidationTypes _consolidation;
        /// <summary>
        /// The type of consolidation used to determine this member's value. 
        /// </summary>
        [DataMember]
        public ConsolidationTypes Consolidation
        {
            get { return  _consolidation; }
            set { _consolidation = value; }
        }

        private FormulaEvaluator _calculation;
        /// <summary>
        /// The calculation interface to use in generating this members value (if not a simple
        /// aggregation only)
        /// </summary>
        [DataMember]
        public FormulaEvaluator Calculation
        {
            get { return _calculation; }
            set { _calculation = value; }
        }

        private bool _iscurrent;
        /// <summary>
        /// Whether the member is a valid current member of the dimension, for example on a time 
        /// dimension the current year and month member's would both be current
        /// </summary>
        [DataMember]
        public bool IsCurrent
        {
            get { return _iscurrent; }
            set { _iscurrent = value; }
        }

        private int _displayorder;
        /// <summary>
        /// The display order of the member
        /// </summary>
        [DataMember]
        public int DisplayOrder
        {
            get { return _displayorder; }
            set { _displayorder = value; }
        }
    }
}
