/*
 * This source file (MemberGenerator.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Utilities;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// Member Generators are used at consolidation time to automatically generate members for a given dimension
    /// </summary>
    [Serializable]
    [DataContract]
    public class MemberGenerator
    {
        /// <summary>
        /// Creates a blank new Member Generator
        /// </summary>
        public MemberGenerator()
        {
            _id = Guid.NewGuid();
            _name = String.Empty;
            _dimension = new Guid();
            _ranges = new RangeGenerator();
            _membertables = new GenericKeyCollection<Guid, MemberTableDetails>();
            _replaceexisting = false;
            _aggregation = AggregationTypes.Sum;
            _sign = Signage.Positive;
        }

        /// <summary>
        /// Creates a Member Generator with the details given
        /// </summary>
        /// <param name="name">The name of the new member generator</param>
        /// <param name="dimension">The dimension this member generator belongs to</param>
        public MemberGenerator(string name, Dimension dimension)
        {
            _id = Guid.NewGuid();
            _name = name;
            _dimension = dimension.ID;
            _membertables = new GenericKeyCollection<Guid, MemberTableDetails>();
            _replaceexisting = false;
            _aggregation = AggregationTypes.Sum;
            _sign = Signage.Positive;
            _ranges = new RangeGenerator(RangeTypes.UniqueValues);
        }

        private Guid _id;
        /// <summary>
        /// The unique identifier of this member generator
        /// </summary>
        [DataMember]
        public Guid ID
        {
            get { return _id; }
            set { _id = value; }
        }

        private string _name;
        /// <summary>
        /// The name of this new member generator
        /// </summary>
        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        private Guid _dimension;
        /// <summary>
        /// The dimension that this member generator belongs to
        /// </summary>
        [DataMember]
        public Guid Dimension
        {
            get { return _dimension; }
            set { _dimension = value; }
        }

        private RangeGenerator _ranges;
        /// <summary>
        /// This defines the ranges, and automatically generates the list of members, if this dimension
        /// uses filter based members
        /// </summary>
        [DataMember]
        public RangeGenerator Ranges
        {
            get { return _ranges; }
            set { _ranges = value; }
        }

        private GenericKeyCollection<Guid, MemberTableDetails> _membertables;
        /// <summary>
        /// This contains the collection of tables with the names of members which will be generated
        /// by this member generator, if this dimension uses identifier based members
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, MemberTableDetails> MemberTables
        {
            get { return _membertables; }
            set { _membertables = value; }
        }

        private bool _replaceexisting;
        /// <summary>
        /// Whether to replace existing members with newly generated measures of the same name at consolidation
        /// time
        /// </summary>
        [DataMember]
        public bool ReplaceExisting
        {
            get { return _replaceexisting; }
            set { _replaceexisting = value; }
        }

        private AggregationTypes _aggregation;
        /// <summary>
        /// The method used to aggregate generated members
        /// </summary>
        [DataMember]
        public AggregationTypes Aggregation
        {
            get { return _aggregation; }
            set { _aggregation = value; }
        }

        private Signage _sign;
        /// <summary>
        /// The natural signage of generated members
        /// </summary>
        [DataMember]
        public Signage Sign
        {
            get { return _sign; }
            set { _sign = value; }
        }
    }
}