/*
 * This source file (Node.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Linq;
using System.Xml.Serialization;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// This represents a node of data in the data matrix
    /// </summary>
    [Serializable]
    [DataContract]
    public class Node
    {
        /// <summary>
        /// Initialises an entirely blank, new node
        /// </summary>
        public Node()
        {
            _measure = new Measure();
            _members = new GenericKeyCollection<Guid, Member>();
            _value = 0;
            _leafnode = false;
        }

        /// <summary>
        /// This is one node of data in the data matrix
        /// </summary>
        /// <param name="members">The members represented by this node</param>
        /// <param name="measure">The measure represented by this node</param>
        /// <param name="value">The value of this node</param>
        /// <param name="leafnode">Whether this new node is a leaf node</param>
        public Node(GenericKeyCollection<Guid, Member> members, Measure measure,
            double value, bool leafnode)
        {
            _measure = measure;
            _members = members;
            _value = value;
            _leafnode = leafnode;
        }

        private string _id;
        /// <summary>
        /// The unique identifier of a node is a composite key made up of the 
        /// individual members and measure of the node
        /// </summary>
        public string ID
        {
            get
            {
                if (_id == null || _id == String.Empty)
                {
                    // Generate the ID
                    _id += _measure.ID.ToString() + "|";
                    foreach (Member thismember in _members)
                        _id += thismember.ID.ToString() + "|";
                }
                return _id;
            }
        }

        private GenericKeyCollection<Guid, Member> _members;
        /// <summary>
        /// This contains the members represented by this node
        /// </summary>
        [DataMember]
        public GenericKeyCollection<Guid, Member> Members
        {
            get { return _members; }
            set { _members = value; }
        }

        private Measure _measure;
        /// <summary>
        /// The unique identifier of the measure represented by this node
        /// </summary>
        [DataMember]
        public Measure Measure
        {
            get { return _measure; }
            set { _measure = value; }
        }

        private double _value;
        /// <summary>
        /// The value of this node
        /// </summary>
        [DataMember]
        public double Value
        {
            get { return _value; }
            set { _value = value; }
        }

        private bool _leafnode;
        /// <summary>
        /// This indicates that all members in this node are leaf node members, and therefore they need to be aggregated from an
        /// underlying data source, or potentially can be written back
        /// </summary>
        [DataMember]
        public bool LeafNode
        {
            get { return _leafnode; }
            set { _leafnode = value; }
        }

        /// <summary>
        /// This tests whether this node has the same members as the given node
        /// </summary>
        /// <param name="previousnode">The node to test against</param>
        /// <returns>True if all members are the same, false if not</returns>
        public bool SameMembers(Node previousnode)
        {
            if (_members.Where(membertest => !previousnode.Members.Contains(membertest.ID)).Count() > 0)
                return false;
            else
                return true;
        }

        /// <summary>
        /// This tests whether this node has the same members as the given node, except for the given
        /// pivoting member
        /// </summary>
        /// <param name="previousnode">The node to test against</param>
        /// <param name="pivotmember">The member to ignore</param>
        /// <returns>True if all members except the pivot member are the same, false if not</returns>
        public bool SameMembers(Node previousnode, Member pivotmember)
        {
            if (_members.Where(membertest => membertest.ID != pivotmember.ID && !previousnode.Members.Contains(membertest.ID)).Count() > 0)
                return false;
            else
                return true;
        }
    }
}
