/*
 * This source file (SQLQueryField.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Utilities;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// This is a SQL Query Field represents a single field used in a query
    /// </summary>
    [Serializable]
    [DataContract]
    public class SQLQueryField
    {
        /// <summary>
        /// Creates a new SQL Query field
        /// </summary>
        public SQLQueryField()
        {
            _tablename = String.Empty;
            _fieldname = String.Empty;
            _alias = String.Empty;
            _function = SQLFieldFunctionTypes.GROUP;
            _isuserfunction = false;
            _functionparameters = new List<object>();
            _userfunctionname = String.Empty;
        }

        /// <summary>
        /// Initialises a new instance of the SQL Query Field with the specified properties
        /// </summary>
        /// <param name="tablename">The name of the table the field belongs in</param>
        /// <param name="fieldname">The name of the field</param>
        /// <param name="alias">An alias to refer to the field</param>
        /// <param name="function">The function used on the field</param>
        public SQLQueryField(string tablename, string fieldname, string alias, SQLFieldFunctionTypes function)
        {
            _tablename = tablename;
            _fieldname = fieldname;
            _alias = alias;
            _function = function;
            _isuserfunction = false;
            _functionparameters = new List<object>();
            _userfunctionname = String.Empty;
        }

        /// <summary>
        /// Initialises a new instance of the SQL Query Field with the specified properties, which is based
        /// upon a user defined function in the database
        /// </summary>
        /// <param name="tablename">The name of the table the field belongs in</param>
        /// <param name="functionname">The name of the field</param>
        /// <param name="alias">An alias to refer to the field</param>
        /// <param name="function">The function used on the field</param>
        /// <param name="functionparameters">As this is a user defined function, these are the parameters
        /// to pass to the function</param>
        public SQLQueryField(string tablename, string alias, string functionname, SQLFieldFunctionTypes function,
            params object[] functionparameters)
        {
            _tablename = tablename;
            _fieldname = alias;
            _alias = alias;
            _function = function;
            _isuserfunction = true;
            _functionparameters = new List<object>();
            _userfunctionname = functionname;
            foreach (object thisparam in functionparameters)
                _functionparameters.Add(thisparam);
        }

        /// <summary>
        /// Returns the unique identifier (name) of the field (which is the name of the table and field)
        /// </summary>
        [DataMember]
        public string ID
        {
            get { return _tablename + "." + _fieldname; }
        }

        private string _tablename;
        /// <summary>
        /// The name of the table that this field belongs to
        /// </summary>
        [DataMember]
        public string Tablename
        {
            get { return _tablename; }
            set { _tablename = value; }
        }

        private string _fieldname;
        /// <summary>
        /// The name of the field
        /// </summary>
        [DataMember]
        public string FieldName
        {
            get { return _fieldname; }
            set { _fieldname = value; }
        }

        private string _alias;
        /// <summary>
        /// The alias (name) of the field returned in the query
        /// </summary>
        [DataMember]
        public string Alias
        {
            get { return _alias; }
            set { _alias = value; }
        }

        private SQLFieldFunctionTypes _function;
        /// <summary>
        /// The function used to return values for this query
        /// </summary>
        [DataMember]
        public SQLFieldFunctionTypes Function
        {
            get { return _function; }
            set { _function = value; }
        }

        private bool _isuserfunction;
        /// <summary>
        /// Whether this field is actually specified as a user defined function, rather than
        /// a standard field
        /// </summary>
        [DataMember]
        public bool IsUserFunction
        {
            get { return _isuserfunction; }
            set { _isuserfunction = value; }
        }

        private string _userfunctionname;
        /// <summary>
        /// If this field is based on a user defined function, this is the name of the function
        /// </summary>
        [DataMember]
        public string UserFunctionName
        {
            get { return _userfunctionname; }
            set { _userfunctionname = value; }
        }

        private List<object> _functionparameters;
        /// <summary>
        /// If this field is based on a user defined function, these are the parameters to pass to this
        /// function
        /// </summary>
        [DataMember]
        public List<object> FunctionParameters
        {
            get { return _functionparameters; }
            set { _functionparameters = value; }
        }
    }
}
