/*
 * This source file (SQLQueryGenerator.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Linq;
using Bamboo.Cubed.Utilities;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// This is a SQL Query Generator class which provides information required to generate a SQL statement against
    /// a database
    /// </summary>
    [Serializable]
    [DataContract]
    public class SQLQueryGenerator
    {
        /// <summary>
        /// Creates a new SQL Query Generator
        /// </summary>
        public SQLQueryGenerator()
        {
            _uniquevalues = false;
            _aggregate = false;
            _tables = new GenericKeyCollection<string, SQLQueryTable>();
            _fields = new GenericKeyCollection<string, SQLQueryField>();
            _filters = new GenericKeyCollection<string, SQLQueryFilter>();
            _sortfields = new GenericKeyCollection<string, SQLQueryField>();
        }

        /// <summary>
        /// Initialises a new instance of the SQL Query Generator with the given values
        /// </summary>
        /// <param name="uniquevalues">Whether this generator will only return unique values from fields</param>
        /// <param name="aggregate">Whether this generator will generate an aggregated SQL statement</param>
        /// <param name="tables">The tables used in this query</param>
        /// <param name="fields">The fields used in this query</param>
        /// <param name="filters">The filters used to limit the results</param>
        public SQLQueryGenerator(bool uniquevalues, bool aggregate, GenericKeyCollection<string, SQLQueryTable> tables,
            GenericKeyCollection<string, SQLQueryField> fields, GenericKeyCollection<string, SQLQueryFilter> filters)
        {
            _uniquevalues = uniquevalues;
            _aggregate = aggregate;
            _tables = tables;
            _fields = fields;
            _filters = filters;
            _sortfields = new GenericKeyCollection<string, SQLQueryField>();
        }

        /// <summary>
        /// Adds a primary table to the table collection
        /// </summary>
        /// <param name="tablename">The name of the table</param>
        /// <param name="primarykey">The primary key field of the table</param>
        public void AddPrimaryTable(string tablename, string primarykey)
        {
            SQLQueryTable newtable = new SQLQueryTable(tablename, primarykey);
            _tables.Add(newtable);
        }

        /// <summary>
        /// Adds a primary table to the table collection, where the table is actually generated from a
        /// table-valued function
        /// </summary>
        /// <param name="tablename">The name of the table</param>
        /// <param name="primarykey">The primary key field of the table</param>
        /// <param name="tablealias">The name to alias the function as in the select statement</param>
        /// <param name="functionparameters">The collection of parameters, on order, to pass to the 
        /// table valued function</param>
        public void AddPrimaryTableFromFunction(string tablename, string primarykey, string tablealias,
            params object[] functionparameters)
        {
            SQLQueryTable newtable = new SQLQueryTable(tablename, primarykey, String.Empty,
                String.Empty, SQLJoinTypes.Inner, tablealias, true, functionparameters);
            _tables.Add(newtable);
        }

        /// <summary>
        /// Adds a joined (subsequent) table to the table collection
        /// </summary>
        /// <param name="tablename">The table to add</param>
        /// <param name="primarykey">The primary key field of the table</param>
        /// <param name="linkingtable">The table used to link this one into the query</param>
        /// <param name="linkingtablefield">The field used to link to this table sprimary key</param>
        /// <param name="jointype">The type of join between the tables</param>
        public void AddJoinedTable(string tablename, string primarykey, string linkingtable,
            string linkingtablefield, SQLJoinTypes jointype)
        {
            SQLQueryTable newtable = new SQLQueryTable(tablename, primarykey, linkingtable,
                linkingtablefield, jointype);
            _tables.Add(newtable);
        }

        /// <summary>
        /// Adds a joined (subsequent) table to the table collection
        /// </summary>
        /// <param name="tablename">The table to add</param>
        /// <param name="primarykey">The primary key field of the table</param>
        /// <param name="linkingtable">The table used to link this one into the query</param>
        /// <param name="linkingtablefield">The field used to link to this table sprimary key</param>
        /// <param name="jointype">The type of join between the tables</param>
        /// <param name="tablealias">The alias to use for the table</param>
        public void AddJoinedTable(string tablename, string primarykey, string linkingtable,
            string linkingtablefield, SQLJoinTypes jointype, string tablealias)
        {
            SQLQueryTable newtable = new SQLQueryTable(tablename, primarykey, linkingtable,
                linkingtablefield, jointype, tablealias);
            _tables.Add(newtable);
        }

        /// <summary>
        /// Adds a joined (subsequent) table to the table collection, where the table is actually generated from a
        /// table-valued function
        /// </summary>
        /// <param name="tablename">The table to add</param>
        /// <param name="primarykey">The primary key field of the table</param>
        /// <param name="linkingtable">The table used to link this one into the query</param>
        /// <param name="linkingtablefield">The field used to link to this table sprimary key</param>
        /// <param name="jointype">The type of join between the tables</param>
        /// <param name="tablealias">The alias to use for the table</param>
        /// <param name="functionparameters">The collection of parameters, on order, to pass to the 
        /// table valued function</param>
        public void AddJoinedTableFromFunction(string tablename, string primarykey, string linkingtable,
            string linkingtablefield, SQLJoinTypes jointype, string tablealias,
            params object[] functionparameters)
        {
            SQLQueryTable newtable = new SQLQueryTable(tablename, primarykey, linkingtable,
                linkingtablefield, jointype, tablealias, true, functionparameters);
            _tables.Add(newtable);
        }

        /// <summary>
        /// Adds a new SQL Query field to the fields collection
        /// </summary>
        /// <param name="tablename">The name of the table the field belongs in</param>
        /// <param name="fieldname">The name of the field</param>
        /// <param name="alias">An alias to refer to the field</param>
        /// <param name="function">The function used on the field</param>
        public void AddField(string tablename, string fieldname, string alias, SQLFieldFunctionTypes function)
        {
            SQLQueryField newfield = new SQLQueryField(tablename, fieldname, alias,
                function);
            _fields.Add(newfield);
        }

        /// <summary>
        /// Adds a new SQL Query field to the fields collection which is based upon a user defined function
        /// in the database
        /// </summary>
        /// <param name="tablename">The name of the table the field belongs in</param>
        /// <param name="functionname">The name of the user defined function the field is based on</param>
        /// <param name="alias">An alias to refer to the field</param>
        /// <param name="function">The function used on the field</param>
        /// <param name="functionparameters">As this is a user defined function based field, these are
        /// the parameters to pass to the function</param>
        public void AddFieldFromFunction(string tablename, string functionname, string alias, SQLFieldFunctionTypes function,
            params object[] functionparameters)
        {
            SQLQueryField newfield = new SQLQueryField(tablename, alias, functionname,
                function, functionparameters);
            _fields.Add(newfield);
        }

        /// <summary>
        /// Adds a new SQL Query field to the sort collection. This field should be added separately to the
        /// standard fields collection, if not - an exception will be thrown
        /// </summary>
        /// <param name="tablename">The name of the table the field exists in</param>
        /// <param name="fieldname">The name of the field</param>
        public void AddSortField(string tablename, string fieldname)
        {
            // Check to ensure the field has already been added on the collection
            if (_fields.FirstOrDefault(thisfield => thisfield.Tablename.ToUpper() == tablename.ToUpper() &&
                thisfield.FieldName.ToUpper() == fieldname.ToUpper()) == null)
                throw new ApplicationException("No field by the name of " + fieldname + " exists in the collection from the table " + tablename);

            // Add the field to the sorted list
            _sortfields.Add(new SQLQueryField(tablename, fieldname, fieldname, SQLFieldFunctionTypes.GROUP));
        }

        /// <summary>
        /// Adds a new SQL Query LIST Filter to the collection
        /// </summary>
        /// <param name="tablename">The name of the table to filter</param>
        /// <param name="fieldname">The name of the field to filter on</param>
        /// <param name="acceptablevalues">The list of acceptable values for the filter</param>
        public void AddListFilter(string tablename, string fieldname, List<object> acceptablevalues)
        {
            SQLQueryFilter newfilter = new SQLQueryFilter(tablename, fieldname, acceptablevalues);
            _filters.Add(newfilter);
        }

        /// <summary>
        /// Adds a new SQL Query filter based on the results of a function
        /// </summary>
        /// <param name="tablename">The name of the table to filter</param>
        /// <param name="fieldname">The name of the field to filter on</param>
        /// <param name="functionname">The name of the function to generate the results to filter with</param>
        /// <param name="tablevalues">Whether this filter returns a table collection of values or a single
        /// scalar value</param>        /// 
        /// <param name="tablevaluefield">If this function is based on a table of values, what is the 
        /// field from the returned table which is used to filter</param>
        /// <param name="functionparameters">The collection of parameters, in the correct order, to
        /// pass to the function</param>
        public void AddFunctionFilter(string tablename, string fieldname, string functionname,
            bool tablevalues, string tablevaluefield, params object[] functionparameters)
        {
            SQLQueryFilter newfilter = new SQLQueryFilter(tablename, fieldname, functionname,
                tablevalues, tablevaluefield, functionparameters);
            _filters.Add(newfilter);
        }

        /// <summary>
        /// Adds a new SQL Query RANGE Filter to the collection
        /// </summary>
        /// <param name="tablename">The name of the table to filter</param>
        /// <param name="fieldname">The name of the field to filter on</param>
        /// <param name="lowervalue">The lower value to match</param>
        /// <param name="uppervalue">The upper value to match</param>
        public void AddRangeFilter(string tablename, string fieldname, object lowervalue, object uppervalue)
        {
            SQLQueryFilter newfilter = new SQLQueryFilter(tablename, fieldname, lowervalue, uppervalue);
            _filters.Add(newfilter);
        }

        private bool _uniquevalues;
        /// <summary>
        /// Whether to only return UNIQUE values from each row
        /// </summary>
        [DataMember]
        public bool UniqueValues
        {
            get { return _uniquevalues; }
            set { _uniquevalues = value; }
        }

        private bool _aggregate;
        /// <summary>
        /// Whether this query should aggregate returned values (using GROUP BY and the functions listed for the
        /// fields) or whether it is a straight select
        /// </summary>
        [DataMember]
        public bool Aggregate
        {
            get { return _aggregate; }
            set { _aggregate = value; }
        }

        private GenericKeyCollection<string, SQLQueryTable> _tables;
        /// <summary>
        /// The tables used in the query
        /// </summary>
        [DataMember]
        public GenericKeyCollection<string, SQLQueryTable> Tables
        {
            get { return _tables; }
            set { _tables = value; }
        }

        private GenericKeyCollection<string, SQLQueryField> _fields;
        /// <summary>
        /// The fields returned by the query
        /// </summary>
        [DataMember]
        public GenericKeyCollection<string, SQLQueryField> Fields
        {
            get { return _fields; }
            set { _fields = value; }
        }

        private GenericKeyCollection<string, SQLQueryFilter> _filters;
        /// <summary>
        /// The filters to restrict returned values
        /// </summary>
        [DataMember]
        public GenericKeyCollection<string, SQLQueryFilter> Filters
        {
            get { return _filters; }
            set { _filters = value; }
        }

        private GenericKeyCollection<string, SQLQueryField> _sortfields;
        /// <summary>
        /// The fields (in order) to sort the results by
        /// </summary>
        [DataMember]
        public GenericKeyCollection<string, SQLQueryField> SortFields
        {
            get { return _sortfields; }
            set { _sortfields = value; }
        }

        /// <summary>
        /// This orders the tables internally to ensure they are in the correct join order
        /// </summary>
        public void OrderTables()
        {
            // Set all table orders to "-1" initially
            foreach (SQLQueryTable thistable in _tables)
                thistable.TableOrder = -1;

            // Cycle through our table order from the first (initial) table onwards
            int currenttableorder = 0;
            while (currenttableorder < _tables.Count)
            {
                if (currenttableorder == 0)
                {
                    // Find the table with no "linking table"
                    SQLQueryTable primarytable = _tables.FirstOrDefault(testtable =>
                        testtable.LinkingTable == String.Empty);
                    if (primarytable != null)
                        primarytable.TableOrder = 0;
                    currenttableorder++;
                }
                else
                {
                    // Cycle through each table in the collection which hasn't already been ordered, and check
                    // if it can be added as the next in order
                    foreach (SQLQueryTable thistable in _tables.Where(testtable => testtable.TableOrder == -1))
                    {
                        // Check to see if this table's "linking table" has already been ordered, and if so - 
                        // use this as the next table
                        SQLQueryTable linkingtableadded = _tables.FirstOrDefault(testtable =>
                            testtable.TableOrder > -1 && testtable.Tablename.ToUpper() == thistable.LinkingTable.ToUpper());
                        if (linkingtableadded != null)
                        {
                            // This tables' "linking table" has already been ordered, so this table can be 
                            // ordered next
                            thistable.TableOrder = currenttableorder;
                            break;
                        }
                    }

                    currenttableorder++;
                }
            }
        }
    }
}
