/*
 * This source file (SQLQueryTable.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Utilities;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// This is a SQL Query Table represents a single table used by a SQL Query Filter
    /// </summary>
    [Serializable]
    [DataContract]
    public class SQLQueryTable
    {
        /// <summary>
        /// Creates a new SQL Query table
        /// </summary>
        public SQLQueryTable()
        {
            _tablename = String.Empty;
            _primarykey = String.Empty;
            _linkingtable = String.Empty;
            _linkingtablefield = String.Empty;
            _jointype = SQLJoinTypes.Inner;
            _tablealias = String.Empty;
            _tableorder = -1;
        }

        /// <summary>
        /// Creates a new instance of a primary table
        /// </summary>
        /// <param name="tablename">The table to create</param>
        /// <param name="primarykey">The primary key field of the table</param>
        public SQLQueryTable(string tablename, string primarykey)
        {
            _tablename = tablename;
            _primarykey = primarykey;
            _linkingtable = String.Empty;
            _linkingtablefield = String.Empty;
            _jointype = SQLJoinTypes.Inner;
            _tablealias = tablename;
            _isfunction = false;
            _tableorder = -1;
            _tablefunctionparameters = new List<object>();
        }

        /// <summary>
        /// Creates a new instance of a joined (subsequent) table
        /// </summary>
        /// <param name="tablename">The table to create</param>
        /// <param name="primarykey">The primary key field of the table</param>
        /// <param name="linkingtable">The table used to link this one into the query</param>
        /// <param name="linkingtablefield">The field used to link to this table sprimary key</param>
        /// <param name="jointype">The type of join between the tables</param>       
        public SQLQueryTable(string tablename, string primarykey, string linkingtable,
            string linkingtablefield, SQLJoinTypes jointype)
        {
            _tablename = tablename;
            _primarykey = primarykey;
            _linkingtable = linkingtable;
            _linkingtablefield = linkingtablefield;
            _jointype = jointype;
            _tablealias = tablename;
            _isfunction = false;
            _tableorder = -1;
            _tablefunctionparameters = new List<object>();
        }

        /// <summary>
        /// Creates a new instance of a joined (subsequent) table with an alias
        /// </summary>
        /// <param name="tablename">The table to create</param>
        /// <param name="primarykey">The primary key field of the table</param>
        /// <param name="linkingtable">The table used to link this one into the query</param>
        /// <param name="linkingtablefield">The field used to link to this table sprimary key</param>
        /// <param name="jointype">The type of join between the tables</param>        
        /// <param name="tablealias">The alias for the table</param>
        public SQLQueryTable(string tablename, string primarykey, string linkingtable,
            string linkingtablefield, SQLJoinTypes jointype, string tablealias)
        {
            _tablename = tablename;
            _primarykey = primarykey;
            _linkingtable = linkingtable;
            _linkingtablefield = linkingtablefield;
            _jointype = jointype;
            _tablealias = tablealias;
            _isfunction = false;
            _tableorder = -1;
            _tablefunctionparameters = new List<object>();
        }

        /// <summary>
        /// Creates a new instance of a joined (subsequent) table with an alias as well as specifying whether
        /// the table is actually generated by a function and the parameters to generate that function
        /// </summary>
        /// <param name="tablename">The table to create</param>
        /// <param name="primarykey">The primary key field of the table</param>
        /// <param name="linkingtable">The table used to link this one into the query</param>
        /// <param name="linkingtablefield">The field used to link to this table sprimary key</param>
        /// <param name="jointype">The type of join between the tables</param>        
        /// <param name="tablealias">The alias for the table</param>
        /// <param name="isfunction">Whether this table is actually generated by a function</param>
        /// <param name="functionparameters">If this table is generated by a function, this is the 
        /// collection of parameters, in the correct order, to pass to the function</param>
        public SQLQueryTable(string tablename, string primarykey, string linkingtable,
            string linkingtablefield, SQLJoinTypes jointype, string tablealias, bool isfunction,
            params object[] functionparameters)
        {
            _tablename = tablename;
            _primarykey = primarykey;
            _linkingtable = linkingtable;
            _linkingtablefield = linkingtablefield;
            _jointype = jointype;
            _tablealias = tablealias;
            _isfunction = isfunction;
            _tableorder = -1;
            _tablefunctionparameters = new List<object>();
            foreach (object thisparam in functionparameters)
                _tablefunctionparameters.Add(thisparam);
        }

        /// <summary>
        /// Returns the unique identifier (name) of the table
        /// </summary>
        public string ID
        {
            get 
            {
                // If there is a valid alias, return it - otherwise return the underlying table
                if (_tablealias != null && _tablealias != String.Empty)
                    return _tablealias;
                else
                    return _tablename; 
            }
        }

        private string _tablename;
        /// <summary>
        /// The name of the table
        /// </summary>
        [DataMember]
        public string Tablename
        {
            get { return _tablename; }
            set { _tablename = value; }
        }

        private string _primarykey;
        /// <summary>
        /// The primary key field for the table
        /// </summary>
        [DataMember]
        public string PrimaryKey
        {
            get { return _primarykey; }
            set { _primarykey = value; }
        }

        private string _linkingtable;
        /// <summary>
        /// The table which this table is linked (JOINED) to in the query (if any)
        /// </summary>
        [DataMember]
        public string LinkingTable
        {
            get { return _linkingtable; }
            set { _linkingtable = value; }
        }

        private string _linkingtablefield;
        /// <summary>
        /// The field in the linking (JOINED) table which relates to this tables' primary key (if any)
        /// </summary>
        [DataMember]
        public string LinkingTableField
        {
            get { return _linkingtablefield; }
            set { _linkingtablefield = value; }
        }

        private SQLJoinTypes _jointype;
        /// <summary>
        /// The type of JOIN between this table and its' linking table (if any)
        /// </summary>
        [DataMember]
        public SQLJoinTypes JoinType
        {
            get { return _jointype; }
            set { _jointype = value; }
        }

        private string _tablealias;
        /// <summary>
        /// The alias for this table
        /// </summary>
        [DataMember]
        public string TableAlias
        {
            get { return _tablealias; }
            set { _tablealias = value; }
        }

        private bool _isfunction;
        /// <summary>
        /// Whether this table is actually a table-valued function rather than a real table (or view)
        /// </summary>
        [DataMember]
        public bool IsFunction
        {
            get { return _isfunction; }
            set { _isfunction = value; }
        }

        private List<object> _tablefunctionparameters;
        /// <summary>
        /// If this table is generated from a function, this is the correctly ordered list of parameters
        /// to pass to the function
        /// </summary>
        [DataMember]
        public List<object> TableFunctionParameters
        {
            get { return _tablefunctionparameters; }
            set { _tablefunctionparameters = value; }
        }

        private int _tableorder;
        /// <summary>
        /// When a table is joined to this query, this is the order in which it should be joined
        /// </summary>
        [DataMember]
        public int TableOrder
        {
            get { return _tableorder; }
            set { _tableorder = value; }
        }
    }
}
