/*
 * This source file (User.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// The user contains information about a user currently connected to the system, or stored in a role
    /// </summary>
    [Serializable]
    [DataContract]
    public class User
    {
        /// <summary>
        /// Initialises a new default user
        /// </summary>
        public User()
        {
            _authenticatedat = DateTime.Now;
            _id = Guid.NewGuid();
            _domain = Environment.UserDomainName;
            _username = Environment.UserName;
            _isadmin = false;
            _fullname = Environment.UserName;
            _email = Environment.UserName + "@" + Environment.UserDomainName;
            _mobile = String.Empty;
            _groups = new List<string>();
            _privatesearchrights = AccessTypes.None;
            _publicsearchrights = AccessTypes.None;
            _readaccesscubes = new List<Guid>();
            _writeaccesscubes = new List<Guid>();
            _grantedreadaccessmembers = new List<Guid>();
            _grantedwriteaccessmembers = new List<Guid>();
            _deniedreadaccessmembers = new List<Guid>();
            _deniedwriteaccessmembers = new List<Guid>();
        }

        /// <summary>
        /// Initialises a new user with the given details
        /// </summary>
        /// <param name="domain">The domain the user is from</param>
        /// <param name="username">The user name the user logged in as</param>
        /// <param name="fullname">The full name of the user</param>
        /// <param name="email">The users email address</param>
        /// <param name="mobile">The users mobile number</param>
        /// <param name="groups">Any system groups the user belongs to</param>
        public User(string domain, string username, string fullname, string email, string mobile,
            string[] groups)
        {
            _authenticatedat = DateTime.Now;
            _id = Guid.NewGuid();
            _domain = domain;
            _username = username;
            _isadmin = false;
            _fullname = fullname;
            _email = email;
            _mobile = mobile;
            _groups = new List<string>(groups);
            _privatesearchrights = AccessTypes.None;
            _publicsearchrights = AccessTypes.None;
            _readaccesscubes = new List<Guid>();
            _writeaccesscubes = new List<Guid>();
            _grantedreadaccessmembers = new List<Guid>();
            _grantedwriteaccessmembers = new List<Guid>();
            _deniedreadaccessmembers = new List<Guid>();
            _deniedwriteaccessmembers = new List<Guid>();
        }

        private DateTime _authenticatedat;
        /// <summary>
        /// The date and time the user was authenticated to the system (user records are cached for
        /// 5 minutes)
        /// </summary>
        [DataMember]
        public DateTime AuthenticatedAt
        {
            get { return _authenticatedat; }
            set { _authenticatedat = value; }
        }

        private int _hashcode;
        /// <summary>
        /// The stored hash code of this user (used for re-retrieval of previously authenticated
        /// users)
        /// </summary>
        [DataMember]
        public int Hashcode
        {
            get 
            {
                if (_hashcode == null)
                    _hashcode = base.GetHashCode();
                return _hashcode; 
            }
            set { _hashcode = value; }
        }

        /// <summary>
        /// Returns the stored hash code of the user object (used for re-retrieval of previously
        /// authenticated users)
        /// </summary>
        /// <returns>The stored hash code (or the standard generated hash code if none has been stored)</returns>
        public override int GetHashCode()
        {
            return Hashcode;
        }

        private Guid _id;
        /// <summary>
        /// The unique identifier of the member
        /// </summary>
        [DataMember]
        public Guid ID
        {
            get { return _id; }
            set { _id = value; }
        }

        private string _domain;
        /// <summary>
        /// The authentication domain the user is from
        /// </summary>
        [DataMember]
        public string Domain
        {
            get { return _domain; }
            set { _domain = value; }
        }

        private string _username;
        /// <summary>
        /// The user name which was used for access
        /// </summary>
        [DataMember]
        public string UserName
        {
            get { return _username; }
            set { _username = value; }
        }

        private string _fullname;
        /// <summary>
        /// The full name of the user
        /// </summary>
        [DataMember]
        public string FullName
        {
            get { return _fullname; }
            set { _fullname = value; }
        }

        private bool _isadmin;
        /// <summary>
        /// Whether this user is a full administrator (root user) of the system
        /// </summary>
        [DataMember]
        public bool IsAdmin
        {
            get { return _isadmin; }
            set { _isadmin = value; }
        }

        private string _email;
        /// <summary>
        /// The email address of the user
        /// </summary>
        [DataMember]
        public string Email
        {
            get { return _email; }
            set { _email = value; }
        }

        private string _mobile;
        /// <summary>
        /// The mobile number of the user
        /// </summary>
        [DataMember]
        public string MobileNo
        {
            get { return _mobile; }
            set { _mobile = value; }
        }

        private List<string> _groups;
        /// <summary>
        /// The names of any groups the user is associated with at the system level, which may give the user
        /// higher access than explicitly defined
        /// </summary>
        [DataMember]
        public List<string> Groups
        {
            get { return _groups; }
            set { _groups = value; }
        }

        private AccessTypes _privatesearchrights;
        /// <summary>
        /// This determines the level of access a user has on PRIVATE searches, which are only visible
        /// by themselves. This is set automatically depending on the users ROLE access when authenticated
        /// </summary>
        [DataMember]
        public AccessTypes PrivateSearchRights
        {
            get { return _privatesearchrights; }
            set { _privatesearchrights = value; }
        }

        private AccessTypes _publicsearchrights;
        /// <summary>
        /// This determines the level of access a user has on PUBLIC searches, which are visible by all 
        /// users of the Cubed Desktop. This is set automatically depending on the users ROLE access when
        /// authenticated
        /// </summary>
        [DataMember]
        public AccessTypes PublicSearchRights
        {
            get { return _publicsearchrights; }
            set { _publicsearchrights = value; }
        }

        private List<Guid> _readaccesscubes;
        /// <summary>
        /// The list of identifiers of Cubes this user has READ access to
        /// </summary>
        [DataMember]
        public List<Guid> ReadAccessCubes
        {
            get { return _readaccesscubes; }
            set { _readaccesscubes = value; }
        }

        private List<Guid> _writeaccesscubes;
        /// <summary>
        /// The list of identifiers of Cubes this user has WRITE access to
        /// </summary>
        [DataMember]
        public List<Guid> WriteAccessCubes
        {
            get { return _writeaccesscubes; }
            set { _writeaccesscubes = value; }
        }

        private List<Guid> _grantedreadaccessmembers;
        /// <summary>
        /// The list of identifiers of Members this user has been granted READ access to
        /// </summary>
        [DataMember]
        public List<Guid> GrantedReadAccessMembers
        {
            get { return _grantedreadaccessmembers; }
            set { _grantedreadaccessmembers = value; }
        }

        private List<Guid> _grantedwriteaccessmembers;
        /// <summary>
        /// The list of identifiers of Members this user has been granted WRITE access to
        /// </summary>
        [DataMember]
        public List<Guid> GrantedWriteAccessMembers
        {
            get { return _grantedwriteaccessmembers; }
            set { _grantedwriteaccessmembers = value; }
        }

        private List<Guid> _deniedreadaccessmembers;
        /// <summary>
        /// The list of identifiers of Members this user has been denied READ access to
        /// </summary>
        [DataMember]
        public List<Guid> DeniedReadAccessMembers
        {
            get { return _deniedreadaccessmembers; }
            set { _deniedreadaccessmembers = value; }
        }

        private List<Guid> _deniedwriteaccessmembers;
        /// <summary>
        /// The list of identifiers of Members this user has been denied WRITE access to
        /// </summary>
        [DataMember]
        public List<Guid> DeniedWriteAccessMembers
        {
            get { return _deniedwriteaccessmembers; }
            set { _deniedwriteaccessmembers = value; }
        }

        /// <summary>
        /// This returns the full user name in the form DOMAIN\Username
        /// </summary>
        public string DomainUserName
        {
            get { return _domain + "\\" + _username; }
        }	
    }
}
