/*
 * This source file (ValueFilter.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// Value Filters allow for the definition of a collection of acceptable values, or an upper and lower
    /// limit value, which will be used to filter the returned data set for a measure or member
    /// </summary>
    [Serializable]
    [DataContract]
    public class ValueFilter
    {
        /// <summary>
        /// Creates a blank new Value Filter
        /// </summary>
        public ValueFilter()
        {
            _id = Guid.NewGuid();
            _fieldname = String.Empty;
            _filtertype = FilterTypes.Range;
            _acceptablevalues = new List<string>();
            _lowervalue = String.Empty;
            _uppervalue = String.Empty;
        }

        /// <summary>
        /// This creates a new filter based on a range of acceptable values
        /// </summary>
        /// <param name="fieldname">The name of the field to look at and filter the values on</param>
        /// <param name="acceptablevalues">The collection of acceptable values</param>
        public ValueFilter(string fieldname, List<string> acceptablevalues)
        {
            _id = Guid.NewGuid();
            _fieldname = fieldname;
            _filtertype = FilterTypes.List;
            _acceptablevalues = acceptablevalues;
            _lowervalue = String.Empty;
            _uppervalue = String.Empty;
        }

        /// <summary>
        /// This creates a new filter based on an upper and lower value
        /// </summary>
        /// <param name="fieldname">The name of the field to filter</param>
        /// <param name="lowervalue">The lower acceptable value</param>
        /// <param name="uppervalue">The upper acceptable value</param>
        public ValueFilter(string fieldname, string lowervalue, string uppervalue)
        {
            _id = Guid.NewGuid();
            _fieldname = fieldname;
            _filtertype = FilterTypes.Range;
            _acceptablevalues = new List<string>();
            _lowervalue = lowervalue;
            _uppervalue = uppervalue;
        }

        private Guid _id;
        /// <summary>
        /// The unique identifier of this value filter
        /// </summary>
        [DataMember]
        public Guid ID
        {
            get { return _id; }
            set { _id = value; }
        }

        private string _fieldname;
        /// <summary>
        /// The name of the field in the data set which is the target of this value filter
        /// </summary>
        [DataMember]
        public string FieldName
        {
            get { return _fieldname; }
            set { _fieldname = value; }
        }

        private FilterTypes _filtertype;
        /// <summary>
        /// The type of filter being applied to the acceptable values
        /// </summary>
        [DataMember]
        public FilterTypes FilterType
        {
            get { return _filtertype; }
            set { _filtertype = value; }
        }        

        private List<string> _acceptablevalues;
        /// <summary>
        /// The list of values which are acceptable for this range, if this is a list filter
        /// </summary>
        [DataMember]
        public List<string> AcceptableValues
        {
            get { return _acceptablevalues; }
            set { _acceptablevalues = value; }
        }

        private string _lowervalue;
        /// <summary>
        /// The value which is acceptable as the lower limit, if this is a definitive range filter rather
        /// than a list filter
        /// </summary>
        [DataMember]
        public string LowerValue
        {
            get { return _lowervalue; }
            set { _lowervalue = value; }
        }

        private string _uppervalue;
        /// <summary>
        /// The value which is acceptable as the upper limit, if this is a definitive range filter rather
        /// than a list filter
        /// </summary>
        [DataMember]
        public string UpperValue
        {
            get { return _uppervalue; }
            set { _uppervalue = value; }
        }
    }    
}
