﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Threading;
using Bamboo.Cubed.UI.WebControls.TrafficLightsGaugeObjects;

namespace Bamboo.Cubed.UI.WebControls
{
    public class TrafficLightsGauge : BaseGauge
    {
        private const String IMAGES_DIRECTORY = "traffic_lights";
        private const String GREEN_LIGHT = "green.png";
        private const String YELLOW_LIGHT = "yellow.png";
        private const String RED_LIGHT = "red.png";


        public TrafficLightsGauge() : base()
        {

        }

        #region Properties

        private Double _lowMaxValue = 0;
        /// <summary>
        /// Gets or sets the maximum lowest value. Every value equal or lower than this one, will be green.
        /// </summary>
        public Double LowMaxValue
        {
            get { return _lowMaxValue; }
            set
            {
                if (_lowMaxValue != value)
                {
                    _lowMaxValue = value;
                }
            }
        }

        private Double _hiMinValue = 0;
        /// <summary>
        /// Gets or sets the minimum high value. Every value equal or higher than this one, will be red.
        /// </summary>
        public Double HiMinValue
        {
            get { return _hiMinValue; }
            set
            {
                if (_hiMinValue != value)
                {
                    _hiMinValue = value;
                }
            }
        }

        private Double _value = 0;
        /// <summary>
        /// Gets or sets the value to represent on the control.
        /// </summary>
        public Double Value
        {
            get { return _value; }
            set
            {
                if (_value != value)
                {
                    _value = value;
                }
            }
        }

        private TrafficLightsGaugeDisplayStyle _displayStyle = TrafficLightsGaugeDisplayStyle.GreenIsHighest;
        /// <summary>
        /// Gets or sets the Display Style of the Gauge.
        /// </summary>
        public TrafficLightsGaugeDisplayStyle DisplayStyle
        {
            get { return _displayStyle;}
            set
            {
                if (_displayStyle !=value)
                {
                    _displayStyle = value;
                }
            }
        }

        private Color _fontColor = Color.Gray;
        /// <summary>
        /// Gets or sents the color of the displayed font.
        /// </summary>
        public Color FontColor
        {
            get { return _fontColor; }
            set
            {
                if (_fontColor != value)
                {
                    _fontColor = value;
                }
            }
        }

        #endregion Properties

        #region Overriden Methods

        protected override void RenderContents(HtmlTextWriter output)
        {
            //Get the image to display.
            String displayImage = GetDisplayImage();

            //Get the bitmap of the image.
            Bitmap bmpImage = (Bitmap)Bitmap.FromStream(GetImageStream(displayImage));

            RenderHtml(bmpImage, output);
        }

        #endregion Overriden Methods



        #region Private Methods

        /// <summary>
        /// Gets the name of the image file to display.
        /// </summary>
        /// <returns>Image filename</returns>
        private String GetDisplayImage()
        {

            String displayImage = "";
            //We get the image to display, according to the current value and the
            //limit values.

            if (_value <= _hiMinValue)
            {
                if (_displayStyle == TrafficLightsGaugeDisplayStyle.GreenIsHighest)
                {
                    displayImage = RED_LIGHT;
                }
                else
                {
                    displayImage = GREEN_LIGHT;
                }
            }
            else if (_value >= _lowMaxValue)
            {
                if (_displayStyle == TrafficLightsGaugeDisplayStyle.GreenIsHighest)
                {
                    displayImage = GREEN_LIGHT;
                }
                else
                {
                    displayImage = RED_LIGHT;
                }
            }
            else
            {
                displayImage = YELLOW_LIGHT;
            }

            return displayImage;
        }

        /// <summary>
        /// Gets the Image Stream from the resources.
        /// </summary>
        /// <param name="imageName">Image Name</param>
        /// <returns>The Image Stream.</returns>
        private Stream GetImageStream(String imageName)
        {
            UnmanagedMemoryStream resourcestream = (UnmanagedMemoryStream)Assembly.GetExecutingAssembly().GetManifestResourceStream(
                    "Bamboo.Cubed.UI.WebControls.traffic_lights." + imageName);
            return resourcestream;
        }

        /// <summary>
        /// Renders everything to the Html.
        /// </summary>
        /// <param name="controlBitmap">The control image to create and display.</param>
        /// <param name="output">The HtmlTextWriter used to write the HTML output.</param>
        private void RenderHtml(Bitmap controlBitmap, HtmlTextWriter output)
        {
            string urldirectory = Assembly.GetExecutingAssembly().CodeBase;
            string realdirectory = urldirectory.Replace('/', '\\');
            realdirectory = realdirectory.Substring(8, realdirectory.Length - 8);
            realdirectory = realdirectory.Substring(0, realdirectory.LastIndexOf('\\') + 1);
            realdirectory = realdirectory.TrimEnd("bin\\".ToCharArray());
            realdirectory += "\\" + IMAGES_DIRECTORY + "\\";

            // Then check whether the flash subdirectory exists
            Guid imagename = Guid.NewGuid();
            if (!Directory.Exists(realdirectory))
                Directory.CreateDirectory(realdirectory);
            string file = realdirectory + imagename.ToString() + ".jpg";
            string imgFile = "./" + IMAGES_DIRECTORY + "/" + imagename.ToString() + ".jpg";
            controlBitmap.Save(file, ImageFormat.Jpeg);
            System.Web.UI.WebControls.Image imgControl = new System.Web.UI.WebControls.Image();
            String imgHtml = "<div id='" + imagename.ToString() + "Div'><center><img src='" + imgFile + "' ><br />";
            imgHtml += _value.ToString() + "</center></div>";
            output.Write(imgHtml);
            Thread cleanupthread = new Thread(new ThreadStart(CleanFiles));
            cleanupthread.Start();
        }

        /// <summary>
        /// Cleans all the old files in the directory.
        /// </summary>
        private void CleanFiles()
        {
            // Grab our real directory for storing of gauge images
            string urldirectory = Assembly.GetExecutingAssembly().CodeBase;
            string realdirectory = urldirectory.Replace('/', '\\');
            realdirectory = realdirectory.Substring(8, realdirectory.Length - 8);
            realdirectory = realdirectory.Substring(0, realdirectory.LastIndexOf('\\') + 1);
            realdirectory = realdirectory.TrimEnd("bin\\".ToCharArray());
            realdirectory += "\\" + IMAGES_DIRECTORY + "\\";

            // Find files which are older than 10 minutes
            try
            {
                DirectoryInfo dirinfo = new DirectoryInfo(realdirectory);
                List<string> deletefiles = new List<string>();
                foreach (FileInfo thisfile in dirinfo.GetFiles().Where(testfile => testfile.CreationTime.AddMinutes(10) < DateTime.Now))
                    deletefiles.Add(thisfile.FullName);
                foreach (string thisfile in deletefiles)
                    File.Delete(thisfile);
            }
            catch
            {
            }
        }
        #endregion Private Methods
    }
}
