/*
 * This source file (LoggingManager.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Bamboo.Cubed.Structures;
using System.Diagnostics;

namespace Bamboo.Cubed.Utilities.Logging
{
    /// <summary>
    /// This class provides access to system logs for capturing and recording events and errors
    /// </summary>
    public static class LoggingManager
    {
        /// <summary>
        /// This writes the given header and message to the event log, depending on the settings specified
        /// and the logging type
        /// </summary>
        /// <param name="header">The header of the message to write</param>
        /// <param name="message">The message to write</param>
        /// <param name="logtype">The type of log entry</param>
        public static void Write(string header, string message, LoggingTypes logtype)
        {
            try
            {
                switch (logtype)
                {
                    case LoggingTypes.Event:
                        if (Information.LoggingLevel == LoggingLevels.High)
                        {
                            // Write the event entry
                            EventLog.WriteEntry(Information.ActiveComponent + " v" + ReleaseInformation.Version,
                                header + Environment.NewLine + message, EventLogEntryType.Warning);
                        }
                        break;
                    case LoggingTypes.MinorError:
                        if (Information.LoggingLevel == LoggingLevels.High || Information.LoggingLevel == LoggingLevels.Medium)
                        {
                            // Write the event entry
                            EventLog.WriteEntry(Information.ActiveComponent + " v" + ReleaseInformation.Version,
                                header + Environment.NewLine + message, EventLogEntryType.Error);
                        }
                        break;
                    case LoggingTypes.MajorError:
                        if (Information.LoggingLevel != LoggingLevels.Disabled)
                        {
                            // Write the event entry
                            EventLog.WriteEntry(Information.ActiveComponent + " v" + ReleaseInformation.Version,
                                header + Environment.NewLine + message, EventLogEntryType.Error);
                        }
                        break;
                }
            }
            catch 
            {
                // An error occurred attempting to write the log
            }
        }

        /// <summary>
        /// This writes the given exception to the event log, depending on the settings specified
        /// and the logging type
        /// </summary>
        /// <param name="ex">The exception to log</param>
        /// <param name="logtype">The type of log entry</param>
        public static void Write(Exception ex, LoggingTypes logtype)
        {
            try
            {
                switch (logtype)
                {
                    case LoggingTypes.Event:
                        if (Information.LoggingLevel == LoggingLevels.High)
                        {
                            // Write the event entry
                            EventLog.WriteEntry(Information.ActiveComponent + " v" + ReleaseInformation.Version,
                                ex.Message + Environment.NewLine + ex.ToString(), EventLogEntryType.Warning);
                        }
                        break;
                    case LoggingTypes.MinorError:
                        if (Information.LoggingLevel == LoggingLevels.High || Information.LoggingLevel == LoggingLevels.Medium)
                        {
                            // Write the event entry
                            EventLog.WriteEntry(Information.ActiveComponent + " v" + ReleaseInformation.Version,
                                ex.Message + Environment.NewLine + ex.ToString(), EventLogEntryType.Error);
                        }
                        break;
                    case LoggingTypes.MajorError:
                        if (Information.LoggingLevel != LoggingLevels.Disabled)
                        {
                            // Write the event entry
                            EventLog.WriteEntry(Information.ActiveComponent + " v" + ReleaseInformation.Version,
                                ex.Message + Environment.NewLine + ex.ToString(), EventLogEntryType.Error);
                        }
                        break;
                }
            }
            catch
            {
                // An error occurred attempting to write the log
            }
        }        
    }
}
