/*
 * This source file (CompressionManager.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.IO.Compression;

namespace Bamboo.Cubed.Utilities
{
    /// <summary>
    /// The Compression Manager provides ease of access to internal Gzip based compression algorithms
    /// available through .NET
    /// </summary>
    public static class CompressionManager
    {
        /// <summary>
        /// This returns a compressed version of the given byte array
        /// </summary>
        /// <param name="uncompressed">An uncompressed array of bytes ready for compression</param>
        /// <returns>The compressed version of the array of bytes</returns>
        public static byte[] Compress(byte[] uncompressed)
        {
            MemoryStream compressedstream = new MemoryStream();            
            DeflateStream ds = new DeflateStream(compressedstream, CompressionMode.Compress);           
            ds.Write(uncompressed, 0, uncompressed.Length);
            ds.Close();
            compressedstream.Close();
            return compressedstream.GetBuffer();
        }

        /// <summary>
        /// Compresses the given string and returns its compressed form
        /// </summary>
        /// <param name="uncompressed">A string to compress</param>
        /// <returns>The compressed version of the string</returns>
        public static string Compress(string uncompressed)
        {
            return UnicodeEncoding.Unicode.GetString(Compress(UnicodeEncoding.Unicode.GetBytes(uncompressed)));
        }

        /// <summary>
        /// This returns a decompressed version of the given byte array
        /// </summary>
        /// <param name="compressed">A previously compressed array of bytes ready for decompression</param>
        /// <returns>The decompressed version of the array of bytes (the original form)</returns>
        public static byte[] Decompress(byte[] compressed)
        {
            MemoryStream compressedstream = new MemoryStream(compressed);
            DeflateStream ds = new DeflateStream(compressedstream, CompressionMode.Decompress);
            List<byte> uncompressedbytes = new List<byte>();
            int readbyte = ds.ReadByte();
            while (readbyte > -1)
            {
                uncompressedbytes.Add(Convert.ToByte(readbyte));
                readbyte = ds.ReadByte();
            }
            ds.Close();
            compressedstream.Close();
            return uncompressedbytes.ToArray();
        }

        /// <summary>
        /// This returns the original uncompressed form of the given string
        /// </summary>
        /// <param name="compressed">A string which was previously compressed</param>
        /// <returns>The original uncompressed form of the string</returns>
        public static string Decompress(string compressed)
        {
            return UnicodeEncoding.Unicode.GetString(Decompress(UnicodeEncoding.Unicode.GetBytes(compressed)));
        }
    }
}