/*
 * This source file (EncryptionManager.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Security.Cryptography;

namespace Bamboo.Cubed.Utilities
{
    /// <summary>
    /// Contains Encryption and Decryption routines used by Cubed
    /// </summary>
    public static class EncryptionManager
    {
        private const string encryptkeyword = "%:1ap&f%)7p:!@'";

        /// <summary>
        /// Returns a legal key bsaed on the string based key passed in
        /// </summary>
        /// <param name="Key">The key to legalise</param>
        /// <returns>A legal key for encryption</returns>
        private static byte[] GetLegalKey(string Key)
        {
            // First check to ensure the passed in key is 15 characters (the legal limit), and adjust if it isn't
            string correctkey = String.Empty;
            if (Key.Length > 15)
                correctkey = Key.Substring(0, 15);
            else if (Key.Length < 15)
            {
                correctkey = Key;
                while (correctkey.Length < 15)
                    correctkey += "0";
            }

            // Now use the correct length key to create the byte array encrypted key
            SymmetricAlgorithm RjiCryptService = new RijndaelManaged();
            string sTemp;
            if (RjiCryptService.LegalKeySizes.Length > 0)
            {
                int lessSize = 0, moreSize = RjiCryptService.LegalKeySizes[0].MinSize;
                while (correctkey.Length * 8 > moreSize)
                {
                    lessSize = moreSize;
                    moreSize += RjiCryptService.LegalKeySizes[0].SkipSize;
                }
                sTemp = correctkey.PadRight(moreSize / 8, ' ');
            }
            else
                sTemp = correctkey;

            return Encoding.UTF8.GetBytes(sTemp);
        }

        /// <summary>
        /// Returns an encrypted form of the passed in string, based on the supplied keyword
        /// </summary>
        /// <param name="Source">The string to encrypt</param>
        /// <param name="KeyWord">The keyword to use to encrypt the string</param>
        /// <returns>An encrypted string</returns>
        public static string EncryptString(string Source, string KeyWord)
        {
            // Check whether anything has been passed through, and if nothing - return nothing
            if (Source == null || Source == String.Empty)
                return String.Empty;

            // Something has been passed through, so let's encrypt it
            SymmetricAlgorithm RjiCryptService = new RijndaelManaged();
            byte[] bytKey = GetLegalKey(KeyWord);
            RjiCryptService.Key = bytKey;
            RjiCryptService.IV = bytKey;

            System.IO.MemoryStream ms = new System.IO.MemoryStream();
            CryptoStream cs = new CryptoStream(ms, RjiCryptService.CreateEncryptor(), CryptoStreamMode.Write);
            Byte[] plaintextbyte = UnicodeEncoding.Unicode.GetBytes(Source);
            cs.Write(plaintextbyte, 0, plaintextbyte.Length);
            cs.Close();
            Byte[] ciphertextbyte = ms.ToArray();
            ms.Close();
            return Convert.ToBase64String(ciphertextbyte);
        }

        /// <summary>
        /// Encrypts a string based on the default keyword
        /// </summary>
        /// <param name="Source">The string to encrypt</param>
        /// <returns>The encrypted string</returns>
        public static string EncryptString(string Source)
        {
            return EncryptString(Source, encryptkeyword);
        }

        /// <summary>
        /// Decrypts a string based on the supplied keyword
        /// </summary>
        /// <param name="Source">The encrypted string to decrypt</param>
        /// <param name="KeyWord">The keyword to use in decryption</param>
        /// <returns>A decrypted string</returns>
        public static string DecryptString(string Source, string KeyWord)
        {
            // Check whether anything has been passed through, and if nothing, return nothing
            if (Source == null || Source == String.Empty)
                return String.Empty;

            // Something has been passed through, so let's decrypt it
            try
            {
                System.IO.MemoryStream ms = new System.IO.MemoryStream(Convert.FromBase64String(Source));

                SymmetricAlgorithm RjiCryptService = new RijndaelManaged();
                byte[] bytKey = GetLegalKey(KeyWord);
                RjiCryptService.Key = bytKey;
                RjiCryptService.IV = bytKey;

                CryptoStream cs = new CryptoStream(ms, RjiCryptService.CreateDecryptor(), CryptoStreamMode.Read);
                System.IO.MemoryStream ds = new System.IO.MemoryStream();
                do
                {
                    Byte[] buf = new Byte[100];
                    int actualbytesread = cs.Read(buf, 0, 100);
                    if (0 == actualbytesread)
                        break;
                    ds.Write(buf, 0, actualbytesread);
                } while (true);

                cs.Close();
                ms.Close();
                Byte[] plaintextbyte = ds.ToArray();
                ds.Close();
                return UnicodeEncoding.Unicode.GetString(plaintextbyte);
            }
            catch
            {
                return String.Empty;
            }
        }

        /// <summary>
        /// Decrypts the supplied string based on the default keyword
        /// </summary>
        /// <param name="Source">The encrypted string to decrypt</param>
        /// <returns>The decrypted string</returns>
        public static string DecryptString(string Source)
        {
            return DecryptString(Source, encryptkeyword);
        }
    }
}