/*
 * This source file (Information.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.Security.Cryptography;
using System.IO;

namespace Bamboo.Cubed.Utilities
{
    /// <summary>
    /// Contains information routines used by Cubed
    /// </summary>
    public static class Information
    {
        /// <summary>
        /// This returns the current executing directory of the application
        /// </summary>
        public static string MyDirectory
        {
            get
            {
                string urldirectory = Assembly.GetExecutingAssembly().CodeBase;
                string realdirectory = urldirectory.Replace('/', '\\');
                realdirectory = realdirectory.Substring(8, realdirectory.Length - 8);
                realdirectory = realdirectory.Substring(0, realdirectory.LastIndexOf('\\') + 1);
                return realdirectory;
            }
        }

        private static string _activecomponent;
        /// <summary>
        /// This returns the currently active component of Cubed
        /// </summary>
        public static string ActiveComponent
        {
            get
            {
                if (_activecomponent == null || _activecomponent == String.Empty)
                    _activecomponent = "Cubed";
                return _activecomponent;
            }
            set { _activecomponent = value; }
        }

        private static LoggingLevels _logginglevel = LoggingLevels.Disabled;
        /// <summary>
        /// Returns the currently active logging level
        /// </summary>
        public static LoggingLevels LoggingLevel
        {
            get
            {
                return _logginglevel;
            }
            set { _logginglevel = value; }
        }

        /// <summary>
        /// This returns the full release and version details
        /// </summary>
        public static string Version
        {
            get 
            {
                return Release + " " + TrueVersion;
            } 
        }

        /// <summary>
        /// This returns the current true version of the application
        /// </summary>
        public static string TrueVersion
        {
            get { return Assembly.GetExecutingAssembly().GetName().Version.ToString(); }
        }

        /// <summary>
        /// This returns the full text of the Release of the application
        /// </summary>
        public static string Release
        {
            get
            {
                // First we grab the name of the release
                string release = ReleaseInformation.Name;

                // Then we add the type of the release                
                switch (ReleaseInformation.ReleaseType)
                {
                    case ReleaseTypes.Prototype:
                        release += " Prototype";
                        break;
                    case ReleaseTypes.PreRelease:
                        release += " Pre Release";
                        break;
                    case ReleaseTypes.Release:
                        release += " Release";
                        break;
                    case ReleaseTypes.ServiceRelease:
                        release += " Service Release";
                        break;
                }

                // Finally we add the number of the release
                release += " " + ReleaseInformation.ReleaseNumber.ToString();
                return release;
            }
        }

        /// <summary>
        /// This gets the header of the release to use in forms to the user
        /// </summary>
        public static string ReleaseHeader
        {
            get { return ReleaseInformation.Name; } 
        }

        /// <summary>
        ///  This gets the details of the release
        /// </summary>
        public static string ReleaseDetails
        {
            get
            {
                // Grab an empty string to begin
                string release = String.Empty;

                // Then we add the type of the release                
                switch (ReleaseInformation.ReleaseType)
                {
                    case ReleaseTypes.Prototype:
                        release += "Prototype";
                        break;
                    case ReleaseTypes.PreRelease:
                        release += "Pre Release";
                        break;
                    case ReleaseTypes.Release:
                        release += "Release";
                        break;
                    case ReleaseTypes.ServiceRelease:
                        release += "Service Release";
                        break;
                }

                // Finally we add the number of the release
                release += " " + ReleaseInformation.ReleaseNumber.ToString();
                return release;
            }
        }

        /// <summary>
        /// Returns the full Cubed Copyright String
        /// </summary>
        public const string FullCopyright = "Copyright  2006-2009 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved";

        /// <summary>
        /// Returns just the header portion of the copyright string
        /// </summary>
        public static string CopyrightHeader
        {
            get
            {
                return "Copyright  2006-2009 Brendan Hayes, Bamboo Software and contributors";
            }
        }
        
        /// <summary>
        /// Returns just the footer portion of the copyright string
        /// </summary>
        public static string CopyrightFooter
        {
            get
            {
                return "All Rights Reserved";
            }
        }
    }
}