/*
 * This source file (IntelligentConverter.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.Security.Cryptography;
using System.IO;

namespace Bamboo.Cubed.Utilities
{
    /// <summary>
    /// This class provides intelligent, reflection-based conversion between compatible objects
    /// </summary>
    public static class IntelligentConverter
    {
        /// <summary>
        /// This converts the FROM object to the TO object by discovering any compatible values between the
        /// two, and copying them across
        /// </summary>
        /// <param name="from">The object to copy from</param>
        /// <param name="to">The object to copy to</param>
        /// <returns>True if the conversion was successful, false if there was a problem</returns>
        public static bool TryConvert(object from, object to)
        {
            try
            {
                // Set a "match-counter". If this is at least one, we will assume the conversion to be 
                // successful
                int matchcounter = 0;

                // First gather the property lists from both objects
                Type fromtype = from.GetType();
                Type totype = to.GetType();
                PropertyInfo[] fromproperties = fromtype.GetProperties();
                PropertyInfo[] toproperties = totype.GetProperties();

                // Now loop through the properties of the FROM object
                foreach (PropertyInfo thisfromproperty in fromproperties)
                {
                    // Now loop through the properties of the TO object to see if it exists
                    foreach (PropertyInfo thistoproperty in toproperties)
                        if (thisfromproperty.Name == thistoproperty.Name)
                        {
                            // We have found a match between the two properties, so perform a copy
                            // on the original objects and increase our match counter
                            matchcounter++;
                            thistoproperty.SetValue(to, thisfromproperty.GetValue(from, null), null);
                        }
                }

                // Return true so long as we've found at least one match
                if (matchcounter > 0)
                    return true;
                else
                    return false;
            }
            catch
            {
                // Return false as an error occured
                return false;
            }
        }
    }
}