/*
 * This source file (XMLManager.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.IO;

namespace Bamboo.Cubed.Utilities
{
    /// <summary>
    /// This is a generic typed XML Manager which is capable of serialising/deserialsing to and from XML based
    /// on a specific type given at execution time
    /// </summary>
    /// <typeparam name="type">The type to serialise</typeparam>
    public static class XMLManager<type>
    {
        #region Serialization

        /// <summary>
        /// This serialises the passed in object to an encoded XML string representation of it
        /// </summary>
        /// <param name="originalobject">The original object which is to be serialised</param>
        /// <returns>An encoded XML string containing the object</returns>
        public static string SerialiseXML(type originalobject)
        {
            if (originalobject != null)
            {
                try
                {
                    // Use regular XML serialisation
                    XmlSerializer s = new XmlSerializer(typeof(type));
                    StringBuilder sb = new StringBuilder();
                    StringWriter sw = new StringWriter(sb);
                    s.Serialize(sw, originalobject);
                    sw.Close();
                    return sw.ToString();
                }
                catch (Exception e)
                {
                    // The serialisation failed so return an exception
                    throw new InvalidXMLException("The XML Serialisation failed." + e.ToString(), e, String.Empty);
                }
            }
            else
            {
                // The passed in object was null so throw an exception
                throw new InvalidXMLException("The object to serialise is null", null, String.Empty);
            }
        }

        /// <summary>
        /// This serialises and object to a file on the disk
        /// </summary>
        /// <param name="filename">The name of the file to save to</param>
        /// <param name="originalobject">The original object to serialise</param>
        public static void SerialiseToFile(string filename, type originalobject)
        {
            if (originalobject != null)
            {
                try
                {
                    // Use regular XML serialisation
                    FileAccess.DeleteFile(filename);
                    XmlSerializer s = new XmlSerializer(typeof(type));
                    TextWriter t = new StreamWriter(filename);
                    s.Serialize(t, originalobject);
                    t.Close();
                }
                catch (Exception e)
                {
                    // The serialisation failed so return an exception
                    throw new InvalidXMLException("The XML Serialisation failed." + e.ToString(), e, String.Empty);
                }
            }
            else
            {
                // The passed in object was null so throw an exception
                throw new InvalidXMLException("The object to serialise is null", null, String.Empty);
            }            
        }

        /// <summary>
        /// This serialises the passed in object to a byte array
        /// </summary>
        /// <param name="originalobject">The original object which is to be serialised</param>
        /// <returns>A byte array of the serialised object</returns>
        public static byte[] SerialiseByteArray(type originalobject)
        {
            if (originalobject != null)
            {
                try
                {
                    // First get the XML using the Serialize XML function
                    string xml = SerialiseXML(originalobject).Replace("utf-16", "utf-8");

                    // Now return the byte array
                    return Encoding.UTF8.GetBytes(xml);
                }
                catch (Exception e)
                {
                    // The serialisation failed so return an exception
                    throw new InvalidXMLException("The Binary Serialisation failed.", e, String.Empty);
                }
            }
            else
            {
                // The passed in object was null so throw an exception
                throw new InvalidXMLException("The object to serialise is null", null, String.Empty);
            }
        }

        /// <summary>
        /// This serialises the passed in object to a Base64 encoded XML string representation of it
        /// </summary>
        /// <param name="originalobject">The original object which is to be serialised</param>
        /// <returns>A Base64 encoded XML string containing the object</returns>
        public static string SerialiseBase64(type originalobject)
        {
            if (originalobject != null)
            {
                try
                {
                    // Get the XML
                    string standardxml = SerialiseXML(originalobject);

                    // The serialisation was successful so return the string
                    return Convert.ToBase64String(Encoding.Unicode.GetBytes(standardxml));
                }
                catch (Exception e)
                {
                    // The serialisation failed so return an exception
                    throw new InvalidXMLException("The XML Serialisation failed.", e, String.Empty);
                }
            }
            else
            {
                // The passed in object was null so throw an exception
                throw new InvalidXMLException("The object to serialise is null", null, String.Empty);
            }
        }

        /// <summary>
        /// This serialises the passed in object to an encrypted, encoded XML string representation of it
        /// </summary>
        /// <param name="originalobject">The original object which is to be serialised</param>
        /// <returns>An encrypted Base64 encoded XML string containing the object</returns>
        public static string SerialiseEncrypted(type originalobject)
        {
            if (originalobject != null)
            {
                try
                {
                    // Get the Base64 encoded XML
                    string base64xml = SerialiseBase64(originalobject);

                    // Return the encrypted XML
                    return EncryptionManager.EncryptString(base64xml);
                }
                catch (Exception e)
                {
                    // The serialisation failed so return an exception
                    throw new InvalidXMLException("The XML Serialisation failed.", e, String.Empty);
                }
            }
            else
            {
                // The passed in object was null so throw an exception
                throw new InvalidXMLException("The object to serialise is null", null, String.Empty);
            }
        }

        #endregion

        #region Deserialization

        /// <summary>
        /// This deserialises a standard XML string representation of an object back into its' original form
        /// </summary>
        /// <param name="standardxml">A standard XML string representation of the XML to return</param>
        /// <returns>The original object</returns>
        public static type DeserialiseXML(string standardxml)
        {
            if (standardxml != null && standardxml != String.Empty)
            {
                try
                {
                    // Use regular deserialisation
                    XmlSerializer s = new XmlSerializer(typeof(type));
                    StringReader sr = new StringReader(standardxml);
                    type o = (type)s.Deserialize(sr);
                    sr.Close();

                    // The deserialisation succeeded, return the object
                    return o;
                }
                catch (Exception e)
                {
                    // The deserialisation of the XML failed, return false
                    throw new InvalidXMLException("The XML deserialisation failed", e, standardxml);
                }
            }
            else
            {
                // The XML was invalid; throw an exception
                throw new InvalidXMLException("The XML was null or empty", null, standardxml);
            }
        }

        /// <summary>
        /// This deserialises an object from a file containing an XML representation of it
        /// </summary>
        /// <param name="filename">The file to deserialise</param>
        /// <returns>The original object</returns>
        public static type DeserialiseFromFile(string filename)
        {
            if (filename != null && filename != String.Empty && File.Exists(filename))
            {
                try
                {
                    // Use regular deserialisation
                    XmlSerializer s = new XmlSerializer(typeof(type));
                    TextReader t = new StreamReader(filename);
                    type o = (type)s.Deserialize(t);
                    t.Close();

                    // The deserialisation succeeded, return the object
                    return o;
                }
                catch (Exception e)
                {
                    // The deserialisation of the XML failed, return false
                    throw new InvalidXMLException("The XML deserialisation failed", e, filename);
                }
            }
            else
            {
                // The XML was invalid; throw an exception
                throw new InvalidXMLException("The file was null, empty or did not exist", null, filename);
            }
        }

        /// <summary>
        /// This function converts a Base64 encoded XML string representation of an object back into
        /// it's original form
        /// </summary>
        /// <param name="base64xml">The Base64 encoded XML string containing the object</param>
        /// <returns>The original object</returns>
        public static type DeserialiseBase64(string base64xml)
        {
            if (base64xml != null && base64xml != String.Empty)
            {
                try
                {
                    // First attempt to convert the string back from Base64
                    string standardxml = Encoding.Unicode.GetString(Convert.FromBase64String(base64xml));

                    // Now try to deserialise the standardised string into an object
                    return DeserialiseXML(standardxml);
                }
                catch (Exception e)
                {
                    // The deserialisation of the XML failed, return false
                    throw new InvalidXMLException("The XML deserialisation failed", e, base64xml);
                }
            }
            else
            {
                // The XML was invalid; throw an exception
                throw new InvalidXMLException("The XML was null or empty", null, base64xml);
            }
        }

        /// <summary>
        /// This function converts an encrypted Base64 encoded XML string representation of an object back into
        /// it's original form
        /// </summary>
        /// <param name="encryptedxml">The encrypted Base64 encoded XML string containing the object</param>
        /// <returns>The original object</returns>
        public static type DeserialiseEncrypted(string encryptedxml)
        {
            if (encryptedxml != null && encryptedxml != String.Empty)
            {
                try
                {
                    // First attempt to convert the string back from Base64
                    string base64xml = EncryptionManager.DecryptString(encryptedxml);

                    // Then return the original object
                    return DeserialiseBase64(base64xml);
                }
                catch (Exception e)
                {
                    // The deserialisation of the XML failed, return false
                    throw new InvalidXMLException("The XML deserialisation failed", e, encryptedxml);
                }
            }
            else
            {
                // The XML was invalid; throw an exception
                throw new InvalidXMLException("The XML was null or empty", null, encryptedxml);
            }
        }

        /// <summary>
        /// This deserialises a passed in byte array to its' original form
        /// </summary>
        /// <param name="bytes">The byte array containing the original object</param>
        /// <returns>The original object</returns>
        public static type DeserialiseByteArray(byte[] bytes)
        {
            if (bytes != null && bytes.Length > 0)
            {
                try
                {
                    // First return the byte array unto a string
                    string xml = Encoding.UTF8.GetString(bytes);

                    // Then return the deserialised object
                    return DeserialiseXML(xml);
                }
                catch (Exception e)
                {
                    // The deserialisation failed so return an exception
                    throw new InvalidXMLException("The Binary deserialisation failed.", e, String.Empty);
                }
            }
            else
            {
                // The XML was invalid; throw an exception
                throw new InvalidXMLException("The XML was null or empty", null, bytes.ToString());
            }
        }        

        #endregion
    }
}